class RCircle:
    circleBGColour = FCircle.circleBGColour
    canvasBGColour = FCircle.canvasBGColour
    lineColour = FCircle.lineColour
    minimumCirclePixels = FCircle.minimumCirclePixels
    # radiusOuterProportion = FCircle.proportion * 1.17
    radiusOuterProportion = FCircle.proportion * 1.3
    radiusOfEachCircle = 0.15