class Unicode:
    chars = {
        "Natural": "♮",
        "Flat": "♭",
        "Sharp": "♯",
        "Double Flat": "𝄫",
        "Double Sharp": "𝄪",
        "minor chord": "-",
        "augmented chord": "+",
        "diminished chord": "⚬",
        "half diminished chord": "⌀",
        "Major Seventh Triangle": "Δ",
        "Whole Note": "𝅝",
        "Half Note": "𝅗𝅥",
        "Quarter Note": "♩",
        "Eighth Note": "♪",
        "Whole Rest": "𝄻",
        "Half Rest": "𝄼",
        "Quarter Rest": "𝄽",
        "Eighth Rest": "𝄾",
        "Way Seperator": ". ",
        "Chinese Seperator": "。",
        "Diad": "◐",
        "Triad": "△",
        "Tetrad": "◇",
        "Pentad": "⬠",
        "Sextad": "⬡",
        "Septad": "✷",
        "Change Number": "⭘",
        "Chapter Number": "🗋",
        "Chapter Change Number": "",
        "Unique Change Number": "❄",
        "Unique Chapter Change Number": '🙉',
        "Index Number": "№",
        "Ring Number": "□",  # 💻
        "Tritone Sub": "⇅",
        "Ascending": "🠥",
        "Descending": "🠧",
        "Braille": "⣿",
        "Binary True": "▣",
        "Binary False": "◻",
        "Key": "🗝",
        "Key Lock": "🔐",
        "Lock": "🔒",
        "Piano": "🎹",
        "Keyboard": "🎘",
        "Mode": "↻",
        "Normalise": "⭳",
        "Wheel": "☸",
        "Abacus": "🧮",
        "Chord": "⦻",
        "Quartal Chord": "➿",
        "Triadic Chord": "➰",
        "Helix": "☤",
        "Step": "±",
        "Treble Clef": "𝄞",
        "Home": "🏠",
        "Hexagram": "卦",
        "Hexagram Chinese": "卦",
        "Tetragram Chinese": "首",
        "Trigram Chinese": "卦",
        "Hexagram Subpage": "🗎",
        "Trigram Subpage": "🗎",
        "Tetragram Subpage": "🗎",
        "Add Note": "+",
        "Remove Note": "-",
        "No Note": "№ ",
        "Mela": "🎪",
        "Raga": "🌈",
        "Word": "🗨",
        "IChing Chinese": "易經",
        "twelve o’clock": "🕛",
        "twelve-thirty": "🕧",
        "one o’clock": "🕐",
        "one-thirty": "🕜",
        "two o’clock": "🕑",
        "two-thirty": "🕝",
        "three o’clock": "🕒",
        "three-thirty": "🕞",
        "four o’clock": "🕓",
        "four-thirty": "🕟",
        "five o’clock": "🕔",
        "five-thirty": "🕠",
        "six o’clock": "🕕",
        "six-thirty": "🕡",
        "seven o’clock": "🕖",
        "seven-thirty": "🕢",
        "eight o’clock": "🕗",
        "eight-thirty": "🕣",
        "nine o’clock": "🕘",
        "nine-thirty": "🕤",
        "ten o’clock": "🕙",
        "ten-thirty": "🕥",
        "eleven o’clock": "🕚",
        "eleven-thirty": "🕦",
        'yin-yang': '☯',
        'giraffe': '🦒'
    }

    chars["Changed Note"] = (
       chars["Add Note"] + "/" + chars["Remove Note"]
    )
    romanNumeralCapitals = [
        "nulla",
        "Ⅰ",
        "Ⅱ",
        "Ⅲ",
        "Ⅳ",
        "Ⅴ",
        "Ⅵ",
        "Ⅶ",
        "Ⅷ",
        "Ⅸ",
        "Ⅹ",
        "Ⅺ",
        "Ⅻ",
    ]
    romanNumeralLowercase = [
        "nulla",
        "ⅰ",
        "ⅱ",
        "ⅲ",
        "ⅳ",
        "ⅴ",
        "ⅵ",
        "ⅶ",
        "ⅷ",
        "ⅸ",
        "ⅹ",
        "ⅺ",
        "ⅻ",
    ]