namesBySequenceIndex = []
namesBySequenceIndexNegative = []
semitonesBySequenceIndex = []
poemBySequenceIndex = []
poemAuthors= []
for i in range(2048):
    namesBySequenceIndex.append('')
    namesBySequenceIndexNegative.append('')
    semitonesBySequenceIndex.append('')
    poemBySequenceIndex.append('')
    poemAuthors.append('')
#1: C :0: 1: 1
namesBySequenceIndex[0] = '''Root,Ohm,Tonal Centre,Key,One,Unison'''
namesBySequenceIndexNegative[0] = ''''''
semitonesBySequenceIndex[0] = [0]
poemBySequenceIndex[0] = '''Begin'''
poemAuthors[0] = ''''''
#1: ䷗䷁ Returning Receptive: 12  : S : Ajaj

#2: C D♭ :0 1: 1 ♭2: sus♭2(no 5)
namesBySequenceIndex[1] = '''Minor Second'''
namesBySequenceIndexNegative[1] = ''''''
semitonesBySequenceIndex[1] = [0, 1]
poemBySequenceIndex[1] = '''B D'''
poemAuthors[1] = ''''''
#2: ䷒䷁ Arrived Receptive: 1 11  : S R1 : Ejaj

#3: C D :0 2: 1 2: sus2(no 5)
namesBySequenceIndex[2] = '''Major Second'''
namesBySequenceIndexNegative[2] = ''''''
semitonesBySequenceIndex[2] = [0, 2]
poemBySequenceIndex[2] = '''B F'''
poemAuthors[2] = ''''''
#3: ䷣䷁ Eclipse Receptive: 2 10  : S R2 : Ijaj

#4: C E♭ :0 3: 1 ♭3: mi(no 5)
namesBySequenceIndex[3] = '''Minor Third,Augmented Second,♯2'''
namesBySequenceIndexNegative[3] = ''''''
semitonesBySequenceIndex[3] = [0, 3]
poemBySequenceIndex[3] = '''B G'''
poemAuthors[3] = ''''''
#4: ䷲䷁ Thunder Receptive: 3 9  : S G2 : Abaj

#5: C E :0 4: 1 3: ma(no 5)
namesBySequenceIndex[4] = '''Major Third'''
namesBySequenceIndexNegative[4] = ''''''
semitonesBySequenceIndex[4] = [0, 4]
poemBySequenceIndex[4] = '''B J'''
poemAuthors[4] = ''''''
#5: ䷂䷁ Sprouting Receptive: 4 8  : S G3 : Akaj

#6: C F :0 5: 1 4: sus4(no 5)
namesBySequenceIndex[5] = '''Fourth,Inverted Power Chord,Perfect Fourth'''
namesBySequenceIndexNegative[5] = ''''''
semitonesBySequenceIndex[5] = [0, 5]
poemBySequenceIndex[5] = '''B K'''
poemAuthors[5] = ''''''
#6: ䷚䷁ Nourishment Receptive: 5 7  : S M1 : Alaj

#7: C G♭ :0 6: 1 ♭5: sus♯4(no 5)
namesBySequenceIndex[6] = '''Tritone,Diminished Fifth,Flat Fifth,Augmented Fourth,Sharp Eleven,Diaboli In Musica'''
namesBySequenceIndexNegative[6] = ''''''
semitonesBySequenceIndex[6] = [0, 6]
poemBySequenceIndex[6] = '''B L'''
poemAuthors[6] = ''''''
#7: ䷗䷗ Returning Returning : 6 6  : S M2 : AjAj

#8: C G :0 7: 1 5: 5
namesBySequenceIndex[7] = '''Fifth,Power Chord,Perfect Fifth'''
namesBySequenceIndexNegative[7] = ''''''
semitonesBySequenceIndex[7] = [0, 7]
poemBySequenceIndex[7] = '''B M'''
poemAuthors[7] = ''''''
#8: ䷗䷆ Returning Leading: 7 5  : S P : Ajej

#9: C G♯ :0 8: 1 ♯5: 1(add ♭6)
namesBySequenceIndex[8] = '''Minor Sixth,Augmented Fifth'''
namesBySequenceIndexNegative[8] = ''''''
semitonesBySequenceIndex[8] = [0, 8]
poemBySequenceIndex[8] = '''B N'''
poemAuthors[8] = ''''''
#9: ䷗䷎ Returning Humble: 8 4  : S D1 : Ajij

#10: C A :0 9: 1 6: 6(no 3 5)
namesBySequenceIndex[9] = '''Major Sixth'''
namesBySequenceIndexNegative[9] = ''''''
semitonesBySequenceIndex[9] = [0, 9]
poemBySequenceIndex[9] = '''B P'''
poemAuthors[9] = ''''''
#10: ䷗䷏ Returning Confident: 9 3  : S D2 : Ajab

#11: C B♭ :0 10: 1 ♭7: 7(no 3 5)
namesBySequenceIndex[10] = '''Minor Seventh'''
namesBySequenceIndexNegative[10] = ''''''
semitonesBySequenceIndex[10] = [0, 10]
poemBySequenceIndex[10] = '''B R'''
poemAuthors[10] = ''''''
#11: ䷗䷇ Returning Comparing: 10 2  : S N2 : Ajak

#12: C B :0 11: 1 7: ma7(no 3 5)
namesBySequenceIndex[11] = '''Major Seventh'''
namesBySequenceIndexNegative[11] = ''''''
semitonesBySequenceIndex[11] = [0, 11]
poemBySequenceIndex[11] = '''B S'''
poemAuthors[11] = ''''''
#12: ䷗䷖ Returning Stripping: 11 1  : S N3 : Ajal

#13: C D♭ D :0 1 2: 1 ♭2 2: sus2(no 5 add ♭9)
namesBySequenceIndex[12] = '''Chromatic Trichord,Chromatic Trimirror'''
namesBySequenceIndexNegative[12] = ''''''
semitonesBySequenceIndex[12] = [0, 1, 2]
poemBySequenceIndex[12] = '''B D F'''
poemAuthors[12] = ''''''
#13: ䷊䷁ Peace Receptive: 1 1 10  : S R1 G1 : Ojaj

#14: C D♭ E♭ :0 1 3: 1 ♭2 ♭3: mi(no 5 add ♭9)
namesBySequenceIndex[13] = '''Phrygian Trichord'''
namesBySequenceIndexNegative[13] = ''''''
semitonesBySequenceIndex[13] = [0, 1, 3]
poemBySequenceIndex[13] = '''B D G'''
poemAuthors[13] = ''''''
#14: ䷵䷁ Betrothing Receptive: 1 2 9  : S R1 G2 : Ebaj

#15: C D♭ E :0 1 4: 1 ♭2 3: ma(no 5 add ♭9)
namesBySequenceIndex[14] = '''Harmonic Trichord'''
namesBySequenceIndexNegative[14] = ''''''
semitonesBySequenceIndex[14] = [0, 1, 4]
poemBySequenceIndex[14] = '''B D J'''
poemAuthors[14] = ''''''
#15: ䷻䷁ Composed Receptive: 1 3 8  : S R1 G3 : Ekaj

#16: C D♭ F :0 1 5: 1 ♭2 4: sus4(no 5 add ♭9)
namesBySequenceIndex[15] = '''Japanese Trichord'''
namesBySequenceIndexNegative[15] = ''''''
semitonesBySequenceIndex[15] = [0, 1, 5]
poemBySequenceIndex[15] = '''B D K'''
poemAuthors[15] = ''''''
#16: ䷨䷁ Diminishing Receptive: 1 4 7  : S R1 M1 : Elaj

#17: C D♭ G♭ :0 1 6: 1 ♭2 ♭5: sus♭2(♭5)
namesBySequenceIndex[16] = '''Viennese Trichord'''
namesBySequenceIndexNegative[16] = ''''''
semitonesBySequenceIndex[16] = [0, 1, 6]
poemBySequenceIndex[16] = '''B D L'''
poemAuthors[16] = ''''''
#17: ䷒䷗ Arrived Returning: 1 5 6  : S R1 M2 : EjAj

#18: C D♭ G :0 1 7: 1 ♭2 5: sus♭2
namesBySequenceIndex[17] = '''Puppets Trichord'''
namesBySequenceIndexNegative[17] = ''''''
semitonesBySequenceIndex[17] = [0, 1, 7]
poemBySequenceIndex[17] = '''B D M'''
poemAuthors[17] = ''''''
#18: ䷒䷆ Arrived Leading: 1 6 5  : S R1 P : Ejej

#19: C D♭ G♯ :0 1 8: 1 ♭2 ♯5: sus♭2(♯5)
namesBySequenceIndex[18] = ''''''
namesBySequenceIndexNegative[18] = ''''''
semitonesBySequenceIndex[18] = [0, 1, 8]
poemBySequenceIndex[18] = '''B D N'''
poemAuthors[18] = ''''''
#19: ䷒䷎ Arrived Humble: 1 7 4  : S R1 D1 : Ejij

#20: C D♭ A :0 1 9: 1 ♭2 6: 6sus♭2(no 5)
namesBySequenceIndex[19] = ''''''
namesBySequenceIndexNegative[19] = ''''''
semitonesBySequenceIndex[19] = [0, 1, 9]
poemBySequenceIndex[19] = '''B D P'''
poemAuthors[19] = ''''''
#20: ䷒䷏ Arrived Confident: 1 8 3  : S R1 D2 : Ejab

#21: C D♭ B♭ :0 1 10: 1 ♭2 ♭7: 7sus♭2(no 5)
namesBySequenceIndex[20] = ''''''
namesBySequenceIndexNegative[20] = ''''''
semitonesBySequenceIndex[20] = [0, 1, 10]
poemBySequenceIndex[20] = '''B D R'''
poemAuthors[20] = ''''''
#21: ䷒䷇ Arrived Comparing: 1 9 2  : S R1 N2 : Ejak

#22: C D♭ B :0 1 11: 1 ♭2 7: ma7sus♭2(no 5)
namesBySequenceIndex[21] = ''''''
namesBySequenceIndexNegative[21] = ''''''
semitonesBySequenceIndex[21] = [0, 1, 11]
poemBySequenceIndex[21] = '''B D S'''
poemAuthors[21] = ''''''
#22: ䷒䷖ Arrived Stripping: 1 10 1  : S R1 N3 : Ejal

#23: C D E♭ :0 2 3: 1 2 ♭3: mi(no 5 add 9)
namesBySequenceIndex[22] = '''Minor Trichord'''
namesBySequenceIndexNegative[22] = ''''''
semitonesBySequenceIndex[22] = [0, 2, 3]
poemBySequenceIndex[22] = '''B F G'''
poemAuthors[22] = ''''''
#23: ䷶䷁ Abounding Receptive: 2 1 9  : S R2 G2 : Ibaj

#24: C D E :0 2 4: 1 2 3: ma(no 5 add 9)
namesBySequenceIndex[23] = '''Whole Tone Trichord'''
namesBySequenceIndexNegative[23] = ''''''
semitonesBySequenceIndex[23] = [0, 2, 4]
poemBySequenceIndex[23] = '''B F J'''
poemAuthors[23] = ''''''
#24: ䷾䷁ Completed Receptive: 2 2 8  : S R2 G3 : Ikaj

#25: C D F :0 2 5: 1 2 4: sus4(no 5 add 9)
namesBySequenceIndex[24] = ''''''
namesBySequenceIndexNegative[24] = ''''''
semitonesBySequenceIndex[24] = [0, 2, 5]
poemBySequenceIndex[24] = '''B F K'''
poemAuthors[24] = ''''''
#25: ䷕䷁ Adorning Receptive: 2 3 7  : S R2 M1 : Ilaj

#26: C D G♭ :0 2 6: 1 2 ♭5: sus2(♭5)
namesBySequenceIndex[25] = ''''''
namesBySequenceIndexNegative[25] = ''''''
semitonesBySequenceIndex[25] = [0, 2, 6]
poemBySequenceIndex[25] = '''B F L'''
poemAuthors[25] = ''''''
#26: ䷣䷗ Eclipse Returning: 2 4 6  : S R2 M2 : IjAj

#27: C D G :0 2 7: 1 2 5: sus2
namesBySequenceIndex[26] = '''Zappa Chord'''
namesBySequenceIndexNegative[26] = ''''''
semitonesBySequenceIndex[26] = [0, 2, 7]
poemBySequenceIndex[26] = '''B F M'''
poemAuthors[26] = ''''''
#27: ䷣䷆ Eclipse Leading: 2 5 5  : S R2 P : Ijej

#28: C D G♯ :0 2 8: 1 2 ♯5: sus2(♯5)
namesBySequenceIndex[27] = ''''''
namesBySequenceIndexNegative[27] = ''''''
semitonesBySequenceIndex[27] = [0, 2, 8]
poemBySequenceIndex[27] = '''B F N'''
poemAuthors[27] = ''''''
#28: ䷣䷎ Eclipse Humble: 2 6 4  : S R2 D1 : Ijij

#29: C D A :0 2 9: 1 2 6: 6sus2(no 5)
namesBySequenceIndex[28] = ''''''
namesBySequenceIndexNegative[28] = ''''''
semitonesBySequenceIndex[28] = [0, 2, 9]
poemBySequenceIndex[28] = '''B F P'''
poemAuthors[28] = ''''''
#29: ䷣䷏ Eclipse Confident: 2 7 3  : S R2 D2 : Ijab

#30: C D B♭ :0 2 10: 1 2 ♭7: 7sus2(no 5)
namesBySequenceIndex[29] = ''''''
namesBySequenceIndexNegative[29] = ''''''
semitonesBySequenceIndex[29] = [0, 2, 10]
poemBySequenceIndex[29] = '''B F R'''
poemAuthors[29] = ''''''
#30: ䷣䷇ Eclipse Comparing: 2 8 2  : S R2 N2 : Ijak

#31: C D B :0 2 11: 1 2 7: ma7sus2(no 5)
namesBySequenceIndex[30] = ''''''
namesBySequenceIndexNegative[30] = ''''''
semitonesBySequenceIndex[30] = [0, 2, 11]
poemBySequenceIndex[30] = '''B F S'''
poemAuthors[30] = ''''''
#31: ䷣䷖ Eclipse Stripping: 2 9 1  : S R2 N3 : Ijal

#32: C D♯ E :0 3 4: 1 ♯2 3: ma(no 5 add ♯9)
namesBySequenceIndex[31] = '''Major-Minor Trichord'''
namesBySequenceIndexNegative[31] = ''''''
semitonesBySequenceIndex[31] = [0, 3, 4]
poemBySequenceIndex[31] = '''B G J'''
poemAuthors[31] = ''''''
#32: ䷐䷁ Following Receptive: 3 1 8  : S R3 G3 : Adaj

#33: C E♭ F :0 3 5: 1 ♭3 4: mi(no 5 add 11)
namesBySequenceIndex[32] = ''''''
namesBySequenceIndexNegative[32] = ''''''
semitonesBySequenceIndex[32] = [0, 3, 5]
poemBySequenceIndex[32] = '''B G K'''
poemAuthors[32] = ''''''
#33: ䷔䷁ Biting Receptive: 3 2 7  : S G2 M1 : Afaj

#34: C E♭ G♭ :0 3 6: 1 ♭3 ♭5: °
namesBySequenceIndex[33] = '''Diminished Chord,Diminished Triad'''
namesBySequenceIndexNegative[33] = ''''''
semitonesBySequenceIndex[33] = [0, 3, 6]
poemBySequenceIndex[33] = '''B G L'''
poemAuthors[33] = ''''''
#34: ䷲䷗ Thunder Returning: 3 3 6  : S G2 M2 : AbAj

#35: C E♭ G :0 3 7: 1 ♭3 5: mi
namesBySequenceIndex[34] = '''Minor Chord,Minor Triad'''
namesBySequenceIndexNegative[34] = ''''''
semitonesBySequenceIndex[34] = [0, 3, 7]
poemBySequenceIndex[34] = '''B G M'''
poemAuthors[34] = ''''''
#35: ䷲䷆ Thunder Leading: 3 4 5  : S G2 P : Abej

#36: C E♭ G♯ :0 3 8: 1 ♭3 ♯5: mi(♯5)
namesBySequenceIndex[35] = ''''''
namesBySequenceIndexNegative[35] = ''''''
semitonesBySequenceIndex[35] = [0, 3, 8]
poemBySequenceIndex[35] = '''B G N'''
poemAuthors[35] = ''''''
#36: ䷲䷎ Thunder Humble: 3 5 4  : S G2 D1 : Abij

#37: C E♭ A :0 3 9: 1 ♭3 6: mi6(no 5)
namesBySequenceIndex[36] = ''''''
namesBySequenceIndexNegative[36] = ''''''
semitonesBySequenceIndex[36] = [0, 3, 9]
poemBySequenceIndex[36] = '''B G P'''
poemAuthors[36] = ''''''
#37: ䷲䷏ Thunder Confident: 3 6 3  : S G2 D2 : Abab

#38: C E♭ B♭ :0 3 10: 1 ♭3 ♭7: mi7(no 5)
namesBySequenceIndex[37] = ''''''
namesBySequenceIndexNegative[37] = ''''''
semitonesBySequenceIndex[37] = [0, 3, 10]
poemBySequenceIndex[37] = '''B G R'''
poemAuthors[37] = ''''''
#38: ䷲䷇ Thunder Comparing: 3 7 2  : S G2 N2 : Abak

#39: C E♭ B :0 3 11: 1 ♭3 7: mima7(no 5)
namesBySequenceIndex[38] = ''''''
namesBySequenceIndexNegative[38] = ''''''
semitonesBySequenceIndex[38] = [0, 3, 11]
poemBySequenceIndex[38] = '''B G S'''
poemAuthors[38] = ''''''
#39: ䷲䷖ Thunder Stripping: 3 8 1  : S G2 N3 : Abal

#40: C E F :0 4 5: 1 3 4: ma(no 5 add 11)
namesBySequenceIndex[39] = ''''''
namesBySequenceIndexNegative[39] = ''''''
semitonesBySequenceIndex[39] = [0, 4, 5]
poemBySequenceIndex[39] = '''B J K'''
poemAuthors[39] = ''''''
#40: ䷩䷁ Augmenting Receptive: 4 1 7  : S G3 M1 : Amaj

#41: C E G♭ :0 4 6: 1 3 ♭5: ma(♭5)
namesBySequenceIndex[40] = ''''''
namesBySequenceIndexNegative[40] = ''''''
semitonesBySequenceIndex[40] = [0, 4, 6]
poemBySequenceIndex[40] = '''B J L'''
poemAuthors[40] = ''''''
#41: ䷂䷗ Sprouting Returning: 4 2 6  : S G3 M2 : AkAj

#42: C E G :0 4 7: 1 3 5: ma
namesBySequenceIndex[41] = '''Major Chord,Major Triad,Raga Malasri,"The Answer"'''
namesBySequenceIndexNegative[41] = ''''''
semitonesBySequenceIndex[41] = [0, 4, 7]
poemBySequenceIndex[41] = '''B J M'''
poemAuthors[41] = ''''''
#42: ䷂䷆ Sprouting Leading: 4 3 5  : S G3 P : Akej

#43: C E G♯ :0 4 8: 1 3 ♯5: ma(♯5)
namesBySequenceIndex[42] = '''Augmented Chord,Augmented Triad,Minoric'''
namesBySequenceIndexNegative[42] = ''''''
semitonesBySequenceIndex[42] = [0, 4, 8]
poemBySequenceIndex[42] = '''B J N'''
poemAuthors[42] = ''''''
#43: ䷂䷎ Sprouting Humble: 4 4 4  : S G3 D1 : Akij

#44: C E A :0 4 9: 1 3 6: 6(no 5)
namesBySequenceIndex[43] = ''''''
namesBySequenceIndexNegative[43] = ''''''
semitonesBySequenceIndex[43] = [0, 4, 9]
poemBySequenceIndex[43] = '''B J P'''
poemAuthors[43] = ''''''
#44: ䷂䷏ Sprouting Confident: 4 5 3  : S G3 D2 : Akab

#45: C E B♭ :0 4 10: 1 3 ♭7: 7(no 5)
namesBySequenceIndex[44] = ''''''
namesBySequenceIndexNegative[44] = ''''''
semitonesBySequenceIndex[44] = [0, 4, 10]
poemBySequenceIndex[44] = '''B J R'''
poemAuthors[44] = ''''''
#45: ䷂䷇ Sprouting Comparing: 4 6 2  : S G3 N2 : Akak

#46: C E B :0 4 11: 1 3 7: ma7(no 5)
namesBySequenceIndex[45] = ''''''
namesBySequenceIndexNegative[45] = ''''''
semitonesBySequenceIndex[45] = [0, 4, 11]
poemBySequenceIndex[45] = '''B J S'''
poemAuthors[45] = ''''''
#46: ䷂䷖ Sprouting Stripping: 4 7 1  : S G3 N3 : Akal

#47: C F G♭ :0 5 6: 1 4 ♭5: sus4(♭5)
namesBySequenceIndex[46] = ''''''
namesBySequenceIndexNegative[46] = ''''''
semitonesBySequenceIndex[46] = [0, 5, 6]
poemBySequenceIndex[46] = '''B K L'''
poemAuthors[46] = ''''''
#47: ䷚䷗ Nourishment Returning: 5 1 6  : S M1 M2 : AlAj

#48: C F G :0 5 7: 1 4 5: sus4
namesBySequenceIndex[47] = '''Suspended Chord'''
namesBySequenceIndexNegative[47] = ''''''
semitonesBySequenceIndex[47] = [0, 5, 7]
poemBySequenceIndex[47] = '''B K M'''
poemAuthors[47] = ''''''
#48: ䷚䷆ Nourishment Leading: 5 2 5  : S M1 P : Alej

#49: C F G♯ :0 5 8: 1 4 ♯5: sus4(♯5)
namesBySequenceIndex[48] = ''''''
namesBySequenceIndexNegative[48] = ''''''
semitonesBySequenceIndex[48] = [0, 5, 8]
poemBySequenceIndex[48] = '''B K N'''
poemAuthors[48] = ''''''
#49: ䷚䷎ Nourishment Humble: 5 3 4  : S M1 D1 : Alij

#50: C F A :0 5 9: 1 4 6: 6sus4(no 5)
namesBySequenceIndex[49] = ''''''
namesBySequenceIndexNegative[49] = ''''''
semitonesBySequenceIndex[49] = [0, 5, 9]
poemBySequenceIndex[49] = '''B K P'''
poemAuthors[49] = ''''''
#50: ䷚䷏ Nourishment Confident: 5 4 3  : S M1 D2 : Alab

#51: C F B♭ :0 5 10: 1 4 ♭7: 7sus4(no 5)
namesBySequenceIndex[50] = '''Quartal Trichord'''
namesBySequenceIndexNegative[50] = ''''''
semitonesBySequenceIndex[50] = [0, 5, 10]
poemBySequenceIndex[50] = '''B K R'''
poemAuthors[50] = ''''''
#51: ䷚䷇ Nourishment Comparing: 5 5 2  : S M1 N2 : Alak

#52: C F B :0 5 11: 1 4 7: ma7sus4(no 5)
namesBySequenceIndex[51] = ''''''
namesBySequenceIndexNegative[51] = ''''''
semitonesBySequenceIndex[51] = [0, 5, 11]
poemBySequenceIndex[51] = '''B K S'''
poemAuthors[51] = ''''''
#52: ䷚䷖ Nourishment Stripping: 5 6 1  : S M1 N3 : Alal

#53: C F♯ G :0 6 7: 1 ♯4 5: sus♯4
namesBySequenceIndex[52] = ''''''
namesBySequenceIndexNegative[52] = ''''''
semitonesBySequenceIndex[52] = [0, 6, 7]
poemBySequenceIndex[52] = '''B L M'''
poemAuthors[52] = ''''''
#53: ䷗䷒ Returning Arrived: 6 1 5  : S M2 P : AjEj

#54: C F♯ G♯ :0 6 8: 1 ♯4 ♯5: sus♯4(♯5)
namesBySequenceIndex[53] = ''''''
namesBySequenceIndexNegative[53] = ''''''
semitonesBySequenceIndex[53] = [0, 6, 8]
poemBySequenceIndex[53] = '''B L N'''
poemAuthors[53] = ''''''
#54: ䷗䷣ Returning Eclipse: 6 2 4  : S M2 D1 : AjIj

#55: C G♭ A :0 6 9: 1 ♭5 6: 6(♭5 no 3)
namesBySequenceIndex[54] = '''Italian Sixth'''
namesBySequenceIndexNegative[54] = ''''''
semitonesBySequenceIndex[54] = [0, 6, 9]
poemBySequenceIndex[54] = '''B L P'''
poemAuthors[54] = ''''''
#55: ䷗䷲ Returning Thunder: 6 3 3  : S M2 D2 : AjAb

#56: C G♭ B♭ :0 6 10: 1 ♭5 ♭7: 7(♭5 no 3)
namesBySequenceIndex[55] = ''''''
namesBySequenceIndexNegative[55] = ''''''
semitonesBySequenceIndex[55] = [0, 6, 10]
poemBySequenceIndex[55] = '''B L R'''
poemAuthors[55] = ''''''
#56: ䷗䷂ Returning Sprouting: 6 4 2  : S M2 N2 : AjAk

#57: C G♭ B :0 6 11: 1 ♭5 7: ma7(♭5 no 3)
namesBySequenceIndex[56] = ''''''
namesBySequenceIndexNegative[56] = ''''''
semitonesBySequenceIndex[56] = [0, 6, 11]
poemBySequenceIndex[56] = '''B L S'''
poemAuthors[56] = ''''''
#57: ䷗䷚ Returning Nourishment: 6 5 1  : S M2 N3 : AjAl

#58: C G A♭ :0 7 8: 1 5 ♭6: 5(add ♭6)
namesBySequenceIndex[57] = ''''''
namesBySequenceIndexNegative[57] = ''''''
semitonesBySequenceIndex[57] = [0, 7, 8]
poemBySequenceIndex[57] = '''B M N'''
poemAuthors[57] = ''''''
#58: ䷗䷭ Returning Ascending: 7 1 4  : S P D1 : Ajoj

#59: C G A :0 7 9: 1 5 6: 5(add 6)
namesBySequenceIndex[58] = ''''''
namesBySequenceIndexNegative[58] = ''''''
semitonesBySequenceIndex[58] = [0, 7, 9]
poemBySequenceIndex[58] = '''B M P'''
poemAuthors[58] = ''''''
#59: ䷗䷧ Returning Deliverance: 7 2 3  : S P D2 : Ajeb

#60: C G B♭ :0 7 10: 1 5 ♭7: 7(no 3)
namesBySequenceIndex[59] = ''''''
namesBySequenceIndexNegative[59] = ''''''
semitonesBySequenceIndex[59] = [0, 7, 10]
poemBySequenceIndex[59] = '''B M R'''
poemAuthors[59] = ''''''
#60: ䷗䷜ Returning Chasm: 7 3 2  : S P N2 : Ajek

#61: C G B :0 7 11: 1 5 7: ma7(no 3)
namesBySequenceIndex[60] = ''''''
namesBySequenceIndexNegative[60] = ''''''
semitonesBySequenceIndex[60] = [0, 7, 11]
poemBySequenceIndex[60] = '''B M S'''
poemAuthors[60] = ''''''
#61: ䷗䷃ Returning Naïve: 7 4 1  : S P N3 : Ajel

#62: C G♯ A :0 8 9: 1 ♯5 6: 6(♯5 no 3)
namesBySequenceIndex[61] = ''''''
namesBySequenceIndexNegative[61] = ''''''
semitonesBySequenceIndex[61] = [0, 8, 9]
poemBySequenceIndex[61] = '''B N P'''
poemAuthors[61] = ''''''
#62: ䷗䷽ Returning Detail: 8 1 3  : S D1 N1 : Ajib

#63: C G♯ B♭ :0 8 10: 1 ♯5 ♭7: 7(♯5 no 3)
namesBySequenceIndex[62] = ''''''
namesBySequenceIndexNegative[62] = ''''''
semitonesBySequenceIndex[62] = [0, 8, 10]
poemBySequenceIndex[62] = '''B N R'''
poemAuthors[62] = ''''''
#63: ䷗䷦ Returning Limping: 8 2 2  : S D1 N2 : Ajik

#64: C G♯ B :0 8 11: 1 ♯5 7: ma7(♯5 no 3)
namesBySequenceIndex[63] = ''''''
namesBySequenceIndexNegative[63] = ''''''
semitonesBySequenceIndex[63] = [0, 8, 11]
poemBySequenceIndex[63] = '''B N S'''
poemAuthors[63] = ''''''
#64: ䷗䷳ Returning Stillness: 8 3 1  : S D1 N3 : Ajil

#65: C A B♭ :0 9 10: 1 6 ♭7: 7(no 3 5 add 13)
namesBySequenceIndex[64] = ''''''
namesBySequenceIndexNegative[64] = ''''''
semitonesBySequenceIndex[64] = [0, 9, 10]
poemBySequenceIndex[64] = '''B P R'''
poemAuthors[64] = ''''''
#65: ䷗䷬ Returning Clustering: 9 1 2  : S D2 N2 : Ajad

#66: C A B :0 9 11: 1 6 7: ma7(no 3 5 add 13)
namesBySequenceIndex[65] = ''''''
namesBySequenceIndexNegative[65] = ''''''
semitonesBySequenceIndex[65] = [0, 9, 11]
poemBySequenceIndex[65] = '''B P S'''
poemAuthors[65] = ''''''
#66: ䷗䷢ Returning Prospering: 9 2 1  : S D2 N3 : Ajaf

#67: C A♯ B :0 10 11: 1 ♯6 7: ma7(no 3 5 add ♭7)
namesBySequenceIndex[66] = ''''''
namesBySequenceIndexNegative[66] = ''''''
semitonesBySequenceIndex[66] = [0, 10, 11]
poemBySequenceIndex[66] = '''B R S'''
poemAuthors[66] = ''''''
#67: ䷗䷓ Returning Seeing: 10 1 1  : S D3 N3 : Ajam

#68: C D♭ D E♭ :0 1 2 3: 1 ♭2 2 ♭3: mi(no 5 add ♭9 9)
namesBySequenceIndex[67] = '''Chromatic Tetrachord'''
namesBySequenceIndexNegative[67] = ''''''
semitonesBySequenceIndex[67] = [0, 1, 2, 3]
poemBySequenceIndex[67] = '''B D F G'''
poemAuthors[67] = ''''''
#68: ䷡䷁ Might Receptive: 1 1 1 9  : S R1 R2 G2 : Obaj

#69: C D♭ D E :0 1 2 4: 1 ♭2 2 3: ma(no 5 add ♭9 9)
namesBySequenceIndex[68] = ''''''
namesBySequenceIndexNegative[68] = ''''''
semitonesBySequenceIndex[68] = [0, 1, 2, 4]
poemBySequenceIndex[68] = '''B D F J'''
poemAuthors[68] = ''''''
#69: ䷄䷁ Biding Receptive: 1 1 2 8  : S R1 R2 G3 : Okaj

#70: C D♭ D F :0 1 2 5: 1 ♭2 2 4: sus4(no 5 add ♭9 9)
namesBySequenceIndex[69] = ''''''
namesBySequenceIndexNegative[69] = ''''''
semitonesBySequenceIndex[69] = [0, 1, 2, 5]
poemBySequenceIndex[69] = '''B D F K'''
poemAuthors[69] = ''''''
#70: ䷙䷁ Cultivating Receptive: 1 1 3 7  : S R1 G1 M1 : Olaj

#71: C D♭ D G♭ :0 1 2 6: 1 ♭2 2 ♭5: sus2(♭5 add ♭9)
namesBySequenceIndex[70] = ''''''
namesBySequenceIndexNegative[70] = ''''''
semitonesBySequenceIndex[70] = [0, 1, 2, 6]
poemBySequenceIndex[70] = '''B D F L'''
poemAuthors[70] = ''''''
#71: ䷊䷗ Peace Returning: 1 1 4 6  : S R1 G1 M2 : OjAj

#72: C D♭ D G :0 1 2 7: 1 ♭2 2 5: sus2(add ♭9)
namesBySequenceIndex[71] = ''''''
namesBySequenceIndexNegative[71] = ''''''
semitonesBySequenceIndex[71] = [0, 1, 2, 7]
poemBySequenceIndex[71] = '''B D F M'''
poemAuthors[71] = ''''''
#72: ䷊䷆ Peace Leading: 1 1 5 5  : S R1 G1 P : Ojej

#73: C D♭ D G♯ :0 1 2 8: 1 ♭2 2 ♯5: sus2(♯5 add ♭9)
namesBySequenceIndex[72] = ''''''
namesBySequenceIndexNegative[72] = ''''''
semitonesBySequenceIndex[72] = [0, 1, 2, 8]
poemBySequenceIndex[72] = '''B D F N'''
poemAuthors[72] = ''''''
#73: ䷊䷎ Peace Humble: 1 1 6 4  : S R1 G1 D1 : Ojij

#74: C D♭ D A :0 1 2 9: 1 ♭2 2 6: 6sus2(no 5 add ♭9)
namesBySequenceIndex[73] = ''''''
namesBySequenceIndexNegative[73] = ''''''
semitonesBySequenceIndex[73] = [0, 1, 2, 9]
poemBySequenceIndex[73] = '''B D F P'''
poemAuthors[73] = ''''''
#74: ䷊䷏ Peace Confident: 1 1 7 3  : S R1 G1 D2 : Ojab

#75: C D♭ D B♭ :0 1 2 10: 1 ♭2 2 ♭7: 9sus♭2(no 5)
namesBySequenceIndex[74] = ''''''
namesBySequenceIndexNegative[74] = ''''''
semitonesBySequenceIndex[74] = [0, 1, 2, 10]
poemBySequenceIndex[74] = '''B D F R'''
poemAuthors[74] = ''''''
#75: ䷊䷇ Peace Comparing: 1 1 8 2  : S R1 G1 N2 : Ojak

#76: C D♭ D B :0 1 2 11: 1 ♭2 2 7: ma9sus♭2(no 5)
namesBySequenceIndex[75] = ''''''
namesBySequenceIndexNegative[75] = ''''''
semitonesBySequenceIndex[75] = [0, 1, 2, 11]
poemBySequenceIndex[75] = '''B D F S'''
poemAuthors[75] = ''''''
#76: ䷊䷖ Peace Stripping: 1 1 9 1  : S R1 G1 N3 : Ojal

#77: C D♭ D♯ E :0 1 3 4: 1 ♭2 ♯2 3: ma(no 5 add ♭9 ♯9)
namesBySequenceIndex[76] = '''Altered Tetrachord'''
namesBySequenceIndexNegative[76] = ''''''
semitonesBySequenceIndex[76] = [0, 1, 3, 4]
poemBySequenceIndex[76] = '''B D G J'''
poemAuthors[76] = ''''''
#77: ䷹䷁ Joyous Receptive: 1 2 1 8  : S R1 R3 G3 : Edaj

#78: C D♭ E♭ F :0 1 3 5: 1 ♭2 ♭3 4: mi(no 5 add ♭9 11)
namesBySequenceIndex[77] = '''Phrygian Tetrachord'''
namesBySequenceIndexNegative[77] = ''''''
semitonesBySequenceIndex[77] = [0, 1, 3, 5]
poemBySequenceIndex[77] = '''B D G K'''
poemAuthors[77] = ''''''
#78: ䷥䷁ Polarising Receptive: 1 2 2 7  : S R1 G2 M1 : Efaj

#79: C D♭ E♭ G♭ :0 1 3 6: 1 ♭2 ♭3 ♭5: °(add ♭9)
namesBySequenceIndex[78] = ''''''
namesBySequenceIndexNegative[78] = ''''''
semitonesBySequenceIndex[78] = [0, 1, 3, 6]
poemBySequenceIndex[78] = '''B D G L'''
poemAuthors[78] = ''''''
#79: ䷵䷗ Betrothing Returning: 1 2 3 6  : S R1 G2 M2 : EbAj

#80: C D♭ E♭ G :0 1 3 7: 1 ♭2 ♭3 5: mi(add ♭9)
namesBySequenceIndex[79] = '''All-In 1,All Interval Tetrachord 1'''
namesBySequenceIndexNegative[79] = ''''''
semitonesBySequenceIndex[79] = [0, 1, 3, 7]
poemBySequenceIndex[79] = '''B D G M'''
poemAuthors[79] = ''''''
#80: ䷵䷆ Betrothing Leading: 1 2 4 5  : S R1 G2 P : Ebej

#81: C D♭ E♭ G♯ :0 1 3 8: 1 ♭2 ♭3 ♯5: mi(♯5 add ♭9)
namesBySequenceIndex[80] = ''''''
namesBySequenceIndexNegative[80] = ''''''
semitonesBySequenceIndex[80] = [0, 1, 3, 8]
poemBySequenceIndex[80] = '''B D G N'''
poemAuthors[80] = ''''''
#81: ䷵䷎ Betrothing Humble: 1 2 5 4  : S R1 G2 D1 : Ebij

#82: C D♭ E♭ A :0 1 3 9: 1 ♭2 ♭3 6: mi6(no 5 add ♭9)
namesBySequenceIndex[81] = ''''''
namesBySequenceIndexNegative[81] = ''''''
semitonesBySequenceIndex[81] = [0, 1, 3, 9]
poemBySequenceIndex[81] = '''B D G P'''
poemAuthors[81] = ''''''
#82: ䷵䷏ Betrothing Confident: 1 2 6 3  : S R1 G2 D2 : Ebab

#83: C D♭ E♭ B♭ :0 1 3 10: 1 ♭2 ♭3 ♭7: mi7(no 5 add ♭9)
namesBySequenceIndex[82] = ''''''
namesBySequenceIndexNegative[82] = ''''''
semitonesBySequenceIndex[82] = [0, 1, 3, 10]
poemBySequenceIndex[82] = '''B D G R'''
poemAuthors[82] = ''''''
#83: ䷵䷇ Betrothing Comparing: 1 2 7 2  : S R1 G2 N2 : Ebak

#84: C D♭ E♭ B :0 1 3 11: 1 ♭2 ♭3 7: mima7(no 5 add ♭9)
namesBySequenceIndex[83] = ''''''
namesBySequenceIndexNegative[83] = ''''''
semitonesBySequenceIndex[83] = [0, 1, 3, 11]
poemBySequenceIndex[83] = '''B D G S'''
poemAuthors[83] = ''''''
#84: ䷵䷖ Betrothing Stripping: 1 2 8 1  : S R1 G2 N3 : Ebal

#85: C D♭ E F :0 1 4 5: 1 ♭2 3 4: ma(no 5 add ♭9 11)
namesBySequenceIndex[84] = ''''''
namesBySequenceIndexNegative[84] = ''''''
semitonesBySequenceIndex[84] = [0, 1, 4, 5]
poemBySequenceIndex[84] = '''B D J K'''
poemAuthors[84] = ''''''
#85: ䷼䷁ Truth Receptive: 1 3 1 7  : S R1 G3 M1 : Emaj

#86: C D♭ E G♭ :0 1 4 6: 1 ♭2 3 ♭5: ma(♭5 add ♭9)
namesBySequenceIndex[85] = '''All-In 2,All Interval Tetrachord 2'''
namesBySequenceIndexNegative[85] = ''''''
semitonesBySequenceIndex[85] = [0, 1, 4, 6]
poemBySequenceIndex[85] = '''B D J L'''
poemAuthors[85] = ''''''
#86: ䷻䷗ Composed Returning: 1 3 2 6  : S R1 G3 M2 : EkAj

#87: C D♭ E G :0 1 4 7: 1 ♭2 3 5: ma(add ♭9)
namesBySequenceIndex[86] = ''''''
namesBySequenceIndexNegative[86] = ''''''
semitonesBySequenceIndex[86] = [0, 1, 4, 7]
poemBySequenceIndex[86] = '''B D J M'''
poemAuthors[86] = ''''''
#87: ䷻䷆ Composed Leading: 1 3 3 5  : S R1 G3 P : Ekej

#88: C D♭ E G♯ :0 1 4 8: 1 ♭2 3 ♯5: ma(♯5 add ♭9)
namesBySequenceIndex[87] = '''Dalic'''
namesBySequenceIndexNegative[87] = ''''''
semitonesBySequenceIndex[87] = [0, 1, 4, 8]
poemBySequenceIndex[87] = '''B D J N'''
poemAuthors[87] = ''''''
#88: ䷻䷎ Composed Humble: 1 3 4 4  : S R1 G3 D1 : Ekij

#89: C D♭ E A :0 1 4 9: 1 ♭2 3 6: 6(no 5 add ♭9)
namesBySequenceIndex[88] = ''''''
namesBySequenceIndexNegative[88] = ''''''
semitonesBySequenceIndex[88] = [0, 1, 4, 9]
poemBySequenceIndex[88] = '''B D J P'''
poemAuthors[88] = ''''''
#89: ䷻䷏ Composed Confident: 1 3 5 3  : S R1 G3 D2 : Ekab

#90: C D♭ E B♭ :0 1 4 10: 1 ♭2 3 ♭7: 7(no 5 add ♭9)
namesBySequenceIndex[89] = ''''''
namesBySequenceIndexNegative[89] = ''''''
semitonesBySequenceIndex[89] = [0, 1, 4, 10]
poemBySequenceIndex[89] = '''B D J R'''
poemAuthors[89] = ''''''
#90: ䷻䷇ Composed Comparing: 1 3 6 2  : S R1 G3 N2 : Ekak

#91: C D♭ E B :0 1 4 11: 1 ♭2 3 7: ma7(no 5 add ♭9)
namesBySequenceIndex[90] = ''''''
namesBySequenceIndexNegative[90] = ''''''
semitonesBySequenceIndex[90] = [0, 1, 4, 11]
poemBySequenceIndex[90] = '''B D J S'''
poemAuthors[90] = ''''''
#91: ䷻䷖ Composed Stripping: 1 3 7 1  : S R1 G3 N3 : Ekal

#92: C D♭ F G♭ :0 1 5 6: 1 ♭2 4 ♭5: sus4(♭5 add ♭9)
namesBySequenceIndex[91] = ''''''
namesBySequenceIndexNegative[91] = ''''''
semitonesBySequenceIndex[91] = [0, 1, 5, 6]
poemBySequenceIndex[91] = '''B D K L'''
poemAuthors[91] = ''''''
#92: ䷨䷗ Diminishing Returning: 1 4 1 6  : S R1 M1 M2 : ElAj

#93: C D♭ F G :0 1 5 7: 1 ♭2 4 5: sus4(add ♭9)
namesBySequenceIndex[92] = '''Vietnamese Tetrachord'''
namesBySequenceIndexNegative[92] = ''''''
semitonesBySequenceIndex[92] = [0, 1, 5, 7]
poemBySequenceIndex[92] = '''B D K M'''
poemAuthors[92] = ''''''
#93: ䷨䷆ Diminishing Leading: 1 4 2 5  : S R1 M1 P : Elej

#94: C D♭ F G♯ :0 1 5 8: 1 ♭2 4 ♯5: sus4(♯5 add ♭9)
namesBySequenceIndex[93] = '''Aerathic'''
namesBySequenceIndexNegative[93] = ''''''
semitonesBySequenceIndex[93] = [0, 1, 5, 8]
poemBySequenceIndex[93] = '''B D K N'''
poemAuthors[93] = ''''''
#94: ䷨䷎ Diminishing Humble: 1 4 3 4  : S R1 M1 D1 : Elij

#95: C D♭ F A :0 1 5 9: 1 ♭2 4 6: 6sus4(no 5 add ♭9)
namesBySequenceIndex[94] = '''Pyrric'''
namesBySequenceIndexNegative[94] = ''''''
semitonesBySequenceIndex[94] = [0, 1, 5, 9]
poemBySequenceIndex[94] = '''B D K P'''
poemAuthors[94] = ''''''
#95: ䷨䷏ Diminishing Confident: 1 4 4 3  : S R1 M1 D2 : Elab

#96: C D♭ F B♭ :0 1 5 10: 1 ♭2 4 ♭7: 7sus4(no 5 add ♭9)
namesBySequenceIndex[95] = ''''''
namesBySequenceIndexNegative[95] = ''''''
semitonesBySequenceIndex[95] = [0, 1, 5, 10]
poemBySequenceIndex[95] = '''B D K R'''
poemAuthors[95] = ''''''
#96: ䷨䷇ Diminishing Comparing: 1 4 5 2  : S R1 M1 N2 : Elak

#97: C D♭ F B :0 1 5 11: 1 ♭2 4 7: ma7sus4(no 5 add ♭9)
namesBySequenceIndex[96] = ''''''
namesBySequenceIndexNegative[96] = ''''''
semitonesBySequenceIndex[96] = [0, 1, 5, 11]
poemBySequenceIndex[96] = '''B D K S'''
poemAuthors[96] = ''''''
#97: ䷨䷖ Diminishing Stripping: 1 4 6 1  : S R1 M1 N3 : Elal

#98: C D♭ F♯ G :0 1 6 7: 1 ♭2 ♯4 5: sus♯4(add ♭9)
namesBySequenceIndex[97] = '''Distance Model 1 5'''
namesBySequenceIndexNegative[97] = ''''''
semitonesBySequenceIndex[97] = [0, 1, 6, 7]
poemBySequenceIndex[97] = '''B D L M'''
poemAuthors[97] = ''''''
#98: ䷒䷒ Arrived Arrived : 1 5 1 5  : S R1 M2 P : EjEj

#99: C D♭ F♯ G♯ :0 1 6 8: 1 ♭2 ♯4 ♯5: sus♯4(♯5 add ♭9)
namesBySequenceIndex[98] = ''''''
namesBySequenceIndexNegative[98] = ''''''
semitonesBySequenceIndex[98] = [0, 1, 6, 8]
poemBySequenceIndex[98] = '''B D L N'''
poemAuthors[98] = ''''''
#99: ䷒䷣ Arrived Eclipse: 1 5 2 4  : S R1 M2 D1 : EjIj

#100: C D♭ G♭ A :0 1 6 9: 1 ♭2 ♭5 6: 6sus♭2(♭5)
namesBySequenceIndex[99] = ''''''
namesBySequenceIndexNegative[99] = ''''''
semitonesBySequenceIndex[99] = [0, 1, 6, 9]
poemBySequenceIndex[99] = '''B D L P'''
poemAuthors[99] = ''''''
#100: ䷒䷲ Arrived Thunder: 1 5 3 3  : S R1 M2 D2 : EjAb

#101: C D♭ G♭ B♭ :0 1 6 10: 1 ♭2 ♭5 ♭7: 7sus♭2(♭5)
namesBySequenceIndex[100] = '''''' #All-In 3,All Interval Tetrachord 3
namesBySequenceIndexNegative[100] = ''''''
semitonesBySequenceIndex[100] = [0, 1, 6, 10]
poemBySequenceIndex[100] = '''B D L R'''
poemAuthors[100] = ''''''
#101: ䷒䷂ Arrived Sprouting: 1 5 4 2  : S R1 M2 N2 : EjAk

#102: C D♭ G♭ B :0 1 6 11: 1 ♭2 ♭5 7: ma7sus♭2(♭5)
namesBySequenceIndex[101] = ''''''
namesBySequenceIndexNegative[101] = ''''''
semitonesBySequenceIndex[101] = [0, 1, 6, 11]
poemBySequenceIndex[101] = '''B D L S'''
poemAuthors[101] = ''''''
#102: ䷒䷚ Arrived Nourishment: 1 5 5 1  : S R1 M2 N3 : EjAl

#103: C D♭ G A♭ :0 1 7 8: 1 ♭2 5 ♭6: sus♭2(add ♭6)
namesBySequenceIndex[102] = ''''''
namesBySequenceIndexNegative[102] = ''''''
semitonesBySequenceIndex[102] = [0, 1, 7, 8]
poemBySequenceIndex[102] = '''B D M N'''
poemAuthors[102] = ''''''
#103: ䷒䷭ Arrived Ascending: 1 6 1 4  : S R1 P D1 : Ejoj

#104: C D♭ G A :0 1 7 9: 1 ♭2 5 6: 6sus♭2
namesBySequenceIndex[103] = '''''' #All-In 4,All Interval Tetrachord 4
namesBySequenceIndexNegative[103] = ''''''
semitonesBySequenceIndex[103] = [0, 1, 7, 9]
poemBySequenceIndex[103] = '''B D M P'''
poemAuthors[103] = ''''''
#104: ䷒䷧ Arrived Deliverance: 1 6 2 3  : S R1 P D2 : Ejeb

#105: C D♭ G B♭ :0 1 7 10: 1 ♭2 5 ♭7: 7sus♭2
namesBySequenceIndex[104] = ''''''
namesBySequenceIndexNegative[104] = ''''''
semitonesBySequenceIndex[104] = [0, 1, 7, 10]
poemBySequenceIndex[104] = '''B D M R'''
poemAuthors[104] = ''''''
#105: ䷒䷜ Arrived Chasm: 1 6 3 2  : S R1 P N2 : Ejek

#106: C D♭ G B :0 1 7 11: 1 ♭2 5 7: ma7sus♭2
namesBySequenceIndex[105] = ''''''
namesBySequenceIndexNegative[105] = ''''''
semitonesBySequenceIndex[105] = [0, 1, 7, 11]
poemBySequenceIndex[105] = '''B D M S'''
poemAuthors[105] = ''''''
#106: ䷒䷃ Arrived Naïve: 1 6 4 1  : S R1 P N3 : Ejel

#107: C D♭ G♯ A :0 1 8 9: 1 ♭2 ♯5 6: 6sus♭2(♯5)
namesBySequenceIndex[106] = ''''''
namesBySequenceIndexNegative[106] = ''''''
semitonesBySequenceIndex[106] = [0, 1, 8, 9]
poemBySequenceIndex[106] = '''B D N P'''
poemAuthors[106] = ''''''
#107: ䷒䷽ Arrived Detail: 1 7 1 3  : S R1 D1 N1 : Ejib

#108: C D♭ G♯ B♭ :0 1 8 10: 1 ♭2 ♯5 ♭7: 7sus♭2(♯5)
namesBySequenceIndex[107] = ''''''
namesBySequenceIndexNegative[107] = ''''''
semitonesBySequenceIndex[107] = [0, 1, 8, 10]
poemBySequenceIndex[107] = '''B D N R'''
poemAuthors[107] = ''''''
#108: ䷒䷦ Arrived Limping: 1 7 2 2  : S R1 D1 N2 : Ejik

#109: C D♭ G♯ B :0 1 8 11: 1 ♭2 ♯5 7: ma7sus♭2(♯5)
namesBySequenceIndex[108] = ''''''
namesBySequenceIndexNegative[108] = ''''''
semitonesBySequenceIndex[108] = [0, 1, 8, 11]
poemBySequenceIndex[108] = '''B D N S'''
poemAuthors[108] = ''''''
#109: ䷒䷳ Arrived Stillness: 1 7 3 1  : S R1 D1 N3 : Ejil

#110: C D♭ A B♭ :0 1 9 10: 1 ♭2 6 ♭7: 7sus♭2(no 5 add 13)
namesBySequenceIndex[109] = ''''''
namesBySequenceIndexNegative[109] = ''''''
semitonesBySequenceIndex[109] = [0, 1, 9, 10]
poemBySequenceIndex[109] = '''B D P R'''
poemAuthors[109] = ''''''
#110: ䷒䷬ Arrived Clustering: 1 8 1 2  : S R1 D2 N2 : Ejad

#111: C D♭ A B :0 1 9 11: 1 ♭2 6 7: ma7sus♭2(no 5 add 13)
namesBySequenceIndex[110] = ''''''
namesBySequenceIndexNegative[110] = ''''''
semitonesBySequenceIndex[110] = [0, 1, 9, 11]
poemBySequenceIndex[110] = '''B D P S'''
poemAuthors[110] = ''''''
#111: ䷒䷢ Arrived Prospering: 1 8 2 1  : S R1 D2 N3 : Ejaf

#112: C D♭ A♯ B :0 1 10 11: 1 ♭2 ♯6 7: ma7sus♭2(no 5 add ♭7)
namesBySequenceIndex[111] = ''''''
namesBySequenceIndexNegative[111] = ''''''
semitonesBySequenceIndex[111] = [0, 1, 10, 11]
poemBySequenceIndex[111] = '''B D R S'''
poemAuthors[111] = ''''''
#112: ䷒䷓ Arrived Seeing: 1 9 1 1  : S R1 D3 N3 : Ejam

#113: C D D♯ E :0 2 3 4: 1 2 ♯2 3: ma(no 5 add 9 ♯9)
namesBySequenceIndex[112] = ''''''
namesBySequenceIndexNegative[112] = ''''''
semitonesBySequenceIndex[112] = [0, 2, 3, 4]
poemBySequenceIndex[112] = '''B F G J'''
poemAuthors[112] = ''''''
#113: ䷰䷁ Revolution Receptive: 2 1 1 8  : S R2 R3 G3 : Idaj

#114: C D E♭ F :0 2 3 5: 1 2 ♭3 4: mi11(no 5 7)
namesBySequenceIndex[113] = '''Minor Tetrachord,Algerian (second octave)'''
namesBySequenceIndexNegative[113] = ''''''
semitonesBySequenceIndex[113] = [0, 2, 3, 5]
poemBySequenceIndex[113] = '''B F G K'''
poemAuthors[113] = ''''''
#114: ䷝䷁ Distanced Receptive: 2 1 2 7  : S R2 G2 M1 : Ifaj

#115: C D E♭ G♭ :0 2 3 6: 1 2 ♭3 ♭5: °(add 9)
namesBySequenceIndex[114] = '''Gypsy Tetrachord'''
namesBySequenceIndexNegative[114] = ''''''
semitonesBySequenceIndex[114] = [0, 2, 3, 6]
poemBySequenceIndex[114] = '''B F G L'''
poemAuthors[114] = ''''''
#115: ䷶䷗ Abounding Returning: 2 1 3 6  : S R2 G2 M2 : IbAj

#116: C D E♭ G :0 2 3 7: 1 2 ♭3 5: mi(add 9)
namesBySequenceIndex[115] = ''''''
namesBySequenceIndexNegative[115] = ''''''
semitonesBySequenceIndex[115] = [0, 2, 3, 7]
poemBySequenceIndex[115] = '''B F G M'''
poemAuthors[115] = ''''''
#116: ䷶䷆ Abounding Leading: 2 1 4 5  : S R2 G2 P : Ibej

#117: C D E♭ G♯ :0 2 3 8: 1 2 ♭3 ♯5: mi(♯5 add 9)
namesBySequenceIndex[116] = '''''' #All-In 5,All Interval Tetrachord 5
namesBySequenceIndexNegative[116] = ''''''
semitonesBySequenceIndex[116] = [0, 2, 3, 8]
poemBySequenceIndex[116] = '''B F G N'''
poemAuthors[116] = ''''''
#117: ䷶䷎ Abounding Humble: 2 1 5 4  : S R2 G2 D1 : Ibij

#118: C D E♭ A :0 2 3 9: 1 2 ♭3 6: mi6/9(no 5)
namesBySequenceIndex[117] = ''''''
namesBySequenceIndexNegative[117] = ''''''
semitonesBySequenceIndex[117] = [0, 2, 3, 9]
poemBySequenceIndex[117] = '''B F G P'''
poemAuthors[117] = ''''''
#118: ䷶䷏ Abounding Confident: 2 1 6 3  : S R2 G2 D2 : Ibab

#119: C D E♭ B♭ :0 2 3 10: 1 2 ♭3 ♭7: mi9(no 5)
namesBySequenceIndex[118] = ''''''
namesBySequenceIndexNegative[118] = ''''''
semitonesBySequenceIndex[118] = [0, 2, 3, 10]
poemBySequenceIndex[118] = '''B F G R'''
poemAuthors[118] = ''''''
#119: ䷶䷇ Abounding Comparing: 2 1 7 2  : S R2 G2 N2 : Ibak

#120: C D E♭ B :0 2 3 11: 1 2 ♭3 7: mima9(no 5)
namesBySequenceIndex[119] = ''''''
namesBySequenceIndexNegative[119] = ''''''
semitonesBySequenceIndex[119] = [0, 2, 3, 11]
poemBySequenceIndex[119] = '''B F G S'''
poemAuthors[119] = ''''''
#120: ䷶䷖ Abounding Stripping: 2 1 8 1  : S R2 G2 N3 : Ibal

#121: C D E F :0 2 4 5: 1 2 3 4: 11(no 5 7)
namesBySequenceIndex[120] = '''Major Tetrachord'''
namesBySequenceIndexNegative[120] = ''''''
semitonesBySequenceIndex[120] = [0, 2, 4, 5]
poemBySequenceIndex[120] = '''B F J K'''
poemAuthors[120] = ''''''
#121: ䷤䷁ Family Receptive: 2 2 1 7  : S R2 G3 M1 : Imaj

#122: C D E G♭ :0 2 4 6: 1 2 3 ♭5: ma(♭5 add 9)
namesBySequenceIndex[121] = '''Lydian Tetrachord'''
namesBySequenceIndexNegative[121] = ''''''
semitonesBySequenceIndex[121] = [0, 2, 4, 6]
poemBySequenceIndex[121] = '''B F J L'''
poemAuthors[121] = ''''''
#122: ䷾䷗ Completed Returning: 2 2 2 6  : S R2 G3 M2 : IkAj

#123: C D E G :0 2 4 7: 1 2 3 5: ma(add 9)
namesBySequenceIndex[122] = '''Mu Major,Wood Chord'''
namesBySequenceIndexNegative[122] = ''''''
semitonesBySequenceIndex[122] = [0, 2, 4, 7]
poemBySequenceIndex[122] = '''B F J M'''
poemAuthors[122] = ''''''
#123: ䷾䷆ Completed Leading: 2 2 3 5  : S R2 G3 P : Ikej

#124: C D E G♯ :0 2 4 8: 1 2 3 ♯5: ma(♯5 add 9)
namesBySequenceIndex[123] = '''Mixolyric'''
namesBySequenceIndexNegative[123] = ''''''
semitonesBySequenceIndex[123] = [0, 2, 4, 8]
poemBySequenceIndex[123] = '''B F J N'''
poemAuthors[123] = ''''''
#124: ䷾䷎ Completed Humble: 2 2 4 4  : S R2 G3 D1 : Ikij

#125: C D E A :0 2 4 9: 1 2 3 6: 6/9(no 5)
namesBySequenceIndex[124] = ''''''
namesBySequenceIndexNegative[124] = ''''''
semitonesBySequenceIndex[124] = [0, 2, 4, 9]
poemBySequenceIndex[124] = '''B F J P'''
poemAuthors[124] = ''''''
#125: ䷾䷏ Completed Confident: 2 2 5 3  : S R2 G3 D2 : Ikab

#126: C D E B♭ :0 2 4 10: 1 2 3 ♭7: 9(no 5)
namesBySequenceIndex[125] = ''''''
namesBySequenceIndexNegative[125] = ''''''
semitonesBySequenceIndex[125] = [0, 2, 4, 10]
poemBySequenceIndex[125] = '''B F J R'''
poemAuthors[125] = ''''''
#126: ䷾䷇ Completed Comparing: 2 2 6 2  : S R2 G3 N2 : Ikak

#127: C D E B :0 2 4 11: 1 2 3 7: ma9(no 5)
namesBySequenceIndex[126] = ''''''
namesBySequenceIndexNegative[126] = ''''''
semitonesBySequenceIndex[126] = [0, 2, 4, 11]
poemBySequenceIndex[126] = '''B F J S'''
poemAuthors[126] = ''''''
#127: ䷾䷖ Completed Stripping: 2 2 7 1  : S R2 G3 N3 : Ikal

#128: C D F G♭ :0 2 5 6: 1 2 4 ♭5: sus4(♭5 add 9)
namesBySequenceIndex[127] = '''''' #All-In 6,All Interval Tetrachord 6
namesBySequenceIndexNegative[127] = ''''''
semitonesBySequenceIndex[127] = [0, 2, 5, 6]
poemBySequenceIndex[127] = '''B F K L'''
poemAuthors[127] = ''''''
#128: ䷕䷗ Adorning Returning: 2 3 1 6  : S R2 M1 M2 : IlAj

#129: C D F G :0 2 5 7: 1 2 4 5: sus4(add 9)
namesBySequenceIndex[128] = '''Quartal Chord,Genus Primum'''
namesBySequenceIndexNegative[128] = ''''''
semitonesBySequenceIndex[128] = [0, 2, 5, 7]
poemBySequenceIndex[128] = '''B F K M'''
poemAuthors[128] = ''''''
#129: ䷕䷆ Adorning Leading: 2 3 2 5  : S R2 M1 P : Ilej

#130: C D F G♯ :0 2 5 8: 1 2 4 ♯5: sus4(♯5 add 9)
namesBySequenceIndex[129] = '''Zoptic'''
namesBySequenceIndexNegative[129] = ''''''
semitonesBySequenceIndex[129] = [0, 2, 5, 8]
poemBySequenceIndex[129] = '''B F K N'''
poemAuthors[129] = ''''''
#130: ䷕䷎ Adorning Humble: 2 3 3 4  : S R2 M1 D1 : Ilij

#131: C D F A :0 2 5 9: 1 2 4 6: 6/9sus4(no 5)
namesBySequenceIndex[130] = '''Rothic'''
namesBySequenceIndexNegative[130] = ''''''
semitonesBySequenceIndex[130] = [0, 2, 5, 9]
poemBySequenceIndex[130] = '''B F K P'''
poemAuthors[130] = ''''''
#131: ䷕䷏ Adorning Confident: 2 3 4 3  : S R2 M1 D2 : Ilab

#132: C D F B♭ :0 2 5 10: 1 2 4 ♭7: 9sus4(no 5)
namesBySequenceIndex[131] = '''Mu Two'''
namesBySequenceIndexNegative[131] = ''''''
semitonesBySequenceIndex[131] = [0, 2, 5, 10]
poemBySequenceIndex[131] = '''B F K R'''
poemAuthors[131] = ''''''
#132: ䷕䷇ Adorning Comparing: 2 3 5 2  : S R2 M1 N2 : Ilak

#133: C D F B :0 2 5 11: 1 2 4 7: ma9sus4(no 5)
namesBySequenceIndex[132] = ''''''
namesBySequenceIndexNegative[132] = ''''''
semitonesBySequenceIndex[132] = [0, 2, 5, 11]
poemBySequenceIndex[132] = '''B F K S'''
poemAuthors[132] = ''''''
#133: ䷕䷖ Adorning Stripping: 2 3 6 1  : S R2 M1 N3 : Ilal

#134: C D F♯ G :0 2 6 7: 1 2 ♯4 5: sus2(add ♯11)
namesBySequenceIndex[133] = ''''''
namesBySequenceIndexNegative[133] = ''''''
semitonesBySequenceIndex[133] = [0, 2, 6, 7]
poemBySequenceIndex[133] = '''B F L M'''
poemAuthors[133] = ''''''
#134: ䷣䷒ Eclipse Arrived: 2 4 1 5  : S R2 M2 P : IjEj

#135: C D F♯ G♯ :0 2 6 8: 1 2 ♯4 ♯5: sus2(♯5 add ♯11)
namesBySequenceIndex[134] = '''French Sixth,Dadic'''
namesBySequenceIndexNegative[134] = ''''''
semitonesBySequenceIndex[134] = [0, 2, 6, 8]
poemBySequenceIndex[134] = '''B F L N'''
poemAuthors[134] = ''''''
#135: ䷣䷣ Eclipse Eclipse : 2 4 2 4  : S R2 M2 D1 : IjIj

#136: C D G♭ A :0 2 6 9: 1 2 ♭5 6: 6sus2(♭5)
namesBySequenceIndex[135] = '''Eporaic'''
namesBySequenceIndexNegative[135] = ''''''
semitonesBySequenceIndex[135] = [0, 2, 6, 9]
poemBySequenceIndex[135] = '''B F L P'''
poemAuthors[135] = ''''''
#136: ䷣䷲ Eclipse Thunder: 2 4 3 3  : S R2 M2 D2 : IjAb

#137: C D G♭ B♭ :0 2 6 10: 1 2 ♭5 ♭7: 7sus2(♭5)
namesBySequenceIndex[136] = '''Lydic'''
namesBySequenceIndexNegative[136] = ''''''
semitonesBySequenceIndex[136] = [0, 2, 6, 10]
poemBySequenceIndex[136] = '''B F L R'''
poemAuthors[136] = ''''''
#137: ䷣䷂ Eclipse Sprouting: 2 4 4 2  : S R2 M2 N2 : IjAk

#138: C D G♭ B :0 2 6 11: 1 2 ♭5 7: ma7sus2(♭5)
namesBySequenceIndex[137] = ''''''
namesBySequenceIndexNegative[137] = ''''''
semitonesBySequenceIndex[137] = [0, 2, 6, 11]
poemBySequenceIndex[137] = '''B F L S'''
poemAuthors[137] = ''''''
#138: ䷣䷚ Eclipse Nourishment: 2 4 5 1  : S R2 M2 N3 : IjAl

#139: C D G A♭ :0 2 7 8: 1 2 5 ♭6: sus2(add ♭6)
namesBySequenceIndex[138] = ''''''
namesBySequenceIndexNegative[138] = ''''''
semitonesBySequenceIndex[138] = [0, 2, 7, 8]
poemBySequenceIndex[138] = '''B F M N'''
poemAuthors[138] = ''''''
#139: ䷣䷭ Eclipse Ascending: 2 5 1 4  : S R2 P D1 : Ijoj

#140: C D G A :0 2 7 9: 1 2 5 6: 6sus2
namesBySequenceIndex[139] = ''''''
namesBySequenceIndexNegative[139] = ''''''
semitonesBySequenceIndex[139] = [0, 2, 7, 9]
poemBySequenceIndex[139] = '''B F M P'''
poemAuthors[139] = ''''''
#140: ䷣䷧ Eclipse Deliverance: 2 5 2 3  : S R2 P D2 : Ijeb

#141: C D G B♭ :0 2 7 10: 1 2 5 ♭7: 7sus2
namesBySequenceIndex[140] = ''''''
namesBySequenceIndexNegative[140] = ''''''
semitonesBySequenceIndex[140] = [0, 2, 7, 10]
poemBySequenceIndex[140] = '''B F M R'''
poemAuthors[140] = ''''''
#141: ䷣䷜ Eclipse Chasm: 2 5 3 2  : S R2 P N2 : Ijek

#142: C D G B :0 2 7 11: 1 2 5 7: ma7sus2
namesBySequenceIndex[141] = ''''''
namesBySequenceIndexNegative[141] = ''''''
semitonesBySequenceIndex[141] = [0, 2, 7, 11]
poemBySequenceIndex[141] = '''B F M S'''
poemAuthors[141] = ''''''
#142: ䷣䷃ Eclipse Naïve: 2 5 4 1  : S R2 P N3 : Ijel

#143: C D G♯ A :0 2 8 9: 1 2 ♯5 6: 6sus2(♯5)
namesBySequenceIndex[142] = ''''''
namesBySequenceIndexNegative[142] = ''''''
semitonesBySequenceIndex[142] = [0, 2, 8, 9]
poemBySequenceIndex[142] = '''B F N P'''
poemAuthors[142] = ''''''
#143: ䷣䷽ Eclipse Detail: 2 6 1 3  : S R2 D1 N1 : Ijib

#144: C D G♯ B♭ :0 2 8 10: 1 2 ♯5 ♭7: 7sus2(♯5)
namesBySequenceIndex[143] = ''''''
namesBySequenceIndexNegative[143] = ''''''
semitonesBySequenceIndex[143] = [0, 2, 8, 10]
poemBySequenceIndex[143] = '''B F N R'''
poemAuthors[143] = ''''''
#144: ䷣䷦ Eclipse Limping: 2 6 2 2  : S R2 D1 N2 : Ijik

#145: C D G♯ B :0 2 8 11: 1 2 ♯5 7: ma7sus2(♯5)
namesBySequenceIndex[144] = ''''''
namesBySequenceIndexNegative[144] = ''''''
semitonesBySequenceIndex[144] = [0, 2, 8, 11]
poemBySequenceIndex[144] = '''B F N S'''
poemAuthors[144] = ''''''
#145: ䷣䷳ Eclipse Stillness: 2 6 3 1  : S R2 D1 N3 : Ijil

#146: C D A B♭ :0 2 9 10: 1 2 6 ♭7: 7sus2(no 5 add 13)
namesBySequenceIndex[145] = ''''''
namesBySequenceIndexNegative[145] = ''''''
semitonesBySequenceIndex[145] = [0, 2, 9, 10]
poemBySequenceIndex[145] = '''B F P R'''
poemAuthors[145] = ''''''
#146: ䷣䷬ Eclipse Clustering: 2 7 1 2  : S R2 D2 N2 : Ijad

#147: C D A B :0 2 9 11: 1 2 6 7: ma7sus2(no 5 add 13)
namesBySequenceIndex[146] = ''''''
namesBySequenceIndexNegative[146] = ''''''
semitonesBySequenceIndex[146] = [0, 2, 9, 11]
poemBySequenceIndex[146] = '''B F P S'''
poemAuthors[146] = ''''''
#147: ䷣䷢ Eclipse Prospering: 2 7 2 1  : S R2 D2 N3 : Ijaf

#148: C D A♯ B :0 2 10 11: 1 2 ♯6 7: ma7sus2(no 5 add ♭7)
namesBySequenceIndex[147] = ''''''
namesBySequenceIndexNegative[147] = ''''''
semitonesBySequenceIndex[147] = [0, 2, 10, 11]
poemBySequenceIndex[147] = '''B F R S'''
poemAuthors[147] = ''''''
#148: ䷣䷓ Eclipse Seeing: 2 8 1 1  : S R2 D3 N3 : Ijam

#149: C D♯ E F :0 3 4 5: 1 ♯2 3 4: ma(no 5 add ♯9 11)
namesBySequenceIndex[148] = ''''''
namesBySequenceIndexNegative[148] = ''''''
semitonesBySequenceIndex[148] = [0, 3, 4, 5]
poemBySequenceIndex[148] = '''B G J K'''
poemAuthors[148] = ''''''
#149: ䷘䷁ Integrity Receptive: 3 1 1 7  : S R3 G3 M1 : Agaj

#150: C D♯ E G♭ :0 3 4 6: 1 ♯2 3 ♭5: ma(♭5 add ♯9)
namesBySequenceIndex[149] = ''''''
namesBySequenceIndexNegative[149] = ''''''
semitonesBySequenceIndex[149] = [0, 3, 4, 6]
poemBySequenceIndex[149] = '''B G J L'''
poemAuthors[149] = ''''''
#150: ䷐䷗ Following Returning: 3 1 2 6  : S R3 G3 M2 : AdAj

#151: C D♯ E G :0 3 4 7: 1 ♯2 3 5: ma(add ♯9)
namesBySequenceIndex[150] = ''''''
namesBySequenceIndexNegative[150] = ''''''
semitonesBySequenceIndex[150] = [0, 3, 4, 7]
poemBySequenceIndex[150] = '''B G J M'''
poemAuthors[150] = ''''''
#151: ䷐䷆ Following Leading: 3 1 3 5  : S R3 G3 P : Adej

#152: C D♯ E G♯ :0 3 4 8: 1 ♯2 3 ♯5: ma(♯5 add ♯9)
namesBySequenceIndex[151] = '''Lanic'''
namesBySequenceIndexNegative[151] = ''''''
semitonesBySequenceIndex[151] = [0, 3, 4, 8]
poemBySequenceIndex[151] = '''B G J N'''
poemAuthors[151] = ''''''
#152: ䷐䷎ Following Humble: 3 1 4 4  : S R3 G3 D1 : Adij

#153: C D♯ E A :0 3 4 9: 1 ♯2 3 6: 6(no 5 add ♯9)
namesBySequenceIndex[152] = ''''''
namesBySequenceIndexNegative[152] = ''''''
semitonesBySequenceIndex[152] = [0, 3, 4, 9]
poemBySequenceIndex[152] = '''B G J P'''
poemAuthors[152] = ''''''
#153: ䷐䷏ Following Confident: 3 1 5 3  : S R3 G3 D2 : Adab

#154: C D♯ E B♭ :0 3 4 10: 1 ♯2 3 ♭7: 7(no 5 add ♯9)
namesBySequenceIndex[153] = '''The Jimi'''
namesBySequenceIndexNegative[153] = ''''''
semitonesBySequenceIndex[153] = [0, 3, 4, 10]
poemBySequenceIndex[153] = '''B G J R'''
poemAuthors[153] = ''''''
#154: ䷐䷇ Following Comparing: 3 1 6 2  : S R3 G3 N2 : Adak

#155: C D♯ E B :0 3 4 11: 1 ♯2 3 7: ma7(no 5 add ♯9)
namesBySequenceIndex[154] = ''''''
namesBySequenceIndexNegative[154] = ''''''
semitonesBySequenceIndex[154] = [0, 3, 4, 11]
poemBySequenceIndex[154] = '''B G J S'''
poemAuthors[154] = ''''''
#155: ䷐䷖ Following Stripping: 3 1 7 1  : S R3 G3 N3 : Adal

#156: C E♭ F G♭ :0 3 5 6: 1 ♭3 4 ♭5: °(add 11)
namesBySequenceIndex[155] = ''''''
namesBySequenceIndexNegative[155] = ''''''
semitonesBySequenceIndex[155] = [0, 3, 5, 6]
poemBySequenceIndex[155] = '''B G K L'''
poemAuthors[155] = ''''''
#156: ䷔䷗ Biting Returning: 3 2 1 6  : S G2 M1 M2 : AfAj

#157: C E♭ F G :0 3 5 7: 1 ♭3 4 5: mi(add 11)
namesBySequenceIndex[156] = ''''''
namesBySequenceIndexNegative[156] = ''''''
semitonesBySequenceIndex[156] = [0, 3, 5, 7]
poemBySequenceIndex[156] = '''B G K M'''
poemAuthors[156] = ''''''
#157: ䷔䷆ Biting Leading: 3 2 2 5  : S G2 M1 P : Afej

#158: C E♭ F G♯ :0 3 5 8: 1 ♭3 4 ♯5: mi(♯5 add 11)
namesBySequenceIndex[157] = '''Mynic'''
namesBySequenceIndexNegative[157] = ''''''
semitonesBySequenceIndex[157] = [0, 3, 5, 8]
poemBySequenceIndex[157] = '''B G K N'''
poemAuthors[157] = ''''''
#158: ䷔䷎ Biting Humble: 3 2 3 4  : S G2 M1 D1 : Afij

#159: C E♭ F A :0 3 5 9: 1 ♭3 4 6: mi6(no 5 add 11)
namesBySequenceIndex[158] = '''Rothaic'''
namesBySequenceIndexNegative[158] = ''''''
semitonesBySequenceIndex[158] = [0, 3, 5, 9]
poemBySequenceIndex[158] = '''B G K P'''
poemAuthors[158] = ''''''
#159: ䷔䷏ Biting Confident: 3 2 4 3  : S G2 M1 D2 : Afab

#160: C E♭ F B♭ :0 3 5 10: 1 ♭3 4 ♭7: mi11(♯9 no 5)
namesBySequenceIndex[159] = ''''''
namesBySequenceIndexNegative[159] = ''''''
semitonesBySequenceIndex[159] = [0, 3, 5, 10]
poemBySequenceIndex[159] = '''B G K R'''
poemAuthors[159] = ''''''
#160: ䷔䷇ Biting Comparing: 3 2 5 2  : S G2 M1 N2 : Afak

#161: C E♭ F B :0 3 5 11: 1 ♭3 4 7: mima11(♯9 no 5)
namesBySequenceIndex[160] = ''''''
namesBySequenceIndexNegative[160] = ''''''
semitonesBySequenceIndex[160] = [0, 3, 5, 11]
poemBySequenceIndex[160] = '''B G K S'''
poemAuthors[160] = ''''''
#161: ䷔䷖ Biting Stripping: 3 2 6 1  : S G2 M1 N3 : Afal

#162: C E♭ F♯ G :0 3 6 7: 1 ♭3 ♯4 5: mi(add ♯11)
namesBySequenceIndex[161] = ''''''
namesBySequenceIndexNegative[161] = ''''''
semitonesBySequenceIndex[161] = [0, 3, 6, 7]
poemBySequenceIndex[161] = '''B G L M'''
poemAuthors[161] = ''''''
#162: ䷲䷒ Thunder Arrived: 3 3 1 5  : S G2 M2 P : AbEj

#163: C E♭ F♯ G♯ :0 3 6 8: 1 ♭3 ♯4 ♯5: °(add ♭6)
namesBySequenceIndex[162] = '''German Sixth,Mynaic'''
namesBySequenceIndexNegative[162] = ''''''
semitonesBySequenceIndex[162] = [0, 3, 6, 8]
poemBySequenceIndex[162] = '''B G L N'''
poemAuthors[162] = ''''''
#163: ䷲䷣ Thunder Eclipse: 3 3 2 4  : S G2 M2 D1 : AbIj

#164: C E♭ G♭ A :0 3 6 9: 1 ♭3 ♭5 6: °7
namesBySequenceIndex[163] = '''Fully Diminished,Phrynic'''
namesBySequenceIndexNegative[163] = ''''''
semitonesBySequenceIndex[163] = [0, 3, 6, 9]
poemBySequenceIndex[163] = '''B G L P'''
poemAuthors[163] = ''''''
#164: ䷲䷲ Thunder Thunder : 3 3 3 3  : S G2 M2 D2 : AbAb

#165: C E♭ G♭ B♭ :0 3 6 10: 1 ♭3 ♭5 ♭7: ⌀7
namesBySequenceIndex[164] = '''Aeraphic,Half Diminished Chord'''
namesBySequenceIndexNegative[164] = ''''''
semitonesBySequenceIndex[164] = [0, 3, 6, 10]
poemBySequenceIndex[164] = '''B G L R'''
poemAuthors[164] = ''''''
#165: ䷲䷂ Thunder Sprouting: 3 3 4 2  : S G2 M2 N2 : AbAk

#166: C E♭ G♭ B :0 3 6 11: 1 ♭3 ♭5 7: °ma7
namesBySequenceIndex[165] = ''''''
namesBySequenceIndexNegative[165] = ''''''
semitonesBySequenceIndex[165] = [0, 3, 6, 11]
poemBySequenceIndex[165] = '''B G L S'''
poemAuthors[165] = ''''''
#166: ䷲䷚ Thunder Nourishment: 3 3 5 1  : S G2 M2 N3 : AbAl

#167: C E♭ G A♭ :0 3 7 8: 1 ♭3 5 ♭6: mi(add ♭6)
namesBySequenceIndex[166] = '''Lothic,Ward'''
namesBySequenceIndexNegative[166] = ''''''
semitonesBySequenceIndex[166] = [0, 3, 7, 8]
poemBySequenceIndex[166] = '''B G M N'''
poemAuthors[166] = ''''''
#167: ䷲䷭ Thunder Ascending: 3 4 1 4  : S G2 P D1 : Aboj

#168: C E♭ G A :0 3 7 9: 1 ♭3 5 6: mi6
namesBySequenceIndex[167] = '''Byptic,Major Pentatonic ♭3'''
namesBySequenceIndexNegative[167] = ''''''
semitonesBySequenceIndex[167] = [0, 3, 7, 9]
poemBySequenceIndex[167] = '''B G M P'''
poemAuthors[167] = ''''''
#168: ䷲䷧ Thunder Deliverance: 3 4 2 3  : S G2 P D2 : Abeb

#169: C E♭ G B♭ :0 3 7 10: 1 ♭3 5 ♭7: mi7
namesBySequenceIndex[168] = '''Bi Yu,Eporic'''
namesBySequenceIndexNegative[168] = ''''''
semitonesBySequenceIndex[168] = [0, 3, 7, 10]
poemBySequenceIndex[168] = '''B G M R'''
poemAuthors[168] = ''''''
#169: ䷲䷜ Thunder Chasm: 3 4 3 2  : S G2 P N2 : Abek

#170: C E♭ G B :0 3 7 11: 1 ♭3 5 7: mima7
namesBySequenceIndex[169] = '''Dygic'''
namesBySequenceIndexNegative[169] = ''''''
semitonesBySequenceIndex[169] = [0, 3, 7, 11]
poemBySequenceIndex[169] = '''B G M S'''
poemAuthors[169] = ''''''
#170: ䷲䷃ Thunder Naïve: 3 4 4 1  : S G2 P N3 : Abel

#171: C E♭ G♯ A :0 3 8 9: 1 ♭3 ♯5 6: mi6(♯5)
namesBySequenceIndex[170] = ''''''
namesBySequenceIndexNegative[170] = ''''''
semitonesBySequenceIndex[170] = [0, 3, 8, 9]
poemBySequenceIndex[170] = '''B G N P'''
poemAuthors[170] = ''''''
#171: ䷲䷽ Thunder Detail: 3 5 1 3  : S G2 D1 N1 : Abib

#172: C E♭ G♯ B♭ :0 3 8 10: 1 ♭3 ♯5 ♭7: mi7(♯5)
namesBySequenceIndex[171] = '''Mu Minor'''
namesBySequenceIndexNegative[171] = ''''''
semitonesBySequenceIndex[171] = [0, 3, 8, 10]
poemBySequenceIndex[171] = '''B G N R'''
poemAuthors[171] = ''''''
#172: ䷲䷦ Thunder Limping: 3 5 2 2  : S G2 D1 N2 : Abik

#173: C E♭ G♯ B :0 3 8 11: 1 ♭3 ♯5 7: mima7(♯5)
namesBySequenceIndex[172] = ''''''
namesBySequenceIndexNegative[172] = ''''''
semitonesBySequenceIndex[172] = [0, 3, 8, 11]
poemBySequenceIndex[172] = '''B G N S'''
poemAuthors[172] = ''''''
#173: ䷲䷳ Thunder Stillness: 3 5 3 1  : S G2 D1 N3 : Abil

#174: C E♭ A B♭ :0 3 9 10: 1 ♭3 6 ♭7: mi7(no 5 add 13)
namesBySequenceIndex[173] = ''''''
namesBySequenceIndexNegative[173] = ''''''
semitonesBySequenceIndex[173] = [0, 3, 9, 10]
poemBySequenceIndex[173] = '''B G P R'''
poemAuthors[173] = ''''''
#174: ䷲䷬ Thunder Clustering: 3 6 1 2  : S G2 D2 N2 : Abad

#175: C E♭ A B :0 3 9 11: 1 ♭3 6 7: mima7(no 5 add 13)
namesBySequenceIndex[174] = ''''''
namesBySequenceIndexNegative[174] = ''''''
semitonesBySequenceIndex[174] = [0, 3, 9, 11]
poemBySequenceIndex[174] = '''B G P S'''
poemAuthors[174] = ''''''
#175: ䷲䷢ Thunder Prospering: 3 6 2 1  : S G2 D2 N3 : Abaf

#176: C E♭ A♯ B :0 3 10 11: 1 ♭3 ♯6 7: mima7(no 5 add ♭7)
namesBySequenceIndex[175] = ''''''
namesBySequenceIndexNegative[175] = ''''''
semitonesBySequenceIndex[175] = [0, 3, 10, 11]
poemBySequenceIndex[175] = '''B G R S'''
poemAuthors[175] = ''''''
#176: ䷲䷓ Thunder Seeing: 3 7 1 1  : S G2 D3 N3 : Abam

#177: C E F G♭ :0 4 5 6: 1 3 4 ♭5: ma(♭5 add 11)
namesBySequenceIndex[176] = ''''''
namesBySequenceIndexNegative[176] = ''''''
semitonesBySequenceIndex[176] = [0, 4, 5, 6]
poemBySequenceIndex[176] = '''B J K L'''
poemAuthors[176] = ''''''
#177: ䷩䷗ Augmenting Returning: 4 1 1 6  : S G3 M1 M2 : AmAj

#178: C E F G :0 4 5 7: 1 3 4 5: ma(add 11)
namesBySequenceIndex[177] = ''''''
namesBySequenceIndexNegative[177] = ''''''
semitonesBySequenceIndex[177] = [0, 4, 5, 7]
poemBySequenceIndex[177] = '''B J K M'''
poemAuthors[177] = ''''''
#178: ䷩䷆ Augmenting Leading: 4 1 2 5  : S G3 M1 P : Amej

#179: C E F G♯ :0 4 5 8: 1 3 4 ♯5: ma(♯5 add 11)
namesBySequenceIndex[178] = '''Gonic'''
namesBySequenceIndexNegative[178] = ''''''
semitonesBySequenceIndex[178] = [0, 4, 5, 8]
poemBySequenceIndex[178] = '''B J K N'''
poemAuthors[178] = ''''''
#179: ䷩䷎ Augmenting Humble: 4 1 3 4  : S G3 M1 D1 : Amij

#180: C E F A :0 4 5 9: 1 3 4 6: 6(no 5 add 11)
namesBySequenceIndex[179] = '''Phratic'''
namesBySequenceIndexNegative[179] = ''''''
semitonesBySequenceIndex[179] = [0, 4, 5, 9]
poemBySequenceIndex[179] = '''B J K P'''
poemAuthors[179] = ''''''
#180: ䷩䷏ Augmenting Confident: 4 1 4 3  : S G3 M1 D2 : Amab

#181: C E F B♭ :0 4 5 10: 1 3 4 ♭7: 7(no 5 add 11)
namesBySequenceIndex[180] = ''''''
namesBySequenceIndexNegative[180] = ''''''
semitonesBySequenceIndex[180] = [0, 4, 5, 10]
poemBySequenceIndex[180] = '''B J K R'''
poemAuthors[180] = ''''''
#181: ䷩䷇ Augmenting Comparing: 4 1 5 2  : S G3 M1 N2 : Amak

#182: C E F B :0 4 5 11: 1 3 4 7: ma7(no 5 add 11)
namesBySequenceIndex[181] = ''''''
namesBySequenceIndexNegative[181] = ''''''
semitonesBySequenceIndex[181] = [0, 4, 5, 11]
poemBySequenceIndex[181] = '''B J K S'''
poemAuthors[181] = ''''''
#182: ䷩䷖ Augmenting Stripping: 4 1 6 1  : S G3 M1 N3 : Amal

#183: C E F♯ G :0 4 6 7: 1 3 ♯4 5: ma(add ♯11)
namesBySequenceIndex[182] = ''''''
namesBySequenceIndexNegative[182] = ''''''
semitonesBySequenceIndex[182] = [0, 4, 6, 7]
poemBySequenceIndex[182] = '''B J L M'''
poemAuthors[182] = ''''''
#183: ䷂䷒ Sprouting Arrived: 4 2 1 5  : S G3 M2 P : AkEj

#184: C E F♯ G♯ :0 4 6 8: 1 3 ♯4 ♯5: ma(♯5 add ♯11)
namesBySequenceIndex[183] = '''Koptic'''
namesBySequenceIndexNegative[183] = ''''''
semitonesBySequenceIndex[183] = [0, 4, 6, 8]
poemBySequenceIndex[183] = '''B J L N'''
poemAuthors[183] = ''''''
#184: ䷂䷣ Sprouting Eclipse: 4 2 2 4  : S G3 M2 D1 : AkIj

#185: C E G♭ A :0 4 6 9: 1 3 ♭5 6: 6(♭5)
namesBySequenceIndex[184] = '''Saric'''
namesBySequenceIndexNegative[184] = ''''''
semitonesBySequenceIndex[184] = [0, 4, 6, 9]
poemBySequenceIndex[184] = '''B J L P'''
poemAuthors[184] = ''''''
#185: ䷂䷲ Sprouting Thunder: 4 2 3 3  : S G3 M2 D2 : AkAb

#186: C E G♭ B♭ :0 4 6 10: 1 3 ♭5 ♭7: 7(♭5)
namesBySequenceIndex[185] = '''Stathic'''
namesBySequenceIndexNegative[185] = ''''''
semitonesBySequenceIndex[185] = [0, 4, 6, 10]
poemBySequenceIndex[185] = '''B J L R'''
poemAuthors[185] = ''''''
#186: ䷂䷂ Sprouting Sprouting : 4 2 4 2  : S G3 M2 N2 : AkAk

#187: C E G♭ B :0 4 6 11: 1 3 ♭5 7: ma7(♭5)
namesBySequenceIndex[186] = ''''''
namesBySequenceIndexNegative[186] = ''''''
semitonesBySequenceIndex[186] = [0, 4, 6, 11]
poemBySequenceIndex[186] = '''B J L S'''
poemAuthors[186] = ''''''
#187: ䷂䷚ Sprouting Nourishment: 4 2 5 1  : S G3 M2 N3 : AkAl

#188: C E G A♭ :0 4 7 8: 1 3 5 ♭6: ma(add ♭6)
namesBySequenceIndex[187] = '''Epogic'''
namesBySequenceIndexNegative[187] = ''''''
semitonesBySequenceIndex[187] = [0, 4, 7, 8]
poemBySequenceIndex[187] = '''B J M N'''
poemAuthors[187] = ''''''
#188: ䷂䷭ Sprouting Ascending: 4 3 1 4  : S G3 P D1 : Akoj

#189: C E G A :0 4 7 9: 1 3 5 6: 6
namesBySequenceIndex[188] = '''Epathic'''
namesBySequenceIndexNegative[188] = ''''''
semitonesBySequenceIndex[188] = [0, 4, 7, 9]
poemBySequenceIndex[188] = '''B J M P'''
poemAuthors[188] = ''''''
#189: ䷂䷧ Sprouting Deliverance: 4 3 2 3  : S G3 P D2 : Akeb

#190: C E G B♭ :0 4 7 10: 1 3 5 ♭7: 7
namesBySequenceIndex[189] = '''Dominant Chord,Raga Mahathi,Epathaic'''
namesBySequenceIndexNegative[189] = ''''''
semitonesBySequenceIndex[189] = [0, 4, 7, 10]
poemBySequenceIndex[189] = '''B J M R'''
poemAuthors[189] = ''''''
#190: ䷂䷜ Sprouting Chasm: 4 3 3 2  : S G3 P N2 : Akek

#191: C E G B :0 4 7 11: 1 3 5 7: ma7
namesBySequenceIndex[190] = '''Thaptic'''
namesBySequenceIndexNegative[190] = ''''''
semitonesBySequenceIndex[190] = [0, 4, 7, 11]
poemBySequenceIndex[190] = '''B J M S'''
poemAuthors[190] = ''''''
#191: ䷂䷃ Sprouting Naïve: 4 3 4 1  : S G3 P N3 : Akel

#192: C E G♯ A :0 4 8 9: 1 3 ♯5 6: 6(♯5)
namesBySequenceIndex[191] = '''Aeoloric'''
namesBySequenceIndexNegative[191] = ''''''
semitonesBySequenceIndex[191] = [0, 4, 8, 9]
poemBySequenceIndex[191] = '''B J N P'''
poemAuthors[191] = ''''''
#192: ䷂䷽ Sprouting Detail: 4 4 1 3  : S G3 D1 N1 : Akib

#193: C E G♯ B♭ :0 4 8 10: 1 3 ♯5 ♭7: 7(♯5)
namesBySequenceIndex[192] = '''Aeolic'''
namesBySequenceIndexNegative[192] = ''''''
semitonesBySequenceIndex[192] = [0, 4, 8, 10]
poemBySequenceIndex[192] = '''B J N R'''
poemAuthors[192] = ''''''
#193: ䷂䷦ Sprouting Limping: 4 4 2 2  : S G3 D1 N2 : Akik

#194: C E G♯ B :0 4 8 11: 1 3 ♯5 7: ma7(♯5)
namesBySequenceIndex[193] = '''Zyphic'''
namesBySequenceIndexNegative[193] = ''''''
semitonesBySequenceIndex[193] = [0, 4, 8, 11]
poemBySequenceIndex[193] = '''B J N S'''
poemAuthors[193] = ''''''
#194: ䷂䷳ Sprouting Stillness: 4 4 3 1  : S G3 D1 N3 : Akil

#195: C E A B♭ :0 4 9 10: 1 3 6 ♭7: 7(no 5 add 13)
namesBySequenceIndex[194] = ''''''
namesBySequenceIndexNegative[194] = ''''''
semitonesBySequenceIndex[194] = [0, 4, 9, 10]
poemBySequenceIndex[194] = '''B J P R'''
poemAuthors[194] = ''''''
#195: ䷂䷬ Sprouting Clustering: 4 5 1 2  : S G3 D2 N2 : Akad

#196: C E A B :0 4 9 11: 1 3 6 7: ma7(no 5 add 13)
namesBySequenceIndex[195] = ''''''
namesBySequenceIndexNegative[195] = ''''''
semitonesBySequenceIndex[195] = [0, 4, 9, 11]
poemBySequenceIndex[195] = '''B J P S'''
poemAuthors[195] = ''''''
#196: ䷂䷢ Sprouting Prospering: 4 5 2 1  : S G3 D2 N3 : Akaf

#197: C E A♯ B :0 4 10 11: 1 3 ♯6 7: ma7(no 5 add ♭7)
namesBySequenceIndex[196] = ''''''
namesBySequenceIndexNegative[196] = ''''''
semitonesBySequenceIndex[196] = [0, 4, 10, 11]
poemBySequenceIndex[196] = '''B J R S'''
poemAuthors[196] = ''''''
#197: ䷂䷓ Sprouting Seeing: 4 6 1 1  : S G3 D3 N3 : Akam

#198: C F F♯ G :0 5 6 7: 1 4 ♯4 5: sus4(add ♯11)
namesBySequenceIndex[197] = '''Dream Chord'''
namesBySequenceIndexNegative[197] = ''''''
semitonesBySequenceIndex[197] = [0, 5, 6, 7]
poemBySequenceIndex[197] = '''B K L M'''
poemAuthors[197] = ''''''
#198: ䷚䷒ Nourishment Arrived: 5 1 1 5  : S M1 M2 P : AlEj

#199: C F G♭ A♭ :0 5 6 8: 1 4 ♭5 ♭6: sus4(♯5 add ♯11)
namesBySequenceIndex[198] = ''''''
namesBySequenceIndexNegative[198] = ''''''
semitonesBySequenceIndex[198] = [0, 5, 6, 8]
poemBySequenceIndex[198] = '''B K L N'''
poemAuthors[198] = ''''''
#199: ䷚䷣ Nourishment Eclipse: 5 1 2 4  : S M1 M2 D1 : AlIj

#200: C F G♭ A :0 5 6 9: 1 4 ♭5 6: 6sus4(♭5)
namesBySequenceIndex[199] = ''''''
namesBySequenceIndexNegative[199] = ''''''
semitonesBySequenceIndex[199] = [0, 5, 6, 9]
poemBySequenceIndex[199] = '''B K L P'''
poemAuthors[199] = ''''''
#200: ䷚䷲ Nourishment Thunder: 5 1 3 3  : S M1 M2 D2 : AlAb

#201: C F G♭ B♭ :0 5 6 10: 1 4 ♭5 ♭7: 7sus4(♭5)
namesBySequenceIndex[200] = ''''''
namesBySequenceIndexNegative[200] = ''''''
semitonesBySequenceIndex[200] = [0, 5, 6, 10]
poemBySequenceIndex[200] = '''B K L R'''
poemAuthors[200] = ''''''
#201: ䷚䷂ Nourishment Sprouting: 5 1 4 2  : S M1 M2 N2 : AlAk

#202: C F G♭ B :0 5 6 11: 1 4 ♭5 7: ma7sus4(♭5)
namesBySequenceIndex[201] = '''Distance Model 5 1'''
namesBySequenceIndexNegative[201] = ''''''
semitonesBySequenceIndex[201] = [0, 5, 6, 11]
poemBySequenceIndex[201] = '''B K L S'''
poemAuthors[201] = ''''''
#202: ䷚䷚ Nourishment Nourishment : 5 1 5 1  : S M1 M2 N3 : AlAl

#203: C F G A♭ :0 5 7 8: 1 4 5 ♭6: sus4(add ♭6)
namesBySequenceIndex[202] = ''''''
namesBySequenceIndexNegative[202] = ''''''
semitonesBySequenceIndex[202] = [0, 5, 7, 8]
poemBySequenceIndex[202] = '''B K M N'''
poemAuthors[202] = ''''''
#203: ䷚䷭ Nourishment Ascending: 5 2 1 4  : S M1 P D1 : Aloj

#204: C F G A :0 5 7 9: 1 4 5 6: 6sus4
namesBySequenceIndex[203] = '''Mu Four'''
namesBySequenceIndexNegative[203] = ''''''
semitonesBySequenceIndex[203] = [0, 5, 7, 9]
poemBySequenceIndex[203] = '''B K M P'''
poemAuthors[203] = ''''''
#204: ䷚䷧ Nourishment Deliverance: 5 2 2 3  : S M1 P D2 : Aleb

#205: C F G B♭ :0 5 7 10: 1 4 5 ♭7: 7sus4
namesBySequenceIndex[204] = ''''''
namesBySequenceIndexNegative[204] = ''''''
semitonesBySequenceIndex[204] = [0, 5, 7, 10]
poemBySequenceIndex[204] = '''B K M R'''
poemAuthors[204] = ''''''
#205: ䷚䷜ Nourishment Chasm: 5 2 3 2  : S M1 P N2 : Alek

#206: C F G B :0 5 7 11: 1 4 5 7: ma7sus4
namesBySequenceIndex[205] = ''''''
namesBySequenceIndexNegative[205] = ''''''
semitonesBySequenceIndex[205] = [0, 5, 7, 11]
poemBySequenceIndex[205] = '''B K M S'''
poemAuthors[205] = ''''''
#206: ䷚䷃ Nourishment Naïve: 5 2 4 1  : S M1 P N3 : Alel

#207: C F G♯ A :0 5 8 9: 1 4 ♯5 6: 6sus4(♯5)
namesBySequenceIndex[206] = ''''''
namesBySequenceIndexNegative[206] = ''''''
semitonesBySequenceIndex[206] = [0, 5, 8, 9]
poemBySequenceIndex[206] = '''B K N P'''
poemAuthors[206] = ''''''
#207: ䷚䷽ Nourishment Detail: 5 3 1 3  : S M1 D1 N1 : Alib

#208: C F G♯ B♭ :0 5 8 10: 1 4 ♯5 ♭7: 7sus4(♯5)
namesBySequenceIndex[207] = ''''''
namesBySequenceIndexNegative[207] = ''''''
semitonesBySequenceIndex[207] = [0, 5, 8, 10]
poemBySequenceIndex[207] = '''B K N R'''
poemAuthors[207] = ''''''
#208: ䷚䷦ Nourishment Limping: 5 3 2 2  : S M1 D1 N2 : Alik

#209: C F G♯ B :0 5 8 11: 1 4 ♯5 7: ma7sus4(♯5)
namesBySequenceIndex[208] = ''''''
namesBySequenceIndexNegative[208] = ''''''
semitonesBySequenceIndex[208] = [0, 5, 8, 11]
poemBySequenceIndex[208] = '''B K N S'''
poemAuthors[208] = ''''''
#209: ䷚䷳ Nourishment Stillness: 5 3 3 1  : S M1 D1 N3 : Alil

#210: C F A B♭ :0 5 9 10: 1 4 6 ♭7: 7sus4(no 5 add 13)
namesBySequenceIndex[209] = ''''''
namesBySequenceIndexNegative[209] = ''''''
semitonesBySequenceIndex[209] = [0, 5, 9, 10]
poemBySequenceIndex[209] = '''B K P R'''
poemAuthors[209] = ''''''
#210: ䷚䷬ Nourishment Clustering: 5 4 1 2  : S M1 D2 N2 : Alad

#211: C F A B :0 5 9 11: 1 4 6 7: ma7sus4(no 5 add 13)
namesBySequenceIndex[210] = ''''''
namesBySequenceIndexNegative[210] = ''''''
semitonesBySequenceIndex[210] = [0, 5, 9, 11]
poemBySequenceIndex[210] = '''B K P S'''
poemAuthors[210] = ''''''
#211: ䷚䷢ Nourishment Prospering: 5 4 2 1  : S M1 D2 N3 : Alaf

#212: C F A♯ B :0 5 10 11: 1 4 ♯6 7: ma7sus4(no 5 add ♭7)
namesBySequenceIndex[211] = ''''''
namesBySequenceIndexNegative[211] = ''''''
semitonesBySequenceIndex[211] = [0, 5, 10, 11]
poemBySequenceIndex[211] = '''B K R S'''
poemAuthors[211] = ''''''
#212: ䷚䷓ Nourishment Seeing: 5 5 1 1  : S M1 D3 N3 : Alam

#213: C F♯ G A♭ :0 6 7 8: 1 ♯4 5 ♭6: sus♯4(add ♭6)
namesBySequenceIndex[212] = ''''''
namesBySequenceIndexNegative[212] = ''''''
semitonesBySequenceIndex[212] = [0, 6, 7, 8]
poemBySequenceIndex[212] = '''B L M N'''
poemAuthors[212] = ''''''
#213: ䷗䷊ Returning Peace: 6 1 1 4  : S M2 P D1 : AjOj

#214: C F♯ G A :0 6 7 9: 1 ♯4 5 6: 6sus♯4
namesBySequenceIndex[213] = ''''''
namesBySequenceIndexNegative[213] = ''''''
semitonesBySequenceIndex[213] = [0, 6, 7, 9]
poemBySequenceIndex[213] = '''B L M P'''
poemAuthors[213] = ''''''
#214: ䷗䷵ Returning Betrothing: 6 1 2 3  : S M2 P D2 : AjEb

#215: C F♯ G B♭ :0 6 7 10: 1 ♯4 5 ♭7: 7sus♯4
namesBySequenceIndex[214] = ''''''
namesBySequenceIndexNegative[214] = ''''''
semitonesBySequenceIndex[214] = [0, 6, 7, 10]
poemBySequenceIndex[214] = '''B L M R'''
poemAuthors[214] = ''''''
#215: ䷗䷻ Returning Composed: 6 1 3 2  : S M2 P N2 : AjEk

#216: C F♯ G B :0 6 7 11: 1 ♯4 5 7: ma7sus♯4
namesBySequenceIndex[215] = ''''''
namesBySequenceIndexNegative[215] = ''''''
semitonesBySequenceIndex[215] = [0, 6, 7, 11]
poemBySequenceIndex[215] = '''B L M S'''
poemAuthors[215] = ''''''
#216: ䷗䷨ Returning Diminishing: 6 1 4 1  : S M2 P N3 : AjEl

#217: C F♯ G♯ A :0 6 8 9: 1 ♯4 ♯5 6: 6sus♯4(♯5)
namesBySequenceIndex[216] = ''''''
namesBySequenceIndexNegative[216] = ''''''
semitonesBySequenceIndex[216] = [0, 6, 8, 9]
poemBySequenceIndex[216] = '''B L N P'''
poemAuthors[216] = ''''''
#217: ䷗䷶ Returning Abounding: 6 2 1 3  : S M2 D1 N1 : AjIb

#218: C F♯ G♯ B♭ :0 6 8 10: 1 ♯4 ♯5 ♭7: 7sus♯4(♯5)
namesBySequenceIndex[217] = ''''''
namesBySequenceIndexNegative[217] = ''''''
semitonesBySequenceIndex[217] = [0, 6, 8, 10]
poemBySequenceIndex[217] = '''B L N R'''
poemAuthors[217] = ''''''
#218: ䷗䷾ Returning Completed: 6 2 2 2  : S M2 D1 N2 : AjIk

#219: C F♯ G♯ B :0 6 8 11: 1 ♯4 ♯5 7: ma7sus♯4(♯5)
namesBySequenceIndex[218] = ''''''
namesBySequenceIndexNegative[218] = ''''''
semitonesBySequenceIndex[218] = [0, 6, 8, 11]
poemBySequenceIndex[218] = '''B L N S'''
poemAuthors[218] = ''''''
#219: ䷗䷕ Returning Adorning: 6 2 3 1  : S M2 D1 N3 : AjIl

#220: C G♭ A B♭ :0 6 9 10: 1 ♭5 6 ♭7: 7(♭5 no 3 add 13)
namesBySequenceIndex[219] = ''''''
namesBySequenceIndexNegative[219] = ''''''
semitonesBySequenceIndex[219] = [0, 6, 9, 10]
poemBySequenceIndex[219] = '''B L P R'''
poemAuthors[219] = ''''''
#220: ䷗䷐ Returning Following: 6 3 1 2  : S M2 D2 N2 : AjAd

#221: C G♭ A B :0 6 9 11: 1 ♭5 6 7: ma7(♭5 no 3 add 13)
namesBySequenceIndex[220] = ''''''
namesBySequenceIndexNegative[220] = ''''''
semitonesBySequenceIndex[220] = [0, 6, 9, 11]
poemBySequenceIndex[220] = '''B L P S'''
poemAuthors[220] = ''''''
#221: ䷗䷔ Returning Biting: 6 3 2 1  : S M2 D2 N3 : AjAf

#222: C G♭ A♯ B :0 6 10 11: 1 ♭5 ♯6 7: ma7(♭5 no 3 add ♭7)
namesBySequenceIndex[221] = ''''''
namesBySequenceIndexNegative[221] = ''''''
semitonesBySequenceIndex[221] = [0, 6, 10, 11]
poemBySequenceIndex[221] = '''B L R S'''
poemAuthors[221] = ''''''
#222: ䷗䷩ Returning Augmenting: 6 4 1 1  : S M2 D3 N3 : AjAm

#223: C G A♭ A :0 7 8 9: 1 5 ♭6 6: 5(add ♭6 6)
namesBySequenceIndex[222] = ''''''
namesBySequenceIndexNegative[222] = ''''''
semitonesBySequenceIndex[222] = [0, 7, 8, 9]
poemBySequenceIndex[222] = '''B M N P'''
poemAuthors[222] = ''''''
#223: ䷗䷟ Returning Persevering: 7 1 1 3  : S P D1 N1 : Ajob

#224: C G A♭ B♭ :0 7 8 10: 1 5 ♭6 ♭7: 7(no 3 add ♭13)
namesBySequenceIndex[223] = ''''''
namesBySequenceIndexNegative[223] = ''''''
semitonesBySequenceIndex[223] = [0, 7, 8, 10]
poemBySequenceIndex[223] = '''B M N R'''
poemAuthors[223] = ''''''
#224: ䷗䷯ Returning Source: 7 1 2 2  : S P D1 N2 : Ajok

#225: C G A♭ B :0 7 8 11: 1 5 ♭6 7: ma7(no 3 add ♭13)
namesBySequenceIndex[224] = ''''''
namesBySequenceIndexNegative[224] = ''''''
semitonesBySequenceIndex[224] = [0, 7, 8, 11]
poemBySequenceIndex[224] = '''B M N S'''
poemAuthors[224] = ''''''
#225: ䷗䷑ Returning Correcting: 7 1 3 1  : S P D1 N3 : Ajol

#226: C G A B♭ :0 7 9 10: 1 5 6 ♭7: 7(no 3 add 13)
namesBySequenceIndex[225] = ''''''
namesBySequenceIndexNegative[225] = ''''''
semitonesBySequenceIndex[225] = [0, 7, 9, 10]
poemBySequenceIndex[225] = '''B M P R'''
poemAuthors[225] = ''''''
#226: ䷗䷮ Returning Confining: 7 2 1 2  : S P D2 N2 : Ajed

#227: C G A B :0 7 9 11: 1 5 6 7: ma7(no 3 add 13)
namesBySequenceIndex[226] = ''''''
namesBySequenceIndexNegative[226] = ''''''
semitonesBySequenceIndex[226] = [0, 7, 9, 11]
poemBySequenceIndex[226] = '''B M P S'''
poemAuthors[226] = ''''''
#227: ䷗䷿ Returning Unfinished: 7 2 2 1  : S P D2 N3 : Ajef

#228: C G A♯ B :0 7 10 11: 1 5 ♯6 7: 5(add ♭7 7)
namesBySequenceIndex[227] = ''''''
namesBySequenceIndexNegative[227] = ''''''
semitonesBySequenceIndex[227] = [0, 7, 10, 11]
poemBySequenceIndex[227] = '''B M R S'''
poemAuthors[227] = ''''''
#228: ䷗䷺ Returning Dispersing: 7 3 1 1  : S P D3 N3 : Ajem

#229: C G♯ A B♭ :0 8 9 10: 1 ♯5 6 ♭7: 7(♯5 no 3 add 13)
namesBySequenceIndex[228] = ''''''
namesBySequenceIndexNegative[228] = ''''''
semitonesBySequenceIndex[228] = [0, 8, 9, 10]
poemBySequenceIndex[228] = '''B N P R'''
poemAuthors[228] = ''''''
#229: ䷗䷞ Returning Conjoining: 8 1 1 2  : S D1 D2 N2 : Ajid

#230: C G♯ A B :0 8 9 11: 1 ♯5 6 7: ma7(♯5 no 3 add 13)
namesBySequenceIndex[229] = ''''''
namesBySequenceIndexNegative[229] = ''''''
semitonesBySequenceIndex[229] = [0, 8, 9, 11]
poemBySequenceIndex[229] = '''B N P S'''
poemAuthors[229] = ''''''
#230: ䷗䷷ Returning Wanderer: 8 1 2 1  : S D1 D2 N3 : Ajif

#231: C G♯ A♯ B :0 8 10 11: 1 ♯5 ♯6 7: ma7(♯5 no 3 add ♭7)
namesBySequenceIndex[230] = ''''''
namesBySequenceIndexNegative[230] = ''''''
semitonesBySequenceIndex[230] = [0, 8, 10, 11]
poemBySequenceIndex[230] = '''B N R S'''
poemAuthors[230] = ''''''
#231: ䷗䷴ Returning Infiltrating: 8 2 1 1  : S D1 N2 N3 : Ajim

#232: C A B♭ B :0 9 10 11: 1 6 ♭7 7: ma7(no 3 5 add ♭7 13)
namesBySequenceIndex[231] = ''''''
namesBySequenceIndexNegative[231] = ''''''
semitonesBySequenceIndex[231] = [0, 9, 10, 11]
poemBySequenceIndex[231] = '''B P R S'''
poemAuthors[231] = ''''''
#232: ䷗䷋ Returning Obstruction: 9 1 1 1  : S D2 N2 N3 : Ajag

#233: C D♭ D E♭ E :0 1 2 3 4: 1 ♭2 2 ♭3 3: ma(no 5 add ♭9 9 ♯9)
namesBySequenceIndex[232] = ''''''
namesBySequenceIndexNegative[232] = ''''''
semitonesBySequenceIndex[232] = [0, 1, 2, 3, 4]
poemBySequenceIndex[232] = '''B D F G J'''
poemAuthors[232] = ''''''
#233: ䷪䷁ Decision Receptive: 1 1 1 1 8  : S R1 R2 G2 G3 : Odaj

#234: C D♭ D E♭ F :0 1 2 3 5: 1 ♭2 2 ♭3 4: mi11(no 5 7 add ♭9)
namesBySequenceIndex[233] = ''''''
namesBySequenceIndexNegative[233] = ''''''
semitonesBySequenceIndex[233] = [0, 1, 2, 3, 5]
poemBySequenceIndex[233] = '''B D F G K'''
poemAuthors[233] = ''''''
#234: ䷍䷁ Resevoir Receptive: 1 1 1 2 7  : S R1 R2 G2 M1 : Ofaj

#235: C D♭ D E♭ G♭ :0 1 2 3 6: 1 ♭2 2 ♭3 ♭5: °(add ♭9 9)
namesBySequenceIndex[234] = ''''''
namesBySequenceIndexNegative[234] = ''''''
semitonesBySequenceIndex[234] = [0, 1, 2, 3, 6]
poemBySequenceIndex[234] = '''B D F G L'''
poemAuthors[234] = ''''''
#235: ䷡䷗ Might Returning: 1 1 1 3 6  : S R1 R2 G2 M2 : ObAj

#236: C D♭ D E♭ G :0 1 2 3 7: 1 ♭2 2 ♭3 5: mi(add ♭9 9)
namesBySequenceIndex[235] = ''''''
namesBySequenceIndexNegative[235] = ''''''
semitonesBySequenceIndex[235] = [0, 1, 2, 3, 7]
poemBySequenceIndex[235] = '''B D F G M'''
poemAuthors[235] = ''''''
#236: ䷡䷆ Might Leading: 1 1 1 4 5  : S R1 R2 G2 P : Obej

#237: C D♭ D E♭ G♯ :0 1 2 3 8: 1 ♭2 2 ♭3 ♯5: mi(♯5 add ♭9 9)
namesBySequenceIndex[236] = ''''''
namesBySequenceIndexNegative[236] = ''''''
semitonesBySequenceIndex[236] = [0, 1, 2, 3, 8]
poemBySequenceIndex[236] = '''B D F G N'''
poemAuthors[236] = ''''''
#237: ䷡䷎ Might Humble: 1 1 1 5 4  : S R1 R2 G2 D1 : Obij

#238: C D♭ D E♭ A :0 1 2 3 9: 1 ♭2 2 ♭3 6: mi6/9(no 5 add ♭9)
namesBySequenceIndex[237] = ''''''
namesBySequenceIndexNegative[237] = ''''''
semitonesBySequenceIndex[237] = [0, 1, 2, 3, 9]
poemBySequenceIndex[237] = '''B D F G P'''
poemAuthors[237] = ''''''
#238: ䷡䷏ Might Confident: 1 1 1 6 3  : S R1 R2 G2 D2 : Obab

#239: C D♭ D E♭ B♭ :0 1 2 3 10: 1 ♭2 2 ♭3 ♭7: mi9(no 5 add ♭9)
namesBySequenceIndex[238] = ''''''
namesBySequenceIndexNegative[238] = ''''''
semitonesBySequenceIndex[238] = [0, 1, 2, 3, 10]
poemBySequenceIndex[238] = '''B D F G R'''
poemAuthors[238] = ''''''
#239: ䷡䷇ Might Comparing: 1 1 1 7 2  : S R1 R2 G2 N2 : Obak

#240: C D♭ D E♭ B :0 1 2 3 11: 1 ♭2 2 ♭3 7: mima9(no 5 add ♭9)
namesBySequenceIndex[239] = ''''''
namesBySequenceIndexNegative[239] = ''''''
semitonesBySequenceIndex[239] = [0, 1, 2, 3, 11]
poemBySequenceIndex[239] = '''B D F G S'''
poemAuthors[239] = ''''''
#240: ䷡䷖ Might Stripping: 1 1 1 8 1  : S R1 R2 G2 N3 : Obal

#241: C D♭ D E F :0 1 2 4 5: 1 ♭2 2 3 4: 11(no 5 7 add ♭9)
namesBySequenceIndex[240] = ''''''
namesBySequenceIndexNegative[240] = ''''''
semitonesBySequenceIndex[240] = [0, 1, 2, 4, 5]
poemBySequenceIndex[240] = '''B D F J K'''
poemAuthors[240] = ''''''
#241: ䷈䷁ Homegrown Receptive: 1 1 2 1 7  : S R1 R2 G3 M1 : Omaj

#242: C D♭ D E G♭ :0 1 2 4 6: 1 ♭2 2 3 ♭5: ma(♭5 add ♭9 9)
namesBySequenceIndex[241] = ''''''
namesBySequenceIndexNegative[241] = ''''''
semitonesBySequenceIndex[241] = [0, 1, 2, 4, 6]
poemBySequenceIndex[241] = '''B D F J L'''
poemAuthors[241] = ''''''
#242: ䷄䷗ Biding Returning: 1 1 2 2 6  : S R1 R2 G3 M2 : OkAj

#243: C D♭ D E G :0 1 2 4 7: 1 ♭2 2 3 5: ma(add ♭9 9)
namesBySequenceIndex[242] = '''Fibonacci (ascending)'''
namesBySequenceIndexNegative[242] = ''''''
semitonesBySequenceIndex[242] = [0, 1, 2, 4, 7]
poemBySequenceIndex[242] = '''B D F J M'''
poemAuthors[242] = ''''''
#243: ䷄䷆ Biding Leading: 1 1 2 3 5  : S R1 R2 G3 P : Okej

#244: C D♭ D E G♯ :0 1 2 4 8: 1 ♭2 2 3 ♯5: ma(♯5 add ♭9 9)
namesBySequenceIndex[243] = '''Poditonic'''
namesBySequenceIndexNegative[243] = ''''''
semitonesBySequenceIndex[243] = [0, 1, 2, 4, 8]
poemBySequenceIndex[243] = '''B D F J N'''
poemAuthors[243] = ''''''
#244: ䷄䷎ Biding Humble: 1 1 2 4 4  : S R1 R2 G3 D1 : Okij

#245: C D♭ D E A :0 1 2 4 9: 1 ♭2 2 3 6: 6/9(no 5 add ♭9)
namesBySequenceIndex[244] = ''''''
namesBySequenceIndexNegative[244] = ''''''
semitonesBySequenceIndex[244] = [0, 1, 2, 4, 9]
poemBySequenceIndex[244] = '''B D F J P'''
poemAuthors[244] = ''''''
#245: ䷄䷏ Biding Confident: 1 1 2 5 3  : S R1 R2 G3 D2 : Okab

#246: C D♭ D E B♭ :0 1 2 4 10: 1 ♭2 2 3 ♭7: 9(no 5 add ♭9)
namesBySequenceIndex[245] = ''''''
namesBySequenceIndexNegative[245] = ''''''
semitonesBySequenceIndex[245] = [0, 1, 2, 4, 10]
poemBySequenceIndex[245] = '''B D F J R'''
poemAuthors[245] = ''''''
#246: ䷄䷇ Biding Comparing: 1 1 2 6 2  : S R1 R2 G3 N2 : Okak

#247: C D♭ D E B :0 1 2 4 11: 1 ♭2 2 3 7: ma9(no 5 add ♭9)
namesBySequenceIndex[246] = ''''''
namesBySequenceIndexNegative[246] = ''''''
semitonesBySequenceIndex[246] = [0, 1, 2, 4, 11]
poemBySequenceIndex[246] = '''B D F J S'''
poemAuthors[246] = ''''''
#247: ䷄䷖ Biding Stripping: 1 1 2 7 1  : S R1 R2 G3 N3 : Okal

#248: C D♭ D F G♭ :0 1 2 5 6: 1 ♭2 2 4 ♭5: sus4(♭5 add ♭9 9)
namesBySequenceIndex[247] = ''''''
namesBySequenceIndexNegative[247] = ''''''
semitonesBySequenceIndex[247] = [0, 1, 2, 5, 6]
poemBySequenceIndex[247] = '''B D F K L'''
poemAuthors[247] = ''''''
#248: ䷙䷗ Cultivating Returning: 1 1 3 1 6  : S R1 G1 M1 M2 : OlAj

#249: C D♭ D F G :0 1 2 5 7: 1 ♭2 2 4 5: sus4(add ♭9 9)
namesBySequenceIndex[248] = ''''''
namesBySequenceIndexNegative[248] = ''''''
semitonesBySequenceIndex[248] = [0, 1, 2, 5, 7]
poemBySequenceIndex[248] = '''B D F K M'''
poemAuthors[248] = ''''''
#249: ䷙䷆ Cultivating Leading: 1 1 3 2 5  : S R1 G1 M1 P : Olej

#250: C D♭ D F G♯ :0 1 2 5 8: 1 ♭2 2 4 ♯5: sus4(♯5 add ♭9 9)
namesBySequenceIndex[249] = '''Gyritonic'''
namesBySequenceIndexNegative[249] = ''''''
semitonesBySequenceIndex[249] = [0, 1, 2, 5, 8]
poemBySequenceIndex[249] = '''B D F K N'''
poemAuthors[249] = ''''''
#250: ䷙䷎ Cultivating Humble: 1 1 3 3 4  : S R1 G1 M1 D1 : Olij

#251: C D♭ D F A :0 1 2 5 9: 1 ♭2 2 4 6: 6/9sus4(no 5 add ♭9)
namesBySequenceIndex[250] = '''Aeoloditonic'''
namesBySequenceIndexNegative[250] = ''''''
semitonesBySequenceIndex[250] = [0, 1, 2, 5, 9]
poemBySequenceIndex[250] = '''B D F K P'''
poemAuthors[250] = ''''''
#251: ䷙䷏ Cultivating Confident: 1 1 3 4 3  : S R1 G1 M1 D2 : Olab

#252: C D♭ D F B♭ :0 1 2 5 10: 1 ♭2 2 4 ♭7: 11sus♭2(no 5)
namesBySequenceIndex[251] = ''''''
namesBySequenceIndexNegative[251] = ''''''
semitonesBySequenceIndex[251] = [0, 1, 2, 5, 10]
poemBySequenceIndex[251] = '''B D F K R'''
poemAuthors[251] = ''''''
#252: ䷙䷇ Cultivating Comparing: 1 1 3 5 2  : S R1 G1 M1 N2 : Olak

#253: C D♭ D F B :0 1 2 5 11: 1 ♭2 2 4 7: ma11sus♭2(no 5)
namesBySequenceIndex[252] = ''''''
namesBySequenceIndexNegative[252] = ''''''
semitonesBySequenceIndex[252] = [0, 1, 2, 5, 11]
poemBySequenceIndex[252] = '''B D F K S'''
poemAuthors[252] = ''''''
#253: ䷙䷖ Cultivating Stripping: 1 1 3 6 1  : S R1 G1 M1 N3 : Olal

#254: C D♭ D F♯ G :0 1 2 6 7: 1 ♭2 2 ♯4 5: sus2(add ♭9 ♯11)
namesBySequenceIndex[253] = '''Raga Nabhomani'''
namesBySequenceIndexNegative[253] = ''''''
semitonesBySequenceIndex[253] = [0, 1, 2, 6, 7]
poemBySequenceIndex[253] = '''B D F L M'''
poemAuthors[253] = ''''''
#254: ䷊䷒ Peace Arrived: 1 1 4 1 5  : S R1 G1 M2 P : OjEj

#255: C D♭ D F♯ G♯ :0 1 2 6 8: 1 ♭2 2 ♯4 ♯5: sus2(♯5 add ♭9 ♯11)
namesBySequenceIndex[254] = '''Syptitonic'''
namesBySequenceIndexNegative[254] = ''''''
semitonesBySequenceIndex[254] = [0, 1, 2, 6, 8]
poemBySequenceIndex[254] = '''B D F L N'''
poemAuthors[254] = ''''''
#255: ䷊䷣ Peace Eclipse: 1 1 4 2 4  : S R1 G1 M2 D1 : OjIj

#256: C D♭ D G♭ A :0 1 2 6 9: 1 ♭2 2 ♭5 6: 6sus2(♭5 add ♭9)
namesBySequenceIndex[255] = '''Aeritonic'''
namesBySequenceIndexNegative[255] = ''''''
semitonesBySequenceIndex[255] = [0, 1, 2, 6, 9]
poemBySequenceIndex[255] = '''B D F L P'''
poemAuthors[255] = ''''''
#256: ䷊䷲ Peace Thunder: 1 1 4 3 3  : S R1 G1 M2 D2 : OjAb

#257: C D♭ D G♭ B♭ :0 1 2 6 10: 1 ♭2 2 ♭5 ♭7: 9sus♭2(♭5)
namesBySequenceIndex[256] = '''Phrythitonic'''
namesBySequenceIndexNegative[256] = ''''''
semitonesBySequenceIndex[256] = [0, 1, 2, 6, 10]
poemBySequenceIndex[256] = '''B D F L R'''
poemAuthors[256] = ''''''
#257: ䷊䷂ Peace Sprouting: 1 1 4 4 2  : S R1 G1 M2 N2 : OjAk

#258: C D♭ D G♭ B :0 1 2 6 11: 1 ♭2 2 ♭5 7: ma9sus♭2(♭5)
namesBySequenceIndex[257] = ''''''
namesBySequenceIndexNegative[257] = ''''''
semitonesBySequenceIndex[257] = [0, 1, 2, 6, 11]
poemBySequenceIndex[257] = '''B D F L S'''
poemAuthors[257] = ''''''
#258: ䷊䷚ Peace Nourishment: 1 1 4 5 1  : S R1 G1 M2 N3 : OjAl

#259: C D♭ D G A♭ :0 1 2 7 8: 1 ♭2 2 5 ♭6: sus2(add ♭6 ♭9)
namesBySequenceIndex[258] = ''''''
namesBySequenceIndexNegative[258] = ''''''
semitonesBySequenceIndex[258] = [0, 1, 2, 7, 8]
poemBySequenceIndex[258] = '''B D F M N'''
poemAuthors[258] = ''''''
#259: ䷊䷭ Peace Ascending: 1 1 5 1 4  : S R1 G1 P D1 : Ojoj

#260: C D♭ D G A :0 1 2 7 9: 1 ♭2 2 5 6: 6sus2(add ♭9)
namesBySequenceIndex[259] = ''''''
namesBySequenceIndexNegative[259] = ''''''
semitonesBySequenceIndex[259] = [0, 1, 2, 7, 9]
poemBySequenceIndex[259] = '''B D F M P'''
poemAuthors[259] = ''''''
#260: ䷊䷧ Peace Deliverance: 1 1 5 2 3  : S R1 G1 P D2 : Ojeb

#261: C D♭ D G B♭ :0 1 2 7 10: 1 ♭2 2 5 ♭7: 9sus♭2
namesBySequenceIndex[260] = ''''''
namesBySequenceIndexNegative[260] = ''''''
semitonesBySequenceIndex[260] = [0, 1, 2, 7, 10]
poemBySequenceIndex[260] = '''B D F M R'''
poemAuthors[260] = ''''''
#261: ䷊䷜ Peace Chasm: 1 1 5 3 2  : S R1 G1 P N2 : Ojek

#262: C D♭ D G B :0 1 2 7 11: 1 ♭2 2 5 7: ma9sus♭2
namesBySequenceIndex[261] = ''''''
namesBySequenceIndexNegative[261] = ''''''
semitonesBySequenceIndex[261] = [0, 1, 2, 7, 11]
poemBySequenceIndex[261] = '''B D F M S'''
poemAuthors[261] = ''''''
#262: ䷊䷃ Peace Naïve: 1 1 5 4 1  : S R1 G1 P N3 : Ojel

#263: C D♭ D G♯ A :0 1 2 8 9: 1 ♭2 2 ♯5 6: 6sus2(♯5 add ♭9)
namesBySequenceIndex[262] = '''Raga Putrika'''
namesBySequenceIndexNegative[262] = ''''''
semitonesBySequenceIndex[262] = [0, 1, 2, 8, 9]
poemBySequenceIndex[262] = '''B D F N P'''
poemAuthors[262] = ''''''
#263: ䷊䷽ Peace Detail: 1 1 6 1 3  : S R1 G1 D1 N1 : Ojib

#264: C D♭ D G♯ B♭ :0 1 2 8 10: 1 ♭2 2 ♯5 ♭7: 9sus♭2(♯5)
namesBySequenceIndex[263] = ''''''
namesBySequenceIndexNegative[263] = ''''''
semitonesBySequenceIndex[263] = [0, 1, 2, 8, 10]
poemBySequenceIndex[263] = '''B D F N R'''
poemAuthors[263] = ''''''
#264: ䷊䷦ Peace Limping: 1 1 6 2 2  : S R1 G1 D1 N2 : Ojik

#265: C D♭ D G♯ B :0 1 2 8 11: 1 ♭2 2 ♯5 7: ma9sus♭2(♯5)
namesBySequenceIndex[264] = '''Raga Kumarapriya'''
namesBySequenceIndexNegative[264] = ''''''
semitonesBySequenceIndex[264] = [0, 1, 2, 8, 11]
poemBySequenceIndex[264] = '''B D F N S'''
poemAuthors[264] = ''''''
#265: ䷊䷳ Peace Stillness: 1 1 6 3 1  : S R1 G1 D1 N3 : Ojil

#266: C D♭ D A B♭ :0 1 2 9 10: 1 ♭2 2 6 ♭7: 9sus♭2(no 5 add 13)
namesBySequenceIndex[265] = ''''''
namesBySequenceIndexNegative[265] = ''''''
semitonesBySequenceIndex[265] = [0, 1, 2, 9, 10]
poemBySequenceIndex[265] = '''B D F P R'''
poemAuthors[265] = ''''''
#266: ䷊䷬ Peace Clustering: 1 1 7 1 2  : S R1 G1 D2 N2 : Ojad

#267: C D♭ D A B :0 1 2 9 11: 1 ♭2 2 6 7: ma9sus♭2(no 5 add 13)
namesBySequenceIndex[266] = ''''''
namesBySequenceIndexNegative[266] = ''''''
semitonesBySequenceIndex[266] = [0, 1, 2, 9, 11]
poemBySequenceIndex[266] = '''B D F P S'''
poemAuthors[266] = ''''''
#267: ䷊䷢ Peace Prospering: 1 1 7 2 1  : S R1 G1 D2 N3 : Ojaf

#268: C D♭ D A♯ B :0 1 2 10 11: 1 ♭2 2 ♯6 7: ma9sus♭2(no 5 add ♭7)
namesBySequenceIndex[267] = ''''''
namesBySequenceIndexNegative[267] = ''''''
semitonesBySequenceIndex[267] = [0, 1, 2, 10, 11]
poemBySequenceIndex[267] = '''B D F R S'''
poemAuthors[267] = ''''''
#268: ䷊䷓ Peace Seeing: 1 1 8 1 1  : S R1 G1 D3 N3 : Ojam

#269: C D♭ D♯ E F :0 1 3 4 5: 1 ♭2 ♯2 3 4: ma(no 5 add ♭9 ♯9 11)
namesBySequenceIndex[268] = ''''''
namesBySequenceIndexNegative[268] = ''''''
semitonesBySequenceIndex[268] = [0, 1, 3, 4, 5]
poemBySequenceIndex[268] = '''B D G J K'''
poemAuthors[268] = ''''''
#269: ䷉䷁ Treading Receptive: 1 2 1 1 7  : S R1 R3 G3 M1 : Egaj

#270: C D♭ D♯ E G♭ :0 1 3 4 6: 1 ♭2 ♯2 3 ♭5: ma(♭5 add ♭9 ♯9)
namesBySequenceIndex[269] = ''''''
namesBySequenceIndexNegative[269] = ''''''
semitonesBySequenceIndex[269] = [0, 1, 3, 4, 6]
poemBySequenceIndex[269] = '''B D G J L'''
poemAuthors[269] = ''''''
#270: ䷹䷗ Joyous Returning: 1 2 1 2 6  : S R1 R3 G3 M2 : EdAj

#271: C D♭ D♯ E G :0 1 3 4 7: 1 ♭2 ♯2 3 5: ma(add ♭9 ♯9)
namesBySequenceIndex[270] = ''''''
namesBySequenceIndexNegative[270] = ''''''
semitonesBySequenceIndex[270] = [0, 1, 3, 4, 7]
poemBySequenceIndex[270] = '''B D G J M'''
poemAuthors[270] = ''''''
#271: ䷹䷆ Joyous Leading: 1 2 1 3 5  : S R1 R3 G3 P : Edej

#272: C D♭ D♯ E G♯ :0 1 3 4 8: 1 ♭2 ♯2 3 ♯5: ma(♯5 add ♭9 ♯9)
namesBySequenceIndex[271] = '''Farben Chord,Aerylitonic'''
namesBySequenceIndexNegative[271] = ''''''
semitonesBySequenceIndex[271] = [0, 1, 3, 4, 8]
poemBySequenceIndex[271] = '''B D G J N'''
poemAuthors[271] = ''''''
#272: ䷹䷎ Joyous Humble: 1 2 1 4 4  : S R1 R3 G3 D1 : Edij

#273: C D♭ D♯ E A :0 1 3 4 9: 1 ♭2 ♯2 3 6: 6(no 5 add ♭9 ♯9)
namesBySequenceIndex[272] = ''''''
namesBySequenceIndexNegative[272] = ''''''
semitonesBySequenceIndex[272] = [0, 1, 3, 4, 9]
poemBySequenceIndex[272] = '''B D G J P'''
poemAuthors[272] = ''''''
#273: ䷹䷏ Joyous Confident: 1 2 1 5 3  : S R1 R3 G3 D2 : Edab

#274: C D♭ D♯ E B♭ :0 1 3 4 10: 1 ♭2 ♯2 3 ♭7: 7(no 5 add ♭9 ♯9)
namesBySequenceIndex[273] = ''''''
namesBySequenceIndexNegative[273] = ''''''
semitonesBySequenceIndex[273] = [0, 1, 3, 4, 10]
poemBySequenceIndex[273] = '''B D G J R'''
poemAuthors[273] = ''''''
#274: ䷹䷇ Joyous Comparing: 1 2 1 6 2  : S R1 R3 G3 N2 : Edak

#275: C D♭ D♯ E B :0 1 3 4 11: 1 ♭2 ♯2 3 7: ma7(no 5 add ♭9 ♯9)
namesBySequenceIndex[274] = ''''''
namesBySequenceIndexNegative[274] = ''''''
semitonesBySequenceIndex[274] = [0, 1, 3, 4, 11]
poemBySequenceIndex[274] = '''B D G J S'''
poemAuthors[274] = ''''''
#275: ䷹䷖ Joyous Stripping: 1 2 1 7 1  : S R1 R3 G3 N3 : Edal

#276: C D♭ E♭ F G♭ :0 1 3 5 6: 1 ♭2 ♭3 4 ♭5: °(add ♭9 11)
namesBySequenceIndex[275] = ''''''
namesBySequenceIndexNegative[275] = ''''''
semitonesBySequenceIndex[275] = [0, 1, 3, 5, 6]
poemBySequenceIndex[275] = '''B D G K L'''
poemAuthors[275] = ''''''
#276: ䷥䷗ Polarising Returning: 1 2 2 1 6  : S R1 G2 M1 M2 : EfAj

#277: C D♭ E♭ F G :0 1 3 5 7: 1 ♭2 ♭3 4 5: mi(add ♭9 11)
namesBySequenceIndex[276] = ''''''
namesBySequenceIndexNegative[276] = ''''''
semitonesBySequenceIndex[276] = [0, 1, 3, 5, 7]
poemBySequenceIndex[276] = '''B D G K M'''
poemAuthors[276] = ''''''
#277: ䷥䷆ Polarising Leading: 1 2 2 2 5  : S R1 G2 M1 P : Efej

#278: C D♭ E♭ F G♯ :0 1 3 5 8: 1 ♭2 ♭3 4 ♯5: mi(♯5 add ♭9 11)
namesBySequenceIndex[277] = '''Raga Chitthakarshini,Phratonic'''
namesBySequenceIndexNegative[277] = ''''''
semitonesBySequenceIndex[277] = [0, 1, 3, 5, 8]
poemBySequenceIndex[277] = '''B D G K N'''
poemAuthors[277] = ''''''
#278: ䷥䷎ Polarising Humble: 1 2 2 3 4  : S R1 G2 M1 D1 : Efij

#279: C D♭ E♭ F A :0 1 3 5 9: 1 ♭2 ♭3 4 6: mi6(no 5 add ♭9 11)
namesBySequenceIndex[278] = '''Aeolycritonic'''
namesBySequenceIndexNegative[278] = ''''''
semitonesBySequenceIndex[278] = [0, 1, 3, 5, 9]
poemBySequenceIndex[278] = '''B D G K P'''
poemAuthors[278] = ''''''
#279: ䷥䷏ Polarising Confident: 1 2 2 4 3  : S R1 G2 M1 D2 : Efab

#280: C D♭ E♭ F B♭ :0 1 3 5 10: 1 ♭2 ♭3 4 ♭7: mi11(♭9 no 5)
namesBySequenceIndex[279] = ''''''
namesBySequenceIndexNegative[279] = ''''''
semitonesBySequenceIndex[279] = [0, 1, 3, 5, 10]
poemBySequenceIndex[279] = '''B D G K R'''
poemAuthors[279] = ''''''
#280: ䷥䷇ Polarising Comparing: 1 2 2 5 2  : S R1 G2 M1 N2 : Efak

#281: C D♭ E♭ F B :0 1 3 5 11: 1 ♭2 ♭3 4 7: mima11(♭9 no 5)
namesBySequenceIndex[280] = ''''''
namesBySequenceIndexNegative[280] = ''''''
semitonesBySequenceIndex[280] = [0, 1, 3, 5, 11]
poemBySequenceIndex[280] = '''B D G K S'''
poemAuthors[280] = ''''''
#281: ䷥䷖ Polarising Stripping: 1 2 2 6 1  : S R1 G2 M1 N3 : Efal

#282: C D♭ E♭ F♯ G :0 1 3 6 7: 1 ♭2 ♭3 ♯4 5: mi(add ♭9 ♯11)
namesBySequenceIndex[281] = ''''''
namesBySequenceIndexNegative[281] = ''''''
semitonesBySequenceIndex[281] = [0, 1, 3, 6, 7]
poemBySequenceIndex[281] = '''B D G L M'''
poemAuthors[281] = ''''''
#282: ䷵䷒ Betrothing Arrived: 1 2 3 1 5  : S R1 G2 M2 P : EbEj

#283: C D♭ E♭ F♯ G♯ :0 1 3 6 8: 1 ♭2 ♭3 ♯4 ♯5: °(add ♭6 ♭9)
namesBySequenceIndex[282] = '''Raga Chaya Todi,Byptitonic'''
namesBySequenceIndexNegative[282] = ''''''
semitonesBySequenceIndex[282] = [0, 1, 3, 6, 8]
poemBySequenceIndex[282] = '''B D G L N'''
poemAuthors[282] = ''''''
#283: ䷵䷣ Betrothing Eclipse: 1 2 3 2 4  : S R1 G2 M2 D1 : EbIj

#284: C D♭ E♭ G♭ A :0 1 3 6 9: 1 ♭2 ♭3 ♭5 6: °7(add ♭9)
namesBySequenceIndex[283] = '''Pathitonic'''
namesBySequenceIndexNegative[283] = ''''''
semitonesBySequenceIndex[283] = [0, 1, 3, 6, 9]
poemBySequenceIndex[283] = '''B D G L P'''
poemAuthors[283] = ''''''
#284: ䷵䷲ Betrothing Thunder: 1 2 3 3 3  : S R1 G2 M2 D2 : EbAb

#285: C D♭ E♭ G♭ B♭ :0 1 3 6 10: 1 ♭2 ♭3 ♭5 ♭7: ⌀7(add ♭9)
namesBySequenceIndex[284] = '''Dyritonic'''
namesBySequenceIndexNegative[284] = ''''''
semitonesBySequenceIndex[284] = [0, 1, 3, 6, 10]
poemBySequenceIndex[284] = '''B D G L R'''
poemAuthors[284] = ''''''
#285: ䷵䷂ Betrothing Sprouting: 1 2 3 4 2  : S R1 G2 M2 N2 : EbAk

#286: C D♭ E♭ G♭ B :0 1 3 6 11: 1 ♭2 ♭3 ♭5 7: °ma7(add ♭9)
namesBySequenceIndex[285] = ''''''
namesBySequenceIndexNegative[285] = ''''''
semitonesBySequenceIndex[285] = [0, 1, 3, 6, 11]
poemBySequenceIndex[285] = '''B D G L S'''
poemAuthors[285] = ''''''
#286: ䷵䷚ Betrothing Nourishment: 1 2 3 5 1  : S R1 G2 M2 N3 : EbAl

#287: C D♭ E♭ G A♭ :0 1 3 7 8: 1 ♭2 ♭3 5 ♭6: mi(add ♭6 ♭9)
namesBySequenceIndex[286] = '''Belinese,Pelog Pentatonic,Phrygian Pentatonic,Madenda Modern,Tezeta Minor (Ethiopia),Raga Bhupal Todi,Raga Bhupalam,Raga Ramkali 2,Dalitonic'''
namesBySequenceIndexNegative[286] = ''''''
semitonesBySequenceIndex[286] = [0, 1, 3, 7, 8]
poemBySequenceIndex[286] = '''B D G M N'''
poemAuthors[286] = ''''''
#287: ䷵䷭ Betrothing Ascending: 1 2 4 1 4  : S R1 G2 P D1 : Eboj

#288: C D♭ E♭ G A :0 1 3 7 9: 1 ♭2 ♭3 5 6: mi6(add ♭9)
namesBySequenceIndex[287] = '''Golitonic'''
namesBySequenceIndexNegative[287] = ''''''
semitonesBySequenceIndex[287] = [0, 1, 3, 7, 9]
poemBySequenceIndex[287] = '''B D G M P'''
poemAuthors[287] = ''''''
#288: ䷵䷧ Betrothing Deliverance: 1 2 4 2 3  : S R1 G2 P D2 : Ebeb

#289: C D♭ E♭ G B♭ :0 1 3 7 10: 1 ♭2 ♭3 5 ♭7: mi7(add ♭9)
namesBySequenceIndex[288] = '''Raga Rukmangi,Pagitonic'''
namesBySequenceIndexNegative[288] = ''''''
semitonesBySequenceIndex[288] = [0, 1, 3, 7, 10]
poemBySequenceIndex[288] = '''B D G M R'''
poemAuthors[288] = ''''''
#289: ䷵䷜ Betrothing Chasm: 1 2 4 3 2  : S R1 G2 P N2 : Ebek

#290: C D♭ E♭ G B :0 1 3 7 11: 1 ♭2 ♭3 5 7: mima7(add ♭9)
namesBySequenceIndex[289] = '''Ionothitonic'''
namesBySequenceIndexNegative[289] = ''''''
semitonesBySequenceIndex[289] = [0, 1, 3, 7, 11]
poemBySequenceIndex[289] = '''B D G M S'''
poemAuthors[289] = ''''''
#290: ䷵䷃ Betrothing Naïve: 1 2 4 4 1  : S R1 G2 P N3 : Ebel

#291: C D♭ E♭ G♯ A :0 1 3 8 9: 1 ♭2 ♭3 ♯5 6: mi6(♯5 add ♭9)
namesBySequenceIndex[290] = ''''''
namesBySequenceIndexNegative[290] = ''''''
semitonesBySequenceIndex[290] = [0, 1, 3, 8, 9]
poemBySequenceIndex[290] = '''B D G N P'''
poemAuthors[290] = ''''''
#291: ䷵䷽ Betrothing Detail: 1 2 5 1 3  : S R1 G2 D1 N1 : Ebib

#292: C D♭ E♭ G♯ B♭ :0 1 3 8 10: 1 ♭2 ♭3 ♯5 ♭7: mi7(♯5 add ♭9)
namesBySequenceIndex[291] = ''''''
namesBySequenceIndexNegative[291] = ''''''
semitonesBySequenceIndex[291] = [0, 1, 3, 8, 10]
poemBySequenceIndex[291] = '''B D G N R'''
poemAuthors[291] = ''''''
#292: ䷵䷦ Betrothing Limping: 1 2 5 2 2  : S R1 G2 D1 N2 : Ebik

#293: C D♭ E♭ G♯ B :0 1 3 8 11: 1 ♭2 ♭3 ♯5 7: mima7(♯5 add ♭9)
namesBySequenceIndex[292] = ''''''
namesBySequenceIndexNegative[292] = ''''''
semitonesBySequenceIndex[292] = [0, 1, 3, 8, 11]
poemBySequenceIndex[292] = '''B D G N S'''
poemAuthors[292] = ''''''
#293: ䷵䷳ Betrothing Stillness: 1 2 5 3 1  : S R1 G2 D1 N3 : Ebil

#294: C D♭ E♭ A B♭ :0 1 3 9 10: 1 ♭2 ♭3 6 ♭7: mi7(no 5 add ♭9 13)
namesBySequenceIndex[293] = ''''''
namesBySequenceIndexNegative[293] = ''''''
semitonesBySequenceIndex[293] = [0, 1, 3, 9, 10]
poemBySequenceIndex[293] = '''B D G P R'''
poemAuthors[293] = ''''''
#294: ䷵䷬ Betrothing Clustering: 1 2 6 1 2  : S R1 G2 D2 N2 : Ebad

#295: C D♭ E♭ A B :0 1 3 9 11: 1 ♭2 ♭3 6 7: mima7(no 5 add ♭9 13)
namesBySequenceIndex[294] = ''''''
namesBySequenceIndexNegative[294] = ''''''
semitonesBySequenceIndex[294] = [0, 1, 3, 9, 11]
poemBySequenceIndex[294] = '''B D G P S'''
poemAuthors[294] = ''''''
#295: ䷵䷢ Betrothing Prospering: 1 2 6 2 1  : S R1 G2 D2 N3 : Ebaf

#296: C D♭ E♭ A♯ B :0 1 3 10 11: 1 ♭2 ♭3 ♯6 7: mima7(no 5 add ♭7 ♭9)
namesBySequenceIndex[295] = ''''''
namesBySequenceIndexNegative[295] = ''''''
semitonesBySequenceIndex[295] = [0, 1, 3, 10, 11]
poemBySequenceIndex[295] = '''B D G R S'''
poemAuthors[295] = ''''''
#296: ䷵䷓ Betrothing Seeing: 1 2 7 1 1  : S R1 G2 D3 N3 : Ebam

#297: C D♭ E F G♭ :0 1 4 5 6: 1 ♭2 3 4 ♭5: ma(♭5 add ♭9 11)
namesBySequenceIndex[296] = '''Arkaik,Arkaik (Greece)'''
namesBySequenceIndexNegative[296] = ''''''
semitonesBySequenceIndex[296] = [0, 1, 4, 5, 6]
poemBySequenceIndex[296] = '''B D J K L'''
poemAuthors[296] = ''''''
#297: ䷼䷗ Truth Returning: 1 3 1 1 6  : S R1 G3 M1 M2 : EmAj

#298: C D♭ E F G :0 1 4 5 7: 1 ♭2 3 4 5: ma(add ♭9 11)
namesBySequenceIndex[297] = ''''''
namesBySequenceIndexNegative[297] = ''''''
semitonesBySequenceIndex[297] = [0, 1, 4, 5, 7]
poemBySequenceIndex[297] = '''B D J K M'''
poemAuthors[297] = ''''''
#298: ䷼䷆ Truth Leading: 1 3 1 2 5  : S R1 G3 M1 P : Emej

#299: C D♭ E F G♯ :0 1 4 5 8: 1 ♭2 3 4 ♯5: ma(♯5 add ♭9 11)
namesBySequenceIndex[298] = '''Syrian Pentatonic,Raga Megharanjani,Phrolitonic'''
namesBySequenceIndexNegative[298] = ''''''
semitonesBySequenceIndex[298] = [0, 1, 4, 5, 8]
poemBySequenceIndex[298] = '''B D J K N'''
poemAuthors[298] = ''''''
#299: ䷼䷎ Truth Humble: 1 3 1 3 4  : S R1 G3 M1 D1 : Emij

#300: C D♭ E F A :0 1 4 5 9: 1 ♭2 3 4 6: 6(no 5 add ♭9 11)
namesBySequenceIndex[299] = '''Thacritonic'''
namesBySequenceIndexNegative[299] = ''''''
semitonesBySequenceIndex[299] = [0, 1, 4, 5, 9]
poemBySequenceIndex[299] = '''B D J K P'''
poemAuthors[299] = ''''''
#300: ䷼䷏ Truth Confident: 1 3 1 4 3  : S R1 G3 M1 D2 : Emab

#301: C D♭ E F B♭ :0 1 4 5 10: 1 ♭2 3 4 ♭7: 11(♭9 no 5)
namesBySequenceIndex[300] = ''''''
namesBySequenceIndexNegative[300] = ''''''
semitonesBySequenceIndex[300] = [0, 1, 4, 5, 10]
poemBySequenceIndex[300] = '''B D J K R'''
poemAuthors[300] = ''''''
#301: ䷼䷇ Truth Comparing: 1 3 1 5 2  : S R1 G3 M1 N2 : Emak

#302: C D♭ E F B :0 1 4 5 11: 1 ♭2 3 4 7: ma11(♭9 no 5)
namesBySequenceIndex[301] = '''Raga Megharanji'''
namesBySequenceIndexNegative[301] = ''''''
semitonesBySequenceIndex[301] = [0, 1, 4, 5, 11]
poemBySequenceIndex[301] = '''B D J K S'''
poemAuthors[301] = ''''''
#302: ䷼䷖ Truth Stripping: 1 3 1 6 1  : S R1 G3 M1 N3 : Emal

#303: C D♭ E F♯ G :0 1 4 6 7: 1 ♭2 3 ♯4 5: ma(add ♭9 ♯11)
namesBySequenceIndex[302] = ''''''
namesBySequenceIndexNegative[302] = ''''''
semitonesBySequenceIndex[302] = [0, 1, 4, 6, 7]
poemBySequenceIndex[302] = '''B D J L M'''
poemAuthors[302] = ''''''
#303: ䷻䷒ Composed Arrived: 1 3 2 1 5  : S R1 G3 M2 P : EkEj

#304: C D♭ E F♯ G♯ :0 1 4 6 8: 1 ♭2 3 ♯4 ♯5: ma(♯5 add ♭9 ♯11)
namesBySequenceIndex[303] = '''Zaptitonic'''
namesBySequenceIndexNegative[303] = ''''''
semitonesBySequenceIndex[303] = [0, 1, 4, 6, 8]
poemBySequenceIndex[303] = '''B D J L N'''
poemAuthors[303] = ''''''
#304: ䷻䷣ Composed Eclipse: 1 3 2 2 4  : S R1 G3 M2 D1 : EkIj

#305: C D♭ E G♭ A :0 1 4 6 9: 1 ♭2 3 ♭5 6: 6(♭5 add ♭9)
namesBySequenceIndex[304] = '''Elektra Chord,Gamma Chord,Major Pentatonic ♭2 ♭5,Sogitonic'''
namesBySequenceIndexNegative[304] = ''''''
semitonesBySequenceIndex[304] = [0, 1, 4, 6, 9]
poemBySequenceIndex[304] = '''B D J L P'''
poemAuthors[304] = ''''''
#305: ䷻䷲ Composed Thunder: 1 3 2 3 3  : S R1 G3 M2 D2 : EkAb

#306: C D♭ E G♭ B♭ :0 1 4 6 10: 1 ♭2 3 ♭5 ♭7: 7(♭5 add ♭9)
namesBySequenceIndex[305] = '''Mogitonic'''
namesBySequenceIndexNegative[305] = ''''''
semitonesBySequenceIndex[305] = [0, 1, 4, 6, 10]
poemBySequenceIndex[305] = '''B D J L R'''
poemAuthors[305] = ''''''
#306: ䷻䷂ Composed Sprouting: 1 3 2 4 2  : S R1 G3 M2 N2 : EkAk

#307: C D♭ E G♭ B :0 1 4 6 11: 1 ♭2 3 ♭5 7: ma7(♭5 add ♭9)
namesBySequenceIndex[306] = ''''''
namesBySequenceIndexNegative[306] = ''''''
semitonesBySequenceIndex[306] = [0, 1, 4, 6, 11]
poemBySequenceIndex[306] = '''B D J L S'''
poemAuthors[306] = ''''''
#307: ䷻䷚ Composed Nourishment: 1 3 2 5 1  : S R1 G3 M2 N3 : EkAl

#308: C D♭ E G A♭ :0 1 4 7 8: 1 ♭2 3 5 ♭6: ma(add ♭6 ♭9)
namesBySequenceIndex[307] = '''Raga Bowli (ascending),Raga Reva,Raga Revagupti,Raga Vibhas,Daptitonic'''
namesBySequenceIndexNegative[307] = ''''''
semitonesBySequenceIndex[307] = [0, 1, 4, 7, 8]
poemBySequenceIndex[307] = '''B D J M N'''
poemAuthors[307] = ''''''
#308: ䷻䷭ Composed Ascending: 1 3 3 1 4  : S R1 G3 P D1 : Ekoj

#309: C D♭ E G A :0 1 4 7 9: 1 ♭2 3 5 6: 6(add ♭9)
namesBySequenceIndex[308] = '''Scriabin,Raga Bibhas,Raga Ra,Doubly Augmented Sixth,Soptitonic'''
namesBySequenceIndexNegative[308] = ''''''
semitonesBySequenceIndex[308] = [0, 1, 4, 7, 9]
poemBySequenceIndex[308] = '''B D J M P'''
poemAuthors[308] = ''''''
#309: ䷻䷧ Composed Deliverance: 1 3 3 2 3  : S R1 G3 P D2 : Ekeb

#310: C D♭ E G B♭ :0 1 4 7 10: 1 ♭2 3 5 ♭7: 7(add ♭9)
namesBySequenceIndex[309] = '''Raga Manaranjani 2,Loptitonic'''
namesBySequenceIndexNegative[309] = ''''''
semitonesBySequenceIndex[309] = [0, 1, 4, 7, 10]
poemBySequenceIndex[309] = '''B D J M R'''
poemAuthors[309] = ''''''
#310: ䷻䷜ Composed Chasm: 1 3 3 3 2  : S R1 G3 P N2 : Ekek

#311: C D♭ E G B :0 1 4 7 11: 1 ♭2 3 5 7: ma7(add ♭9)
namesBySequenceIndex[310] = '''Zalitonic'''
namesBySequenceIndexNegative[310] = ''''''
semitonesBySequenceIndex[310] = [0, 1, 4, 7, 11]
poemBySequenceIndex[310] = '''B D J M S'''
poemAuthors[310] = ''''''
#311: ䷻䷃ Composed Naïve: 1 3 3 4 1  : S R1 G3 P N3 : Ekel

#312: C D♭ E G♯ A :0 1 4 8 9: 1 ♭2 3 ♯5 6: 6(♯5 add ♭9)
namesBySequenceIndex[311] = '''Aeolapritonic'''
namesBySequenceIndexNegative[311] = ''''''
semitonesBySequenceIndex[311] = [0, 1, 4, 8, 9]
poemBySequenceIndex[311] = '''B D J N P'''
poemAuthors[311] = ''''''
#312: ䷻䷽ Composed Detail: 1 3 4 1 3  : S R1 G3 D1 N1 : Ekib

#313: C D♭ E G♯ B♭ :0 1 4 8 10: 1 ♭2 3 ♯5 ♭7: 7(♯5 add ♭9)
namesBySequenceIndex[312] = '''Aerophitonic'''
namesBySequenceIndexNegative[312] = ''''''
semitonesBySequenceIndex[312] = [0, 1, 4, 8, 10]
poemBySequenceIndex[312] = '''B D J N R'''
poemAuthors[312] = ''''''
#313: ䷻䷦ Composed Limping: 1 3 4 2 2  : S R1 G3 D1 N2 : Ekik

#314: C D♭ E G♯ B :0 1 4 8 11: 1 ♭2 3 ♯5 7: ma7(♯5 add ♭9)
namesBySequenceIndex[313] = '''Doptitonic'''
namesBySequenceIndexNegative[313] = ''''''
semitonesBySequenceIndex[313] = [0, 1, 4, 8, 11]
poemBySequenceIndex[313] = '''B D J N S'''
poemAuthors[313] = ''''''
#314: ䷻䷳ Composed Stillness: 1 3 4 3 1  : S R1 G3 D1 N3 : Ekil

#315: C D♭ E A B♭ :0 1 4 9 10: 1 ♭2 3 6 ♭7: 7(no 5 add ♭9 13)
namesBySequenceIndex[314] = ''''''
namesBySequenceIndexNegative[314] = ''''''
semitonesBySequenceIndex[314] = [0, 1, 4, 9, 10]
poemBySequenceIndex[314] = '''B D J P R'''
poemAuthors[314] = ''''''
#315: ䷻䷬ Composed Clustering: 1 3 5 1 2  : S R1 G3 D2 N2 : Ekad

#316: C D♭ E A B :0 1 4 9 11: 1 ♭2 3 6 7: ma7(no 5 add ♭9 13)
namesBySequenceIndex[315] = ''''''
namesBySequenceIndexNegative[315] = ''''''
semitonesBySequenceIndex[315] = [0, 1, 4, 9, 11]
poemBySequenceIndex[315] = '''B D J P S'''
poemAuthors[315] = ''''''
#316: ䷻䷢ Composed Prospering: 1 3 5 2 1  : S R1 G3 D2 N3 : Ekaf

#317: C D♭ E A♯ B :0 1 4 10 11: 1 ♭2 3 ♯6 7: ma7(no 5 add ♭7 ♭9)
namesBySequenceIndex[316] = ''''''
namesBySequenceIndexNegative[316] = ''''''
semitonesBySequenceIndex[316] = [0, 1, 4, 10, 11]
poemBySequenceIndex[316] = '''B D J R S'''
poemAuthors[316] = ''''''
#317: ䷻䷓ Composed Seeing: 1 3 6 1 1  : S R1 G3 D3 N3 : Ekam

#318: C D♭ F F♯ G :0 1 5 6 7: 1 ♭2 4 ♯4 5: sus4(add ♭9 ♯11)
namesBySequenceIndex[317] = ''''''
namesBySequenceIndexNegative[317] = ''''''
semitonesBySequenceIndex[317] = [0, 1, 5, 6, 7]
poemBySequenceIndex[317] = '''B D K L M'''
poemAuthors[317] = ''''''
#318: ䷨䷒ Diminishing Arrived: 1 4 1 1 5  : S R1 M1 M2 P : ElEj

#319: C D♭ F G♭ A♭ :0 1 5 6 8: 1 ♭2 4 ♭5 ♭6: sus4(♯5 add ♭9 ♯11)
namesBySequenceIndex[318] = '''Aeoloritonic'''
namesBySequenceIndexNegative[318] = ''''''
semitonesBySequenceIndex[318] = [0, 1, 5, 6, 8]
poemBySequenceIndex[318] = '''B D K L N'''
poemAuthors[318] = ''''''
#319: ䷨䷣ Diminishing Eclipse: 1 4 1 2 4  : S R1 M1 M2 D1 : ElIj

#320: C D♭ F G♭ A :0 1 5 6 9: 1 ♭2 4 ♭5 6: 6sus4(♭5 add ♭9)
namesBySequenceIndex[319] = '''Zynitonic'''
namesBySequenceIndexNegative[319] = ''''''
semitonesBySequenceIndex[319] = [0, 1, 5, 6, 9]
poemBySequenceIndex[319] = '''B D K L P'''
poemAuthors[319] = ''''''
#320: ䷨䷲ Diminishing Thunder: 1 4 1 3 3  : S R1 M1 M2 D2 : ElAb

#321: C D♭ F G♭ B♭ :0 1 5 6 10: 1 ♭2 4 ♭5 ♭7: 7sus4(♭5 add ♭9)
namesBySequenceIndex[320] = '''Iwato,Aeolian Harmonic,Zokuso Pentatonic (Japan),Hon-Kumoi-Joshi (Japan),Lanitonic'''
namesBySequenceIndexNegative[320] = ''''''
semitonesBySequenceIndex[320] = [0, 1, 5, 6, 10]
poemBySequenceIndex[320] = '''B D K L R'''
poemAuthors[320] = ''''''
#321: ䷨䷂ Diminishing Sprouting: 1 4 1 4 2  : S R1 M1 M2 N2 : ElAk

#322: C D♭ F G♭ B :0 1 5 6 11: 1 ♭2 4 ♭5 7: ma7sus4(♭5 add ♭9)
namesBySequenceIndex[321] = ''''''
namesBySequenceIndexNegative[321] = ''''''
semitonesBySequenceIndex[321] = [0, 1, 5, 6, 11]
poemBySequenceIndex[321] = '''B D K L S'''
poemAuthors[321] = ''''''
#322: ䷨䷚ Diminishing Nourishment: 1 4 1 5 1  : S R1 M1 M2 N3 : ElAl

#323: C D♭ F G A♭ :0 1 5 7 8: 1 ♭2 4 5 ♭6: sus4(add ♭6 ♭9)
namesBySequenceIndex[322] = '''In,Sakura (Japan),Naka Zora (Japan),Soft Descend (Japan),Ambassel (Ethiopia),Olympos Enharmonic (Greece),Raga Malahari (ascending),Raga Saveri (ascending),Raga Gunkali,Raga Kunakri,Raga Latantapryia,Raga Salanganata,Raga Geyahejjajji,Raga Kannadabangala,In (Japan),Ionacritonic'''
namesBySequenceIndexNegative[322] = ''''''
semitonesBySequenceIndex[322] = [0, 1, 5, 7, 8]
poemBySequenceIndex[322] = '''B D K M N'''
poemAuthors[322] = ''''''
#323: ䷨䷭ Diminishing Ascending: 1 4 2 1 4  : S R1 M1 P D1 : Eloj

#324: C D♭ F G A :0 1 5 7 9: 1 ♭2 4 5 6: 6sus4(add ♭9)
namesBySequenceIndex[323] = '''Raga Manaranjani 1,Zyditonic'''
namesBySequenceIndexNegative[323] = ''''''
semitonesBySequenceIndex[323] = [0, 1, 5, 7, 9]
poemBySequenceIndex[323] = '''B D K M P'''
poemAuthors[323] = ''''''
#324: ䷨䷧ Diminishing Deliverance: 1 4 2 2 3  : S R1 M1 P D2 : Eleb

#325: C D♭ F G B♭ :0 1 5 7 10: 1 ♭2 4 5 ♭7: 7sus4(add ♭9)
namesBySequenceIndex[324] = '''Kokin-Choshi,Ian Iwato (Japan),Miyakobushi (Japan),Insen Pentatonic (Japan),Soft Ascend (Japan),Raga Bairagi (or Baira),Raga Lasaki,Raga Ribhavari (or Revati),Thalitonic'''
namesBySequenceIndexNegative[324] = ''''''
semitonesBySequenceIndex[324] = [0, 1, 5, 7, 10]
poemBySequenceIndex[324] = '''B D K M R'''
poemAuthors[324] = ''''''
#325: ䷨䷜ Diminishing Chasm: 1 4 2 3 2  : S R1 M1 P N2 : Elek

#326: C D♭ F G B :0 1 5 7 11: 1 ♭2 4 5 7: ma7sus4(add ♭9)
namesBySequenceIndex[325] = '''Raga Gowla (ascending),Raga Gauri,Ionythitonic'''
namesBySequenceIndexNegative[325] = ''''''
semitonesBySequenceIndex[325] = [0, 1, 5, 7, 11]
poemBySequenceIndex[325] = '''B D K M S'''
poemAuthors[325] = ''''''
#326: ䷨䷃ Diminishing Naïve: 1 4 2 4 1  : S R1 M1 P N3 : Elel

#327: C D♭ F G♯ A :0 1 5 8 9: 1 ♭2 4 ♯5 6: 6sus4(♯5 add ♭9)
namesBySequenceIndex[326] = '''Loritonic'''
namesBySequenceIndexNegative[326] = ''''''
semitonesBySequenceIndex[326] = [0, 1, 5, 8, 9]
poemBySequenceIndex[326] = '''B D K N P'''
poemAuthors[326] = ''''''
#327: ䷨䷽ Diminishing Detail: 1 4 3 1 3  : S R1 M1 D1 N1 : Elib

#328: C D♭ F G♯ B♭ :0 1 5 8 10: 1 ♭2 4 ♯5 ♭7: 7sus4(♯5 add ♭9)
namesBySequenceIndex[327] = '''Pyritonic'''
namesBySequenceIndexNegative[327] = ''''''
semitonesBySequenceIndex[327] = [0, 1, 5, 8, 10]
poemBySequenceIndex[327] = '''B D K N R'''
poemAuthors[327] = ''''''
#328: ䷨䷦ Diminishing Limping: 1 4 3 2 2  : S R1 M1 D1 N2 : Elik

#329: C D♭ F G♯ B :0 1 5 8 11: 1 ♭2 4 ♯5 7: ma7sus4(♯5 add ♭9)
namesBySequenceIndex[328] = '''Raga Kshanika,Ragitonic'''
namesBySequenceIndexNegative[328] = ''''''
semitonesBySequenceIndex[328] = [0, 1, 5, 8, 11]
poemBySequenceIndex[328] = '''B D K N S'''
poemAuthors[328] = ''''''
#329: ䷨䷳ Diminishing Stillness: 1 4 3 3 1  : S R1 M1 D1 N3 : Elil

#330: C D♭ F A B♭ :0 1 5 9 10: 1 ♭2 4 6 ♭7: 13(♭9 no 3 5)
namesBySequenceIndex[329] = '''Lagitonic'''
namesBySequenceIndexNegative[329] = ''''''
semitonesBySequenceIndex[329] = [0, 1, 5, 9, 10]
poemBySequenceIndex[329] = '''B D K P R'''
poemAuthors[329] = ''''''
#330: ䷨䷬ Diminishing Clustering: 1 4 4 1 2  : S R1 M1 D2 N2 : Elad

#331: C D♭ F A B :0 1 5 9 11: 1 ♭2 4 6 7: ma13(♭9 no 3 5)
namesBySequenceIndex[330] = '''Rolitonic'''
namesBySequenceIndexNegative[330] = ''''''
semitonesBySequenceIndex[330] = [0, 1, 5, 9, 11]
poemBySequenceIndex[330] = '''B D K P S'''
poemAuthors[330] = ''''''
#331: ䷨䷢ Diminishing Prospering: 1 4 4 2 1  : S R1 M1 D2 N3 : Elaf

#332: C D♭ F A♯ B :0 1 5 10 11: 1 ♭2 4 ♯6 7: ma7sus4(no 5 add ♭7 ♭9)
namesBySequenceIndex[331] = ''''''
namesBySequenceIndexNegative[331] = ''''''
semitonesBySequenceIndex[331] = [0, 1, 5, 10, 11]
poemBySequenceIndex[331] = '''B D K R S'''
poemAuthors[331] = ''''''
#332: ䷨䷓ Diminishing Seeing: 1 4 5 1 1  : S R1 M1 D3 N3 : Elam

#333: C D♭ F♯ G A♭ :0 1 6 7 8: 1 ♭2 ♯4 5 ♭6: sus♯4(add ♭6 ♭9)
namesBySequenceIndex[332] = '''Raga Saugandhini,Raga Yashranjani'''
namesBySequenceIndexNegative[332] = ''''''
semitonesBySequenceIndex[332] = [0, 1, 6, 7, 8]
poemBySequenceIndex[332] = '''B D L M N'''
poemAuthors[332] = ''''''
#333: ䷒䷊ Arrived Peace: 1 5 1 1 4  : S R1 M2 P D1 : EjOj

#334: C D♭ F♯ G A :0 1 6 7 9: 1 ♭2 ♯4 5 6: 6sus♯4(add ♭9)
namesBySequenceIndex[333] = ''''''
namesBySequenceIndexNegative[333] = ''''''
semitonesBySequenceIndex[333] = [0, 1, 6, 7, 9]
poemBySequenceIndex[333] = '''B D L M P'''
poemAuthors[333] = ''''''
#334: ䷒䷵ Arrived Betrothing: 1 5 1 2 3  : S R1 M2 P D2 : EjEb

#335: C D♭ F♯ G B♭ :0 1 6 7 10: 1 ♭2 ♯4 5 ♭7: 7sus♯4(add ♭9)
namesBySequenceIndex[334] = ''''''
namesBySequenceIndexNegative[334] = ''''''
semitonesBySequenceIndex[334] = [0, 1, 6, 7, 10]
poemBySequenceIndex[334] = '''B D L M R'''
poemAuthors[334] = ''''''
#335: ䷒䷻ Arrived Composed: 1 5 1 3 2  : S R1 M2 P N2 : EjEk

#336: C D♭ F♯ G B :0 1 6 7 11: 1 ♭2 ♯4 5 7: ma7sus♯4(add ♭9)
namesBySequenceIndex[335] = ''''''
namesBySequenceIndexNegative[335] = ''''''
semitonesBySequenceIndex[335] = [0, 1, 6, 7, 11]
poemBySequenceIndex[335] = '''B D L M S'''
poemAuthors[335] = ''''''
#336: ䷒䷨ Arrived Diminishing: 1 5 1 4 1  : S R1 M2 P N3 : EjEl

#337: C D♭ F♯ G♯ A :0 1 6 8 9: 1 ♭2 ♯4 ♯5 6: 6sus♯4(♯5 add ♭9)
namesBySequenceIndex[336] = ''''''
namesBySequenceIndexNegative[336] = ''''''
semitonesBySequenceIndex[336] = [0, 1, 6, 8, 9]
poemBySequenceIndex[336] = '''B D L N P'''
poemAuthors[336] = ''''''
#337: ䷒䷶ Arrived Abounding: 1 5 2 1 3  : S R1 M2 D1 N1 : EjIb

#338: C D♭ F♯ G♯ B♭ :0 1 6 8 10: 1 ♭2 ♯4 ♯5 ♭7: 7sus♯4(♯5 add ♭9)
namesBySequenceIndex[337] = ''''''
namesBySequenceIndexNegative[337] = ''''''
semitonesBySequenceIndex[337] = [0, 1, 6, 8, 10]
poemBySequenceIndex[337] = '''B D L N R'''
poemAuthors[337] = ''''''
#338: ䷒䷾ Arrived Completed: 1 5 2 2 2  : S R1 M2 D1 N2 : EjIk

#339: C D♭ F♯ G♯ B :0 1 6 8 11: 1 ♭2 ♯4 ♯5 7: ma7sus♯4(♯5 add ♭9)
namesBySequenceIndex[338] = ''''''
namesBySequenceIndexNegative[338] = ''''''
semitonesBySequenceIndex[338] = [0, 1, 6, 8, 11]
poemBySequenceIndex[338] = '''B D L N S'''
poemAuthors[338] = ''''''
#339: ䷒䷕ Arrived Adorning: 1 5 2 3 1  : S R1 M2 D1 N3 : EjIl

#340: C D♭ G♭ A B♭ :0 1 6 9 10: 1 ♭2 ♭5 6 ♭7: 7sus♭2(♭5 add 13)
namesBySequenceIndex[339] = ''''''
namesBySequenceIndexNegative[339] = ''''''
semitonesBySequenceIndex[339] = [0, 1, 6, 9, 10]
poemBySequenceIndex[339] = '''B D L P R'''
poemAuthors[339] = ''''''
#340: ䷒䷐ Arrived Following: 1 5 3 1 2  : S R1 M2 D2 N2 : EjAd

#341: C D♭ G♭ A B :0 1 6 9 11: 1 ♭2 ♭5 6 7: ma7sus♭2(♭5 add 13)
namesBySequenceIndex[340] = ''''''
namesBySequenceIndexNegative[340] = ''''''
semitonesBySequenceIndex[340] = [0, 1, 6, 9, 11]
poemBySequenceIndex[340] = '''B D L P S'''
poemAuthors[340] = ''''''
#341: ䷒䷔ Arrived Biting: 1 5 3 2 1  : S R1 M2 D2 N3 : EjAf

#342: C D♭ G♭ A♯ B :0 1 6 10 11: 1 ♭2 ♭5 ♯6 7: ma7sus♭2(♭5 add ♭7)
namesBySequenceIndex[341] = ''''''
namesBySequenceIndexNegative[341] = ''''''
semitonesBySequenceIndex[341] = [0, 1, 6, 10, 11]
poemBySequenceIndex[341] = '''B D L R S'''
poemAuthors[341] = ''''''
#342: ䷒䷩ Arrived Augmenting: 1 5 4 1 1  : S R1 M2 D3 N3 : EjAm

#343: C D♭ G A♭ A :0 1 7 8 9: 1 ♭2 5 ♭6 6: 6sus♭2(add ♭6)
namesBySequenceIndex[342] = ''''''
namesBySequenceIndexNegative[342] = ''''''
semitonesBySequenceIndex[342] = [0, 1, 7, 8, 9]
poemBySequenceIndex[342] = '''B D M N P'''
poemAuthors[342] = ''''''
#343: ䷒䷟ Arrived Persevering: 1 6 1 1 3  : S R1 P D1 N1 : Ejob

#344: C D♭ G A♭ B♭ :0 1 7 8 10: 1 ♭2 5 ♭6 ♭7: 7sus♭2(add ♭13)
namesBySequenceIndex[343] = ''''''
namesBySequenceIndexNegative[343] = ''''''
semitonesBySequenceIndex[343] = [0, 1, 7, 8, 10]
poemBySequenceIndex[343] = '''B D M N R'''
poemAuthors[343] = ''''''
#344: ䷒䷯ Arrived Source: 1 6 1 2 2  : S R1 P D1 N2 : Ejok

#345: C D♭ G A♭ B :0 1 7 8 11: 1 ♭2 5 ♭6 7: ma7sus♭2(add ♭13)
namesBySequenceIndex[344] = '''Raga Deshgaur'''
namesBySequenceIndexNegative[344] = ''''''
semitonesBySequenceIndex[344] = [0, 1, 7, 8, 11]
poemBySequenceIndex[344] = '''B D M N S'''
poemAuthors[344] = ''''''
#345: ䷒䷑ Arrived Correcting: 1 6 1 3 1  : S R1 P D1 N3 : Ejol

#346: C D♭ G A B♭ :0 1 7 9 10: 1 ♭2 5 6 ♭7: 7sus♭2(add 13)
namesBySequenceIndex[345] = ''''''
namesBySequenceIndexNegative[345] = ''''''
semitonesBySequenceIndex[345] = [0, 1, 7, 9, 10]
poemBySequenceIndex[345] = '''B D M P R'''
poemAuthors[345] = ''''''
#346: ䷒䷮ Arrived Confining: 1 6 2 1 2  : S R1 P D2 N2 : Ejed

#347: C D♭ G A B :0 1 7 9 11: 1 ♭2 5 6 7: ma7sus♭2(add 13)
namesBySequenceIndex[346] = ''''''
namesBySequenceIndexNegative[346] = ''''''
semitonesBySequenceIndex[346] = [0, 1, 7, 9, 11]
poemBySequenceIndex[346] = '''B D M P S'''
poemAuthors[346] = ''''''
#347: ䷒䷿ Arrived Unfinished: 1 6 2 2 1  : S R1 P D2 N3 : Ejef

#348: C D♭ G A♯ B :0 1 7 10 11: 1 ♭2 5 ♯6 7: ma7sus♭2(add ♭7)
namesBySequenceIndex[347] = ''''''
namesBySequenceIndexNegative[347] = ''''''
semitonesBySequenceIndex[347] = [0, 1, 7, 10, 11]
poemBySequenceIndex[347] = '''B D M R S'''
poemAuthors[347] = ''''''
#348: ䷒䷺ Arrived Dispersing: 1 6 3 1 1  : S R1 P D3 N3 : Ejem

#349: C D♭ G♯ A B♭ :0 1 8 9 10: 1 ♭2 ♯5 6 ♭7: 7sus♭2(♯5 add 13)
namesBySequenceIndex[348] = ''''''
namesBySequenceIndexNegative[348] = ''''''
semitonesBySequenceIndex[348] = [0, 1, 8, 9, 10]
poemBySequenceIndex[348] = '''B D N P R'''
poemAuthors[348] = ''''''
#349: ䷒䷞ Arrived Conjoining: 1 7 1 1 2  : S R1 D1 D2 N2 : Ejid

#350: C D♭ G♯ A B :0 1 8 9 11: 1 ♭2 ♯5 6 7: ma7sus♭2(♯5 add 13)
namesBySequenceIndex[349] = ''''''
namesBySequenceIndexNegative[349] = ''''''
semitonesBySequenceIndex[349] = [0, 1, 8, 9, 11]
poemBySequenceIndex[349] = '''B D N P S'''
poemAuthors[349] = ''''''
#350: ䷒䷷ Arrived Wanderer: 1 7 1 2 1  : S R1 D1 D2 N3 : Ejif

#351: C D♭ G♯ A♯ B :0 1 8 10 11: 1 ♭2 ♯5 ♯6 7: ma7sus♭2(♯5 add ♭7)
namesBySequenceIndex[350] = ''''''
namesBySequenceIndexNegative[350] = ''''''
semitonesBySequenceIndex[350] = [0, 1, 8, 10, 11]
poemBySequenceIndex[350] = '''B D N R S'''
poemAuthors[350] = ''''''
#351: ䷒䷴ Arrived Infiltrating: 1 7 2 1 1  : S R1 D1 N2 N3 : Ejim

#352: C D♭ A B♭ B :0 1 9 10 11: 1 ♭2 6 ♭7 7: ma7sus♭2(no 5 add ♭7 13)
namesBySequenceIndex[351] = ''''''
namesBySequenceIndexNegative[351] = ''''''
semitonesBySequenceIndex[351] = [0, 1, 9, 10, 11]
poemBySequenceIndex[351] = '''B D P R S'''
poemAuthors[351] = ''''''
#352: ䷒䷋ Arrived Obstruction: 1 8 1 1 1  : S R1 D2 N2 N3 : Ejag

#353: C D D♯ E F :0 2 3 4 5: 1 2 ♯2 3 4: 11(no 5 7 add ♯9)
namesBySequenceIndex[352] = ''''''
namesBySequenceIndexNegative[352] = ''''''
semitonesBySequenceIndex[352] = [0, 2, 3, 4, 5]
poemBySequenceIndex[352] = '''B F G J K'''
poemAuthors[352] = ''''''
#353: ䷌䷁ Fellowship Receptive: 2 1 1 1 7  : S R2 R3 G3 M1 : Igaj

#354: C D D♯ E G♭ :0 2 3 4 6: 1 2 ♯2 3 ♭5: ma(♭5 add 9 ♯9)
namesBySequenceIndex[353] = ''''''
namesBySequenceIndexNegative[353] = ''''''
semitonesBySequenceIndex[353] = [0, 2, 3, 4, 6]
poemBySequenceIndex[353] = '''B F G J L'''
poemAuthors[353] = ''''''
#354: ䷰䷗ Revolution Returning: 2 1 1 2 6  : S R2 R3 G3 M2 : IdAj

#355: C D D♯ E G :0 2 3 4 7: 1 2 ♯2 3 5: ma(add 9 ♯9)
namesBySequenceIndex[354] = ''''''
namesBySequenceIndexNegative[354] = ''''''
semitonesBySequenceIndex[354] = [0, 2, 3, 4, 7]
poemBySequenceIndex[354] = '''B F G J M'''
poemAuthors[354] = ''''''
#355: ䷰䷆ Revolution Leading: 2 1 1 3 5  : S R2 R3 G3 P : Idej

#356: C D D♯ E G♯ :0 2 3 4 8: 1 2 ♯2 3 ♯5: ma(♯5 add 9 ♯9)
namesBySequenceIndex[355] = '''Zaritonic'''
namesBySequenceIndexNegative[355] = ''''''
semitonesBySequenceIndex[355] = [0, 2, 3, 4, 8]
poemBySequenceIndex[355] = '''B F G J N'''
poemAuthors[355] = ''''''
#356: ䷰䷎ Revolution Humble: 2 1 1 4 4  : S R2 R3 G3 D1 : Idij

#357: C D D♯ E A :0 2 3 4 9: 1 2 ♯2 3 6: 6/9(no 5 add ♯9)
namesBySequenceIndex[356] = ''''''
namesBySequenceIndexNegative[356] = ''''''
semitonesBySequenceIndex[356] = [0, 2, 3, 4, 9]
poemBySequenceIndex[356] = '''B F G J P'''
poemAuthors[356] = ''''''
#357: ䷰䷏ Revolution Confident: 2 1 1 5 3  : S R2 R3 G3 D2 : Idab

#358: C D D♯ E B♭ :0 2 3 4 10: 1 2 ♯2 3 ♭7: 9(no 5 add ♯9)
namesBySequenceIndex[357] = ''''''
namesBySequenceIndexNegative[357] = ''''''
semitonesBySequenceIndex[357] = [0, 2, 3, 4, 10]
poemBySequenceIndex[357] = '''B F G J R'''
poemAuthors[357] = ''''''
#358: ䷰䷇ Revolution Comparing: 2 1 1 6 2  : S R2 R3 G3 N2 : Idak

#359: C D D♯ E B :0 2 3 4 11: 1 2 ♯2 3 7: ma9(no 5 add ♯9)
namesBySequenceIndex[358] = ''''''
namesBySequenceIndexNegative[358] = ''''''
semitonesBySequenceIndex[358] = [0, 2, 3, 4, 11]
poemBySequenceIndex[358] = '''B F G J S'''
poemAuthors[358] = ''''''
#359: ䷰䷖ Revolution Stripping: 2 1 1 7 1  : S R2 R3 G3 N3 : Idal

#360: C D E♭ F G♭ :0 2 3 5 6: 1 2 ♭3 4 ♭5: °11(no 7)
namesBySequenceIndex[359] = ''''''
namesBySequenceIndexNegative[359] = ''''''
semitonesBySequenceIndex[359] = [0, 2, 3, 5, 6]
poemBySequenceIndex[359] = '''B F G K L'''
poemAuthors[359] = ''''''
#360: ䷝䷗ Distanced Returning: 2 1 2 1 6  : S R2 G2 M1 M2 : IfAj

#361: C D E♭ F G :0 2 3 5 7: 1 2 ♭3 4 5: mi11(no 7)
namesBySequenceIndex[360] = '''Nando-Kyemyonjo,Ghana Pentatonic 1,Chad Gadyo (Jewish),Raga Kuntvarali,Raga Purnanalita'''
namesBySequenceIndexNegative[360] = ''''''
semitonesBySequenceIndex[360] = [0, 2, 3, 5, 7]
poemBySequenceIndex[360] = '''B F G K M'''
poemAuthors[360] = ''''''
#361: ䷝䷆ Distanced Leading: 2 1 2 2 5  : S R2 G2 M1 P : Ifej

#362: C D E♭ F G♯ :0 2 3 5 8: 1 2 ♭3 4 ♯5: mi11(♯5 no 7)
namesBySequenceIndex[361] = '''Raga Audav Tukhari,Zythitonic'''
namesBySequenceIndexNegative[361] = ''''''
semitonesBySequenceIndex[361] = [0, 2, 3, 5, 8]
poemBySequenceIndex[361] = '''B F G K N'''
poemAuthors[361] = ''''''
#362: ䷝䷎ Distanced Humble: 2 1 2 3 4  : S R2 G2 M1 D1 : Ifij

#363: C D E♭ F A :0 2 3 5 9: 1 2 ♭3 4 6: mi13(no 5 7)
namesBySequenceIndex[362] = '''Raga Abhogi,Gythitonic'''
namesBySequenceIndexNegative[362] = ''''''
semitonesBySequenceIndex[362] = [0, 2, 3, 5, 9]
poemBySequenceIndex[362] = '''B F G K P'''
poemAuthors[362] = ''''''
#363: ䷝䷏ Distanced Confident: 2 1 2 4 3  : S R2 G2 M1 D2 : Ifab

#364: C D E♭ F B♭ :0 2 3 5 10: 1 2 ♭3 4 ♭7: mi11(no 5)
namesBySequenceIndex[363] = ''''''
namesBySequenceIndexNegative[363] = ''''''
semitonesBySequenceIndex[363] = [0, 2, 3, 5, 10]
poemBySequenceIndex[363] = '''B F G K R'''
poemAuthors[363] = ''''''
#364: ䷝䷇ Distanced Comparing: 2 1 2 5 2  : S R2 G2 M1 N2 : Ifak

#365: C D E♭ F B :0 2 3 5 11: 1 2 ♭3 4 7: mima11(no 5)
namesBySequenceIndex[364] = ''''''
namesBySequenceIndexNegative[364] = ''''''
semitonesBySequenceIndex[364] = [0, 2, 3, 5, 11]
poemBySequenceIndex[364] = '''B F G K S'''
poemAuthors[364] = ''''''
#365: ䷝䷖ Distanced Stripping: 2 1 2 6 1  : S R2 G2 M1 N3 : Ifal

#366: C D E♭ F♯ G :0 2 3 6 7: 1 2 ♭3 ♯4 5: mi(add 9 ♯11)
namesBySequenceIndex[365] = ''''''
namesBySequenceIndexNegative[365] = ''''''
semitonesBySequenceIndex[365] = [0, 2, 3, 6, 7]
poemBySequenceIndex[365] = '''B F G L M'''
poemAuthors[365] = ''''''
#366: ䷶䷒ Abounding Arrived: 2 1 3 1 5  : S R2 G2 M2 P : IbEj

#367: C D E♭ F♯ G♯ :0 2 3 6 8: 1 2 ♭3 ♯4 ♯5: °(add ♭6 9)
namesBySequenceIndex[366] = '''Bogitonic'''
namesBySequenceIndexNegative[366] = ''''''
semitonesBySequenceIndex[366] = [0, 2, 3, 6, 8]
poemBySequenceIndex[366] = '''B F G L N'''
poemAuthors[366] = ''''''
#367: ䷶䷣ Abounding Eclipse: 2 1 3 2 4  : S R2 G2 M2 D1 : IbIj

#368: C D E♭ G♭ A :0 2 3 6 9: 1 2 ♭3 ♭5 6: °7(add 9)
namesBySequenceIndex[367] = '''Ionalitonic'''
namesBySequenceIndexNegative[367] = ''''''
semitonesBySequenceIndex[367] = [0, 2, 3, 6, 9]
poemBySequenceIndex[367] = '''B F G L P'''
poemAuthors[367] = ''''''
#368: ䷶䷲ Abounding Thunder: 2 1 3 3 3  : S R2 G2 M2 D2 : IbAb

#369: C D E♭ G♭ B♭ :0 2 3 6 10: 1 2 ♭3 ♭5 ♭7: ⌀9
namesBySequenceIndex[368] = '''Stothitonic'''
namesBySequenceIndexNegative[368] = ''''''
semitonesBySequenceIndex[368] = [0, 2, 3, 6, 10]
poemBySequenceIndex[368] = '''B F G L R'''
poemAuthors[368] = ''''''
#369: ䷶䷂ Abounding Sprouting: 2 1 3 4 2  : S R2 G2 M2 N2 : IbAk

#370: C D E♭ G♭ B :0 2 3 6 11: 1 2 ♭3 ♭5 7: °ma9
namesBySequenceIndex[369] = ''''''
namesBySequenceIndexNegative[369] = ''''''
semitonesBySequenceIndex[369] = [0, 2, 3, 6, 11]
poemBySequenceIndex[369] = '''B F G L S'''
poemAuthors[369] = ''''''
#370: ䷶䷚ Abounding Nourishment: 2 1 3 5 1  : S R2 G2 M2 N3 : IbAl

#371: C D E♭ G A♭ :0 2 3 7 8: 1 2 ♭3 5 ♭6: mi(add ♭6 9)
namesBySequenceIndex[370] = '''Ake-Bono,Aeolian Pentatonic,Kata-Kumoi (Japan),Yona Nuki Minor (Japan),Epogitonic'''
namesBySequenceIndexNegative[370] = ''''''
semitonesBySequenceIndex[370] = [0, 2, 3, 7, 8]
poemBySequenceIndex[370] = '''B F G M N'''
poemAuthors[370] = ''''''
#371: ䷶䷭ Abounding Ascending: 2 1 4 1 4  : S R2 G2 P D1 : Iboj

#372: C D E♭ G A :0 2 3 7 9: 1 2 ♭3 5 6: mi6/9
namesBySequenceIndex[371] = '''Kumoi,Dorian Pentatonic,Stathitonic'''
namesBySequenceIndexNegative[371] = ''''''
semitonesBySequenceIndex[371] = [0, 2, 3, 7, 9]
poemBySequenceIndex[371] = '''B F G M P'''
poemAuthors[371] = ''''''
#372: ䷶䷧ Abounding Deliverance: 2 1 4 2 3  : S R2 G2 P D2 : Ibeb

#373: C D E♭ G B♭ :0 2 3 7 10: 1 2 ♭3 5 ♭7: mi9
namesBySequenceIndex[372] = '''Pygmy,Gycritonic'''
namesBySequenceIndexNegative[372] = ''''''
semitonesBySequenceIndex[372] = [0, 2, 3, 7, 10]
poemBySequenceIndex[372] = '''B F G M R'''
poemAuthors[372] = ''''''
#373: ䷶䷜ Abounding Chasm: 2 1 4 3 2  : S R2 G2 P N2 : Ibek

#374: C D E♭ G B :0 2 3 7 11: 1 2 ♭3 5 7: mima9
namesBySequenceIndex[373] = '''Spiderman Chord,Raga Hamsadhvani 1,Zagitonic'''
namesBySequenceIndexNegative[373] = ''''''
semitonesBySequenceIndex[373] = [0, 2, 3, 7, 11]
poemBySequenceIndex[373] = '''B F G M S'''
poemAuthors[373] = ''''''
#374: ䷶䷃ Abounding Naïve: 2 1 4 4 1  : S R2 G2 P N3 : Ibel

#375: C D E♭ G♯ A :0 2 3 8 9: 1 2 ♭3 ♯5 6: mi6/9(♯5)
namesBySequenceIndex[374] = ''''''
namesBySequenceIndexNegative[374] = ''''''
semitonesBySequenceIndex[374] = [0, 2, 3, 8, 9]
poemBySequenceIndex[374] = '''B F G N P'''
poemAuthors[374] = ''''''
#375: ䷶䷽ Abounding Detail: 2 1 5 1 3  : S R2 G2 D1 N1 : Ibib

#376: C D E♭ G♯ B♭ :0 2 3 8 10: 1 2 ♭3 ♯5 ♭7: mi9(♯5)
namesBySequenceIndex[375] = ''''''
namesBySequenceIndexNegative[375] = ''''''
semitonesBySequenceIndex[375] = [0, 2, 3, 8, 10]
poemBySequenceIndex[375] = '''B F G N R'''
poemAuthors[375] = ''''''
#376: ䷶䷦ Abounding Limping: 2 1 5 2 2  : S R2 G2 D1 N2 : Ibik

#377: C D E♭ G♯ B :0 2 3 8 11: 1 2 ♭3 ♯5 7: mima9(♯5)
namesBySequenceIndex[376] = ''''''
namesBySequenceIndexNegative[376] = ''''''
semitonesBySequenceIndex[376] = [0, 2, 3, 8, 11]
poemBySequenceIndex[376] = '''B F G N S'''
poemAuthors[376] = ''''''
#377: ䷶䷳ Abounding Stillness: 2 1 5 3 1  : S R2 G2 D1 N3 : Ibil

#378: C D E♭ A B♭ :0 2 3 9 10: 1 2 ♭3 6 ♭7: mi9(no 5 add 13)
namesBySequenceIndex[377] = ''''''
namesBySequenceIndexNegative[377] = ''''''
semitonesBySequenceIndex[377] = [0, 2, 3, 9, 10]
poemBySequenceIndex[377] = '''B F G P R'''
poemAuthors[377] = ''''''
#378: ䷶䷬ Abounding Clustering: 2 1 6 1 2  : S R2 G2 D2 N2 : Ibad

#379: C D E♭ A B :0 2 3 9 11: 1 2 ♭3 6 7: mima9(no 5 add 13)
namesBySequenceIndex[378] = ''''''
namesBySequenceIndexNegative[378] = ''''''
semitonesBySequenceIndex[378] = [0, 2, 3, 9, 11]
poemBySequenceIndex[378] = '''B F G P S'''
poemAuthors[378] = ''''''
#379: ䷶䷢ Abounding Prospering: 2 1 6 2 1  : S R2 G2 D2 N3 : Ibaf

#380: C D E♭ A♯ B :0 2 3 10 11: 1 2 ♭3 ♯6 7: mima9(no 5 add ♭7)
namesBySequenceIndex[379] = ''''''
namesBySequenceIndexNegative[379] = ''''''
semitonesBySequenceIndex[379] = [0, 2, 3, 10, 11]
poemBySequenceIndex[379] = '''B F G R S'''
poemAuthors[379] = ''''''
#380: ䷶䷓ Abounding Seeing: 2 1 7 1 1  : S R2 G2 D3 N3 : Ibam

#381: C D E F G♭ :0 2 4 5 6: 1 2 3 4 ♭5: 11(♭5 no 7)
namesBySequenceIndex[380] = ''''''
namesBySequenceIndexNegative[380] = ''''''
semitonesBySequenceIndex[380] = [0, 2, 4, 5, 6]
poemBySequenceIndex[380] = '''B F J K L'''
poemAuthors[380] = ''''''
#381: ䷤䷗ Family Returning: 2 2 1 1 6  : S R2 G3 M1 M2 : ImAj

#382: C D E F G :0 2 4 5 7: 1 2 3 4 5: 11(no 7)
namesBySequenceIndex[381] = '''Raga Budhamanohari'''
namesBySequenceIndexNegative[381] = ''''''
semitonesBySequenceIndex[381] = [0, 2, 4, 5, 7]
poemBySequenceIndex[381] = '''B F J K M'''
poemAuthors[381] = ''''''
#382: ䷤䷆ Family Leading: 2 2 1 2 5  : S R2 G3 M1 P : Imej

#383: C D E F G♯ :0 2 4 5 8: 1 2 3 4 ♯5: 11(♯5 no 7)
namesBySequenceIndex[382] = '''Palitonic,Altered Major Pentatonic'''
namesBySequenceIndexNegative[382] = ''''''
semitonesBySequenceIndex[382] = [0, 2, 4, 5, 8]
poemBySequenceIndex[382] = '''B F J K N'''
poemAuthors[382] = ''''''
#383: ䷤䷎ Family Humble: 2 2 1 3 4  : S R2 G3 M1 D1 : Imij

#384: C D E F A :0 2 4 5 9: 1 2 3 4 6: 13(no 5 7)
namesBySequenceIndex[383] = '''Aeolyphritonic'''
namesBySequenceIndexNegative[383] = ''''''
semitonesBySequenceIndex[383] = [0, 2, 4, 5, 9]
poemBySequenceIndex[383] = '''B F J K P'''
poemAuthors[383] = ''''''
#384: ䷤䷏ Family Confident: 2 2 1 4 3  : S R2 G3 M1 D2 : Imab

#385: C D E F B♭ :0 2 4 5 10: 1 2 3 4 ♭7: 11(no 5)
namesBySequenceIndex[384] = ''''''
namesBySequenceIndexNegative[384] = ''''''
semitonesBySequenceIndex[384] = [0, 2, 4, 5, 10]
poemBySequenceIndex[384] = '''B F J K R'''
poemAuthors[384] = ''''''
#385: ䷤䷇ Family Comparing: 2 2 1 5 2  : S R2 G3 M1 N2 : Imak

#386: C D E F B :0 2 4 5 11: 1 2 3 4 7: ma11(no 5)
namesBySequenceIndex[385] = ''''''
namesBySequenceIndexNegative[385] = ''''''
semitonesBySequenceIndex[385] = [0, 2, 4, 5, 11]
poemBySequenceIndex[385] = '''B F J K S'''
poemAuthors[385] = ''''''
#386: ䷤䷖ Family Stripping: 2 2 1 6 1  : S R2 G3 M1 N3 : Imal

#387: C D E F♯ G :0 2 4 6 7: 1 2 3 ♯4 5: ma(add 9 ♯11)
namesBySequenceIndex[386] = ''''''
namesBySequenceIndexNegative[386] = ''''''
semitonesBySequenceIndex[386] = [0, 2, 4, 6, 7]
poemBySequenceIndex[386] = '''B F J L M'''
poemAuthors[386] = ''''''
#387: ䷾䷒ Completed Arrived: 2 2 2 1 5  : S R2 G3 M2 P : IkEj

#388: C D E F♯ G♯ :0 2 4 6 8: 1 2 3 ♯4 ♯5: ma(♯5 add 9 ♯11)
namesBySequenceIndex[387] = '''Bothitonic'''
namesBySequenceIndexNegative[387] = ''''''
semitonesBySequenceIndex[387] = [0, 2, 4, 6, 8]
poemBySequenceIndex[387] = '''B F J L N'''
poemAuthors[387] = ''''''
#388: ䷾䷣ Completed Eclipse: 2 2 2 2 4  : S R2 G3 M2 D1 : IkIj

#389: C D E G♭ A :0 2 4 6 9: 1 2 3 ♭5 6: 6/9(♭5)
namesBySequenceIndex[388] = '''Kung,Thonitonic'''
namesBySequenceIndexNegative[388] = ''''''
semitonesBySequenceIndex[388] = [0, 2, 4, 6, 9]
poemBySequenceIndex[388] = '''B F J L P'''
poemAuthors[388] = ''''''
#389: ䷾䷲ Completed Thunder: 2 2 2 3 3  : S R2 G3 M2 D2 : IkAb

#390: C D E G♭ B♭ :0 2 4 6 10: 1 2 3 ♭5 ♭7: 9(♭5)
namesBySequenceIndex[389] = '''Kataditonic'''
namesBySequenceIndexNegative[389] = ''''''
semitonesBySequenceIndex[389] = [0, 2, 4, 6, 10]
poemBySequenceIndex[389] = '''B F J L R'''
poemAuthors[389] = ''''''
#390: ䷾䷂ Completed Sprouting: 2 2 2 4 2  : S R2 G3 M2 N2 : IkAk

#391: C D E G♭ B :0 2 4 6 11: 1 2 3 ♭5 7: ma9(♭5)
namesBySequenceIndex[390] = '''Raga Kumurdaki,Raga Kumudki'''
namesBySequenceIndexNegative[390] = ''''''
semitonesBySequenceIndex[390] = [0, 2, 4, 6, 11]
poemBySequenceIndex[390] = '''B F J L S'''
poemAuthors[390] = ''''''
#391: ䷾䷚ Completed Nourishment: 2 2 2 5 1  : S R2 G3 M2 N3 : IkAl

#392: C D E G A♭ :0 2 4 7 8: 1 2 3 5 ♭6: ma(add ♭6 9)
namesBySequenceIndex[391] = '''Raga Janasammodini,Raga Bhupeshwari,Major Pentatonic ♭6,Danitonic'''
namesBySequenceIndexNegative[391] = ''''''
semitonesBySequenceIndex[391] = [0, 2, 4, 7, 8]
poemBySequenceIndex[391] = '''B F J M N'''
poemAuthors[391] = ''''''
#392: ䷾䷭ Completed Ascending: 2 2 3 1 4  : S R2 G3 P D1 : Ikoj

#393: C D E G A :0 2 4 7 9: 1 2 3 5 6: 6/9
namesBySequenceIndex[392] = '''Major Pentatonic,Ryosen (Japan),Yona Nuki Major (Japan),Man Jue (China),Gong (China),Ghana Pentatonic 2,Tezeta Major (Ethiopia),Raga Bilahari (ascending),Raga Mohanam,Raga Bhupali,Raga Deskar,Raga Kokila,Raga Jait Kalyan,Raga Bhup,Pentatonic'''
namesBySequenceIndexNegative[392] = ''''''
semitonesBySequenceIndex[392] = [0, 2, 4, 7, 9]
poemBySequenceIndex[392] = '''B F J M P'''
poemAuthors[392] = ''''''
#393: ䷾䷧ Completed Deliverance: 2 2 3 2 3  : S R2 G3 P D2 : Ikeb

#394: C D E G B♭ :0 2 4 7 10: 1 2 3 5 ♭7: 9
namesBySequenceIndex[393] = '''Dominant Pentatonic,Phropitonic'''
namesBySequenceIndexNegative[393] = ''''''
semitonesBySequenceIndex[393] = [0, 2, 4, 7, 10]
poemBySequenceIndex[393] = '''B F J M R'''
poemAuthors[393] = ''''''
#394: ䷾䷜ Completed Chasm: 2 2 3 3 2  : S R2 G3 P N2 : Ikek

#395: C D E G B :0 2 4 7 11: 1 2 3 5 7: ma9
namesBySequenceIndex[394] = '''Raga Hamsadhvani 2,Aerathitonic'''
namesBySequenceIndexNegative[394] = ''''''
semitonesBySequenceIndex[394] = [0, 2, 4, 7, 11]
poemBySequenceIndex[394] = '''B F J M S'''
poemAuthors[394] = ''''''
#395: ䷾䷃ Completed Naïve: 2 2 3 4 1  : S R2 G3 P N3 : Ikel

#396: C D E G♯ A :0 2 4 8 9: 1 2 3 ♯5 6: 6/9(♯5)
namesBySequenceIndex[395] = '''Zogitonic'''
namesBySequenceIndexNegative[395] = ''''''
semitonesBySequenceIndex[395] = [0, 2, 4, 8, 9]
poemBySequenceIndex[395] = '''B F J N P'''
poemAuthors[395] = ''''''
#396: ䷾䷽ Completed Detail: 2 2 4 1 3  : S R2 G3 D1 N1 : Ikib

#397: C D E G♯ B♭ :0 2 4 8 10: 1 2 3 ♯5 ♭7: 9(♯5)
namesBySequenceIndex[396] = '''Koditonic'''
namesBySequenceIndexNegative[396] = ''''''
semitonesBySequenceIndex[396] = [0, 2, 4, 8, 10]
poemBySequenceIndex[396] = '''B F J N R'''
poemAuthors[396] = ''''''
#397: ䷾䷦ Completed Limping: 2 2 4 2 2  : S R2 G3 D1 N2 : Ikik

#398: C D E G♯ B :0 2 4 8 11: 1 2 3 ♯5 7: ma9(♯5)
namesBySequenceIndex[397] = '''Pynitonic'''
namesBySequenceIndexNegative[397] = ''''''
semitonesBySequenceIndex[397] = [0, 2, 4, 8, 11]
poemBySequenceIndex[397] = '''B F J N S'''
poemAuthors[397] = ''''''
#398: ䷾䷳ Completed Stillness: 2 2 4 3 1  : S R2 G3 D1 N3 : Ikil

#399: C D E A B♭ :0 2 4 9 10: 1 2 3 6 ♭7: 9(no 5 add 13)
namesBySequenceIndex[398] = ''''''
namesBySequenceIndexNegative[398] = ''''''
semitonesBySequenceIndex[398] = [0, 2, 4, 9, 10]
poemBySequenceIndex[398] = '''B F J P R'''
poemAuthors[398] = ''''''
#399: ䷾䷬ Completed Clustering: 2 2 5 1 2  : S R2 G3 D2 N2 : Ikad

#400: C D E A B :0 2 4 9 11: 1 2 3 6 7: ma9(no 5 add 13)
namesBySequenceIndex[399] = '''Raga Neroshta'''
namesBySequenceIndexNegative[399] = ''''''
semitonesBySequenceIndex[399] = [0, 2, 4, 9, 11]
poemBySequenceIndex[399] = '''B F J P S'''
poemAuthors[399] = ''''''
#400: ䷾䷢ Completed Prospering: 2 2 5 2 1  : S R2 G3 D2 N3 : Ikaf

#401: C D E A♯ B :0 2 4 10 11: 1 2 3 ♯6 7: ma9(no 5 add ♭7)
namesBySequenceIndex[400] = ''''''
namesBySequenceIndexNegative[400] = ''''''
semitonesBySequenceIndex[400] = [0, 2, 4, 10, 11]
poemBySequenceIndex[400] = '''B F J R S'''
poemAuthors[400] = ''''''
#401: ䷾䷓ Completed Seeing: 2 2 6 1 1  : S R2 G3 D3 N3 : Ikam

#402: C D F F♯ G :0 2 5 6 7: 1 2 4 ♯4 5: sus4(add 9 ♯11)
namesBySequenceIndex[401] = ''''''
namesBySequenceIndexNegative[401] = ''''''
semitonesBySequenceIndex[401] = [0, 2, 5, 6, 7]
poemBySequenceIndex[401] = '''B F K L M'''
poemAuthors[401] = ''''''
#402: ䷕䷒ Adorning Arrived: 2 3 1 1 5  : S R2 M1 M2 P : IlEj

#403: C D F G♭ A♭ :0 2 5 6 8: 1 2 4 ♭5 ♭6: sus4(♯5 add 9 ♯11)
namesBySequenceIndex[402] = '''Banitonic'''
namesBySequenceIndexNegative[402] = ''''''
semitonesBySequenceIndex[402] = [0, 2, 5, 6, 8]
poemBySequenceIndex[402] = '''B F K L N'''
poemAuthors[402] = ''''''
#403: ䷕䷣ Adorning Eclipse: 2 3 1 2 4  : S R2 M1 M2 D1 : IlIj

#404: C D F G♭ A :0 2 5 6 9: 1 2 4 ♭5 6: 6/9sus4(♭5)
namesBySequenceIndex[403] = '''Phralitonic'''
namesBySequenceIndexNegative[403] = ''''''
semitonesBySequenceIndex[403] = [0, 2, 5, 6, 9]
poemBySequenceIndex[403] = '''B F K L P'''
poemAuthors[403] = ''''''
#404: ䷕䷲ Adorning Thunder: 2 3 1 3 3  : S R2 M1 M2 D2 : IlAb

#405: C D F G♭ B♭ :0 2 5 6 10: 1 2 4 ♭5 ♭7: 9sus4(♭5)
namesBySequenceIndex[404] = '''Ionaritonic'''
namesBySequenceIndexNegative[404] = ''''''
semitonesBySequenceIndex[404] = [0, 2, 5, 6, 10]
poemBySequenceIndex[404] = '''B F K L R'''
poemAuthors[404] = ''''''
#405: ䷕䷂ Adorning Sprouting: 2 3 1 4 2  : S R2 M1 M2 N2 : IlAk

#406: C D F G♭ B :0 2 5 6 11: 1 2 4 ♭5 7: ma9sus4(♭5)
namesBySequenceIndex[405] = ''''''
namesBySequenceIndexNegative[405] = ''''''
semitonesBySequenceIndex[405] = [0, 2, 5, 6, 11]
poemBySequenceIndex[405] = '''B F K L S'''
poemAuthors[405] = ''''''
#406: ䷕䷚ Adorning Nourishment: 2 3 1 5 1  : S R2 M1 M2 N3 : IlAl

#407: C D F G A♭ :0 2 5 7 8: 1 2 4 5 ♭6: sus4(add ♭6 9)
namesBySequenceIndex[406] = '''Han-Kumoi,Raga Shobhavari,Raga Sutradhari,Ionitonic'''
namesBySequenceIndexNegative[406] = ''''''
semitonesBySequenceIndex[406] = [0, 2, 5, 7, 8]
poemBySequenceIndex[406] = '''B F K M N'''
poemAuthors[406] = ''''''
#407: ䷕䷭ Adorning Ascending: 2 3 2 1 4  : S R2 M1 P D1 : Iloj

#408: C D F G A :0 2 5 7 9: 1 2 4 5 6: 6/9sus4
namesBySequenceIndex[407] = '''Ritusen,Blues Major,Scottish Pentatonic,Yo (Japan),Ritsu Gagaku (Japan),Ujo (Korea),Zhi (China),Zheng (China),Bac (Vietnam),Lai Soutsanaen (Laos),Lai Po Sai (Laos),Lai Soi (Laos),Raga Arabhi (ascending),Raga Devakriya,Raga Shree (ascending),Raga Yadukua Kambodhi (ascending),Raga Suddha Saveri,Mynitonic'''
namesBySequenceIndexNegative[407] = ''''''
semitonesBySequenceIndex[407] = [0, 2, 5, 7, 9]
poemBySequenceIndex[407] = '''B F K M P'''
poemAuthors[407] = ''''''
#408: ䷕䷧ Adorning Deliverance: 2 3 2 2 3  : S R2 M1 P D2 : Ileb

#409: C D F G B♭ :0 2 5 7 10: 1 2 4 5 ♭7: 9sus4
namesBySequenceIndex[408] = '''Egyptian,Suspended Pentatonic,Shang-Diao (China),Jin-Yu or Quin-Yu (China),Rui Bin (China),Raga Madhmat Sarang,Raga Madhyamavati,Distance Model 2 3,Thaptitonic'''
namesBySequenceIndexNegative[408] = ''''''
semitonesBySequenceIndex[408] = [0, 2, 5, 7, 10]
poemBySequenceIndex[408] = '''B F K M R'''
poemAuthors[408] = ''''''
#409: ䷕䷜ Adorning Chasm: 2 3 2 3 2  : S R2 M1 P N2 : Ilek

#410: C D F G B :0 2 5 7 11: 1 2 4 5 7: ma9sus4
namesBySequenceIndex[409] = '''Raga Desh,Tcherepnin Major Pentatonic,Daritonic'''
namesBySequenceIndexNegative[409] = ''''''
semitonesBySequenceIndex[409] = [0, 2, 5, 7, 11]
poemBySequenceIndex[409] = '''B F K M S'''
poemAuthors[409] = ''''''
#410: ䷕䷃ Adorning Naïve: 2 3 2 4 1  : S R2 M1 P N3 : Ilel

#411: C D F G♯ A :0 2 5 8 9: 1 2 4 ♯5 6: 6/9sus4(♯5)
namesBySequenceIndex[410] = '''Rothitonic'''
namesBySequenceIndexNegative[410] = ''''''
semitonesBySequenceIndex[410] = [0, 2, 5, 8, 9]
poemBySequenceIndex[410] = '''B F K N P'''
poemAuthors[410] = ''''''
#411: ䷕䷽ Adorning Detail: 2 3 3 1 3  : S R2 M1 D1 N1 : Ilib

#412: C D F G♯ B♭ :0 2 5 8 10: 1 2 4 ♯5 ♭7: 9sus4(♯5)
namesBySequenceIndex[411] = '''Chaio,Staditonic'''
namesBySequenceIndexNegative[411] = ''''''
semitonesBySequenceIndex[411] = [0, 2, 5, 8, 10]
poemBySequenceIndex[411] = '''B F K N R'''
poemAuthors[411] = ''''''
#412: ䷕䷦ Adorning Limping: 2 3 3 2 2  : S R2 M1 D1 N2 : Ilik

#413: C D F G♯ B :0 2 5 8 11: 1 2 4 ♯5 7: ma9sus4(♯5)
namesBySequenceIndex[412] = '''Raga Priyadharshini,Myditonic'''
namesBySequenceIndexNegative[412] = ''''''
semitonesBySequenceIndex[412] = [0, 2, 5, 8, 11]
poemBySequenceIndex[412] = '''B F K N S'''
poemAuthors[412] = ''''''
#413: ䷕䷳ Adorning Stillness: 2 3 3 3 1  : S R2 M1 D1 N3 : Ilil

#414: C D F A B♭ :0 2 5 9 10: 1 2 4 6 ♭7: 13(no 3 5)
namesBySequenceIndex[413] = '''Raga Guhamanohari,Pyeong Jo,Saritonic'''
namesBySequenceIndexNegative[413] = ''''''
semitonesBySequenceIndex[413] = [0, 2, 5, 9, 10]
poemBySequenceIndex[413] = '''B F K P R'''
poemAuthors[413] = ''''''
#414: ䷕䷬ Adorning Clustering: 2 3 4 1 2  : S R2 M1 D2 N2 : Ilad

#415: C D F A B :0 2 5 9 11: 1 2 4 6 7: ma13(no 3 5)
namesBySequenceIndex[414] = '''Raga Rasranjani,Koptitonic'''
namesBySequenceIndexNegative[414] = ''''''
semitonesBySequenceIndex[414] = [0, 2, 5, 9, 11]
poemBySequenceIndex[414] = '''B F K P S'''
poemAuthors[414] = ''''''
#415: ䷕䷢ Adorning Prospering: 2 3 4 2 1  : S R2 M1 D2 N3 : Ilaf

#416: C D F A♯ B :0 2 5 10 11: 1 2 4 ♯6 7: ma9sus4(no 5 add ♭7)
namesBySequenceIndex[415] = ''''''
namesBySequenceIndexNegative[415] = ''''''
semitonesBySequenceIndex[415] = [0, 2, 5, 10, 11]
poemBySequenceIndex[415] = '''B F K R S'''
poemAuthors[415] = ''''''
#416: ䷕䷓ Adorning Seeing: 2 3 5 1 1  : S R2 M1 D3 N3 : Ilam

#417: C D F♯ G A♭ :0 2 6 7 8: 1 2 ♯4 5 ♭6: sus2(add ♭6 ♯11)
namesBySequenceIndex[416] = '''Raga Asavari (ascending),Raditonic'''
namesBySequenceIndexNegative[416] = ''''''
semitonesBySequenceIndex[416] = [0, 2, 6, 7, 8]
poemBySequenceIndex[416] = '''B F L M N'''
poemAuthors[416] = ''''''
#417: ䷣䷊ Eclipse Peace: 2 4 1 1 4  : S R2 M2 P D1 : IjOj

#418: C D F♯ G A :0 2 6 7 9: 1 2 ♯4 5 6: 6sus2(add ♯11)
namesBySequenceIndex[417] = '''Raga Shri Kalyan,Ionycritonic'''
namesBySequenceIndexNegative[417] = ''''''
semitonesBySequenceIndex[417] = [0, 2, 6, 7, 9]
poemBySequenceIndex[417] = '''B F L M P'''
poemAuthors[417] = ''''''
#418: ䷣䷵ Eclipse Betrothing: 2 4 1 2 3  : S R2 M2 P D2 : IjEb

#419: C D F♯ G B♭ :0 2 6 7 10: 1 2 ♯4 5 ♭7: 9sus♯4
namesBySequenceIndex[418] = '''Epyritonic'''
namesBySequenceIndexNegative[418] = ''''''
semitonesBySequenceIndex[418] = [0, 2, 6, 7, 10]
poemBySequenceIndex[418] = '''B F L M R'''
poemAuthors[418] = ''''''
#419: ䷣䷻ Eclipse Composed: 2 4 1 3 2  : S R2 M2 P N2 : IjEk

#420: C D F♯ G B :0 2 6 7 11: 1 2 ♯4 5 7: ma9sus♯4
namesBySequenceIndex[419] = '''Raga Hamsanada,Raga Vaijayanti,Dygitonic'''
namesBySequenceIndexNegative[419] = ''''''
semitonesBySequenceIndex[419] = [0, 2, 6, 7, 11]
poemBySequenceIndex[419] = '''B F L M S'''
poemAuthors[419] = ''''''
#420: ䷣䷨ Eclipse Diminishing: 2 4 1 4 1  : S R2 M2 P N3 : IjEl

#421: C D F♯ G♯ A :0 2 6 8 9: 1 2 ♯4 ♯5 6: 6sus2(♯5 add ♯11)
namesBySequenceIndex[420] = '''Epaditonic'''
namesBySequenceIndexNegative[420] = ''''''
semitonesBySequenceIndex[420] = [0, 2, 6, 8, 9]
poemBySequenceIndex[420] = '''B F L N P'''
poemAuthors[420] = ''''''
#421: ䷣䷶ Eclipse Abounding: 2 4 2 1 3  : S R2 M2 D1 N1 : IjIb

#422: C D F♯ G♯ B♭ :0 2 6 8 10: 1 2 ♯4 ♯5 ♭7: 9sus♯4(♯5)
namesBySequenceIndex[421] = '''Tholitonic'''
namesBySequenceIndexNegative[421] = ''''''
semitonesBySequenceIndex[421] = [0, 2, 6, 8, 10]
poemBySequenceIndex[421] = '''B F L N R'''
poemAuthors[421] = ''''''
#422: ䷣䷾ Eclipse Completed: 2 4 2 2 2  : S R2 M2 D1 N2 : IjIk

#423: C D F♯ G♯ B :0 2 6 8 11: 1 2 ♯4 ♯5 7: ma9sus♯4(♯5)
namesBySequenceIndex[422] = '''Dyptitonic'''
namesBySequenceIndexNegative[422] = ''''''
semitonesBySequenceIndex[422] = [0, 2, 6, 8, 11]
poemBySequenceIndex[422] = '''B F L N S'''
poemAuthors[422] = ''''''
#423: ䷣䷕ Eclipse Adorning: 2 4 2 3 1  : S R2 M2 D1 N3 : IjIl

#424: C D G♭ A B♭ :0 2 6 9 10: 1 2 ♭5 6 ♭7: 7sus2(♭5 add 13)
namesBySequenceIndex[423] = '''Raga Shubravarni,Zanitonic'''
namesBySequenceIndexNegative[423] = ''''''
semitonesBySequenceIndex[423] = [0, 2, 6, 9, 10]
poemBySequenceIndex[423] = '''B F L P R'''
poemAuthors[423] = ''''''
#424: ䷣䷐ Eclipse Following: 2 4 3 1 2  : S R2 M2 D2 N2 : IjAd

#425: C D G♭ A B :0 2 6 9 11: 1 2 ♭5 6 7: ma7sus2(♭5 add 13)
namesBySequenceIndex[424] = '''Aeolythitonic'''
namesBySequenceIndexNegative[424] = ''''''
semitonesBySequenceIndex[424] = [0, 2, 6, 9, 11]
poemBySequenceIndex[424] = '''B F L P S'''
poemAuthors[424] = ''''''
#425: ䷣䷔ Eclipse Biting: 2 4 3 2 1  : S R2 M2 D2 N3 : IjAf

#426: C D G♭ A♯ B :0 2 6 10 11: 1 2 ♭5 ♯6 7: ma7sus2(♭5 add ♭7)
namesBySequenceIndex[425] = '''Kanitonic'''
namesBySequenceIndexNegative[425] = ''''''
semitonesBySequenceIndex[425] = [0, 2, 6, 10, 11]
poemBySequenceIndex[425] = '''B F L R S'''
poemAuthors[425] = ''''''
#426: ䷣䷩ Eclipse Augmenting: 2 4 4 1 1  : S R2 M2 D3 N3 : IjAm

#427: C D G A♭ A :0 2 7 8 9: 1 2 5 ♭6 6: 6sus2(add ♭6)
namesBySequenceIndex[426] = ''''''
namesBySequenceIndexNegative[426] = ''''''
semitonesBySequenceIndex[426] = [0, 2, 7, 8, 9]
poemBySequenceIndex[426] = '''B F M N P'''
poemAuthors[426] = ''''''
#427: ䷣䷟ Eclipse Persevering: 2 5 1 1 3  : S R2 P D1 N1 : Ijob

#428: C D G A♭ B♭ :0 2 7 8 10: 1 2 5 ♭6 ♭7: 7sus2(add ♭13)
namesBySequenceIndex[427] = ''''''
namesBySequenceIndexNegative[427] = ''''''
semitonesBySequenceIndex[427] = [0, 2, 7, 8, 10]
poemBySequenceIndex[427] = '''B F M N R'''
poemAuthors[427] = ''''''
#428: ䷣䷯ Eclipse Source: 2 5 1 2 2  : S R2 P D1 N2 : Ijok

#429: C D G A♭ B :0 2 7 8 11: 1 2 5 ♭6 7: ma7sus2(add ♭13)
namesBySequenceIndex[428] = ''''''
namesBySequenceIndexNegative[428] = ''''''
semitonesBySequenceIndex[428] = [0, 2, 7, 8, 11]
poemBySequenceIndex[428] = '''B F M N S'''
poemAuthors[428] = ''''''
#429: ䷣䷑ Eclipse Correcting: 2 5 1 3 1  : S R2 P D1 N3 : Ijol

#430: C D G A B♭ :0 2 7 9 10: 1 2 5 6 ♭7: 7sus2(add 13)
namesBySequenceIndex[429] = '''Raga Matkokil,Raga Matha Kokila'''
namesBySequenceIndexNegative[429] = ''''''
semitonesBySequenceIndex[429] = [0, 2, 7, 9, 10]
poemBySequenceIndex[429] = '''B F M P R'''
poemAuthors[429] = ''''''
#430: ䷣䷮ Eclipse Confining: 2 5 2 1 2  : S R2 P D2 N2 : Ijed

#431: C D G A B :0 2 7 9 11: 1 2 5 6 7: ma7sus2(add 13)
namesBySequenceIndex[430] = ''''''
namesBySequenceIndexNegative[430] = ''''''
semitonesBySequenceIndex[430] = [0, 2, 7, 9, 11]
poemBySequenceIndex[430] = '''B F M P S'''
poemAuthors[430] = ''''''
#431: ䷣䷿ Eclipse Unfinished: 2 5 2 2 1  : S R2 P D2 N3 : Ijef

#432: C D G A♯ B :0 2 7 10 11: 1 2 5 ♯6 7: ma7sus2(add ♭7)
namesBySequenceIndex[431] = ''''''
namesBySequenceIndexNegative[431] = ''''''
semitonesBySequenceIndex[431] = [0, 2, 7, 10, 11]
poemBySequenceIndex[431] = '''B F M R S'''
poemAuthors[431] = ''''''
#432: ䷣䷺ Eclipse Dispersing: 2 5 3 1 1  : S R2 P D3 N3 : Ijem

#433: C D G♯ A B♭ :0 2 8 9 10: 1 2 ♯5 6 ♭7: 7sus2(♯5 add 13)
namesBySequenceIndex[432] = ''''''
namesBySequenceIndexNegative[432] = ''''''
semitonesBySequenceIndex[432] = [0, 2, 8, 9, 10]
poemBySequenceIndex[432] = '''B F N P R'''
poemAuthors[432] = ''''''
#433: ䷣䷞ Eclipse Conjoining: 2 6 1 1 2  : S R2 D1 D2 N2 : Ijid

#434: C D G♯ A B :0 2 8 9 11: 1 2 ♯5 6 7: ma7sus2(♯5 add 13)
namesBySequenceIndex[433] = ''''''
namesBySequenceIndexNegative[433] = ''''''
semitonesBySequenceIndex[433] = [0, 2, 8, 9, 11]
poemBySequenceIndex[433] = '''B F N P S'''
poemAuthors[433] = ''''''
#434: ䷣䷷ Eclipse Wanderer: 2 6 1 2 1  : S R2 D1 D2 N3 : Ijif

#435: C D G♯ A♯ B :0 2 8 10 11: 1 2 ♯5 ♯6 7: ma7sus2(♯5 add ♭7)
namesBySequenceIndex[434] = ''''''
namesBySequenceIndexNegative[434] = ''''''
semitonesBySequenceIndex[434] = [0, 2, 8, 10, 11]
poemBySequenceIndex[434] = '''B F N R S'''
poemAuthors[434] = ''''''
#435: ䷣䷴ Eclipse Infiltrating: 2 6 2 1 1  : S R2 D1 N2 N3 : Ijim

#436: C D A B♭ B :0 2 9 10 11: 1 2 6 ♭7 7: ma7sus2(no 5 add ♭7 13)
namesBySequenceIndex[435] = ''''''
namesBySequenceIndexNegative[435] = ''''''
semitonesBySequenceIndex[435] = [0, 2, 9, 10, 11]
poemBySequenceIndex[435] = '''B F P R S'''
poemAuthors[435] = ''''''
#436: ䷣䷋ Eclipse Obstruction: 2 7 1 1 1  : S R2 D2 N2 N3 : Ijag

#437: C D♯ E F G♭ :0 3 4 5 6: 1 ♯2 3 4 ♭5: ma(♭5 add ♯9 11)
namesBySequenceIndex[436] = ''''''
namesBySequenceIndexNegative[436] = ''''''
semitonesBySequenceIndex[436] = [0, 3, 4, 5, 6]
poemBySequenceIndex[436] = '''B G J K L'''
poemAuthors[436] = ''''''
#437: ䷘䷗ Integrity Returning: 3 1 1 1 6  : S R3 G3 M1 M2 : AgAj

#438: C D♯ E F G :0 3 4 5 7: 1 ♯2 3 4 5: ma(add ♯9 11)
namesBySequenceIndex[437] = ''''''
namesBySequenceIndexNegative[437] = ''''''
semitonesBySequenceIndex[437] = [0, 3, 4, 5, 7]
poemBySequenceIndex[437] = '''B G J K M'''
poemAuthors[437] = ''''''
#438: ䷘䷆ Integrity Leading: 3 1 1 2 5  : S R3 G3 M1 P : Agej

#439: C D♯ E F G♯ :0 3 4 5 8: 1 ♯2 3 4 ♯5: ma(♯5 add ♯9 11)
namesBySequenceIndex[438] = '''Goritonic,Center-Cluster PentaMirror'''
namesBySequenceIndexNegative[438] = ''''''
semitonesBySequenceIndex[438] = [0, 3, 4, 5, 8]
poemBySequenceIndex[438] = '''B G J K N'''
poemAuthors[438] = ''''''
#439: ䷘䷎ Integrity Humble: 3 1 1 3 4  : S R3 G3 M1 D1 : Agij

#440: C D♯ E F A :0 3 4 5 9: 1 ♯2 3 4 6: 6(no 5 add ♯9 11)
namesBySequenceIndex[439] = '''Mothitonic'''
namesBySequenceIndexNegative[439] = ''''''
semitonesBySequenceIndex[439] = [0, 3, 4, 5, 9]
poemBySequenceIndex[439] = '''B G J K P'''
poemAuthors[439] = ''''''
#440: ䷘䷏ Integrity Confident: 3 1 1 4 3  : S R3 G3 M1 D2 : Agab

#441: C D♯ E F B♭ :0 3 4 5 10: 1 ♯2 3 4 ♭7: 11(♯9 no 5)
namesBySequenceIndex[440] = ''''''
namesBySequenceIndexNegative[440] = ''''''
semitonesBySequenceIndex[440] = [0, 3, 4, 5, 10]
poemBySequenceIndex[440] = '''B G J K R'''
poemAuthors[440] = ''''''
#441: ䷘䷇ Integrity Comparing: 3 1 1 5 2  : S R3 G3 M1 N2 : Agak

#442: C D♯ E F B :0 3 4 5 11: 1 ♯2 3 4 7: ma11(♯9 no 5)
namesBySequenceIndex[441] = ''''''
namesBySequenceIndexNegative[441] = ''''''
semitonesBySequenceIndex[441] = [0, 3, 4, 5, 11]
poemBySequenceIndex[441] = '''B G J K S'''
poemAuthors[441] = ''''''
#442: ䷘䷖ Integrity Stripping: 3 1 1 6 1  : S R3 G3 M1 N3 : Agal

#443: C D♯ E F♯ G :0 3 4 6 7: 1 ♯2 3 ♯4 5: ma(add ♯9 ♯11)
namesBySequenceIndex[442] = ''''''
namesBySequenceIndexNegative[442] = ''''''
semitonesBySequenceIndex[442] = [0, 3, 4, 6, 7]
poemBySequenceIndex[442] = '''B G J L M'''
poemAuthors[442] = ''''''
#443: ䷐䷒ Following Arrived: 3 1 2 1 5  : S R3 G3 M2 P : AdEj

#444: C D♯ E F♯ G♯ :0 3 4 6 8: 1 ♯2 3 ♯4 ♯5: ma(♯5 add ♯9 ♯11)
namesBySequenceIndex[443] = '''Gylitonic'''
namesBySequenceIndexNegative[443] = ''''''
semitonesBySequenceIndex[443] = [0, 3, 4, 6, 8]
poemBySequenceIndex[443] = '''B G J L N'''
poemAuthors[443] = ''''''
#444: ䷐䷣ Following Eclipse: 3 1 2 2 4  : S R3 G3 M2 D1 : AdIj

#445: C D♯ E G♭ A :0 3 4 6 9: 1 ♯2 3 ♭5 6: 6(♭5 add ♯9)
namesBySequenceIndex[444] = '''Bycritonic'''
namesBySequenceIndexNegative[444] = ''''''
semitonesBySequenceIndex[444] = [0, 3, 4, 6, 9]
poemBySequenceIndex[444] = '''B G J L P'''
poemAuthors[444] = ''''''
#445: ䷐䷲ Following Thunder: 3 1 2 3 3  : S R3 G3 M2 D2 : AdAb

#446: C D♯ E G♭ B♭ :0 3 4 6 10: 1 ♯2 3 ♭5 ♭7: 7(♭5 add ♯9)
namesBySequenceIndex[445] = '''Locrian Pentatonic,Aeronitonic'''
namesBySequenceIndexNegative[445] = ''''''
semitonesBySequenceIndex[445] = [0, 3, 4, 6, 10]
poemBySequenceIndex[445] = '''B G J L R'''
poemAuthors[445] = ''''''
#446: ䷐䷂ Following Sprouting: 3 1 2 4 2  : S R3 G3 M2 N2 : AdAk

#447: C D♯ E G♭ B :0 3 4 6 11: 1 ♯2 3 ♭5 7: ma7(♭5 add ♯9)
namesBySequenceIndex[446] = ''''''
namesBySequenceIndexNegative[446] = ''''''
semitonesBySequenceIndex[446] = [0, 3, 4, 6, 11]
poemBySequenceIndex[446] = '''B G J L S'''
poemAuthors[446] = ''''''
#447: ䷐䷚ Following Nourishment: 3 1 2 5 1  : S R3 G3 M2 N3 : AdAl

#448: C D♯ E G A♭ :0 3 4 7 8: 1 ♯2 3 5 ♭6: ma(add ♭6 ♯9)
namesBySequenceIndex[447] = '''Laritonic,Altered Pentatonic'''
namesBySequenceIndexNegative[447] = ''''''
semitonesBySequenceIndex[447] = [0, 3, 4, 7, 8]
poemBySequenceIndex[447] = '''B G J M N'''
poemAuthors[447] = ''''''
#448: ䷐䷭ Following Ascending: 3 1 3 1 4  : S R3 G3 P D1 : Adoj

#449: C D♯ E G A :0 3 4 7 9: 1 ♯2 3 5 6: 6(add ♯9)
namesBySequenceIndex[448] = '''Raga Mohanangi,Mythitonic'''
namesBySequenceIndexNegative[448] = ''''''
semitonesBySequenceIndex[448] = [0, 3, 4, 7, 9]
poemBySequenceIndex[448] = '''B G J M P'''
poemAuthors[448] = ''''''
#449: ䷐䷧ Following Deliverance: 3 1 3 2 3  : S R3 G3 P D2 : Adeb

#450: C D♯ E G B♭ :0 3 4 7 10: 1 ♯2 3 5 ♭7: 7(add ♯9)
namesBySequenceIndex[449] = '''Garitonic,Major Pentatonic ♭7 ♯9'''
namesBySequenceIndexNegative[449] = ''''''
semitonesBySequenceIndex[449] = [0, 3, 4, 7, 10]
poemBySequenceIndex[449] = '''B G J M R'''
poemAuthors[449] = ''''''
#450: ䷐䷜ Following Chasm: 3 1 3 3 2  : S R3 G3 P N2 : Adek

#451: C D♯ E G B :0 3 4 7 11: 1 ♯2 3 5 7: ma7(add ♯9)
namesBySequenceIndex[450] = '''Ionagitonic'''
namesBySequenceIndexNegative[450] = ''''''
semitonesBySequenceIndex[450] = [0, 3, 4, 7, 11]
poemBySequenceIndex[450] = '''B G J M S'''
poemAuthors[450] = ''''''
#451: ䷐䷃ Following Naïve: 3 1 3 4 1  : S R3 G3 P N3 : Adel

#452: C D♯ E G♯ A :0 3 4 8 9: 1 ♯2 3 ♯5 6: 6(♯5 add ♯9)
namesBySequenceIndex[451] = '''Mocritonic'''
namesBySequenceIndexNegative[451] = ''''''
semitonesBySequenceIndex[451] = [0, 3, 4, 8, 9]
poemBySequenceIndex[451] = '''B G J N P'''
poemAuthors[451] = ''''''
#452: ䷐䷽ Following Detail: 3 1 4 1 3  : S R3 G3 D1 N1 : Adib

#453: C D♯ E G♯ B♭ :0 3 4 8 10: 1 ♯2 3 ♯5 ♭7: 7(♯5 add ♯9)
namesBySequenceIndex[452] = '''Dynitonic'''
namesBySequenceIndexNegative[452] = ''''''
semitonesBySequenceIndex[452] = [0, 3, 4, 8, 10]
poemBySequenceIndex[452] = '''B G J N R'''
poemAuthors[452] = ''''''
#453: ䷐䷦ Following Limping: 3 1 4 2 2  : S R3 G3 D1 N2 : Adik

#454: C D♯ E G♯ B :0 3 4 8 11: 1 ♯2 3 ♯5 7: ma7(♯5 add ♯9)
namesBySequenceIndex[453] = '''Styditonic'''
namesBySequenceIndexNegative[453] = ''''''
semitonesBySequenceIndex[453] = [0, 3, 4, 8, 11]
poemBySequenceIndex[453] = '''B G J N S'''
poemAuthors[453] = ''''''
#454: ䷐䷳ Following Stillness: 3 1 4 3 1  : S R3 G3 D1 N3 : Adil

#455: C D♯ E A B♭ :0 3 4 9 10: 1 ♯2 3 6 ♭7: 7(no 5 add ♯9 13)
namesBySequenceIndex[454] = ''''''
namesBySequenceIndexNegative[454] = ''''''
semitonesBySequenceIndex[454] = [0, 3, 4, 9, 10]
poemBySequenceIndex[454] = '''B G J P R'''
poemAuthors[454] = ''''''
#455: ䷐䷬ Following Clustering: 3 1 5 1 2  : S R3 G3 D2 N2 : Adad

#456: C D♯ E A B :0 3 4 9 11: 1 ♯2 3 6 7: ma7(no 5 add ♯9 13)
namesBySequenceIndex[455] = ''''''
namesBySequenceIndexNegative[455] = ''''''
semitonesBySequenceIndex[455] = [0, 3, 4, 9, 11]
poemBySequenceIndex[455] = '''B G J P S'''
poemAuthors[455] = ''''''
#456: ䷐䷢ Following Prospering: 3 1 5 2 1  : S R3 G3 D2 N3 : Adaf

#457: C D♯ E A♯ B :0 3 4 10 11: 1 ♯2 3 ♯6 7: ma7(no 5 add ♭7 ♯9)
namesBySequenceIndex[456] = ''''''
namesBySequenceIndexNegative[456] = ''''''
semitonesBySequenceIndex[456] = [0, 3, 4, 10, 11]
poemBySequenceIndex[456] = '''B G J R S'''
poemAuthors[456] = ''''''
#457: ䷐䷓ Following Seeing: 3 1 6 1 1  : S R3 G3 D3 N3 : Adam

#458: C E♭ F F♯ G :0 3 5 6 7: 1 ♭3 4 ♯4 5: mi(add 11 ♯11)
namesBySequenceIndex[457] = ''''''
namesBySequenceIndexNegative[457] = ''''''
semitonesBySequenceIndex[457] = [0, 3, 5, 6, 7]
poemBySequenceIndex[457] = '''B G K L M'''
poemAuthors[457] = ''''''
#458: ䷔䷒ Biting Arrived: 3 2 1 1 5  : S G2 M1 M2 P : AfEj

#459: C E♭ F G♭ A♭ :0 3 5 6 8: 1 ♭3 4 ♭5 ♭6: °(add ♭6 11)
namesBySequenceIndex[458] = '''Bocritonic'''
namesBySequenceIndexNegative[458] = ''''''
semitonesBySequenceIndex[458] = [0, 3, 5, 6, 8]
poemBySequenceIndex[458] = '''B G K L N'''
poemAuthors[458] = ''''''
#459: ䷔䷣ Biting Eclipse: 3 2 1 2 4  : S G2 M1 M2 D1 : AfIj

#460: C E♭ F G♭ A :0 3 5 6 9: 1 ♭3 4 ♭5 6: °7(add 11)
namesBySequenceIndex[459] = '''Katycritonic'''
namesBySequenceIndexNegative[459] = ''''''
semitonesBySequenceIndex[459] = [0, 3, 5, 6, 9]
poemBySequenceIndex[459] = '''B G K L P'''
poemAuthors[459] = ''''''
#460: ䷔䷲ Biting Thunder: 3 2 1 3 3  : S G2 M1 M2 D2 : AfAb

#461: C E♭ F G♭ B♭ :0 3 5 6 10: 1 ♭3 4 ♭5 ♭7: ⌀11(♯9)
namesBySequenceIndex[460] = '''Raga Jayakauns,The Spook,Minor Pentatonic ♭5,Phrynitonic'''
namesBySequenceIndexNegative[460] = ''''''
semitonesBySequenceIndex[460] = [0, 3, 5, 6, 10]
poemBySequenceIndex[460] = '''B G K L R'''
poemAuthors[460] = ''''''
#461: ䷔䷂ Biting Sprouting: 3 2 1 4 2  : S G2 M1 M2 N2 : AfAk

#462: C E♭ F G♭ B :0 3 5 6 11: 1 ♭3 4 ♭5 7: °ma11(♯9)
namesBySequenceIndex[461] = ''''''
namesBySequenceIndexNegative[461] = ''''''
semitonesBySequenceIndex[461] = [0, 3, 5, 6, 11]
poemBySequenceIndex[461] = '''B G K L S'''
poemAuthors[461] = ''''''
#462: ䷔䷚ Biting Nourishment: 3 2 1 5 1  : S G2 M1 M2 N3 : AfAl

#463: C E♭ F G A♭ :0 3 5 7 8: 1 ♭3 4 5 ♭6: mi(add ♭6 11)
namesBySequenceIndex[462] = '''Raga Kokil Pancham,Daditonic'''
namesBySequenceIndexNegative[462] = ''''''
semitonesBySequenceIndex[462] = [0, 3, 5, 7, 8]
poemBySequenceIndex[462] = '''B G K M N'''
poemAuthors[462] = ''''''
#463: ䷔䷭ Biting Ascending: 3 2 2 1 4  : S G2 M1 P D1 : Afoj

#464: C E♭ F G A :0 3 5 7 9: 1 ♭3 4 5 6: mi6(add 11)
namesBySequenceIndex[463] = '''Kyemyonjo,Kyemyonjo (Korea),Sylitonic'''
namesBySequenceIndexNegative[463] = ''''''
semitonesBySequenceIndex[463] = [0, 3, 5, 7, 9]
poemBySequenceIndex[463] = '''B G K M P'''
poemAuthors[463] = ''''''
#464: ䷔䷧ Biting Deliverance: 3 2 2 2 3  : S G2 M1 P D2 : Afeb

#465: C E♭ F G B♭ :0 3 5 7 10: 1 ♭3 4 5 ♭7: mi11(♯9)
namesBySequenceIndex[464] = '''Minor Pentatonic,Blues Minor Pentatonic,Bati (Ethiopia),Qing Shang (China),Gu Xian (China),Jia Zhong (China),Yu 2 (China),P'yongjo-kyemyonjo (Korea),Lai Yai (Laos),Lai Noi (Laos),Raga Dhani,Raga Abheri,Raga Dhanyasi (ascending),Raga Bhimpalasi (ascending),Raga Udhayaravi,Rocritonic'''
namesBySequenceIndexNegative[464] = ''''''
semitonesBySequenceIndex[464] = [0, 3, 5, 7, 10]
poemBySequenceIndex[464] = '''B G K M R'''
poemAuthors[464] = ''''''
#465: ䷔䷜ Biting Chasm: 3 2 2 3 2  : S G2 M1 P N2 : Afek

#466: C E♭ F G B :0 3 5 7 11: 1 ♭3 4 5 7: mima11(♯9)
namesBySequenceIndex[465] = '''Raga Udayaravicandrika,Kagitonic'''
namesBySequenceIndexNegative[465] = ''''''
semitonesBySequenceIndex[465] = [0, 3, 5, 7, 11]
poemBySequenceIndex[465] = '''B G K M S'''
poemAuthors[465] = ''''''
#466: ䷔䷃ Biting Naïve: 3 2 2 4 1  : S G2 M1 P N3 : Afel

#467: C E♭ F G♯ A :0 3 5 8 9: 1 ♭3 4 ♯5 6: mi6(♯5 add 11)
namesBySequenceIndex[466] = '''Dogitonic'''
namesBySequenceIndexNegative[466] = ''''''
semitonesBySequenceIndex[466] = [0, 3, 5, 8, 9]
poemBySequenceIndex[466] = '''B G K N P'''
poemAuthors[466] = ''''''
#467: ䷔䷽ Biting Detail: 3 2 3 1 3  : S G2 M1 D1 N1 : Afib

#468: C E♭ F G♯ B♭ :0 3 5 8 10: 1 ♭3 4 ♯5 ♭7: mi11(♯5 ♯9)
namesBySequenceIndex[467] = '''Man Gong,Blues Minor,Minyo (Japan),Jiao (China),Quan Ming (China),Yi Ze (China),Raga Hindolam,Epathitonic'''
namesBySequenceIndexNegative[467] = ''''''
semitonesBySequenceIndex[467] = [0, 3, 5, 8, 10]
poemBySequenceIndex[467] = '''B G K N R'''
poemAuthors[467] = ''''''
#468: ䷔䷦ Biting Limping: 3 2 3 2 2  : S G2 M1 D1 N2 : Afik

#469: C E♭ F G♯ B :0 3 5 8 11: 1 ♭3 4 ♯5 7: mima11(♯5 ♯9)
namesBySequenceIndex[468] = '''Gothitonic,Raga Chandrakauns Kiravani'''
namesBySequenceIndexNegative[468] = ''''''
semitonesBySequenceIndex[468] = [0, 3, 5, 8, 11]
poemBySequenceIndex[468] = '''B G K N S'''
poemAuthors[468] = ''''''
#469: ䷔䷳ Biting Stillness: 3 2 3 3 1  : S G2 M1 D1 N3 : Afil

#470: C E♭ F A B♭ :0 3 5 9 10: 1 ♭3 4 6 ♭7: mi13(♯9 no 5)
namesBySequenceIndex[469] = '''Raga Surya,Raga Varamu,Raga Chandrakauns Kafi,Lonitonic'''
namesBySequenceIndexNegative[469] = ''''''
semitonesBySequenceIndex[469] = [0, 3, 5, 9, 10]
poemBySequenceIndex[469] = '''B G K P R'''
poemAuthors[469] = ''''''
#470: ䷔䷬ Biting Clustering: 3 2 4 1 2  : S G2 M1 D2 N2 : Afad

#471: C E♭ F A B :0 3 5 9 11: 1 ♭3 4 6 7: mima13(♯9 no 5)
namesBySequenceIndex[470] = '''Raga Rajeshwari,Raga Chandrakauns Modern,Raga Marga Hindola,Docritonic'''
namesBySequenceIndexNegative[470] = ''''''
semitonesBySequenceIndex[470] = [0, 3, 5, 9, 11]
poemBySequenceIndex[470] = '''B G K P S'''
poemAuthors[470] = ''''''
#471: ䷔䷢ Biting Prospering: 3 2 4 2 1  : S G2 M1 D2 N3 : Afaf

#472: C E♭ F A♯ B :0 3 5 10 11: 1 ♭3 4 ♯6 7: mima11(♯9 no 5 add ♭7)
namesBySequenceIndex[471] = ''''''
namesBySequenceIndexNegative[471] = ''''''
semitonesBySequenceIndex[471] = [0, 3, 5, 10, 11]
poemBySequenceIndex[471] = '''B G K R S'''
poemAuthors[471] = ''''''
#472: ䷔䷓ Biting Seeing: 3 2 5 1 1  : S G2 M1 D3 N3 : Afam

#473: C E♭ F♯ G A♭ :0 3 6 7 8: 1 ♭3 ♯4 5 ♭6: mi(add ♭6 ♯11)
namesBySequenceIndex[472] = '''Staptitonic'''
namesBySequenceIndexNegative[472] = ''''''
semitonesBySequenceIndex[472] = [0, 3, 6, 7, 8]
poemBySequenceIndex[472] = '''B G L M N'''
poemAuthors[472] = ''''''
#473: ䷲䷊ Thunder Peace: 3 3 1 1 4  : S G2 M2 P D1 : AbOj

#474: C E♭ F♯ G A :0 3 6 7 9: 1 ♭3 ♯4 5 6: mi6(add ♯11)
namesBySequenceIndex[473] = '''Thoptitonic'''
namesBySequenceIndexNegative[473] = ''''''
semitonesBySequenceIndex[473] = [0, 3, 6, 7, 9]
poemBySequenceIndex[473] = '''B G L M P'''
poemAuthors[473] = ''''''
#474: ䷲䷵ Thunder Betrothing: 3 3 1 2 3  : S G2 M2 P D2 : AbEb

#475: C E♭ F♯ G B♭ :0 3 6 7 10: 1 ♭3 ♯4 5 ♭7: mi7(add ♯11)
namesBySequenceIndex[474] = '''Raga Samudhra Priya,Lyditonic'''
namesBySequenceIndexNegative[474] = ''''''
semitonesBySequenceIndex[474] = [0, 3, 6, 7, 10]
poemBySequenceIndex[474] = '''B G L M R'''
poemAuthors[474] = ''''''
#475: ䷲䷻ Thunder Composed: 3 3 1 3 2  : S G2 M2 P N2 : AbEk

#476: C E♭ F♯ G B :0 3 6 7 11: 1 ♭3 ♯4 5 7: mima7(add ♯11)
namesBySequenceIndex[475] = '''Raga Multani 2,Kygitonic'''
namesBySequenceIndexNegative[475] = ''''''
semitonesBySequenceIndex[475] = [0, 3, 6, 7, 11]
poemBySequenceIndex[475] = '''B G L M S'''
poemAuthors[475] = ''''''
#476: ䷲䷨ Thunder Diminishing: 3 3 1 4 1  : S G2 M2 P N3 : AbEl

#477: C E♭ F♯ G♯ A :0 3 6 8 9: 1 ♭3 ♯4 ♯5 6: °7(add ♭6)
namesBySequenceIndex[476] = '''Phrothitonic'''
namesBySequenceIndexNegative[476] = ''''''
semitonesBySequenceIndex[476] = [0, 3, 6, 8, 9]
poemBySequenceIndex[476] = '''B G L N P'''
poemAuthors[476] = ''''''
#477: ䷲䷶ Thunder Abounding: 3 3 2 1 3  : S G2 M2 D1 N1 : AbIb

#478: C E♭ F♯ G♯ B♭ :0 3 6 8 10: 1 ♭3 ♯4 ♯5 ♭7: ⌀7(add ♭13)
namesBySequenceIndex[477] = '''Chin,Raga Harikauns,Kataritonic'''
namesBySequenceIndexNegative[477] = ''''''
semitonesBySequenceIndex[477] = [0, 3, 6, 8, 10]
poemBySequenceIndex[477] = '''B G L N R'''
poemAuthors[477] = ''''''
#478: ䷲䷾ Thunder Completed: 3 3 2 2 2  : S G2 M2 D1 N2 : AbIk

#479: C E♭ F♯ G♯ B :0 3 6 8 11: 1 ♭3 ♯4 ♯5 7: °ma7(add ♭13)
namesBySequenceIndex[478] = '''Diminished Pentatonic,Thoditonic'''
namesBySequenceIndexNegative[478] = ''''''
semitonesBySequenceIndex[478] = [0, 3, 6, 8, 11]
poemBySequenceIndex[478] = '''B G L N S'''
poemAuthors[478] = ''''''
#479: ䷲䷕ Thunder Adorning: 3 3 2 3 1  : S G2 M2 D1 N3 : AbIl

#480: C E♭ G♭ A B♭ :0 3 6 9 10: 1 ♭3 ♭5 6 ♭7: ⌀7(add 13)
namesBySequenceIndex[479] = '''Raga Bageshri (ascending),Thyritonic'''
namesBySequenceIndexNegative[479] = ''''''
semitonesBySequenceIndex[479] = [0, 3, 6, 9, 10]
poemBySequenceIndex[479] = '''B G L P R'''
poemAuthors[479] = ''''''
#480: ䷲䷐ Thunder Following: 3 3 3 1 2  : S G2 M2 D2 N2 : AbAd

#481: C E♭ G♭ A B :0 3 6 9 11: 1 ♭3 ♭5 6 7: °ma7(add 13)
namesBySequenceIndex[480] = '''Beta Chord,Mixitonic'''
namesBySequenceIndexNegative[480] = ''''''
semitonesBySequenceIndex[480] = [0, 3, 6, 9, 11]
poemBySequenceIndex[480] = '''B G L P S'''
poemAuthors[480] = ''''''
#481: ䷲䷔ Thunder Biting: 3 3 3 2 1  : S G2 M2 D2 N3 : AbAf

#482: C E♭ G♭ A♯ B :0 3 6 10 11: 1 ♭3 ♭5 ♯6 7: °ma7(add ♭7)
namesBySequenceIndex[481] = '''Stolitonic'''
namesBySequenceIndexNegative[481] = ''''''
semitonesBySequenceIndex[481] = [0, 3, 6, 10, 11]
poemBySequenceIndex[481] = '''B G L R S'''
poemAuthors[481] = ''''''
#482: ䷲䷩ Thunder Augmenting: 3 3 4 1 1  : S G2 M2 D3 N3 : AbAm

#483: C E♭ G A♭ A :0 3 7 8 9: 1 ♭3 5 ♭6 6: mi6(add ♭6)
namesBySequenceIndex[482] = '''Bylitonic'''
namesBySequenceIndexNegative[482] = ''''''
semitonesBySequenceIndex[482] = [0, 3, 7, 8, 9]
poemBySequenceIndex[482] = '''B G M N P'''
poemAuthors[482] = ''''''
#483: ䷲䷟ Thunder Persevering: 3 4 1 1 3  : S G2 P D1 N1 : Abob

#484: C E♭ G A♭ B♭ :0 3 7 8 10: 1 ♭3 5 ♭6 ♭7: mi7(add ♭13)
namesBySequenceIndex[483] = '''Raga Varini,Raga Shailaja,Zoptitonic'''
namesBySequenceIndexNegative[483] = ''''''
semitonesBySequenceIndex[483] = [0, 3, 7, 8, 10]
poemBySequenceIndex[483] = '''B G M N R'''
poemAuthors[483] = ''''''
#484: ䷲䷯ Thunder Source: 3 4 1 2 2  : S G2 P D1 N2 : Abok

#485: C E♭ G A♭ B :0 3 7 8 11: 1 ♭3 5 ♭6 7: mima7(add ♭13)
namesBySequenceIndex[484] = '''Kyritonic'''
namesBySequenceIndexNegative[484] = ''''''
semitonesBySequenceIndex[484] = [0, 3, 7, 8, 11]
poemBySequenceIndex[484] = '''B G M N S'''
poemAuthors[484] = ''''''
#485: ䷲䷑ Thunder Correcting: 3 4 1 3 1  : S G2 P D1 N3 : Abol

#486: C E♭ G A B♭ :0 3 7 9 10: 1 ♭3 5 6 ♭7: mi7(add 13)
namesBySequenceIndex[485] = '''Thocritonic'''
namesBySequenceIndexNegative[485] = ''''''
semitonesBySequenceIndex[485] = [0, 3, 7, 9, 10]
poemBySequenceIndex[485] = '''B G M P R'''
poemAuthors[485] = ''''''
#486: ䷲䷮ Thunder Confining: 3 4 2 1 2  : S G2 P D2 N2 : Abed

#487: C E♭ G A B :0 3 7 9 11: 1 ♭3 5 6 7: mima7(add 13)
namesBySequenceIndex[486] = '''Katagitonic'''
namesBySequenceIndexNegative[486] = ''''''
semitonesBySequenceIndex[486] = [0, 3, 7, 9, 11]
poemBySequenceIndex[486] = '''B G M P S'''
poemAuthors[486] = ''''''
#487: ䷲䷿ Thunder Unfinished: 3 4 2 2 1  : S G2 P D2 N3 : Abef

#488: C E♭ G A♯ B :0 3 7 10 11: 1 ♭3 5 ♯6 7: mima7(add ♭7)
namesBySequenceIndex[487] = '''Aeraphitonic'''
namesBySequenceIndexNegative[487] = ''''''
semitonesBySequenceIndex[487] = [0, 3, 7, 10, 11]
poemBySequenceIndex[487] = '''B G M R S'''
poemAuthors[487] = ''''''
#488: ䷲䷺ Thunder Dispersing: 3 4 3 1 1  : S G2 P D3 N3 : Abem

#489: C E♭ G♯ A B♭ :0 3 8 9 10: 1 ♭3 ♯5 6 ♭7: mi7(♯5 add 13)
namesBySequenceIndex[488] = ''''''
namesBySequenceIndexNegative[488] = ''''''
semitonesBySequenceIndex[488] = [0, 3, 8, 9, 10]
poemBySequenceIndex[488] = '''B G N P R'''
poemAuthors[488] = ''''''
#489: ䷲䷞ Thunder Conjoining: 3 5 1 1 2  : S G2 D1 D2 N2 : Abid

#490: C E♭ G♯ A B :0 3 8 9 11: 1 ♭3 ♯5 6 7: mima7(♯5 add 13)
namesBySequenceIndex[489] = ''''''
namesBySequenceIndexNegative[489] = ''''''
semitonesBySequenceIndex[489] = [0, 3, 8, 9, 11]
poemBySequenceIndex[489] = '''B G N P S'''
poemAuthors[489] = ''''''
#490: ䷲䷷ Thunder Wanderer: 3 5 1 2 1  : S G2 D1 D2 N3 : Abif

#491: C E♭ G♯ A♯ B :0 3 8 10 11: 1 ♭3 ♯5 ♯6 7: mima7(♯5 add ♭7)
namesBySequenceIndex[490] = ''''''
namesBySequenceIndexNegative[490] = ''''''
semitonesBySequenceIndex[490] = [0, 3, 8, 10, 11]
poemBySequenceIndex[490] = '''B G N R S'''
poemAuthors[490] = ''''''
#491: ䷲䷴ Thunder Infiltrating: 3 5 2 1 1  : S G2 D1 N2 N3 : Abim

#492: C E♭ A B♭ B :0 3 9 10 11: 1 ♭3 6 ♭7 7: mima7(no 5 add ♭7 13)
namesBySequenceIndex[491] = ''''''
namesBySequenceIndexNegative[491] = ''''''
semitonesBySequenceIndex[491] = [0, 3, 9, 10, 11]
poemBySequenceIndex[491] = '''B G P R S'''
poemAuthors[491] = ''''''
#492: ䷲䷋ Thunder Obstruction: 3 6 1 1 1  : S G2 D2 N2 N3 : Abag

#493: C E F F♯ G :0 4 5 6 7: 1 3 4 ♯4 5: ma(add 11 ♯11)
namesBySequenceIndex[492] = ''''''
namesBySequenceIndexNegative[492] = ''''''
semitonesBySequenceIndex[492] = [0, 4, 5, 6, 7]
poemBySequenceIndex[492] = '''B J K L M'''
poemAuthors[492] = ''''''
#493: ䷩䷒ Augmenting Arrived: 4 1 1 1 5  : S G3 M1 M2 P : AmEj

#494: C E F G♭ A♭ :0 4 5 6 8: 1 3 4 ♭5 ♭6: ma(♯5 add 11 ♯11)
namesBySequenceIndex[493] = '''Laditonic'''
namesBySequenceIndexNegative[493] = ''''''
semitonesBySequenceIndex[493] = [0, 4, 5, 6, 8]
poemBySequenceIndex[493] = '''B J K L N'''
poemAuthors[493] = ''''''
#494: ䷩䷣ Augmenting Eclipse: 4 1 1 2 4  : S G3 M1 M2 D1 : AmIj

#495: C E F G♭ A :0 4 5 6 9: 1 3 4 ♭5 6: 6(♭5 add 11)
namesBySequenceIndex[494] = '''Ionyptitonic'''
namesBySequenceIndexNegative[494] = ''''''
semitonesBySequenceIndex[494] = [0, 4, 5, 6, 9]
poemBySequenceIndex[494] = '''B J K L P'''
poemAuthors[494] = ''''''
#495: ䷩䷲ Augmenting Thunder: 4 1 1 3 3  : S G3 M1 M2 D2 : AmAb

#496: C E F G♭ B♭ :0 4 5 6 10: 1 3 4 ♭5 ♭7: 7(♭5 add 11)
namesBySequenceIndex[495] = '''Stonitonic'''
namesBySequenceIndexNegative[495] = ''''''
semitonesBySequenceIndex[495] = [0, 4, 5, 6, 10]
poemBySequenceIndex[495] = '''B J K L R'''
poemAuthors[495] = ''''''
#496: ䷩䷂ Augmenting Sprouting: 4 1 1 4 2  : S G3 M1 M2 N2 : AmAk

#497: C E F G♭ B :0 4 5 6 11: 1 3 4 ♭5 7: ma7(♭5 add 11)
namesBySequenceIndex[496] = ''''''
namesBySequenceIndexNegative[496] = ''''''
semitonesBySequenceIndex[496] = [0, 4, 5, 6, 11]
poemBySequenceIndex[496] = '''B J K L S'''
poemAuthors[496] = ''''''
#497: ䷩䷚ Augmenting Nourishment: 4 1 1 5 1  : S G3 M1 M2 N3 : AmAl

#498: C E F G A♭ :0 4 5 7 8: 1 3 4 5 ♭6: ma(add ♭6 11)
namesBySequenceIndex[497] = '''Poritonic'''
namesBySequenceIndexNegative[497] = ''''''
semitonesBySequenceIndex[497] = [0, 4, 5, 7, 8]
poemBySequenceIndex[497] = '''B J K M N'''
poemAuthors[497] = ''''''
#498: ䷩䷭ Augmenting Ascending: 4 1 2 1 4  : S G3 M1 P D1 : Amoj

#499: C E F G A :0 4 5 7 9: 1 3 4 5 6: 6(add 11)
namesBySequenceIndex[498] = '''Raga Mand,Raga Nagaswaravali,Lothitonic'''
namesBySequenceIndexNegative[498] = ''''''
semitonesBySequenceIndex[498] = [0, 4, 5, 7, 9]
poemBySequenceIndex[498] = '''B J K M P'''
poemAuthors[498] = ''''''
#499: ䷩䷧ Augmenting Deliverance: 4 1 2 2 3  : S G3 M1 P D2 : Ameb

#500: C E F G B♭ :0 4 5 7 10: 1 3 4 5 ♭7: 7(add 11)
namesBySequenceIndex[499] = '''Jog,Raga Savethri,Mixolydian Pentatonic,Jog (India),Aeracritonic'''
namesBySequenceIndexNegative[499] = ''''''
semitonesBySequenceIndex[499] = [0, 4, 5, 7, 10]
poemBySequenceIndex[499] = '''B J K M R'''
poemAuthors[499] = ''''''
#500: ䷩䷜ Augmenting Chasm: 4 1 2 3 2  : S G3 M1 P N2 : Amek

#501: C E F G B :0 4 5 7 11: 1 3 4 5 7: ma7(add 11)
namesBySequenceIndex[500] = '''Ryukyu,Raga Keda,Ionian Pentatonic,Pelog Degung Modern (Bali),Melog Selisir,Raga Jaganmohini (ascending),Raga Kedaram (ascending),Raga Nata/Nattai (descending),Raga Bihag (ascending),Raga Gambhiranata,Raga Neelambari,Raga Nalinakanti,Ryukyu (Japan),Gonitonic'''
namesBySequenceIndexNegative[500] = ''''''
semitonesBySequenceIndex[500] = [0, 4, 5, 7, 11]
poemBySequenceIndex[500] = '''B J K M S'''
poemAuthors[500] = ''''''
#501: ䷩䷃ Augmenting Naïve: 4 1 2 4 1  : S G3 M1 P N3 : Amel

#502: C E F G♯ A :0 4 5 8 9: 1 3 4 ♯5 6: 6(♯5 add 11)
namesBySequenceIndex[501] = '''Zothitonic'''
namesBySequenceIndexNegative[501] = ''''''
semitonesBySequenceIndex[501] = [0, 4, 5, 8, 9]
poemBySequenceIndex[501] = '''B J K N P'''
poemAuthors[501] = ''''''
#502: ䷩䷽ Augmenting Detail: 4 1 3 1 3  : S G3 M1 D1 N1 : Amib

#503: C E F G♯ B♭ :0 4 5 8 10: 1 3 4 ♯5 ♭7: 7(♯5 add 11)
namesBySequenceIndex[502] = '''Epygitonic'''
namesBySequenceIndexNegative[502] = ''''''
semitonesBySequenceIndex[502] = [0, 4, 5, 8, 10]
poemBySequenceIndex[502] = '''B J K N R'''
poemAuthors[502] = ''''''
#503: ䷩䷦ Augmenting Limping: 4 1 3 2 2  : S G3 M1 D1 N2 : Amik

#504: C E F G♯ B :0 4 5 8 11: 1 3 4 ♯5 7: ma7(♯5 add 11)
namesBySequenceIndex[503] = '''Bacovia,Raga Girija,Raga Bhairav (ascending),Lycritonic,Bocavia (Romania)'''
namesBySequenceIndexNegative[503] = ''''''
semitonesBySequenceIndex[503] = [0, 4, 5, 8, 11]
poemBySequenceIndex[503] = '''B J K N S'''
poemAuthors[503] = ''''''
#504: ䷩䷳ Augmenting Stillness: 4 1 3 3 1  : S G3 M1 D1 N3 : Amil

#505: C E F A B♭ :0 4 5 9 10: 1 3 4 6 ♭7: 13(no 5 9)
namesBySequenceIndex[504] = '''Raga Khamaji Durga,Phraditonic'''
namesBySequenceIndexNegative[504] = ''''''
semitonesBySequenceIndex[504] = [0, 4, 5, 9, 10]
poemBySequenceIndex[504] = '''B J K P R'''
poemAuthors[504] = ''''''
#505: ䷩䷬ Augmenting Clustering: 4 1 4 1 2  : S G3 M1 D2 N2 : Amad

#506: C E F A B :0 4 5 9 11: 1 3 4 6 7: ma13(no 5 9)
namesBySequenceIndex[505] = '''Raga Vasanta (ascending),Raga Kaushikdhvani,Paptitonic'''
namesBySequenceIndexNegative[505] = ''''''
semitonesBySequenceIndex[505] = [0, 4, 5, 9, 11]
poemBySequenceIndex[505] = '''B J K P S'''
poemAuthors[505] = ''''''
#506: ䷩䷢ Augmenting Prospering: 4 1 4 2 1  : S G3 M1 D2 N3 : Amaf

#507: C E F A♯ B :0 4 5 10 11: 1 3 4 ♯6 7: ma7(no 5 add ♭7 11)
namesBySequenceIndex[506] = ''''''
namesBySequenceIndexNegative[506] = ''''''
semitonesBySequenceIndex[506] = [0, 4, 5, 10, 11]
poemBySequenceIndex[506] = '''B J K R S'''
poemAuthors[506] = ''''''
#507: ䷩䷓ Augmenting Seeing: 4 1 5 1 1  : S G3 M1 D3 N3 : Amam

#508: C E F♯ G A♭ :0 4 6 7 8: 1 3 ♯4 5 ♭6: ma(add ♭6 ♯11)
namesBySequenceIndex[507] = '''Zoditonic'''
namesBySequenceIndexNegative[507] = ''''''
semitonesBySequenceIndex[507] = [0, 4, 6, 7, 8]
poemBySequenceIndex[507] = '''B J L M N'''
poemAuthors[507] = ''''''
#508: ䷂䷊ Sprouting Peace: 4 2 1 1 4  : S G3 M2 P D1 : AkOj

#509: C E F♯ G A :0 4 6 7 9: 1 3 ♯4 5 6: 6(add ♯11)
namesBySequenceIndex[508] = '''Raga Dhavalashri,Aeolacritonic'''
namesBySequenceIndexNegative[508] = ''''''
semitonesBySequenceIndex[508] = [0, 4, 6, 7, 9]
poemBySequenceIndex[508] = '''B J L M P'''
poemAuthors[508] = ''''''
#509: ䷂䷵ Sprouting Betrothing: 4 2 1 2 3  : S G3 M2 P D2 : AkEb

#510: C E F♯ G B♭ :0 4 6 7 10: 1 3 ♯4 5 ♭7: 7(add ♯11)
namesBySequenceIndex[509] = '''Ionoditonic'''
namesBySequenceIndexNegative[509] = ''''''
semitonesBySequenceIndex[509] = [0, 4, 6, 7, 10]
poemBySequenceIndex[509] = '''B J L M R'''
poemAuthors[509] = ''''''
#510: ䷂䷻ Sprouting Composed: 4 2 1 3 2  : S G3 M2 P N2 : AkEk

#511: C E F♯ G B :0 4 6 7 11: 1 3 ♯4 5 7: ma7(add ♯11)
namesBySequenceIndex[510] = '''Hirajoshi,Lydian Pentatonic,Chinese,Ching (China),Raga Amrtavarshini,Raga Malashri,Raga Shilangi,Zolitonic'''
namesBySequenceIndexNegative[510] = ''''''
semitonesBySequenceIndex[510] = [0, 4, 6, 7, 11]
poemBySequenceIndex[510] = '''B J L M S'''
poemAuthors[510] = ''''''
#511: ䷂䷨ Sprouting Diminishing: 4 2 1 4 1  : S G3 M2 P N3 : AkEl

#512: C E F♯ G♯ A :0 4 6 8 9: 1 3 ♯4 ♯5 6: 6(♯5 add ♯11)
namesBySequenceIndex[511] = '''Aerynitonic'''
namesBySequenceIndexNegative[511] = ''''''
semitonesBySequenceIndex[511] = [0, 4, 6, 8, 9]
poemBySequenceIndex[511] = '''B J L N P'''
poemAuthors[511] = ''''''
#512: ䷂䷶ Sprouting Abounding: 4 2 2 1 3  : S G3 M2 D1 N1 : AkIb

#513: C E F♯ G♯ B♭ :0 4 6 8 10: 1 3 ♯4 ♯5 ♭7: 7(♯5 add ♯11)
namesBySequenceIndex[512] = '''Bolitonic,Pentatonic Whole-Tone'''
namesBySequenceIndexNegative[512] = ''''''
semitonesBySequenceIndex[512] = [0, 4, 6, 8, 10]
poemBySequenceIndex[512] = '''B J L N R'''
poemAuthors[512] = ''''''
#513: ䷂䷾ Sprouting Completed: 4 2 2 2 2  : S G3 M2 D1 N2 : AkIk

#514: C E F♯ G♯ B :0 4 6 8 11: 1 3 ♯4 ♯5 7: ma7(♯5 add ♯11)
namesBySequenceIndex[513] = '''Raga Basant (ascending),Raga Bhairav (descending),Aeolanitonic,Augmented Pentatonic'''
namesBySequenceIndexNegative[513] = ''''''
semitonesBySequenceIndex[513] = [0, 4, 6, 8, 11]
poemBySequenceIndex[513] = '''B J L N S'''
poemAuthors[513] = ''''''
#514: ䷂䷕ Sprouting Adorning: 4 2 2 3 1  : S G3 M2 D1 N3 : AkIl

#515: C E G♭ A B♭ :0 4 6 9 10: 1 3 ♭5 6 ♭7: 7(♭5 add 13)
namesBySequenceIndex[514] = '''Mystic Chord,Phronitonic'''
namesBySequenceIndexNegative[514] = ''''''
semitonesBySequenceIndex[514] = [0, 4, 6, 9, 10]
poemBySequenceIndex[514] = '''B J L P R'''
poemAuthors[514] = ''''''
#515: ䷂䷐ Sprouting Following: 4 2 3 1 2  : S G3 M2 D2 N2 : AkAd

#516: C E G♭ A B :0 4 6 9 11: 1 3 ♭5 6 7: ma7(♭5 add 13)
namesBySequenceIndex[515] = '''Raga Hindol,Raga Sunada Vinodini,Raga Sanjk Ka Hindol,Gathitonic'''
namesBySequenceIndexNegative[515] = ''''''
semitonesBySequenceIndex[515] = [0, 4, 6, 9, 11]
poemBySequenceIndex[515] = '''B J L P S'''
poemAuthors[515] = ''''''
#516: ䷂䷔ Sprouting Biting: 4 2 3 2 1  : S G3 M2 D2 N3 : AkAf

#517: C E G♭ A♯ B :0 4 6 10 11: 1 3 ♭5 ♯6 7: ma7(♭5 add ♭7)
namesBySequenceIndex[516] = '''Zathitonic'''
namesBySequenceIndexNegative[516] = ''''''
semitonesBySequenceIndex[516] = [0, 4, 6, 10, 11]
poemBySequenceIndex[516] = '''B J L R S'''
poemAuthors[516] = ''''''
#517: ䷂䷩ Sprouting Augmenting: 4 2 4 1 1  : S G3 M2 D3 N3 : AkAm

#518: C E G A♭ A :0 4 7 8 9: 1 3 5 ♭6 6: 6(add ♭6)
namesBySequenceIndex[517] = '''Aeolyritonic'''
namesBySequenceIndexNegative[517] = ''''''
semitonesBySequenceIndex[517] = [0, 4, 7, 8, 9]
poemBySequenceIndex[517] = '''B J M N P'''
poemAuthors[517] = ''''''
#518: ䷂䷟ Sprouting Persevering: 4 3 1 1 3  : S G3 P D1 N1 : Akob

#519: C E G A♭ B♭ :0 4 7 8 10: 1 3 5 ♭6 ♭7: 7(add ♭13)
namesBySequenceIndex[518] = '''Ryphitonic'''
namesBySequenceIndexNegative[518] = ''''''
semitonesBySequenceIndex[518] = [0, 4, 7, 8, 10]
poemBySequenceIndex[518] = '''B J M N R'''
poemAuthors[518] = ''''''
#519: ䷂䷯ Sprouting Source: 4 3 1 2 2  : S G3 P D1 N2 : Akok

#520: C E G A♭ B :0 4 7 8 11: 1 3 5 ♭6 7: ma7(add ♭13)
namesBySequenceIndex[519] = '''Zacritonic'''
namesBySequenceIndexNegative[519] = ''''''
semitonesBySequenceIndex[519] = [0, 4, 7, 8, 11]
poemBySequenceIndex[519] = '''B J M N S'''
poemAuthors[519] = ''''''
#520: ䷂䷑ Sprouting Correcting: 4 3 1 3 1  : S G3 P D1 N3 : Akol

#521: C E G A B♭ :0 4 7 9 10: 1 3 5 6 ♭7: 7(add 13)
namesBySequenceIndex[520] = '''Raga Valaji,Ionaditonic'''
namesBySequenceIndexNegative[520] = ''''''
semitonesBySequenceIndex[520] = [0, 4, 7, 9, 10]
poemBySequenceIndex[520] = '''B J M P R'''
poemAuthors[520] = ''''''
#521: ䷂䷮ Sprouting Confining: 4 3 2 1 2  : S G3 P D2 N2 : Aked

#522: C E G A B :0 4 7 9 11: 1 3 5 6 7: ma7(add 13)
namesBySequenceIndex[521] = '''Raga Mamata,Magitonic'''
namesBySequenceIndexNegative[521] = ''''''
semitonesBySequenceIndex[521] = [0, 4, 7, 9, 11]
poemBySequenceIndex[521] = '''B J M P S'''
poemAuthors[521] = ''''''
#522: ䷂䷿ Sprouting Unfinished: 4 3 2 2 1  : S G3 P D2 N3 : Akef

#523: C E G A♯ B :0 4 7 10 11: 1 3 5 ♯6 7: ma7(add ♭7)
namesBySequenceIndex[522] = '''Molitonic'''
namesBySequenceIndexNegative[522] = ''''''
semitonesBySequenceIndex[522] = [0, 4, 7, 10, 11]
poemBySequenceIndex[522] = '''B J M R S'''
poemAuthors[522] = ''''''
#523: ䷂䷺ Sprouting Dispersing: 4 3 3 1 1  : S G3 P D3 N3 : Akem

#524: C E G♯ A B♭ :0 4 8 9 10: 1 3 ♯5 6 ♭7: 7(♯5 add 13)
namesBySequenceIndex[523] = '''Ranitonic'''
namesBySequenceIndexNegative[523] = ''''''
semitonesBySequenceIndex[523] = [0, 4, 8, 9, 10]
poemBySequenceIndex[523] = '''B J N P R'''
poemAuthors[523] = ''''''
#524: ䷂䷞ Sprouting Conjoining: 4 4 1 1 2  : S G3 D1 D2 N2 : Akid

#525: C E G♯ A B :0 4 8 9 11: 1 3 ♯5 6 7: ma7(♯5 add 13)
namesBySequenceIndex[524] = '''Dolitonic'''
namesBySequenceIndexNegative[524] = ''''''
semitonesBySequenceIndex[524] = [0, 4, 8, 9, 11]
poemBySequenceIndex[524] = '''B J N P S'''
poemAuthors[524] = ''''''
#525: ䷂䷷ Sprouting Wanderer: 4 4 1 2 1  : S G3 D1 D2 N3 : Akif

#526: C E G♯ A♯ B :0 4 8 10 11: 1 3 ♯5 ♯6 7: ma7(♯5 add ♭7)
namesBySequenceIndex[525] = '''Zylitonic'''
namesBySequenceIndexNegative[525] = ''''''
semitonesBySequenceIndex[525] = [0, 4, 8, 10, 11]
poemBySequenceIndex[525] = '''B J N R S'''
poemAuthors[525] = ''''''
#526: ䷂䷴ Sprouting Infiltrating: 4 4 2 1 1  : S G3 D1 N2 N3 : Akim

#527: C E A B♭ B :0 4 9 10 11: 1 3 6 ♭7 7: ma7(no 5 add ♭7 13)
namesBySequenceIndex[526] = ''''''
namesBySequenceIndexNegative[526] = ''''''
semitonesBySequenceIndex[526] = [0, 4, 9, 10, 11]
poemBySequenceIndex[526] = '''B J P R S'''
poemAuthors[526] = ''''''
#527: ䷂䷋ Sprouting Obstruction: 4 5 1 1 1  : S G3 D2 N2 N3 : Akag

#528: C F F♯ G A♭ :0 5 6 7 8: 1 4 ♯4 5 ♭6: sus4(add ♭6 ♯11)
namesBySequenceIndex[527] = ''''''
namesBySequenceIndexNegative[527] = ''''''
semitonesBySequenceIndex[527] = [0, 5, 6, 7, 8]
poemBySequenceIndex[527] = '''B K L M N'''
poemAuthors[527] = ''''''
#528: ䷚䷊ Nourishment Peace: 5 1 1 1 4  : S M1 M2 P D1 : AlOj

#529: C F F♯ G A :0 5 6 7 9: 1 4 ♯4 5 6: 6sus4(add ♯11)
namesBySequenceIndex[528] = ''''''
namesBySequenceIndexNegative[528] = ''''''
semitonesBySequenceIndex[528] = [0, 5, 6, 7, 9]
poemBySequenceIndex[528] = '''B K L M P'''
poemAuthors[528] = ''''''
#529: ䷚䷵ Nourishment Betrothing: 5 1 1 2 3  : S M1 M2 P D2 : AlEb

#530: C F F♯ G B♭ :0 5 6 7 10: 1 4 ♯4 5 ♭7: 7sus4(add ♯11)
namesBySequenceIndex[529] = ''''''
namesBySequenceIndexNegative[529] = ''''''
semitonesBySequenceIndex[529] = [0, 5, 6, 7, 10]
poemBySequenceIndex[529] = '''B K L M R'''
poemAuthors[529] = ''''''
#530: ䷚䷻ Nourishment Composed: 5 1 1 3 2  : S M1 M2 P N2 : AlEk

#531: C F F♯ G B :0 5 6 7 11: 1 4 ♯4 5 7: ma7sus4(add ♯11)
namesBySequenceIndex[530] = ''''''
namesBySequenceIndexNegative[530] = ''''''
semitonesBySequenceIndex[530] = [0, 5, 6, 7, 11]
poemBySequenceIndex[530] = '''B K L M S'''
poemAuthors[530] = ''''''
#531: ䷚䷨ Nourishment Diminishing: 5 1 1 4 1  : S M1 M2 P N3 : AlEl

#532: C F G♭ A♭ A :0 5 6 8 9: 1 4 ♭5 ♭6 6: 6sus4(♯5 add ♯11)
namesBySequenceIndex[531] = ''''''
namesBySequenceIndexNegative[531] = ''''''
semitonesBySequenceIndex[531] = [0, 5, 6, 8, 9]
poemBySequenceIndex[531] = '''B K L N P'''
poemAuthors[531] = ''''''
#532: ䷚䷶ Nourishment Abounding: 5 1 2 1 3  : S M1 M2 D1 N1 : AlIb

#533: C F G♭ A♭ B♭ :0 5 6 8 10: 1 4 ♭5 ♭6 ♭7: 7sus4(♯5 add ♯11)
namesBySequenceIndex[532] = ''''''
namesBySequenceIndexNegative[532] = ''''''
semitonesBySequenceIndex[532] = [0, 5, 6, 8, 10]
poemBySequenceIndex[532] = '''B K L N R'''
poemAuthors[532] = ''''''
#533: ䷚䷾ Nourishment Completed: 5 1 2 2 2  : S M1 M2 D1 N2 : AlIk

#534: C F G♭ A♭ B :0 5 6 8 11: 1 4 ♭5 ♭6 7: ma7sus4(♯5 add ♯11)
namesBySequenceIndex[533] = ''''''
namesBySequenceIndexNegative[533] = ''''''
semitonesBySequenceIndex[533] = [0, 5, 6, 8, 11]
poemBySequenceIndex[533] = '''B K L N S'''
poemAuthors[533] = ''''''
#534: ䷚䷕ Nourishment Adorning: 5 1 2 3 1  : S M1 M2 D1 N3 : AlIl

#535: C F G♭ A B♭ :0 5 6 9 10: 1 4 ♭5 6 ♭7: 7sus4(♭5 add 13)
namesBySequenceIndex[534] = ''''''
namesBySequenceIndexNegative[534] = ''''''
semitonesBySequenceIndex[534] = [0, 5, 6, 9, 10]
poemBySequenceIndex[534] = '''B K L P R'''
poemAuthors[534] = ''''''
#535: ䷚䷐ Nourishment Following: 5 1 3 1 2  : S M1 M2 D2 N2 : AlAd

#536: C F G♭ A B :0 5 6 9 11: 1 4 ♭5 6 7: ma7sus4(♭5 add 13)
namesBySequenceIndex[535] = ''''''
namesBySequenceIndexNegative[535] = ''''''
semitonesBySequenceIndex[535] = [0, 5, 6, 9, 11]
poemBySequenceIndex[535] = '''B K L P S'''
poemAuthors[535] = ''''''
#536: ䷚䷔ Nourishment Biting: 5 1 3 2 1  : S M1 M2 D2 N3 : AlAf

#537: C F G♭ A♯ B :0 5 6 10 11: 1 4 ♭5 ♯6 7: ma7sus4(♭5 add ♭7)
namesBySequenceIndex[536] = ''''''
namesBySequenceIndexNegative[536] = ''''''
semitonesBySequenceIndex[536] = [0, 5, 6, 10, 11]
poemBySequenceIndex[536] = '''B K L R S'''
poemAuthors[536] = ''''''
#537: ䷚䷩ Nourishment Augmenting: 5 1 4 1 1  : S M1 M2 D3 N3 : AlAm

#538: C F G A♭ A :0 5 7 8 9: 1 4 5 ♭6 6: 6sus4(add ♭6)
namesBySequenceIndex[537] = ''''''
namesBySequenceIndexNegative[537] = ''''''
semitonesBySequenceIndex[537] = [0, 5, 7, 8, 9]
poemBySequenceIndex[537] = '''B K M N P'''
poemAuthors[537] = ''''''
#538: ䷚䷟ Nourishment Persevering: 5 2 1 1 3  : S M1 P D1 N1 : Alob

#539: C F G A♭ B♭ :0 5 7 8 10: 1 4 5 ♭6 ♭7: 7sus4(add ♭13)
namesBySequenceIndex[538] = ''''''
namesBySequenceIndexNegative[538] = ''''''
semitonesBySequenceIndex[538] = [0, 5, 7, 8, 10]
poemBySequenceIndex[538] = '''B K M N R'''
poemAuthors[538] = ''''''
#539: ䷚䷯ Nourishment Source: 5 2 1 2 2  : S M1 P D1 N2 : Alok

#540: C F G A♭ B :0 5 7 8 11: 1 4 5 ♭6 7: ma7sus4(add ♭13)
namesBySequenceIndex[539] = '''Raga Devaranji,Raga Devaranjani 1'''
namesBySequenceIndexNegative[539] = ''''''
semitonesBySequenceIndex[539] = [0, 5, 7, 8, 11]
poemBySequenceIndex[539] = '''B K M N S'''
poemAuthors[539] = ''''''
#540: ䷚䷑ Nourishment Correcting: 5 2 1 3 1  : S M1 P D1 N3 : Alol

#541: C F G A B♭ :0 5 7 9 10: 1 4 5 6 ♭7: 7sus4(add 13)
namesBySequenceIndex[540] = '''Raga Kuntvarali,Raga Kuntalavarali'''
namesBySequenceIndexNegative[540] = ''''''
semitonesBySequenceIndex[540] = [0, 5, 7, 9, 10]
poemBySequenceIndex[540] = '''B K M P R'''
poemAuthors[540] = ''''''
#541: ䷚䷮ Nourishment Confining: 5 2 2 1 2  : S M1 P D2 N2 : Aled

#542: C F G A B :0 5 7 9 11: 1 4 5 6 7: ma7sus4(add 13)
namesBySequenceIndex[541] = '''Raga Puruhutika,Raga Purvaholica'''
namesBySequenceIndexNegative[541] = ''''''
semitonesBySequenceIndex[541] = [0, 5, 7, 9, 11]
poemBySequenceIndex[541] = '''B K M P S'''
poemAuthors[541] = ''''''
#542: ䷚䷿ Nourishment Unfinished: 5 2 2 2 1  : S M1 P D2 N3 : Alef

#543: C F G A♯ B :0 5 7 10 11: 1 4 5 ♯6 7: ma7sus4(add ♭7)
namesBySequenceIndex[542] = ''''''
namesBySequenceIndexNegative[542] = ''''''
semitonesBySequenceIndex[542] = [0, 5, 7, 10, 11]
poemBySequenceIndex[542] = '''B K M R S'''
poemAuthors[542] = ''''''
#543: ䷚䷺ Nourishment Dispersing: 5 2 3 1 1  : S M1 P D3 N3 : Alem

#544: C F G♯ A B♭ :0 5 8 9 10: 1 4 ♯5 6 ♭7: 7sus4(♯5 add 13)
namesBySequenceIndex[543] = ''''''
namesBySequenceIndexNegative[543] = ''''''
semitonesBySequenceIndex[543] = [0, 5, 8, 9, 10]
poemBySequenceIndex[543] = '''B K N P R'''
poemAuthors[543] = ''''''
#544: ䷚䷞ Nourishment Conjoining: 5 3 1 1 2  : S M1 D1 D2 N2 : Alid

#545: C F G♯ A B :0 5 8 9 11: 1 4 ♯5 6 7: ma7sus4(♯5 add 13)
namesBySequenceIndex[544] = ''''''
namesBySequenceIndexNegative[544] = ''''''
semitonesBySequenceIndex[544] = [0, 5, 8, 9, 11]
poemBySequenceIndex[544] = '''B K N P S'''
poemAuthors[544] = ''''''
#545: ䷚䷷ Nourishment Wanderer: 5 3 1 2 1  : S M1 D1 D2 N3 : Alif

#546: C F G♯ A♯ B :0 5 8 10 11: 1 4 ♯5 ♯6 7: ma7sus4(♯5 add ♭7)
namesBySequenceIndex[545] = '''Fibonacci (descending)'''
namesBySequenceIndexNegative[545] = ''''''
semitonesBySequenceIndex[545] = [0, 5, 8, 10, 11]
poemBySequenceIndex[545] = '''B K N R S'''
poemAuthors[545] = ''''''
#546: ䷚䷴ Nourishment Infiltrating: 5 3 2 1 1  : S M1 D1 N2 N3 : Alim

#547: C F A B♭ B :0 5 9 10 11: 1 4 6 ♭7 7: ma7sus4(no 5 add ♭7 13)
namesBySequenceIndex[546] = ''''''
namesBySequenceIndexNegative[546] = ''''''
semitonesBySequenceIndex[546] = [0, 5, 9, 10, 11]
poemBySequenceIndex[546] = '''B K P R S'''
poemAuthors[546] = ''''''
#547: ䷚䷋ Nourishment Obstruction: 5 4 1 1 1  : S M1 D2 N2 N3 : Alag

#548: C F♯ G A♭ A :0 6 7 8 9: 1 ♯4 5 ♭6 6: 6sus♯4(add ♭6)
namesBySequenceIndex[547] = ''''''
namesBySequenceIndexNegative[547] = ''''''
semitonesBySequenceIndex[547] = [0, 6, 7, 8, 9]
poemBySequenceIndex[547] = '''B L M N P'''
poemAuthors[547] = ''''''
#548: ䷗䷡ Returning Might: 6 1 1 1 3  : S M2 P D1 N1 : AjOb

#549: C F♯ G A♭ B♭ :0 6 7 8 10: 1 ♯4 5 ♭6 ♭7: 7sus♯4(add ♭13)
namesBySequenceIndex[548] = ''''''
namesBySequenceIndexNegative[548] = ''''''
semitonesBySequenceIndex[548] = [0, 6, 7, 8, 10]
poemBySequenceIndex[548] = '''B L M N R'''
poemAuthors[548] = ''''''
#549: ䷗䷄ Returning Biding: 6 1 1 2 2  : S M2 P D1 N2 : AjOk

#550: C F♯ G A♭ B :0 6 7 8 11: 1 ♯4 5 ♭6 7: ma7sus♯4(add ♭13)
namesBySequenceIndex[549] = ''''''
namesBySequenceIndexNegative[549] = ''''''
semitonesBySequenceIndex[549] = [0, 6, 7, 8, 11]
poemBySequenceIndex[549] = '''B L M N S'''
poemAuthors[549] = ''''''
#550: ䷗䷙ Returning Cultivating: 6 1 1 3 1  : S M2 P D1 N3 : AjOl

#551: C F♯ G A B♭ :0 6 7 9 10: 1 ♯4 5 6 ♭7: 7sus♯4(add 13)
namesBySequenceIndex[550] = ''''''
namesBySequenceIndexNegative[550] = ''''''
semitonesBySequenceIndex[550] = [0, 6, 7, 9, 10]
poemBySequenceIndex[550] = '''B L M P R'''
poemAuthors[550] = ''''''
#551: ䷗䷹ Returning Joyous: 6 1 2 1 2  : S M2 P D2 N2 : AjEd

#552: C F♯ G A B :0 6 7 9 11: 1 ♯4 5 6 7: ma7sus♯4(add 13)
namesBySequenceIndex[551] = ''''''
namesBySequenceIndexNegative[551] = ''''''
semitonesBySequenceIndex[551] = [0, 6, 7, 9, 11]
poemBySequenceIndex[551] = '''B L M P S'''
poemAuthors[551] = ''''''
#552: ䷗䷥ Returning Polarising: 6 1 2 2 1  : S M2 P D2 N3 : AjEf

#553: C F♯ G A♯ B :0 6 7 10 11: 1 ♯4 5 ♯6 7: ma7sus♯4(add ♭7)
namesBySequenceIndex[552] = ''''''
namesBySequenceIndexNegative[552] = ''''''
semitonesBySequenceIndex[552] = [0, 6, 7, 10, 11]
poemBySequenceIndex[552] = '''B L M R S'''
poemAuthors[552] = ''''''
#553: ䷗䷼ Returning Truth: 6 1 3 1 1  : S M2 P D3 N3 : AjEm

#554: C F♯ G♯ A B♭ :0 6 8 9 10: 1 ♯4 ♯5 6 ♭7: 7sus♯4(♯5 add 13)
namesBySequenceIndex[553] = ''''''
namesBySequenceIndexNegative[553] = ''''''
semitonesBySequenceIndex[553] = [0, 6, 8, 9, 10]
poemBySequenceIndex[553] = '''B L N P R'''
poemAuthors[553] = ''''''
#554: ䷗䷰ Returning Revolution: 6 2 1 1 2  : S M2 D1 D2 N2 : AjId

#555: C F♯ G♯ A B :0 6 8 9 11: 1 ♯4 ♯5 6 7: ma7sus♯4(♯5 add 13)
namesBySequenceIndex[554] = ''''''
namesBySequenceIndexNegative[554] = ''''''
semitonesBySequenceIndex[554] = [0, 6, 8, 9, 11]
poemBySequenceIndex[554] = '''B L N P S'''
poemAuthors[554] = ''''''
#555: ䷗䷝ Returning Distanced: 6 2 1 2 1  : S M2 D1 D2 N3 : AjIf

#556: C F♯ G♯ A♯ B :0 6 8 10 11: 1 ♯4 ♯5 ♯6 7: ma7sus♯4(♯5 add ♭7)
namesBySequenceIndex[555] = ''''''
namesBySequenceIndexNegative[555] = ''''''
semitonesBySequenceIndex[555] = [0, 6, 8, 10, 11]
poemBySequenceIndex[555] = '''B L N R S'''
poemAuthors[555] = ''''''
#556: ䷗䷤ Returning Family: 6 2 2 1 1  : S M2 D1 N2 N3 : AjIm

#557: C G♭ A B♭ B :0 6 9 10 11: 1 ♭5 6 ♭7 7: ma7(♭5 no 3 add ♭7 13)
namesBySequenceIndex[556] = ''''''
namesBySequenceIndexNegative[556] = ''''''
semitonesBySequenceIndex[556] = [0, 6, 9, 10, 11]
poemBySequenceIndex[556] = '''B L P R S'''
poemAuthors[556] = ''''''
#557: ䷗䷘ Returning Integrity: 6 3 1 1 1  : S M2 D2 N2 N3 : AjAg

#558: C G A♭ A B♭ :0 7 8 9 10: 1 5 ♭6 6 ♭7: 7(no 3 add ♭13 13)
namesBySequenceIndex[557] = ''''''
namesBySequenceIndexNegative[557] = ''''''
semitonesBySequenceIndex[557] = [0, 7, 8, 9, 10]
poemBySequenceIndex[557] = '''B M N P R'''
poemAuthors[557] = ''''''
#558: ䷗䷛ Returning Excess: 7 1 1 1 2  : S P D1 D2 N2 : Ajod

#559: C G A♭ A B :0 7 8 9 11: 1 5 ♭6 6 7: ma7(no 3 add ♭13 13)
namesBySequenceIndex[558] = ''''''
namesBySequenceIndexNegative[558] = ''''''
semitonesBySequenceIndex[558] = [0, 7, 8, 9, 11]
poemBySequenceIndex[558] = '''B M N P S'''
poemAuthors[558] = ''''''
#559: ䷗䷱ Returning Holding: 7 1 1 2 1  : S P D1 D2 N3 : Ajof

#560: C G A♭ B♭ B :0 7 8 10 11: 1 5 ♭6 ♭7 7: 5(add ♭6 ♭7 7)
namesBySequenceIndex[559] = ''''''
namesBySequenceIndexNegative[559] = ''''''
semitonesBySequenceIndex[559] = [0, 7, 8, 10, 11]
poemBySequenceIndex[559] = '''B M N R S'''
poemAuthors[559] = ''''''
#560: ䷗䷸ Returning Osmosis: 7 1 2 1 1  : S P D1 N2 N3 : Ajom

#561: C G A B♭ B :0 7 9 10 11: 1 5 6 ♭7 7: 5(add 6 ♭7 7)
namesBySequenceIndex[560] = ''''''
namesBySequenceIndexNegative[560] = ''''''
semitonesBySequenceIndex[560] = [0, 7, 9, 10, 11]
poemBySequenceIndex[560] = '''B M P R S'''
poemAuthors[560] = ''''''
#561: ䷗䷅ Returning Arguing: 7 2 1 1 1  : S P D2 N2 N3 : Ajeg

#562: C G♯ A B♭ B :0 8 9 10 11: 1 ♯5 6 ♭7 7: ma7(♯5 no 3 add ♭7 13)
namesBySequenceIndex[561] = ''''''
namesBySequenceIndexNegative[561] = ''''''
semitonesBySequenceIndex[561] = [0, 8, 9, 10, 11]
poemBySequenceIndex[561] = '''B N P R S'''
poemAuthors[561] = ''''''
#562: ䷗䷠ Returning Retiring: 8 1 1 1 1  : S D1 D2 N2 N3 : Ajig

#563: C D♭ D E♭ E F :0 1 2 3 4 5: 1 ♭2 2 ♭3 3 4: 11(no 5 7 add ♭9 ♯9)
namesBySequenceIndex[562] = '''Chromatic Hexachord'''
namesBySequenceIndexNegative[562] = ''''''
semitonesBySequenceIndex[562] = [0, 1, 2, 3, 4, 5]
poemBySequenceIndex[562] = '''B D F G J K'''
poemAuthors[562] = ''''''
#563: ䷀䷁ Creative Receptive: 1 1 1 1 1 7  : S R1 R2 G2 G3 M1 : Ogaj

#564: C D♭ D E♭ E G♭ :0 1 2 3 4 6: 1 ♭2 2 ♭3 3 ♭5: ma(♭5 add ♭9 9 ♯9)
namesBySequenceIndex[563] = ''''''
namesBySequenceIndexNegative[563] = ''''''
semitonesBySequenceIndex[563] = [0, 1, 2, 3, 4, 6]
poemBySequenceIndex[563] = '''B D F G J L'''
poemAuthors[563] = ''''''
#564: ䷪䷗ Decision Returning: 1 1 1 1 2 6  : S R1 R2 G2 G3 M2 : OdAj

#565: C D♭ D E♭ E G :0 1 2 3 4 7: 1 ♭2 2 ♭3 3 5: ma(add ♭9 9 ♯9)
namesBySequenceIndex[564] = ''''''
namesBySequenceIndexNegative[564] = ''''''
semitonesBySequenceIndex[564] = [0, 1, 2, 3, 4, 7]
poemBySequenceIndex[564] = '''B D F G J M'''
poemAuthors[564] = ''''''
#565: ䷪䷆ Decision Leading: 1 1 1 1 3 5  : S R1 R2 G2 G3 P : Odej

#566: C D♭ D E♭ E G♯ :0 1 2 3 4 8: 1 ♭2 2 ♭3 3 ♯5: ma(♯5 add ♭9 9 ♯9)
namesBySequenceIndex[565] = '''Gynimic'''
namesBySequenceIndexNegative[565] = ''''''
semitonesBySequenceIndex[565] = [0, 1, 2, 3, 4, 8]
poemBySequenceIndex[565] = '''B D F G J N'''
poemAuthors[565] = ''''''
#566: ䷪䷎ Decision Humble: 1 1 1 1 4 4  : S R1 R2 G2 G3 D1 : Odij

#567: C D♭ D E♭ E A :0 1 2 3 4 9: 1 ♭2 2 ♭3 3 6: 6/9(no 5 add ♭9 ♯9)
namesBySequenceIndex[566] = ''''''
namesBySequenceIndexNegative[566] = ''''''
semitonesBySequenceIndex[566] = [0, 1, 2, 3, 4, 9]
poemBySequenceIndex[566] = '''B D F G J P'''
poemAuthors[566] = ''''''
#567: ䷪䷏ Decision Confident: 1 1 1 1 5 3  : S R1 R2 G2 G3 D2 : Odab

#568: C D♭ D E♭ E B♭ :0 1 2 3 4 10: 1 ♭2 2 ♭3 3 ♭7: 9(no 5 add ♭9 ♯9)
namesBySequenceIndex[567] = ''''''
namesBySequenceIndexNegative[567] = ''''''
semitonesBySequenceIndex[567] = [0, 1, 2, 3, 4, 10]
poemBySequenceIndex[567] = '''B D F G J R'''
poemAuthors[567] = ''''''
#568: ䷪䷇ Decision Comparing: 1 1 1 1 6 2  : S R1 R2 G2 G3 N2 : Odak

#569: C D♭ D E♭ E B :0 1 2 3 4 11: 1 ♭2 2 ♭3 3 7: ma9(no 5 add ♭9 ♯9)
namesBySequenceIndex[568] = ''''''
namesBySequenceIndexNegative[568] = ''''''
semitonesBySequenceIndex[568] = [0, 1, 2, 3, 4, 11]
poemBySequenceIndex[568] = '''B D F G J S'''
poemAuthors[568] = ''''''
#569: ䷪䷖ Decision Stripping: 1 1 1 1 7 1  : S R1 R2 G2 G3 N3 : Odal

#570: C D♭ D E♭ F G♭ :0 1 2 3 5 6: 1 ♭2 2 ♭3 4 ♭5: °11(no 7 add ♭9)
namesBySequenceIndex[569] = ''''''
namesBySequenceIndexNegative[569] = ''''''
semitonesBySequenceIndex[569] = [0, 1, 2, 3, 5, 6]
poemBySequenceIndex[569] = '''B D F G K L'''
poemAuthors[569] = ''''''
#570: ䷍䷗ Resevoir Returning: 1 1 1 2 1 6  : S R1 R2 G2 M1 M2 : OfAj

#571: C D♭ D E♭ F G :0 1 2 3 5 7: 1 ♭2 2 ♭3 4 5: mi11(no 7 add ♭9)
namesBySequenceIndex[570] = ''''''
namesBySequenceIndexNegative[570] = ''''''
semitonesBySequenceIndex[570] = [0, 1, 2, 3, 5, 7]
poemBySequenceIndex[570] = '''B D F G K M'''
poemAuthors[570] = ''''''
#571: ䷍䷆ Resevoir Leading: 1 1 1 2 2 5  : S R1 R2 G2 M1 P : Ofej

#572: C D♭ D E♭ F G♯ :0 1 2 3 5 8: 1 ♭2 2 ♭3 4 ♯5: mi11(♯5 no 7 add ♭9)
namesBySequenceIndex[571] = '''Golimic'''
namesBySequenceIndexNegative[571] = ''''''
semitonesBySequenceIndex[571] = [0, 1, 2, 3, 5, 8]
poemBySequenceIndex[571] = '''B D F G K N'''
poemAuthors[571] = ''''''
#572: ䷍䷎ Resevoir Humble: 1 1 1 2 3 4  : S R1 R2 G2 M1 D1 : Ofij

#573: C D♭ D E♭ F A :0 1 2 3 5 9: 1 ♭2 2 ♭3 4 6: mi13(no 5 7 add ♭9)
namesBySequenceIndex[572] = '''Lylimic'''
namesBySequenceIndexNegative[572] = ''''''
semitonesBySequenceIndex[572] = [0, 1, 2, 3, 5, 9]
poemBySequenceIndex[572] = '''B D F G K P'''
poemAuthors[572] = ''''''
#573: ䷍䷏ Resevoir Confident: 1 1 1 2 4 3  : S R1 R2 G2 M1 D2 : Ofab

#574: C D♭ D E♭ F B♭ :0 1 2 3 5 10: 1 ♭2 2 ♭3 4 ♭7: mi11(no 5 add ♭9)
namesBySequenceIndex[573] = ''''''
namesBySequenceIndexNegative[573] = ''''''
semitonesBySequenceIndex[573] = [0, 1, 2, 3, 5, 10]
poemBySequenceIndex[573] = '''B D F G K R'''
poemAuthors[573] = ''''''
#574: ䷍䷇ Resevoir Comparing: 1 1 1 2 5 2  : S R1 R2 G2 M1 N2 : Ofak

#575: C D♭ D E♭ F B :0 1 2 3 5 11: 1 ♭2 2 ♭3 4 7: mima11(no 5 add ♭9)
namesBySequenceIndex[574] = ''''''
namesBySequenceIndexNegative[574] = ''''''
semitonesBySequenceIndex[574] = [0, 1, 2, 3, 5, 11]
poemBySequenceIndex[574] = '''B D F G K S'''
poemAuthors[574] = ''''''
#575: ䷍䷖ Resevoir Stripping: 1 1 1 2 6 1  : S R1 R2 G2 M1 N3 : Ofal

#576: C D♭ D E♭ F♯ G :0 1 2 3 6 7: 1 ♭2 2 ♭3 ♯4 5: mi(add ♭9 9 ♯11)
namesBySequenceIndex[575] = ''''''
namesBySequenceIndexNegative[575] = ''''''
semitonesBySequenceIndex[575] = [0, 1, 2, 3, 6, 7]
poemBySequenceIndex[575] = '''B D F G L M'''
poemAuthors[575] = ''''''
#576: ䷡䷒ Might Arrived: 1 1 1 3 1 5  : S R1 R2 G2 M2 P : ObEj

#577: C D♭ D E♭ F♯ G♯ :0 1 2 3 6 8: 1 ♭2 2 ♭3 ♯4 ♯5: °(add ♭6 ♭9 9)
namesBySequenceIndex[576] = '''Zanimic'''
namesBySequenceIndexNegative[576] = ''''''
semitonesBySequenceIndex[576] = [0, 1, 2, 3, 6, 8]
poemBySequenceIndex[576] = '''B D F G L N'''
poemAuthors[576] = ''''''
#577: ䷡䷣ Might Eclipse: 1 1 1 3 2 4  : S R1 R2 G2 M2 D1 : ObIj

#578: C D♭ D E♭ G♭ A :0 1 2 3 6 9: 1 ♭2 2 ♭3 ♭5 6: °7(add ♭9 9)
namesBySequenceIndex[577] = '''Gaptimic'''
namesBySequenceIndexNegative[577] = ''''''
semitonesBySequenceIndex[577] = [0, 1, 2, 3, 6, 9]
poemBySequenceIndex[577] = '''B D F G L P'''
poemAuthors[577] = ''''''
#578: ䷡䷲ Might Thunder: 1 1 1 3 3 3  : S R1 R2 G2 M2 D2 : ObAb

#579: C D♭ D E♭ G♭ B♭ :0 1 2 3 6 10: 1 ♭2 2 ♭3 ♭5 ♭7: ⌀9(add ♭9)
namesBySequenceIndex[578] = '''Lynimic'''
namesBySequenceIndexNegative[578] = ''''''
semitonesBySequenceIndex[578] = [0, 1, 2, 3, 6, 10]
poemBySequenceIndex[578] = '''B D F G L R'''
poemAuthors[578] = ''''''
#579: ䷡䷂ Might Sprouting: 1 1 1 3 4 2  : S R1 R2 G2 M2 N2 : ObAk

#580: C D♭ D E♭ G♭ B :0 1 2 3 6 11: 1 ♭2 2 ♭3 ♭5 7: °ma9(add ♭9)
namesBySequenceIndex[579] = ''''''
namesBySequenceIndexNegative[579] = ''''''
semitonesBySequenceIndex[579] = [0, 1, 2, 3, 6, 11]
poemBySequenceIndex[579] = '''B D F G L S'''
poemAuthors[579] = ''''''
#580: ䷡䷚ Might Nourishment: 1 1 1 3 5 1  : S R1 R2 G2 M2 N3 : ObAl

#581: C D♭ D E♭ G A♭ :0 1 2 3 7 8: 1 ♭2 2 ♭3 5 ♭6: mi(add ♭6 ♭9 9)
namesBySequenceIndex[580] = '''Zynimic'''
namesBySequenceIndexNegative[580] = ''''''
semitonesBySequenceIndex[580] = [0, 1, 2, 3, 7, 8]
poemBySequenceIndex[580] = '''B D F G M N'''
poemAuthors[580] = ''''''
#581: ䷡䷭ Might Ascending: 1 1 1 4 1 4  : S R1 R2 G2 P D1 : Oboj

#582: C D♭ D E♭ G A :0 1 2 3 7 9: 1 ♭2 2 ♭3 5 6: mi6/9(add ♭9)
namesBySequenceIndex[581] = '''Kataptimic'''
namesBySequenceIndexNegative[581] = ''''''
semitonesBySequenceIndex[581] = [0, 1, 2, 3, 7, 9]
poemBySequenceIndex[581] = '''B D F G M P'''
poemAuthors[581] = ''''''
#582: ䷡䷧ Might Deliverance: 1 1 1 4 2 3  : S R1 R2 G2 P D2 : Obeb

#583: C D♭ D E♭ G B♭ :0 1 2 3 7 10: 1 ♭2 2 ♭3 5 ♭7: mi9(add ♭9)
namesBySequenceIndex[582] = '''Aerodimic'''
namesBySequenceIndexNegative[582] = ''''''
semitonesBySequenceIndex[582] = [0, 1, 2, 3, 7, 10]
poemBySequenceIndex[582] = '''B D F G M R'''
poemAuthors[582] = ''''''
#583: ䷡䷜ Might Chasm: 1 1 1 4 3 2  : S R1 R2 G2 P N2 : Obek

#584: C D♭ D E♭ G B :0 1 2 3 7 11: 1 ♭2 2 ♭3 5 7: mima9(add ♭9)
namesBySequenceIndex[583] = '''Thydimic'''
namesBySequenceIndexNegative[583] = ''''''
semitonesBySequenceIndex[583] = [0, 1, 2, 3, 7, 11]
poemBySequenceIndex[583] = '''B D F G M S'''
poemAuthors[583] = ''''''
#584: ䷡䷃ Might Naïve: 1 1 1 4 4 1  : S R1 R2 G2 P N3 : Obel

#585: C D♭ D E♭ G♯ A :0 1 2 3 8 9: 1 ♭2 2 ♭3 ♯5 6: mi6/9(♯5 add ♭9)
namesBySequenceIndex[584] = ''''''
namesBySequenceIndexNegative[584] = ''''''
semitonesBySequenceIndex[584] = [0, 1, 2, 3, 8, 9]
poemBySequenceIndex[584] = '''B D F G N P'''
poemAuthors[584] = ''''''
#585: ䷡䷽ Might Detail: 1 1 1 5 1 3  : S R1 R2 G2 D1 N1 : Obib

#586: C D♭ D E♭ G♯ B♭ :0 1 2 3 8 10: 1 ♭2 2 ♭3 ♯5 ♭7: mi9(♯5 add ♭9)
namesBySequenceIndex[585] = ''''''
namesBySequenceIndexNegative[585] = ''''''
semitonesBySequenceIndex[585] = [0, 1, 2, 3, 8, 10]
poemBySequenceIndex[585] = '''B D F G N R'''
poemAuthors[585] = ''''''
#586: ䷡䷦ Might Limping: 1 1 1 5 2 2  : S R1 R2 G2 D1 N2 : Obik

#587: C D♭ D E♭ G♯ B :0 1 2 3 8 11: 1 ♭2 2 ♭3 ♯5 7: mima9(♯5 add ♭9)
namesBySequenceIndex[586] = ''''''
namesBySequenceIndexNegative[586] = ''''''
semitonesBySequenceIndex[586] = [0, 1, 2, 3, 8, 11]
poemBySequenceIndex[586] = '''B D F G N S'''
poemAuthors[586] = ''''''
#587: ䷡䷳ Might Stillness: 1 1 1 5 3 1  : S R1 R2 G2 D1 N3 : Obil

#588: C D♭ D E♭ A B♭ :0 1 2 3 9 10: 1 ♭2 2 ♭3 6 ♭7: mi9(no 5 add ♭9 13)
namesBySequenceIndex[587] = ''''''
namesBySequenceIndexNegative[587] = ''''''
semitonesBySequenceIndex[587] = [0, 1, 2, 3, 9, 10]
poemBySequenceIndex[587] = '''B D F G P R'''
poemAuthors[587] = ''''''
#588: ䷡䷬ Might Clustering: 1 1 1 6 1 2  : S R1 R2 G2 D2 N2 : Obad

#589: C D♭ D E♭ A B :0 1 2 3 9 11: 1 ♭2 2 ♭3 6 7: mima9(no 5 add ♭9 13)
namesBySequenceIndex[588] = ''''''
namesBySequenceIndexNegative[588] = ''''''
semitonesBySequenceIndex[588] = [0, 1, 2, 3, 9, 11]
poemBySequenceIndex[588] = '''B D F G P S'''
poemAuthors[588] = ''''''
#589: ䷡䷢ Might Prospering: 1 1 1 6 2 1  : S R1 R2 G2 D2 N3 : Obaf

#590: C D♭ D E♭ A♯ B :0 1 2 3 10 11: 1 ♭2 2 ♭3 ♯6 7: mima9(no 5 add ♭7 ♭9)
namesBySequenceIndex[589] = ''''''
namesBySequenceIndexNegative[589] = ''''''
semitonesBySequenceIndex[589] = [0, 1, 2, 3, 10, 11]
poemBySequenceIndex[589] = '''B D F G R S'''
poemAuthors[589] = ''''''
#590: ䷡䷓ Might Seeing: 1 1 1 7 1 1  : S R1 R2 G2 D3 N3 : Obam

#591: C D♭ D E F G♭ :0 1 2 4 5 6: 1 ♭2 2 3 4 ♭5: 11(♭5 no 7 add ♭9)
namesBySequenceIndex[590] = ''''''
namesBySequenceIndexNegative[590] = ''''''
semitonesBySequenceIndex[590] = [0, 1, 2, 4, 5, 6]
poemBySequenceIndex[590] = '''B D F J K L'''
poemAuthors[590] = ''''''
#591: ䷈䷗ Homegrown Returning: 1 1 2 1 1 6  : S R1 R2 G3 M1 M2 : OmAj

#592: C D♭ D E F G :0 1 2 4 5 7: 1 ♭2 2 3 4 5: 11(no 7 add ♭9)
namesBySequenceIndex[591] = '''Sacher Hexachord'''
namesBySequenceIndexNegative[591] = ''''''
semitonesBySequenceIndex[591] = [0, 1, 2, 4, 5, 7]
poemBySequenceIndex[591] = '''B D F J K M'''
poemAuthors[591] = ''''''
#592: ䷈䷆ Homegrown Leading: 1 1 2 1 2 5  : S R1 R2 G3 M1 P : Omej

#593: C D♭ D E F G♯ :0 1 2 4 5 8: 1 ♭2 2 3 4 ♯5: 11(♯5 no 7 add ♭9)
namesBySequenceIndex[592] = '''Stagimic'''
namesBySequenceIndexNegative[592] = ''''''
semitonesBySequenceIndex[592] = [0, 1, 2, 4, 5, 8]
poemBySequenceIndex[592] = '''B D F J K N'''
poemAuthors[592] = ''''''
#593: ䷈䷎ Homegrown Humble: 1 1 2 1 3 4  : S R1 R2 G3 M1 D1 : Omij

#594: C D♭ D E F A :0 1 2 4 5 9: 1 ♭2 2 3 4 6: 13(no 5 7 add ♭9)
namesBySequenceIndex[593] = '''Aeoladimic'''
namesBySequenceIndexNegative[593] = ''''''
semitonesBySequenceIndex[593] = [0, 1, 2, 4, 5, 9]
poemBySequenceIndex[593] = '''B D F J K P'''
poemAuthors[593] = ''''''
#594: ䷈䷏ Homegrown Confident: 1 1 2 1 4 3  : S R1 R2 G3 M1 D2 : Omab

#595: C D♭ D E F B♭ :0 1 2 4 5 10: 1 ♭2 2 3 4 ♭7: 11(no 5 add ♭9)
namesBySequenceIndex[594] = ''''''
namesBySequenceIndexNegative[594] = ''''''
semitonesBySequenceIndex[594] = [0, 1, 2, 4, 5, 10]
poemBySequenceIndex[594] = '''B D F J K R'''
poemAuthors[594] = ''''''
#595: ䷈䷇ Homegrown Comparing: 1 1 2 1 5 2  : S R1 R2 G3 M1 N2 : Omak

#596: C D♭ D E F B :0 1 2 4 5 11: 1 ♭2 2 3 4 7: ma11(no 5 add ♭9)
namesBySequenceIndex[595] = ''''''
namesBySequenceIndexNegative[595] = ''''''
semitonesBySequenceIndex[595] = [0, 1, 2, 4, 5, 11]
poemBySequenceIndex[595] = '''B D F J K S'''
poemAuthors[595] = ''''''
#596: ䷈䷖ Homegrown Stripping: 1 1 2 1 6 1  : S R1 R2 G3 M1 N3 : Omal

#597: C D♭ D E F♯ G :0 1 2 4 6 7: 1 ♭2 2 3 ♯4 5: ma(add ♭9 9 ♯11)
namesBySequenceIndex[596] = ''''''
namesBySequenceIndexNegative[596] = ''''''
semitonesBySequenceIndex[596] = [0, 1, 2, 4, 6, 7]
poemBySequenceIndex[596] = '''B D F J L M'''
poemAuthors[596] = ''''''
#597: ䷄䷒ Biding Arrived: 1 1 2 2 1 5  : S R1 R2 G3 M2 P : OkEj

#598: C D♭ D E F♯ G♯ :0 1 2 4 6 8: 1 ♭2 2 3 ♯4 ♯5: ma(♯5 add ♭9 9 ♯11)
namesBySequenceIndex[597] = '''Ionorimic'''
namesBySequenceIndexNegative[597] = ''''''
semitonesBySequenceIndex[597] = [0, 1, 2, 4, 6, 8]
poemBySequenceIndex[597] = '''B D F J L N'''
poemAuthors[597] = ''''''
#598: ䷄䷣ Biding Eclipse: 1 1 2 2 2 4  : S R1 R2 G3 M2 D1 : OkIj

#599: C D♭ D E G♭ A :0 1 2 4 6 9: 1 ♭2 2 3 ♭5 6: 6/9(♭5 add ♭9)
namesBySequenceIndex[598] = '''Thyrimic'''
namesBySequenceIndexNegative[598] = ''''''
semitonesBySequenceIndex[598] = [0, 1, 2, 4, 6, 9]
poemBySequenceIndex[598] = '''B D F J L P'''
poemAuthors[598] = ''''''
#599: ䷄䷲ Biding Thunder: 1 1 2 2 3 3  : S R1 R2 G3 M2 D2 : OkAb

#600: C D♭ D E G♭ B♭ :0 1 2 4 6 10: 1 ♭2 2 3 ♭5 ♭7: 9(♭5 add ♭9)
namesBySequenceIndex[599] = '''Sycrimic'''
namesBySequenceIndexNegative[599] = ''''''
semitonesBySequenceIndex[599] = [0, 1, 2, 4, 6, 10]
poemBySequenceIndex[599] = '''B D F J L R'''
poemAuthors[599] = ''''''
#600: ䷄䷂ Biding Sprouting: 1 1 2 2 4 2  : S R1 R2 G3 M2 N2 : OkAk

#601: C D♭ D E G♭ B :0 1 2 4 6 11: 1 ♭2 2 3 ♭5 7: ma9(♭5 add ♭9)
namesBySequenceIndex[600] = ''''''
namesBySequenceIndexNegative[600] = ''''''
semitonesBySequenceIndex[600] = [0, 1, 2, 4, 6, 11]
poemBySequenceIndex[600] = '''B D F J L S'''
poemAuthors[600] = ''''''
#601: ䷄䷚ Biding Nourishment: 1 1 2 2 5 1  : S R1 R2 G3 M2 N3 : OkAl

#602: C D♭ D E G A♭ :0 1 2 4 7 8: 1 ♭2 2 3 5 ♭6: ma(add ♭6 ♭9 9)
namesBySequenceIndex[601] = '''Zylimic,All Trichord Hexachord'''
namesBySequenceIndexNegative[601] = ''''''
semitonesBySequenceIndex[601] = [0, 1, 2, 4, 7, 8]
poemBySequenceIndex[601] = '''B D F J M N'''
poemAuthors[601] = ''''''
#602: ䷄䷭ Biding Ascending: 1 1 2 3 1 4  : S R1 R2 G3 P D1 : Okoj

#603: C D♭ D E G A :0 1 2 4 7 9: 1 ♭2 2 3 5 6: 6/9(add ♭9)
namesBySequenceIndex[602] = '''Phrynimic'''
namesBySequenceIndexNegative[602] = ''''''
semitonesBySequenceIndex[602] = [0, 1, 2, 4, 7, 9]
poemBySequenceIndex[602] = '''B D F J M P'''
poemAuthors[602] = ''''''
#603: ䷄䷧ Biding Deliverance: 1 1 2 3 2 3  : S R1 R2 G3 P D2 : Okeb

#604: C D♭ D E G B♭ :0 1 2 4 7 10: 1 ♭2 2 3 5 ♭7: 9(add ♭9)
namesBySequenceIndex[603] = '''Epycrimic'''
namesBySequenceIndexNegative[603] = ''''''
semitonesBySequenceIndex[603] = [0, 1, 2, 4, 7, 10]
poemBySequenceIndex[603] = '''B D F J M R'''
poemAuthors[603] = ''''''
#604: ䷄䷜ Biding Chasm: 1 1 2 3 3 2  : S R1 R2 G3 P N2 : Okek

#605: C D♭ D E G B :0 1 2 4 7 11: 1 ♭2 2 3 5 7: ma9(add ♭9)
namesBySequenceIndex[604] = '''Dyptimic'''
namesBySequenceIndexNegative[604] = ''''''
semitonesBySequenceIndex[604] = [0, 1, 2, 4, 7, 11]
poemBySequenceIndex[604] = '''B D F J M S'''
poemAuthors[604] = ''''''
#605: ䷄䷃ Biding Naïve: 1 1 2 3 4 1  : S R1 R2 G3 P N3 : Okel

#606: C D♭ D E G♯ A :0 1 2 4 8 9: 1 ♭2 2 3 ♯5 6: 6/9(♯5 add ♭9)
namesBySequenceIndex[605] = '''Aeoloptimic'''
namesBySequenceIndexNegative[605] = ''''''
semitonesBySequenceIndex[605] = [0, 1, 2, 4, 8, 9]
poemBySequenceIndex[605] = '''B D F J N P'''
poemAuthors[605] = ''''''
#606: ䷄䷽ Biding Detail: 1 1 2 4 1 3  : S R1 R2 G3 D1 N1 : Okib

#607: C D♭ D E G♯ B♭ :0 1 2 4 8 10: 1 ♭2 2 3 ♯5 ♭7: 9(♯5 add ♭9)
namesBySequenceIndex[606] = '''Epolimic'''
namesBySequenceIndexNegative[606] = ''''''
semitonesBySequenceIndex[606] = [0, 1, 2, 4, 8, 10]
poemBySequenceIndex[606] = '''B D F J N R'''
poemAuthors[606] = ''''''
#607: ䷄䷦ Biding Limping: 1 1 2 4 2 2  : S R1 R2 G3 D1 N2 : Okik

#608: C D♭ D E G♯ B :0 1 2 4 8 11: 1 ♭2 2 3 ♯5 7: ma9(♯5 add ♭9)
namesBySequenceIndex[607] = '''Epalimic'''
namesBySequenceIndexNegative[607] = ''''''
semitonesBySequenceIndex[607] = [0, 1, 2, 4, 8, 11]
poemBySequenceIndex[607] = '''B D F J N S'''
poemAuthors[607] = ''''''
#608: ䷄䷳ Biding Stillness: 1 1 2 4 3 1  : S R1 R2 G3 D1 N3 : Okil

#609: C D♭ D E A B♭ :0 1 2 4 9 10: 1 ♭2 2 3 6 ♭7: 9(no 5 add ♭9 13)
namesBySequenceIndex[608] = ''''''
namesBySequenceIndexNegative[608] = ''''''
semitonesBySequenceIndex[608] = [0, 1, 2, 4, 9, 10]
poemBySequenceIndex[608] = '''B D F J P R'''
poemAuthors[608] = ''''''
#609: ䷄䷬ Biding Clustering: 1 1 2 5 1 2  : S R1 R2 G3 D2 N2 : Okad

#610: C D♭ D E A B :0 1 2 4 9 11: 1 ♭2 2 3 6 7: ma9(no 5 add ♭9 13)
namesBySequenceIndex[609] = ''''''
namesBySequenceIndexNegative[609] = ''''''
semitonesBySequenceIndex[609] = [0, 1, 2, 4, 9, 11]
poemBySequenceIndex[609] = '''B D F J P S'''
poemAuthors[609] = ''''''
#610: ䷄䷢ Biding Prospering: 1 1 2 5 2 1  : S R1 R2 G3 D2 N3 : Okaf

#611: C D♭ D E A♯ B :0 1 2 4 10 11: 1 ♭2 2 3 ♯6 7: ma9(no 5 add ♭7 ♭9)
namesBySequenceIndex[610] = ''''''
namesBySequenceIndexNegative[610] = ''''''
semitonesBySequenceIndex[610] = [0, 1, 2, 4, 10, 11]
poemBySequenceIndex[610] = '''B D F J R S'''
poemAuthors[610] = ''''''
#611: ䷄䷓ Biding Seeing: 1 1 2 6 1 1  : S R1 R2 G3 D3 N3 : Okam

#612: C D♭ D F F♯ G :0 1 2 5 6 7: 1 ♭2 2 4 ♯4 5: sus4(add ♭9 9 ♯11)
namesBySequenceIndex[611] = ''''''
namesBySequenceIndexNegative[611] = ''''''
semitonesBySequenceIndex[611] = [0, 1, 2, 5, 6, 7]
poemBySequenceIndex[611] = '''B D F K L M'''
poemAuthors[611] = ''''''
#612: ䷙䷒ Cultivating Arrived: 1 1 3 1 1 5  : S R1 G1 M1 M2 P : OlEj

#613: C D♭ D F G♭ A♭ :0 1 2 5 6 8: 1 ♭2 2 4 ♭5 ♭6: sus4(♯5 add ♭9 9 ♯11)
namesBySequenceIndex[612] = '''Bothimic'''
namesBySequenceIndexNegative[612] = ''''''
semitonesBySequenceIndex[612] = [0, 1, 2, 5, 6, 8]
poemBySequenceIndex[612] = '''B D F K L N'''
poemAuthors[612] = ''''''
#613: ䷙䷣ Cultivating Eclipse: 1 1 3 1 2 4  : S R1 G1 M1 M2 D1 : OlIj

#614: C D♭ D F G♭ A :0 1 2 5 6 9: 1 ♭2 2 4 ♭5 6: 6/9sus4(♭5 add ♭9)
namesBySequenceIndex[613] = '''Schoenberg Hexachord,Phrothimic'''
namesBySequenceIndexNegative[613] = ''''''
semitonesBySequenceIndex[613] = [0, 1, 2, 5, 6, 9]
poemBySequenceIndex[613] = '''B D F K L P'''
poemAuthors[613] = ''''''
#614: ䷙䷲ Cultivating Thunder: 1 1 3 1 3 3  : S R1 G1 M1 M2 D2 : OlAb

#615: C D♭ D F G♭ B♭ :0 1 2 5 6 10: 1 ♭2 2 4 ♭5 ♭7: 11sus♭2(♭5)
namesBySequenceIndex[614] = '''Eparimic'''
namesBySequenceIndexNegative[614] = ''''''
semitonesBySequenceIndex[614] = [0, 1, 2, 5, 6, 10]
poemBySequenceIndex[614] = '''B D F K L R'''
poemAuthors[614] = ''''''
#615: ䷙䷂ Cultivating Sprouting: 1 1 3 1 4 2  : S R1 G1 M1 M2 N2 : OlAk

#616: C D♭ D F G♭ B :0 1 2 5 6 11: 1 ♭2 2 4 ♭5 7: ma11sus♭2(♭5)
namesBySequenceIndex[615] = ''''''
namesBySequenceIndexNegative[615] = ''''''
semitonesBySequenceIndex[615] = [0, 1, 2, 5, 6, 11]
poemBySequenceIndex[615] = '''B D F K L S'''
poemAuthors[615] = ''''''
#616: ䷙䷚ Cultivating Nourishment: 1 1 3 1 5 1  : S R1 G1 M1 M2 N3 : OlAl

#617: C D♭ D F G A♭ :0 1 2 5 7 8: 1 ♭2 2 4 5 ♭6: sus4(add ♭6 ♭9 9)
namesBySequenceIndex[616] = '''Sogimic'''
namesBySequenceIndexNegative[616] = ''''''
semitonesBySequenceIndex[616] = [0, 1, 2, 5, 7, 8]
poemBySequenceIndex[616] = '''B D F K M N'''
poemAuthors[616] = ''''''
#617: ䷙䷭ Cultivating Ascending: 1 1 3 2 1 4  : S R1 G1 M1 P D1 : Oloj

#618: C D♭ D F G A :0 1 2 5 7 9: 1 ♭2 2 4 5 6: 6/9sus4(add ♭9)
namesBySequenceIndex[617] = '''Lanimic'''
namesBySequenceIndexNegative[617] = ''''''
semitonesBySequenceIndex[617] = [0, 1, 2, 5, 7, 9]
poemBySequenceIndex[617] = '''B D F K M P'''
poemAuthors[617] = ''''''
#618: ䷙䷧ Cultivating Deliverance: 1 1 3 2 2 3  : S R1 G1 M1 P D2 : Oleb

#619: C D♭ D F G B♭ :0 1 2 5 7 10: 1 ♭2 2 4 5 ♭7: 11sus♭2
namesBySequenceIndex[618] = '''Pyrimic'''
namesBySequenceIndexNegative[618] = ''''''
semitonesBySequenceIndex[618] = [0, 1, 2, 5, 7, 10]
poemBySequenceIndex[618] = '''B D F K M R'''
poemAuthors[618] = ''''''
#619: ䷙䷜ Cultivating Chasm: 1 1 3 2 3 2  : S R1 G1 M1 P N2 : Olek

#620: C D♭ D F G B :0 1 2 5 7 11: 1 ♭2 2 4 5 7: ma11sus♭2
namesBySequenceIndex[619] = '''Ranimic'''
namesBySequenceIndexNegative[619] = ''''''
semitonesBySequenceIndex[619] = [0, 1, 2, 5, 7, 11]
poemBySequenceIndex[619] = '''B D F K M S'''
poemAuthors[619] = ''''''
#620: ䷙䷃ Cultivating Naïve: 1 1 3 2 4 1  : S R1 G1 M1 P N3 : Olel

#621: C D♭ D F G♯ A :0 1 2 5 8 9: 1 ♭2 2 4 ♯5 6: 6/9sus4(♯5 add ♭9)
namesBySequenceIndex[620] = '''Raga Suddha Mukhari,Epadimic'''
namesBySequenceIndexNegative[620] = ''''''
semitonesBySequenceIndex[620] = [0, 1, 2, 5, 8, 9]
poemBySequenceIndex[620] = '''B D F K N P'''
poemAuthors[620] = ''''''
#621: ䷙䷽ Cultivating Detail: 1 1 3 3 1 3  : S R1 G1 M1 D1 N1 : Olib

#622: C D♭ D F G♯ B♭ :0 1 2 5 8 10: 1 ♭2 2 4 ♯5 ♭7: 11sus♭2(♯5)
namesBySequenceIndex[621] = '''Phronimic'''
namesBySequenceIndexNegative[621] = ''''''
semitonesBySequenceIndex[621] = [0, 1, 2, 5, 8, 10]
poemBySequenceIndex[621] = '''B D F K N R'''
poemAuthors[621] = ''''''
#622: ䷙䷦ Cultivating Limping: 1 1 3 3 2 2  : S R1 G1 M1 D1 N2 : Olik

#623: C D♭ D F G♯ B :0 1 2 5 8 11: 1 ♭2 2 4 ♯5 7: ma11sus♭2(♯5)
namesBySequenceIndex[622] = '''Tharimic'''
namesBySequenceIndexNegative[622] = ''''''
semitonesBySequenceIndex[622] = [0, 1, 2, 5, 8, 11]
poemBySequenceIndex[622] = '''B D F K N S'''
poemAuthors[622] = ''''''
#623: ䷙䷳ Cultivating Stillness: 1 1 3 3 3 1  : S R1 G1 M1 D1 N3 : Olil

#624: C D♭ D F A B♭ :0 1 2 5 9 10: 1 ♭2 2 4 6 ♭7: 13sus♭2(no 5)
namesBySequenceIndex[623] = '''Zycrimic'''
namesBySequenceIndexNegative[623] = ''''''
semitonesBySequenceIndex[623] = [0, 1, 2, 5, 9, 10]
poemBySequenceIndex[623] = '''B D F K P R'''
poemAuthors[623] = ''''''
#624: ䷙䷬ Cultivating Clustering: 1 1 3 4 1 2  : S R1 G1 M1 D2 N2 : Olad

#625: C D♭ D F A B :0 1 2 5 9 11: 1 ♭2 2 4 6 7: ma13sus♭2(no 5)
namesBySequenceIndex[624] = '''Malimic'''
namesBySequenceIndexNegative[624] = ''''''
semitonesBySequenceIndex[624] = [0, 1, 2, 5, 9, 11]
poemBySequenceIndex[624] = '''B D F K P S'''
poemAuthors[624] = ''''''
#625: ䷙䷢ Cultivating Prospering: 1 1 3 4 2 1  : S R1 G1 M1 D2 N3 : Olaf

#626: C D♭ D F A♯ B :0 1 2 5 10 11: 1 ♭2 2 4 ♯6 7: ma11sus♭2(no 5 add ♭7)
namesBySequenceIndex[625] = ''''''
namesBySequenceIndexNegative[625] = ''''''
semitonesBySequenceIndex[625] = [0, 1, 2, 5, 10, 11]
poemBySequenceIndex[625] = '''B D F K R S'''
poemAuthors[625] = ''''''
#626: ䷙䷓ Cultivating Seeing: 1 1 3 5 1 1  : S R1 G1 M1 D3 N3 : Olam

#627: C D♭ D F♯ G A♭ :0 1 2 6 7 8: 1 ♭2 2 ♯4 5 ♭6: sus2(add ♭6 ♭9 ♯11)
namesBySequenceIndex[626] = '''Two Semitone Tritone,Stadimic'''
namesBySequenceIndexNegative[626] = ''''''
semitonesBySequenceIndex[626] = [0, 1, 2, 6, 7, 8]
poemBySequenceIndex[626] = '''B D F L M N'''
poemAuthors[626] = ''''''
#627: ䷊䷊ Peace Peace : 1 1 4 1 1 4  : S R1 G1 M2 P D1 : OjOj

#628: C D♭ D F♯ G A :0 1 2 6 7 9: 1 ♭2 2 ♯4 5 6: 6sus2(add ♭9 ♯11)
namesBySequenceIndex[627] = '''Raga Chandrajyoti,Epyrimic'''
namesBySequenceIndexNegative[627] = ''''''
semitonesBySequenceIndex[627] = [0, 1, 2, 6, 7, 9]
poemBySequenceIndex[627] = '''B D F L M P'''
poemAuthors[627] = ''''''
#628: ䷊䷵ Peace Betrothing: 1 1 4 1 2 3  : S R1 G1 M2 P D2 : OjEb

#629: C D♭ D F♯ G B♭ :0 1 2 6 7 10: 1 ♭2 2 ♯4 5 ♭7: 9sus♯4(add ♭9)
namesBySequenceIndex[628] = '''Phryptimic'''
namesBySequenceIndexNegative[628] = ''''''
semitonesBySequenceIndex[628] = [0, 1, 2, 6, 7, 10]
poemBySequenceIndex[628] = '''B D F L M R'''
poemAuthors[628] = ''''''
#629: ䷊䷻ Peace Composed: 1 1 4 1 3 2  : S R1 G1 M2 P N2 : OjEk

#630: C D♭ D F♯ G B :0 1 2 6 7 11: 1 ♭2 2 ♯4 5 7: ma9sus♯4(add ♭9)
namesBySequenceIndex[629] = '''Raga Vijayasri,Aeolimic'''
namesBySequenceIndexNegative[629] = ''''''
semitonesBySequenceIndex[629] = [0, 1, 2, 6, 7, 11]
poemBySequenceIndex[629] = '''B D F L M S'''
poemAuthors[629] = ''''''
#630: ䷊䷨ Peace Diminishing: 1 1 4 1 4 1  : S R1 G1 M2 P N3 : OjEl

#631: C D♭ D F♯ G♯ A :0 1 2 6 8 9: 1 ♭2 2 ♯4 ♯5 6: 6sus2(♯5 add ♭9 ♯11)
namesBySequenceIndex[630] = '''Ionathimic'''
namesBySequenceIndexNegative[630] = ''''''
semitonesBySequenceIndex[630] = [0, 1, 2, 6, 8, 9]
poemBySequenceIndex[630] = '''B D F L N P'''
poemAuthors[630] = ''''''
#631: ䷊䷶ Peace Abounding: 1 1 4 2 1 3  : S R1 G1 M2 D1 N1 : OjIb

#632: C D♭ D F♯ G♯ B♭ :0 1 2 6 8 10: 1 ♭2 2 ♯4 ♯5 ♭7: 9sus♯4(♯5 add ♭9)
namesBySequenceIndex[631] = '''Aeraptimic'''
namesBySequenceIndexNegative[631] = ''''''
semitonesBySequenceIndex[631] = [0, 1, 2, 6, 8, 10]
poemBySequenceIndex[631] = '''B D F L N R'''
poemAuthors[631] = ''''''
#632: ䷊䷾ Peace Completed: 1 1 4 2 2 2  : S R1 G1 M2 D1 N2 : OjIk

#633: C D♭ D F♯ G♯ B :0 1 2 6 8 11: 1 ♭2 2 ♯4 ♯5 7: ma9sus♯4(♯5 add ♭9)
namesBySequenceIndex[632] = '''Aeolaptimic'''
namesBySequenceIndexNegative[632] = ''''''
semitonesBySequenceIndex[632] = [0, 1, 2, 6, 8, 11]
poemBySequenceIndex[632] = '''B D F L N S'''
poemAuthors[632] = ''''''
#633: ䷊䷕ Peace Adorning: 1 1 4 2 3 1  : S R1 G1 M2 D1 N3 : OjIl

#634: C D♭ D G♭ A B♭ :0 1 2 6 9 10: 1 ♭2 2 ♭5 6 ♭7: 9sus♭2(♭5 add 13)
namesBySequenceIndex[633] = '''Epytimic'''
namesBySequenceIndexNegative[633] = ''''''
semitonesBySequenceIndex[633] = [0, 1, 2, 6, 9, 10]
poemBySequenceIndex[633] = '''B D F L P R'''
poemAuthors[633] = ''''''
#634: ䷊䷐ Peace Following: 1 1 4 3 1 2  : S R1 G1 M2 D2 N2 : OjAd

#635: C D♭ D G♭ A B :0 1 2 6 9 11: 1 ♭2 2 ♭5 6 7: ma9sus♭2(♭5 add 13)
namesBySequenceIndex[634] = '''Macrimic'''
namesBySequenceIndexNegative[634] = ''''''
semitonesBySequenceIndex[634] = [0, 1, 2, 6, 9, 11]
poemBySequenceIndex[634] = '''B D F L P S'''
poemAuthors[634] = ''''''
#635: ䷊䷔ Peace Biting: 1 1 4 3 2 1  : S R1 G1 M2 D2 N3 : OjAf

#636: C D♭ D G♭ A♯ B :0 1 2 6 10 11: 1 ♭2 2 ♭5 ♯6 7: ma9sus♭2(♭5 add ♭7)
namesBySequenceIndex[635] = '''Polimic'''
namesBySequenceIndexNegative[635] = ''''''
semitonesBySequenceIndex[635] = [0, 1, 2, 6, 10, 11]
poemBySequenceIndex[635] = '''B D F L R S'''
poemAuthors[635] = ''''''
#636: ䷊䷩ Peace Augmenting: 1 1 4 4 1 1  : S R1 G1 M2 D3 N3 : OjAm

#637: C D♭ D G A♭ A :0 1 2 7 8 9: 1 ♭2 2 5 ♭6 6: 6sus2(add ♭6 ♭9)
namesBySequenceIndex[636] = ''''''
namesBySequenceIndexNegative[636] = ''''''
semitonesBySequenceIndex[636] = [0, 1, 2, 7, 8, 9]
poemBySequenceIndex[636] = '''B D F M N P'''
poemAuthors[636] = ''''''
#637: ䷊䷟ Peace Persevering: 1 1 5 1 1 3  : S R1 G1 P D1 N1 : Ojob

#638: C D♭ D G A♭ B♭ :0 1 2 7 8 10: 1 ♭2 2 5 ♭6 ♭7: 9sus♭2(add ♭13)
namesBySequenceIndex[637] = ''''''
namesBySequenceIndexNegative[637] = ''''''
semitonesBySequenceIndex[637] = [0, 1, 2, 7, 8, 10]
poemBySequenceIndex[637] = '''B D F M N R'''
poemAuthors[637] = ''''''
#638: ䷊䷯ Peace Source: 1 1 5 1 2 2  : S R1 G1 P D1 N2 : Ojok

#639: C D♭ D G A♭ B :0 1 2 7 8 11: 1 ♭2 2 5 ♭6 7: ma9sus♭2(add ♭13)
namesBySequenceIndex[638] = ''''''
namesBySequenceIndexNegative[638] = ''''''
semitonesBySequenceIndex[638] = [0, 1, 2, 7, 8, 11]
poemBySequenceIndex[638] = '''B D F M N S'''
poemAuthors[638] = ''''''
#639: ䷊䷑ Peace Correcting: 1 1 5 1 3 1  : S R1 G1 P D1 N3 : Ojol

#640: C D♭ D G A B♭ :0 1 2 7 9 10: 1 ♭2 2 5 6 ♭7: 9sus♭2(add 13)
namesBySequenceIndex[639] = ''''''
namesBySequenceIndexNegative[639] = ''''''
semitonesBySequenceIndex[639] = [0, 1, 2, 7, 9, 10]
poemBySequenceIndex[639] = '''B D F M P R'''
poemAuthors[639] = ''''''
#640: ䷊䷮ Peace Confining: 1 1 5 2 1 2  : S R1 G1 P D2 N2 : Ojed

#641: C D♭ D G A B :0 1 2 7 9 11: 1 ♭2 2 5 6 7: ma9sus♭2(add 13)
namesBySequenceIndex[640] = ''''''
namesBySequenceIndexNegative[640] = ''''''
semitonesBySequenceIndex[640] = [0, 1, 2, 7, 9, 11]
poemBySequenceIndex[640] = '''B D F M P S'''
poemAuthors[640] = ''''''
#641: ䷊䷿ Peace Unfinished: 1 1 5 2 2 1  : S R1 G1 P D2 N3 : Ojef

#642: C D♭ D G A♯ B :0 1 2 7 10 11: 1 ♭2 2 5 ♯6 7: ma9sus♭2(add ♭7)
namesBySequenceIndex[641] = ''''''
namesBySequenceIndexNegative[641] = ''''''
semitonesBySequenceIndex[641] = [0, 1, 2, 7, 10, 11]
poemBySequenceIndex[641] = '''B D F M R S'''
poemAuthors[641] = ''''''
#642: ䷊䷺ Peace Dispersing: 1 1 5 3 1 1  : S R1 G1 P D3 N3 : Ojem

#643: C D♭ D G♯ A B♭ :0 1 2 8 9 10: 1 ♭2 2 ♯5 6 ♭7: 9sus♭2(♯5 add 13)
namesBySequenceIndex[642] = ''''''
namesBySequenceIndexNegative[642] = ''''''
semitonesBySequenceIndex[642] = [0, 1, 2, 8, 9, 10]
poemBySequenceIndex[642] = '''B D F N P R'''
poemAuthors[642] = ''''''
#643: ䷊䷞ Peace Conjoining: 1 1 6 1 1 2  : S R1 G1 D1 D2 N2 : Ojid

#644: C D♭ D G♯ A B :0 1 2 8 9 11: 1 ♭2 2 ♯5 6 7: ma9sus♭2(♯5 add 13)
namesBySequenceIndex[643] = ''''''
namesBySequenceIndexNegative[643] = ''''''
semitonesBySequenceIndex[643] = [0, 1, 2, 8, 9, 11]
poemBySequenceIndex[643] = '''B D F N P S'''
poemAuthors[643] = ''''''
#644: ䷊䷷ Peace Wanderer: 1 1 6 1 2 1  : S R1 G1 D1 D2 N3 : Ojif

#645: C D♭ D G♯ A♯ B :0 1 2 8 10 11: 1 ♭2 2 ♯5 ♯6 7: ma9sus♭2(♯5 add ♭7)
namesBySequenceIndex[644] = ''''''
namesBySequenceIndexNegative[644] = ''''''
semitonesBySequenceIndex[644] = [0, 1, 2, 8, 10, 11]
poemBySequenceIndex[644] = '''B D F N R S'''
poemAuthors[644] = ''''''
#645: ䷊䷴ Peace Infiltrating: 1 1 6 2 1 1  : S R1 G1 D1 N2 N3 : Ojim

#646: C D♭ D A B♭ B :0 1 2 9 10 11: 1 ♭2 2 6 ♭7 7: ma9sus♭2(no 5 add ♭7 13)
namesBySequenceIndex[645] = ''''''
namesBySequenceIndexNegative[645] = ''''''
semitonesBySequenceIndex[645] = [0, 1, 2, 9, 10, 11]
poemBySequenceIndex[645] = '''B D F P R S'''
poemAuthors[645] = ''''''
#646: ䷊䷋ Peace Obstruction: 1 1 7 1 1 1  : S R1 G1 D2 N2 N3 : Ojag

#647: C D♭ D♯ E F G♭ :0 1 3 4 5 6: 1 ♭2 ♯2 3 4 ♭5: ma(♭5 add ♭9 ♯9 11)
namesBySequenceIndex[646] = ''''''
namesBySequenceIndexNegative[646] = ''''''
semitonesBySequenceIndex[646] = [0, 1, 3, 4, 5, 6]
poemBySequenceIndex[646] = '''B D G J K L'''
poemAuthors[646] = ''''''
#647: ䷉䷗ Treading Returning: 1 2 1 1 1 6  : S R1 R3 G3 M1 M2 : EgAj

#648: C D♭ D♯ E F G :0 1 3 4 5 7: 1 ♭2 ♯2 3 4 5: ma(add ♭9 ♯9 11)
namesBySequenceIndex[647] = ''''''
namesBySequenceIndexNegative[647] = ''''''
semitonesBySequenceIndex[647] = [0, 1, 3, 4, 5, 7]
poemBySequenceIndex[647] = '''B D G J K M'''
poemAuthors[647] = ''''''
#648: ䷉䷆ Treading Leading: 1 2 1 1 2 5  : S R1 R3 G3 M1 P : Egej

#649: C D♭ D♯ E F G♯ :0 1 3 4 5 8: 1 ♭2 ♯2 3 4 ♯5: ma(♯5 add ♭9 ♯9 11)
namesBySequenceIndex[648] = '''Stodimic'''
namesBySequenceIndexNegative[648] = ''''''
semitonesBySequenceIndex[648] = [0, 1, 3, 4, 5, 8]
poemBySequenceIndex[648] = '''B D G J K N'''
poemAuthors[648] = ''''''
#649: ䷉䷎ Treading Humble: 1 2 1 1 3 4  : S R1 R3 G3 M1 D1 : Egij

#650: C D♭ D♯ E F A :0 1 3 4 5 9: 1 ♭2 ♯2 3 4 6: 6(no 5 add ♭9 ♯9 11)
namesBySequenceIndex[649] = '''Kynimic'''
namesBySequenceIndexNegative[649] = ''''''
semitonesBySequenceIndex[649] = [0, 1, 3, 4, 5, 9]
poemBySequenceIndex[649] = '''B D G J K P'''
poemAuthors[649] = ''''''
#650: ䷉䷏ Treading Confident: 1 2 1 1 4 3  : S R1 R3 G3 M1 D2 : Egab

#651: C D♭ D♯ E F B♭ :0 1 3 4 5 10: 1 ♭2 ♯2 3 4 ♭7: 11(♭9 no 5 add ♯9)
namesBySequenceIndex[650] = ''''''
namesBySequenceIndexNegative[650] = ''''''
semitonesBySequenceIndex[650] = [0, 1, 3, 4, 5, 10]
poemBySequenceIndex[650] = '''B D G J K R'''
poemAuthors[650] = ''''''
#651: ䷉䷇ Treading Comparing: 1 2 1 1 5 2  : S R1 R3 G3 M1 N2 : Egak

#652: C D♭ D♯ E F B :0 1 3 4 5 11: 1 ♭2 ♯2 3 4 7: ma11(♭9 no 5 add ♯9)
namesBySequenceIndex[651] = ''''''
namesBySequenceIndexNegative[651] = ''''''
semitonesBySequenceIndex[651] = [0, 1, 3, 4, 5, 11]
poemBySequenceIndex[651] = '''B D G J K S'''
poemAuthors[651] = ''''''
#652: ䷉䷖ Treading Stripping: 1 2 1 1 6 1  : S R1 R3 G3 M1 N3 : Egal

#653: C D♭ D♯ E F♯ G :0 1 3 4 6 7: 1 ♭2 ♯2 3 ♯4 5: ma(add ♭9 ♯9 ♯11)
namesBySequenceIndex[652] = '''Istrian,Istrian (Croatia)'''
namesBySequenceIndexNegative[652] = ''''''
semitonesBySequenceIndex[652] = [0, 1, 3, 4, 6, 7]
poemBySequenceIndex[652] = '''B D G J L M'''
poemAuthors[652] = ''''''
#653: ䷹䷒ Joyous Arrived: 1 2 1 2 1 5  : S R1 R3 G3 M2 P : EdEj

#654: C D♭ D♯ E F♯ G♯ :0 1 3 4 6 8: 1 ♭2 ♯2 3 ♯4 ♯5: ma(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[653] = '''Barimic'''
namesBySequenceIndexNegative[653] = ''''''
semitonesBySequenceIndex[653] = [0, 1, 3, 4, 6, 8]
poemBySequenceIndex[653] = '''B D G J L N'''
poemAuthors[653] = ''''''
#654: ䷹䷣ Joyous Eclipse: 1 2 1 2 2 4  : S R1 R3 G3 M2 D1 : EdIj

#655: C D♭ D♯ E G♭ A :0 1 3 4 6 9: 1 ♭2 ♯2 3 ♭5 6: 6(♭5 add ♭9 ♯9)
namesBySequenceIndex[654] = '''Aeolygimic'''
namesBySequenceIndexNegative[654] = ''''''
semitonesBySequenceIndex[654] = [0, 1, 3, 4, 6, 9]
poemBySequenceIndex[654] = '''B D G J L P'''
poemAuthors[654] = ''''''
#655: ䷹䷲ Joyous Thunder: 1 2 1 2 3 3  : S R1 R3 G3 M2 D2 : EdAb

#656: C D♭ D♯ E G♭ B♭ :0 1 3 4 6 10: 1 ♭2 ♯2 3 ♭5 ♭7: 7(♭5 add ♭9 ♯9)
namesBySequenceIndex[655] = '''Sho 2,Locrimic'''
namesBySequenceIndexNegative[655] = ''''''
semitonesBySequenceIndex[655] = [0, 1, 3, 4, 6, 10]
poemBySequenceIndex[655] = '''B D G J L R'''
poemAuthors[655] = ''''''
#656: ䷹䷂ Joyous Sprouting: 1 2 1 2 4 2  : S R1 R3 G3 M2 N2 : EdAk

#657: C D♭ D♯ E G♭ B :0 1 3 4 6 11: 1 ♭2 ♯2 3 ♭5 7: ma7(♭5 add ♭9 ♯9)
namesBySequenceIndex[656] = ''''''
namesBySequenceIndexNegative[656] = ''''''
semitonesBySequenceIndex[656] = [0, 1, 3, 4, 6, 11]
poemBySequenceIndex[656] = '''B D G J L S'''
poemAuthors[656] = ''''''
#657: ䷹䷚ Joyous Nourishment: 1 2 1 2 5 1  : S R1 R3 G3 M2 N3 : EdAl

#658: C D♭ D♯ E G A♭ :0 1 3 4 7 8: 1 ♭2 ♯2 3 5 ♭6: ma(add ♭6 ♭9 ♯9)
namesBySequenceIndex[657] = '''Lygimic'''
namesBySequenceIndexNegative[657] = ''''''
semitonesBySequenceIndex[657] = [0, 1, 3, 4, 7, 8]
poemBySequenceIndex[657] = '''B D G J M N'''
poemAuthors[657] = ''''''
#658: ䷹䷭ Joyous Ascending: 1 2 1 3 1 4  : S R1 R3 G3 P D1 : Edoj

#659: C D♭ D♯ E G A :0 1 3 4 7 9: 1 ♭2 ♯2 3 5 6: 6(add ♭9 ♯9)
namesBySequenceIndex[658] = '''Rodimic,Blues Dorian Sexatonic,'''
namesBySequenceIndexNegative[658] = ''''''
semitonesBySequenceIndex[658] = [0, 1, 3, 4, 7, 9]
poemBySequenceIndex[658] = '''B D G J M P'''
poemAuthors[658] = ''''''
#659: ䷹䷧ Joyous Deliverance: 1 2 1 3 2 3  : S R1 R3 G3 P D2 : Edeb

#660: C D♭ D♯ E G B♭ :0 1 3 4 7 10: 1 ♭2 ♯2 3 5 ♭7: 7(add ♭9 ♯9)
namesBySequenceIndex[659] = '''Sonimic'''
namesBySequenceIndexNegative[659] = ''''''
semitonesBySequenceIndex[659] = [0, 1, 3, 4, 7, 10]
poemBySequenceIndex[659] = '''B D G J M R'''
poemAuthors[659] = ''''''
#660: ䷹䷜ Joyous Chasm: 1 2 1 3 3 2  : S R1 R3 G3 P N2 : Edek

#661: C D♭ D♯ E G B :0 1 3 4 7 11: 1 ♭2 ♯2 3 5 7: ma7(add ♭9 ♯9)
namesBySequenceIndex[660] = '''Dorimic'''
namesBySequenceIndexNegative[660] = ''''''
semitonesBySequenceIndex[660] = [0, 1, 3, 4, 7, 11]
poemBySequenceIndex[660] = '''B D G J M S'''
poemAuthors[660] = ''''''
#661: ䷹䷃ Joyous Naïve: 1 2 1 3 4 1  : S R1 R3 G3 P N3 : Edel

#662: C D♭ D♯ E G♯ A :0 1 3 4 8 9: 1 ♭2 ♯2 3 ♯5 6: 6(♯5 add ♭9 ♯9)
namesBySequenceIndex[661] = '''Aeologimic'''
namesBySequenceIndexNegative[661] = ''''''
semitonesBySequenceIndex[661] = [0, 1, 3, 4, 8, 9]
poemBySequenceIndex[661] = '''B D G J N P'''
poemAuthors[661] = ''''''
#662: ䷹䷽ Joyous Detail: 1 2 1 4 1 3  : S R1 R3 G3 D1 N1 : Edib

#663: C D♭ D♯ E G♯ B♭ :0 1 3 4 8 10: 1 ♭2 ♯2 3 ♯5 ♭7: 7(♯5 add ♭9 ♯9)
namesBySequenceIndex[662] = '''Katorimic'''
namesBySequenceIndexNegative[662] = ''''''
semitonesBySequenceIndex[662] = [0, 1, 3, 4, 8, 10]
poemBySequenceIndex[662] = '''B D G J N R'''
poemAuthors[662] = ''''''
#663: ䷹䷦ Joyous Limping: 1 2 1 4 2 2  : S R1 R3 G3 D1 N2 : Edik

#664: C D♭ D♯ E G♯ B :0 1 3 4 8 11: 1 ♭2 ♯2 3 ♯5 7: ma7(♯5 add ♭9 ♯9)
namesBySequenceIndex[663] = '''Dylimic'''
namesBySequenceIndexNegative[663] = ''''''
semitonesBySequenceIndex[663] = [0, 1, 3, 4, 8, 11]
poemBySequenceIndex[663] = '''B D G J N S'''
poemAuthors[663] = ''''''
#664: ䷹䷳ Joyous Stillness: 1 2 1 4 3 1  : S R1 R3 G3 D1 N3 : Edil

#665: C D♭ D♯ E A B♭ :0 1 3 4 9 10: 1 ♭2 ♯2 3 6 ♭7: 7(no 5 add ♭9 ♯9 13)
namesBySequenceIndex[664] = ''''''
namesBySequenceIndexNegative[664] = ''''''
semitonesBySequenceIndex[664] = [0, 1, 3, 4, 9, 10]
poemBySequenceIndex[664] = '''B D G J P R'''
poemAuthors[664] = ''''''
#665: ䷹䷬ Joyous Clustering: 1 2 1 5 1 2  : S R1 R3 G3 D2 N2 : Edad

#666: C D♭ D♯ E A B :0 1 3 4 9 11: 1 ♭2 ♯2 3 6 7: ma7(no 5 add ♭9 ♯9 13)
namesBySequenceIndex[665] = ''''''
namesBySequenceIndexNegative[665] = ''''''
semitonesBySequenceIndex[665] = [0, 1, 3, 4, 9, 11]
poemBySequenceIndex[665] = '''B D G J P S'''
poemAuthors[665] = ''''''
#666: ䷹䷢ Joyous Prospering: 1 2 1 5 2 1  : S R1 R3 G3 D2 N3 : Edaf

#667: C D♭ D♯ E A♯ B :0 1 3 4 10 11: 1 ♭2 ♯2 3 ♯6 7: ma7(no 5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[666] = ''''''
namesBySequenceIndexNegative[666] = ''''''
semitonesBySequenceIndex[666] = [0, 1, 3, 4, 10, 11]
poemBySequenceIndex[666] = '''B D G J R S'''
poemAuthors[666] = ''''''
#667: ䷹䷓ Joyous Seeing: 1 2 1 6 1 1  : S R1 R3 G3 D3 N3 : Edam

#668: C D♭ E♭ F F♯ G :0 1 3 5 6 7: 1 ♭2 ♭3 4 ♯4 5: mi(add ♭9 11 ♯11)
namesBySequenceIndex[667] = ''''''
namesBySequenceIndexNegative[667] = ''''''
semitonesBySequenceIndex[667] = [0, 1, 3, 5, 6, 7]
poemBySequenceIndex[667] = '''B D G K L M'''
poemAuthors[667] = ''''''
#668: ䷥䷒ Polarising Arrived: 1 2 2 1 1 5  : S R1 G2 M1 M2 P : EfEj

#669: C D♭ E♭ F G♭ A♭ :0 1 3 5 6 8: 1 ♭2 ♭3 4 ♭5 ♭6: °(add ♭6 ♭9 11)
namesBySequenceIndex[668] = '''Soptimic'''
namesBySequenceIndexNegative[668] = ''''''
semitonesBySequenceIndex[668] = [0, 1, 3, 5, 6, 8]
poemBySequenceIndex[668] = '''B D G K L N'''
poemAuthors[668] = ''''''
#669: ䷥䷣ Polarising Eclipse: 1 2 2 1 2 4  : S R1 G2 M1 M2 D1 : EfIj

#670: C D♭ E♭ F G♭ A :0 1 3 5 6 9: 1 ♭2 ♭3 4 ♭5 6: °7(add ♭9 11)
namesBySequenceIndex[669] = '''Double Phrygian,Parimic'''
namesBySequenceIndexNegative[669] = ''''''
semitonesBySequenceIndex[669] = [0, 1, 3, 5, 6, 9]
poemBySequenceIndex[669] = '''B D G K L P'''
poemAuthors[669] = ''''''
#670: ䷥䷲ Polarising Thunder: 1 2 2 1 3 3  : S R1 G2 M1 M2 D2 : EfAb

#671: C D♭ E♭ F G♭ B♭ :0 1 3 5 6 10: 1 ♭2 ♭3 4 ♭5 ♭7: ⌀11(♭9)
namesBySequenceIndex[670] = '''Honkoshi,Thocrimic,Honkoshi (Japan)'''
namesBySequenceIndexNegative[670] = ''''''
semitonesBySequenceIndex[670] = [0, 1, 3, 5, 6, 10]
poemBySequenceIndex[670] = '''B D G K L R'''
poemAuthors[670] = ''''''
#671: ䷥䷂ Polarising Sprouting: 1 2 2 1 4 2  : S R1 G2 M1 M2 N2 : EfAk

#672: C D♭ E♭ F G♭ B :0 1 3 5 6 11: 1 ♭2 ♭3 4 ♭5 7: °ma11(♭9)
namesBySequenceIndex[671] = ''''''
namesBySequenceIndexNegative[671] = ''''''
semitonesBySequenceIndex[671] = [0, 1, 3, 5, 6, 11]
poemBySequenceIndex[671] = '''B D G K L S'''
poemAuthors[671] = ''''''
#672: ䷥䷚ Polarising Nourishment: 1 2 2 1 5 1  : S R1 G2 M1 M2 N3 : EfAl

#673: C D♭ E♭ F G A♭ :0 1 3 5 7 8: 1 ♭2 ♭3 4 5 ♭6: mi(add ♭6 ♭9 11)
namesBySequenceIndex[672] = '''Raga Suddha Simantini,Zothimic'''
namesBySequenceIndexNegative[672] = ''''''
semitonesBySequenceIndex[672] = [0, 1, 3, 5, 7, 8]
poemBySequenceIndex[672] = '''B D G K M N'''
poemAuthors[672] = ''''''
#673: ䷥䷭ Polarising Ascending: 1 2 2 2 1 4  : S R1 G2 M1 P D1 : Efoj

#674: C D♭ E♭ F G A :0 1 3 5 7 9: 1 ♭2 ♭3 4 5 6: mi6(add ♭9 11)
namesBySequenceIndex[673] = '''Stogimic'''
namesBySequenceIndexNegative[673] = ''''''
semitonesBySequenceIndex[673] = [0, 1, 3, 5, 7, 9]
poemBySequenceIndex[673] = '''B D G K M P'''
poemAuthors[673] = ''''''
#674: ䷥䷧ Polarising Deliverance: 1 2 2 2 2 3  : S R1 G2 M1 P D2 : Efeb

#675: C D♭ E♭ F G B♭ :0 1 3 5 7 10: 1 ♭2 ♭3 4 5 ♭7: mi11(♭9)
namesBySequenceIndex[674] = '''Raga Gandharavam,Sarimic'''
namesBySequenceIndexNegative[674] = ''''''
semitonesBySequenceIndex[674] = [0, 1, 3, 5, 7, 10]
poemBySequenceIndex[674] = '''B D G K M R'''
poemAuthors[674] = ''''''
#675: ䷥䷜ Polarising Chasm: 1 2 2 2 3 2  : S R1 G2 M1 P N2 : Efek

#676: C D♭ E♭ F G B :0 1 3 5 7 11: 1 ♭2 ♭3 4 5 7: mima11(♭9)
namesBySequenceIndex[675] = '''Phrydimic'''
namesBySequenceIndexNegative[675] = ''''''
semitonesBySequenceIndex[675] = [0, 1, 3, 5, 7, 11]
poemBySequenceIndex[675] = '''B D G K M S'''
poemAuthors[675] = ''''''
#676: ䷥䷃ Polarising Naïve: 1 2 2 2 4 1  : S R1 G2 M1 P N3 : Efel

#677: C D♭ E♭ F G♯ A :0 1 3 5 8 9: 1 ♭2 ♭3 4 ♯5 6: mi6(♯5 add ♭9 11)
namesBySequenceIndex[676] = '''Radimic'''
namesBySequenceIndexNegative[676] = ''''''
semitonesBySequenceIndex[676] = [0, 1, 3, 5, 8, 9]
poemBySequenceIndex[676] = '''B D G K N P'''
poemAuthors[676] = ''''''
#677: ䷥䷽ Polarising Detail: 1 2 2 3 1 3  : S R1 G2 M1 D1 N1 : Efib

#678: C D♭ E♭ F G♯ B♭ :0 1 3 5 8 10: 1 ♭2 ♭3 4 ♯5 ♭7: mi11(♯5 ♭9)
namesBySequenceIndex[677] = '''Ritzu,Raga Suddha Todi,Ritzu (Japan),Eporimic'''
namesBySequenceIndexNegative[677] = ''''''
semitonesBySequenceIndex[677] = [0, 1, 3, 5, 8, 10]
poemBySequenceIndex[677] = '''B D G K N R'''
poemAuthors[677] = ''''''
#678: ䷥䷦ Polarising Limping: 1 2 2 3 2 2  : S R1 G2 M1 D1 N2 : Efik

#679: C D♭ E♭ F G♯ B :0 1 3 5 8 11: 1 ♭2 ♭3 4 ♯5 7: mima11(♯5 ♭9)
namesBySequenceIndex[678] = '''Raga Viyogavarali,Thothimic'''
namesBySequenceIndexNegative[678] = ''''''
semitonesBySequenceIndex[678] = [0, 1, 3, 5, 8, 11]
poemBySequenceIndex[678] = '''B D G K N S'''
poemAuthors[678] = ''''''
#679: ䷥䷳ Polarising Stillness: 1 2 2 3 3 1  : S R1 G2 M1 D1 N3 : Efil

#680: C D♭ E♭ F A B♭ :0 1 3 5 9 10: 1 ♭2 ♭3 4 6 ♭7: mi13(♭9 no 5)
namesBySequenceIndex[679] = '''Sagimic'''
namesBySequenceIndexNegative[679] = ''''''
semitonesBySequenceIndex[679] = [0, 1, 3, 5, 9, 10]
poemBySequenceIndex[679] = '''B D G K P R'''
poemAuthors[679] = ''''''
#680: ䷥䷬ Polarising Clustering: 1 2 2 4 1 2  : S R1 G2 M1 D2 N2 : Efad

#681: C D♭ E♭ F A B :0 1 3 5 9 11: 1 ♭2 ♭3 4 6 7: mima13(♭9 no 5)
namesBySequenceIndex[680] = '''Gadimic'''
namesBySequenceIndexNegative[680] = ''''''
semitonesBySequenceIndex[680] = [0, 1, 3, 5, 9, 11]
poemBySequenceIndex[680] = '''B D G K P S'''
poemAuthors[680] = ''''''
#681: ䷥䷢ Polarising Prospering: 1 2 2 4 2 1  : S R1 G2 M1 D2 N3 : Efaf

#682: C D♭ E♭ F A♯ B :0 1 3 5 10 11: 1 ♭2 ♭3 4 ♯6 7: mima11(♭9 no 5 add ♭7)
namesBySequenceIndex[681] = ''''''
namesBySequenceIndexNegative[681] = ''''''
semitonesBySequenceIndex[681] = [0, 1, 3, 5, 10, 11]
poemBySequenceIndex[681] = '''B D G K R S'''
poemAuthors[681] = ''''''
#682: ䷥䷓ Polarising Seeing: 1 2 2 5 1 1  : S R1 G2 M1 D3 N3 : Efam

#683: C D♭ E♭ F♯ G A♭ :0 1 3 6 7 8: 1 ♭2 ♭3 ♯4 5 ♭6: mi(add ♭6 ♭9 ♯11)
namesBySequenceIndex[682] = '''Zaptimic'''
namesBySequenceIndexNegative[682] = ''''''
semitonesBySequenceIndex[682] = [0, 1, 3, 6, 7, 8]
poemBySequenceIndex[682] = '''B D G L M N'''
poemAuthors[682] = ''''''
#683: ䷵䷊ Betrothing Peace: 1 2 3 1 1 4  : S R1 G2 M2 P D1 : EbOj

#684: C D♭ E♭ F♯ G A :0 1 3 6 7 9: 1 ♭2 ♭3 ♯4 5 6: mi6(add ♭9 ♯11)
namesBySequenceIndex[683] = '''Dodimic,Messiaen 2nd Mode Truncated'''
namesBySequenceIndexNegative[683] = ''''''
semitonesBySequenceIndex[683] = [0, 1, 3, 6, 7, 9]
poemBySequenceIndex[683] = '''B D G L M P'''
poemAuthors[683] = ''''''
#684: ䷵䷵ Betrothing Betrothing : 1 2 3 1 2 3  : S R1 G2 M2 P D2 : EbEb

#685: C D♭ E♭ F♯ G B♭ :0 1 3 6 7 10: 1 ♭2 ♭3 ♯4 5 ♭7: mi7(add ♭9 ♯11)
namesBySequenceIndex[684] = '''Thacrimic'''
namesBySequenceIndexNegative[684] = ''''''
semitonesBySequenceIndex[684] = [0, 1, 3, 6, 7, 10]
poemBySequenceIndex[684] = '''B D G L M R'''
poemAuthors[684] = ''''''
#685: ䷵䷻ Betrothing Composed: 1 2 3 1 3 2  : S R1 G2 M2 P N2 : EbEk

#686: C D♭ E♭ F♯ G B :0 1 3 6 7 11: 1 ♭2 ♭3 ♯4 5 7: mima7(add ♭9 ♯11)
namesBySequenceIndex[685] = '''Zodimic'''
namesBySequenceIndexNegative[685] = ''''''
semitonesBySequenceIndex[685] = [0, 1, 3, 6, 7, 11]
poemBySequenceIndex[685] = '''B D G L M S'''
poemAuthors[685] = ''''''
#686: ䷵䷨ Betrothing Diminishing: 1 2 3 1 4 1  : S R1 G2 M2 P N3 : EbEl

#687: C D♭ E♭ F♯ G♯ A :0 1 3 6 8 9: 1 ♭2 ♭3 ♯4 ♯5 6: °7(add ♭6 ♭9)
namesBySequenceIndex[686] = '''Molimic'''
namesBySequenceIndexNegative[686] = ''''''
semitonesBySequenceIndex[686] = [0, 1, 3, 6, 8, 9]
poemBySequenceIndex[686] = '''B D G L N P'''
poemAuthors[686] = ''''''
#687: ䷵䷶ Betrothing Abounding: 1 2 3 2 1 3  : S R1 G2 M2 D1 N1 : EbIb

#688: C D♭ E♭ F♯ G♯ B♭ :0 1 3 6 8 10: 1 ♭2 ♭3 ♯4 ♯5 ♭7: ⌀7(add ♭9 ♭13)
namesBySequenceIndex[687] = '''Raga Gurjari Todi,Aeolorimic'''
namesBySequenceIndexNegative[687] = ''''''
semitonesBySequenceIndex[687] = [0, 1, 3, 6, 8, 10]
poemBySequenceIndex[687] = '''B D G L N R'''
poemAuthors[687] = ''''''
#688: ䷵䷾ Betrothing Completed: 1 2 3 2 2 2  : S R1 G2 M2 D1 N2 : EbIk

#689: C D♭ E♭ F♯ G♯ B :0 1 3 6 8 11: 1 ♭2 ♭3 ♯4 ♯5 7: °ma7(add ♭9 ♭13)
namesBySequenceIndex[688] = '''Stathimic'''
namesBySequenceIndexNegative[688] = ''''''
semitonesBySequenceIndex[688] = [0, 1, 3, 6, 8, 11]
poemBySequenceIndex[688] = '''B D G L N S'''
poemAuthors[688] = ''''''
#689: ䷵䷕ Betrothing Adorning: 1 2 3 2 3 1  : S R1 G2 M2 D1 N3 : EbIl

#690: C D♭ E♭ G♭ A B♭ :0 1 3 6 9 10: 1 ♭2 ♭3 ♭5 6 ♭7: ⌀7(add ♭9 13)
namesBySequenceIndex[689] = '''Dacrimic'''
namesBySequenceIndexNegative[689] = ''''''
semitonesBySequenceIndex[689] = [0, 1, 3, 6, 9, 10]
poemBySequenceIndex[689] = '''B D G L P R'''
poemAuthors[689] = ''''''
#690: ䷵䷐ Betrothing Following: 1 2 3 3 1 2  : S R1 G2 M2 D2 N2 : EbAd

#691: C D♭ E♭ G♭ A B :0 1 3 6 9 11: 1 ♭2 ♭3 ♭5 6 7: °ma7(add ♭9 13)
namesBySequenceIndex[690] = '''Gocrimic'''
namesBySequenceIndexNegative[690] = ''''''
semitonesBySequenceIndex[690] = [0, 1, 3, 6, 9, 11]
poemBySequenceIndex[690] = '''B D G L P S'''
poemAuthors[690] = ''''''
#691: ䷵䷔ Betrothing Biting: 1 2 3 3 2 1  : S R1 G2 M2 D2 N3 : EbAf

#692: C D♭ E♭ G♭ A♯ B :0 1 3 6 10 11: 1 ♭2 ♭3 ♭5 ♯6 7: °ma7(add ♭7 ♭9)
namesBySequenceIndex[691] = '''Ryrimic'''
namesBySequenceIndexNegative[691] = ''''''
semitonesBySequenceIndex[691] = [0, 1, 3, 6, 10, 11]
poemBySequenceIndex[691] = '''B D G L R S'''
poemAuthors[691] = ''''''
#692: ䷵䷩ Betrothing Augmenting: 1 2 3 4 1 1  : S R1 G2 M2 D3 N3 : EbAm

#693: C D♭ E♭ G A♭ A :0 1 3 7 8 9: 1 ♭2 ♭3 5 ♭6 6: mi6(add ♭6 ♭9)
namesBySequenceIndex[692] = '''Tholimic'''
namesBySequenceIndexNegative[692] = ''''''
semitonesBySequenceIndex[692] = [0, 1, 3, 7, 8, 9]
poemBySequenceIndex[692] = '''B D G M N P'''
poemAuthors[692] = ''''''
#693: ䷵䷟ Betrothing Persevering: 1 2 4 1 1 3  : S R1 G2 P D1 N1 : Ebob

#694: C D♭ E♭ G A♭ B♭ :0 1 3 7 8 10: 1 ♭2 ♭3 5 ♭6 ♭7: mi7(add ♭9 ♭13)
namesBySequenceIndex[693] = '''Raga Kashyapi,Zalimic'''
namesBySequenceIndexNegative[693] = ''''''
semitonesBySequenceIndex[693] = [0, 1, 3, 7, 8, 10]
poemBySequenceIndex[693] = '''B D G M N R'''
poemAuthors[693] = ''''''
#694: ䷵䷯ Betrothing Source: 1 2 4 1 2 2  : S R1 G2 P D1 N2 : Ebok

#695: C D♭ E♭ G A♭ B :0 1 3 7 8 11: 1 ♭2 ♭3 5 ♭6 7: mima7(add ♭9 ♭13)
namesBySequenceIndex[694] = '''Panimic'''
namesBySequenceIndexNegative[694] = ''''''
semitonesBySequenceIndex[694] = [0, 1, 3, 7, 8, 11]
poemBySequenceIndex[694] = '''B D G M N S'''
poemAuthors[694] = ''''''
#695: ䷵䷑ Betrothing Correcting: 1 2 4 1 3 1  : S R1 G2 P D1 N3 : Ebol

#696: C D♭ E♭ G A B♭ :0 1 3 7 9 10: 1 ♭2 ♭3 5 6 ♭7: mi7(add ♭9 13)
namesBySequenceIndex[695] = '''Raga Salagavarali,Epatimic'''
namesBySequenceIndexNegative[695] = ''''''
semitonesBySequenceIndex[695] = [0, 1, 3, 7, 9, 10]
poemBySequenceIndex[695] = '''B D G M P R'''
poemAuthors[695] = ''''''
#696: ䷵䷮ Betrothing Confining: 1 2 4 2 1 2  : S R1 G2 P D2 N2 : Ebed

#697: C D♭ E♭ G A B :0 1 3 7 9 11: 1 ♭2 ♭3 5 6 7: mima7(add ♭9 13)
namesBySequenceIndex[696] = '''Sythimic'''
namesBySequenceIndexNegative[696] = ''''''
semitonesBySequenceIndex[696] = [0, 1, 3, 7, 9, 11]
poemBySequenceIndex[696] = '''B D G M P S'''
poemAuthors[696] = ''''''
#697: ䷵䷿ Betrothing Unfinished: 1 2 4 2 2 1  : S R1 G2 P D2 N3 : Ebef

#698: C D♭ E♭ G A♯ B :0 1 3 7 10 11: 1 ♭2 ♭3 5 ♯6 7: mima7(add ♭7 ♭9)
namesBySequenceIndex[697] = '''Epacrimic'''
namesBySequenceIndexNegative[697] = ''''''
semitonesBySequenceIndex[697] = [0, 1, 3, 7, 10, 11]
poemBySequenceIndex[697] = '''B D G M R S'''
poemAuthors[697] = ''''''
#698: ䷵䷺ Betrothing Dispersing: 1 2 4 3 1 1  : S R1 G2 P D3 N3 : Ebem

#699: C D♭ E♭ G♯ A B♭ :0 1 3 8 9 10: 1 ♭2 ♭3 ♯5 6 ♭7: mi7(♯5 add ♭9 13)
namesBySequenceIndex[698] = ''''''
namesBySequenceIndexNegative[698] = ''''''
semitonesBySequenceIndex[698] = [0, 1, 3, 8, 9, 10]
poemBySequenceIndex[698] = '''B D G N P R'''
poemAuthors[698] = ''''''
#699: ䷵䷞ Betrothing Conjoining: 1 2 5 1 1 2  : S R1 G2 D1 D2 N2 : Ebid

#700: C D♭ E♭ G♯ A B :0 1 3 8 9 11: 1 ♭2 ♭3 ♯5 6 7: mima7(♯5 add ♭9 13)
namesBySequenceIndex[699] = ''''''
namesBySequenceIndexNegative[699] = ''''''
semitonesBySequenceIndex[699] = [0, 1, 3, 8, 9, 11]
poemBySequenceIndex[699] = '''B D G N P S'''
poemAuthors[699] = ''''''
#700: ䷵䷷ Betrothing Wanderer: 1 2 5 1 2 1  : S R1 G2 D1 D2 N3 : Ebif

#701: C D♭ E♭ G♯ A♯ B :0 1 3 8 10 11: 1 ♭2 ♭3 ♯5 ♯6 7: mima7(♯5 add ♭7 ♭9)
namesBySequenceIndex[700] = ''''''
namesBySequenceIndexNegative[700] = ''''''
semitonesBySequenceIndex[700] = [0, 1, 3, 8, 10, 11]
poemBySequenceIndex[700] = '''B D G N R S'''
poemAuthors[700] = ''''''
#701: ䷵䷴ Betrothing Infiltrating: 1 2 5 2 1 1  : S R1 G2 D1 N2 N3 : Ebim

#702: C D♭ E♭ A B♭ B :0 1 3 9 10 11: 1 ♭2 ♭3 6 ♭7 7: mima7(no 5 add ♭7 ♭9 13)
namesBySequenceIndex[701] = ''''''
namesBySequenceIndexNegative[701] = ''''''
semitonesBySequenceIndex[701] = [0, 1, 3, 9, 10, 11]
poemBySequenceIndex[701] = '''B D G P R S'''
poemAuthors[701] = ''''''
#702: ䷵䷋ Betrothing Obstruction: 1 2 6 1 1 1  : S R1 G2 D2 N2 N3 : Ebag

#703: C D♭ E F F♯ G :0 1 4 5 6 7: 1 ♭2 3 4 ♯4 5: ma(add ♭9 11 ♯11)
namesBySequenceIndex[702] = ''''''
namesBySequenceIndexNegative[702] = ''''''
semitonesBySequenceIndex[702] = [0, 1, 4, 5, 6, 7]
poemBySequenceIndex[702] = '''B D J K L M'''
poemAuthors[702] = ''''''
#703: ䷼䷒ Truth Arrived: 1 3 1 1 1 5  : S R1 G3 M1 M2 P : EmEj

#704: C D♭ E F G♭ A♭ :0 1 4 5 6 8: 1 ♭2 3 4 ♭5 ♭6: ma(♯5 add ♭9 11 ♯11)
namesBySequenceIndex[703] = '''Rythimic'''
namesBySequenceIndexNegative[703] = ''''''
semitonesBySequenceIndex[703] = [0, 1, 4, 5, 6, 8]
poemBySequenceIndex[703] = '''B D J K L N'''
poemAuthors[703] = ''''''
#704: ䷼䷣ Truth Eclipse: 1 3 1 1 2 4  : S R1 G3 M1 M2 D1 : EmIj

#705: C D♭ E F G♭ A :0 1 4 5 6 9: 1 ♭2 3 4 ♭5 6: 6(♭5 add ♭9 11)
namesBySequenceIndex[704] = '''Mogimic'''
namesBySequenceIndexNegative[704] = ''''''
semitonesBySequenceIndex[704] = [0, 1, 4, 5, 6, 9]
poemBySequenceIndex[704] = '''B D J K L P'''
poemAuthors[704] = ''''''
#705: ䷼䷲ Truth Thunder: 1 3 1 1 3 3  : S R1 G3 M1 M2 D2 : EmAb

#706: C D♭ E F G♭ B♭ :0 1 4 5 6 10: 1 ♭2 3 4 ♭5 ♭7: 11(♭5 ♭9)
namesBySequenceIndex[705] = '''Aerygimic'''
namesBySequenceIndexNegative[705] = ''''''
semitonesBySequenceIndex[705] = [0, 1, 4, 5, 6, 10]
poemBySequenceIndex[705] = '''B D J K L R'''
poemAuthors[705] = ''''''
#706: ䷼䷂ Truth Sprouting: 1 3 1 1 4 2  : S R1 G3 M1 M2 N2 : EmAk

#707: C D♭ E F G♭ B :0 1 4 5 6 11: 1 ♭2 3 4 ♭5 7: ma11(♭5 ♭9)
namesBySequenceIndex[706] = ''''''
namesBySequenceIndexNegative[706] = ''''''
semitonesBySequenceIndex[706] = [0, 1, 4, 5, 6, 11]
poemBySequenceIndex[706] = '''B D J K L S'''
poemAuthors[706] = ''''''
#707: ䷼䷚ Truth Nourishment: 1 3 1 1 5 1  : S R1 G3 M1 M2 N3 : EmAl

#708: C D♭ E F G A♭ :0 1 4 5 7 8: 1 ♭2 3 4 5 ♭6: ma(add ♭6 ♭9 11)
namesBySequenceIndex[707] = '''Raga Malahari (descending),Raga Purna Pancama,Ionolimic'''
namesBySequenceIndexNegative[707] = ''''''
semitonesBySequenceIndex[707] = [0, 1, 4, 5, 7, 8]
poemBySequenceIndex[707] = '''B D J K M N'''
poemAuthors[707] = ''''''
#708: ䷼䷭ Truth Ascending: 1 3 1 2 1 4  : S R1 G3 M1 P D1 : Emoj

#709: C D♭ E F G A :0 1 4 5 7 9: 1 ♭2 3 4 5 6: 6(add ♭9 11)
namesBySequenceIndex[708] = '''Raga Kalavati,Raga Ragamalini,Zydimic'''
namesBySequenceIndexNegative[708] = ''''''
semitonesBySequenceIndex[708] = [0, 1, 4, 5, 7, 9]
poemBySequenceIndex[708] = '''B D J K M P'''
poemAuthors[708] = ''''''
#709: ䷼䷧ Truth Deliverance: 1 3 1 2 2 3  : S R1 G3 M1 P D2 : Emeb

#710: C D♭ E F G B♭ :0 1 4 5 7 10: 1 ♭2 3 4 5 ♭7: 11(♭9)
namesBySequenceIndex[709] = '''Raga Gaula,Pagimic'''
namesBySequenceIndexNegative[709] = ''''''
semitonesBySequenceIndex[709] = [0, 1, 4, 5, 7, 10]
poemBySequenceIndex[709] = '''B D J K M R'''
poemAuthors[709] = ''''''
#710: ䷼䷜ Truth Chasm: 1 3 1 2 3 2  : S R1 G3 M1 P N2 : Emek

#711: C D♭ E F G B :0 1 4 5 7 11: 1 ♭2 3 4 5 7: ma11(♭9)
namesBySequenceIndex[710] = '''Raga Gowla (descending),Raga Jaganmohini (descending),Katadimic'''
namesBySequenceIndexNegative[710] = ''''''
semitonesBySequenceIndex[710] = [0, 1, 4, 5, 7, 11]
poemBySequenceIndex[710] = '''B D J K M S'''
poemAuthors[710] = ''''''
#711: ䷼䷃ Truth Naïve: 1 3 1 2 4 1  : S R1 G3 M1 P N3 : Emel

#712: C D♭ E F G♯ A :0 1 4 5 8 9: 1 ♭2 3 4 ♯5 6: 6(♯5 add ♭9 11)
namesBySequenceIndex[711] = '''Inverted Augmented,Prometheus Liszt,"Ode-to-Napoleon" Hexachord,Aeologimic'''
namesBySequenceIndexNegative[711] = ''''''
semitonesBySequenceIndex[711] = [0, 1, 4, 5, 8, 9]
poemBySequenceIndex[711] = '''B D J K N P'''
poemAuthors[711] = ''''''
#712: ䷼䷽ Truth Detail: 1 3 1 3 1 3  : S R1 G3 M1 D1 N1 : Emib

#713: C D♭ E F G♯ B♭ :0 1 4 5 8 10: 1 ♭2 3 4 ♯5 ♭7: 11(♯5 ♭9)
namesBySequenceIndex[712] = '''Raga Vasantha,Raga Lalita Bhairav,Dolimic'''
namesBySequenceIndexNegative[712] = ''''''
semitonesBySequenceIndex[712] = [0, 1, 4, 5, 8, 10]
poemBySequenceIndex[712] = '''B D J K N R'''
poemAuthors[712] = ''''''
#713: ䷼䷦ Truth Limping: 1 3 1 3 2 2  : S R1 G3 M1 D1 N2 : Emik

#714: C D♭ E F G♯ B :0 1 4 5 8 11: 1 ♭2 3 4 ♯5 7: ma11(♯5 ♭9)
namesBySequenceIndex[713] = '''Raga Lalita,Raga Sohini,Katycrimic'''
namesBySequenceIndexNegative[713] = ''''''
semitonesBySequenceIndex[713] = [0, 1, 4, 5, 8, 11]
poemBySequenceIndex[713] = '''B D J K N S'''
poemAuthors[713] = ''''''
#714: ䷼䷳ Truth Stillness: 1 3 1 3 3 1  : S R1 G3 M1 D1 N3 : Emil

#715: C D♭ E F A B♭ :0 1 4 5 9 10: 1 ♭2 3 4 6 ♭7: 13(♭9 no 5)
namesBySequenceIndex[714] = '''Raga Rudra Pancama,Lalimic'''
namesBySequenceIndexNegative[714] = ''''''
semitonesBySequenceIndex[714] = [0, 1, 4, 5, 9, 10]
poemBySequenceIndex[714] = '''B D J K P R'''
poemAuthors[714] = ''''''
#715: ䷼䷬ Truth Clustering: 1 3 1 4 1 2  : S R1 G3 M1 D2 N2 : Emad

#716: C D♭ E F A B :0 1 4 5 9 11: 1 ♭2 3 4 6 7: ma13(♭9 no 5)
namesBySequenceIndex[715] = '''Raga Vasanta (descending),Raga Chaya Vati,Lyrimic'''
namesBySequenceIndexNegative[715] = ''''''
semitonesBySequenceIndex[715] = [0, 1, 4, 5, 9, 11]
poemBySequenceIndex[715] = '''B D J K P S'''
poemAuthors[715] = ''''''
#716: ䷼䷢ Truth Prospering: 1 3 1 4 2 1  : S R1 G3 M1 D2 N3 : Emaf

#717: C D♭ E F A♯ B :0 1 4 5 10 11: 1 ♭2 3 4 ♯6 7: ma11(♭9 no 5 add ♭7)
namesBySequenceIndex[716] = ''''''
namesBySequenceIndexNegative[716] = ''''''
semitonesBySequenceIndex[716] = [0, 1, 4, 5, 10, 11]
poemBySequenceIndex[716] = '''B D J K R S'''
poemAuthors[716] = ''''''
#717: ䷼䷓ Truth Seeing: 1 3 1 5 1 1  : S R1 G3 M1 D3 N3 : Emam

#718: C D♭ E F♯ G A♭ :0 1 4 6 7 8: 1 ♭2 3 ♯4 5 ♭6: ma(add ♭6 ♭9 ♯11)
namesBySequenceIndex[717] = '''Raga Indupriya,Raga Dhaval,Phrogimic'''
namesBySequenceIndexNegative[717] = ''''''
semitonesBySequenceIndex[717] = [0, 1, 4, 6, 7, 8]
poemBySequenceIndex[717] = '''B D J L M N'''
poemAuthors[717] = ''''''
#718: ䷻䷊ Composed Peace: 1 3 2 1 1 4  : S R1 G3 M2 P D1 : EkOj

#719: C D♭ E F♯ G A :0 1 4 6 7 9: 1 ♭2 3 ♯4 5 6: 6(add ♭9 ♯11)
namesBySequenceIndex[718] = '''Ionadimic'''
namesBySequenceIndexNegative[718] = ''''''
semitonesBySequenceIndex[718] = [0, 1, 4, 6, 7, 9]
poemBySequenceIndex[718] = '''B D J L M P'''
poemAuthors[718] = ''''''
#719: ䷻䷵ Composed Betrothing: 1 3 2 1 2 3  : S R1 G3 M2 P D2 : EkEb

#720: C D♭ E F♯ G B♭ :0 1 4 6 7 10: 1 ♭2 3 ♯4 5 ♭7: 7(add ♭9 ♯11)
namesBySequenceIndex[719] = '''Petrushka Chord,Tritone,Stylimic'''
namesBySequenceIndexNegative[719] = ''''''
semitonesBySequenceIndex[719] = [0, 1, 4, 6, 7, 10]
poemBySequenceIndex[719] = '''B D J L M R'''
poemAuthors[719] = ''''''
#720: ䷻䷻ Composed Composed : 1 3 2 1 3 2  : S R1 G3 M2 P N2 : EkEk

#721: C D♭ E F♯ G B :0 1 4 6 7 11: 1 ♭2 3 ♯4 5 7: ma7(add ♭9 ♯11)
namesBySequenceIndex[720] = '''Raga Mandari,Raga Gamakakriya,Raga Hamsanarayami,Gogimic'''
namesBySequenceIndexNegative[720] = ''''''
semitonesBySequenceIndex[720] = [0, 1, 4, 6, 7, 11]
poemBySequenceIndex[720] = '''B D J L M S'''
poemAuthors[720] = ''''''
#721: ䷻䷨ Composed Diminishing: 1 3 2 1 4 1  : S R1 G3 M2 P N3 : EkEl

#722: C D♭ E F♯ G♯ A :0 1 4 6 8 9: 1 ♭2 3 ♯4 ♯5 6: 6(♯5 add ♭9 ♯11)
namesBySequenceIndex[721] = '''Raga Hejjajji,Aerylimic'''
namesBySequenceIndexNegative[721] = ''''''
semitonesBySequenceIndex[721] = [0, 1, 4, 6, 8, 9]
poemBySequenceIndex[721] = '''B D J L N P'''
poemAuthors[721] = ''''''
#722: ䷻䷶ Composed Abounding: 1 3 2 2 1 3  : S R1 G3 M2 D1 N1 : EkIb

#723: C D♭ E F♯ G♯ B♭ :0 1 4 6 8 10: 1 ♭2 3 ♯4 ♯5 ♭7: 7(♯5 add ♭9 ♯11)
namesBySequenceIndex[722] = '''Gygimic'''
namesBySequenceIndexNegative[722] = ''''''
semitonesBySequenceIndex[722] = [0, 1, 4, 6, 8, 10]
poemBySequenceIndex[722] = '''B D J L N R'''
poemAuthors[722] = ''''''
#723: ䷻䷾ Composed Completed: 1 3 2 2 2 2  : S R1 G3 M2 D1 N2 : EkIk

#724: C D♭ E F♯ G♯ B :0 1 4 6 8 11: 1 ♭2 3 ♯4 ♯5 7: ma7(♯5 add ♭9 ♯11)
namesBySequenceIndex[723] = '''Paptimic'''
namesBySequenceIndexNegative[723] = ''''''
semitonesBySequenceIndex[723] = [0, 1, 4, 6, 8, 11]
poemBySequenceIndex[723] = '''B D J L N S'''
poemAuthors[723] = ''''''
#724: ䷻䷕ Composed Adorning: 1 3 2 2 3 1  : S R1 G3 M2 D1 N3 : EkIl

#725: C D♭ E G♭ A B♭ :0 1 4 6 9 10: 1 ♭2 3 ♭5 6 ♭7: 7(♭5 add ♭9 13)
namesBySequenceIndex[724] = '''Prometheus Neapolitan,Monimic'''
namesBySequenceIndexNegative[724] = ''''''
semitonesBySequenceIndex[724] = [0, 1, 4, 6, 9, 10]
poemBySequenceIndex[724] = '''B D J L P R'''
poemAuthors[724] = ''''''
#725: ䷻䷐ Composed Following: 1 3 2 3 1 2  : S R1 G3 M2 D2 N2 : EkAd

#726: C D♭ E G♭ A B :0 1 4 6 9 11: 1 ♭2 3 ♭5 6 7: ma7(♭5 add ♭9 13)
namesBySequenceIndex[725] = '''Raga Hamsanandi,Raga Pancama,Raga Puriya 2,Raga Marva,Raga Sohni,Lydimic'''
namesBySequenceIndexNegative[725] = ''''''
semitonesBySequenceIndex[725] = [0, 1, 4, 6, 9, 11]
poemBySequenceIndex[725] = '''B D J L P S'''
poemAuthors[725] = ''''''
#726: ䷻䷔ Composed Biting: 1 3 2 3 2 1  : S R1 G3 M2 D2 N3 : EkAf

#727: C D♭ E G♭ A♯ B :0 1 4 6 10 11: 1 ♭2 3 ♭5 ♯6 7: ma7(♭5 add ♭7 ♭9)
namesBySequenceIndex[726] = '''Ladimic'''
namesBySequenceIndexNegative[726] = ''''''
semitonesBySequenceIndex[726] = [0, 1, 4, 6, 10, 11]
poemBySequenceIndex[726] = '''B D J L R S'''
poemAuthors[726] = ''''''
#727: ䷻䷩ Composed Augmenting: 1 3 2 4 1 1  : S R1 G3 M2 D3 N3 : EkAm

#728: C D♭ E G A♭ A :0 1 4 7 8 9: 1 ♭2 3 5 ♭6 6: 6(add ♭6 ♭9)
namesBySequenceIndex[727] = '''Raga Kalagada,Loptimic'''
namesBySequenceIndexNegative[727] = ''''''
semitonesBySequenceIndex[727] = [0, 1, 4, 7, 8, 9]
poemBySequenceIndex[727] = '''B D J M N P'''
poemAuthors[727] = ''''''
#728: ䷻䷟ Composed Persevering: 1 3 3 1 1 3  : S R1 G3 P D1 N1 : Ekob

#729: C D♭ E G A♭ B♭ :0 1 4 7 8 10: 1 ♭2 3 5 ♭6 ♭7: 7(add ♭9 ♭13)
namesBySequenceIndex[728] = '''Lolimic'''
namesBySequenceIndexNegative[728] = ''''''
semitonesBySequenceIndex[728] = [0, 1, 4, 7, 8, 10]
poemBySequenceIndex[728] = '''B D J M N R'''
poemAuthors[728] = ''''''
#729: ䷻䷯ Composed Source: 1 3 3 1 2 2  : S R1 G3 P D1 N2 : Ekok

#730: C D♭ E G A♭ B :0 1 4 7 8 11: 1 ♭2 3 5 ♭6 7: ma7(add ♭9 ♭13)
namesBySequenceIndex[729] = '''Raga Bowli (descending),Raga Bauli,Aerynimic'''
namesBySequenceIndexNegative[729] = ''''''
semitonesBySequenceIndex[729] = [0, 1, 4, 7, 8, 11]
poemBySequenceIndex[729] = '''B D J M N S'''
poemAuthors[729] = ''''''
#730: ䷻䷑ Composed Correcting: 1 3 3 1 3 1  : S R1 G3 P D1 N3 : Ekol

#731: C D♭ E G A B♭ :0 1 4 7 9 10: 1 ♭2 3 5 6 ♭7: 7(add ♭9 13)
namesBySequenceIndex[730] = '''Raga Malayamarutam,Rygimic'''
namesBySequenceIndexNegative[730] = ''''''
semitonesBySequenceIndex[730] = [0, 1, 4, 7, 9, 10]
poemBySequenceIndex[730] = '''B D J M P R'''
poemAuthors[730] = ''''''
#731: ䷻䷮ Composed Confining: 1 3 3 2 1 2  : S R1 G3 P D2 N2 : Eked

#732: C D♭ E G A B :0 1 4 7 9 11: 1 ♭2 3 5 6 7: ma7(add ♭9 13)
namesBySequenceIndex[731] = '''Banimic'''
namesBySequenceIndexNegative[731] = ''''''
semitonesBySequenceIndex[731] = [0, 1, 4, 7, 9, 11]
poemBySequenceIndex[731] = '''B D J M P S'''
poemAuthors[731] = ''''''
#732: ䷻䷿ Composed Unfinished: 1 3 3 2 2 1  : S R1 G3 P D2 N3 : Ekef

#733: C D♭ E G A♯ B :0 1 4 7 10 11: 1 ♭2 3 5 ♯6 7: ma7(add ♭7 ♭9)
namesBySequenceIndex[732] = '''Ionaphimic'''
namesBySequenceIndexNegative[732] = ''''''
semitonesBySequenceIndex[732] = [0, 1, 4, 7, 10, 11]
poemBySequenceIndex[732] = '''B D J M R S'''
poemAuthors[732] = ''''''
#733: ䷻䷺ Composed Dispersing: 1 3 3 3 1 1  : S R1 G3 P D3 N3 : Ekem

#734: C D♭ E G♯ A B♭ :0 1 4 8 9 10: 1 ♭2 3 ♯5 6 ♭7: 7(♯5 add ♭9 13)
namesBySequenceIndex[733] = '''Kyptimic'''
namesBySequenceIndexNegative[733] = ''''''
semitonesBySequenceIndex[733] = [0, 1, 4, 8, 9, 10]
poemBySequenceIndex[733] = '''B D J N P R'''
poemAuthors[733] = ''''''
#734: ䷻䷞ Composed Conjoining: 1 3 4 1 1 2  : S R1 G3 D1 D2 N2 : Ekid

#735: C D♭ E G♯ A B :0 1 4 8 9 11: 1 ♭2 3 ♯5 6 7: ma7(♯5 add ♭9 13)
namesBySequenceIndex[734] = '''Ionygimic'''
namesBySequenceIndexNegative[734] = ''''''
semitonesBySequenceIndex[734] = [0, 1, 4, 8, 9, 11]
poemBySequenceIndex[734] = '''B D J N P S'''
poemAuthors[734] = ''''''
#735: ䷻䷷ Composed Wanderer: 1 3 4 1 2 1  : S R1 G3 D1 D2 N3 : Ekif

#736: C D♭ E G♯ A♯ B :0 1 4 8 10 11: 1 ♭2 3 ♯5 ♯6 7: ma7(♯5 add ♭7 ♭9)
namesBySequenceIndex[735] = '''Synimic'''
namesBySequenceIndexNegative[735] = ''''''
semitonesBySequenceIndex[735] = [0, 1, 4, 8, 10, 11]
poemBySequenceIndex[735] = '''B D J N R S'''
poemAuthors[735] = ''''''
#736: ䷻䷴ Composed Infiltrating: 1 3 4 2 1 1  : S R1 G3 D1 N2 N3 : Ekim

#737: C D♭ E A B♭ B :0 1 4 9 10 11: 1 ♭2 3 6 ♭7 7: ma7(no 5 add ♭7 ♭9 13)
namesBySequenceIndex[736] = ''''''
namesBySequenceIndexNegative[736] = ''''''
semitonesBySequenceIndex[736] = [0, 1, 4, 9, 10, 11]
poemBySequenceIndex[736] = '''B D J P R S'''
poemAuthors[736] = ''''''
#737: ䷻䷋ Composed Obstruction: 1 3 5 1 1 1  : S R1 G3 D2 N2 N3 : Ekag

#738: C D♭ F F♯ G A♭ :0 1 5 6 7 8: 1 ♭2 4 ♯4 5 ♭6: sus4(add ♭6 ♭9 ♯11)
namesBySequenceIndex[737] = '''Kygimic'''
namesBySequenceIndexNegative[737] = ''''''
semitonesBySequenceIndex[737] = [0, 1, 5, 6, 7, 8]
poemBySequenceIndex[737] = '''B D K L M N'''
poemAuthors[737] = ''''''
#738: ䷨䷊ Diminishing Peace: 1 4 1 1 1 4  : S R1 M1 M2 P D1 : ElOj

#739: C D♭ F F♯ G A :0 1 5 6 7 9: 1 ♭2 4 ♯4 5 6: 6sus4(add ♭9 ♯11)
namesBySequenceIndex[738] = '''Rorimic'''
namesBySequenceIndexNegative[738] = ''''''
semitonesBySequenceIndex[738] = [0, 1, 5, 6, 7, 9]
poemBySequenceIndex[738] = '''B D K L M P'''
poemAuthors[738] = ''''''
#739: ䷨䷵ Diminishing Betrothing: 1 4 1 1 2 3  : S R1 M1 M2 P D2 : ElEb

#740: C D♭ F F♯ G B♭ :0 1 5 6 7 10: 1 ♭2 4 ♯4 5 ♭7: 11sus♯4(♭9)
namesBySequenceIndex[739] = '''Sylimic'''
namesBySequenceIndexNegative[739] = ''''''
semitonesBySequenceIndex[739] = [0, 1, 5, 6, 7, 10]
poemBySequenceIndex[739] = '''B D K L M R'''
poemAuthors[739] = ''''''
#740: ䷨䷻ Diminishing Composed: 1 4 1 1 3 2  : S R1 M1 M2 P N2 : ElEk

#741: C D♭ F F♯ G B :0 1 5 6 7 11: 1 ♭2 4 ♯4 5 7: ma11sus♯4(♭9)
namesBySequenceIndex[740] = '''Messiaen 5th Mode,Thodimic'''
namesBySequenceIndexNegative[740] = ''''''
semitonesBySequenceIndex[740] = [0, 1, 5, 6, 7, 11]
poemBySequenceIndex[740] = '''B D K L M S'''
poemAuthors[740] = ''''''
#741: ䷨䷨ Diminishing Diminishing : 1 4 1 1 4 1  : S R1 M1 M2 P N3 : ElEl

#742: C D♭ F G♭ A♭ A :0 1 5 6 8 9: 1 ♭2 4 ♭5 ♭6 6: 6sus4(♯5 add ♭9 ♯11)
namesBySequenceIndex[741] = '''Phrocrimic'''
namesBySequenceIndexNegative[741] = ''''''
semitonesBySequenceIndex[741] = [0, 1, 5, 6, 8, 9]
poemBySequenceIndex[741] = '''B D K L N P'''
poemAuthors[741] = ''''''
#742: ䷨䷶ Diminishing Abounding: 1 4 1 2 1 3  : S R1 M1 M2 D1 N1 : ElIb

#743: C D♭ F G♭ A♭ B♭ :0 1 5 6 8 10: 1 ♭2 4 ♭5 ♭6 ♭7: 11sus♯4(♯5 ♭9)
namesBySequenceIndex[742] = '''Kycrimic'''
namesBySequenceIndexNegative[742] = ''''''
semitonesBySequenceIndex[742] = [0, 1, 5, 6, 8, 10]
poemBySequenceIndex[742] = '''B D K L N R'''
poemAuthors[742] = ''''''
#743: ䷨䷾ Diminishing Completed: 1 4 1 2 2 2  : S R1 M1 M2 D1 N2 : ElIk

#744: C D♭ F G♭ A♭ B :0 1 5 6 8 11: 1 ♭2 4 ♭5 ♭6 7: ma11sus♯4(♯5 ♭9)
namesBySequenceIndex[743] = '''Lycrimic'''
namesBySequenceIndexNegative[743] = ''''''
semitonesBySequenceIndex[743] = [0, 1, 5, 6, 8, 11]
poemBySequenceIndex[743] = '''B D K L N S'''
poemAuthors[743] = ''''''
#744: ䷨䷕ Diminishing Adorning: 1 4 1 2 3 1  : S R1 M1 M2 D1 N3 : ElIl

#745: C D♭ F G♭ A B♭ :0 1 5 6 9 10: 1 ♭2 4 ♭5 6 ♭7: 13(♭5 ♭9 no 3)
namesBySequenceIndex[744] = '''Sygimic'''
namesBySequenceIndexNegative[744] = ''''''
semitonesBySequenceIndex[744] = [0, 1, 5, 6, 9, 10]
poemBySequenceIndex[744] = '''B D K L P R'''
poemAuthors[744] = ''''''
#745: ䷨䷐ Diminishing Following: 1 4 1 3 1 2  : S R1 M1 M2 D2 N2 : ElAd

#746: C D♭ F G♭ A B :0 1 5 6 9 11: 1 ♭2 4 ♭5 6 7: ma13(♭5 ♭9 no 3)
namesBySequenceIndex[745] = '''Katynimic'''
namesBySequenceIndexNegative[745] = ''''''
semitonesBySequenceIndex[745] = [0, 1, 5, 6, 9, 11]
poemBySequenceIndex[745] = '''B D K L P S'''
poemAuthors[745] = ''''''
#746: ䷨䷔ Diminishing Biting: 1 4 1 3 2 1  : S R1 M1 M2 D2 N3 : ElAf

#747: C D♭ F G♭ A♯ B :0 1 5 6 10 11: 1 ♭2 4 ♭5 ♯6 7: ma7sus4(♭5 add ♭7 ♭9)
namesBySequenceIndex[746] = '''Zythimic'''
namesBySequenceIndexNegative[746] = ''''''
semitonesBySequenceIndex[746] = [0, 1, 5, 6, 10, 11]
poemBySequenceIndex[746] = '''B D K L R S'''
poemAuthors[746] = ''''''
#747: ䷨䷩ Diminishing Augmenting: 1 4 1 4 1 1  : S R1 M1 M2 D3 N3 : ElAm

#748: C D♭ F G A♭ A :0 1 5 7 8 9: 1 ♭2 4 5 ♭6 6: 6sus4(add ♭6 ♭9)
namesBySequenceIndex[747] = '''Raga Kalakanthi 2,Bacrimic'''
namesBySequenceIndexNegative[747] = ''''''
semitonesBySequenceIndex[747] = [0, 1, 5, 7, 8, 9]
poemBySequenceIndex[747] = '''B D K M N P'''
poemAuthors[747] = ''''''
#748: ䷨䷟ Diminishing Persevering: 1 4 2 1 1 3  : S R1 M1 P D1 N1 : Elob

#749: C D♭ F G A♭ B♭ :0 1 5 7 8 10: 1 ♭2 4 5 ♭6 ♭7: 7sus4(add ♭9 ♭13)
namesBySequenceIndex[748] = '''Insen,Niagari (Japan),Raga Phenadyuti 2,Ionarimic'''
namesBySequenceIndexNegative[748] = ''''''
semitonesBySequenceIndex[748] = [0, 1, 5, 7, 8, 10]
poemBySequenceIndex[748] = '''B D K M N R'''
poemAuthors[748] = ''''''
#749: ䷨䷯ Diminishing Source: 1 4 2 1 2 2  : S R1 M1 P D1 N2 : Elok

#750: C D♭ F G A♭ B :0 1 5 7 8 11: 1 ♭2 4 5 ♭6 7: ma7sus4(add ♭9 ♭13)
namesBySequenceIndex[749] = '''Raga Padi,Morimic'''
namesBySequenceIndexNegative[749] = ''''''
semitonesBySequenceIndex[749] = [0, 1, 5, 7, 8, 11]
poemBySequenceIndex[749] = '''B D K M N S'''
poemAuthors[749] = ''''''
#750: ䷨䷑ Diminishing Correcting: 1 4 2 1 3 1  : S R1 M1 P D1 N3 : Elol

#751: C D♭ F G A B♭ :0 1 5 7 9 10: 1 ♭2 4 5 6 ♭7: 13(♭9 no 3)
namesBySequenceIndex[750] = '''Raga Rasavali,Kaptimic'''
namesBySequenceIndexNegative[750] = ''''''
semitonesBySequenceIndex[750] = [0, 1, 5, 7, 9, 10]
poemBySequenceIndex[750] = '''B D K M P R'''
poemAuthors[750] = ''''''
#751: ䷨䷮ Diminishing Confining: 1 4 2 2 1 2  : S R1 M1 P D2 N2 : Eled

#752: C D♭ F G A B :0 1 5 7 9 11: 1 ♭2 4 5 6 7: ma13(♭9 no 3)
namesBySequenceIndex[751] = '''Raga Jivantika,Epylimic'''
namesBySequenceIndexNegative[751] = ''''''
semitonesBySequenceIndex[751] = [0, 1, 5, 7, 9, 11]
poemBySequenceIndex[751] = '''B D K M P S'''
poemAuthors[751] = ''''''
#752: ䷨䷿ Diminishing Unfinished: 1 4 2 2 2 1  : S R1 M1 P D2 N3 : Elef

#753: C D♭ F G A♯ B :0 1 5 7 10 11: 1 ♭2 4 5 ♯6 7: ma7sus4(add ♭7 ♭9)
namesBySequenceIndex[752] = '''Kokin-Joshi,Pothimic'''
namesBySequenceIndexNegative[752] = ''''''
semitonesBySequenceIndex[752] = [0, 1, 5, 7, 10, 11]
poemBySequenceIndex[752] = '''B D K M R S'''
poemAuthors[752] = ''''''
#753: ䷨䷺ Diminishing Dispersing: 1 4 2 3 1 1  : S R1 M1 P D3 N3 : Elem

#754: C D♭ F G♯ A B♭ :0 1 5 8 9 10: 1 ♭2 4 ♯5 6 ♭7: 13(♯5 ♭9 no 3)
namesBySequenceIndex[753] = '''Katygimic'''
namesBySequenceIndexNegative[753] = ''''''
semitonesBySequenceIndex[753] = [0, 1, 5, 8, 9, 10]
poemBySequenceIndex[753] = '''B D K N P R'''
poemAuthors[753] = ''''''
#754: ䷨䷞ Diminishing Conjoining: 1 4 3 1 1 2  : S R1 M1 D1 D2 N2 : Elid

#755: C D♭ F G♯ A B :0 1 5 8 9 11: 1 ♭2 4 ♯5 6 7: ma13(♯5 ♭9 no 3)
namesBySequenceIndex[754] = '''Katoptimic'''
namesBySequenceIndexNegative[754] = ''''''
semitonesBySequenceIndex[754] = [0, 1, 5, 8, 9, 11]
poemBySequenceIndex[754] = '''B D K N P S'''
poemAuthors[754] = ''''''
#755: ䷨䷷ Diminishing Wanderer: 1 4 3 1 2 1  : S R1 M1 D1 D2 N3 : Elif

#756: C D♭ F G♯ A♯ B :0 1 5 8 10 11: 1 ♭2 4 ♯5 ♯6 7: ma7sus4(♯5 add ♭7 ♭9)
namesBySequenceIndex[755] = '''Rogimic'''
namesBySequenceIndexNegative[755] = ''''''
semitonesBySequenceIndex[755] = [0, 1, 5, 8, 10, 11]
poemBySequenceIndex[755] = '''B D K N R S'''
poemAuthors[755] = ''''''
#756: ䷨䷴ Diminishing Infiltrating: 1 4 3 2 1 1  : S R1 M1 D1 N2 N3 : Elim

#757: C D♭ F A B♭ B :0 1 5 9 10 11: 1 ♭2 4 6 ♭7 7: ma13(♭9 no 3 5 add ♭7)
namesBySequenceIndex[756] = '''Thanimic'''
namesBySequenceIndexNegative[756] = ''''''
semitonesBySequenceIndex[756] = [0, 1, 5, 9, 10, 11]
poemBySequenceIndex[756] = '''B D K P R S'''
poemAuthors[756] = ''''''
#757: ䷨䷋ Diminishing Obstruction: 1 4 4 1 1 1  : S R1 M1 D2 N2 N3 : Elag

#758: C D♭ F♯ G A♭ A :0 1 6 7 8 9: 1 ♭2 ♯4 5 ♭6 6: 6sus♯4(add ♭6 ♭9)
namesBySequenceIndex[757] = ''''''
namesBySequenceIndexNegative[757] = ''''''
semitonesBySequenceIndex[757] = [0, 1, 6, 7, 8, 9]
poemBySequenceIndex[757] = '''B D L M N P'''
poemAuthors[757] = ''''''
#758: ䷒䷡ Arrived Might: 1 5 1 1 1 3  : S R1 M2 P D1 N1 : EjOb

#759: C D♭ F♯ G A♭ B♭ :0 1 6 7 8 10: 1 ♭2 ♯4 5 ♭6 ♭7: 7sus♯4(add ♭9 ♭13)
namesBySequenceIndex[758] = ''''''
namesBySequenceIndexNegative[758] = ''''''
semitonesBySequenceIndex[758] = [0, 1, 6, 7, 8, 10]
poemBySequenceIndex[758] = '''B D L M N R'''
poemAuthors[758] = ''''''
#759: ䷒䷄ Arrived Biding: 1 5 1 1 2 2  : S R1 M2 P D1 N2 : EjOk

#760: C D♭ F♯ G A♭ B :0 1 6 7 8 11: 1 ♭2 ♯4 5 ♭6 7: ma7sus♯4(add ♭9 ♭13)
namesBySequenceIndex[759] = ''''''
namesBySequenceIndexNegative[759] = ''''''
semitonesBySequenceIndex[759] = [0, 1, 6, 7, 8, 11]
poemBySequenceIndex[759] = '''B D L M N S'''
poemAuthors[759] = ''''''
#760: ䷒䷙ Arrived Cultivating: 1 5 1 1 3 1  : S R1 M2 P D1 N3 : EjOl

#761: C D♭ F♯ G A B♭ :0 1 6 7 9 10: 1 ♭2 ♯4 5 6 ♭7: 13(♭9 ♯11 no 3)
namesBySequenceIndex[760] = ''''''
namesBySequenceIndexNegative[760] = ''''''
semitonesBySequenceIndex[760] = [0, 1, 6, 7, 9, 10]
poemBySequenceIndex[760] = '''B D L M P R'''
poemAuthors[760] = ''''''
#761: ䷒䷹ Arrived Joyous: 1 5 1 2 1 2  : S R1 M2 P D2 N2 : EjEd

#762: C D♭ F♯ G A B :0 1 6 7 9 11: 1 ♭2 ♯4 5 6 7: ma13(♭9 ♯11 no 3)
namesBySequenceIndex[761] = ''''''
namesBySequenceIndexNegative[761] = ''''''
semitonesBySequenceIndex[761] = [0, 1, 6, 7, 9, 11]
poemBySequenceIndex[761] = '''B D L M P S'''
poemAuthors[761] = ''''''
#762: ䷒䷥ Arrived Polarising: 1 5 1 2 2 1  : S R1 M2 P D2 N3 : EjEf

#763: C D♭ F♯ G A♯ B :0 1 6 7 10 11: 1 ♭2 ♯4 5 ♯6 7: ma7sus♯4(add ♭7 ♭9)
namesBySequenceIndex[762] = ''''''
namesBySequenceIndexNegative[762] = ''''''
semitonesBySequenceIndex[762] = [0, 1, 6, 7, 10, 11]
poemBySequenceIndex[762] = '''B D L M R S'''
poemAuthors[762] = ''''''
#763: ䷒䷼ Arrived Truth: 1 5 1 3 1 1  : S R1 M2 P D3 N3 : EjEm

#764: C D♭ F♯ G♯ A B♭ :0 1 6 8 9 10: 1 ♭2 ♯4 ♯5 6 ♭7: 13(♯5 ♭9 ♯11 no 3)
namesBySequenceIndex[763] = ''''''
namesBySequenceIndexNegative[763] = ''''''
semitonesBySequenceIndex[763] = [0, 1, 6, 8, 9, 10]
poemBySequenceIndex[763] = '''B D L N P R'''
poemAuthors[763] = ''''''
#764: ䷒䷰ Arrived Revolution: 1 5 2 1 1 2  : S R1 M2 D1 D2 N2 : EjId

#765: C D♭ F♯ G♯ A B :0 1 6 8 9 11: 1 ♭2 ♯4 ♯5 6 7: ma13(♯5 ♭9 ♯11 no 3)
namesBySequenceIndex[764] = ''''''
namesBySequenceIndexNegative[764] = ''''''
semitonesBySequenceIndex[764] = [0, 1, 6, 8, 9, 11]
poemBySequenceIndex[764] = '''B D L N P S'''
poemAuthors[764] = ''''''
#765: ䷒䷝ Arrived Distanced: 1 5 2 1 2 1  : S R1 M2 D1 D2 N3 : EjIf

#766: C D♭ F♯ G♯ A♯ B :0 1 6 8 10 11: 1 ♭2 ♯4 ♯5 ♯6 7: ma7sus♯4(♯5 add ♭7 ♭9)
namesBySequenceIndex[765] = ''''''
namesBySequenceIndexNegative[765] = ''''''
semitonesBySequenceIndex[765] = [0, 1, 6, 8, 10, 11]
poemBySequenceIndex[765] = '''B D L N R S'''
poemAuthors[765] = ''''''
#766: ䷒䷤ Arrived Family: 1 5 2 2 1 1  : S R1 M2 D1 N2 N3 : EjIm

#767: C D♭ G♭ A B♭ B :0 1 6 9 10 11: 1 ♭2 ♭5 6 ♭7 7: ma7sus♭2(♭5 add ♭7 13)
namesBySequenceIndex[766] = ''''''
namesBySequenceIndexNegative[766] = ''''''
semitonesBySequenceIndex[766] = [0, 1, 6, 9, 10, 11]
poemBySequenceIndex[766] = '''B D L P R S'''
poemAuthors[766] = ''''''
#767: ䷒䷘ Arrived Integrity: 1 5 3 1 1 1  : S R1 M2 D2 N2 N3 : EjAg

#768: C D♭ G A♭ A B♭ :0 1 7 8 9 10: 1 ♭2 5 ♭6 6 ♭7: 7sus♭2(add ♭13 13)
namesBySequenceIndex[767] = ''''''
namesBySequenceIndexNegative[767] = ''''''
semitonesBySequenceIndex[767] = [0, 1, 7, 8, 9, 10]
poemBySequenceIndex[767] = '''B D M N P R'''
poemAuthors[767] = ''''''
#768: ䷒䷛ Arrived Excess: 1 6 1 1 1 2  : S R1 P D1 D2 N2 : Ejod

#769: C D♭ G A♭ A B :0 1 7 8 9 11: 1 ♭2 5 ♭6 6 7: ma7sus♭2(add ♭13 13)
namesBySequenceIndex[768] = ''''''
namesBySequenceIndexNegative[768] = ''''''
semitonesBySequenceIndex[768] = [0, 1, 7, 8, 9, 11]
poemBySequenceIndex[768] = '''B D M N P S'''
poemAuthors[768] = ''''''
#769: ䷒䷱ Arrived Holding: 1 6 1 1 2 1  : S R1 P D1 D2 N3 : Ejof

#770: C D♭ G A♭ B♭ B :0 1 7 8 10 11: 1 ♭2 5 ♭6 ♭7 7: ma7sus♭2(add ♭7 ♭13)
namesBySequenceIndex[769] = ''''''
namesBySequenceIndexNegative[769] = ''''''
semitonesBySequenceIndex[769] = [0, 1, 7, 8, 10, 11]
poemBySequenceIndex[769] = '''B D M N R S'''
poemAuthors[769] = ''''''
#770: ䷒䷸ Arrived Osmosis: 1 6 1 2 1 1  : S R1 P D1 N2 N3 : Ejom

#771: C D♭ G A B♭ B :0 1 7 9 10 11: 1 ♭2 5 6 ♭7 7: ma7sus♭2(add ♭7 13)
namesBySequenceIndex[770] = ''''''
namesBySequenceIndexNegative[770] = ''''''
semitonesBySequenceIndex[770] = [0, 1, 7, 9, 10, 11]
poemBySequenceIndex[770] = '''B D M P R S'''
poemAuthors[770] = ''''''
#771: ䷒䷅ Arrived Arguing: 1 6 2 1 1 1  : S R1 P D2 N2 N3 : Ejeg

#772: C D♭ G♯ A B♭ B :0 1 8 9 10 11: 1 ♭2 ♯5 6 ♭7 7: ma7sus♭2(♯5 add ♭7 13)
namesBySequenceIndex[771] = ''''''
namesBySequenceIndexNegative[771] = ''''''
semitonesBySequenceIndex[771] = [0, 1, 8, 9, 10, 11]
poemBySequenceIndex[771] = '''B D N P R S'''
poemAuthors[771] = ''''''
#772: ䷒䷠ Arrived Retiring: 1 7 1 1 1 1  : S R1 D1 D2 N2 N3 : Ejig

#773: C D D♯ E F G♭ :0 2 3 4 5 6: 1 2 ♯2 3 4 ♭5: 11(♭5 no 7 add ♯9)
namesBySequenceIndex[772] = ''''''
namesBySequenceIndexNegative[772] = ''''''
semitonesBySequenceIndex[772] = [0, 2, 3, 4, 5, 6]
poemBySequenceIndex[772] = '''B F G J K L'''
poemAuthors[772] = ''''''
#773: ䷌䷗ Fellowship Returning: 2 1 1 1 1 6  : S R2 R3 G3 M1 M2 : IgAj

#774: C D D♯ E F G :0 2 3 4 5 7: 1 2 ♯2 3 4 5: 11(no 7 add ♯9)
namesBySequenceIndex[773] = ''''''
namesBySequenceIndexNegative[773] = ''''''
semitonesBySequenceIndex[773] = [0, 2, 3, 4, 5, 7]
poemBySequenceIndex[773] = '''B F G J K M'''
poemAuthors[773] = ''''''
#774: ䷌䷆ Fellowship Leading: 2 1 1 1 2 5  : S R2 R3 G3 M1 P : Igej

#775: C D D♯ E F G♯ :0 2 3 4 5 8: 1 2 ♯2 3 4 ♯5: 11(♯5 no 7 add ♯9)
namesBySequenceIndex[774] = '''Korimic'''
namesBySequenceIndexNegative[774] = ''''''
semitonesBySequenceIndex[774] = [0, 2, 3, 4, 5, 8]
poemBySequenceIndex[774] = '''B F G J K N'''
poemAuthors[774] = ''''''
#775: ䷌䷎ Fellowship Humble: 2 1 1 1 3 4  : S R2 R3 G3 M1 D1 : Igij

#776: C D D♯ E F A :0 2 3 4 5 9: 1 2 ♯2 3 4 6: 13(no 5 7 add ♯9)
namesBySequenceIndex[775] = '''Saptimic'''
namesBySequenceIndexNegative[775] = ''''''
semitonesBySequenceIndex[775] = [0, 2, 3, 4, 5, 9]
poemBySequenceIndex[775] = '''B F G J K P'''
poemAuthors[775] = ''''''
#776: ䷌䷏ Fellowship Confident: 2 1 1 1 4 3  : S R2 R3 G3 M1 D2 : Igab

#777: C D D♯ E F B♭ :0 2 3 4 5 10: 1 2 ♯2 3 4 ♭7: 11(no 5 add ♯9)
namesBySequenceIndex[776] = ''''''
namesBySequenceIndexNegative[776] = ''''''
semitonesBySequenceIndex[776] = [0, 2, 3, 4, 5, 10]
poemBySequenceIndex[776] = '''B F G J K R'''
poemAuthors[776] = ''''''
#777: ䷌䷇ Fellowship Comparing: 2 1 1 1 5 2  : S R2 R3 G3 M1 N2 : Igak

#778: C D D♯ E F B :0 2 3 4 5 11: 1 2 ♯2 3 4 7: ma11(no 5 add ♯9)
namesBySequenceIndex[777] = ''''''
namesBySequenceIndexNegative[777] = ''''''
semitonesBySequenceIndex[777] = [0, 2, 3, 4, 5, 11]
poemBySequenceIndex[777] = '''B F G J K S'''
poemAuthors[777] = ''''''
#778: ䷌䷖ Fellowship Stripping: 2 1 1 1 6 1  : S R2 R3 G3 M1 N3 : Igal

#779: C D D♯ E F♯ G :0 2 3 4 6 7: 1 2 ♯2 3 ♯4 5: ma(add 9 ♯9 ♯11)
namesBySequenceIndex[778] = ''''''
namesBySequenceIndexNegative[778] = ''''''
semitonesBySequenceIndex[778] = [0, 2, 3, 4, 6, 7]
poemBySequenceIndex[778] = '''B F G J L M'''
poemAuthors[778] = ''''''
#779: ䷰䷒ Revolution Arrived: 2 1 1 2 1 5  : S R2 R3 G3 M2 P : IdEj

#780: C D D♯ E F♯ G♯ :0 2 3 4 6 8: 1 2 ♯2 3 ♯4 ♯5: ma(♯5 add 9 ♯9 ♯11)
namesBySequenceIndex[779] = '''Borimic'''
namesBySequenceIndexNegative[779] = ''''''
semitonesBySequenceIndex[779] = [0, 2, 3, 4, 6, 8]
poemBySequenceIndex[779] = '''B F G J L N'''
poemAuthors[779] = ''''''
#780: ䷰䷣ Revolution Eclipse: 2 1 1 2 2 4  : S R2 R3 G3 M2 D1 : IdIj

#781: C D D♯ E G♭ A :0 2 3 4 6 9: 1 2 ♯2 3 ♭5 6: 6/9(♭5 add ♯9)
namesBySequenceIndex[780] = '''Dycrimic'''
namesBySequenceIndexNegative[780] = ''''''
semitonesBySequenceIndex[780] = [0, 2, 3, 4, 6, 9]
poemBySequenceIndex[780] = '''B F G J L P'''
poemAuthors[780] = ''''''
#781: ䷰䷲ Revolution Thunder: 2 1 1 2 3 3  : S R2 R3 G3 M2 D2 : IdAb

#782: C D D♯ E G♭ B♭ :0 2 3 4 6 10: 1 2 ♯2 3 ♭5 ♭7: 9(♭5 add ♯9)
namesBySequenceIndex[781] = '''Raptimic'''
namesBySequenceIndexNegative[781] = ''''''
semitonesBySequenceIndex[781] = [0, 2, 3, 4, 6, 10]
poemBySequenceIndex[781] = '''B F G J L R'''
poemAuthors[781] = ''''''
#782: ䷰䷂ Revolution Sprouting: 2 1 1 2 4 2  : S R2 R3 G3 M2 N2 : IdAk

#783: C D D♯ E G♭ B :0 2 3 4 6 11: 1 2 ♯2 3 ♭5 7: ma9(♭5 add ♯9)
namesBySequenceIndex[782] = ''''''
namesBySequenceIndexNegative[782] = ''''''
semitonesBySequenceIndex[782] = [0, 2, 3, 4, 6, 11]
poemBySequenceIndex[782] = '''B F G J L S'''
poemAuthors[782] = ''''''
#783: ䷰䷚ Revolution Nourishment: 2 1 1 2 5 1  : S R2 R3 G3 M2 N3 : IdAl

#784: C D D♯ E G A♭ :0 2 3 4 7 8: 1 2 ♯2 3 5 ♭6: ma(add ♭6 9 ♯9)
namesBySequenceIndex[783] = '''Ganimic'''
namesBySequenceIndexNegative[783] = ''''''
semitonesBySequenceIndex[783] = [0, 2, 3, 4, 7, 8]
poemBySequenceIndex[783] = '''B F G J M N'''
poemAuthors[783] = ''''''
#784: ䷰䷭ Revolution Ascending: 2 1 1 3 1 4  : S R2 R3 G3 P D1 : Idoj

#785: C D D♯ E G A :0 2 3 4 7 9: 1 2 ♯2 3 5 6: 6/9(add ♯9)
namesBySequenceIndex[784] = '''Gycrimic'''
namesBySequenceIndexNegative[784] = ''''''
semitonesBySequenceIndex[784] = [0, 2, 3, 4, 7, 9]
poemBySequenceIndex[784] = '''B F G J M P'''
poemAuthors[784] = ''''''
#785: ䷰䷧ Revolution Deliverance: 2 1 1 3 2 3  : S R2 R3 G3 P D2 : Ideb

#786: C D D♯ E G B♭ :0 2 3 4 7 10: 1 2 ♯2 3 5 ♭7: 9(add ♯9)
namesBySequenceIndex[785] = '''Katagimic'''
namesBySequenceIndexNegative[785] = ''''''
semitonesBySequenceIndex[785] = [0, 2, 3, 4, 7, 10]
poemBySequenceIndex[785] = '''B F G J M R'''
poemAuthors[785] = ''''''
#786: ䷰䷜ Revolution Chasm: 2 1 1 3 3 2  : S R2 R3 G3 P N2 : Idek

#787: C D D♯ E G B :0 2 3 4 7 11: 1 2 ♯2 3 5 7: ma9(add ♯9)
namesBySequenceIndex[786] = '''Ionocrimic'''
namesBySequenceIndexNegative[786] = ''''''
semitonesBySequenceIndex[786] = [0, 2, 3, 4, 7, 11]
poemBySequenceIndex[786] = '''B F G J M S'''
poemAuthors[786] = ''''''
#787: ䷰䷃ Revolution Naïve: 2 1 1 3 4 1  : S R2 R3 G3 P N3 : Idel

#788: C D D♯ E G♯ A :0 2 3 4 8 9: 1 2 ♯2 3 ♯5 6: 6/9(♯5 add ♯9)
namesBySequenceIndex[787] = '''Katocrimic'''
namesBySequenceIndexNegative[787] = ''''''
semitonesBySequenceIndex[787] = [0, 2, 3, 4, 8, 9]
poemBySequenceIndex[787] = '''B F G J N P'''
poemAuthors[787] = ''''''
#788: ䷰䷽ Revolution Detail: 2 1 1 4 1 3  : S R2 R3 G3 D1 N1 : Idib

#789: C D D♯ E G♯ B♭ :0 2 3 4 8 10: 1 2 ♯2 3 ♯5 ♭7: 9(♯5 add ♯9)
namesBySequenceIndex[788] = '''Pogimic'''
namesBySequenceIndexNegative[788] = ''''''
semitonesBySequenceIndex[788] = [0, 2, 3, 4, 8, 10]
poemBySequenceIndex[788] = '''B F G J N R'''
poemAuthors[788] = ''''''
#789: ䷰䷦ Revolution Limping: 2 1 1 4 2 2  : S R2 R3 G3 D1 N2 : Idik

#790: C D D♯ E G♯ B :0 2 3 4 8 11: 1 2 ♯2 3 ♯5 7: ma9(♯5 add ♯9)
namesBySequenceIndex[789] = '''Stynimic'''
namesBySequenceIndexNegative[789] = ''''''
semitonesBySequenceIndex[789] = [0, 2, 3, 4, 8, 11]
poemBySequenceIndex[789] = '''B F G J N S'''
poemAuthors[789] = ''''''
#790: ䷰䷳ Revolution Stillness: 2 1 1 4 3 1  : S R2 R3 G3 D1 N3 : Idil

#791: C D D♯ E A B♭ :0 2 3 4 9 10: 1 2 ♯2 3 6 ♭7: 9(no 5 add ♯9 13)
namesBySequenceIndex[790] = ''''''
namesBySequenceIndexNegative[790] = ''''''
semitonesBySequenceIndex[790] = [0, 2, 3, 4, 9, 10]
poemBySequenceIndex[790] = '''B F G J P R'''
poemAuthors[790] = ''''''
#791: ䷰䷬ Revolution Clustering: 2 1 1 5 1 2  : S R2 R3 G3 D2 N2 : Idad

#792: C D D♯ E A B :0 2 3 4 9 11: 1 2 ♯2 3 6 7: ma9(no 5 add ♯9 13)
namesBySequenceIndex[791] = ''''''
namesBySequenceIndexNegative[791] = ''''''
semitonesBySequenceIndex[791] = [0, 2, 3, 4, 9, 11]
poemBySequenceIndex[791] = '''B F G J P S'''
poemAuthors[791] = ''''''
#792: ䷰䷢ Revolution Prospering: 2 1 1 5 2 1  : S R2 R3 G3 D2 N3 : Idaf

#793: C D D♯ E A♯ B :0 2 3 4 10 11: 1 2 ♯2 3 ♯6 7: ma9(no 5 add ♭7 ♯9)
namesBySequenceIndex[792] = ''''''
namesBySequenceIndexNegative[792] = ''''''
semitonesBySequenceIndex[792] = [0, 2, 3, 4, 10, 11]
poemBySequenceIndex[792] = '''B F G J R S'''
poemAuthors[792] = ''''''
#793: ䷰䷓ Revolution Seeing: 2 1 1 6 1 1  : S R2 R3 G3 D3 N3 : Idam

#794: C D E♭ F F♯ G :0 2 3 5 6 7: 1 2 ♭3 4 ♯4 5: mi11(no 7 add ♯11)
namesBySequenceIndex[793] = ''''''
namesBySequenceIndexNegative[793] = ''''''
semitonesBySequenceIndex[793] = [0, 2, 3, 5, 6, 7]
poemBySequenceIndex[793] = '''B F G K L M'''
poemAuthors[793] = ''''''
#794: ䷝䷒ Distanced Arrived: 2 1 2 1 1 5  : S R2 G2 M1 M2 P : IfEj

#795: C D E♭ F G♭ A♭ :0 2 3 5 6 8: 1 2 ♭3 4 ♭5 ♭6: °11(no 7 add ♭6)
namesBySequenceIndex[794] = '''Marimic'''
namesBySequenceIndexNegative[794] = ''''''
semitonesBySequenceIndex[794] = [0, 2, 3, 5, 6, 8]
poemBySequenceIndex[794] = '''B F G K L N'''
poemAuthors[794] = ''''''
#795: ䷝䷣ Distanced Eclipse: 2 1 2 1 2 4  : S R2 G2 M1 M2 D1 : IfIj

#796: C D E♭ F G♭ A :0 2 3 5 6 9: 1 2 ♭3 4 ♭5 6: °13(no 7)
namesBySequenceIndex[795] = '''Pyramid Sexatonic,Kyrimic'''
namesBySequenceIndexNegative[795] = ''''''
semitonesBySequenceIndex[795] = [0, 2, 3, 5, 6, 9]
poemBySequenceIndex[795] = '''B F G K L P'''
poemAuthors[795] = ''''''
#796: ䷝䷲ Distanced Thunder: 2 1 2 1 3 3  : S R2 G2 M1 M2 D2 : IfAb

#797: C D E♭ F G♭ B♭ :0 2 3 5 6 10: 1 2 ♭3 4 ♭5 ♭7: ⌀11
namesBySequenceIndex[796] = '''Stycrimic'''
namesBySequenceIndexNegative[796] = ''''''
semitonesBySequenceIndex[796] = [0, 2, 3, 5, 6, 10]
poemBySequenceIndex[796] = '''B F G K L R'''
poemAuthors[796] = ''''''
#797: ䷝䷂ Distanced Sprouting: 2 1 2 1 4 2  : S R2 G2 M1 M2 N2 : IfAk

#798: C D E♭ F G♭ B :0 2 3 5 6 11: 1 2 ♭3 4 ♭5 7: °ma11
namesBySequenceIndex[797] = ''''''
namesBySequenceIndexNegative[797] = ''''''
semitonesBySequenceIndex[797] = [0, 2, 3, 5, 6, 11]
poemBySequenceIndex[797] = '''B F G K L S'''
poemAuthors[797] = ''''''
#798: ䷝䷚ Distanced Nourishment: 2 1 2 1 5 1  : S R2 G2 M1 M2 N3 : IfAl

#799: C D E♭ F G A♭ :0 2 3 5 7 8: 1 2 ♭3 4 5 ♭6: mi11(no 7 add ♭6)
namesBySequenceIndex[798] = '''Koptimic'''
namesBySequenceIndexNegative[798] = ''''''
semitonesBySequenceIndex[798] = [0, 2, 3, 5, 7, 8]
poemBySequenceIndex[798] = '''B F G K M N'''
poemAuthors[798] = ''''''
#799: ䷝䷭ Distanced Ascending: 2 1 2 2 1 4  : S R2 G2 M1 P D1 : Ifoj

#800: C D E♭ F G A :0 2 3 5 7 9: 1 2 ♭3 4 5 6: mi13(no 7)
namesBySequenceIndex[799] = '''Sho 1,Raga Gauri Velavali,Raga Suddha Bangala,Aerathimic'''
namesBySequenceIndexNegative[799] = ''''''
semitonesBySequenceIndex[799] = [0, 2, 3, 5, 7, 9]
poemBySequenceIndex[799] = '''B F G K M P'''
poemAuthors[799] = ''''''
#800: ䷝䷧ Distanced Deliverance: 2 1 2 2 2 3  : S R2 G2 M1 P D2 : Ifeb

#801: C D E♭ F G B♭ :0 2 3 5 7 10: 1 2 ♭3 4 5 ♭7: mi11
namesBySequenceIndex[800] = '''Raga Palasi,Raga Adana (ascending),Minor Sexatonic,Raga Manirangu,Raga Nayaki,Raga Pushpalithika,Raga Suha Sughrai,Rocrimic'''
namesBySequenceIndexNegative[800] = ''''''
semitonesBySequenceIndex[800] = [0, 2, 3, 5, 7, 10]
poemBySequenceIndex[800] = '''B F G K M R'''
poemAuthors[800] = ''''''
#801: ䷝䷜ Distanced Chasm: 2 1 2 2 3 2  : S R2 G2 M1 P N2 : Ifek

#802: C D E♭ F G B :0 2 3 5 7 11: 1 2 ♭3 4 5 7: mima11
namesBySequenceIndex[801] = '''Raga Sindhura Kafi,Poptimic'''
namesBySequenceIndexNegative[801] = ''''''
semitonesBySequenceIndex[801] = [0, 2, 3, 5, 7, 11]
poemBySequenceIndex[801] = '''B F G K M S'''
poemAuthors[801] = ''''''
#802: ䷝䷃ Distanced Naïve: 2 1 2 2 4 1  : S R2 G2 M1 P N3 : Ifel

#803: C D E♭ F G♯ A :0 2 3 5 8 9: 1 2 ♭3 4 ♯5 6: mi13(♯5 no 7)
namesBySequenceIndex[802] = '''Larimic'''
namesBySequenceIndexNegative[802] = ''''''
semitonesBySequenceIndex[802] = [0, 2, 3, 5, 8, 9]
poemBySequenceIndex[802] = '''B F G K N P'''
poemAuthors[802] = ''''''
#803: ䷝䷽ Distanced Detail: 2 1 2 3 1 3  : S R2 G2 M1 D1 N1 : Ifib

#804: C D E♭ F G♯ B♭ :0 2 3 5 8 10: 1 2 ♭3 4 ♯5 ♭7: mi11(♯5)
namesBySequenceIndex[803] = '''Phradimic'''
namesBySequenceIndexNegative[803] = ''''''
semitonesBySequenceIndex[803] = [0, 2, 3, 5, 8, 10]
poemBySequenceIndex[803] = '''B F G K N R'''
poemAuthors[803] = ''''''
#804: ䷝䷦ Distanced Limping: 2 1 2 3 2 2  : S R2 G2 M1 D1 N2 : Ifik

#805: C D E♭ F G♯ B :0 2 3 5 8 11: 1 2 ♭3 4 ♯5 7: mima11(♯5)
namesBySequenceIndex[804] = '''Raga Ghantana,Raga Kaishikiranjani,Raga Kaushiranjani,Aerogimic'''
namesBySequenceIndexNegative[804] = ''''''
semitonesBySequenceIndex[804] = [0, 2, 3, 5, 8, 11]
poemBySequenceIndex[804] = '''B F G K N S'''
poemAuthors[804] = ''''''
#805: ䷝䷳ Distanced Stillness: 2 1 2 3 3 1  : S R2 G2 M1 D1 N3 : Ifil

#806: C D E♭ F A B♭ :0 2 3 5 9 10: 1 2 ♭3 4 6 ♭7: mi13(no 5)
namesBySequenceIndex[805] = '''Raga Shreeranjani,Raga Shee Ranjani or Sriranjiani,Raga Kapijingla,Raga Bageshri 2,Raga Jayamanohari,Gyrimic'''
namesBySequenceIndexNegative[805] = ''''''
semitonesBySequenceIndex[805] = [0, 2, 3, 5, 9, 10]
poemBySequenceIndex[805] = '''B F G K P R'''
poemAuthors[805] = ''''''
#806: ䷝䷬ Distanced Clustering: 2 1 2 4 1 2  : S R2 G2 M1 D2 N2 : Ifad

#807: C D E♭ F A B :0 2 3 5 9 11: 1 2 ♭3 4 6 7: mima13(no 5)
namesBySequenceIndex[806] = '''Rylimic'''
namesBySequenceIndexNegative[806] = ''''''
semitonesBySequenceIndex[806] = [0, 2, 3, 5, 9, 11]
poemBySequenceIndex[806] = '''B F G K P S'''
poemAuthors[806] = ''''''
#807: ䷝䷢ Distanced Prospering: 2 1 2 4 2 1  : S R2 G2 M1 D2 N3 : Ifaf

#808: C D E♭ F A♯ B :0 2 3 5 10 11: 1 2 ♭3 4 ♯6 7: mima11(no 5 add ♭7)
namesBySequenceIndex[807] = ''''''
namesBySequenceIndexNegative[807] = ''''''
semitonesBySequenceIndex[807] = [0, 2, 3, 5, 10, 11]
poemBySequenceIndex[807] = '''B F G K R S'''
poemAuthors[807] = ''''''
#808: ䷝䷓ Distanced Seeing: 2 1 2 5 1 1  : S R2 G2 M1 D3 N3 : Ifam

#809: C D E♭ F♯ G A♭ :0 2 3 6 7 8: 1 2 ♭3 ♯4 5 ♭6: mi(add ♭6 9 ♯11)
namesBySequenceIndex[808] = '''Raga Syamalam,Madimic'''
namesBySequenceIndexNegative[808] = ''''''
semitonesBySequenceIndex[808] = [0, 2, 3, 6, 7, 8]
poemBySequenceIndex[808] = '''B F G L M N'''
poemAuthors[808] = ''''''
#809: ䷶䷊ Abounding Peace: 2 1 3 1 1 4  : S R2 G2 M2 P D1 : IbOj

#810: C D E♭ F♯ G A :0 2 3 6 7 9: 1 2 ♭3 ♯4 5 6: mi6/9(add ♯11)
namesBySequenceIndex[809] = '''Raga Vijayanagari,Bridge Chord,Gythimic'''
namesBySequenceIndexNegative[809] = ''''''
semitonesBySequenceIndex[809] = [0, 2, 3, 6, 7, 9]
poemBySequenceIndex[809] = '''B F G L M P'''
poemAuthors[809] = ''''''
#810: ䷶䷵ Abounding Betrothing: 2 1 3 1 2 3  : S R2 G2 M2 P D2 : IbEb

#811: C D E♭ F♯ G B♭ :0 2 3 6 7 10: 1 2 ♭3 ♯4 5 ♭7: mi9(add ♯11)
namesBySequenceIndex[810] = '''Raga Simharava,Raga Gopikatilaka,Ragimic'''
namesBySequenceIndexNegative[810] = ''''''
semitonesBySequenceIndex[810] = [0, 2, 3, 6, 7, 10]
poemBySequenceIndex[810] = '''B F G L M R'''
poemAuthors[810] = ''''''
#811: ䷶䷻ Abounding Composed: 2 1 3 1 3 2  : S R2 G2 M2 P N2 : IbEk

#812: C D E♭ F♯ G B :0 2 3 6 7 11: 1 2 ♭3 ♯4 5 7: mima9(add ♯11)
namesBySequenceIndex[811] = '''Raga Amarasenapriya,Logimic'''
namesBySequenceIndexNegative[811] = ''''''
semitonesBySequenceIndex[811] = [0, 2, 3, 6, 7, 11]
poemBySequenceIndex[811] = '''B F G L M S'''
poemAuthors[811] = ''''''
#812: ䷶䷨ Abounding Diminishing: 2 1 3 1 4 1  : S R2 G2 M2 P N3 : IbEl

#813: C D E♭ F♯ G♯ A :0 2 3 6 8 9: 1 2 ♭3 ♯4 ♯5 6: °7(add ♭6 9)
namesBySequenceIndex[812] = '''Raga Neelangi,Zyrimic'''
namesBySequenceIndexNegative[812] = ''''''
semitonesBySequenceIndex[812] = [0, 2, 3, 6, 8, 9]
poemBySequenceIndex[812] = '''B F G L N P'''
poemAuthors[812] = ''''''
#813: ䷶䷶ Abounding Abounding : 2 1 3 2 1 3  : S R2 G2 M2 D1 N1 : IbIb

#814: C D E♭ F♯ G♯ B♭ :0 2 3 6 8 10: 1 2 ♭3 ♯4 ♯5 ♭7: ⌀9(add ♭13)
namesBySequenceIndex[813] = '''Takemitzu Tree 2,Katonimic'''
namesBySequenceIndexNegative[813] = ''''''
semitonesBySequenceIndex[813] = [0, 2, 3, 6, 8, 10]
poemBySequenceIndex[813] = '''B F G L N R'''
poemAuthors[813] = ''''''
#814: ䷶䷾ Abounding Completed: 2 1 3 2 2 2  : S R2 G2 M2 D1 N2 : IbIk

#815: C D E♭ F♯ G♯ B :0 2 3 6 8 11: 1 2 ♭3 ♯4 ♯5 7: °ma9(add ♭13)
namesBySequenceIndex[814] = '''Takemitzu Tree 1,Takemitzu Tree (Japan),Sorimic'''
namesBySequenceIndexNegative[814] = ''''''
semitonesBySequenceIndex[814] = [0, 2, 3, 6, 8, 11]
poemBySequenceIndex[814] = '''B F G L N S'''
poemAuthors[814] = ''''''
#815: ䷶䷕ Abounding Adorning: 2 1 3 2 3 1  : S R2 G2 M2 D1 N3 : IbIl

#816: C D E♭ G♭ A B♭ :0 2 3 6 9 10: 1 2 ♭3 ♭5 6 ♭7: ⌀9(add 13)
namesBySequenceIndex[815] = '''Raga Bageshri (descending),Thylimic'''
namesBySequenceIndexNegative[815] = ''''''
semitonesBySequenceIndex[815] = [0, 2, 3, 6, 9, 10]
poemBySequenceIndex[815] = '''B F G L P R'''
poemAuthors[815] = ''''''
#816: ䷶䷐ Abounding Following: 2 1 3 3 1 2  : S R2 G2 M2 D2 N2 : IbAd

#817: C D E♭ G♭ A B :0 2 3 6 9 11: 1 2 ♭3 ♭5 6 7: °ma9(add 13)
namesBySequenceIndex[816] = '''Raga Ranjani,Raga Rangini,Aeolonimic'''
namesBySequenceIndexNegative[816] = ''''''
semitonesBySequenceIndex[816] = [0, 2, 3, 6, 9, 11]
poemBySequenceIndex[816] = '''B F G L P S'''
poemAuthors[816] = ''''''
#817: ䷶䷔ Abounding Biting: 2 1 3 3 2 1  : S R2 G2 M2 D2 N3 : IbAf

#818: C D E♭ G♭ A♯ B :0 2 3 6 10 11: 1 2 ♭3 ♭5 ♯6 7: °ma9(add ♭7)
namesBySequenceIndex[817] = '''Phrycrimic'''
namesBySequenceIndexNegative[817] = ''''''
semitonesBySequenceIndex[817] = [0, 2, 3, 6, 10, 11]
poemBySequenceIndex[817] = '''B F G L R S'''
poemAuthors[817] = ''''''
#818: ䷶䷩ Abounding Augmenting: 2 1 3 4 1 1  : S R2 G2 M2 D3 N3 : IbAm

#819: C D E♭ G A♭ A :0 2 3 7 8 9: 1 2 ♭3 5 ♭6 6: mi6/9(add ♭13)
namesBySequenceIndex[818] = '''Katarimic'''
namesBySequenceIndexNegative[818] = ''''''
semitonesBySequenceIndex[818] = [0, 2, 3, 7, 8, 9]
poemBySequenceIndex[818] = '''B F G M N P'''
poemAuthors[818] = ''''''
#819: ䷶䷟ Abounding Persevering: 2 1 4 1 1 3  : S R2 G2 P D1 N1 : Ibob

#820: C D E♭ G A♭ B♭ :0 2 3 7 8 10: 1 2 ♭3 5 ♭6 ♭7: mi9(add ♭13)
namesBySequenceIndex[819] = '''Raga Trimurti,Aeolaphimic'''
namesBySequenceIndexNegative[819] = ''''''
semitonesBySequenceIndex[819] = [0, 2, 3, 7, 8, 10]
poemBySequenceIndex[819] = '''B F G M N R'''
poemAuthors[819] = ''''''
#820: ䷶䷯ Abounding Source: 2 1 4 1 2 2  : S R2 G2 P D1 N2 : Ibok

#821: C D E♭ G A♭ B :0 2 3 7 8 11: 1 2 ♭3 5 ♭6 7: mima9(add ♭13)
namesBySequenceIndex[820] = '''Zadimic'''
namesBySequenceIndexNegative[820] = ''''''
semitonesBySequenceIndex[820] = [0, 2, 3, 7, 8, 11]
poemBySequenceIndex[820] = '''B F G M N S'''
poemAuthors[820] = ''''''
#821: ䷶䷑ Abounding Correcting: 2 1 4 1 3 1  : S R2 G2 P D1 N3 : Ibol

#822: C D E♭ G A B♭ :0 2 3 7 9 10: 1 2 ♭3 5 6 ♭7: mi9(add 13)
namesBySequenceIndex[821] = '''Raga Manavi,Danimic'''
namesBySequenceIndexNegative[821] = ''''''
semitonesBySequenceIndex[821] = [0, 2, 3, 7, 9, 10]
poemBySequenceIndex[821] = '''B F G M P R'''
poemAuthors[821] = ''''''
#822: ䷶䷮ Abounding Confining: 2 1 4 2 1 2  : S R2 G2 P D2 N2 : Ibed

#823: C D E♭ G A B :0 2 3 7 9 11: 1 2 ♭3 5 6 7: mima9(add 13)
namesBySequenceIndex[822] = '''Hawaiian,Epythimic'''
namesBySequenceIndexNegative[822] = ''''''
semitonesBySequenceIndex[822] = [0, 2, 3, 7, 9, 11]
poemBySequenceIndex[822] = '''B F G M P S'''
poemAuthors[822] = ''''''
#823: ䷶䷿ Abounding Unfinished: 2 1 4 2 2 1  : S R2 G2 P D2 N3 : Ibef

#824: C D E♭ G A♯ B :0 2 3 7 10 11: 1 2 ♭3 5 ♯6 7: mima9(add ♭7)
namesBySequenceIndex[823] = '''Eponimic'''
namesBySequenceIndexNegative[823] = ''''''
semitonesBySequenceIndex[823] = [0, 2, 3, 7, 10, 11]
poemBySequenceIndex[823] = '''B F G M R S'''
poemAuthors[823] = ''''''
#824: ䷶䷺ Abounding Dispersing: 2 1 4 3 1 1  : S R2 G2 P D3 N3 : Ibem

#825: C D E♭ G♯ A B♭ :0 2 3 8 9 10: 1 2 ♭3 ♯5 6 ♭7: mi9(♯5 add 13)
namesBySequenceIndex[824] = ''''''
namesBySequenceIndexNegative[824] = ''''''
semitonesBySequenceIndex[824] = [0, 2, 3, 8, 9, 10]
poemBySequenceIndex[824] = '''B F G N P R'''
poemAuthors[824] = ''''''
#825: ䷶䷞ Abounding Conjoining: 2 1 5 1 1 2  : S R2 G2 D1 D2 N2 : Ibid

#826: C D E♭ G♯ A B :0 2 3 8 9 11: 1 2 ♭3 ♯5 6 7: mima9(♯5 add 13)
namesBySequenceIndex[825] = ''''''
namesBySequenceIndexNegative[825] = ''''''
semitonesBySequenceIndex[825] = [0, 2, 3, 8, 9, 11]
poemBySequenceIndex[825] = '''B F G N P S'''
poemAuthors[825] = ''''''
#826: ䷶䷷ Abounding Wanderer: 2 1 5 1 2 1  : S R2 G2 D1 D2 N3 : Ibif

#827: C D E♭ G♯ A♯ B :0 2 3 8 10 11: 1 2 ♭3 ♯5 ♯6 7: mima9(♯5 add ♭7)
namesBySequenceIndex[826] = ''''''
namesBySequenceIndexNegative[826] = ''''''
semitonesBySequenceIndex[826] = [0, 2, 3, 8, 10, 11]
poemBySequenceIndex[826] = '''B F G N R S'''
poemAuthors[826] = ''''''
#827: ䷶䷴ Abounding Infiltrating: 2 1 5 2 1 1  : S R2 G2 D1 N2 N3 : Ibim

#828: C D E♭ A B♭ B :0 2 3 9 10 11: 1 2 ♭3 6 ♭7 7: mima9(no 5 add ♭7 13)
namesBySequenceIndex[827] = ''''''
namesBySequenceIndexNegative[827] = ''''''
semitonesBySequenceIndex[827] = [0, 2, 3, 9, 10, 11]
poemBySequenceIndex[827] = '''B F G P R S'''
poemAuthors[827] = ''''''
#828: ䷶䷋ Abounding Obstruction: 2 1 6 1 1 1  : S R2 G2 D2 N2 N3 : Ibag

#829: C D E F F♯ G :0 2 4 5 6 7: 1 2 3 4 ♯4 5: 11(no 7 add ♯11)
namesBySequenceIndex[828] = '''Raga Dipak'''
namesBySequenceIndexNegative[828] = ''''''
semitonesBySequenceIndex[828] = [0, 2, 4, 5, 6, 7]
poemBySequenceIndex[828] = '''B F J K L M'''
poemAuthors[828] = ''''''
#829: ䷤䷒ Family Arrived: 2 2 1 1 1 5  : S R2 G3 M1 M2 P : ImEj

#830: C D E F G♭ A♭ :0 2 4 5 6 8: 1 2 3 4 ♭5 ♭6: 11(♯5 no 7 add ♯11)
namesBySequenceIndex[829] = '''Epagimic'''
namesBySequenceIndexNegative[829] = ''''''
semitonesBySequenceIndex[829] = [0, 2, 4, 5, 6, 8]
poemBySequenceIndex[829] = '''B F J K L N'''
poemAuthors[829] = ''''''
#830: ䷤䷣ Family Eclipse: 2 2 1 1 2 4  : S R2 G3 M1 M2 D1 : ImIj

#831: C D E F G♭ A :0 2 4 5 6 9: 1 2 3 4 ♭5 6: 13(♭5 no 7)
namesBySequenceIndex[830] = '''Aeronimic'''
namesBySequenceIndexNegative[830] = ''''''
semitonesBySequenceIndex[830] = [0, 2, 4, 5, 6, 9]
poemBySequenceIndex[830] = '''B F J K L P'''
poemAuthors[830] = ''''''
#831: ䷤䷲ Family Thunder: 2 2 1 1 3 3  : S R2 G3 M1 M2 D2 : ImAb

#832: C D E F G♭ B♭ :0 2 4 5 6 10: 1 2 3 4 ♭5 ♭7: 11(♭5)
namesBySequenceIndex[831] = '''Rynimic'''
namesBySequenceIndexNegative[831] = ''''''
semitonesBySequenceIndex[831] = [0, 2, 4, 5, 6, 10]
poemBySequenceIndex[831] = '''B F J K L R'''
poemAuthors[831] = ''''''
#832: ䷤䷂ Family Sprouting: 2 2 1 1 4 2  : S R2 G3 M1 M2 N2 : ImAk

#833: C D E F G♭ B :0 2 4 5 6 11: 1 2 3 4 ♭5 7: ma11(♭5)
namesBySequenceIndex[832] = ''''''
namesBySequenceIndexNegative[832] = ''''''
semitonesBySequenceIndex[832] = [0, 2, 4, 5, 6, 11]
poemBySequenceIndex[832] = '''B F J K L S'''
poemAuthors[832] = ''''''
#833: ䷤䷚ Family Nourishment: 2 2 1 1 5 1  : S R2 G3 M1 M2 N3 : ImAl

#834: C D E F G A♭ :0 2 4 5 7 8: 1 2 3 4 5 ♭6: 11(no 7 add ♭6)
namesBySequenceIndex[833] = '''Ronimic'''
namesBySequenceIndexNegative[833] = ''''''
semitonesBySequenceIndex[833] = [0, 2, 4, 5, 7, 8]
poemBySequenceIndex[833] = '''B F J K M N'''
poemAuthors[833] = ''''''
#834: ䷤䷭ Family Ascending: 2 2 1 2 1 4  : S R2 G3 M1 P D1 : Imoj

#835: C D E F G A :0 2 4 5 7 9: 1 2 3 4 5 6: 13(no 7)
namesBySequenceIndex[834] = '''Raga Kambhoj,Arezzo Major Diatonic Hexachord,Raga Kambodhi (ascending),Raga Yadukua Kambodhi (descending),Raga Devaranjani 2,Mynimic'''
namesBySequenceIndexNegative[834] = ''''''
semitonesBySequenceIndex[834] = [0, 2, 4, 5, 7, 9]
poemBySequenceIndex[834] = '''B F J K M P'''
poemAuthors[834] = ''''''
#835: ䷤䷧ Family Deliverance: 2 2 1 2 2 3  : S R2 G3 M1 P D2 : Imeb

#836: C D E F G B♭ :0 2 4 5 7 10: 1 2 3 4 5 ♭7: 11
namesBySequenceIndex[835] = '''Raga Andhali,Raga Siva Kambhoji,Raga Vivardhini,Ionycrimic'''
namesBySequenceIndexNegative[835] = ''''''
semitonesBySequenceIndex[835] = [0, 2, 4, 5, 7, 10]
poemBySequenceIndex[835] = '''B F J K M R'''
poemAuthors[835] = ''''''
#836: ䷤䷜ Family Chasm: 2 2 1 2 3 2  : S R2 G3 M1 P N2 : Imek

#837: C D E F G B :0 2 4 5 7 11: 1 2 3 4 5 7: ma11
namesBySequenceIndex[836] = '''Raga Kedaram (descending),Raga Vilasini,Ionyptimic'''
namesBySequenceIndexNegative[836] = ''''''
semitonesBySequenceIndex[836] = [0, 2, 4, 5, 7, 11]
poemBySequenceIndex[836] = '''B F J K M S'''
poemAuthors[836] = ''''''
#837: ䷤䷃ Family Naïve: 2 2 1 2 4 1  : S R2 G3 M1 P N3 : Imel

#838: C D E F G♯ A :0 2 4 5 8 9: 1 2 3 4 ♯5 6: 13(♯5 no 7)
namesBySequenceIndex[837] = '''Aeranimic'''
namesBySequenceIndexNegative[837] = ''''''
semitonesBySequenceIndex[837] = [0, 2, 4, 5, 8, 9]
poemBySequenceIndex[837] = '''B F J K N P'''
poemAuthors[837] = ''''''
#838: ䷤䷽ Family Detail: 2 2 1 3 1 3  : S R2 G3 M1 D1 N1 : Imib

#839: C D E F G♯ B♭ :0 2 4 5 8 10: 1 2 3 4 ♯5 ♭7: 11(♯5)
namesBySequenceIndex[838] = '''Lyptimic'''
namesBySequenceIndexNegative[838] = ''''''
semitonesBySequenceIndex[838] = [0, 2, 4, 5, 8, 10]
poemBySequenceIndex[838] = '''B F J K N R'''
poemAuthors[838] = ''''''
#839: ䷤䷦ Family Limping: 2 2 1 3 2 2  : S R2 G3 M1 D1 N2 : Imik

#840: C D E F G♯ B :0 2 4 5 8 11: 1 2 3 4 ♯5 7: ma11(♯5)
namesBySequenceIndex[839] = '''Raga Sarasanana,Ionaptimic'''
namesBySequenceIndexNegative[839] = ''''''
semitonesBySequenceIndex[839] = [0, 2, 4, 5, 8, 11]
poemBySequenceIndex[839] = '''B F J K N S'''
poemAuthors[839] = ''''''
#840: ䷤䷳ Family Stillness: 2 2 1 3 3 1  : S R2 G3 M1 D1 N3 : Imil

#841: C D E F A B♭ :0 2 4 5 9 10: 1 2 3 4 6 ♭7: 13(no 5)
namesBySequenceIndex[840] = '''Raga Rageshri,Raga Nattaikurinji,Raga Rageshwari,Ionagimic'''
namesBySequenceIndexNegative[840] = ''''''
semitonesBySequenceIndex[840] = [0, 2, 4, 5, 9, 10]
poemBySequenceIndex[840] = '''B F J K P R'''
poemAuthors[840] = ''''''
#841: ䷤䷬ Family Clustering: 2 2 1 4 1 2  : S R2 G3 M1 D2 N2 : Imad

#842: C D E F A B :0 2 4 5 9 11: 1 2 3 4 6 7: ma13(no 5)
namesBySequenceIndex[841] = '''Raga Hamsa Vinodini,Aeolanimic'''
namesBySequenceIndexNegative[841] = ''''''
semitonesBySequenceIndex[841] = [0, 2, 4, 5, 9, 11]
poemBySequenceIndex[841] = '''B F J K P S'''
poemAuthors[841] = ''''''
#842: ䷤䷢ Family Prospering: 2 2 1 4 2 1  : S R2 G3 M1 D2 N3 : Imaf

#843: C D E F A♯ B :0 2 4 5 10 11: 1 2 3 4 ♯6 7: ma11(no 5 add ♭7)
namesBySequenceIndex[842] = ''''''
namesBySequenceIndexNegative[842] = ''''''
semitonesBySequenceIndex[842] = [0, 2, 4, 5, 10, 11]
poemBySequenceIndex[842] = '''B F J K R S'''
poemAuthors[842] = ''''''
#843: ䷤䷓ Family Seeing: 2 2 1 5 1 1  : S R2 G3 M1 D3 N3 : Imam

#844: C D E F♯ G A♭ :0 2 4 6 7 8: 1 2 3 ♯4 5 ♭6: ma(add ♭6 9 ♯11)
namesBySequenceIndex[843] = '''Katyrimic'''
namesBySequenceIndexNegative[843] = ''''''
semitonesBySequenceIndex[843] = [0, 2, 4, 6, 7, 8]
poemBySequenceIndex[843] = '''B F J L M N'''
poemAuthors[843] = ''''''
#844: ䷾䷊ Completed Peace: 2 2 2 1 1 4  : S R2 G3 M2 P D1 : IkOj

#845: C D E F♯ G A :0 2 4 6 7 9: 1 2 3 ♯4 5 6: 6/9(add ♯11)
namesBySequenceIndex[844] = '''Raga Airavati,Ancient Chinese,Raga Kalyani,Raga Yamuna Kalyani,Raga Kalyani Keseri,Lonimic'''
namesBySequenceIndexNegative[844] = ''''''
semitonesBySequenceIndex[844] = [0, 2, 4, 6, 7, 9]
poemBySequenceIndex[844] = '''B F J L M P'''
poemAuthors[844] = ''''''
#845: ䷾䷵ Completed Betrothing: 2 2 2 1 2 3  : S R2 G3 M2 P D2 : IkEb

#846: C D E F♯ G B♭ :0 2 4 6 7 10: 1 2 3 ♯4 5 ♭7: 9(add ♯11)
namesBySequenceIndex[845] = '''Salimic'''
namesBySequenceIndexNegative[845] = ''''''
semitonesBySequenceIndex[845] = [0, 2, 4, 6, 7, 10]
poemBySequenceIndex[845] = '''B F J L M R'''
poemAuthors[845] = ''''''
#846: ䷾䷻ Completed Composed: 2 2 2 1 3 2  : S R2 G3 M2 P N2 : IkEk

#847: C D E F♯ G B :0 2 4 6 7 11: 1 2 3 ♯4 5 7: ma9(add ♯11)
namesBySequenceIndex[846] = '''Raga Ratnakanthi,Raga Chaturangini 2,Phrolimic'''
namesBySequenceIndexNegative[846] = ''''''
semitonesBySequenceIndex[846] = [0, 2, 4, 6, 7, 11]
poemBySequenceIndex[846] = '''B F J L M S'''
poemAuthors[846] = ''''''
#847: ䷾䷨ Completed Diminishing: 2 2 2 1 4 1  : S R2 G3 M2 P N3 : IkEl

#848: C D E F♯ G♯ A :0 2 4 6 8 9: 1 2 3 ♯4 ♯5 6: 6/9(♯5 add ♯11)
namesBySequenceIndex[847] = '''Eskimo Sexatonic 1,Epothimic'''
namesBySequenceIndexNegative[847] = ''''''
semitonesBySequenceIndex[847] = [0, 2, 4, 6, 8, 9]
poemBySequenceIndex[847] = '''B F J L N P'''
poemAuthors[847] = ''''''
#848: ䷾䷶ Completed Abounding: 2 2 2 2 1 3  : S R2 G3 M2 D1 N1 : IkIb

#849: C D E F♯ G♯ B♭ :0 2 4 6 8 10: 1 2 3 ♯4 ♯5 ♭7: 9(♯5 add ♯11)
namesBySequenceIndex[848] = '''Whole Tone,Sexatonic,Anhemitonic Sexatonic,Messiaen 1st Mode,Raga Sahera,Raga Gopriya,Kylimic'''
namesBySequenceIndexNegative[848] = ''''''
semitonesBySequenceIndex[848] = [0, 2, 4, 6, 8, 10]
poemBySequenceIndex[848] = '''B F J L N R'''
poemAuthors[848] = ''''''
#849: ䷾䷾ Completed Completed : 2 2 2 2 2 2  : S R2 G3 M2 D1 N2 : IkIk

#850: C D E F♯ G♯ B :0 2 4 6 8 11: 1 2 3 ♯4 ♯5 7: ma9(♯5 add ♯11)
namesBySequenceIndex[849] = '''Eskimo Sexatonic 2,Thynimic'''
namesBySequenceIndexNegative[849] = ''''''
semitonesBySequenceIndex[849] = [0, 2, 4, 6, 8, 11]
poemBySequenceIndex[849] = '''B F J L N S'''
poemAuthors[849] = ''''''
#850: ䷾䷕ Completed Adorning: 2 2 2 2 3 1  : S R2 G3 M2 D1 N3 : IkIl

#851: C D E G♭ A B♭ :0 2 4 6 9 10: 1 2 3 ♭5 6 ♭7: 9(♭5 add 13)
namesBySequenceIndex[850] = '''Prometheus,Raga Barbara,Aeolathimic'''
namesBySequenceIndexNegative[850] = ''''''
semitonesBySequenceIndex[850] = [0, 2, 4, 6, 9, 10]
poemBySequenceIndex[850] = '''B F J L P R'''
poemAuthors[850] = ''''''
#851: ䷾䷐ Completed Following: 2 2 2 3 1 2  : S R2 G3 M2 D2 N2 : IkAd

#852: C D E G♭ A B :0 2 4 6 9 11: 1 2 3 ♭5 6 7: ma9(♭5 add 13)
namesBySequenceIndex[851] = '''Raga Mruganandana,Zoptimic'''
namesBySequenceIndexNegative[851] = ''''''
semitonesBySequenceIndex[851] = [0, 2, 4, 6, 9, 11]
poemBySequenceIndex[851] = '''B F J L P S'''
poemAuthors[851] = ''''''
#852: ䷾䷔ Completed Biting: 2 2 2 3 2 1  : S R2 G3 M2 D2 N3 : IkAf

#853: C D E G♭ A♯ B :0 2 4 6 10 11: 1 2 3 ♭5 ♯6 7: ma9(♭5 add ♭7)
namesBySequenceIndex[852] = '''Zyptimic'''
namesBySequenceIndexNegative[852] = ''''''
semitonesBySequenceIndex[852] = [0, 2, 4, 6, 10, 11]
poemBySequenceIndex[852] = '''B F J L R S'''
poemAuthors[852] = ''''''
#853: ䷾䷩ Completed Augmenting: 2 2 2 4 1 1  : S R2 G3 M2 D3 N3 : IkAm

#854: C D E G A♭ A :0 2 4 7 8 9: 1 2 3 5 ♭6 6: 6/9(add ♭13)
namesBySequenceIndex[853] = '''Bebop Major Sexatonic,Dygimic'''
namesBySequenceIndexNegative[853] = ''''''
semitonesBySequenceIndex[853] = [0, 2, 4, 7, 8, 9]
poemBySequenceIndex[853] = '''B F J M N P'''
poemAuthors[853] = ''''''
#854: ䷾䷟ Completed Persevering: 2 2 3 1 1 3  : S R2 G3 P D1 N1 : Ikob

#855: C D E G A♭ B♭ :0 2 4 7 8 10: 1 2 3 5 ♭6 ♭7: 9(add ♭13)
namesBySequenceIndex[854] = '''Bythimic'''
namesBySequenceIndexNegative[854] = ''''''
semitonesBySequenceIndex[854] = [0, 2, 4, 7, 8, 10]
poemBySequenceIndex[854] = '''B F J M N R'''
poemAuthors[854] = ''''''
#855: ䷾䷯ Completed Source: 2 2 3 1 2 2  : S R2 G3 P D1 N2 : Ikok

#856: C D E G A♭ B :0 2 4 7 8 11: 1 2 3 5 ♭6 7: ma9(add ♭13)
namesBySequenceIndex[855] = '''Raga Latika,Stonimic'''
namesBySequenceIndexNegative[855] = ''''''
semitonesBySequenceIndex[855] = [0, 2, 4, 7, 8, 11]
poemBySequenceIndex[855] = '''B F J M N S'''
poemAuthors[855] = ''''''
#856: ䷾䷑ Completed Correcting: 2 2 3 1 3 1  : S R2 G3 P D1 N3 : Ikol

#857: C D E G A B♭ :0 2 4 7 9 10: 1 2 3 5 6 ♭7: 9(add 13)
namesBySequenceIndex[856] = '''Zeracrimic,Mixolydian No Four,Mixolydian Sexatonic 1'''
namesBySequenceIndexNegative[856] = ''''''
semitonesBySequenceIndex[856] = [0, 2, 4, 7, 9, 10]
poemBySequenceIndex[856] = '''B F J M P R'''
poemAuthors[856] = ''''''
#857: ䷾䷮ Completed Confining: 2 2 3 2 1 2  : S R2 G3 P D2 N2 : Iked

#858: C D E G A B :0 2 4 7 9 11: 1 2 3 5 6 7: ma9(add 13)
namesBySequenceIndex[857] = '''Raga Sankara,Raga Kumud,Raga Shankara,Raga Prabhati,Lydian Sexatonic,Major Pentatonic Add Ma7,Thaptimic'''
namesBySequenceIndexNegative[857] = ''''''
semitonesBySequenceIndex[857] = [0, 2, 4, 7, 9, 11]
poemBySequenceIndex[857] = '''B F J M P S'''
poemAuthors[857] = ''''''
#858: ䷾䷿ Completed Unfinished: 2 2 3 2 2 1  : S R2 G3 P D2 N3 : Ikef

#859: C D E G A♯ B :0 2 4 7 10 11: 1 2 3 5 ♯6 7: ma9(add ♭7)
namesBySequenceIndex[858] = '''Bycrimic'''
namesBySequenceIndexNegative[858] = ''''''
semitonesBySequenceIndex[858] = [0, 2, 4, 7, 10, 11]
poemBySequenceIndex[858] = '''B F J M R S'''
poemAuthors[858] = ''''''
#859: ䷾䷺ Completed Dispersing: 2 2 3 3 1 1  : S R2 G3 P D3 N3 : Ikem

#860: C D E G♯ A B♭ :0 2 4 8 9 10: 1 2 3 ♯5 6 ♭7: 9(♯5 add 13)
namesBySequenceIndex[859] = '''Katothimic'''
namesBySequenceIndexNegative[859] = ''''''
semitonesBySequenceIndex[859] = [0, 2, 4, 8, 9, 10]
poemBySequenceIndex[859] = '''B F J N P R'''
poemAuthors[859] = ''''''
#860: ䷾䷞ Completed Conjoining: 2 2 4 1 1 2  : S R2 G3 D1 D2 N2 : Ikid

#861: C D E G♯ A B :0 2 4 8 9 11: 1 2 3 ♯5 6 7: ma9(♯5 add 13)
namesBySequenceIndex[860] = '''Aelothimic'''
namesBySequenceIndexNegative[860] = ''''''
semitonesBySequenceIndex[860] = [0, 2, 4, 8, 9, 11]
poemBySequenceIndex[860] = '''B F J N P S'''
poemAuthors[860] = ''''''
#861: ䷾䷷ Completed Wanderer: 2 2 4 1 2 1  : S R2 G3 D1 D2 N3 : Ikif

#862: C D E G♯ A♯ B :0 2 4 8 10 11: 1 2 3 ♯5 ♯6 7: ma9(♯5 add ♭7)
namesBySequenceIndex[861] = '''Aeolocrimic'''
namesBySequenceIndexNegative[861] = ''''''
semitonesBySequenceIndex[861] = [0, 2, 4, 8, 10, 11]
poemBySequenceIndex[861] = '''B F J N R S'''
poemAuthors[861] = ''''''
#862: ䷾䷴ Completed Infiltrating: 2 2 4 2 1 1  : S R2 G3 D1 N2 N3 : Ikim

#863: C D E A B♭ B :0 2 4 9 10 11: 1 2 3 6 ♭7 7: ma9(no 5 add ♭7 13)
namesBySequenceIndex[862] = ''''''
namesBySequenceIndexNegative[862] = ''''''
semitonesBySequenceIndex[862] = [0, 2, 4, 9, 10, 11]
poemBySequenceIndex[862] = '''B F J P R S'''
poemAuthors[862] = ''''''
#863: ䷾䷋ Completed Obstruction: 2 2 5 1 1 1  : S R2 G3 D2 N2 N3 : Ikag

#864: C D F F♯ G A♭ :0 2 5 6 7 8: 1 2 4 ♯4 5 ♭6: sus4(add ♭6 9 ♯11)
namesBySequenceIndex[863] = '''Stoptimic'''
namesBySequenceIndexNegative[863] = ''''''
semitonesBySequenceIndex[863] = [0, 2, 5, 6, 7, 8]
poemBySequenceIndex[863] = '''B F K L M N'''
poemAuthors[863] = ''''''
#864: ䷕䷊ Adorning Peace: 2 3 1 1 1 4  : S R2 M1 M2 P D1 : IlOj

#865: C D F F♯ G A :0 2 5 6 7 9: 1 2 4 ♯4 5 6: 6/9sus4(add ♯11)
namesBySequenceIndex[864] = '''Gathimic'''
namesBySequenceIndexNegative[864] = ''''''
semitonesBySequenceIndex[864] = [0, 2, 5, 6, 7, 9]
poemBySequenceIndex[864] = '''B F K L M P'''
poemAuthors[864] = ''''''
#865: ䷕䷵ Adorning Betrothing: 2 3 1 1 2 3  : S R2 M1 M2 P D2 : IlEb

#866: C D F F♯ G B♭ :0 2 5 6 7 10: 1 2 4 ♯4 5 ♭7: 11sus♯4
namesBySequenceIndex[865] = '''Zolimic'''
namesBySequenceIndexNegative[865] = ''''''
semitonesBySequenceIndex[865] = [0, 2, 5, 6, 7, 10]
poemBySequenceIndex[865] = '''B F K L M R'''
poemAuthors[865] = ''''''
#866: ䷕䷻ Adorning Composed: 2 3 1 1 3 2  : S R2 M1 M2 P N2 : IlEk

#867: C D F F♯ G B :0 2 5 6 7 11: 1 2 4 ♯4 5 7: ma11sus♯4
namesBySequenceIndex[866] = '''Kagimic'''
namesBySequenceIndexNegative[866] = ''''''
semitonesBySequenceIndex[866] = [0, 2, 5, 6, 7, 11]
poemBySequenceIndex[866] = '''B F K L M S'''
poemAuthors[866] = ''''''
#867: ䷕䷨ Adorning Diminishing: 2 3 1 1 4 1  : S R2 M1 M2 P N3 : IlEl

#868: C D F G♭ A♭ A :0 2 5 6 8 9: 1 2 4 ♭5 ♭6 6: 6/9sus4(♯5 add ♯11)
namesBySequenceIndex[867] = '''Kothimic'''
namesBySequenceIndexNegative[867] = ''''''
semitonesBySequenceIndex[867] = [0, 2, 5, 6, 8, 9]
poemBySequenceIndex[867] = '''B F K L N P'''
poemAuthors[867] = ''''''
#868: ䷕䷶ Adorning Abounding: 2 3 1 2 1 3  : S R2 M1 M2 D1 N1 : IlIb

#869: C D F G♭ A♭ B♭ :0 2 5 6 8 10: 1 2 4 ♭5 ♭6 ♭7: 11sus♯4(♯5)
namesBySequenceIndex[868] = '''Padimic'''
namesBySequenceIndexNegative[868] = ''''''
semitonesBySequenceIndex[868] = [0, 2, 5, 6, 8, 10]
poemBySequenceIndex[868] = '''B F K L N R'''
poemAuthors[868] = ''''''
#869: ䷕䷾ Adorning Completed: 2 3 1 2 2 2  : S R2 M1 M2 D1 N2 : IlIk

#870: C D F G♭ A♭ B :0 2 5 6 8 11: 1 2 4 ♭5 ♭6 7: ma11sus♯4(♯5)
namesBySequenceIndex[869] = '''Katalimic'''
namesBySequenceIndexNegative[869] = ''''''
semitonesBySequenceIndex[869] = [0, 2, 5, 6, 8, 11]
poemBySequenceIndex[869] = '''B F K L N S'''
poemAuthors[869] = ''''''
#870: ䷕䷕ Adorning Adorning : 2 3 1 2 3 1  : S R2 M1 M2 D1 N3 : IlIl

#871: C D F G♭ A B♭ :0 2 5 6 9 10: 1 2 4 ♭5 6 ♭7: 13(♭5 no 3)
namesBySequenceIndex[870] = '''Syptimic'''
namesBySequenceIndexNegative[870] = ''''''
semitonesBySequenceIndex[870] = [0, 2, 5, 6, 9, 10]
poemBySequenceIndex[870] = '''B F K L P R'''
poemAuthors[870] = ''''''
#871: ䷕䷐ Adorning Following: 2 3 1 3 1 2  : S R2 M1 M2 D2 N2 : IlAd

#872: C D F G♭ A B :0 2 5 6 9 11: 1 2 4 ♭5 6 7: ma13(♭5 no 3)
namesBySequenceIndex[871] = '''Stydimic'''
namesBySequenceIndexNegative[871] = ''''''
semitonesBySequenceIndex[871] = [0, 2, 5, 6, 9, 11]
poemBySequenceIndex[871] = '''B F K L P S'''
poemAuthors[871] = ''''''
#872: ䷕䷔ Adorning Biting: 2 3 1 3 2 1  : S R2 M1 M2 D2 N3 : IlAf

#873: C D F G♭ A♯ B :0 2 5 6 10 11: 1 2 4 ♭5 ♯6 7: ma9sus4(♭5 add ♭7)
namesBySequenceIndex[872] = '''Zarimic'''
namesBySequenceIndexNegative[872] = ''''''
semitonesBySequenceIndex[872] = [0, 2, 5, 6, 10, 11]
poemBySequenceIndex[872] = '''B F K L R S'''
poemAuthors[872] = ''''''
#873: ䷕䷩ Adorning Augmenting: 2 3 1 4 1 1  : S R2 M1 M2 D3 N3 : IlAm

#874: C D F G A♭ A :0 2 5 7 8 9: 1 2 4 5 ♭6 6: 6/9sus4(add ♭13)
namesBySequenceIndex[873] = '''Dadimic'''
namesBySequenceIndexNegative[873] = ''''''
semitonesBySequenceIndex[873] = [0, 2, 5, 7, 8, 9]
poemBySequenceIndex[873] = '''B F K M N P'''
poemAuthors[873] = ''''''
#874: ䷕䷟ Adorning Persevering: 2 3 2 1 1 3  : S R2 M1 P D1 N1 : Ilob

#875: C D F G A♭ B♭ :0 2 5 7 8 10: 1 2 4 5 ♭6 ♭7: 9sus4(add ♭13)
namesBySequenceIndex[874] = '''Raga Navamanohari,Byptimic'''
namesBySequenceIndexNegative[874] = ''''''
semitonesBySequenceIndex[874] = [0, 2, 5, 7, 8, 10]
poemBySequenceIndex[874] = '''B F K M N R'''
poemAuthors[874] = ''''''
#875: ䷕䷯ Adorning Source: 2 3 2 1 2 2  : S R2 M1 P D1 N2 : Ilok

#876: C D F G A♭ B :0 2 5 7 8 11: 1 2 4 5 ♭6 7: ma9sus4(add ♭13)
namesBySequenceIndex[875] = '''Raga Bhinna Pancama,Staptimic'''
namesBySequenceIndexNegative[875] = ''''''
semitonesBySequenceIndex[875] = [0, 2, 5, 7, 8, 11]
poemBySequenceIndex[875] = '''B F K M N S'''
poemAuthors[875] = ''''''
#876: ䷕䷑ Adorning Correcting: 2 3 2 1 3 1  : S R2 M1 P D1 N3 : Ilol

#877: C D F G A B♭ :0 2 5 7 9 10: 1 2 4 5 6 ♭7: 13(no 3)
namesBySequenceIndex[876] = '''P’Yongjo,Yosen (Japan),Raga Devamanohari,Raga Andolika,Raga Darbar,Raga Gorakh,Raga Narayani,Raga Suposhini,Mixolydian Sexatonic 2,P’Yongjo (Korea),Lothimic'''
namesBySequenceIndexNegative[876] = ''''''
semitonesBySequenceIndex[876] = [0, 2, 5, 7, 9, 10]
poemBySequenceIndex[876] = '''B F K M P R'''
poemAuthors[876] = ''''''
#877: ䷕䷮ Adorning Confining: 2 3 2 2 1 2  : S R2 M1 P D2 N2 : Iled

#878: C D F G A B :0 2 5 7 9 11: 1 2 4 5 6 7: ma13(no 3)
namesBySequenceIndex[877] = '''Raga Nagagandhari,Gonimic'''
namesBySequenceIndexNegative[877] = ''''''
semitonesBySequenceIndex[877] = [0, 2, 5, 7, 9, 11]
poemBySequenceIndex[877] = '''B F K M P S'''
poemAuthors[877] = ''''''
#878: ䷕䷿ Adorning Unfinished: 2 3 2 2 2 1  : S R2 M1 P D2 N3 : Ilef

#879: C D F G A♯ B :0 2 5 7 10 11: 1 2 4 5 ♯6 7: ma9sus4(add ♭7)
namesBySequenceIndex[878] = '''Raga Brindabani,Raga Brindabani Sarang,Raga Megh,Raga Megh Malhar,Raga Adana (ascending),Thatimic'''
namesBySequenceIndexNegative[878] = ''''''
semitonesBySequenceIndex[878] = [0, 2, 5, 7, 10, 11]
poemBySequenceIndex[878] = '''B F K M R S'''
poemAuthors[878] = ''''''
#879: ䷕䷺ Adorning Dispersing: 2 3 2 3 1 1  : S R2 M1 P D3 N3 : Ilem

#880: C D F G♯ A B♭ :0 2 5 8 9 10: 1 2 4 ♯5 6 ♭7: 13(♯5 no 3)
namesBySequenceIndex[879] = '''Pathimic'''
namesBySequenceIndexNegative[879] = ''''''
semitonesBySequenceIndex[879] = [0, 2, 5, 8, 9, 10]
poemBySequenceIndex[879] = '''B F K N P R'''
poemAuthors[879] = ''''''
#880: ䷕䷞ Adorning Conjoining: 2 3 3 1 1 2  : S R2 M1 D1 D2 N2 : Ilid

#881: C D F G♯ A B :0 2 5 8 9 11: 1 2 4 ♯5 6 7: ma13(♯5 no 3)
namesBySequenceIndex[880] = '''Baptimic'''
namesBySequenceIndexNegative[880] = ''''''
semitonesBySequenceIndex[880] = [0, 2, 5, 8, 9, 11]
poemBySequenceIndex[880] = '''B F K N P S'''
poemAuthors[880] = ''''''
#881: ䷕䷷ Adorning Wanderer: 2 3 3 1 2 1  : S R2 M1 D1 D2 N3 : Ilif

#882: C D F G♯ A♯ B :0 2 5 8 10 11: 1 2 4 ♯5 ♯6 7: ma9sus4(♯5 add ♭7)
namesBySequenceIndex[881] = '''Katolimic'''
namesBySequenceIndexNegative[881] = ''''''
semitonesBySequenceIndex[881] = [0, 2, 5, 8, 10, 11]
poemBySequenceIndex[881] = '''B F K N R S'''
poemAuthors[881] = ''''''
#882: ䷕䷴ Adorning Infiltrating: 2 3 3 2 1 1  : S R2 M1 D1 N2 N3 : Ilim

#883: C D F A B♭ B :0 2 5 9 10 11: 1 2 4 6 ♭7 7: ma13(no 3 5 add ♭7)
namesBySequenceIndex[882] = '''Gylimic'''
namesBySequenceIndexNegative[882] = ''''''
semitonesBySequenceIndex[882] = [0, 2, 5, 9, 10, 11]
poemBySequenceIndex[882] = '''B F K P R S'''
poemAuthors[882] = ''''''
#883: ䷕䷋ Adorning Obstruction: 2 3 4 1 1 1  : S R2 M1 D2 N2 N3 : Ilag

#884: C D F♯ G A♭ A :0 2 6 7 8 9: 1 2 ♯4 5 ♭6 6: 6sus2(add ♭6 ♯11)
namesBySequenceIndex[883] = '''Ionothimic'''
namesBySequenceIndexNegative[883] = ''''''
semitonesBySequenceIndex[883] = [0, 2, 6, 7, 8, 9]
poemBySequenceIndex[883] = '''B F L M N P'''
poemAuthors[883] = ''''''
#884: ䷣䷡ Eclipse Might: 2 4 1 1 1 3  : S R2 M2 P D1 N1 : IjOb

#885: C D F♯ G A♭ B♭ :0 2 6 7 8 10: 1 2 ♯4 5 ♭6 ♭7: 9sus♯4(add ♭13)
namesBySequenceIndex[884] = '''Raga Jaganmohanam,Phrylimic'''
namesBySequenceIndexNegative[884] = ''''''
semitonesBySequenceIndex[884] = [0, 2, 6, 7, 8, 10]
poemBySequenceIndex[884] = '''B F L M N R'''
poemAuthors[884] = ''''''
#885: ䷣䷄ Eclipse Biding: 2 4 1 1 2 2  : S R2 M2 P D1 N2 : IjOk

#886: C D F♯ G A♭ B :0 2 6 7 8 11: 1 2 ♯4 5 ♭6 7: ma9sus♯4(add ♭13)
namesBySequenceIndex[885] = '''Ralimic'''
namesBySequenceIndexNegative[885] = ''''''
semitonesBySequenceIndex[885] = [0, 2, 6, 7, 8, 11]
poemBySequenceIndex[885] = '''B F L M N S'''
poemAuthors[885] = ''''''
#886: ䷣䷙ Eclipse Cultivating: 2 4 1 1 3 1  : S R2 M2 P D1 N3 : IjOl

#887: C D F♯ G A B♭ :0 2 6 7 9 10: 1 2 ♯4 5 6 ♭7: 13(♯11 no 3)
namesBySequenceIndex[886] = '''Raga Sarasvati,Socrimic'''
namesBySequenceIndexNegative[886] = ''''''
semitonesBySequenceIndex[886] = [0, 2, 6, 7, 9, 10]
poemBySequenceIndex[886] = '''B F L M P R'''
poemAuthors[886] = ''''''
#887: ䷣䷹ Eclipse Joyous: 2 4 1 2 1 2  : S R2 M2 P D2 N2 : IjEd

#888: C D F♯ G A B :0 2 6 7 9 11: 1 2 ♯4 5 6 7: ma13(♯11 no 3)
namesBySequenceIndex[887] = '''Raga Nishadi,Stolimic'''
namesBySequenceIndexNegative[887] = ''''''
semitonesBySequenceIndex[887] = [0, 2, 6, 7, 9, 11]
poemBySequenceIndex[887] = '''B F L M P S'''
poemAuthors[887] = ''''''
#888: ䷣䷥ Eclipse Polarising: 2 4 1 2 2 1  : S R2 M2 P D2 N3 : IjEf

#889: C D F♯ G A♯ B :0 2 6 7 10 11: 1 2 ♯4 5 ♯6 7: ma9sus♯4(add ♭7)
namesBySequenceIndex[888] = '''Raga Malarani,Lodimic'''
namesBySequenceIndexNegative[888] = ''''''
semitonesBySequenceIndex[888] = [0, 2, 6, 7, 10, 11]
poemBySequenceIndex[888] = '''B F L M R S'''
poemAuthors[888] = ''''''
#889: ䷣䷼ Eclipse Truth: 2 4 1 3 1 1  : S R2 M2 P D3 N3 : IjEm

#890: C D F♯ G♯ A B♭ :0 2 6 8 9 10: 1 2 ♯4 ♯5 6 ♭7: 13(♯5 ♯11 no 3)
namesBySequenceIndex[889] = '''Phrygimic'''
namesBySequenceIndexNegative[889] = ''''''
semitonesBySequenceIndex[889] = [0, 2, 6, 8, 9, 10]
poemBySequenceIndex[889] = '''B F L N P R'''
poemAuthors[889] = ''''''
#890: ䷣䷰ Eclipse Revolution: 2 4 2 1 1 2  : S R2 M2 D1 D2 N2 : IjId

#891: C D F♯ G♯ A B :0 2 6 8 9 11: 1 2 ♯4 ♯5 6 7: ma13(♯5 ♯11 no 3)
namesBySequenceIndex[890] = '''Byrimic'''
namesBySequenceIndexNegative[890] = ''''''
semitonesBySequenceIndex[890] = [0, 2, 6, 8, 9, 11]
poemBySequenceIndex[890] = '''B F L N P S'''
poemAuthors[890] = ''''''
#891: ䷣䷝ Eclipse Distanced: 2 4 2 1 2 1  : S R2 M2 D1 D2 N3 : IjIf

#892: C D F♯ G♯ A♯ B :0 2 6 8 10 11: 1 2 ♯4 ♯5 ♯6 7: ma9sus♯4(♯5 add ♭7)
namesBySequenceIndex[891] = '''Sydimic'''
namesBySequenceIndexNegative[891] = ''''''
semitonesBySequenceIndex[891] = [0, 2, 6, 8, 10, 11]
poemBySequenceIndex[891] = '''B F L N R S'''
poemAuthors[891] = ''''''
#892: ䷣䷤ Eclipse Family: 2 4 2 2 1 1  : S R2 M2 D1 N2 N3 : IjIm

#893: C D G♭ A B♭ B :0 2 6 9 10 11: 1 2 ♭5 6 ♭7 7: ma7sus2(♭5 add ♭7 13)
namesBySequenceIndex[892] = '''Sathimic'''
namesBySequenceIndexNegative[892] = ''''''
semitonesBySequenceIndex[892] = [0, 2, 6, 9, 10, 11]
poemBySequenceIndex[892] = '''B F L P R S'''
poemAuthors[892] = ''''''
#893: ䷣䷘ Eclipse Integrity: 2 4 3 1 1 1  : S R2 M2 D2 N2 N3 : IjAg

#894: C D G A♭ A B♭ :0 2 7 8 9 10: 1 2 5 ♭6 6 ♭7: 7sus2(add ♭13 13)
namesBySequenceIndex[893] = ''''''
namesBySequenceIndexNegative[893] = ''''''
semitonesBySequenceIndex[893] = [0, 2, 7, 8, 9, 10]
poemBySequenceIndex[893] = '''B F M N P R'''
poemAuthors[893] = ''''''
#894: ䷣䷛ Eclipse Excess: 2 5 1 1 1 2  : S R2 P D1 D2 N2 : Ijod

#895: C D G A♭ A B :0 2 7 8 9 11: 1 2 5 ♭6 6 7: ma7sus2(add ♭13 13)
namesBySequenceIndex[894] = ''''''
namesBySequenceIndexNegative[894] = ''''''
semitonesBySequenceIndex[894] = [0, 2, 7, 8, 9, 11]
poemBySequenceIndex[894] = '''B F M N P S'''
poemAuthors[894] = ''''''
#895: ䷣䷱ Eclipse Holding: 2 5 1 1 2 1  : S R2 P D1 D2 N3 : Ijof

#896: C D G A♭ B♭ B :0 2 7 8 10 11: 1 2 5 ♭6 ♭7 7: ma7sus2(add ♭7 ♭13)
namesBySequenceIndex[895] = ''''''
namesBySequenceIndexNegative[895] = ''''''
semitonesBySequenceIndex[895] = [0, 2, 7, 8, 10, 11]
poemBySequenceIndex[895] = '''B F M N R S'''
poemAuthors[895] = ''''''
#896: ䷣䷸ Eclipse Osmosis: 2 5 1 2 1 1  : S R2 P D1 N2 N3 : Ijom

#897: C D G A B♭ B :0 2 7 9 10 11: 1 2 5 6 ♭7 7: ma7sus2(add ♭7 13)
namesBySequenceIndex[896] = ''''''
namesBySequenceIndexNegative[896] = ''''''
semitonesBySequenceIndex[896] = [0, 2, 7, 9, 10, 11]
poemBySequenceIndex[896] = '''B F M P R S'''
poemAuthors[896] = ''''''
#897: ䷣䷅ Eclipse Arguing: 2 5 2 1 1 1  : S R2 P D2 N2 N3 : Ijeg

#898: C D G♯ A B♭ B :0 2 8 9 10 11: 1 2 ♯5 6 ♭7 7: ma7sus2(♯5 add ♭7 13)
namesBySequenceIndex[897] = ''''''
namesBySequenceIndexNegative[897] = ''''''
semitonesBySequenceIndex[897] = [0, 2, 8, 9, 10, 11]
poemBySequenceIndex[897] = '''B F N P R S'''
poemAuthors[897] = ''''''
#898: ䷣䷠ Eclipse Retiring: 2 6 1 1 1 1  : S R2 D1 D2 N2 N3 : Ijig

#899: C D♯ E F F♯ G :0 3 4 5 6 7: 1 ♯2 3 4 ♯4 5: ma(add ♯9 11 ♯11)
namesBySequenceIndex[898] = ''''''
namesBySequenceIndexNegative[898] = ''''''
semitonesBySequenceIndex[898] = [0, 3, 4, 5, 6, 7]
poemBySequenceIndex[898] = '''B G J K L M'''
poemAuthors[898] = ''''''
#899: ䷘䷒ Integrity Arrived: 3 1 1 1 1 5  : S R3 G3 M1 M2 P : AgEj

#900: C D♯ E F G♭ A♭ :0 3 4 5 6 8: 1 ♯2 3 4 ♭5 ♭6: ma(♯5 add ♯9 11 ♯11)
namesBySequenceIndex[899] = '''Kathimic'''
namesBySequenceIndexNegative[899] = ''''''
semitonesBySequenceIndex[899] = [0, 3, 4, 5, 6, 8]
poemBySequenceIndex[899] = '''B G J K L N'''
poemAuthors[899] = ''''''
#900: ䷘䷣ Integrity Eclipse: 3 1 1 1 2 4  : S R3 G3 M1 M2 D1 : AgIj

#901: C D♯ E F G♭ A :0 3 4 5 6 9: 1 ♯2 3 4 ♭5 6: 6(♭5 add ♯9 11)
namesBySequenceIndex[900] = '''Kydimic'''
namesBySequenceIndexNegative[900] = ''''''
semitonesBySequenceIndex[900] = [0, 3, 4, 5, 6, 9]
poemBySequenceIndex[900] = '''B G J K L P'''
poemAuthors[900] = ''''''
#901: ䷘䷲ Integrity Thunder: 3 1 1 1 3 3  : S R3 G3 M1 M2 D2 : AgAb

#902: C D♯ E F G♭ B♭ :0 3 4 5 6 10: 1 ♯2 3 4 ♭5 ♭7: 11(♭5 ♯9)
namesBySequenceIndex[901] = '''Zygimic'''
namesBySequenceIndexNegative[901] = ''''''
semitonesBySequenceIndex[901] = [0, 3, 4, 5, 6, 10]
poemBySequenceIndex[901] = '''B G J K L R'''
poemAuthors[901] = ''''''
#902: ䷘䷂ Integrity Sprouting: 3 1 1 1 4 2  : S R3 G3 M1 M2 N2 : AgAk

#903: C D♯ E F G♭ B :0 3 4 5 6 11: 1 ♯2 3 4 ♭5 7: ma11(♭5 ♯9)
namesBySequenceIndex[902] = ''''''
namesBySequenceIndexNegative[902] = ''''''
semitonesBySequenceIndex[902] = [0, 3, 4, 5, 6, 11]
poemBySequenceIndex[902] = '''B G J K L S'''
poemAuthors[902] = ''''''
#903: ䷘䷚ Integrity Nourishment: 3 1 1 1 5 1  : S R3 G3 M1 M2 N3 : AgAl

#904: C D♯ E F G A♭ :0 3 4 5 7 8: 1 ♯2 3 4 5 ♭6: ma(add ♭6 ♯9 11)
namesBySequenceIndex[903] = '''Thycrimic'''
namesBySequenceIndexNegative[903] = ''''''
semitonesBySequenceIndex[903] = [0, 3, 4, 5, 7, 8]
poemBySequenceIndex[903] = '''B G J K M N'''
poemAuthors[903] = ''''''
#904: ䷘䷭ Integrity Ascending: 3 1 1 2 1 4  : S R3 G3 M1 P D1 : Agoj

#905: C D♯ E F G A :0 3 4 5 7 9: 1 ♯2 3 4 5 6: 6(add ♯9 11)
namesBySequenceIndex[904] = '''Lagimic'''
namesBySequenceIndexNegative[904] = ''''''
semitonesBySequenceIndex[904] = [0, 3, 4, 5, 7, 9]
poemBySequenceIndex[904] = '''B G J K M P'''
poemAuthors[904] = ''''''
#905: ䷘䷧ Integrity Deliverance: 3 1 1 2 2 3  : S R3 G3 M1 P D2 : Ageb

#906: C D♯ E F G B♭ :0 3 4 5 7 10: 1 ♯2 3 4 5 ♭7: 11(♯9)
namesBySequenceIndex[905] = '''Mela Calanata,Raga Bhanumanjari,Raga Jog,Ionynimic'''
namesBySequenceIndexNegative[905] = ''''''
semitonesBySequenceIndex[905] = [0, 3, 4, 5, 7, 10]
poemBySequenceIndex[905] = '''B G J K M R'''
poemAuthors[905] = ''''''
#906: ䷘䷜ Integrity Chasm: 3 1 1 2 3 2  : S R3 G3 M1 P N2 : Agek

#907: C D♯ E F G B :0 3 4 5 7 11: 1 ♯2 3 4 5 7: ma11(♯9)
namesBySequenceIndex[906] = '''Donimic'''
namesBySequenceIndexNegative[906] = ''''''
semitonesBySequenceIndex[906] = [0, 3, 4, 5, 7, 11]
poemBySequenceIndex[906] = '''B G J K M S'''
poemAuthors[906] = ''''''
#907: ䷘䷃ Integrity Naïve: 3 1 1 2 4 1  : S R3 G3 M1 P N3 : Agel

#908: C D♯ E F G♯ A :0 3 4 5 8 9: 1 ♯2 3 4 ♯5 6: 6(♯5 add ♯9 11)
namesBySequenceIndex[907] = '''Thyptimic'''
namesBySequenceIndexNegative[907] = ''''''
semitonesBySequenceIndex[907] = [0, 3, 4, 5, 8, 9]
poemBySequenceIndex[907] = '''B G J K N P'''
poemAuthors[907] = ''''''
#908: ䷘䷽ Integrity Detail: 3 1 1 3 1 3  : S R3 G3 M1 D1 N1 : Agib

#909: C D♯ E F G♯ B♭ :0 3 4 5 8 10: 1 ♯2 3 4 ♯5 ♭7: 11(♯5 ♯9)
namesBySequenceIndex[908] = '''Epogimic'''
namesBySequenceIndexNegative[908] = ''''''
semitonesBySequenceIndex[908] = [0, 3, 4, 5, 8, 10]
poemBySequenceIndex[908] = '''B G J K N R'''
poemAuthors[908] = ''''''
#909: ䷘䷦ Integrity Limping: 3 1 1 3 2 2  : S R3 G3 M1 D1 N2 : Agik

#910: C D♯ E F G♯ B :0 3 4 5 8 11: 1 ♯2 3 4 ♯5 7: ma11(♯5 ♯9)
namesBySequenceIndex[909] = '''Docrimic'''
namesBySequenceIndexNegative[909] = ''''''
semitonesBySequenceIndex[909] = [0, 3, 4, 5, 8, 11]
poemBySequenceIndex[909] = '''B G J K N S'''
poemAuthors[909] = ''''''
#910: ䷘䷳ Integrity Stillness: 3 1 1 3 3 1  : S R3 G3 M1 D1 N3 : Agil

#911: C D♯ E F A B♭ :0 3 4 5 9 10: 1 ♯2 3 4 6 ♭7: 13(♯9 no 5)
namesBySequenceIndex[910] = '''Zogimic'''
namesBySequenceIndexNegative[910] = ''''''
semitonesBySequenceIndex[910] = [0, 3, 4, 5, 9, 10]
poemBySequenceIndex[910] = '''B G J K P R'''
poemAuthors[910] = ''''''
#911: ䷘䷬ Integrity Clustering: 3 1 1 4 1 2  : S R3 G3 M1 D2 N2 : Agad

#912: C D♯ E F A B :0 3 4 5 9 11: 1 ♯2 3 4 6 7: ma13(♯9 no 5)
namesBySequenceIndex[911] = '''Pylimic'''
namesBySequenceIndexNegative[911] = ''''''
semitonesBySequenceIndex[911] = [0, 3, 4, 5, 9, 11]
poemBySequenceIndex[911] = '''B G J K P S'''
poemAuthors[911] = ''''''
#912: ䷘䷢ Integrity Prospering: 3 1 1 4 2 1  : S R3 G3 M1 D2 N3 : Agaf

#913: C D♯ E F A♯ B :0 3 4 5 10 11: 1 ♯2 3 4 ♯6 7: ma11(♯9 no 5 add ♭7)
namesBySequenceIndex[912] = ''''''
namesBySequenceIndexNegative[912] = ''''''
semitonesBySequenceIndex[912] = [0, 3, 4, 5, 10, 11]
poemBySequenceIndex[912] = '''B G J K R S'''
poemAuthors[912] = ''''''
#913: ䷘䷓ Integrity Seeing: 3 1 1 5 1 1  : S R3 G3 M1 D3 N3 : Agam

#914: C D♯ E F♯ G A♭ :0 3 4 6 7 8: 1 ♯2 3 ♯4 5 ♭6: ma(add ♭6 ♯9 ♯11)
namesBySequenceIndex[913] = '''Aeralimic'''
namesBySequenceIndexNegative[913] = ''''''
semitonesBySequenceIndex[913] = [0, 3, 4, 6, 7, 8]
poemBySequenceIndex[913] = '''B G J L M N'''
poemAuthors[913] = ''''''
#914: ䷐䷊ Following Peace: 3 1 2 1 1 4  : S R3 G3 M2 P D1 : AdOj

#915: C D♯ E F♯ G A :0 3 4 6 7 9: 1 ♯2 3 ♯4 5 6: 6(add ♯9 ♯11)
namesBySequenceIndex[914] = '''Stygimic'''
namesBySequenceIndexNegative[914] = ''''''
semitonesBySequenceIndex[914] = [0, 3, 4, 6, 7, 9]
poemBySequenceIndex[914] = '''B G J L M P'''
poemAuthors[914] = ''''''
#915: ䷐䷵ Following Betrothing: 3 1 2 1 2 3  : S R3 G3 M2 P D2 : AdEb

#916: C D♯ E F♯ G B♭ :0 3 4 6 7 10: 1 ♯2 3 ♯4 5 ♭7: 7(add ♯9 ♯11)
namesBySequenceIndex[915] = '''Pygimic'''
namesBySequenceIndexNegative[915] = ''''''
semitonesBySequenceIndex[915] = [0, 3, 4, 6, 7, 10]
poemBySequenceIndex[915] = '''B G J L M R'''
poemAuthors[915] = ''''''
#916: ䷐䷻ Following Composed: 3 1 2 1 3 2  : S R3 G3 M2 P N2 : AdEk

#917: C D♯ E F♯ G B :0 3 4 6 7 11: 1 ♯2 3 ♯4 5 7: ma7(add ♯9 ♯11)
namesBySequenceIndex[916] = '''Raga Rasamanjari 2,Ionophimic'''
namesBySequenceIndexNegative[916] = ''''''
semitonesBySequenceIndex[916] = [0, 3, 4, 6, 7, 11]
poemBySequenceIndex[916] = '''B G J L M S'''
poemAuthors[916] = ''''''
#917: ䷐䷨ Following Diminishing: 3 1 2 1 4 1  : S R3 G3 M2 P N3 : AdEl

#918: C D♯ E F♯ G♯ A :0 3 4 6 8 9: 1 ♯2 3 ♯4 ♯5 6: 6(♯5 add ♯9 ♯11)
namesBySequenceIndex[917] = '''Aeolydimic'''
namesBySequenceIndexNegative[917] = ''''''
semitonesBySequenceIndex[917] = [0, 3, 4, 6, 8, 9]
poemBySequenceIndex[917] = '''B G J L N P'''
poemAuthors[917] = ''''''
#918: ䷐䷶ Following Abounding: 3 1 2 2 1 3  : S R3 G3 M2 D1 N1 : AdIb

#919: C D♯ E F♯ G♯ B♭ :0 3 4 6 8 10: 1 ♯2 3 ♯4 ♯5 ♭7: 7(♯5 add ♯9 ♯11)
namesBySequenceIndex[918] = '''Boptimic'''
namesBySequenceIndexNegative[918] = ''''''
semitonesBySequenceIndex[918] = [0, 3, 4, 6, 8, 10]
poemBySequenceIndex[918] = '''B G J L N R'''
poemAuthors[918] = ''''''
#919: ䷐䷾ Following Completed: 3 1 2 2 2 2  : S R3 G3 M2 D1 N2 : AdIk

#920: C D♯ E F♯ G♯ B :0 3 4 6 8 11: 1 ♯2 3 ♯4 ♯5 7: ma7(♯5 add ♯9 ♯11)
namesBySequenceIndex[919] = '''Zathimic'''
namesBySequenceIndexNegative[919] = ''''''
semitonesBySequenceIndex[919] = [0, 3, 4, 6, 8, 11]
poemBySequenceIndex[919] = '''B G J L N S'''
poemAuthors[919] = ''''''
#920: ䷐䷕ Following Adorning: 3 1 2 2 3 1  : S R3 G3 M2 D1 N3 : AdIl

#921: C D♯ E G♭ A B♭ :0 3 4 6 9 10: 1 ♯2 3 ♭5 6 ♭7: 7(♭5 add ♯9 13)
namesBySequenceIndex[920] = '''Lythimic'''
namesBySequenceIndexNegative[920] = ''''''
semitonesBySequenceIndex[920] = [0, 3, 4, 6, 9, 10]
poemBySequenceIndex[920] = '''B G J L P R'''
poemAuthors[920] = ''''''
#921: ䷐䷐ Following Following : 3 1 2 3 1 2  : S R3 G3 M2 D2 N2 : AdAd

#922: C D♯ E G♭ A B :0 3 4 6 9 11: 1 ♯2 3 ♭5 6 7: ma7(♭5 add ♯9 13)
namesBySequenceIndex[921] = '''Aeolythimic'''
namesBySequenceIndexNegative[921] = ''''''
semitonesBySequenceIndex[921] = [0, 3, 4, 6, 9, 11]
poemBySequenceIndex[921] = '''B G J L P S'''
poemAuthors[921] = ''''''
#922: ䷐䷔ Following Biting: 3 1 2 3 2 1  : S R3 G3 M2 D2 N3 : AdAf

#923: C D♯ E G♭ A♯ B :0 3 4 6 10 11: 1 ♯2 3 ♭5 ♯6 7: ma7(♭5 add ♭7 ♯9)
namesBySequenceIndex[922] = '''Kodimic'''
namesBySequenceIndexNegative[922] = ''''''
semitonesBySequenceIndex[922] = [0, 3, 4, 6, 10, 11]
poemBySequenceIndex[922] = '''B G J L R S'''
poemAuthors[922] = ''''''
#923: ䷐䷩ Following Augmenting: 3 1 2 4 1 1  : S R3 G3 M2 D3 N3 : AdAm

#924: C D♯ E G A♭ A :0 3 4 7 8 9: 1 ♯2 3 5 ♭6 6: 6(add ♭6 ♯9)
namesBySequenceIndex[923] = '''Bogimic'''
namesBySequenceIndexNegative[923] = ''''''
semitonesBySequenceIndex[923] = [0, 3, 4, 7, 8, 9]
poemBySequenceIndex[923] = '''B G J M N P'''
poemAuthors[923] = ''''''
#924: ䷐䷟ Following Persevering: 3 1 3 1 1 3  : S R3 G3 P D1 N1 : Adob

#925: C D♯ E G A♭ B♭ :0 3 4 7 8 10: 1 ♯2 3 5 ♭6 ♭7: 7(add ♯9 ♭13)
namesBySequenceIndex[924] = '''Dynimic'''
namesBySequenceIndexNegative[924] = ''''''
semitonesBySequenceIndex[924] = [0, 3, 4, 7, 8, 10]
poemBySequenceIndex[924] = '''B G J M N R'''
poemAuthors[924] = ''''''
#925: ䷐䷯ Following Source: 3 1 3 1 2 2  : S R3 G3 P D1 N2 : Adok

#926: C D♯ E G A♭ B :0 3 4 7 8 11: 1 ♯2 3 5 ♭6 7: ma7(add ♯9 ♭13)
namesBySequenceIndex[925] = '''Augmented,Major Augmented,Genus Tertium,Raga Devamani,Ionythimic'''
namesBySequenceIndexNegative[925] = ''''''
semitonesBySequenceIndex[925] = [0, 3, 4, 7, 8, 11]
poemBySequenceIndex[925] = '''B G J M N S'''
poemAuthors[925] = ''''''
#926: ䷐䷑ Following Correcting: 3 1 3 1 3 1  : S R3 G3 P D1 N3 : Adol

#927: C D♯ E G A B♭ :0 3 4 7 9 10: 1 ♯2 3 5 6 ♭7: 7(add ♯9 13)
namesBySequenceIndex[926] = '''Bitches Brew,Lorimic'''
namesBySequenceIndexNegative[926] = ''''''
semitonesBySequenceIndex[926] = [0, 3, 4, 7, 9, 10]
poemBySequenceIndex[926] = '''B G J M P R'''
poemAuthors[926] = ''''''
#927: ䷐䷮ Following Confining: 3 1 3 2 1 2  : S R3 G3 P D2 N2 : Aded

#928: C D♯ E G A B :0 3 4 7 9 11: 1 ♯2 3 5 6 7: ma7(add ♯9 13)
namesBySequenceIndex[927] = '''Porimic,Lydian ♯2 Sexatonic'''
namesBySequenceIndexNegative[927] = ''''''
semitonesBySequenceIndex[927] = [0, 3, 4, 7, 9, 11]
poemBySequenceIndex[927] = '''B G J M P S'''
poemAuthors[927] = ''''''
#928: ䷐䷿ Following Unfinished: 3 1 3 2 2 1  : S R3 G3 P D2 N3 : Adef

#929: C D♯ E G A♯ B :0 3 4 7 10 11: 1 ♯2 3 5 ♯6 7: ma7(add ♭7 ♯9)
namesBySequenceIndex[928] = '''Ionalimic'''
namesBySequenceIndexNegative[928] = ''''''
semitonesBySequenceIndex[928] = [0, 3, 4, 7, 10, 11]
poemBySequenceIndex[928] = '''B G J M R S'''
poemAuthors[928] = ''''''
#929: ䷐䷺ Following Dispersing: 3 1 3 3 1 1  : S R3 G3 P D3 N3 : Adem

#930: C D♯ E G♯ A B♭ :0 3 4 8 9 10: 1 ♯2 3 ♯5 6 ♭7: 7(♯5 add ♯9 13)
namesBySequenceIndex[929] = '''Phrythimic'''
namesBySequenceIndexNegative[929] = ''''''
semitonesBySequenceIndex[929] = [0, 3, 4, 8, 9, 10]
poemBySequenceIndex[929] = '''B G J N P R'''
poemAuthors[929] = ''''''
#930: ䷐䷞ Following Conjoining: 3 1 4 1 1 2  : S R3 G3 D1 D2 N2 : Adid

#931: C D♯ E G♯ A B :0 3 4 8 9 11: 1 ♯2 3 ♯5 6 7: ma7(♯5 add ♯9 13)
namesBySequenceIndex[930] = '''Sothimic'''
namesBySequenceIndexNegative[930] = ''''''
semitonesBySequenceIndex[930] = [0, 3, 4, 8, 9, 11]
poemBySequenceIndex[930] = '''B G J N P S'''
poemAuthors[930] = ''''''
#931: ䷐䷷ Following Wanderer: 3 1 4 1 2 1  : S R3 G3 D1 D2 N3 : Adif

#932: C D♯ E G♯ A♯ B :0 3 4 8 10 11: 1 ♯2 3 ♯5 ♯6 7: ma7(♯5 add ♭7 ♯9)
namesBySequenceIndex[931] = '''Weird Chord,Phraptimic'''
namesBySequenceIndexNegative[931] = ''''''
semitonesBySequenceIndex[931] = [0, 3, 4, 8, 10, 11]
poemBySequenceIndex[931] = '''B G J N R S'''
poemAuthors[931] = ''''''
#932: ䷐䷴ Following Infiltrating: 3 1 4 2 1 1  : S R3 G3 D1 N2 N3 : Adim

#933: C D♯ E A B♭ B :0 3 4 9 10 11: 1 ♯2 3 6 ♭7 7: ma7(no 5 add ♭7 ♯9 13)
namesBySequenceIndex[932] = ''''''
namesBySequenceIndexNegative[932] = ''''''
semitonesBySequenceIndex[932] = [0, 3, 4, 9, 10, 11]
poemBySequenceIndex[932] = '''B G J P R S'''
poemAuthors[932] = ''''''
#933: ䷐䷋ Following Obstruction: 3 1 5 1 1 1  : S R3 G3 D2 N2 N3 : Adag

#934: C E♭ F F♯ G A♭ :0 3 5 6 7 8: 1 ♭3 4 ♯4 5 ♭6: mi(add ♭6 11 ♯11)
namesBySequenceIndex[933] = '''Phrathimic'''
namesBySequenceIndexNegative[933] = ''''''
semitonesBySequenceIndex[933] = [0, 3, 5, 6, 7, 8]
poemBySequenceIndex[933] = '''B G K L M N'''
poemAuthors[933] = ''''''
#934: ䷔䷊ Biting Peace: 3 2 1 1 1 4  : S G2 M1 M2 P D1 : AfOj

#935: C E♭ F F♯ G A :0 3 5 6 7 9: 1 ♭3 4 ♯4 5 6: mi6(add 11 ♯11)
namesBySequenceIndex[934] = '''Kolimic'''
namesBySequenceIndexNegative[934] = ''''''
semitonesBySequenceIndex[934] = [0, 3, 5, 6, 7, 9]
poemBySequenceIndex[934] = '''B G K L M P'''
poemAuthors[934] = ''''''
#935: ䷔䷵ Biting Betrothing: 3 2 1 1 2 3  : S G2 M1 M2 P D2 : AfEb

#936: C E♭ F F♯ G B♭ :0 3 5 6 7 10: 1 ♭3 4 ♯4 5 ♭7: mi11(♯9 add ♯11)
namesBySequenceIndex[935] = '''Blues,Raga Nileshwari,Blues Sexatonic,Aeolyphimic'''
namesBySequenceIndexNegative[935] = ''''''
semitonesBySequenceIndex[935] = [0, 3, 5, 6, 7, 10]
poemBySequenceIndex[935] = '''B G K L M R'''
poemAuthors[935] = ''''''
#936: ䷔䷻ Biting Composed: 3 2 1 1 3 2  : S G2 M1 M2 P N2 : AfEk

#937: C E♭ F F♯ G B :0 3 5 6 7 11: 1 ♭3 4 ♯4 5 7: mima11(♯9 add ♯11)
namesBySequenceIndex[936] = '''Rathimic'''
namesBySequenceIndexNegative[936] = ''''''
semitonesBySequenceIndex[936] = [0, 3, 5, 6, 7, 11]
poemBySequenceIndex[936] = '''B G K L M S'''
poemAuthors[936] = ''''''
#937: ䷔䷨ Biting Diminishing: 3 2 1 1 4 1  : S G2 M1 M2 P N3 : AfEl

#938: C E♭ F G♭ A♭ A :0 3 5 6 8 9: 1 ♭3 4 ♭5 ♭6 6: °7(add ♭6 11)
namesBySequenceIndex[937] = '''Bagimic'''
namesBySequenceIndexNegative[937] = ''''''
semitonesBySequenceIndex[937] = [0, 3, 5, 6, 8, 9]
poemBySequenceIndex[937] = '''B G K L N P'''
poemAuthors[937] = ''''''
#938: ䷔䷶ Biting Abounding: 3 2 1 2 1 3  : S G2 M1 M2 D1 N1 : AfIb

#939: C E♭ F G♭ A♭ B♭ :0 3 5 6 8 10: 1 ♭3 4 ♭5 ♭6 ♭7: ⌀11(♯9 add ♭13)
namesBySequenceIndex[938] = '''Phracrimic'''
namesBySequenceIndexNegative[938] = ''''''
semitonesBySequenceIndex[938] = [0, 3, 5, 6, 8, 10]
poemBySequenceIndex[938] = '''B G K L N R'''
poemAuthors[938] = ''''''
#939: ䷔䷾ Biting Completed: 3 2 1 2 2 2  : S G2 M1 M2 D1 N2 : AfIk

#940: C E♭ F G♭ A♭ B :0 3 5 6 8 11: 1 ♭3 4 ♭5 ♭6 7: °ma11(♯9 add ♭13)
namesBySequenceIndex[939] = '''Zacrimic'''
namesBySequenceIndexNegative[939] = ''''''
semitonesBySequenceIndex[939] = [0, 3, 5, 6, 8, 11]
poemBySequenceIndex[939] = '''B G K L N S'''
poemAuthors[939] = ''''''
#940: ䷔䷕ Biting Adorning: 3 2 1 2 3 1  : S G2 M1 M2 D1 N3 : AfIl

#941: C E♭ F G♭ A B♭ :0 3 5 6 9 10: 1 ♭3 4 ♭5 6 ♭7: ⌀13(♯9)
namesBySequenceIndex[940] = '''Bocrimic'''
namesBySequenceIndexNegative[940] = ''''''
semitonesBySequenceIndex[940] = [0, 3, 5, 6, 9, 10]
poemBySequenceIndex[940] = '''B G K L P R'''
poemAuthors[940] = ''''''
#941: ䷔䷐ Biting Following: 3 2 1 3 1 2  : S G2 M1 M2 D2 N2 : AfAd

#942: C E♭ F G♭ A B :0 3 5 6 9 11: 1 ♭3 4 ♭5 6 7: °ma13(♯9)
namesBySequenceIndex[941] = '''Aeradimic'''
namesBySequenceIndexNegative[941] = ''''''
semitonesBySequenceIndex[941] = [0, 3, 5, 6, 9, 11]
poemBySequenceIndex[941] = '''B G K L P S'''
poemAuthors[941] = ''''''
#942: ䷔䷔ Biting Biting : 3 2 1 3 2 1  : S G2 M1 M2 D2 N3 : AfAf

#943: C E♭ F G♭ A♯ B :0 3 5 6 10 11: 1 ♭3 4 ♭5 ♯6 7: °ma11(♯9 add ♭7)
namesBySequenceIndex[942] = '''Rothimic'''
namesBySequenceIndexNegative[942] = ''''''
semitonesBySequenceIndex[942] = [0, 3, 5, 6, 10, 11]
poemBySequenceIndex[942] = '''B G K L R S'''
poemAuthors[942] = ''''''
#943: ䷔䷩ Biting Augmenting: 3 2 1 4 1 1  : S G2 M1 M2 D3 N3 : AfAm

#944: C E♭ F G A♭ A :0 3 5 7 8 9: 1 ♭3 4 5 ♭6 6: mi6(add ♭6 11)
namesBySequenceIndex[943] = '''Stothimic'''
namesBySequenceIndexNegative[943] = ''''''
semitonesBySequenceIndex[943] = [0, 3, 5, 7, 8, 9]
poemBySequenceIndex[943] = '''B G K M N P'''
poemAuthors[943] = ''''''
#944: ䷔䷟ Biting Persevering: 3 2 2 1 1 3  : S G2 M1 P D1 N1 : Afob

#945: C E♭ F G A♭ B♭ :0 3 5 7 8 10: 1 ♭3 4 5 ♭6 ♭7: mi11(♯9 add ♭13)
namesBySequenceIndex[944] = '''Raga Desya Todi,Phrygian Sexatonic,Raga Gopikavasantam,Epathimic'''
namesBySequenceIndexNegative[944] = ''''''
semitonesBySequenceIndex[944] = [0, 3, 5, 7, 8, 10]
poemBySequenceIndex[944] = '''B G K M N R'''
poemAuthors[944] = ''''''
#945: ䷔䷯ Biting Source: 3 2 2 1 2 2  : S G2 M1 P D1 N2 : Afok

#946: C E♭ F G A♭ B :0 3 5 7 8 11: 1 ♭3 4 5 ♭6 7: mima11(♯9 add ♭13)
namesBySequenceIndex[945] = '''Raga Takka,Mothimic'''
namesBySequenceIndexNegative[945] = ''''''
semitonesBySequenceIndex[945] = [0, 3, 5, 7, 8, 11]
poemBySequenceIndex[945] = '''B G K M N S'''
poemAuthors[945] = ''''''
#946: ䷔䷑ Biting Correcting: 3 2 2 1 3 1  : S G2 M1 P D1 N3 : Afol

#947: C E♭ F G A B♭ :0 3 5 7 9 10: 1 ♭3 4 5 6 ♭7: mi13(♯9)
namesBySequenceIndex[946] = '''Raga Manohari,Raga Malavasri,Darmic'''
namesBySequenceIndexNegative[946] = ''''''
semitonesBySequenceIndex[946] = [0, 3, 5, 7, 9, 10]
poemBySequenceIndex[946] = '''B G K M P R'''
poemAuthors[946] = ''''''
#947: ䷔䷮ Biting Confining: 3 2 2 2 1 2  : S G2 M1 P D2 N2 : Afed

#948: C E♭ F G A B :0 3 5 7 9 11: 1 ♭3 4 5 6 7: mima13(♯9)
namesBySequenceIndex[947] = '''Aeragimic'''
namesBySequenceIndexNegative[947] = ''''''
semitonesBySequenceIndex[947] = [0, 3, 5, 7, 9, 11]
poemBySequenceIndex[947] = '''B G K M P S'''
poemAuthors[947] = ''''''
#948: ䷔䷿ Biting Unfinished: 3 2 2 2 2 1  : S G2 M1 P D2 N3 : Afef

#949: C E♭ F G A♯ B :0 3 5 7 10 11: 1 ♭3 4 5 ♯6 7: mima11(♯9 add ♭7)
namesBySequenceIndex[948] = '''Dalimic'''
namesBySequenceIndexNegative[948] = ''''''
semitonesBySequenceIndex[948] = [0, 3, 5, 7, 10, 11]
poemBySequenceIndex[948] = '''B G K M R S'''
poemAuthors[948] = ''''''
#949: ䷔䷺ Biting Dispersing: 3 2 2 3 1 1  : S G2 M1 P D3 N3 : Afem

#950: C E♭ F G♯ A B♭ :0 3 5 8 9 10: 1 ♭3 4 ♯5 6 ♭7: mi13(♯5 ♯9)
namesBySequenceIndex[949] = '''Ionacrimic'''
namesBySequenceIndexNegative[949] = ''''''
semitonesBySequenceIndex[949] = [0, 3, 5, 8, 9, 10]
poemBySequenceIndex[949] = '''B G K N P R'''
poemAuthors[949] = ''''''
#950: ䷔䷞ Biting Conjoining: 3 2 3 1 1 2  : S G2 M1 D1 D2 N2 : Afid

#951: C E♭ F G♯ A B :0 3 5 8 9 11: 1 ♭3 4 ♯5 6 7: mima13(♯5 ♯9)
namesBySequenceIndex[950] = '''Stythimic'''
namesBySequenceIndexNegative[950] = ''''''
semitonesBySequenceIndex[950] = [0, 3, 5, 8, 9, 11]
poemBySequenceIndex[950] = '''B G K N P S'''
poemAuthors[950] = ''''''
#951: ䷔䷷ Biting Wanderer: 3 2 3 1 2 1  : S G2 M1 D1 D2 N3 : Afif

#952: C E♭ F G♯ A♯ B :0 3 5 8 10 11: 1 ♭3 4 ♯5 ♯6 7: mima11(♯5 ♯9 add ♭7)
namesBySequenceIndex[951] = '''Raga Malkauns,Mixolimic'''
namesBySequenceIndexNegative[951] = ''''''
semitonesBySequenceIndex[951] = [0, 3, 5, 8, 10, 11]
poemBySequenceIndex[951] = '''B G K N R S'''
poemAuthors[951] = ''''''
#952: ䷔䷴ Biting Infiltrating: 3 2 3 2 1 1  : S G2 M1 D1 N2 N3 : Afim

#953: C E♭ F A B♭ B :0 3 5 9 10 11: 1 ♭3 4 6 ♭7 7: mima13(♯9 no 5 add ♭7)
namesBySequenceIndex[952] = '''Podimic'''
namesBySequenceIndexNegative[952] = ''''''
semitonesBySequenceIndex[952] = [0, 3, 5, 9, 10, 11]
poemBySequenceIndex[952] = '''B G K P R S'''
poemAuthors[952] = ''''''
#953: ䷔䷋ Biting Obstruction: 3 2 4 1 1 1  : S G2 M1 D2 N2 N3 : Afag

#954: C E♭ F♯ G A♭ A :0 3 6 7 8 9: 1 ♭3 ♯4 5 ♭6 6: mi6(add ♭6 ♯11)
namesBySequenceIndex[953] = '''Ionogimic'''
namesBySequenceIndexNegative[953] = ''''''
semitonesBySequenceIndex[953] = [0, 3, 6, 7, 8, 9]
poemBySequenceIndex[953] = '''B G L M N P'''
poemAuthors[953] = ''''''
#954: ䷲䷡ Thunder Might: 3 3 1 1 1 3  : S G2 M2 P D1 N1 : AbOb

#955: C E♭ F♯ G A♭ B♭ :0 3 6 7 8 10: 1 ♭3 ♯4 5 ♭6 ♭7: mi7(add ♯11 ♭13)
namesBySequenceIndex[954] = '''Zagimic'''
namesBySequenceIndexNegative[954] = ''''''
semitonesBySequenceIndex[954] = [0, 3, 6, 7, 8, 10]
poemBySequenceIndex[954] = '''B G L M N R'''
poemAuthors[954] = ''''''
#955: ䷲䷄ Thunder Biding: 3 3 1 1 2 2  : S G2 M2 P D1 N2 : AbOk

#956: C E♭ F♯ G A♭ B :0 3 6 7 8 11: 1 ♭3 ♯4 5 ♭6 7: mima7(add ♯11 ♭13)
namesBySequenceIndex[955] = '''Mydimic'''
namesBySequenceIndexNegative[955] = ''''''
semitonesBySequenceIndex[955] = [0, 3, 6, 7, 8, 11]
poemBySequenceIndex[955] = '''B G L M N S'''
poemAuthors[955] = ''''''
#956: ䷲䷙ Thunder Cultivating: 3 3 1 1 3 1  : S G2 M2 P D1 N3 : AbOl

#957: C E♭ F♯ G A B♭ :0 3 6 7 9 10: 1 ♭3 ♯4 5 6 ♭7: mi13(♯9 ♯11)
namesBySequenceIndex[956] = '''Raga Madhukauns,Thalimic'''
namesBySequenceIndexNegative[956] = ''''''
semitonesBySequenceIndex[956] = [0, 3, 6, 7, 9, 10]
poemBySequenceIndex[956] = '''B G L M P R'''
poemAuthors[956] = ''''''
#957: ䷲䷹ Thunder Joyous: 3 3 1 2 1 2  : S G2 M2 P D2 N2 : AbEd

#958: C E♭ F♯ G A B :0 3 6 7 9 11: 1 ♭3 ♯4 5 6 7: mima13(♯9 ♯11)
namesBySequenceIndex[957] = '''Dagimic'''
namesBySequenceIndexNegative[957] = ''''''
semitonesBySequenceIndex[957] = [0, 3, 6, 7, 9, 11]
poemBySequenceIndex[957] = '''B G L M P S'''
poemAuthors[957] = ''''''
#958: ䷲䷥ Thunder Polarising: 3 3 1 2 2 1  : S G2 M2 P D2 N3 : AbEf

#959: C E♭ F♯ G A♯ B :0 3 6 7 10 11: 1 ♭3 ♯4 5 ♯6 7: mima7(add ♭7 ♯11)
namesBySequenceIndex[958] = '''Raga Jivantini,Raga Gaurikriya,Ionodimic'''
namesBySequenceIndexNegative[958] = ''''''
semitonesBySequenceIndex[958] = [0, 3, 6, 7, 10, 11]
poemBySequenceIndex[958] = '''B G L M R S'''
poemAuthors[958] = ''''''
#959: ䷲䷼ Thunder Truth: 3 3 1 3 1 1  : S G2 M2 P D3 N3 : AbEm

#960: C E♭ F♯ G♯ A B♭ :0 3 6 8 9 10: 1 ♭3 ♯4 ♯5 6 ♭7: ⌀7(add ♭13 13)
namesBySequenceIndex[959] = '''Thagimic'''
namesBySequenceIndexNegative[959] = ''''''
semitonesBySequenceIndex[959] = [0, 3, 6, 8, 9, 10]
poemBySequenceIndex[959] = '''B G L N P R'''
poemAuthors[959] = ''''''
#960: ䷲䷰ Thunder Revolution: 3 3 2 1 1 2  : S G2 M2 D1 D2 N2 : AbId

#961: C E♭ F♯ G♯ A B :0 3 6 8 9 11: 1 ♭3 ♯4 ♯5 6 7: °ma7(add ♭13 13)
namesBySequenceIndex[960] = '''Thoptimic'''
namesBySequenceIndexNegative[960] = ''''''
semitonesBySequenceIndex[960] = [0, 3, 6, 8, 9, 11]
poemBySequenceIndex[960] = '''B G L N P S'''
poemAuthors[960] = ''''''
#961: ䷲䷝ Thunder Distanced: 3 3 2 1 2 1  : S G2 M2 D1 D2 N3 : AbIf

#962: C E♭ F♯ G♯ A♯ B :0 3 6 8 10 11: 1 ♭3 ♯4 ♯5 ♯6 7: °ma7(add ♭7 ♭13)
namesBySequenceIndex[961] = '''Palimic'''
namesBySequenceIndexNegative[961] = ''''''
semitonesBySequenceIndex[961] = [0, 3, 6, 8, 10, 11]
poemBySequenceIndex[961] = '''B G L N R S'''
poemAuthors[961] = ''''''
#962: ䷲䷤ Thunder Family: 3 3 2 2 1 1  : S G2 M2 D1 N2 N3 : AbIm

#963: C E♭ G♭ A B♭ B :0 3 6 9 10 11: 1 ♭3 ♭5 6 ♭7 7: °ma7(add ♭7 13)
namesBySequenceIndex[962] = '''Epynimic'''
namesBySequenceIndexNegative[962] = ''''''
semitonesBySequenceIndex[962] = [0, 3, 6, 9, 10, 11]
poemBySequenceIndex[962] = '''B G L P R S'''
poemAuthors[962] = ''''''
#963: ䷲䷘ Thunder Integrity: 3 3 3 1 1 1  : S G2 M2 D2 N2 N3 : AbAg

#964: C E♭ G A♭ A B♭ :0 3 7 8 9 10: 1 ♭3 5 ♭6 6 ♭7: mi7(add ♭13 13)
namesBySequenceIndex[963] = '''Aeolycrimic'''
namesBySequenceIndexNegative[963] = ''''''
semitonesBySequenceIndex[963] = [0, 3, 7, 8, 9, 10]
poemBySequenceIndex[963] = '''B G M N P R'''
poemAuthors[963] = ''''''
#964: ䷲䷛ Thunder Excess: 3 4 1 1 1 2  : S G2 P D1 D2 N2 : Abod

#965: C E♭ G A♭ A B :0 3 7 8 9 11: 1 ♭3 5 ♭6 6 7: mima7(add ♭13 13)
namesBySequenceIndex[964] = '''Ionylimic'''
namesBySequenceIndexNegative[964] = ''''''
semitonesBySequenceIndex[964] = [0, 3, 7, 8, 9, 11]
poemBySequenceIndex[964] = '''B G M N P S'''
poemAuthors[964] = ''''''
#965: ䷲䷱ Thunder Holding: 3 4 1 1 2 1  : S G2 P D1 D2 N3 : Abof

#966: C E♭ G A♭ B♭ B :0 3 7 8 10 11: 1 ♭3 5 ♭6 ♭7 7: mima7(add ♭7 ♭13)
namesBySequenceIndex[965] = '''Katathimic'''
namesBySequenceIndexNegative[965] = ''''''
semitonesBySequenceIndex[965] = [0, 3, 7, 8, 10, 11]
poemBySequenceIndex[965] = '''B G M N R S'''
poemAuthors[965] = ''''''
#966: ䷲䷸ Thunder Osmosis: 3 4 1 2 1 1  : S G2 P D1 N2 N3 : Abom

#967: C E♭ G A B♭ B :0 3 7 9 10 11: 1 ♭3 5 6 ♭7 7: mima7(add ♭7 13)
namesBySequenceIndex[966] = '''Phragimic'''
namesBySequenceIndexNegative[966] = ''''''
semitonesBySequenceIndex[966] = [0, 3, 7, 9, 10, 11]
poemBySequenceIndex[966] = '''B G M P R S'''
poemAuthors[966] = ''''''
#967: ䷲䷅ Thunder Arguing: 3 4 2 1 1 1  : S G2 P D2 N2 N3 : Abeg

#968: C E♭ G♯ A B♭ B :0 3 8 9 10 11: 1 ♭3 ♯5 6 ♭7 7: mima7(♯5 add ♭7 13)
namesBySequenceIndex[967] = ''''''
namesBySequenceIndexNegative[967] = ''''''
semitonesBySequenceIndex[967] = [0, 3, 8, 9, 10, 11]
poemBySequenceIndex[967] = '''B G N P R S'''
poemAuthors[967] = ''''''
#968: ䷲䷠ Thunder Retiring: 3 5 1 1 1 1  : S G2 D1 D2 N2 N3 : Abig

#969: C E F F♯ G A♭ :0 4 5 6 7 8: 1 3 4 ♯4 5 ♭6: ma(add ♭6 11 ♯11)
namesBySequenceIndex[968] = '''Kadimic'''
namesBySequenceIndexNegative[968] = ''''''
semitonesBySequenceIndex[968] = [0, 4, 5, 6, 7, 8]
poemBySequenceIndex[968] = '''B J K L M N'''
poemAuthors[968] = ''''''
#969: ䷩䷊ Augmenting Peace: 4 1 1 1 1 4  : S G3 M1 M2 P D1 : AmOj

#970: C E F F♯ G A :0 4 5 6 7 9: 1 3 4 ♯4 5 6: 6(add 11 ♯11)
namesBySequenceIndex[969] = '''Raga Bhatiyar (ascending),Aeronimic'''
namesBySequenceIndexNegative[969] = ''''''
semitonesBySequenceIndex[969] = [0, 4, 5, 6, 7, 9]
poemBySequenceIndex[969] = '''B J K L M P'''
poemAuthors[969] = ''''''
#970: ䷩䷵ Augmenting Betrothing: 4 1 1 1 2 3  : S G3 M1 M2 P D2 : AmEb

#971: C E F F♯ G B♭ :0 4 5 6 7 10: 1 3 4 ♯4 5 ♭7: 7(add 11 ♯11)
namesBySequenceIndex[970] = '''Pynimic'''
namesBySequenceIndexNegative[970] = ''''''
semitonesBySequenceIndex[970] = [0, 4, 5, 6, 7, 10]
poemBySequenceIndex[970] = '''B J K L M R'''
poemAuthors[970] = ''''''
#971: ䷩䷻ Augmenting Composed: 4 1 1 1 3 2  : S G3 M1 M2 P N2 : AmEk

#972: C E F F♯ G B :0 4 5 6 7 11: 1 3 4 ♯4 5 7: ma7(add 11 ♯11)
namesBySequenceIndex[971] = '''Mocrimic'''
namesBySequenceIndexNegative[971] = ''''''
semitonesBySequenceIndex[971] = [0, 4, 5, 6, 7, 11]
poemBySequenceIndex[971] = '''B J K L M S'''
poemAuthors[971] = ''''''
#972: ䷩䷨ Augmenting Diminishing: 4 1 1 1 4 1  : S G3 M1 M2 P N3 : AmEl

#973: C E F G♭ A♭ A :0 4 5 6 8 9: 1 3 4 ♭5 ♭6 6: 6(♯5 add 11 ♯11)
namesBySequenceIndex[972] = '''Aerothimic'''
namesBySequenceIndexNegative[972] = ''''''
semitonesBySequenceIndex[972] = [0, 4, 5, 6, 8, 9]
poemBySequenceIndex[972] = '''B J K L N P'''
poemAuthors[972] = ''''''
#973: ䷩䷶ Augmenting Abounding: 4 1 1 2 1 3  : S G3 M1 M2 D1 N1 : AmIb

#974: C E F G♭ A♭ B♭ :0 4 5 6 8 10: 1 3 4 ♭5 ♭6 ♭7: 7(♯5 add 11 ♯11)
namesBySequenceIndex[973] = '''Mycrimic'''
namesBySequenceIndexNegative[973] = ''''''
semitonesBySequenceIndex[973] = [0, 4, 5, 6, 8, 10]
poemBySequenceIndex[973] = '''B J K L N R'''
poemAuthors[973] = ''''''
#974: ䷩䷾ Augmenting Completed: 4 1 1 2 2 2  : S G3 M1 M2 D1 N2 : AmIk

#975: C E F G♭ A♭ B :0 4 5 6 8 11: 1 3 4 ♭5 ♭6 7: ma7(♯5 add 11 ♯11)
namesBySequenceIndex[974] = '''Raga Gangatarangini,Kanimic'''
namesBySequenceIndexNegative[974] = ''''''
semitonesBySequenceIndex[974] = [0, 4, 5, 6, 8, 11]
poemBySequenceIndex[974] = '''B J K L N S'''
poemAuthors[974] = ''''''
#975: ䷩䷕ Augmenting Adorning: 4 1 1 2 3 1  : S G3 M1 M2 D1 N3 : AmIl

#976: C E F G♭ A B♭ :0 4 5 6 9 10: 1 3 4 ♭5 6 ♭7: 13(♭5 no 9)
namesBySequenceIndex[975] = '''Bolimic'''
namesBySequenceIndexNegative[975] = ''''''
semitonesBySequenceIndex[975] = [0, 4, 5, 6, 9, 10]
poemBySequenceIndex[975] = '''B J K L P R'''
poemAuthors[975] = ''''''
#976: ䷩䷐ Augmenting Following: 4 1 1 3 1 2  : S G3 M1 M2 D2 N2 : AmAd

#977: C E F G♭ A B :0 4 5 6 9 11: 1 3 4 ♭5 6 7: ma13(♭5 no 9)
namesBySequenceIndex[976] = '''Mythimic'''
namesBySequenceIndexNegative[976] = ''''''
semitonesBySequenceIndex[976] = [0, 4, 5, 6, 9, 11]
poemBySequenceIndex[976] = '''B J K L P S'''
poemAuthors[976] = ''''''
#977: ䷩䷔ Augmenting Biting: 4 1 1 3 2 1  : S G3 M1 M2 D2 N3 : AmAf

#978: C E F G♭ A♯ B :0 4 5 6 10 11: 1 3 4 ♭5 ♯6 7: ma7(♭5 add ♭7 11)
namesBySequenceIndex[977] = '''Thonimic'''
namesBySequenceIndexNegative[977] = ''''''
semitonesBySequenceIndex[977] = [0, 4, 5, 6, 10, 11]
poemBySequenceIndex[977] = '''B J K L R S'''
poemAuthors[977] = ''''''
#978: ䷩䷩ Augmenting Augmenting : 4 1 1 4 1 1  : S G3 M1 M2 D3 N3 : AmAm

#979: C E F G A♭ A :0 4 5 7 8 9: 1 3 4 5 ♭6 6: 6(add ♭6 11)
namesBySequenceIndex[978] = '''Raga Saravati,Syrimic'''
namesBySequenceIndexNegative[978] = ''''''
semitonesBySequenceIndex[978] = [0, 4, 5, 7, 8, 9]
poemBySequenceIndex[978] = '''B J K M N P'''
poemAuthors[978] = ''''''
#979: ䷩䷟ Augmenting Persevering: 4 1 2 1 1 3  : S G3 M1 P D1 N1 : Amob

#980: C E F G A♭ B♭ :0 4 5 7 8 10: 1 3 4 5 ♭6 ♭7: 7(add 11 ♭13)
namesBySequenceIndex[979] = '''Raga Kamalamanohari,Modimic'''
namesBySequenceIndexNegative[979] = ''''''
semitonesBySequenceIndex[979] = [0, 4, 5, 7, 8, 10]
poemBySequenceIndex[979] = '''B J K M N R'''
poemAuthors[979] = ''''''
#980: ䷩䷯ Augmenting Source: 4 1 2 1 2 2  : S G3 M1 P D1 N2 : Amok

#981: C E F G A♭ B :0 4 5 7 8 11: 1 3 4 5 ♭6 7: ma7(add 11 ♭13)
namesBySequenceIndex[980] = '''Raga Paras (ascending),Laptimic'''
namesBySequenceIndexNegative[980] = ''''''
semitonesBySequenceIndex[980] = [0, 4, 5, 7, 8, 11]
poemBySequenceIndex[980] = '''B J K M N S'''
poemAuthors[980] = ''''''
#981: ䷩䷑ Augmenting Correcting: 4 1 2 1 3 1  : S G3 M1 P D1 N3 : Amol

#982: C E F G A B♭ :0 4 5 7 9 10: 1 3 4 5 6 ♭7: 13(no 9)
namesBySequenceIndex[981] = '''Raga Vegavahini (ascending),Raga Bahudari,Garimic'''
namesBySequenceIndexNegative[981] = ''''''
semitonesBySequenceIndex[981] = [0, 4, 5, 7, 9, 10]
poemBySequenceIndex[981] = '''B J K M P R'''
poemAuthors[981] = ''''''
#982: ䷩䷮ Augmenting Confining: 4 1 2 2 1 2  : S G3 M1 P D2 N2 : Amed

#983: C E F G A B :0 4 5 7 9 11: 1 3 4 5 6 7: ma13(no 9)
namesBySequenceIndex[982] = '''Raga Hari Nata,Genus Secundum,Bylimic'''
namesBySequenceIndexNegative[982] = ''''''
semitonesBySequenceIndex[982] = [0, 4, 5, 7, 9, 11]
poemBySequenceIndex[982] = '''B J K M P S'''
poemAuthors[982] = ''''''
#983: ䷩䷿ Augmenting Unfinished: 4 1 2 2 2 1  : S G3 M1 P D2 N3 : Amef

#984: C E F G A♯ B :0 4 5 7 10 11: 1 3 4 5 ♯6 7: ma7(add ♭7 11)
namesBySequenceIndex[983] = '''Raga Tilang,Raga Bridabani Tilang,Raga Savitri,Epygimic'''
namesBySequenceIndexNegative[983] = ''''''
semitonesBySequenceIndex[983] = [0, 4, 5, 7, 10, 11]
poemBySequenceIndex[983] = '''B J K M R S'''
poemAuthors[983] = ''''''
#984: ䷩䷺ Augmenting Dispersing: 4 1 2 3 1 1  : S G3 M1 P D3 N3 : Amem

#985: C E F G♯ A B♭ :0 4 5 8 9 10: 1 3 4 ♯5 6 ♭7: 13(♯5 no 9)
namesBySequenceIndex[984] = '''Thogimic'''
namesBySequenceIndexNegative[984] = ''''''
semitonesBySequenceIndex[984] = [0, 4, 5, 8, 9, 10]
poemBySequenceIndex[984] = '''B J K N P R'''
poemAuthors[984] = ''''''
#985: ䷩䷞ Augmenting Conjoining: 4 1 3 1 1 2  : S G3 M1 D1 D2 N2 : Amid

#986: C E F G♯ A B :0 4 5 8 9 11: 1 3 4 ♯5 6 7: ma13(♯5 no 9)
namesBySequenceIndex[985] = '''Solimic'''
namesBySequenceIndexNegative[985] = ''''''
semitonesBySequenceIndex[985] = [0, 4, 5, 8, 9, 11]
poemBySequenceIndex[985] = '''B J K N P S'''
poemAuthors[985] = ''''''
#986: ䷩䷷ Augmenting Wanderer: 4 1 3 1 2 1  : S G3 M1 D1 D2 N3 : Amif

#987: C E F G♯ A♯ B :0 4 5 8 10 11: 1 3 4 ♯5 ♯6 7: ma7(♯5 add ♭7 11)
namesBySequenceIndex[986] = '''Phralimic'''
namesBySequenceIndexNegative[986] = ''''''
semitonesBySequenceIndex[986] = [0, 4, 5, 8, 10, 11]
poemBySequenceIndex[986] = '''B J K N R S'''
poemAuthors[986] = ''''''
#987: ䷩䷴ Augmenting Infiltrating: 4 1 3 2 1 1  : S G3 M1 D1 N2 N3 : Amim

#988: C E F A B♭ B :0 4 5 9 10 11: 1 3 4 6 ♭7 7: ma13(no 5 9 add ♭7)
namesBySequenceIndex[987] = '''Daptimic'''
namesBySequenceIndexNegative[987] = ''''''
semitonesBySequenceIndex[987] = [0, 4, 5, 9, 10, 11]
poemBySequenceIndex[987] = '''B J K P R S'''
poemAuthors[987] = ''''''
#988: ䷩䷋ Augmenting Obstruction: 4 1 4 1 1 1  : S G3 M1 D2 N2 N3 : Amag

#989: C E F♯ G A♭ A :0 4 6 7 8 9: 1 3 ♯4 5 ♭6 6: 6(add ♭6 ♯11)
namesBySequenceIndex[988] = '''Kocrimic'''
namesBySequenceIndexNegative[988] = ''''''
semitonesBySequenceIndex[988] = [0, 4, 6, 7, 8, 9]
poemBySequenceIndex[988] = '''B J L M N P'''
poemAuthors[988] = ''''''
#989: ䷂䷡ Sprouting Might: 4 2 1 1 1 3  : S G3 M2 P D1 N1 : AkOb

#990: C E F♯ G A♭ B♭ :0 4 6 7 8 10: 1 3 ♯4 5 ♭6 ♭7: 7(add ♯11 ♭13)
namesBySequenceIndex[989] = '''Raga Jyoti,Gacrimic'''
namesBySequenceIndexNegative[989] = ''''''
semitonesBySequenceIndex[989] = [0, 4, 6, 7, 8, 10]
poemBySequenceIndex[989] = '''B J L M N R'''
poemAuthors[989] = ''''''
#990: ䷂䷄ Sprouting Biding: 4 2 1 1 2 2  : S G3 M2 P D1 N2 : AkOk

#991: C E F♯ G A♭ B :0 4 6 7 8 11: 1 3 ♯4 5 ♭6 7: ma7(add ♯11 ♭13)
namesBySequenceIndex[990] = '''Aerycrimic'''
namesBySequenceIndexNegative[990] = ''''''
semitonesBySequenceIndex[990] = [0, 4, 6, 7, 8, 11]
poemBySequenceIndex[990] = '''B J L M N S'''
poemAuthors[990] = ''''''
#991: ䷂䷙ Sprouting Cultivating: 4 2 1 1 3 1  : S G3 M2 P D1 N3 : AkOl

#992: C E F♯ G A B♭ :0 4 6 7 9 10: 1 3 ♯4 5 6 ♭7: 7(add ♯11 13)
namesBySequenceIndex[991] = '''Raga Vutari,Raga Zilaf,Manimic'''
namesBySequenceIndexNegative[991] = ''''''
semitonesBySequenceIndex[991] = [0, 4, 6, 7, 9, 10]
poemBySequenceIndex[991] = '''B J L M P R'''
poemAuthors[991] = ''''''
#992: ䷂䷹ Sprouting Joyous: 4 2 1 2 1 2  : S G3 M2 P D2 N2 : AkEd

#993: C E F♯ G A B :0 4 6 7 9 11: 1 3 ♯4 5 6 7: ma7(add ♯11 13)
namesBySequenceIndex[992] = '''Dyrimic'''
namesBySequenceIndexNegative[992] = ''''''
semitonesBySequenceIndex[992] = [0, 4, 6, 7, 9, 11]
poemBySequenceIndex[992] = '''B J L M P S'''
poemAuthors[992] = ''''''
#993: ䷂䷥ Sprouting Polarising: 4 2 1 2 2 1  : S G3 M2 P D2 N3 : AkEf

#994: C E F♯ G A♯ B :0 4 6 7 10 11: 1 3 ♯4 5 ♯6 7: ma7(add ♭7 ♯11)
namesBySequenceIndex[993] = '''Raga Vijayavasanta,Katythimic'''
namesBySequenceIndexNegative[993] = ''''''
semitonesBySequenceIndex[993] = [0, 4, 6, 7, 10, 11]
poemBySequenceIndex[993] = '''B J L M R S'''
poemAuthors[993] = ''''''
#994: ䷂䷼ Sprouting Truth: 4 2 1 3 1 1  : S G3 M2 P D3 N3 : AkEm

#995: C E F♯ G♯ A B♭ :0 4 6 8 9 10: 1 3 ♯4 ♯5 6 ♭7: 7(♯5 add ♯11 13)
namesBySequenceIndex[994] = '''Dathimic'''
namesBySequenceIndexNegative[994] = ''''''
semitonesBySequenceIndex[994] = [0, 4, 6, 8, 9, 10]
poemBySequenceIndex[994] = '''B J L N P R'''
poemAuthors[994] = ''''''
#995: ䷂䷰ Sprouting Revolution: 4 2 2 1 1 2  : S G3 M2 D1 D2 N2 : AkId

#996: C E F♯ G♯ A B :0 4 6 8 9 11: 1 3 ♯4 ♯5 6 7: ma7(♯5 add ♯11 13)
namesBySequenceIndex[995] = '''Rycrimic'''
namesBySequenceIndexNegative[995] = ''''''
semitonesBySequenceIndex[995] = [0, 4, 6, 8, 9, 11]
poemBySequenceIndex[995] = '''B J L N P S'''
poemAuthors[995] = ''''''
#996: ䷂䷝ Sprouting Distanced: 4 2 2 1 2 1  : S G3 M2 D1 D2 N3 : AkIf

#997: C E F♯ G♯ A♯ B :0 4 6 8 10 11: 1 3 ♯4 ♯5 ♯6 7: ma7(♯5 add ♭7 ♯11)
namesBySequenceIndex[996] = '''Katanimic'''
namesBySequenceIndexNegative[996] = ''''''
semitonesBySequenceIndex[996] = [0, 4, 6, 8, 10, 11]
poemBySequenceIndex[996] = '''B J L N R S'''
poemAuthors[996] = ''''''
#997: ䷂䷤ Sprouting Family: 4 2 2 2 1 1  : S G3 M2 D1 N2 N3 : AkIm

#998: C E G♭ A B♭ B :0 4 6 9 10 11: 1 3 ♭5 6 ♭7 7: ma7(♭5 add ♭7 13)
namesBySequenceIndex[997] = '''Stalimic'''
namesBySequenceIndexNegative[997] = ''''''
semitonesBySequenceIndex[997] = [0, 4, 6, 9, 10, 11]
poemBySequenceIndex[997] = '''B J L P R S'''
poemAuthors[997] = ''''''
#998: ䷂䷘ Sprouting Integrity: 4 2 3 1 1 1  : S G3 M2 D2 N2 N3 : AkAg

#999: C E G A♭ A B♭ :0 4 7 8 9 10: 1 3 5 ♭6 6 ♭7: 7(add ♭13 13)
namesBySequenceIndex[998] = '''Galimic'''
namesBySequenceIndexNegative[998] = ''''''
semitonesBySequenceIndex[998] = [0, 4, 7, 8, 9, 10]
poemBySequenceIndex[998] = '''B J M N P R'''
poemAuthors[998] = ''''''
#999: ䷂䷛ Sprouting Excess: 4 3 1 1 1 2  : S G3 P D1 D2 N2 : Akod

#1000: C E G A♭ A B :0 4 7 8 9 11: 1 3 5 ♭6 6 7: ma7(add ♭13 13)
namesBySequenceIndex[999] = '''Bygimic'''
namesBySequenceIndexNegative[999] = ''''''
semitonesBySequenceIndex[999] = [0, 4, 7, 8, 9, 11]
poemBySequenceIndex[999] = '''B J M N P S'''
poemAuthors[999] = ''''''
#1000: ䷂䷱ Sprouting Holding: 4 3 1 1 2 1  : S G3 P D1 D2 N3 : Akof

#1001: C E G A♭ B♭ B :0 4 7 8 10 11: 1 3 5 ♭6 ♭7 7: ma7(add ♭7 ♭13)
namesBySequenceIndex[1000] = '''Lathimic'''
namesBySequenceIndexNegative[1000] = ''''''
semitonesBySequenceIndex[1000] = [0, 4, 7, 8, 10, 11]
poemBySequenceIndex[1000] = '''B J M N R S'''
poemAuthors[1000] = ''''''
#1001: ䷂䷸ Sprouting Osmosis: 4 3 1 2 1 1  : S G3 P D1 N2 N3 : Akom

#1002: C E G A B♭ B :0 4 7 9 10 11: 1 3 5 6 ♭7 7: ma7(add ♭7 13)
namesBySequenceIndex[1001] = '''Starimic'''
namesBySequenceIndexNegative[1001] = ''''''
semitonesBySequenceIndex[1001] = [0, 4, 7, 9, 10, 11]
poemBySequenceIndex[1001] = '''B J M P R S'''
poemAuthors[1001] = ''''''
#1002: ䷂䷅ Sprouting Arguing: 4 3 2 1 1 1  : S G3 P D2 N2 N3 : Akeg

#1003: C E G♯ A B♭ B :0 4 8 9 10 11: 1 3 ♯5 6 ♭7 7: ma7(♯5 add ♭7 13)
namesBySequenceIndex[1002] = '''Ponimic'''
namesBySequenceIndexNegative[1002] = ''''''
semitonesBySequenceIndex[1002] = [0, 4, 8, 9, 10, 11]
poemBySequenceIndex[1002] = '''B J N P R S'''
poemAuthors[1002] = ''''''
#1003: ䷂䷠ Sprouting Retiring: 4 4 1 1 1 1  : S G3 D1 D2 N2 N3 : Akig

#1004: C F F♯ G A♭ A :0 5 6 7 8 9: 1 4 ♯4 5 ♭6 6: 6sus4(add ♭6 ♯11)
namesBySequenceIndex[1003] = ''''''
namesBySequenceIndexNegative[1003] = ''''''
semitonesBySequenceIndex[1003] = [0, 5, 6, 7, 8, 9]
poemBySequenceIndex[1003] = '''B K L M N P'''
poemAuthors[1003] = ''''''
#1004: ䷚䷡ Nourishment Might: 5 1 1 1 1 3  : S M1 M2 P D1 N1 : AlOb

#1005: C F F♯ G A♭ B♭ :0 5 6 7 8 10: 1 4 ♯4 5 ♭6 ♭7: 7sus4(add ♯11 ♭13)
namesBySequenceIndex[1004] = ''''''
namesBySequenceIndexNegative[1004] = ''''''
semitonesBySequenceIndex[1004] = [0, 5, 6, 7, 8, 10]
poemBySequenceIndex[1004] = '''B K L M N R'''
poemAuthors[1004] = ''''''
#1005: ䷚䷄ Nourishment Biding: 5 1 1 1 2 2  : S M1 M2 P D1 N2 : AlOk

#1006: C F F♯ G A♭ B :0 5 6 7 8 11: 1 4 ♯4 5 ♭6 7: ma7sus4(add ♯11 ♭13)
namesBySequenceIndex[1005] = ''''''
namesBySequenceIndexNegative[1005] = ''''''
semitonesBySequenceIndex[1005] = [0, 5, 6, 7, 8, 11]
poemBySequenceIndex[1005] = '''B K L M N S'''
poemAuthors[1005] = ''''''
#1006: ䷚䷙ Nourishment Cultivating: 5 1 1 1 3 1  : S M1 M2 P D1 N3 : AlOl

#1007: C F F♯ G A B♭ :0 5 6 7 9 10: 1 4 ♯4 5 6 ♭7: 13sus♯4(no 9)
namesBySequenceIndex[1006] = ''''''
namesBySequenceIndexNegative[1006] = ''''''
semitonesBySequenceIndex[1006] = [0, 5, 6, 7, 9, 10]
poemBySequenceIndex[1006] = '''B K L M P R'''
poemAuthors[1006] = ''''''
#1007: ䷚䷹ Nourishment Joyous: 5 1 1 2 1 2  : S M1 M2 P D2 N2 : AlEd

#1008: C F F♯ G A B :0 5 6 7 9 11: 1 4 ♯4 5 6 7: ma13sus♯4(no 9)
namesBySequenceIndex[1007] = ''''''
namesBySequenceIndexNegative[1007] = ''''''
semitonesBySequenceIndex[1007] = [0, 5, 6, 7, 9, 11]
poemBySequenceIndex[1007] = '''B K L M P S'''
poemAuthors[1007] = ''''''
#1008: ䷚䷥ Nourishment Polarising: 5 1 1 2 2 1  : S M1 M2 P D2 N3 : AlEf

#1009: C F F♯ G A♯ B :0 5 6 7 10 11: 1 4 ♯4 5 ♯6 7: ma7sus4(add ♭7 ♯11)
namesBySequenceIndex[1008] = ''''''
namesBySequenceIndexNegative[1008] = ''''''
semitonesBySequenceIndex[1008] = [0, 5, 6, 7, 10, 11]
poemBySequenceIndex[1008] = '''B K L M R S'''
poemAuthors[1008] = ''''''
#1009: ䷚䷼ Nourishment Truth: 5 1 1 3 1 1  : S M1 M2 P D3 N3 : AlEm

#1010: C F G♭ A♭ A B♭ :0 5 6 8 9 10: 1 4 ♭5 ♭6 6 ♭7: 13sus♯4(♯5 no 9)
namesBySequenceIndex[1009] = ''''''
namesBySequenceIndexNegative[1009] = ''''''
semitonesBySequenceIndex[1009] = [0, 5, 6, 8, 9, 10]
poemBySequenceIndex[1009] = '''B K L N P R'''
poemAuthors[1009] = ''''''
#1010: ䷚䷰ Nourishment Revolution: 5 1 2 1 1 2  : S M1 M2 D1 D2 N2 : AlId

#1011: C F G♭ A♭ A B :0 5 6 8 9 11: 1 4 ♭5 ♭6 6 7: ma13sus♯4(♯5 no 9)
namesBySequenceIndex[1010] = ''''''
namesBySequenceIndexNegative[1010] = ''''''
semitonesBySequenceIndex[1010] = [0, 5, 6, 8, 9, 11]
poemBySequenceIndex[1010] = '''B K L N P S'''
poemAuthors[1010] = ''''''
#1011: ䷚䷝ Nourishment Distanced: 5 1 2 1 2 1  : S M1 M2 D1 D2 N3 : AlIf

#1012: C F G♭ A♭ B♭ B :0 5 6 8 10 11: 1 4 ♭5 ♭6 ♭7 7: ma7sus4(♯5 add ♭7 ♯11)
namesBySequenceIndex[1011] = ''''''
namesBySequenceIndexNegative[1011] = ''''''
semitonesBySequenceIndex[1011] = [0, 5, 6, 8, 10, 11]
poemBySequenceIndex[1011] = '''B K L N R S'''
poemAuthors[1011] = ''''''
#1012: ䷚䷤ Nourishment Family: 5 1 2 2 1 1  : S M1 M2 D1 N2 N3 : AlIm

#1013: C F G♭ A B♭ B :0 5 6 9 10 11: 1 4 ♭5 6 ♭7 7: ma7sus4(♭5 add ♭7 13)
namesBySequenceIndex[1012] = ''''''
namesBySequenceIndexNegative[1012] = ''''''
semitonesBySequenceIndex[1012] = [0, 5, 6, 9, 10, 11]
poemBySequenceIndex[1012] = '''B K L P R S'''
poemAuthors[1012] = ''''''
#1013: ䷚䷘ Nourishment Integrity: 5 1 3 1 1 1  : S M1 M2 D2 N2 N3 : AlAg

#1014: C F G A♭ A B♭ :0 5 7 8 9 10: 1 4 5 ♭6 6 ♭7: 7sus4(add ♭13 13)
namesBySequenceIndex[1013] = ''''''
namesBySequenceIndexNegative[1013] = ''''''
semitonesBySequenceIndex[1013] = [0, 5, 7, 8, 9, 10]
poemBySequenceIndex[1013] = '''B K M N P R'''
poemAuthors[1013] = ''''''
#1014: ䷚䷛ Nourishment Excess: 5 2 1 1 1 2  : S M1 P D1 D2 N2 : Alod

#1015: C F G A♭ A B :0 5 7 8 9 11: 1 4 5 ♭6 6 7: ma7sus4(add ♭13 13)
namesBySequenceIndex[1014] = ''''''
namesBySequenceIndexNegative[1014] = ''''''
semitonesBySequenceIndex[1014] = [0, 5, 7, 8, 9, 11]
poemBySequenceIndex[1014] = '''B K M N P S'''
poemAuthors[1014] = ''''''
#1015: ䷚䷱ Nourishment Holding: 5 2 1 1 2 1  : S M1 P D1 D2 N3 : Alof

#1016: C F G A♭ B♭ B :0 5 7 8 10 11: 1 4 5 ♭6 ♭7 7: ma7sus4(add ♭7 ♭13)
namesBySequenceIndex[1015] = ''''''
namesBySequenceIndexNegative[1015] = ''''''
semitonesBySequenceIndex[1015] = [0, 5, 7, 8, 10, 11]
poemBySequenceIndex[1015] = '''B K M N R S'''
poemAuthors[1015] = ''''''
#1016: ䷚䷸ Nourishment Osmosis: 5 2 1 2 1 1  : S M1 P D1 N2 N3 : Alom

#1017: C F G A B♭ B :0 5 7 9 10 11: 1 4 5 6 ♭7 7: ma7sus4(add ♭7 13)
namesBySequenceIndex[1016] = ''''''
namesBySequenceIndexNegative[1016] = ''''''
semitonesBySequenceIndex[1016] = [0, 5, 7, 9, 10, 11]
poemBySequenceIndex[1016] = '''B K M P R S'''
poemAuthors[1016] = ''''''
#1017: ䷚䷅ Nourishment Arguing: 5 2 2 1 1 1  : S M1 P D2 N2 N3 : Aleg

#1018: C F G♯ A B♭ B :0 5 8 9 10 11: 1 4 ♯5 6 ♭7 7: ma7sus4(♯5 add ♭7 13)
namesBySequenceIndex[1017] = ''''''
namesBySequenceIndexNegative[1017] = ''''''
semitonesBySequenceIndex[1017] = [0, 5, 8, 9, 10, 11]
poemBySequenceIndex[1017] = '''B K N P R S'''
poemAuthors[1017] = ''''''
#1018: ䷚䷠ Nourishment Retiring: 5 3 1 1 1 1  : S M1 D1 D2 N2 N3 : Alig

#1019: C F♯ G A♭ A B♭ :0 6 7 8 9 10: 1 ♯4 5 ♭6 6 ♭7: 7sus♯4(add ♭13 13)
namesBySequenceIndex[1018] = ''''''
namesBySequenceIndexNegative[1018] = ''''''
semitonesBySequenceIndex[1018] = [0, 6, 7, 8, 9, 10]
poemBySequenceIndex[1018] = '''B L M N P R'''
poemAuthors[1018] = ''''''
#1019: ䷗䷪ Returning Decision: 6 1 1 1 1 2  : S M2 P D1 D2 N2 : AjOd

#1020: C F♯ G A♭ A B :0 6 7 8 9 11: 1 ♯4 5 ♭6 6 7: ma7sus♯4(add ♭13 13)
namesBySequenceIndex[1019] = ''''''
namesBySequenceIndexNegative[1019] = ''''''
semitonesBySequenceIndex[1019] = [0, 6, 7, 8, 9, 11]
poemBySequenceIndex[1019] = '''B L M N P S'''
poemAuthors[1019] = ''''''
#1020: ䷗䷍ Returning Resevoir: 6 1 1 1 2 1  : S M2 P D1 D2 N3 : AjOf

#1021: C F♯ G A♭ B♭ B :0 6 7 8 10 11: 1 ♯4 5 ♭6 ♭7 7: ma7sus♯4(add ♭7 ♭13)
namesBySequenceIndex[1020] = ''''''
namesBySequenceIndexNegative[1020] = ''''''
semitonesBySequenceIndex[1020] = [0, 6, 7, 8, 10, 11]
poemBySequenceIndex[1020] = '''B L M N R S'''
poemAuthors[1020] = ''''''
#1021: ䷗䷈ Returning Homegrown: 6 1 1 2 1 1  : S M2 P D1 N2 N3 : AjOm

#1022: C F♯ G A B♭ B :0 6 7 9 10 11: 1 ♯4 5 6 ♭7 7: ma7sus♯4(add ♭7 13)
namesBySequenceIndex[1021] = ''''''
namesBySequenceIndexNegative[1021] = ''''''
semitonesBySequenceIndex[1021] = [0, 6, 7, 9, 10, 11]
poemBySequenceIndex[1021] = '''B L M P R S'''
poemAuthors[1021] = ''''''
#1022: ䷗䷉ Returning Treading: 6 1 2 1 1 1  : S M2 P D2 N2 N3 : AjEg

#1023: C F♯ G♯ A B♭ B :0 6 8 9 10 11: 1 ♯4 ♯5 6 ♭7 7: ma7sus♯4(♯5 add ♭7 13)
namesBySequenceIndex[1022] = ''''''
namesBySequenceIndexNegative[1022] = ''''''
semitonesBySequenceIndex[1022] = [0, 6, 8, 9, 10, 11]
poemBySequenceIndex[1022] = '''B L N P R S'''
poemAuthors[1022] = ''''''
#1023: ䷗䷌ Returning Fellowship: 6 2 1 1 1 1  : S M2 D1 D2 N2 N3 : AjIg

#1024: C G A♭ A B♭ B :0 7 8 9 10 11: 1 5 ♭6 6 ♭7 7: 5(add ♭6 6 ♭7 7)
namesBySequenceIndex[1023] = ''''''
namesBySequenceIndexNegative[1023] = ''''''
semitonesBySequenceIndex[1023] = [0, 7, 8, 9, 10, 11]
poemBySequenceIndex[1023] = '''B M N P R S'''
poemAuthors[1023] = ''''''
#1024: ䷗䷫ Returning Coupling: 7 1 1 1 1 1  : S P D1 D2 N2 N3 : Ajog

#1025: C D♭ D E♭ E F G♭ :0 1 2 3 4 5 6: 1 ♭2 2 ♭3 3 4 ♭5: 11(♭5 no 7 add ♭9 ♯9)
namesBySequenceIndex[1024] = '''Chromatic Heptachord'''
namesBySequenceIndexNegative[1024] = ''''''
semitonesBySequenceIndex[1024] = [0, 1, 2, 3, 4, 5, 6]
poemBySequenceIndex[1024] = '''B D F G J K L'''
poemAuthors[1024] = ''''''
#1025: ䷀䷗ Creative Returning: 1 1 1 1 1 1 6  : S R1 R2 G2 G3 M1 M2 : OgAj

#1026: C D♭ D E♭ E F G :0 1 2 3 4 5 7: 1 ♭2 2 ♭3 3 4 5: 11(no 7 add ♭9 ♯9)
namesBySequenceIndex[1025] = ''''''
namesBySequenceIndexNegative[1025] = ''''''
semitonesBySequenceIndex[1025] = [0, 1, 2, 3, 4, 5, 7]
poemBySequenceIndex[1025] = '''B D F G J K M'''
poemAuthors[1025] = ''''''
#1026: ䷀䷆ Creative Leading: 1 1 1 1 1 2 5  : S R1 R2 G2 G3 M1 P : Ogej

#1027: C D♭ D E♭ E F G♯ :0 1 2 3 4 5 8: 1 ♭2 2 ♭3 3 4 ♯5: 11(♯5 no 7 add ♭9 ♯9)
namesBySequenceIndex[1026] = '''Epodian'''
namesBySequenceIndexNegative[1026] = ''''''
semitonesBySequenceIndex[1026] = [0, 1, 2, 3, 4, 5, 8]
poemBySequenceIndex[1026] = '''B D F G J K N'''
poemAuthors[1026] = ''''''
#1027: ䷀䷎ Creative Humble: 1 1 1 1 1 3 4  : S R1 R2 G2 G3 M1 D1 : Ogij

#1028: C D♭ D E♭ E F A :0 1 2 3 4 5 9: 1 ♭2 2 ♭3 3 4 6: 13(no 5 7 add ♭9 ♯9)
namesBySequenceIndex[1027] = '''Ionydian'''
namesBySequenceIndexNegative[1027] = ''''''
semitonesBySequenceIndex[1027] = [0, 1, 2, 3, 4, 5, 9]
poemBySequenceIndex[1027] = '''B D F G J K P'''
poemAuthors[1027] = ''''''
#1028: ䷀䷏ Creative Confident: 1 1 1 1 1 4 3  : S R1 R2 G2 G3 M1 D2 : Ogab

#1029: C D♭ D E♭ E F B♭ :0 1 2 3 4 5 10: 1 ♭2 2 ♭3 3 4 ♭7: 11(no 5 add ♭9 ♯9)
namesBySequenceIndex[1028] = ''''''
namesBySequenceIndexNegative[1028] = ''''''
semitonesBySequenceIndex[1028] = [0, 1, 2, 3, 4, 5, 10]
poemBySequenceIndex[1028] = '''B D F G J K R'''
poemAuthors[1028] = ''''''
#1029: ䷀䷇ Creative Comparing: 1 1 1 1 1 5 2  : S R1 R2 G2 G3 M1 N2 : Ogak

#1030: C D♭ D E♭ E F B :0 1 2 3 4 5 11: 1 ♭2 2 ♭3 3 4 7: ma11(no 5 add ♭9 ♯9)
namesBySequenceIndex[1029] = ''''''
namesBySequenceIndexNegative[1029] = ''''''
semitonesBySequenceIndex[1029] = [0, 1, 2, 3, 4, 5, 11]
poemBySequenceIndex[1029] = '''B D F G J K S'''
poemAuthors[1029] = ''''''
#1030: ䷀䷖ Creative Stripping: 1 1 1 1 1 6 1  : S R1 R2 G2 G3 M1 N3 : Ogal

#1031: C D♭ D E♭ E F♯ G :0 1 2 3 4 6 7: 1 ♭2 2 ♭3 3 ♯4 5: ma(add ♭9 9 ♯9 ♯11)
namesBySequenceIndex[1030] = ''''''
namesBySequenceIndexNegative[1030] = ''''''
semitonesBySequenceIndex[1030] = [0, 1, 2, 3, 4, 6, 7]
poemBySequenceIndex[1030] = '''B D F G J L M'''
poemAuthors[1030] = ''''''
#1031: ䷪䷒ Decision Arrived: 1 1 1 1 2 1 5  : S R1 R2 G2 G3 M2 P : OdEj

#1032: C D♭ D E♭ E F♯ G♯ :0 1 2 3 4 6 8: 1 ♭2 2 ♭3 3 ♯4 ♯5: ma(♯5 add ♭9 9 ♯9 ♯11)
namesBySequenceIndex[1031] = '''Epanian'''
namesBySequenceIndexNegative[1031] = ''''''
semitonesBySequenceIndex[1031] = [0, 1, 2, 3, 4, 6, 8]
poemBySequenceIndex[1031] = '''B D F G J L N'''
poemAuthors[1031] = ''''''
#1032: ䷪䷣ Decision Eclipse: 1 1 1 1 2 2 4  : S R1 R2 G2 G3 M2 D1 : OdIj

#1033: C D♭ D E♭ E G♭ A :0 1 2 3 4 6 9: 1 ♭2 2 ♭3 3 ♭5 6: 6/9(♭5 add ♭9 ♯9)
namesBySequenceIndex[1032] = '''Kadian'''
namesBySequenceIndexNegative[1032] = ''''''
semitonesBySequenceIndex[1032] = [0, 1, 2, 3, 4, 6, 9]
poemBySequenceIndex[1032] = '''B D F G J L P'''
poemAuthors[1032] = ''''''
#1033: ䷪䷲ Decision Thunder: 1 1 1 1 2 3 3  : S R1 R2 G2 G3 M2 D2 : OdAb

#1034: C D♭ D E♭ E G♭ B♭ :0 1 2 3 4 6 10: 1 ♭2 2 ♭3 3 ♭5 ♭7: 9(♭5 add ♭9 ♯9)
namesBySequenceIndex[1033] = '''Rarian'''
namesBySequenceIndexNegative[1033] = ''''''
semitonesBySequenceIndex[1033] = [0, 1, 2, 3, 4, 6, 10]
poemBySequenceIndex[1033] = '''B D F G J L R'''
poemAuthors[1033] = ''''''
#1034: ䷪䷂ Decision Sprouting: 1 1 1 1 2 4 2  : S R1 R2 G2 G3 M2 N2 : OdAk

#1035: C D♭ D E♭ E G♭ B :0 1 2 3 4 6 11: 1 ♭2 2 ♭3 3 ♭5 7: ma9(♭5 add ♭9 ♯9)
namesBySequenceIndex[1034] = ''''''
namesBySequenceIndexNegative[1034] = ''''''
semitonesBySequenceIndex[1034] = [0, 1, 2, 3, 4, 6, 11]
poemBySequenceIndex[1034] = '''B D F G J L S'''
poemAuthors[1034] = ''''''
#1035: ䷪䷚ Decision Nourishment: 1 1 1 1 2 5 1  : S R1 R2 G2 G3 M2 N3 : OdAl

#1036: C D♭ D E♭ E G A♭ :0 1 2 3 4 7 8: 1 ♭2 2 ♭3 3 5 ♭6: ma(add ♭6 ♭9 9 ♯9)
namesBySequenceIndex[1035] = '''Aeoladian'''
namesBySequenceIndexNegative[1035] = ''''''
semitonesBySequenceIndex[1035] = [0, 1, 2, 3, 4, 7, 8]
poemBySequenceIndex[1035] = '''B D F G J M N'''
poemAuthors[1035] = ''''''
#1036: ䷪䷭ Decision Ascending: 1 1 1 1 3 1 4  : S R1 R2 G2 G3 P D1 : Odoj

#1037: C D♭ D E♭ E G A :0 1 2 3 4 7 9: 1 ♭2 2 ♭3 3 5 6: 6/9(add ♭9 ♯9)
namesBySequenceIndex[1036] = '''Stycrian'''
namesBySequenceIndexNegative[1036] = ''''''
semitonesBySequenceIndex[1036] = [0, 1, 2, 3, 4, 7, 9]
poemBySequenceIndex[1036] = '''B D F G J M P'''
poemAuthors[1036] = ''''''
#1037: ䷪䷧ Decision Deliverance: 1 1 1 1 3 2 3  : S R1 R2 G2 G3 P D2 : Odeb

#1038: C D♭ D E♭ E G B♭ :0 1 2 3 4 7 10: 1 ♭2 2 ♭3 3 5 ♭7: 9(add ♭9 ♯9)
namesBySequenceIndex[1037] = '''Sadian'''
namesBySequenceIndexNegative[1037] = ''''''
semitonesBySequenceIndex[1037] = [0, 1, 2, 3, 4, 7, 10]
poemBySequenceIndex[1037] = '''B D F G J M R'''
poemAuthors[1037] = ''''''
#1038: ䷪䷜ Decision Chasm: 1 1 1 1 3 3 2  : S R1 R2 G2 G3 P N2 : Odek

#1039: C D♭ D E♭ E G B :0 1 2 3 4 7 11: 1 ♭2 2 ♭3 3 5 7: ma9(add ♭9 ♯9)
namesBySequenceIndex[1038] = '''Mygian'''
namesBySequenceIndexNegative[1038] = ''''''
semitonesBySequenceIndex[1038] = [0, 1, 2, 3, 4, 7, 11]
poemBySequenceIndex[1038] = '''B D F G J M S'''
poemAuthors[1038] = ''''''
#1039: ䷪䷃ Decision Naïve: 1 1 1 1 3 4 1  : S R1 R2 G2 G3 P N3 : Odel

#1040: C D♭ D E♭ E G♯ A :0 1 2 3 4 8 9: 1 ♭2 2 ♭3 3 ♯5 6: 6/9(♯5 add ♭9 ♯9)
namesBySequenceIndex[1039] = '''Lolian'''
namesBySequenceIndexNegative[1039] = ''''''
semitonesBySequenceIndex[1039] = [0, 1, 2, 3, 4, 8, 9]
poemBySequenceIndex[1039] = '''B D F G J N P'''
poemAuthors[1039] = ''''''
#1040: ䷪䷽ Decision Detail: 1 1 1 1 4 1 3  : S R1 R2 G2 G3 D1 N1 : Odib

#1041: C D♭ D E♭ E G♯ B♭ :0 1 2 3 4 8 10: 1 ♭2 2 ♭3 3 ♯5 ♭7: 9(♯5 add ♭9 ♯9)
namesBySequenceIndex[1040] = '''Bynian'''
namesBySequenceIndexNegative[1040] = ''''''
semitonesBySequenceIndex[1040] = [0, 1, 2, 3, 4, 8, 10]
poemBySequenceIndex[1040] = '''B D F G J N R'''
poemAuthors[1040] = ''''''
#1041: ䷪䷦ Decision Limping: 1 1 1 1 4 2 2  : S R1 R2 G2 G3 D1 N2 : Odik

#1042: C D♭ D E♭ E G♯ B :0 1 2 3 4 8 11: 1 ♭2 2 ♭3 3 ♯5 7: ma9(♯5 add ♭9 ♯9)
namesBySequenceIndex[1041] = '''Epydian'''
namesBySequenceIndexNegative[1041] = ''''''
semitonesBySequenceIndex[1041] = [0, 1, 2, 3, 4, 8, 11]
poemBySequenceIndex[1041] = '''B D F G J N S'''
poemAuthors[1041] = ''''''
#1042: ䷪䷳ Decision Stillness: 1 1 1 1 4 3 1  : S R1 R2 G2 G3 D1 N3 : Odil

#1043: C D♭ D E♭ E A B♭ :0 1 2 3 4 9 10: 1 ♭2 2 ♭3 3 6 ♭7: 9(no 5 add ♭9 ♯9 13)
namesBySequenceIndex[1042] = ''''''
namesBySequenceIndexNegative[1042] = ''''''
semitonesBySequenceIndex[1042] = [0, 1, 2, 3, 4, 9, 10]
poemBySequenceIndex[1042] = '''B D F G J P R'''
poemAuthors[1042] = ''''''
#1043: ䷪䷬ Decision Clustering: 1 1 1 1 5 1 2  : S R1 R2 G2 G3 D2 N2 : Odad

#1044: C D♭ D E♭ E A B :0 1 2 3 4 9 11: 1 ♭2 2 ♭3 3 6 7: ma9(no 5 add ♭9 ♯9 13)
namesBySequenceIndex[1043] = ''''''
namesBySequenceIndexNegative[1043] = ''''''
semitonesBySequenceIndex[1043] = [0, 1, 2, 3, 4, 9, 11]
poemBySequenceIndex[1043] = '''B D F G J P S'''
poemAuthors[1043] = ''''''
#1044: ䷪䷢ Decision Prospering: 1 1 1 1 5 2 1  : S R1 R2 G2 G3 D2 N3 : Odaf

#1045: C D♭ D E♭ E A♯ B :0 1 2 3 4 10 11: 1 ♭2 2 ♭3 3 ♯6 7: ma9(no 5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1044] = ''''''
namesBySequenceIndexNegative[1044] = ''''''
semitonesBySequenceIndex[1044] = [0, 1, 2, 3, 4, 10, 11]
poemBySequenceIndex[1044] = '''B D F G J R S'''
poemAuthors[1044] = ''''''
#1045: ䷪䷓ Decision Seeing: 1 1 1 1 6 1 1  : S R1 R2 G2 G3 D3 N3 : Odam

#1046: C D♭ D E♭ F F♯ G :0 1 2 3 5 6 7: 1 ♭2 2 ♭3 4 ♯4 5: mi11(no 7 add ♭9 ♯11)
namesBySequenceIndex[1045] = ''''''
namesBySequenceIndexNegative[1045] = ''''''
semitonesBySequenceIndex[1045] = [0, 1, 2, 3, 5, 6, 7]
poemBySequenceIndex[1045] = '''B D F G K L M'''
poemAuthors[1045] = ''''''
#1046: ䷍䷒ Resevoir Arrived: 1 1 1 2 1 1 5  : S R1 R2 G2 M1 M2 P : OfEj

#1047: C D♭ D E♭ F G♭ A♭ :0 1 2 3 5 6 8: 1 ♭2 2 ♭3 4 ♭5 ♭6: °11(no 7 add ♭6 ♭9)
namesBySequenceIndex[1046] = '''Aerodian'''
namesBySequenceIndexNegative[1046] = ''''''
semitonesBySequenceIndex[1046] = [0, 1, 2, 3, 5, 6, 8]
poemBySequenceIndex[1046] = '''B D F G K L N'''
poemAuthors[1046] = ''''''
#1047: ䷍䷣ Resevoir Eclipse: 1 1 1 2 1 2 4  : S R1 R2 G2 M1 M2 D1 : OfIj

#1048: C D♭ D E♭ F G♭ A :0 1 2 3 5 6 9: 1 ♭2 2 ♭3 4 ♭5 6: °13(no 7 add ♭9)
namesBySequenceIndex[1047] = '''Sycrian'''
namesBySequenceIndexNegative[1047] = ''''''
semitonesBySequenceIndex[1047] = [0, 1, 2, 3, 5, 6, 9]
poemBySequenceIndex[1047] = '''B D F G K L P'''
poemAuthors[1047] = ''''''
#1048: ䷍䷲ Resevoir Thunder: 1 1 1 2 1 3 3  : S R1 R2 G2 M1 M2 D2 : OfAb

#1049: C D♭ D E♭ F G♭ B♭ :0 1 2 3 5 6 10: 1 ♭2 2 ♭3 4 ♭5 ♭7: ⌀11(add ♭9)
namesBySequenceIndex[1048] = '''Katolian'''
namesBySequenceIndexNegative[1048] = ''''''
semitonesBySequenceIndex[1048] = [0, 1, 2, 3, 5, 6, 10]
poemBySequenceIndex[1048] = '''B D F G K L R'''
poemAuthors[1048] = ''''''
#1049: ䷍䷂ Resevoir Sprouting: 1 1 1 2 1 4 2  : S R1 R2 G2 M1 M2 N2 : OfAk

#1050: C D♭ D E♭ F G♭ B :0 1 2 3 5 6 11: 1 ♭2 2 ♭3 4 ♭5 7: °ma11(add ♭9)
namesBySequenceIndex[1049] = ''''''
namesBySequenceIndexNegative[1049] = ''''''
semitonesBySequenceIndex[1049] = [0, 1, 2, 3, 5, 6, 11]
poemBySequenceIndex[1049] = '''B D F G K L S'''
poemAuthors[1049] = ''''''
#1050: ䷍䷚ Resevoir Nourishment: 1 1 1 2 1 5 1  : S R1 R2 G2 M1 M2 N3 : OfAl

#1051: C D♭ D E♭ F G A♭ :0 1 2 3 5 7 8: 1 ♭2 2 ♭3 4 5 ♭6: mi11(no 7 add ♭6 ♭9)
namesBySequenceIndex[1050] = '''Epyrian'''
namesBySequenceIndexNegative[1050] = ''''''
semitonesBySequenceIndex[1050] = [0, 1, 2, 3, 5, 7, 8]
poemBySequenceIndex[1050] = '''B D F G K M N'''
poemAuthors[1050] = ''''''
#1051: ䷍䷭ Resevoir Ascending: 1 1 1 2 2 1 4  : S R1 R2 G2 M1 P D1 : Ofoj

#1052: C D♭ D E♭ F G A :0 1 2 3 5 7 9: 1 ♭2 2 ♭3 4 5 6: mi13(no 7 add ♭9)
namesBySequenceIndex[1051] = '''Aeolythian'''
namesBySequenceIndexNegative[1051] = ''''''
semitonesBySequenceIndex[1051] = [0, 1, 2, 3, 5, 7, 9]
poemBySequenceIndex[1051] = '''B D F G K M P'''
poemAuthors[1051] = ''''''
#1052: ䷍䷧ Resevoir Deliverance: 1 1 1 2 2 2 3  : S R1 R2 G2 M1 P D2 : Ofeb

#1053: C D♭ D E♭ F G B♭ :0 1 2 3 5 7 10: 1 ♭2 2 ♭3 4 5 ♭7: mi11(add ♭9)
namesBySequenceIndex[1052] = '''Magian'''
namesBySequenceIndexNegative[1052] = ''''''
semitonesBySequenceIndex[1052] = [0, 1, 2, 3, 5, 7, 10]
poemBySequenceIndex[1052] = '''B D F G K M R'''
poemAuthors[1052] = ''''''
#1053: ䷍䷜ Resevoir Chasm: 1 1 1 2 2 3 2  : S R1 R2 G2 M1 P N2 : Ofek

#1054: C D♭ D E♭ F G B :0 1 2 3 5 7 11: 1 ♭2 2 ♭3 4 5 7: mima11(add ♭9)
namesBySequenceIndex[1053] = '''Konian'''
namesBySequenceIndexNegative[1053] = ''''''
semitonesBySequenceIndex[1053] = [0, 1, 2, 3, 5, 7, 11]
poemBySequenceIndex[1053] = '''B D F G K M S'''
poemAuthors[1053] = ''''''
#1054: ䷍䷃ Resevoir Naïve: 1 1 1 2 2 4 1  : S R1 R2 G2 M1 P N3 : Ofel

#1055: C D♭ D E♭ F G♯ A :0 1 2 3 5 8 9: 1 ♭2 2 ♭3 4 ♯5 6: mi13(♯5 no 7 add ♭9)
namesBySequenceIndex[1054] = '''Bolian'''
namesBySequenceIndexNegative[1054] = ''''''
semitonesBySequenceIndex[1054] = [0, 1, 2, 3, 5, 8, 9]
poemBySequenceIndex[1054] = '''B D F G K N P'''
poemAuthors[1054] = ''''''
#1055: ䷍䷽ Resevoir Detail: 1 1 1 2 3 1 3  : S R1 R2 G2 M1 D1 N1 : Ofib

#1056: C D♭ D E♭ F G♯ B♭ :0 1 2 3 5 8 10: 1 ♭2 2 ♭3 4 ♯5 ♭7: mi11(♯5 add ♭9)
namesBySequenceIndex[1055] = '''Zalian'''
namesBySequenceIndexNegative[1055] = ''''''
semitonesBySequenceIndex[1055] = [0, 1, 2, 3, 5, 8, 10]
poemBySequenceIndex[1055] = '''B D F G K N R'''
poemAuthors[1055] = ''''''
#1056: ䷍䷦ Resevoir Limping: 1 1 1 2 3 2 2  : S R1 R2 G2 M1 D1 N2 : Ofik

#1057: C D♭ D E♭ F G♯ B :0 1 2 3 5 8 11: 1 ♭2 2 ♭3 4 ♯5 7: mima11(♯5 add ♭9)
namesBySequenceIndex[1056] = '''Gynian'''
namesBySequenceIndexNegative[1056] = ''''''
semitonesBySequenceIndex[1056] = [0, 1, 2, 3, 5, 8, 11]
poemBySequenceIndex[1056] = '''B D F G K N S'''
poemAuthors[1056] = ''''''
#1057: ䷍䷳ Resevoir Stillness: 1 1 1 2 3 3 1  : S R1 R2 G2 M1 D1 N3 : Ofil

#1058: C D♭ D E♭ F A B♭ :0 1 2 3 5 9 10: 1 ♭2 2 ♭3 4 6 ♭7: mi13(no 5 add ♭9)
namesBySequenceIndex[1057] = '''Salian'''
namesBySequenceIndexNegative[1057] = ''''''
semitonesBySequenceIndex[1057] = [0, 1, 2, 3, 5, 9, 10]
poemBySequenceIndex[1057] = '''B D F G K P R'''
poemAuthors[1057] = ''''''
#1058: ䷍䷬ Resevoir Clustering: 1 1 1 2 4 1 2  : S R1 R2 G2 M1 D2 N2 : Ofad

#1059: C D♭ D E♭ F A B :0 1 2 3 5 9 11: 1 ♭2 2 ♭3 4 6 7: mima13(no 5 add ♭9)
namesBySequenceIndex[1058] = '''Aerolian'''
namesBySequenceIndexNegative[1058] = ''''''
semitonesBySequenceIndex[1058] = [0, 1, 2, 3, 5, 9, 11]
poemBySequenceIndex[1058] = '''B D F G K P S'''
poemAuthors[1058] = ''''''
#1059: ䷍䷢ Resevoir Prospering: 1 1 1 2 4 2 1  : S R1 R2 G2 M1 D2 N3 : Ofaf

#1060: C D♭ D E♭ F A♯ B :0 1 2 3 5 10 11: 1 ♭2 2 ♭3 4 ♯6 7: mima11(no 5 add ♭7 ♭9)
namesBySequenceIndex[1059] = ''''''
namesBySequenceIndexNegative[1059] = ''''''
semitonesBySequenceIndex[1059] = [0, 1, 2, 3, 5, 10, 11]
poemBySequenceIndex[1059] = '''B D F G K R S'''
poemAuthors[1059] = ''''''
#1060: ䷍䷓ Resevoir Seeing: 1 1 1 2 5 1 1  : S R1 R2 G2 M1 D3 N3 : Ofam

#1061: C D♭ D E♭ F♯ G A♭ :0 1 2 3 6 7 8: 1 ♭2 2 ♭3 ♯4 5 ♭6: mi(add ♭6 ♭9 9 ♯11)
namesBySequenceIndex[1060] = '''Zythian'''
namesBySequenceIndexNegative[1060] = ''''''
semitonesBySequenceIndex[1060] = [0, 1, 2, 3, 6, 7, 8]
poemBySequenceIndex[1060] = '''B D F G L M N'''
poemAuthors[1060] = ''''''
#1061: ䷡䷊ Might Peace: 1 1 1 3 1 1 4  : S R1 R2 G2 M2 P D1 : ObOj

#1062: C D♭ D E♭ F♯ G A :0 1 2 3 6 7 9: 1 ♭2 2 ♭3 ♯4 5 6: mi6/9(add ♭9 ♯11)
namesBySequenceIndex[1061] = '''Kanian'''
namesBySequenceIndexNegative[1061] = ''''''
semitonesBySequenceIndex[1061] = [0, 1, 2, 3, 6, 7, 9]
poemBySequenceIndex[1061] = '''B D F G L M P'''
poemAuthors[1061] = ''''''
#1062: ䷡䷵ Might Betrothing: 1 1 1 3 1 2 3  : S R1 R2 G2 M2 P D2 : ObEb

#1063: C D♭ D E♭ F♯ G B♭ :0 1 2 3 6 7 10: 1 ♭2 2 ♭3 ♯4 5 ♭7: mi9(add ♭9 ♯11)
namesBySequenceIndex[1062] = '''Logian'''
namesBySequenceIndexNegative[1062] = ''''''
semitonesBySequenceIndex[1062] = [0, 1, 2, 3, 6, 7, 10]
poemBySequenceIndex[1062] = '''B D F G L M R'''
poemAuthors[1062] = ''''''
#1063: ䷡䷻ Might Composed: 1 1 1 3 1 3 2  : S R1 R2 G2 M2 P N2 : ObEk

#1064: C D♭ D E♭ F♯ G B :0 1 2 3 6 7 11: 1 ♭2 2 ♭3 ♯4 5 7: mima9(add ♭9 ♯11)
namesBySequenceIndex[1063] = '''Dylian'''
namesBySequenceIndexNegative[1063] = ''''''
semitonesBySequenceIndex[1063] = [0, 1, 2, 3, 6, 7, 11]
poemBySequenceIndex[1063] = '''B D F G L M S'''
poemAuthors[1063] = ''''''
#1064: ䷡䷨ Might Diminishing: 1 1 1 3 1 4 1  : S R1 R2 G2 M2 P N3 : ObEl

#1065: C D♭ D E♭ F♯ G♯ A :0 1 2 3 6 8 9: 1 ♭2 2 ♭3 ♯4 ♯5 6: °7(add ♭6 ♭9 9)
namesBySequenceIndex[1064] = '''Ganian'''
namesBySequenceIndexNegative[1064] = ''''''
semitonesBySequenceIndex[1064] = [0, 1, 2, 3, 6, 8, 9]
poemBySequenceIndex[1064] = '''B D F G L N P'''
poemAuthors[1064] = ''''''
#1065: ䷡䷶ Might Abounding: 1 1 1 3 2 1 3  : S R1 R2 G2 M2 D1 N1 : ObIb

#1066: C D♭ D E♭ F♯ G♯ B♭ :0 1 2 3 6 8 10: 1 ♭2 2 ♭3 ♯4 ♯5 ♭7: ⌀9(add ♭9 ♭13)
namesBySequenceIndex[1065] = '''Aerygian'''
namesBySequenceIndexNegative[1065] = ''''''
semitonesBySequenceIndex[1065] = [0, 1, 2, 3, 6, 8, 10]
poemBySequenceIndex[1065] = '''B D F G L N R'''
poemAuthors[1065] = ''''''
#1066: ䷡䷾ Might Completed: 1 1 1 3 2 2 2  : S R1 R2 G2 M2 D1 N2 : ObIk

#1067: C D♭ D E♭ F♯ G♯ B :0 1 2 3 6 8 11: 1 ♭2 2 ♭3 ♯4 ♯5 7: °ma9(add ♭9 ♭13)
namesBySequenceIndex[1066] = '''Katorian'''
namesBySequenceIndexNegative[1066] = ''''''
semitonesBySequenceIndex[1066] = [0, 1, 2, 3, 6, 8, 11]
poemBySequenceIndex[1066] = '''B D F G L N S'''
poemAuthors[1066] = ''''''
#1067: ䷡䷕ Might Adorning: 1 1 1 3 2 3 1  : S R1 R2 G2 M2 D1 N3 : ObIl

#1068: C D♭ D E♭ G♭ A B♭ :0 1 2 3 6 9 10: 1 ♭2 2 ♭3 ♭5 6 ♭7: ⌀9(add ♭9 13)
namesBySequenceIndex[1067] = '''Sydian'''
namesBySequenceIndexNegative[1067] = ''''''
semitonesBySequenceIndex[1067] = [0, 1, 2, 3, 6, 9, 10]
poemBySequenceIndex[1067] = '''B D F G L P R'''
poemAuthors[1067] = ''''''
#1068: ䷡䷐ Might Following: 1 1 1 3 3 1 2  : S R1 R2 G2 M2 D2 N2 : ObAd

#1069: C D♭ D E♭ G♭ A B :0 1 2 3 6 9 11: 1 ♭2 2 ♭3 ♭5 6 7: °ma9(add ♭9 13)
namesBySequenceIndex[1068] = '''Dothian'''
namesBySequenceIndexNegative[1068] = ''''''
semitonesBySequenceIndex[1068] = [0, 1, 2, 3, 6, 9, 11]
poemBySequenceIndex[1068] = '''B D F G L P S'''
poemAuthors[1068] = ''''''
#1069: ䷡䷔ Might Biting: 1 1 1 3 3 2 1  : S R1 R2 G2 M2 D2 N3 : ObAf

#1070: C D♭ D E♭ G♭ A♯ B :0 1 2 3 6 10 11: 1 ♭2 2 ♭3 ♭5 ♯6 7: °ma9(add ♭7 ♭9)
namesBySequenceIndex[1069] = '''Pacrian'''
namesBySequenceIndexNegative[1069] = ''''''
semitonesBySequenceIndex[1069] = [0, 1, 2, 3, 6, 10, 11]
poemBySequenceIndex[1069] = '''B D F G L R S'''
poemAuthors[1069] = ''''''
#1070: ䷡䷩ Might Augmenting: 1 1 1 3 4 1 1  : S R1 R2 G2 M2 D3 N3 : ObAm

#1071: C D♭ D E♭ G A♭ A :0 1 2 3 7 8 9: 1 ♭2 2 ♭3 5 ♭6 6: mi6/9(add ♭9 ♭13)
namesBySequenceIndex[1070] = '''Radian'''
namesBySequenceIndexNegative[1070] = ''''''
semitonesBySequenceIndex[1070] = [0, 1, 2, 3, 7, 8, 9]
poemBySequenceIndex[1070] = '''B D F G M N P'''
poemAuthors[1070] = ''''''
#1071: ䷡䷟ Might Persevering: 1 1 1 4 1 1 3  : S R1 R2 G2 P D1 N1 : Obob

#1072: C D♭ D E♭ G A♭ B♭ :0 1 2 3 7 8 10: 1 ♭2 2 ♭3 5 ♭6 ♭7: mi9(add ♭9 ♭13)
namesBySequenceIndex[1071] = '''Doptian'''
namesBySequenceIndexNegative[1071] = ''''''
semitonesBySequenceIndex[1071] = [0, 1, 2, 3, 7, 8, 10]
poemBySequenceIndex[1071] = '''B D F G M N R'''
poemAuthors[1071] = ''''''
#1072: ䷡䷯ Might Source: 1 1 1 4 1 2 2  : S R1 R2 G2 P D1 N2 : Obok

#1073: C D♭ D E♭ G A♭ B :0 1 2 3 7 8 11: 1 ♭2 2 ♭3 5 ♭6 7: mima9(add ♭9 ♭13)
namesBySequenceIndex[1072] = '''Thagian'''
namesBySequenceIndexNegative[1072] = ''''''
semitonesBySequenceIndex[1072] = [0, 1, 2, 3, 7, 8, 11]
poemBySequenceIndex[1072] = '''B D F G M N S'''
poemAuthors[1072] = ''''''
#1073: ䷡䷑ Might Correcting: 1 1 1 4 1 3 1  : S R1 R2 G2 P D1 N3 : Obol

#1074: C D♭ D E♭ G A B♭ :0 1 2 3 7 9 10: 1 ♭2 2 ♭3 5 6 ♭7: mi9(add ♭9 13)
namesBySequenceIndex[1073] = '''Kydian'''
namesBySequenceIndexNegative[1073] = ''''''
semitonesBySequenceIndex[1073] = [0, 1, 2, 3, 7, 9, 10]
poemBySequenceIndex[1073] = '''B D F G M P R'''
poemAuthors[1073] = ''''''
#1074: ䷡䷮ Might Confining: 1 1 1 4 2 1 2  : S R1 R2 G2 P D2 N2 : Obed

#1075: C D♭ D E♭ G A B :0 1 2 3 7 9 11: 1 ♭2 2 ♭3 5 6 7: mima9(add ♭9 13)
namesBySequenceIndex[1074] = '''Galian'''
namesBySequenceIndexNegative[1074] = ''''''
semitonesBySequenceIndex[1074] = [0, 1, 2, 3, 7, 9, 11]
poemBySequenceIndex[1074] = '''B D F G M P S'''
poemAuthors[1074] = ''''''
#1075: ䷡䷿ Might Unfinished: 1 1 1 4 2 2 1  : S R1 R2 G2 P D2 N3 : Obef

#1076: C D♭ D E♭ G A♯ B :0 1 2 3 7 10 11: 1 ♭2 2 ♭3 5 ♯6 7: mima9(add ♭7 ♭9)
namesBySequenceIndex[1075] = '''Katydian'''
namesBySequenceIndexNegative[1075] = ''''''
semitonesBySequenceIndex[1075] = [0, 1, 2, 3, 7, 10, 11]
poemBySequenceIndex[1075] = '''B D F G M R S'''
poemAuthors[1075] = ''''''
#1076: ䷡䷺ Might Dispersing: 1 1 1 4 3 1 1  : S R1 R2 G2 P D3 N3 : Obem

#1077: C D♭ D E♭ G♯ A B♭ :0 1 2 3 8 9 10: 1 ♭2 2 ♭3 ♯5 6 ♭7: mi9(♯5 add ♭9 13)
namesBySequenceIndex[1076] = ''''''
namesBySequenceIndexNegative[1076] = ''''''
semitonesBySequenceIndex[1076] = [0, 1, 2, 3, 8, 9, 10]
poemBySequenceIndex[1076] = '''B D F G N P R'''
poemAuthors[1076] = ''''''
#1077: ䷡䷞ Might Conjoining: 1 1 1 5 1 1 2  : S R1 R2 G2 D1 D2 N2 : Obid

#1078: C D♭ D E♭ G♯ A B :0 1 2 3 8 9 11: 1 ♭2 2 ♭3 ♯5 6 7: mima9(♯5 add ♭9 13)
namesBySequenceIndex[1077] = ''''''
namesBySequenceIndexNegative[1077] = ''''''
semitonesBySequenceIndex[1077] = [0, 1, 2, 3, 8, 9, 11]
poemBySequenceIndex[1077] = '''B D F G N P S'''
poemAuthors[1077] = ''''''
#1078: ䷡䷷ Might Wanderer: 1 1 1 5 1 2 1  : S R1 R2 G2 D1 D2 N3 : Obif

#1079: C D♭ D E♭ G♯ A♯ B :0 1 2 3 8 10 11: 1 ♭2 2 ♭3 ♯5 ♯6 7: mima9(♯5 add ♭7 ♭9)
namesBySequenceIndex[1078] = ''''''
namesBySequenceIndexNegative[1078] = ''''''
semitonesBySequenceIndex[1078] = [0, 1, 2, 3, 8, 10, 11]
poemBySequenceIndex[1078] = '''B D F G N R S'''
poemAuthors[1078] = ''''''
#1079: ䷡䷴ Might Infiltrating: 1 1 1 5 2 1 1  : S R1 R2 G2 D1 N2 N3 : Obim

#1080: C D♭ D E♭ A B♭ B :0 1 2 3 9 10 11: 1 ♭2 2 ♭3 6 ♭7 7: mima9(no 5 add ♭7 ♭9 13)
namesBySequenceIndex[1079] = ''''''
namesBySequenceIndexNegative[1079] = ''''''
semitonesBySequenceIndex[1079] = [0, 1, 2, 3, 9, 10, 11]
poemBySequenceIndex[1079] = '''B D F G P R S'''
poemAuthors[1079] = ''''''
#1080: ䷡䷋ Might Obstruction: 1 1 1 6 1 1 1  : S R1 R2 G2 D2 N2 N3 : Obag

#1081: C D♭ D E F F♯ G :0 1 2 4 5 6 7: 1 ♭2 2 3 4 ♯4 5: 11(no 7 add ♭9 ♯11)
namesBySequenceIndex[1080] = ''''''
namesBySequenceIndexNegative[1080] = ''''''
semitonesBySequenceIndex[1080] = [0, 1, 2, 4, 5, 6, 7]
poemBySequenceIndex[1080] = '''B D F J K L M'''
poemAuthors[1080] = ''''''
#1081: ䷈䷒ Homegrown Arrived: 1 1 2 1 1 1 5  : S R1 R2 G3 M1 M2 P : OmEj

#1082: C D♭ D E F G♭ A♭ :0 1 2 4 5 6 8: 1 ♭2 2 3 4 ♭5 ♭6: 11(♯5 no 7 add ♭9 ♯11)
namesBySequenceIndex[1081] = '''Sodian'''
namesBySequenceIndexNegative[1081] = ''''''
semitonesBySequenceIndex[1081] = [0, 1, 2, 4, 5, 6, 8]
poemBySequenceIndex[1081] = '''B D F J K L N'''
poemAuthors[1081] = ''''''
#1082: ䷈䷣ Homegrown Eclipse: 1 1 2 1 1 2 4  : S R1 R2 G3 M1 M2 D1 : OmIj

#1083: C D♭ D E F G♭ A :0 1 2 4 5 6 9: 1 ♭2 2 3 4 ♭5 6: 13(♭5 no 7 add ♭9)
namesBySequenceIndex[1082] = '''Zygian'''
namesBySequenceIndexNegative[1082] = ''''''
semitonesBySequenceIndex[1082] = [0, 1, 2, 4, 5, 6, 9]
poemBySequenceIndex[1082] = '''B D F J K L P'''
poemAuthors[1082] = ''''''
#1083: ䷈䷲ Homegrown Thunder: 1 1 2 1 1 3 3  : S R1 R2 G3 M1 M2 D2 : OmAb

#1084: C D♭ D E F G♭ B♭ :0 1 2 4 5 6 10: 1 ♭2 2 3 4 ♭5 ♭7: 11(♭5 add ♭9)
namesBySequenceIndex[1083] = '''Styrian'''
namesBySequenceIndexNegative[1083] = ''''''
semitonesBySequenceIndex[1083] = [0, 1, 2, 4, 5, 6, 10]
poemBySequenceIndex[1083] = '''B D F J K L R'''
poemAuthors[1083] = ''''''
#1084: ䷈䷂ Homegrown Sprouting: 1 1 2 1 1 4 2  : S R1 R2 G3 M1 M2 N2 : OmAk

#1085: C D♭ D E F G♭ B :0 1 2 4 5 6 11: 1 ♭2 2 3 4 ♭5 7: ma11(♭5 add ♭9)
namesBySequenceIndex[1084] = ''''''
namesBySequenceIndexNegative[1084] = ''''''
semitonesBySequenceIndex[1084] = [0, 1, 2, 4, 5, 6, 11]
poemBySequenceIndex[1084] = '''B D F J K L S'''
poemAuthors[1084] = ''''''
#1085: ䷈䷚ Homegrown Nourishment: 1 1 2 1 1 5 1  : S R1 R2 G3 M1 M2 N3 : OmAl

#1086: C D♭ D E F G A♭ :0 1 2 4 5 7 8: 1 ♭2 2 3 4 5 ♭6: 11(no 7 add ♭6 ♭9)
namesBySequenceIndex[1085] = '''Bythian'''
namesBySequenceIndexNegative[1085] = ''''''
semitonesBySequenceIndex[1085] = [0, 1, 2, 4, 5, 7, 8]
poemBySequenceIndex[1085] = '''B D F J K M N'''
poemAuthors[1085] = ''''''
#1086: ䷈䷭ Homegrown Ascending: 1 1 2 1 2 1 4  : S R1 R2 G3 M1 P D1 : Omoj

#1087: C D♭ D E F G A :0 1 2 4 5 7 9: 1 ♭2 2 3 4 5 6: 13(no 7 add ♭9)
namesBySequenceIndex[1086] = '''Sarian'''
namesBySequenceIndexNegative[1086] = ''''''
semitonesBySequenceIndex[1086] = [0, 1, 2, 4, 5, 7, 9]
poemBySequenceIndex[1086] = '''B D F J K M P'''
poemAuthors[1086] = ''''''
#1087: ䷈䷧ Homegrown Deliverance: 1 1 2 1 2 2 3  : S R1 R2 G3 M1 P D2 : Omeb

#1088: C D♭ D E F G B♭ :0 1 2 4 5 7 10: 1 ♭2 2 3 4 5 ♭7: 11(add ♭9)
namesBySequenceIndex[1087] = '''Aeoloptian'''
namesBySequenceIndexNegative[1087] = ''''''
semitonesBySequenceIndex[1087] = [0, 1, 2, 4, 5, 7, 10]
poemBySequenceIndex[1087] = '''B D F J K M R'''
poemAuthors[1087] = ''''''
#1088: ䷈䷜ Homegrown Chasm: 1 1 2 1 2 3 2  : S R1 R2 G3 M1 P N2 : Omek

#1089: C D♭ D E F G B :0 1 2 4 5 7 11: 1 ♭2 2 3 4 5 7: ma11(add ♭9)
namesBySequenceIndex[1088] = '''Macrian'''
namesBySequenceIndexNegative[1088] = ''''''
semitonesBySequenceIndex[1088] = [0, 1, 2, 4, 5, 7, 11]
poemBySequenceIndex[1088] = '''B D F J K M S'''
poemAuthors[1088] = ''''''
#1089: ䷈䷃ Homegrown Naïve: 1 1 2 1 2 4 1  : S R1 R2 G3 M1 P N3 : Omel

#1090: C D♭ D E F G♯ A :0 1 2 4 5 8 9: 1 ♭2 2 3 4 ♯5 6: 13(♯5 no 7 add ♭9)
namesBySequenceIndex[1089] = '''Stodian'''
namesBySequenceIndexNegative[1089] = ''''''
semitonesBySequenceIndex[1089] = [0, 1, 2, 4, 5, 8, 9]
poemBySequenceIndex[1089] = '''B D F J K N P'''
poemAuthors[1089] = ''''''
#1090: ䷈䷽ Homegrown Detail: 1 1 2 1 3 1 3  : S R1 R2 G3 M1 D1 N1 : Omib

#1091: C D♭ D E F G♯ B♭ :0 1 2 4 5 8 10: 1 ♭2 2 3 4 ♯5 ♭7: 11(♯5 add ♭9)
namesBySequenceIndex[1090] = '''Aeralian'''
namesBySequenceIndexNegative[1090] = ''''''
semitonesBySequenceIndex[1090] = [0, 1, 2, 4, 5, 8, 10]
poemBySequenceIndex[1090] = '''B D F J K N R'''
poemAuthors[1090] = ''''''
#1091: ䷈䷦ Homegrown Limping: 1 1 2 1 3 2 2  : S R1 R2 G3 M1 D1 N2 : Omik

#1092: C D♭ D E F G♯ B :0 1 2 4 5 8 11: 1 ♭2 2 3 4 ♯5 7: ma11(♯5 add ♭9)
namesBySequenceIndex[1091] = '''Gadian'''
namesBySequenceIndexNegative[1091] = ''''''
semitonesBySequenceIndex[1091] = [0, 1, 2, 4, 5, 8, 11]
poemBySequenceIndex[1091] = '''B D F J K N S'''
poemAuthors[1091] = ''''''
#1092: ䷈䷳ Homegrown Stillness: 1 1 2 1 3 3 1  : S R1 R2 G3 M1 D1 N3 : Omil

#1093: C D♭ D E F A B♭ :0 1 2 4 5 9 10: 1 ♭2 2 3 4 6 ♭7: 13(no 5 add ♭9)
namesBySequenceIndex[1092] = '''Rodian'''
namesBySequenceIndexNegative[1092] = ''''''
semitonesBySequenceIndex[1092] = [0, 1, 2, 4, 5, 9, 10]
poemBySequenceIndex[1092] = '''B D F J K P R'''
poemAuthors[1092] = ''''''
#1093: ䷈䷬ Homegrown Clustering: 1 1 2 1 4 1 2  : S R1 R2 G3 M1 D2 N2 : Omad

#1094: C D♭ D E F A B :0 1 2 4 5 9 11: 1 ♭2 2 3 4 6 7: ma13(no 5 add ♭9)
namesBySequenceIndex[1093] = '''Thoptian'''
namesBySequenceIndexNegative[1093] = ''''''
semitonesBySequenceIndex[1093] = [0, 1, 2, 4, 5, 9, 11]
poemBySequenceIndex[1093] = '''B D F J K P S'''
poemAuthors[1093] = ''''''
#1094: ䷈䷢ Homegrown Prospering: 1 1 2 1 4 2 1  : S R1 R2 G3 M1 D2 N3 : Omaf

#1095: C D♭ D E F A♯ B :0 1 2 4 5 10 11: 1 ♭2 2 3 4 ♯6 7: ma11(no 5 add ♭7 ♭9)
namesBySequenceIndex[1094] = ''''''
namesBySequenceIndexNegative[1094] = ''''''
semitonesBySequenceIndex[1094] = [0, 1, 2, 4, 5, 10, 11]
poemBySequenceIndex[1094] = '''B D F J K R S'''
poemAuthors[1094] = ''''''
#1095: ䷈䷓ Homegrown Seeing: 1 1 2 1 5 1 1  : S R1 R2 G3 M1 D3 N3 : Omam

#1096: C D♭ D E F♯ G A♭ :0 1 2 4 6 7 8: 1 ♭2 2 3 ♯4 5 ♭6: ma(add ♭6 ♭9 9 ♯11)
namesBySequenceIndex[1095] = '''Dodian'''
namesBySequenceIndexNegative[1095] = ''''''
semitonesBySequenceIndex[1095] = [0, 1, 2, 4, 6, 7, 8]
poemBySequenceIndex[1095] = '''B D F J L M N'''
poemAuthors[1095] = ''''''
#1096: ䷄䷊ Biding Peace: 1 1 2 2 1 1 4  : S R1 R2 G3 M2 P D1 : OkOj

#1097: C D♭ D E F♯ G A :0 1 2 4 6 7 9: 1 ♭2 2 3 ♯4 5 6: 6/9(add ♭9 ♯11)
namesBySequenceIndex[1096] = '''Phradian'''
namesBySequenceIndexNegative[1096] = ''''''
semitonesBySequenceIndex[1096] = [0, 1, 2, 4, 6, 7, 9]
poemBySequenceIndex[1096] = '''B D F J L M P'''
poemAuthors[1096] = ''''''
#1097: ䷄䷵ Biding Betrothing: 1 1 2 2 1 2 3  : S R1 R2 G3 M2 P D2 : OkEb

#1098: C D♭ D E F♯ G B♭ :0 1 2 4 6 7 10: 1 ♭2 2 3 ♯4 5 ♭7: 9(add ♭9 ♯11)
namesBySequenceIndex[1097] = '''Epaptian'''
namesBySequenceIndexNegative[1097] = ''''''
semitonesBySequenceIndex[1097] = [0, 1, 2, 4, 6, 7, 10]
poemBySequenceIndex[1097] = '''B D F J L M R'''
poemAuthors[1097] = ''''''
#1098: ䷄䷻ Biding Composed: 1 1 2 2 1 3 2  : S R1 R2 G3 M2 P N2 : OkEk

#1099: C D♭ D E F♯ G B :0 1 2 4 6 7 11: 1 ♭2 2 3 ♯4 5 7: ma9(add ♭9 ♯11)
namesBySequenceIndex[1098] = '''Lycrian'''
namesBySequenceIndexNegative[1098] = ''''''
semitonesBySequenceIndex[1098] = [0, 1, 2, 4, 6, 7, 11]
poemBySequenceIndex[1098] = '''B D F J L M S'''
poemAuthors[1098] = ''''''
#1099: ䷄䷨ Biding Diminishing: 1 1 2 2 1 4 1  : S R1 R2 G3 M2 P N3 : OkEl

#1100: C D♭ D E F♯ G♯ A :0 1 2 4 6 8 9: 1 ♭2 2 3 ♯4 ♯5 6: 6/9(♯5 add ♭9 ♯11)
namesBySequenceIndex[1099] = '''Porian,Ultralocrian ♭♭3'''
namesBySequenceIndexNegative[1099] = ''''''
semitonesBySequenceIndex[1099] = [0, 1, 2, 4, 6, 8, 9]
poemBySequenceIndex[1099] = '''B D F J L N P'''
poemAuthors[1099] = ''''''
#1100: ䷄䷶ Biding Abounding: 1 1 2 2 2 1 3  : S R1 R2 G3 M2 D1 N1 : OkIb

#1101: C D♭ D E F♯ G♯ B♭ :0 1 2 4 6 8 10: 1 ♭2 2 3 ♯4 ♯5 ♭7: 9(♯5 add ♭9 ♯11)
namesBySequenceIndex[1100] = '''Pyptian,Superlocrian ♭♭3'''
namesBySequenceIndexNegative[1100] = ''''''
semitonesBySequenceIndex[1100] = [0, 1, 2, 4, 6, 8, 10]
poemBySequenceIndex[1100] = '''B D F J L N R'''
poemAuthors[1100] = ''''''
#1101: ䷄䷾ Biding Completed: 1 1 2 2 2 2 2  : S R1 R2 G3 M2 D1 N2 : OkIk

#1102: C D♭ D E F♯ G♯ B :0 1 2 4 6 8 11: 1 ♭2 2 3 ♯4 ♯5 7: ma9(♯5 add ♭9 ♯11)
namesBySequenceIndex[1101] = '''Molian'''
namesBySequenceIndexNegative[1101] = ''''''
semitonesBySequenceIndex[1101] = [0, 1, 2, 4, 6, 8, 11]
poemBySequenceIndex[1101] = '''B D F J L N S'''
poemAuthors[1101] = ''''''
#1102: ䷄䷕ Biding Adorning: 1 1 2 2 2 3 1  : S R1 R2 G3 M2 D1 N3 : OkIl

#1103: C D♭ D E G♭ A B♭ :0 1 2 4 6 9 10: 1 ♭2 2 3 ♭5 6 ♭7: 9(♭5 add ♭9 13)
namesBySequenceIndex[1102] = '''Lothian'''
namesBySequenceIndexNegative[1102] = ''''''
semitonesBySequenceIndex[1102] = [0, 1, 2, 4, 6, 9, 10]
poemBySequenceIndex[1102] = '''B D F J L P R'''
poemAuthors[1102] = ''''''
#1103: ䷄䷐ Biding Following: 1 1 2 2 3 1 2  : S R1 R2 G3 M2 D2 N2 : OkAd

#1104: C D♭ D E G♭ A B :0 1 2 4 6 9 11: 1 ♭2 2 3 ♭5 6 7: ma9(♭5 add ♭9 13)
namesBySequenceIndex[1103] = '''Dadian'''
namesBySequenceIndexNegative[1103] = ''''''
semitonesBySequenceIndex[1103] = [0, 1, 2, 4, 6, 9, 11]
poemBySequenceIndex[1103] = '''B D F J L P S'''
poemAuthors[1103] = ''''''
#1104: ䷄䷔ Biding Biting: 1 1 2 2 3 2 1  : S R1 R2 G3 M2 D2 N3 : OkAf

#1105: C D♭ D E G♭ A♯ B :0 1 2 4 6 10 11: 1 ♭2 2 3 ♭5 ♯6 7: ma9(♭5 add ♭7 ♭9)
namesBySequenceIndex[1104] = '''Stocrian'''
namesBySequenceIndexNegative[1104] = ''''''
semitonesBySequenceIndex[1104] = [0, 1, 2, 4, 6, 10, 11]
poemBySequenceIndex[1104] = '''B D F J L R S'''
poemAuthors[1104] = ''''''
#1105: ䷄䷩ Biding Augmenting: 1 1 2 2 4 1 1  : S R1 R2 G3 M2 D3 N3 : OkAm

#1106: C D♭ D E G A♭ A :0 1 2 4 7 8 9: 1 ♭2 2 3 5 ♭6 6: 6/9(add ♭9 ♭13)
namesBySequenceIndex[1105] = '''Gathain,Gathian'''
namesBySequenceIndexNegative[1105] = ''''''
semitonesBySequenceIndex[1105] = [0, 1, 2, 4, 7, 8, 9]
poemBySequenceIndex[1105] = '''B D F J M N P'''
poemAuthors[1105] = ''''''
#1106: ䷄䷟ Biding Persevering: 1 1 2 3 1 1 3  : S R1 R2 G3 P D1 N1 : Okob

#1107: C D♭ D E G A♭ B♭ :0 1 2 4 7 8 10: 1 ♭2 2 3 5 ♭6 ♭7: 9(add ♭9 ♭13)
namesBySequenceIndex[1106] = '''Phragian'''
namesBySequenceIndexNegative[1106] = ''''''
semitonesBySequenceIndex[1106] = [0, 1, 2, 4, 7, 8, 10]
poemBySequenceIndex[1106] = '''B D F J M N R'''
poemAuthors[1106] = ''''''
#1107: ䷄䷯ Biding Source: 1 1 2 3 1 2 2  : S R1 R2 G3 P D1 N2 : Okok

#1108: C D♭ D E G A♭ B :0 1 2 4 7 8 11: 1 ♭2 2 3 5 ♭6 7: ma9(add ♭9 ♭13)
namesBySequenceIndex[1107] = '''Bothian'''
namesBySequenceIndexNegative[1107] = ''''''
semitonesBySequenceIndex[1107] = [0, 1, 2, 4, 7, 8, 11]
poemBySequenceIndex[1107] = '''B D F J M N S'''
poemAuthors[1107] = ''''''
#1108: ䷄䷑ Biding Correcting: 1 1 2 3 1 3 1  : S R1 R2 G3 P D1 N3 : Okol

#1109: C D♭ D E G A B♭ :0 1 2 4 7 9 10: 1 ♭2 2 3 5 6 ♭7: 9(add ♭9 13)
namesBySequenceIndex[1108] = '''Phralian'''
namesBySequenceIndexNegative[1108] = ''''''
semitonesBySequenceIndex[1108] = [0, 1, 2, 4, 7, 9, 10]
poemBySequenceIndex[1108] = '''B D F J M P R'''
poemAuthors[1108] = ''''''
#1109: ䷄䷮ Biding Confining: 1 1 2 3 2 1 2  : S R1 R2 G3 P D2 N2 : Oked

#1110: C D♭ D E G A B :0 1 2 4 7 9 11: 1 ♭2 2 3 5 6 7: ma9(add ♭9 13)
namesBySequenceIndex[1109] = '''Stolian'''
namesBySequenceIndexNegative[1109] = ''''''
semitonesBySequenceIndex[1109] = [0, 1, 2, 4, 7, 9, 11]
poemBySequenceIndex[1109] = '''B D F J M P S'''
poemAuthors[1109] = ''''''
#1110: ䷄䷿ Biding Unfinished: 1 1 2 3 2 2 1  : S R1 R2 G3 P D2 N3 : Okef

#1111: C D♭ D E G A♯ B :0 1 2 4 7 10 11: 1 ♭2 2 3 5 ♯6 7: ma9(add ♭7 ♭9)
namesBySequenceIndex[1110] = '''Thyphian'''
namesBySequenceIndexNegative[1110] = ''''''
semitonesBySequenceIndex[1110] = [0, 1, 2, 4, 7, 10, 11]
poemBySequenceIndex[1110] = '''B D F J M R S'''
poemAuthors[1110] = ''''''
#1111: ䷄䷺ Biding Dispersing: 1 1 2 3 3 1 1  : S R1 R2 G3 P D3 N3 : Okem

#1112: C D♭ D E G♯ A B♭ :0 1 2 4 8 9 10: 1 ♭2 2 3 ♯5 6 ♭7: 9(♯5 add ♭9 13)
namesBySequenceIndex[1111] = '''Godian'''
namesBySequenceIndexNegative[1111] = ''''''
semitonesBySequenceIndex[1111] = [0, 1, 2, 4, 8, 9, 10]
poemBySequenceIndex[1111] = '''B D F J N P R'''
poemAuthors[1111] = ''''''
#1112: ䷄䷞ Biding Conjoining: 1 1 2 4 1 1 2  : S R1 R2 G3 D1 D2 N2 : Okid

#1113: C D♭ D E G♯ A B :0 1 2 4 8 9 11: 1 ♭2 2 3 ♯5 6 7: ma9(♯5 add ♭9 13)
namesBySequenceIndex[1112] = '''Lyptian'''
namesBySequenceIndexNegative[1112] = ''''''
semitonesBySequenceIndex[1112] = [0, 1, 2, 4, 8, 9, 11]
poemBySequenceIndex[1112] = '''B D F J N P S'''
poemAuthors[1112] = ''''''
#1113: ䷄䷷ Biding Wanderer: 1 1 2 4 1 2 1  : S R1 R2 G3 D1 D2 N3 : Okif

#1114: C D♭ D E G♯ A♯ B :0 1 2 4 8 10 11: 1 ♭2 2 3 ♯5 ♯6 7: ma9(♯5 add ♭7 ♭9)
namesBySequenceIndex[1113] = '''Karian'''
namesBySequenceIndexNegative[1113] = ''''''
semitonesBySequenceIndex[1113] = [0, 1, 2, 4, 8, 10, 11]
poemBySequenceIndex[1113] = '''B D F J N R S'''
poemAuthors[1113] = ''''''
#1114: ䷄䷴ Biding Infiltrating: 1 1 2 4 2 1 1  : S R1 R2 G3 D1 N2 N3 : Okim

#1115: C D♭ D E A B♭ B :0 1 2 4 9 10 11: 1 ♭2 2 3 6 ♭7 7: ma9(no 5 add ♭7 ♭9 13)
namesBySequenceIndex[1114] = ''''''
namesBySequenceIndexNegative[1114] = ''''''
semitonesBySequenceIndex[1114] = [0, 1, 2, 4, 9, 10, 11]
poemBySequenceIndex[1114] = '''B D F J P R S'''
poemAuthors[1114] = ''''''
#1115: ䷄䷋ Biding Obstruction: 1 1 2 5 1 1 1  : S R1 R2 G3 D2 N2 N3 : Okag

#1116: C D♭ D F F♯ G A♭ :0 1 2 5 6 7 8: 1 ♭2 2 4 ♯4 5 ♭6: sus4(add ♭6 ♭9 9 ♯11)
namesBySequenceIndex[1115] = '''Dynian'''
namesBySequenceIndexNegative[1115] = ''''''
semitonesBySequenceIndex[1115] = [0, 1, 2, 5, 6, 7, 8]
poemBySequenceIndex[1115] = '''B D F K L M N'''
poemAuthors[1115] = ''''''
#1116: ䷙䷊ Cultivating Peace: 1 1 3 1 1 1 4  : S R1 G1 M1 M2 P D1 : OlOj

#1117: C D♭ D F F♯ G A :0 1 2 5 6 7 9: 1 ♭2 2 4 ♯4 5 6: 6/9sus4(add ♭9 ♯11)
namesBySequenceIndex[1116] = '''Lanian'''
namesBySequenceIndexNegative[1116] = ''''''
semitonesBySequenceIndex[1116] = [0, 1, 2, 5, 6, 7, 9]
poemBySequenceIndex[1116] = '''B D F K L M P'''
poemAuthors[1116] = ''''''
#1117: ䷙䷵ Cultivating Betrothing: 1 1 3 1 1 2 3  : S R1 G1 M1 M2 P D2 : OlEb

#1118: C D♭ D F F♯ G B♭ :0 1 2 5 6 7 10: 1 ♭2 2 4 ♯4 5 ♭7: 11sus♯4(add ♭9)
namesBySequenceIndex[1117] = '''Sogian,Chromatic Mixolydian'''
namesBySequenceIndexNegative[1117] = ''''''
semitonesBySequenceIndex[1117] = [0, 1, 2, 5, 6, 7, 10]
poemBySequenceIndex[1117] = '''B D F K L M R'''
poemAuthors[1117] = ''''''
#1118: ䷙䷻ Cultivating Composed: 1 1 3 1 1 3 2  : S R1 G1 M1 M2 P N2 : OlEk

#1119: C D♭ D F F♯ G B :0 1 2 5 6 7 11: 1 ♭2 2 4 ♯4 5 7: ma11sus♯4(add ♭9)
namesBySequenceIndex[1118] = '''Dyrian'''
namesBySequenceIndexNegative[1118] = ''''''
semitonesBySequenceIndex[1118] = [0, 1, 2, 5, 6, 7, 11]
poemBySequenceIndex[1118] = '''B D F K L M S'''
poemAuthors[1118] = ''''''
#1119: ䷙䷨ Cultivating Diminishing: 1 1 3 1 1 4 1  : S R1 G1 M1 M2 P N3 : OlEl

#1120: C D♭ D F G♭ A♭ A :0 1 2 5 6 8 9: 1 ♭2 2 4 ♭5 ♭6 6: 6/9sus4(♯5 add ♭9 ♯11)
namesBySequenceIndex[1119] = '''Epadian,Locrian ♭♭3 ♭♭7'''
namesBySequenceIndexNegative[1119] = ''''''
semitonesBySequenceIndex[1119] = [0, 1, 2, 5, 6, 8, 9]
poemBySequenceIndex[1119] = '''B D F K L N P'''
poemAuthors[1119] = ''''''
#1120: ䷙䷶ Cultivating Abounding: 1 1 3 1 2 1 3  : S R1 G1 M1 M2 D1 N1 : OlIb

#1121: C D♭ D F G♭ A♭ B♭ :0 1 2 5 6 8 10: 1 ♭2 2 4 ♭5 ♭6 ♭7: 11sus♯4(♯5 add ♭9)
namesBySequenceIndex[1120] = '''Pynian'''
namesBySequenceIndexNegative[1120] = ''''''
semitonesBySequenceIndex[1120] = [0, 1, 2, 5, 6, 8, 10]
poemBySequenceIndex[1120] = '''B D F K L N R'''
poemAuthors[1120] = ''''''
#1121: ䷙䷾ Cultivating Completed: 1 1 3 1 2 2 2  : S R1 G1 M1 M2 D1 N2 : OlIk

#1122: C D♭ D F G♭ A♭ B :0 1 2 5 6 8 11: 1 ♭2 2 4 ♭5 ♭6 7: ma11sus♯4(♯5 add ♭9)
namesBySequenceIndex[1121] = '''Zylian'''
namesBySequenceIndexNegative[1121] = ''''''
semitonesBySequenceIndex[1121] = [0, 1, 2, 5, 6, 8, 11]
poemBySequenceIndex[1121] = '''B D F K L N S'''
poemAuthors[1121] = ''''''
#1122: ䷙䷕ Cultivating Adorning: 1 1 3 1 2 3 1  : S R1 G1 M1 M2 D1 N3 : OlIl

#1123: C D♭ D F G♭ A B♭ :0 1 2 5 6 9 10: 1 ♭2 2 4 ♭5 6 ♭7: 13sus♭2(♭5)
namesBySequenceIndex[1122] = '''Aeolothian'''
namesBySequenceIndexNegative[1122] = ''''''
semitonesBySequenceIndex[1122] = [0, 1, 2, 5, 6, 9, 10]
poemBySequenceIndex[1122] = '''B D F K L P R'''
poemAuthors[1122] = ''''''
#1123: ䷙䷐ Cultivating Following: 1 1 3 1 3 1 2  : S R1 G1 M1 M2 D2 N2 : OlAd

#1124: C D♭ D F G♭ A B :0 1 2 5 6 9 11: 1 ♭2 2 4 ♭5 6 7: ma13sus♭2(♭5)
namesBySequenceIndex[1123] = '''Lalian'''
namesBySequenceIndexNegative[1123] = ''''''
semitonesBySequenceIndex[1123] = [0, 1, 2, 5, 6, 9, 11]
poemBySequenceIndex[1123] = '''B D F K L P S'''
poemAuthors[1123] = ''''''
#1124: ䷙䷔ Cultivating Biting: 1 1 3 1 3 2 1  : S R1 G1 M1 M2 D2 N3 : OlAf

#1125: C D♭ D F G♭ A♯ B :0 1 2 5 6 10 11: 1 ♭2 2 4 ♭5 ♯6 7: ma11sus♭2(♭5 add ♭7)
namesBySequenceIndex[1124] = '''Eponian'''
namesBySequenceIndexNegative[1124] = ''''''
semitonesBySequenceIndex[1124] = [0, 1, 2, 5, 6, 10, 11]
poemBySequenceIndex[1124] = '''B D F K L R S'''
poemAuthors[1124] = ''''''
#1125: ䷙䷩ Cultivating Augmenting: 1 1 3 1 4 1 1  : S R1 G1 M1 M2 D3 N3 : OlAm

#1126: C D♭ D F G A♭ A :0 1 2 5 7 8 9: 1 ♭2 2 4 5 ♭6 6: 6/9sus4(add ♭9 ♭13)
namesBySequenceIndex[1125] = '''Mela Kanakangi,Chromatic Dorian,Raga Kanakambari,Katarian'''
namesBySequenceIndexNegative[1125] = ''''''
semitonesBySequenceIndex[1125] = [0, 1, 2, 5, 7, 8, 9]
poemBySequenceIndex[1125] = '''B D F K M N P'''
poemAuthors[1125] = ''''''
#1126: ䷙䷟ Cultivating Persevering: 1 1 3 2 1 1 3  : S R1 G1 M1 P D1 N1 : Olob

#1127: C D♭ D F G A♭ B♭ :0 1 2 5 7 8 10: 1 ♭2 2 4 5 ♭6 ♭7: 11sus♭2(add ♭13)
namesBySequenceIndex[1126] = '''Mela Ratnangi,Raga Phenadyuti 1,Mixopyrian'''
namesBySequenceIndexNegative[1126] = ''''''
semitonesBySequenceIndex[1126] = [0, 1, 2, 5, 7, 8, 10]
poemBySequenceIndex[1126] = '''B D F K M N R'''
poemAuthors[1126] = ''''''
#1127: ䷙䷯ Cultivating Source: 1 1 3 2 1 2 2  : S R1 G1 M1 P D1 N2 : Olok

#1128: C D♭ D F G A♭ B :0 1 2 5 7 8 11: 1 ♭2 2 4 5 ♭6 7: ma11sus♭2(add ♭13)
namesBySequenceIndex[1127] = '''Mela Ganamurti,Raga Ganasamavarali,Eparian'''
namesBySequenceIndexNegative[1127] = ''''''
semitonesBySequenceIndex[1127] = [0, 1, 2, 5, 7, 8, 11]
poemBySequenceIndex[1127] = '''B D F K M N S'''
poemAuthors[1127] = ''''''
#1128: ䷙䷑ Cultivating Correcting: 1 1 3 2 1 3 1  : S R1 G1 M1 P D1 N3 : Olol

#1129: C D♭ D F G A B♭ :0 1 2 5 7 9 10: 1 ♭2 2 4 5 6 ♭7: 13sus♭2
namesBySequenceIndex[1128] = '''Mela Venaspati,Mela Vanaspati,Raga Bhanumati,Zaptian'''
namesBySequenceIndexNegative[1128] = ''''''
semitonesBySequenceIndex[1128] = [0, 1, 2, 5, 7, 9, 10]
poemBySequenceIndex[1128] = '''B D F K M P R'''
poemAuthors[1128] = ''''''
#1129: ䷙䷮ Cultivating Confining: 1 1 3 2 2 1 2  : S R1 G1 M1 P D2 N2 : Oled

#1130: C D♭ D F G A B :0 1 2 5 7 9 11: 1 ♭2 2 4 5 6 7: ma13sus♭2
namesBySequenceIndex[1129] = '''Mela Manavati,Raga Manoranjani,Pylian'''
namesBySequenceIndexNegative[1129] = ''''''
semitonesBySequenceIndex[1129] = [0, 1, 2, 5, 7, 9, 11]
poemBySequenceIndex[1129] = '''B D F K M P S'''
poemAuthors[1129] = ''''''
#1130: ䷙䷿ Cultivating Unfinished: 1 1 3 2 2 2 1  : S R1 G1 M1 P D2 N3 : Olef

#1131: C D♭ D F G A♯ B :0 1 2 5 7 10 11: 1 ♭2 2 4 5 ♯6 7: ma11sus♭2(add ♭7)
namesBySequenceIndex[1130] = '''Mela Tenarupi,Mela Tanarupi,Raga Tanukirti,Epythian'''
namesBySequenceIndexNegative[1130] = ''''''
semitonesBySequenceIndex[1130] = [0, 1, 2, 5, 7, 10, 11]
poemBySequenceIndex[1130] = '''B D F K M R S'''
poemAuthors[1130] = ''''''
#1131: ䷙䷺ Cultivating Dispersing: 1 1 3 2 3 1 1  : S R1 G1 M1 P D3 N3 : Olem

#1132: C D♭ D F G♯ A B♭ :0 1 2 5 8 9 10: 1 ♭2 2 4 ♯5 6 ♭7: 13sus♭2(♯5)
namesBySequenceIndex[1131] = '''Pothian'''
namesBySequenceIndexNegative[1131] = ''''''
semitonesBySequenceIndex[1131] = [0, 1, 2, 5, 8, 9, 10]
poemBySequenceIndex[1131] = '''B D F K N P R'''
poemAuthors[1131] = ''''''
#1132: ䷙䷞ Cultivating Conjoining: 1 1 3 3 1 1 2  : S R1 G1 M1 D1 D2 N2 : Olid

#1133: C D♭ D F G♯ A B :0 1 2 5 8 9 11: 1 ♭2 2 4 ♯5 6 7: ma13sus♭2(♯5)
namesBySequenceIndex[1132] = '''Epocrain,Kylian'''
namesBySequenceIndexNegative[1132] = ''''''
semitonesBySequenceIndex[1132] = [0, 1, 2, 5, 8, 9, 11]
poemBySequenceIndex[1132] = '''B D F K N P S'''
poemAuthors[1132] = ''''''
#1133: ䷙䷷ Cultivating Wanderer: 1 1 3 3 1 2 1  : S R1 G1 M1 D1 D2 N3 : Olif

#1134: C D♭ D F G♯ A♯ B :0 1 2 5 8 10 11: 1 ♭2 2 4 ♯5 ♯6 7: ma11sus♭2(♯5 add ♭7)
namesBySequenceIndex[1133] = '''Moptian'''
namesBySequenceIndexNegative[1133] = ''''''
semitonesBySequenceIndex[1133] = [0, 1, 2, 5, 8, 10, 11]
poemBySequenceIndex[1133] = '''B D F K N R S'''
poemAuthors[1133] = ''''''
#1134: ䷙䷴ Cultivating Infiltrating: 1 1 3 3 2 1 1  : S R1 G1 M1 D1 N2 N3 : Olim

#1135: C D♭ D F A B♭ B :0 1 2 5 9 10 11: 1 ♭2 2 4 6 ♭7 7: ma13sus♭2(no 5 add ♭7)
namesBySequenceIndex[1134] = '''Aerocrian'''
namesBySequenceIndexNegative[1134] = ''''''
semitonesBySequenceIndex[1134] = [0, 1, 2, 5, 9, 10, 11]
poemBySequenceIndex[1134] = '''B D F K P R S'''
poemAuthors[1134] = ''''''
#1135: ䷙䷋ Cultivating Obstruction: 1 1 3 4 1 1 1  : S R1 G1 M1 D2 N2 N3 : Olag

#1136: C D♭ D F♯ G A♭ A :0 1 2 6 7 8 9: 1 ♭2 2 ♯4 5 ♭6 6: 6sus2(add ♭6 ♭9 ♯11)
namesBySequenceIndex[1135] = '''Mela Salaga,Aeolanian'''
namesBySequenceIndexNegative[1135] = ''''''
semitonesBySequenceIndex[1135] = [0, 1, 2, 6, 7, 8, 9]
poemBySequenceIndex[1135] = '''B D F L M N P'''
poemAuthors[1135] = ''''''
#1136: ䷊䷡ Peace Might: 1 1 4 1 1 1 3  : S R1 G1 M2 P D1 N1 : OjOb

#1137: C D♭ D F♯ G A♭ B♭ :0 1 2 6 7 8 10: 1 ♭2 2 ♯4 5 ♭6 ♭7: 9sus♯4(add ♭9 ♭13)
namesBySequenceIndex[1136] = '''Mela Jalarnava,Aeolagian'''
namesBySequenceIndexNegative[1136] = ''''''
semitonesBySequenceIndex[1136] = [0, 1, 2, 6, 7, 8, 10]
poemBySequenceIndex[1136] = '''B D F L M N R'''
poemAuthors[1136] = ''''''
#1137: ䷊䷄ Peace Biding: 1 1 4 1 1 2 2  : S R1 G1 M2 P D1 N2 : OjOk

#1138: C D♭ D F♯ G A♭ B :0 1 2 6 7 8 11: 1 ♭2 2 ♯4 5 ♭6 7: ma9sus♯4(add ♭9 ♭13)
namesBySequenceIndex[1137] = '''Mela Jhalavarli,Raga Varali,Raga Jinavali,Stonian'''
namesBySequenceIndexNegative[1137] = ''''''
semitonesBySequenceIndex[1137] = [0, 1, 2, 6, 7, 8, 11]
poemBySequenceIndex[1137] = '''B D F L M N S'''
poemAuthors[1137] = ''''''
#1138: ䷊䷙ Peace Cultivating: 1 1 4 1 1 3 1  : S R1 G1 M2 P D1 N3 : OjOl

#1139: C D♭ D F♯ G A B♭ :0 1 2 6 7 9 10: 1 ♭2 2 ♯4 5 6 ♭7: 13sus♭2(♯11)
namesBySequenceIndex[1138] = '''Mela Navanitam,Dagian'''
namesBySequenceIndexNegative[1138] = ''''''
semitonesBySequenceIndex[1138] = [0, 1, 2, 6, 7, 9, 10]
poemBySequenceIndex[1138] = '''B D F L M P R'''
poemAuthors[1138] = ''''''
#1139: ䷊䷹ Peace Joyous: 1 1 4 1 2 1 2  : S R1 G1 M2 P D2 N2 : OjEd

#1140: C D♭ D F♯ G A B :0 1 2 6 7 9 11: 1 ♭2 2 ♯4 5 6 7: ma13sus♭2(♯11)
namesBySequenceIndex[1139] = '''Mela Pavani,Raga Kunbhini,Aeraphian'''
namesBySequenceIndexNegative[1139] = ''''''
semitonesBySequenceIndex[1139] = [0, 1, 2, 6, 7, 9, 11]
poemBySequenceIndex[1139] = '''B D F L M P S'''
poemAuthors[1139] = ''''''
#1140: ䷊䷥ Peace Polarising: 1 1 4 1 2 2 1  : S R1 G1 M2 P D2 N3 : OjEf

#1141: C D♭ D F♯ G A♯ B :0 1 2 6 7 10 11: 1 ♭2 2 ♯4 5 ♯6 7: ma9sus♯4(add ♭7 ♭9)
namesBySequenceIndex[1140] = '''Mela Raghupriya,Raga Ghandarva,Raga Ravikriya,Kolian'''
namesBySequenceIndexNegative[1140] = ''''''
semitonesBySequenceIndex[1140] = [0, 1, 2, 6, 7, 10, 11]
poemBySequenceIndex[1140] = '''B D F L M R S'''
poemAuthors[1140] = ''''''
#1141: ䷊䷼ Peace Truth: 1 1 4 1 3 1 1  : S R1 G1 M2 P D3 N3 : OjEm

#1142: C D♭ D F♯ G♯ A B♭ :0 1 2 6 8 9 10: 1 ♭2 2 ♯4 ♯5 6 ♭7: 13sus♭2(♯5 ♯11)
namesBySequenceIndex[1141] = '''Pycrian'''
namesBySequenceIndexNegative[1141] = ''''''
semitonesBySequenceIndex[1141] = [0, 1, 2, 6, 8, 9, 10]
poemBySequenceIndex[1141] = '''B D F L N P R'''
poemAuthors[1141] = ''''''
#1142: ䷊䷰ Peace Revolution: 1 1 4 2 1 1 2  : S R1 G1 M2 D1 D2 N2 : OjId

#1143: C D♭ D F♯ G♯ A B :0 1 2 6 8 9 11: 1 ♭2 2 ♯4 ♯5 6 7: ma13sus♭2(♯5 ♯11)
namesBySequenceIndex[1142] = '''Gaptian'''
namesBySequenceIndexNegative[1142] = ''''''
semitonesBySequenceIndex[1142] = [0, 1, 2, 6, 8, 9, 11]
poemBySequenceIndex[1142] = '''B D F L N P S'''
poemAuthors[1142] = ''''''
#1143: ䷊䷝ Peace Distanced: 1 1 4 2 1 2 1  : S R1 G1 M2 D1 D2 N3 : OjIf

#1144: C D♭ D F♯ G♯ A♯ B :0 1 2 6 8 10 11: 1 ♭2 2 ♯4 ♯5 ♯6 7: ma9sus♯4(♯5 add ♭7 ♭9)
namesBySequenceIndex[1143] = '''Zonian'''
namesBySequenceIndexNegative[1143] = ''''''
semitonesBySequenceIndex[1143] = [0, 1, 2, 6, 8, 10, 11]
poemBySequenceIndex[1143] = '''B D F L N R S'''
poemAuthors[1143] = ''''''
#1144: ䷊䷤ Peace Family: 1 1 4 2 2 1 1  : S R1 G1 M2 D1 N2 N3 : OjIm

#1145: C D♭ D G♭ A B♭ B :0 1 2 6 9 10 11: 1 ♭2 2 ♭5 6 ♭7 7: ma9sus♭2(♭5 add ♭7 13)
namesBySequenceIndex[1144] = '''Mathian'''
namesBySequenceIndexNegative[1144] = ''''''
semitonesBySequenceIndex[1144] = [0, 1, 2, 6, 9, 10, 11]
poemBySequenceIndex[1144] = '''B D F L P R S'''
poemAuthors[1144] = ''''''
#1145: ䷊䷘ Peace Integrity: 1 1 4 3 1 1 1  : S R1 G1 M2 D2 N2 N3 : OjAg

#1146: C D♭ D G A♭ A B♭ :0 1 2 7 8 9 10: 1 ♭2 2 5 ♭6 6 ♭7: 9sus♭2(add ♭13 13)
namesBySequenceIndex[1145] = ''''''
namesBySequenceIndexNegative[1145] = ''''''
semitonesBySequenceIndex[1145] = [0, 1, 2, 7, 8, 9, 10]
poemBySequenceIndex[1145] = '''B D F M N P R'''
poemAuthors[1145] = ''''''
#1146: ䷊䷛ Peace Excess: 1 1 5 1 1 1 2  : S R1 G1 P D1 D2 N2 : Ojod

#1147: C D♭ D G A♭ A B :0 1 2 7 8 9 11: 1 ♭2 2 5 ♭6 6 7: ma9sus♭2(add ♭13 13)
namesBySequenceIndex[1146] = ''''''
namesBySequenceIndexNegative[1146] = ''''''
semitonesBySequenceIndex[1146] = [0, 1, 2, 7, 8, 9, 11]
poemBySequenceIndex[1146] = '''B D F M N P S'''
poemAuthors[1146] = ''''''
#1147: ䷊䷱ Peace Holding: 1 1 5 1 1 2 1  : S R1 G1 P D1 D2 N3 : Ojof

#1148: C D♭ D G A♭ B♭ B :0 1 2 7 8 10 11: 1 ♭2 2 5 ♭6 ♭7 7: ma9sus♭2(add ♭7 ♭13)
namesBySequenceIndex[1147] = ''''''
namesBySequenceIndexNegative[1147] = ''''''
semitonesBySequenceIndex[1147] = [0, 1, 2, 7, 8, 10, 11]
poemBySequenceIndex[1147] = '''B D F M N R S'''
poemAuthors[1147] = ''''''
#1148: ䷊䷸ Peace Osmosis: 1 1 5 1 2 1 1  : S R1 G1 P D1 N2 N3 : Ojom

#1149: C D♭ D G A B♭ B :0 1 2 7 9 10 11: 1 ♭2 2 5 6 ♭7 7: ma9sus♭2(add ♭7 13)
namesBySequenceIndex[1148] = ''''''
namesBySequenceIndexNegative[1148] = ''''''
semitonesBySequenceIndex[1148] = [0, 1, 2, 7, 9, 10, 11]
poemBySequenceIndex[1148] = '''B D F M P R S'''
poemAuthors[1148] = ''''''
#1149: ䷊䷅ Peace Arguing: 1 1 5 2 1 1 1  : S R1 G1 P D2 N2 N3 : Ojeg

#1150: C D♭ D G♯ A B♭ B :0 1 2 8 9 10 11: 1 ♭2 2 ♯5 6 ♭7 7: ma9sus♭2(♯5 add ♭7 13)
namesBySequenceIndex[1149] = ''''''
namesBySequenceIndexNegative[1149] = ''''''
semitonesBySequenceIndex[1149] = [0, 1, 2, 8, 9, 10, 11]
poemBySequenceIndex[1149] = '''B D F N P R S'''
poemAuthors[1149] = ''''''
#1150: ䷊䷠ Peace Retiring: 1 1 6 1 1 1 1  : S R1 G1 D1 D2 N2 N3 : Ojig

#1151: C D♭ D♯ E F F♯ G :0 1 3 4 5 6 7: 1 ♭2 ♯2 3 4 ♯4 5: ma(add ♭9 ♯9 11 ♯11)
namesBySequenceIndex[1150] = ''''''
namesBySequenceIndexNegative[1150] = ''''''
semitonesBySequenceIndex[1150] = [0, 1, 3, 4, 5, 6, 7]
poemBySequenceIndex[1150] = '''B D G J K L M'''
poemAuthors[1150] = ''''''
#1151: ䷉䷒ Treading Arrived: 1 2 1 1 1 1 5  : S R1 R3 G3 M1 M2 P : EgEj

#1152: C D♭ D♯ E F G♭ A♭ :0 1 3 4 5 6 8: 1 ♭2 ♯2 3 4 ♭5 ♭6: ma(♯5 add ♭9 ♯9 11 ♯11)
namesBySequenceIndex[1151] = '''Aeragian'''
namesBySequenceIndexNegative[1151] = ''''''
semitonesBySequenceIndex[1151] = [0, 1, 3, 4, 5, 6, 8]
poemBySequenceIndex[1151] = '''B D G J K L N'''
poemAuthors[1151] = ''''''
#1152: ䷉䷣ Treading Eclipse: 1 2 1 1 1 2 4  : S R1 R3 G3 M1 M2 D1 : EgIj

#1153: C D♭ D♯ E F G♭ A :0 1 3 4 5 6 9: 1 ♭2 ♯2 3 4 ♭5 6: 6(♭5 add ♭9 ♯9 11)
namesBySequenceIndex[1152] = '''Epolian'''
namesBySequenceIndexNegative[1152] = ''''''
semitonesBySequenceIndex[1152] = [0, 1, 3, 4, 5, 6, 9]
poemBySequenceIndex[1152] = '''B D G J K L P'''
poemAuthors[1152] = ''''''
#1153: ䷉䷲ Treading Thunder: 1 2 1 1 1 3 3  : S R1 R3 G3 M1 M2 D2 : EgAb

#1154: C D♭ D♯ E F G♭ B♭ :0 1 3 4 5 6 10: 1 ♭2 ♯2 3 4 ♭5 ♭7: 11(♭5 ♭9 add ♯9)
namesBySequenceIndex[1153] = '''Epynian'''
namesBySequenceIndexNegative[1153] = ''''''
semitonesBySequenceIndex[1153] = [0, 1, 3, 4, 5, 6, 10]
poemBySequenceIndex[1153] = '''B D G J K L R'''
poemAuthors[1153] = ''''''
#1154: ䷉䷂ Treading Sprouting: 1 2 1 1 1 4 2  : S R1 R3 G3 M1 M2 N2 : EgAk

#1155: C D♭ D♯ E F G♭ B :0 1 3 4 5 6 11: 1 ♭2 ♯2 3 4 ♭5 7: ma11(♭5 ♭9 add ♯9)
namesBySequenceIndex[1154] = ''''''
namesBySequenceIndexNegative[1154] = ''''''
semitonesBySequenceIndex[1154] = [0, 1, 3, 4, 5, 6, 11]
poemBySequenceIndex[1154] = '''B D G J K L S'''
poemAuthors[1154] = ''''''
#1155: ䷉䷚ Treading Nourishment: 1 2 1 1 1 5 1  : S R1 R3 G3 M1 M2 N3 : EgAl

#1156: C D♭ D♯ E F G A♭ :0 1 3 4 5 7 8: 1 ♭2 ♯2 3 4 5 ♭6: ma(add ♭6 ♭9 ♯9 11)
namesBySequenceIndex[1155] = '''Kothian'''
namesBySequenceIndexNegative[1155] = ''''''
semitonesBySequenceIndex[1155] = [0, 1, 3, 4, 5, 7, 8]
poemBySequenceIndex[1155] = '''B D G J K M N'''
poemAuthors[1155] = ''''''
#1156: ䷉䷭ Treading Ascending: 1 2 1 1 2 1 4  : S R1 R3 G3 M1 P D1 : Egoj

#1157: C D♭ D♯ E F G A :0 1 3 4 5 7 9: 1 ♭2 ♯2 3 4 5 6: 6(add ♭9 ♯9 11)
namesBySequenceIndex[1156] = '''Aerothian'''
namesBySequenceIndexNegative[1156] = ''''''
semitonesBySequenceIndex[1156] = [0, 1, 3, 4, 5, 7, 9]
poemBySequenceIndex[1156] = '''B D G J K M P'''
poemAuthors[1156] = ''''''
#1157: ䷉䷧ Treading Deliverance: 1 2 1 1 2 2 3  : S R1 R3 G3 M1 P D2 : Egeb

#1158: C D♭ D♯ E F G B♭ :0 1 3 4 5 7 10: 1 ♭2 ♯2 3 4 5 ♭7: 11(♭9 add ♯9)
namesBySequenceIndex[1157] = '''Zadian'''
namesBySequenceIndexNegative[1157] = ''''''
semitonesBySequenceIndex[1157] = [0, 1, 3, 4, 5, 7, 10]
poemBySequenceIndex[1157] = '''B D G J K M R'''
poemAuthors[1157] = ''''''
#1158: ䷉䷜ Treading Chasm: 1 2 1 1 2 3 2  : S R1 R3 G3 M1 P N2 : Egek

#1159: C D♭ D♯ E F G B :0 1 3 4 5 7 11: 1 ♭2 ♯2 3 4 5 7: ma11(♭9 add ♯9)
namesBySequenceIndex[1158] = '''Bathian'''
namesBySequenceIndexNegative[1158] = ''''''
semitonesBySequenceIndex[1158] = [0, 1, 3, 4, 5, 7, 11]
poemBySequenceIndex[1158] = '''B D G J K M S'''
poemAuthors[1158] = ''''''
#1159: ䷉䷃ Treading Naïve: 1 2 1 1 2 4 1  : S R1 R3 G3 M1 P N3 : Egel

#1160: C D♭ D♯ E F G♯ A :0 1 3 4 5 8 9: 1 ♭2 ♯2 3 4 ♯5 6: 6(♯5 add ♭9 ♯9 11)
namesBySequenceIndex[1159] = '''Mixolocrian'''
namesBySequenceIndexNegative[1159] = ''''''
semitonesBySequenceIndex[1159] = [0, 1, 3, 4, 5, 8, 9]
poemBySequenceIndex[1159] = '''B D G J K N P'''
poemAuthors[1159] = ''''''
#1160: ䷉䷽ Treading Detail: 1 2 1 1 3 1 3  : S R1 R3 G3 M1 D1 N1 : Egib

#1161: C D♭ D♯ E F G♯ B♭ :0 1 3 4 5 8 10: 1 ♭2 ♯2 3 4 ♯5 ♭7: 11(♯5 ♭9 add ♯9)
namesBySequenceIndex[1160] = '''Kycrian'''
namesBySequenceIndexNegative[1160] = ''''''
semitonesBySequenceIndex[1160] = [0, 1, 3, 4, 5, 8, 10]
poemBySequenceIndex[1160] = '''B D G J K N R'''
poemAuthors[1160] = ''''''
#1161: ䷉䷦ Treading Limping: 1 2 1 1 3 2 2  : S R1 R3 G3 M1 D1 N2 : Egik

#1162: C D♭ D♯ E F G♯ B :0 1 3 4 5 8 11: 1 ♭2 ♯2 3 4 ♯5 7: ma11(♯5 ♭9 add ♯9)
namesBySequenceIndex[1161] = '''Kataptian'''
namesBySequenceIndexNegative[1161] = ''''''
semitonesBySequenceIndex[1161] = [0, 1, 3, 4, 5, 8, 11]
poemBySequenceIndex[1161] = '''B D G J K N S'''
poemAuthors[1161] = ''''''
#1162: ䷉䷳ Treading Stillness: 1 2 1 1 3 3 1  : S R1 R3 G3 M1 D1 N3 : Egil

#1163: C D♭ D♯ E F A B♭ :0 1 3 4 5 9 10: 1 ♭2 ♯2 3 4 6 ♭7: 13(♭9 no 5 add ♯9)
namesBySequenceIndex[1162] = '''Dacrian'''
namesBySequenceIndexNegative[1162] = ''''''
semitonesBySequenceIndex[1162] = [0, 1, 3, 4, 5, 9, 10]
poemBySequenceIndex[1162] = '''B D G J K P R'''
poemAuthors[1162] = ''''''
#1163: ䷉䷬ Treading Clustering: 1 2 1 1 4 1 2  : S R1 R3 G3 M1 D2 N2 : Egad

#1164: C D♭ D♯ E F A B :0 1 3 4 5 9 11: 1 ♭2 ♯2 3 4 6 7: ma13(♭9 no 5 add ♯9)
namesBySequenceIndex[1163] = '''Ionyrian'''
namesBySequenceIndexNegative[1163] = ''''''
semitonesBySequenceIndex[1163] = [0, 1, 3, 4, 5, 9, 11]
poemBySequenceIndex[1163] = '''B D G J K P S'''
poemAuthors[1163] = ''''''
#1164: ䷉䷢ Treading Prospering: 1 2 1 1 4 2 1  : S R1 R3 G3 M1 D2 N3 : Egaf

#1165: C D♭ D♯ E F A♯ B :0 1 3 4 5 10 11: 1 ♭2 ♯2 3 4 ♯6 7: ma11(♭9 no 5 add ♭7 ♯9)
namesBySequenceIndex[1164] = ''''''
namesBySequenceIndexNegative[1164] = ''''''
semitonesBySequenceIndex[1164] = [0, 1, 3, 4, 5, 10, 11]
poemBySequenceIndex[1164] = '''B D G J K R S'''
poemAuthors[1164] = ''''''
#1165: ䷉䷓ Treading Seeing: 1 2 1 1 5 1 1  : S R1 R3 G3 M1 D3 N3 : Egam

#1166: C D♭ D♯ E F♯ G A♭ :0 1 3 4 6 7 8: 1 ♭2 ♯2 3 ♯4 5 ♭6: ma(add ♭6 ♭9 ♯9 ♯11)
namesBySequenceIndex[1165] = '''Aeolygian'''
namesBySequenceIndexNegative[1165] = ''''''
semitonesBySequenceIndex[1165] = [0, 1, 3, 4, 6, 7, 8]
poemBySequenceIndex[1165] = '''B D G J L M N'''
poemAuthors[1165] = ''''''
#1166: ䷹䷊ Joyous Peace: 1 2 1 2 1 1 4  : S R1 R3 G3 M2 P D1 : EdOj

#1167: C D♭ D♯ E F♯ G A :0 1 3 4 6 7 9: 1 ♭2 ♯2 3 ♯4 5 6: 6(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1166] = '''Ionorian,Superlocrian ♭♭6 ♭♭7'''
namesBySequenceIndexNegative[1166] = ''''''
semitonesBySequenceIndex[1166] = [0, 1, 3, 4, 6, 7, 9]
poemBySequenceIndex[1166] = '''B D G J L M P'''
poemAuthors[1166] = ''''''
#1167: ䷹䷵ Joyous Betrothing: 1 2 1 2 1 2 3  : S R1 R3 G3 M2 P D2 : EdEb

#1168: C D♭ D♯ E F♯ G B♭ :0 1 3 4 6 7 10: 1 ♭2 ♯2 3 ♯4 5 ♭7: 7(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1167] = '''Epylian'''
namesBySequenceIndexNegative[1167] = ''''''
semitonesBySequenceIndex[1167] = [0, 1, 3, 4, 6, 7, 10]
poemBySequenceIndex[1167] = '''B D G J L M R'''
poemAuthors[1167] = ''''''
#1168: ䷹䷻ Joyous Composed: 1 2 1 2 1 3 2  : S R1 R3 G3 M2 P N2 : EdEk

#1169: C D♭ D♯ E F♯ G B :0 1 3 4 6 7 11: 1 ♭2 ♯2 3 ♯4 5 7: ma7(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1168] = '''Padian'''
namesBySequenceIndexNegative[1168] = ''''''
semitonesBySequenceIndex[1168] = [0, 1, 3, 4, 6, 7, 11]
poemBySequenceIndex[1168] = '''B D G J L M S'''
poemAuthors[1168] = ''''''
#1169: ䷹䷨ Joyous Diminishing: 1 2 1 2 1 4 1  : S R1 R3 G3 M2 P N3 : EdEl

#1170: C D♭ D♯ E F♯ G♯ A :0 1 3 4 6 8 9: 1 ♭2 ♯2 3 ♯4 ♯5 6: 6(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1169] = '''Ultralocrian,Mixolydian ♯1,Pathian'''
namesBySequenceIndexNegative[1169] = ''''''
semitonesBySequenceIndex[1169] = [0, 1, 3, 4, 6, 8, 9]
poemBySequenceIndex[1169] = '''B D G J L N P'''
poemAuthors[1169] = ''''''
#1170: ䷹䷶ Joyous Abounding: 1 2 1 2 2 1 3  : S R1 R3 G3 M2 D1 N1 : EdIb

#1171: C D♭ D♯ E F♯ G♯ B♭ :0 1 3 4 6 8 10: 1 ♭2 ♯2 3 ♯4 ♯5 ♭7: 7(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1170] = '''Altered,Altered Dominant,Superlocrian,Locrian ♭4,Pomeroy,Rave,Ionadian'''
namesBySequenceIndexNegative[1170] = ''''''
semitonesBySequenceIndex[1170] = [0, 1, 3, 4, 6, 8, 10]
poemBySequenceIndex[1170] = '''B D G J L N R'''
poemAuthors[1170] = ''''''
#1171: ䷹䷾ Joyous Completed: 1 2 1 2 2 2 2  : S R1 R3 G3 M2 D1 N2 : EdIk

#1172: C D♭ D♯ E F♯ G♯ B :0 1 3 4 6 8 11: 1 ♭2 ♯2 3 ♯4 ♯5 7: ma7(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1171] = '''Zoptian,Superlocrian Ma7'''
namesBySequenceIndexNegative[1171] = ''''''
semitonesBySequenceIndex[1171] = [0, 1, 3, 4, 6, 8, 11]
poemBySequenceIndex[1171] = '''B D G J L N S'''
poemAuthors[1171] = ''''''
#1172: ䷹䷕ Joyous Adorning: 1 2 1 2 2 3 1  : S R1 R3 G3 M2 D1 N3 : EdIl

#1173: C D♭ D♯ E G♭ A B♭ :0 1 3 4 6 9 10: 1 ♭2 ♯2 3 ♭5 6 ♭7: 7(♭5 add ♭9 ♯9 13)
namesBySequenceIndex[1172] = '''Zyptian,Superlocrian ♯6'''
namesBySequenceIndexNegative[1172] = ''''''
semitonesBySequenceIndex[1172] = [0, 1, 3, 4, 6, 9, 10]
poemBySequenceIndex[1172] = '''B D G J L P R'''
poemAuthors[1172] = ''''''
#1173: ䷹䷐ Joyous Following: 1 2 1 2 3 1 2  : S R1 R3 G3 M2 D2 N2 : EdAd

#1174: C D♭ D♯ E G♭ A B :0 1 3 4 6 9 11: 1 ♭2 ♯2 3 ♭5 6 7: ma7(♭5 add ♭9 ♯9 13)
namesBySequenceIndex[1173] = '''Panian,Crunchy Altered Major'''
namesBySequenceIndexNegative[1173] = ''''''
semitonesBySequenceIndex[1173] = [0, 1, 3, 4, 6, 9, 11]
poemBySequenceIndex[1173] = '''B D G J L P S'''
poemAuthors[1173] = ''''''
#1174: ䷹䷔ Joyous Biting: 1 2 1 2 3 2 1  : S R1 R3 G3 M2 D2 N3 : EdAf

#1175: C D♭ D♯ E G♭ A♯ B :0 1 3 4 6 10 11: 1 ♭2 ♯2 3 ♭5 ♯6 7: ma7(♭5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1174] = '''Rogian'''
namesBySequenceIndexNegative[1174] = ''''''
semitonesBySequenceIndex[1174] = [0, 1, 3, 4, 6, 10, 11]
poemBySequenceIndex[1174] = '''B D G J L R S'''
poemAuthors[1174] = ''''''
#1175: ䷹䷩ Joyous Augmenting: 1 2 1 2 4 1 1  : S R1 R3 G3 M2 D3 N3 : EdAm

#1176: C D♭ D♯ E G A♭ A :0 1 3 4 7 8 9: 1 ♭2 ♯2 3 5 ♭6 6: 6(add ♭6 ♭9 ♯9)
namesBySequenceIndex[1175] = '''Ultraphrygian,Ionodian'''
namesBySequenceIndexNegative[1175] = ''''''
semitonesBySequenceIndex[1175] = [0, 1, 3, 4, 7, 8, 9]
poemBySequenceIndex[1175] = '''B D G J M N P'''
poemAuthors[1175] = ''''''
#1176: ䷹䷟ Joyous Persevering: 1 2 1 3 1 1 3  : S R1 R3 G3 P D1 N1 : Edob

#1177: C D♭ D♯ E G A♭ B♭ :0 1 3 4 7 8 10: 1 ♭2 ♯2 3 5 ♭6 ♭7: 7(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1176] = '''Maqam Huzzam,Maqam Saba Zamzam (Iraq),Maqam Huzzam (Iraq),Phronian,Phrygian ♭4'''
namesBySequenceIndexNegative[1176] = ''''''
semitonesBySequenceIndex[1176] = [0, 1, 3, 4, 7, 8, 10]
poemBySequenceIndex[1176] = '''B D G J M N R'''
poemAuthors[1176] = ''''''
#1177: ䷹䷯ Joyous Source: 1 2 1 3 1 2 2  : S R1 R3 G3 P D1 N2 : Edok

#1178: C D♭ D♯ E G A♭ B :0 1 3 4 7 8 11: 1 ♭2 ♯2 3 5 ♭6 7: ma7(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1177] = '''Harmonic Gypsy,Phrygian ♭4 Ma7,Ionocrian'''
namesBySequenceIndexNegative[1177] = ''''''
semitonesBySequenceIndex[1177] = [0, 1, 3, 4, 7, 8, 11]
poemBySequenceIndex[1177] = '''B D G J M N S'''
poemAuthors[1177] = ''''''
#1178: ䷹䷑ Joyous Correcting: 1 2 1 3 1 3 1  : S R1 R3 G3 P D1 N3 : Edol

#1179: C D♭ D♯ E G A B♭ :0 1 3 4 7 9 10: 1 ♭2 ♯2 3 5 6 ♭7: 7(add ♭9 ♯9 13)
namesBySequenceIndex[1178] = '''Kathian,Dorian ♭2 ♭4'''
namesBySequenceIndexNegative[1178] = ''''''
semitonesBySequenceIndex[1178] = [0, 1, 3, 4, 7, 9, 10]
poemBySequenceIndex[1178] = '''B D G J M P R'''
poemAuthors[1178] = ''''''
#1179: ䷹䷮ Joyous Confining: 1 2 1 3 2 1 2  : S R1 R3 G3 P D2 N2 : Eded

#1180: C D♭ D♯ E G A B :0 1 3 4 7 9 11: 1 ♭2 ♯2 3 5 6 7: ma7(add ♭9 ♯9 13)
namesBySequenceIndex[1179] = '''Kynian,Neapolitan Major ♭4'''
namesBySequenceIndexNegative[1179] = ''''''
semitonesBySequenceIndex[1179] = [0, 1, 3, 4, 7, 9, 11]
poemBySequenceIndex[1179] = '''B D G J M P S'''
poemAuthors[1179] = ''''''
#1180: ䷹䷿ Joyous Unfinished: 1 2 1 3 2 2 1  : S R1 R3 G3 P D2 N3 : Edef

#1181: C D♭ D♯ E G A♯ B :0 1 3 4 7 10 11: 1 ♭2 ♯2 3 5 ♯6 7: ma7(add ♭7 ♭9 ♯9)
namesBySequenceIndex[1180] = '''Aeolocrian'''
namesBySequenceIndexNegative[1180] = ''''''
semitonesBySequenceIndex[1180] = [0, 1, 3, 4, 7, 10, 11]
poemBySequenceIndex[1180] = '''B D G J M R S'''
poemAuthors[1180] = ''''''
#1181: ䷹䷺ Joyous Dispersing: 1 2 1 3 3 1 1  : S R1 R3 G3 P D3 N3 : Edem

#1182: C D♭ D♯ E G♯ A B♭ :0 1 3 4 8 9 10: 1 ♭2 ♯2 3 ♯5 6 ♭7: 7(♯5 add ♭9 ♯9 13)
namesBySequenceIndex[1181] = '''Pydian'''
namesBySequenceIndexNegative[1181] = ''''''
semitonesBySequenceIndex[1181] = [0, 1, 3, 4, 8, 9, 10]
poemBySequenceIndex[1181] = '''B D G J N P R'''
poemAuthors[1181] = ''''''
#1182: ䷹䷞ Joyous Conjoining: 1 2 1 4 1 1 2  : S R1 R3 G3 D1 D2 N2 : Edid

#1183: C D♭ D♯ E G♯ A B :0 1 3 4 8 9 11: 1 ♭2 ♯2 3 ♯5 6 7: ma7(♯5 add ♭9 ♯9 13)
namesBySequenceIndex[1182] = '''Sorian'''
namesBySequenceIndexNegative[1182] = ''''''
semitonesBySequenceIndex[1182] = [0, 1, 3, 4, 8, 9, 11]
poemBySequenceIndex[1182] = '''B D G J N P S'''
poemAuthors[1182] = ''''''
#1183: ䷹䷷ Joyous Wanderer: 1 2 1 4 1 2 1  : S R1 R3 G3 D1 D2 N3 : Edif

#1184: C D♭ D♯ E G♯ A♯ B :0 1 3 4 8 10 11: 1 ♭2 ♯2 3 ♯5 ♯6 7: ma7(♯5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1183] = '''Bagian'''
namesBySequenceIndexNegative[1183] = ''''''
semitonesBySequenceIndex[1183] = [0, 1, 3, 4, 8, 10, 11]
poemBySequenceIndex[1183] = '''B D G J N R S'''
poemAuthors[1183] = ''''''
#1184: ䷹䷴ Joyous Infiltrating: 1 2 1 4 2 1 1  : S R1 R3 G3 D1 N2 N3 : Edim

#1185: C D♭ D♯ E A B♭ B :0 1 3 4 9 10 11: 1 ♭2 ♯2 3 6 ♭7 7: ma7(no 5 add ♭7 ♭9 ♯9 13)
namesBySequenceIndex[1184] = ''''''
namesBySequenceIndexNegative[1184] = ''''''
semitonesBySequenceIndex[1184] = [0, 1, 3, 4, 9, 10, 11]
poemBySequenceIndex[1184] = '''B D G J P R S'''
poemAuthors[1184] = ''''''
#1185: ䷹䷋ Joyous Obstruction: 1 2 1 5 1 1 1  : S R1 R3 G3 D2 N2 N3 : Edag

#1186: C D♭ E♭ F F♯ G A♭ :0 1 3 5 6 7 8: 1 ♭2 ♭3 4 ♯4 5 ♭6: mi(add ♭6 ♭9 11 ♯11)
namesBySequenceIndex[1185] = '''Aeolyrian'''
namesBySequenceIndexNegative[1185] = ''''''
semitonesBySequenceIndex[1185] = [0, 1, 3, 5, 6, 7, 8]
poemBySequenceIndex[1185] = '''B D G K L M N'''
poemAuthors[1185] = ''''''
#1186: ䷥䷊ Polarising Peace: 1 2 2 1 1 1 4  : S R1 G2 M1 M2 P D1 : EfOj

#1187: C D♭ E♭ F F♯ G A :0 1 3 5 6 7 9: 1 ♭2 ♭3 4 ♯4 5 6: mi6(add ♭9 11 ♯11)
namesBySequenceIndex[1186] = '''Lyneian,Lynian'''
namesBySequenceIndexNegative[1186] = ''''''
semitonesBySequenceIndex[1186] = [0, 1, 3, 5, 6, 7, 9]
poemBySequenceIndex[1186] = '''B D G K L M P'''
poemAuthors[1186] = ''''''
#1187: ䷥䷵ Polarising Betrothing: 1 2 2 1 1 2 3  : S R1 G2 M1 M2 P D2 : EfEb

#1188: C D♭ E♭ F F♯ G B♭ :0 1 3 5 6 7 10: 1 ♭2 ♭3 4 ♯4 5 ♭7: mi11(♭9 add ♯11)
namesBySequenceIndex[1187] = '''Stadian,Blues Minor (add ♭9)'''
namesBySequenceIndexNegative[1187] = ''''''
semitonesBySequenceIndex[1187] = [0, 1, 3, 5, 6, 7, 10]
poemBySequenceIndex[1187] = '''B D G K L M R'''
poemAuthors[1187] = ''''''
#1188: ䷥䷻ Polarising Composed: 1 2 2 1 1 3 2  : S R1 G2 M1 M2 P N2 : EfEk

#1189: C D♭ E♭ F F♯ G B :0 1 3 5 6 7 11: 1 ♭2 ♭3 4 ♯4 5 7: mima11(♭9 add ♯11)
namesBySequenceIndex[1188] = '''Aeolyptian'''
namesBySequenceIndexNegative[1188] = ''''''
semitonesBySequenceIndex[1188] = [0, 1, 3, 5, 6, 7, 11]
poemBySequenceIndex[1188] = '''B D G K L M S'''
poemAuthors[1188] = ''''''
#1189: ䷥䷨ Polarising Diminishing: 1 2 2 1 1 4 1  : S R1 G2 M1 M2 P N3 : EfEl

#1190: C D♭ E♭ F G♭ A♭ A :0 1 3 5 6 8 9: 1 ♭2 ♭3 4 ♭5 ♭6 6: °7(add ♭6 ♭9 11)
namesBySequenceIndex[1189] = '''Stothian,Locrian ♭♭7'''
namesBySequenceIndexNegative[1189] = ''''''
semitonesBySequenceIndex[1189] = [0, 1, 3, 5, 6, 8, 9]
poemBySequenceIndex[1189] = '''B D G K L N P'''
poemAuthors[1189] = ''''''
#1190: ䷥䷶ Polarising Abounding: 1 2 2 1 2 1 3  : S R1 G2 M1 M2 D1 N1 : EfIb

#1191: C D♭ E♭ F G♭ A♭ B♭ :0 1 3 5 6 8 10: 1 ♭2 ♭3 4 ♭5 ♭6 ♭7: ⌀11(♭9 add ♭13)
namesBySequenceIndex[1190] = '''Locrian,Pien Chih (China),Makam Lami (Jewish),Yishtabach (Jewish)'''
namesBySequenceIndexNegative[1190] = ''''''
semitonesBySequenceIndex[1190] = [0, 1, 3, 5, 6, 8, 10]
poemBySequenceIndex[1190] = '''B D G K L N R'''
poemAuthors[1190] = ''''''
#1191: ䷥䷾ Polarising Completed: 1 2 2 1 2 2 2  : S R1 G2 M1 M2 D1 N2 : EfIk

#1192: C D♭ E♭ F G♭ A♭ B :0 1 3 5 6 8 11: 1 ♭2 ♭3 4 ♭5 ♭6 7: °ma11(♭9 add ♭13)
namesBySequenceIndex[1191] = '''Aeolorian,Locrian Ma7'''
namesBySequenceIndexNegative[1191] = ''''''
semitonesBySequenceIndex[1191] = [0, 1, 3, 5, 6, 8, 11]
poemBySequenceIndex[1191] = '''B D G K L N S'''
poemAuthors[1191] = ''''''
#1192: ䷥䷕ Polarising Adorning: 1 2 2 1 2 3 1  : S R1 G2 M1 M2 D1 N3 : EfIl

#1193: C D♭ E♭ F G♭ A B♭ :0 1 3 5 6 9 10: 1 ♭2 ♭3 4 ♭5 6 ♭7: ⌀13(♭9)
namesBySequenceIndex[1192] = '''Thyptian,Locrian ♮6,Locrian ♯6,Dorian ♭9'''
namesBySequenceIndexNegative[1192] = ''''''
semitonesBySequenceIndex[1192] = [0, 1, 3, 5, 6, 9, 10]
poemBySequenceIndex[1192] = '''B D G K L P R'''
poemAuthors[1192] = ''''''
#1193: ䷥䷐ Polarising Following: 1 2 2 1 3 1 2  : S R1 G2 M1 M2 D2 N2 : EfAd

#1194: C D♭ E♭ F G♭ A B :0 1 3 5 6 9 11: 1 ♭2 ♭3 4 ♭5 6 7: °ma13(♭9)
namesBySequenceIndex[1193] = '''Byrian,Neapolitan Major ♭5'''
namesBySequenceIndexNegative[1193] = ''''''
semitonesBySequenceIndex[1193] = [0, 1, 3, 5, 6, 9, 11]
poemBySequenceIndex[1193] = '''B D G K L P S'''
poemAuthors[1193] = ''''''
#1194: ䷥䷔ Polarising Biting: 1 2 2 1 3 2 1  : S R1 G2 M1 M2 D2 N3 : EfAf

#1195: C D♭ E♭ F G♭ A♯ B :0 1 3 5 6 10 11: 1 ♭2 ♭3 4 ♭5 ♯6 7: °ma11(♭9 add ♭7)
namesBySequenceIndex[1194] = '''Daptian'''
namesBySequenceIndexNegative[1194] = ''''''
semitonesBySequenceIndex[1194] = [0, 1, 3, 5, 6, 10, 11]
poemBySequenceIndex[1194] = '''B D G K L R S'''
poemAuthors[1194] = ''''''
#1195: ䷥䷩ Polarising Augmenting: 1 2 2 1 4 1 1  : S R1 G2 M1 M2 D3 N3 : EfAm

#1196: C D♭ E♭ F G A♭ A :0 1 3 5 7 8 9: 1 ♭2 ♭3 4 5 ♭6 6: mi6(add ♭6 ♭9 11)
namesBySequenceIndex[1195] = '''Mela Senavati,Raga Malini,Raga Senagrani,Dyptian'''
namesBySequenceIndexNegative[1195] = ''''''
semitonesBySequenceIndex[1195] = [0, 1, 3, 5, 7, 8, 9]
poemBySequenceIndex[1195] = '''B D G K M N P'''
poemAuthors[1195] = ''''''
#1196: ䷥䷟ Polarising Persevering: 1 2 2 2 1 1 3  : S R1 G2 M1 P D1 N1 : Efob

#1197: C D♭ E♭ F G A♭ B♭ :0 1 3 5 7 8 10: 1 ♭2 ♭3 4 5 ♭6 ♭7: mi11(♭9 add ♭13)
namesBySequenceIndex[1196] = '''Phrygian,Ousak (Greece),Zokuso (Japan),Maqam Kurd (Iraq),Raga Dhanyasi (descending),Mela Hanumatodi,Mela Bhairavi That,Raga Bilashkhani Todi,Raga Ghanta'''
namesBySequenceIndexNegative[1196] = ''''''
semitonesBySequenceIndex[1196] = [0, 1, 3, 5, 7, 8, 10]
poemBySequenceIndex[1196] = '''B D G K M N R'''
poemAuthors[1196] = ''''''
#1197: ䷥䷯ Polarising Source: 1 2 2 2 1 2 2  : S R1 G2 M1 P D1 N2 : Efok

#1198: C D♭ E♭ F G A♭ B :0 1 3 5 7 8 11: 1 ♭2 ♭3 4 5 ♭6 7: mima11(♭9 add ♭13)
namesBySequenceIndex[1197] = '''Mela Dhenuka,Neapolitan Minor,Hungarian Gypsy,Maqam Shahnaz or Maqam Shahnaz Kurdi (Iraq),Raga Dhunibinnashadjam,Raga Bhinnasadjam,Aerylian'''
namesBySequenceIndexNegative[1197] = ''''''
semitonesBySequenceIndex[1197] = [0, 1, 3, 5, 7, 8, 11]
poemBySequenceIndex[1197] = '''B D G K M N S'''
poemAuthors[1197] = ''''''
#1198: ䷥䷑ Polarising Correcting: 1 2 2 2 1 3 1  : S R1 G2 M1 P D1 N3 : Efol

#1199: C D♭ E♭ F G A B♭ :0 1 3 5 7 9 10: 1 ♭2 ♭3 4 5 6 ♭7: mi13(♭9)
namesBySequenceIndex[1198] = '''Mela Natakapriya,Dorian ♭2,Javanese,Phrygian ♮6,Phrygian Mixolydian,Raga Natabharanam,Raga Ahiri Todi,Mixolythian'''
namesBySequenceIndexNegative[1198] = ''''''
semitonesBySequenceIndex[1198] = [0, 1, 3, 5, 7, 9, 10]
poemBySequenceIndex[1198] = '''B D G K M P R'''
poemAuthors[1198] = ''''''
#1199: ䷥䷮ Polarising Confining: 1 2 2 2 2 1 2  : S R1 G2 M1 P D2 N2 : Efed

#1200: C D♭ E♭ F G A B :0 1 3 5 7 9 11: 1 ♭2 ♭3 4 5 6 7: mima13(♭9)
namesBySequenceIndex[1199] = '''Mela Kokilapriya,Neapolitan,Raga Kokilaravam,Neopolitan Major,Thydian'''
namesBySequenceIndexNegative[1199] = ''''''
semitonesBySequenceIndex[1199] = [0, 1, 3, 5, 7, 9, 11]
poemBySequenceIndex[1199] = '''B D G K M P S'''
poemAuthors[1199] = ''''''
#1200: ䷥䷿ Polarising Unfinished: 1 2 2 2 2 2 1  : S R1 G2 M1 P D2 N3 : Efef

#1201: C D♭ E♭ F G A♯ B :0 1 3 5 7 10 11: 1 ♭2 ♭3 4 5 ♯6 7: mima11(♭9 add ♭7)
namesBySequenceIndex[1200] = '''Mela Rupavati,Staptian'''
namesBySequenceIndexNegative[1200] = ''''''
semitonesBySequenceIndex[1200] = [0, 1, 3, 5, 7, 10, 11]
poemBySequenceIndex[1200] = '''B D G K M R S'''
poemAuthors[1200] = ''''''
#1201: ䷥䷺ Polarising Dispersing: 1 2 2 2 3 1 1  : S R1 G2 M1 P D3 N3 : Efem

#1202: C D♭ E♭ F G♯ A B♭ :0 1 3 5 8 9 10: 1 ♭2 ♭3 4 ♯5 6 ♭7: mi13(♯5 ♭9)
namesBySequenceIndex[1201] = '''Byptian'''
namesBySequenceIndexNegative[1201] = ''''''
semitonesBySequenceIndex[1201] = [0, 1, 3, 5, 8, 9, 10]
poemBySequenceIndex[1201] = '''B D G K N P R'''
poemAuthors[1201] = ''''''
#1202: ䷥䷞ Polarising Conjoining: 1 2 2 3 1 1 2  : S R1 G2 M1 D1 D2 N2 : Efid

#1203: C D♭ E♭ F G♯ A B :0 1 3 5 8 9 11: 1 ♭2 ♭3 4 ♯5 6 7: mima13(♯5 ♭9)
namesBySequenceIndex[1202] = '''Phrycrian'''
namesBySequenceIndexNegative[1202] = ''''''
semitonesBySequenceIndex[1202] = [0, 1, 3, 5, 8, 9, 11]
poemBySequenceIndex[1202] = '''B D G K N P S'''
poemAuthors[1202] = ''''''
#1203: ䷥䷷ Polarising Wanderer: 1 2 2 3 1 2 1  : S R1 G2 M1 D1 D2 N3 : Efif

#1204: C D♭ E♭ F G♯ A♯ B :0 1 3 5 8 10 11: 1 ♭2 ♭3 4 ♯5 ♯6 7: mima11(♯5 ♭9 add ♭7)
namesBySequenceIndex[1203] = '''Aeolylian'''
namesBySequenceIndexNegative[1203] = ''''''
semitonesBySequenceIndex[1203] = [0, 1, 3, 5, 8, 10, 11]
poemBySequenceIndex[1203] = '''B D G K N R S'''
poemAuthors[1203] = ''''''
#1204: ䷥䷴ Polarising Infiltrating: 1 2 2 3 2 1 1  : S R1 G2 M1 D1 N2 N3 : Efim

#1205: C D♭ E♭ F A B♭ B :0 1 3 5 9 10 11: 1 ♭2 ♭3 4 6 ♭7 7: mima13(♭9 no 5 add ♭7)
namesBySequenceIndex[1204] = '''Kalian'''
namesBySequenceIndexNegative[1204] = ''''''
semitonesBySequenceIndex[1204] = [0, 1, 3, 5, 9, 10, 11]
poemBySequenceIndex[1204] = '''B D G K P R S'''
poemAuthors[1204] = ''''''
#1205: ䷥䷋ Polarising Obstruction: 1 2 2 4 1 1 1  : S R1 G2 M1 D2 N2 N3 : Efag

#1206: C D♭ E♭ F♯ G A♭ A :0 1 3 6 7 8 9: 1 ♭2 ♭3 ♯4 5 ♭6 6: mi6(add ♭6 ♭9 ♯11)
namesBySequenceIndex[1205] = '''Mela Gavambodhi,Raga Girvani,Ladian'''
namesBySequenceIndexNegative[1205] = ''''''
semitonesBySequenceIndex[1205] = [0, 1, 3, 6, 7, 8, 9]
poemBySequenceIndex[1205] = '''B D G L M N P'''
poemAuthors[1205] = ''''''
#1206: ䷵䷡ Betrothing Might: 1 2 3 1 1 1 3  : S R1 G2 M2 P D1 N1 : EbOb

#1207: C D♭ E♭ F♯ G A♭ B♭ :0 1 3 6 7 8 10: 1 ♭2 ♭3 ♯4 5 ♭6 ♭7: mi7(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1206] = '''Mela Bhavapriya,Raga Bhavani,Raga Kalamurti,Dygian'''
namesBySequenceIndexNegative[1206] = ''''''
semitonesBySequenceIndex[1206] = [0, 1, 3, 6, 7, 8, 10]
poemBySequenceIndex[1206] = '''B D G L M N R'''
poemAuthors[1206] = ''''''
#1207: ䷵䷄ Betrothing Biding: 1 2 3 1 1 2 2  : S R1 G2 M2 P D1 N2 : EbOk

#1208: C D♭ E♭ F♯ G A♭ B :0 1 3 6 7 8 11: 1 ♭2 ♭3 ♯4 5 ♭6 7: mima7(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1207] = '''Mela Shubhapanturavali,Mela Shadvidamargini,Raga Multani 1,Maqam Athar Kurd (Iraq),Raga Gamakasamantam,Ionyphian'''
namesBySequenceIndexNegative[1207] = ''''''
semitonesBySequenceIndex[1207] = [0, 1, 3, 6, 7, 8, 11]
poemBySequenceIndex[1207] = '''B D G L M N S'''
poemAuthors[1207] = ''''''
#1208: ䷵䷙ Betrothing Cultivating: 1 2 3 1 1 3 1  : S R1 G2 M2 P D1 N3 : EbOl

#1209: C D♭ E♭ F♯ G A B♭ :0 1 3 6 7 9 10: 1 ♭2 ♭3 ♯4 5 6 ♭7: mi13(♭9 ♯11)
namesBySequenceIndex[1208] = '''Mela Sadvidhmargini,Raga Tivravahini,Raga Sthavarajam,Dorian ♭9 ♯11,Phrylian'''
namesBySequenceIndexNegative[1208] = ''''''
semitonesBySequenceIndex[1208] = [0, 1, 3, 6, 7, 9, 10]
poemBySequenceIndex[1208] = '''B D G L M P R'''
poemAuthors[1208] = ''''''
#1209: ䷵䷹ Betrothing Joyous: 1 2 3 1 2 1 2  : S R1 G2 M2 P D2 N2 : EbEd

#1210: C D♭ E♭ F♯ G A B :0 1 3 6 7 9 11: 1 ♭2 ♭3 ♯4 5 6 7: mima13(♭9 ♯11)
namesBySequenceIndex[1209] = '''Mela Suvarnangi,Raga Sauviram,Manian'''
namesBySequenceIndexNegative[1209] = ''''''
semitonesBySequenceIndex[1209] = [0, 1, 3, 6, 7, 9, 11]
poemBySequenceIndex[1209] = '''B D G L M P S'''
poemAuthors[1209] = ''''''
#1210: ䷵䷥ Betrothing Polarising: 1 2 3 1 2 2 1  : S R1 G2 M2 P D2 N3 : EbEf

#1211: C D♭ E♭ F♯ G A♯ B :0 1 3 6 7 10 11: 1 ♭2 ♭3 ♯4 5 ♯6 7: mima7(add ♭7 ♭9 ♯11)
namesBySequenceIndex[1210] = '''Mela Divyamani,Raga Vamsavathi,Katadian'''
namesBySequenceIndexNegative[1210] = ''''''
semitonesBySequenceIndex[1210] = [0, 1, 3, 6, 7, 10, 11]
poemBySequenceIndex[1210] = '''B D G L M R S'''
poemAuthors[1210] = ''''''
#1211: ䷵䷼ Betrothing Truth: 1 2 3 1 3 1 1  : S R1 G2 M2 P D3 N3 : EbEm

#1212: C D♭ E♭ F♯ G♯ A B♭ :0 1 3 6 8 9 10: 1 ♭2 ♭3 ♯4 ♯5 6 ♭7: ⌀13(♭9 ♯11 add ♭13)
namesBySequenceIndex[1211] = '''Solian'''
namesBySequenceIndexNegative[1211] = ''''''
semitonesBySequenceIndex[1211] = [0, 1, 3, 6, 8, 9, 10]
poemBySequenceIndex[1211] = '''B D G L N P R'''
poemAuthors[1211] = ''''''
#1212: ䷵䷰ Betrothing Revolution: 1 2 3 2 1 1 2  : S R1 G2 M2 D1 D2 N2 : EbId

#1213: C D♭ E♭ F♯ G♯ A B :0 1 3 6 8 9 11: 1 ♭2 ♭3 ♯4 ♯5 6 7: °ma13(♭9 ♯11 add ♭13)
namesBySequenceIndex[1212] = '''Phrogian'''
namesBySequenceIndexNegative[1212] = ''''''
semitonesBySequenceIndex[1212] = [0, 1, 3, 6, 8, 9, 11]
poemBySequenceIndex[1212] = '''B D G L N P S'''
poemAuthors[1212] = ''''''
#1213: ䷵䷝ Betrothing Distanced: 1 2 3 2 1 2 1  : S R1 G2 M2 D1 D2 N3 : EbIf

#1214: C D♭ E♭ F♯ G♯ A♯ B :0 1 3 6 8 10 11: 1 ♭2 ♭3 ♯4 ♯5 ♯6 7: °ma7(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1213] = '''Enigmatic Minor,Bylian'''
namesBySequenceIndexNegative[1213] = ''''''
semitonesBySequenceIndex[1213] = [0, 1, 3, 6, 8, 10, 11]
poemBySequenceIndex[1213] = '''B D G L N R S'''
poemAuthors[1213] = ''''''
#1214: ䷵䷤ Betrothing Family: 1 2 3 2 2 1 1  : S R1 G2 M2 D1 N2 N3 : EbIm

#1215: C D♭ E♭ G♭ A B♭ B :0 1 3 6 9 10 11: 1 ♭2 ♭3 ♭5 6 ♭7 7: °ma7(add ♭7 ♭9 13)
namesBySequenceIndex[1214] = '''Polian'''
namesBySequenceIndexNegative[1214] = ''''''
semitonesBySequenceIndex[1214] = [0, 1, 3, 6, 9, 10, 11]
poemBySequenceIndex[1214] = '''B D G L P R S'''
poemAuthors[1214] = ''''''
#1215: ䷵䷘ Betrothing Integrity: 1 2 3 3 1 1 1  : S R1 G2 M2 D2 N2 N3 : EbAg

#1216: C D♭ E♭ G A♭ A B♭ :0 1 3 7 8 9 10: 1 ♭2 ♭3 5 ♭6 6 ♭7: mi7(add ♭9 ♭13 13)
namesBySequenceIndex[1215] = '''Stogian'''
namesBySequenceIndexNegative[1215] = ''''''
semitonesBySequenceIndex[1215] = [0, 1, 3, 7, 8, 9, 10]
poemBySequenceIndex[1215] = '''B D G M N P R'''
poemAuthors[1215] = ''''''
#1216: ䷵䷛ Betrothing Excess: 1 2 4 1 1 1 2  : S R1 G2 P D1 D2 N2 : Ebod

#1217: C D♭ E♭ G A♭ A B :0 1 3 7 8 9 11: 1 ♭2 ♭3 5 ♭6 6 7: mima7(add ♭9 ♭13 13)
namesBySequenceIndex[1216] = '''Thorian'''
namesBySequenceIndexNegative[1216] = ''''''
semitonesBySequenceIndex[1216] = [0, 1, 3, 7, 8, 9, 11]
poemBySequenceIndex[1216] = '''B D G M N P S'''
poemAuthors[1216] = ''''''
#1217: ䷵䷱ Betrothing Holding: 1 2 4 1 1 2 1  : S R1 G2 P D1 D2 N3 : Ebof

#1218: C D♭ E♭ G A♭ B♭ B :0 1 3 7 8 10 11: 1 ♭2 ♭3 5 ♭6 ♭7 7: mima7(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1217] = '''Katonian'''
namesBySequenceIndexNegative[1217] = ''''''
semitonesBySequenceIndex[1217] = [0, 1, 3, 7, 8, 10, 11]
poemBySequenceIndex[1217] = '''B D G M N R S'''
poemAuthors[1217] = ''''''
#1218: ䷵䷸ Betrothing Osmosis: 1 2 4 1 2 1 1  : S R1 G2 P D1 N2 N3 : Ebom

#1219: C D♭ E♭ G A B♭ B :0 1 3 7 9 10 11: 1 ♭2 ♭3 5 6 ♭7 7: mima7(add ♭7 ♭9 13)
namesBySequenceIndex[1218] = '''Myptian'''
namesBySequenceIndexNegative[1218] = ''''''
semitonesBySequenceIndex[1218] = [0, 1, 3, 7, 9, 10, 11]
poemBySequenceIndex[1218] = '''B D G M P R S'''
poemAuthors[1218] = ''''''
#1219: ䷵䷅ Betrothing Arguing: 1 2 4 2 1 1 1  : S R1 G2 P D2 N2 N3 : Ebeg

#1220: C D♭ E♭ G♯ A B♭ B :0 1 3 8 9 10 11: 1 ♭2 ♭3 ♯5 6 ♭7 7: mima7(♯5 add ♭7 ♭9 13)
namesBySequenceIndex[1219] = ''''''
namesBySequenceIndexNegative[1219] = ''''''
semitonesBySequenceIndex[1219] = [0, 1, 3, 8, 9, 10, 11]
poemBySequenceIndex[1219] = '''B D G N P R S'''
poemAuthors[1219] = ''''''
#1220: ䷵䷠ Betrothing Retiring: 1 2 5 1 1 1 1  : S R1 G2 D1 D2 N2 N3 : Ebig

#1221: C D♭ E F F♯ G A♭ :0 1 4 5 6 7 8: 1 ♭2 3 4 ♯4 5 ♭6: ma(add ♭6 ♭9 11 ♯11)
namesBySequenceIndex[1220] = '''Ionaptian'''
namesBySequenceIndexNegative[1220] = ''''''
semitonesBySequenceIndex[1220] = [0, 1, 4, 5, 6, 7, 8]
poemBySequenceIndex[1220] = '''B D J K L M N'''
poemAuthors[1220] = ''''''
#1221: ䷼䷊ Truth Peace: 1 3 1 1 1 1 4  : S R1 G3 M1 M2 P D1 : EmOj

#1222: C D♭ E F F♯ G A :0 1 4 5 6 7 9: 1 ♭2 3 4 ♯4 5 6: 6(add ♭9 11 ♯11)
namesBySequenceIndex[1221] = '''Phrythian'''
namesBySequenceIndexNegative[1221] = ''''''
semitonesBySequenceIndex[1221] = [0, 1, 4, 5, 6, 7, 9]
poemBySequenceIndex[1221] = '''B D J K L M P'''
poemAuthors[1221] = ''''''
#1222: ䷼䷵ Truth Betrothing: 1 3 1 1 1 2 3  : S R1 G3 M1 M2 P D2 : EmEb

#1223: C D♭ E F F♯ G B♭ :0 1 4 5 6 7 10: 1 ♭2 3 4 ♯4 5 ♭7: 11(♭9 add ♯11)
namesBySequenceIndex[1222] = '''Katynian'''
namesBySequenceIndexNegative[1222] = ''''''
semitonesBySequenceIndex[1222] = [0, 1, 4, 5, 6, 7, 10]
poemBySequenceIndex[1222] = '''B D J K L M R'''
poemAuthors[1222] = ''''''
#1223: ䷼䷻ Truth Composed: 1 3 1 1 1 3 2  : S R1 G3 M1 M2 P N2 : EmEk

#1224: C D♭ E F F♯ G B :0 1 4 5 6 7 11: 1 ♭2 3 4 ♯4 5 7: ma11(♭9 add ♯11)
namesBySequenceIndex[1223] = '''Zydian'''
namesBySequenceIndexNegative[1223] = ''''''
semitonesBySequenceIndex[1223] = [0, 1, 4, 5, 6, 7, 11]
poemBySequenceIndex[1223] = '''B D J K L M S'''
poemAuthors[1223] = ''''''
#1224: ䷼䷨ Truth Diminishing: 1 3 1 1 1 4 1  : S R1 G3 M1 M2 P N3 : EmEl

#1225: C D♭ E F G♭ A♭ A :0 1 4 5 6 8 9: 1 ♭2 3 4 ♭5 ♭6 6: 6(♯5 add ♭9 11 ♯11)
namesBySequenceIndex[1224] = '''Ralian'''
namesBySequenceIndexNegative[1224] = ''''''
semitonesBySequenceIndex[1224] = [0, 1, 4, 5, 6, 8, 9]
poemBySequenceIndex[1224] = '''B D J K L N P'''
poemAuthors[1224] = ''''''
#1225: ䷼䷶ Truth Abounding: 1 3 1 1 2 1 3  : S R1 G3 M1 M2 D1 N1 : EmIb

#1226: C D♭ E F G♭ A♭ B♭ :0 1 4 5 6 8 10: 1 ♭2 3 4 ♭5 ♭6 ♭7: 11(♯5 ♭9 add ♯11)
namesBySequenceIndex[1225] = '''Rahawi,Locrian Dominant,Rahawi (Iraq),Mixonorian'''
namesBySequenceIndexNegative[1225] = ''''''
semitonesBySequenceIndex[1225] = [0, 1, 4, 5, 6, 8, 10]
poemBySequenceIndex[1225] = '''B D J K L N R'''
poemAuthors[1225] = ''''''
#1226: ䷼䷾ Truth Completed: 1 3 1 1 2 2 2  : S R1 G3 M1 M2 D1 N2 : EmIk

#1227: C D♭ E F G♭ A♭ B :0 1 4 5 6 8 11: 1 ♭2 3 4 ♭5 ♭6 7: ma11(♯5 ♭9 add ♯11)
namesBySequenceIndex[1226] = '''Persian,Raga Suddha Pancama,Paptian'''
namesBySequenceIndexNegative[1226] = ''''''
semitonesBySequenceIndex[1226] = [0, 1, 4, 5, 6, 8, 11]
poemBySequenceIndex[1226] = '''B D J K L N S'''
poemAuthors[1226] = ''''''
#1227: ䷼䷕ Truth Adorning: 1 3 1 1 2 3 1  : S R1 G3 M1 M2 D1 N3 : EmIl

#1228: C D♭ E F G♭ A B♭ :0 1 4 5 6 9 10: 1 ♭2 3 4 ♭5 6 ♭7: 13(♭5 ♭9)
namesBySequenceIndex[1227] = '''Tsinganikos,Raga Ahira Lalita,Tsinganikos (Greece),Mogian'''
namesBySequenceIndexNegative[1227] = ''''''
semitonesBySequenceIndex[1227] = [0, 1, 4, 5, 6, 9, 10]
poemBySequenceIndex[1227] = '''B D J K L P R'''
poemAuthors[1227] = ''''''
#1228: ䷼䷐ Truth Following: 1 3 1 1 3 1 2  : S R1 G3 M1 M2 D2 N2 : EmAd

#1229: C D♭ E F G♭ A B :0 1 4 5 6 9 11: 1 ♭2 3 4 ♭5 6 7: ma13(♭5 ♭9)
namesBySequenceIndex[1228] = '''Raga Lalit,Raga Bhankar,Chromatic Lydian,Gogian'''
namesBySequenceIndexNegative[1228] = ''''''
semitonesBySequenceIndex[1228] = [0, 1, 4, 5, 6, 9, 11]
poemBySequenceIndex[1228] = '''B D J K L P S'''
poemAuthors[1228] = ''''''
#1229: ䷼䷔ Truth Biting: 1 3 1 1 3 2 1  : S R1 G3 M1 M2 D2 N3 : EmAf

#1230: C D♭ E F G♭ A♯ B :0 1 4 5 6 10 11: 1 ♭2 3 4 ♭5 ♯6 7: ma11(♭5 ♭9 add ♭7)
namesBySequenceIndex[1229] = '''Koptian'''
namesBySequenceIndexNegative[1229] = ''''''
semitonesBySequenceIndex[1229] = [0, 1, 4, 5, 6, 10, 11]
poemBySequenceIndex[1229] = '''B D J K L R S'''
poemAuthors[1229] = ''''''
#1230: ䷼䷩ Truth Augmenting: 1 3 1 1 4 1 1  : S R1 G3 M1 M2 D3 N3 : EmAm

#1231: C D♭ E F G A♭ A :0 1 4 5 7 8 9: 1 ♭2 3 4 5 ♭6 6: 6(add ♭6 ♭9 11)
namesBySequenceIndex[1230] = '''Mela Gayakapriya,Raga Kalakanthi 1,Gypsy Sexatonic,Modian'''
namesBySequenceIndexNegative[1230] = ''''''
semitonesBySequenceIndex[1230] = [0, 1, 4, 5, 7, 8, 9]
poemBySequenceIndex[1230] = '''B D J K M N P'''
poemAuthors[1230] = ''''''
#1231: ䷼䷟ Truth Persevering: 1 3 1 2 1 1 3  : S R1 G3 M1 P D1 N1 : Emob

#1232: C D♭ E F G A♭ B♭ :0 1 4 5 7 8 10: 1 ♭2 3 4 5 ♭6 ♭7: 11(♭9 add ♭13)
namesBySequenceIndex[1231] = '''Phrygian Dominant,Phrygian Major,Spanish,Spanish Gypsy,Zilof (Spain),Dorico Flamenco (Spain),Jewish,Avaha or Ahava Rabba (Jewish),Freygish (or Fraigish),Hitzaz (or Hijaz,Greece),Alhijaz (Saudi Arabian),Maqam Humayun (Iraq),Maqam Zengule (Iraq),Maqam Hijaz-Nahawand (Iraq),Humayun (Iraq),Mela Vakulabharanam,Raga Jogiya,Raga Vativasanta,Ionalian'''
namesBySequenceIndexNegative[1231] = ''''''
semitonesBySequenceIndex[1231] = [0, 1, 4, 5, 7, 8, 10]
poemBySequenceIndex[1231] = '''B D J K M N R'''
poemAuthors[1231] = ''''''
#1232: ䷼䷯ Truth Source: 1 3 1 2 1 2 2  : S R1 G3 M1 P D1 N2 : Emok

#1233: C D♭ E F G A♭ B :0 1 4 5 7 8 11: 1 ♭2 3 4 5 ♭6 7: ma11(♭9 add ♭13)
namesBySequenceIndex[1232] = '''Double Harmonic,Double Harmonic Major,Major Phrygian,Byzantine or Byzantine Liturgical Chromatic,Major Gypsy,Hungarian Folk,Hitzazkiar or Hijazskiar (Greece),Maqam Hijaz Kar (Iraq),Maqam Suzidil (Iraq),Hijaz Kar (Saudi Arabian),Arabic,Mela Mayamalavagowla,Raga Saveri (descending),Raga Paras (descending),Raga Kalingada,Raga Paraj,Raga Bhairav That,Raga Gaulipantu,Raga Lalita Panchami,Aerynian'''
namesBySequenceIndexNegative[1232] = ''''''
semitonesBySequenceIndex[1232] = [0, 1, 4, 5, 7, 8, 11]
poemBySequenceIndex[1232] = '''B D J K M N S'''
poemAuthors[1232] = ''''''
#1233: ䷼䷑ Truth Correcting: 1 3 1 2 1 3 1  : S R1 G3 M1 P D1 N3 : Emol

#1234: C D♭ E F G A B♭ :0 1 4 5 7 9 10: 1 ♭2 3 4 5 6 ♭7: 13(♭9)
namesBySequenceIndex[1233] = '''Mela Cakravka,Mela Chakravakam,Mixolydian ♭2,Maqam Zanjaran,Maqam Hicaz (Iraq),Raga Ahir Bhairav,Raga Bindumalini,Raga Vagadeeshwari,Raga Vegavahini (descending),Maqam Zanjaran (Iraq),Aeronian'''
namesBySequenceIndexNegative[1233] = ''''''
semitonesBySequenceIndex[1233] = [0, 1, 4, 5, 7, 9, 10]
poemBySequenceIndex[1233] = '''B D J K M P R'''
poemAuthors[1233] = ''''''
#1234: ䷼䷮ Truth Confining: 1 3 1 2 2 1 2  : S R1 G3 M1 P D2 N2 : Emed

#1235: C D♭ E F G A B :0 1 4 5 7 9 11: 1 ♭2 3 4 5 6 7: ma13(♭9)
namesBySequenceIndex[1234] = '''Mela Suryakanta,Mela Suryakantam,Raga Bhairubahar,Raga Sowrashtram,Raga Supradhipam,Zanian,Raga Bhatiyar (descending)'''
namesBySequenceIndexNegative[1234] = ''''''
semitonesBySequenceIndex[1234] = [0, 1, 4, 5, 7, 9, 11]
poemBySequenceIndex[1234] = '''B D J K M P S'''
poemAuthors[1234] = ''''''
#1235: ䷼䷿ Truth Unfinished: 1 3 1 2 2 2 1  : S R1 G3 M1 P D2 N3 : Emef

#1236: C D♭ E F G A♯ B :0 1 4 5 7 10 11: 1 ♭2 3 4 5 ♯6 7: ma11(♭9 add ♭7)
namesBySequenceIndex[1235] = '''Mela Hatakambari,Raga Jeyasuddhamalavi,Zodian'''
namesBySequenceIndexNegative[1235] = ''''''
semitonesBySequenceIndex[1235] = [0, 1, 4, 5, 7, 10, 11]
poemBySequenceIndex[1235] = '''B D J K M R S'''
poemAuthors[1235] = ''''''
#1236: ䷼䷺ Truth Dispersing: 1 3 1 2 3 1 1  : S R1 G3 M1 P D3 N3 : Emem

#1237: C D♭ E F G♯ A B♭ :0 1 4 5 8 9 10: 1 ♭2 3 4 ♯5 6 ♭7: 13(♯5 ♭9)
namesBySequenceIndex[1236] = '''Ionygian,Mixolydian Augmented ♭9,Mixolydian ♯5 ♭9'''
namesBySequenceIndexNegative[1236] = ''''''
semitonesBySequenceIndex[1236] = [0, 1, 4, 5, 8, 9, 10]
poemBySequenceIndex[1236] = '''B D J K N P R'''
poemAuthors[1236] = ''''''
#1237: ䷼䷞ Truth Conjoining: 1 3 1 3 1 1 2  : S R1 G3 M1 D1 D2 N2 : Emid

#1238: C D♭ E F G♯ A B :0 1 4 5 8 9 11: 1 ♭2 3 4 ♯5 6 7: ma13(♯5 ♭9)
namesBySequenceIndex[1237] = '''Socrian,Ionian ♯5 ♭9,Ionian Augmented ♭9'''
namesBySequenceIndexNegative[1237] = ''''''
semitonesBySequenceIndex[1237] = [0, 1, 4, 5, 8, 9, 11]
poemBySequenceIndex[1237] = '''B D J K N P S'''
poemAuthors[1237] = ''''''
#1238: ䷼䷷ Truth Wanderer: 1 3 1 3 1 2 1  : S R1 G3 M1 D1 D2 N3 : Emif

#1239: C D♭ E F G♯ A♯ B :0 1 4 5 8 10 11: 1 ♭2 3 4 ♯5 ♯6 7: ma11(♯5 ♭9 add ♭7)
namesBySequenceIndex[1238] = '''Enigmatic (descending),Verdi,Sothian'''
namesBySequenceIndexNegative[1238] = ''''''
semitonesBySequenceIndex[1238] = [0, 1, 4, 5, 8, 10, 11]
poemBySequenceIndex[1238] = '''B D J K N R S'''
poemAuthors[1238] = ''''''
#1239: ䷼䷴ Truth Infiltrating: 1 3 1 3 2 1 1  : S R1 G3 M1 D1 N2 N3 : Emim

#1240: C D♭ E F A B♭ B :0 1 4 5 9 10 11: 1 ♭2 3 4 6 ♭7 7: ma13(♭9 no 5 add ♭7)
namesBySequenceIndex[1239] = '''Katygian'''
namesBySequenceIndexNegative[1239] = ''''''
semitonesBySequenceIndex[1239] = [0, 1, 4, 5, 9, 10, 11]
poemBySequenceIndex[1239] = '''B D J K P R S'''
poemAuthors[1239] = ''''''
#1240: ䷼䷋ Truth Obstruction: 1 3 1 4 1 1 1  : S R1 G3 M1 D2 N2 N3 : Emag

#1241: C D♭ E F♯ G A♭ A :0 1 4 6 7 8 9: 1 ♭2 3 ♯4 5 ♭6 6: 6(add ♭6 ♭9 ♯11)
namesBySequenceIndex[1240] = '''Mela Dhavalambari,John Fould's Mantra of Will,Thogian'''
namesBySequenceIndexNegative[1240] = ''''''
semitonesBySequenceIndex[1240] = [0, 1, 4, 6, 7, 8, 9]
poemBySequenceIndex[1240] = '''B D J L M N P'''
poemAuthors[1240] = ''''''
#1241: ䷻䷡ Composed Might: 1 3 2 1 1 1 3  : S R1 G3 M2 P D1 N1 : EkOb

#1242: C D♭ E F♯ G A♭ B♭ :0 1 4 6 7 8 10: 1 ♭2 3 ♯4 5 ♭6 ♭7: 7(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1241] = '''Mela Namanarayani,Raga Pratapa,Raga Purvi Thaat,Raga Narmada,Rynian'''
namesBySequenceIndexNegative[1241] = ''''''
semitonesBySequenceIndex[1241] = [0, 1, 4, 6, 7, 8, 10]
poemBySequenceIndex[1241] = '''B D J L M N R'''
poemAuthors[1241] = ''''''
#1242: ䷻䷄ Composed Biding: 1 3 2 1 1 2 2  : S R1 G3 M2 P D1 N2 : EkOk

#1243: C D♭ E F♯ G A♭ B :0 1 4 6 7 8 11: 1 ♭2 3 ♯4 5 ♭6 7: ma7(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1242] = '''Mela Kamavardhani,Raga Dhipaka,Raga Bhairav (ascending),Pireotikos (Greece),Raga Pantuvarali,Raga Purvi,Raga Basant (descending),Raga Kasiramakryia,Raga Puriya Dhanashri,Raga Shri,Raga Suddha Ramakriya,Chromatic Hypolydian,Stylian'''
namesBySequenceIndexNegative[1242] = ''''''
semitonesBySequenceIndex[1242] = [0, 1, 4, 6, 7, 8, 11]
poemBySequenceIndex[1242] = '''B D J L M N S'''
poemAuthors[1242] = ''''''
#1243: ䷻䷙ Composed Cultivating: 1 3 2 1 1 3 1  : S R1 G3 M2 P D1 N3 : EkOl

#1244: C D♭ E F♯ G A B♭ :0 1 4 6 7 9 10: 1 ♭2 3 ♯4 5 6 ♭7: 13(♭9 ♯11)
namesBySequenceIndex[1243] = '''Mela Ramapriya,Romanian Major,Raga Nasikabhushani,Raga Ramamahohari,Epalian'''
namesBySequenceIndexNegative[1243] = ''''''
semitonesBySequenceIndex[1243] = [0, 1, 4, 6, 7, 9, 10]
poemBySequenceIndex[1243] = '''B D J L M P R'''
poemAuthors[1243] = ''''''
#1244: ䷻䷹ Composed Joyous: 1 3 2 1 2 1 2  : S R1 G3 M2 P D2 N2 : EkEd

#1245: C D♭ E F♯ G A B :0 1 4 6 7 9 11: 1 ♭2 3 ♯4 5 6 7: ma13(♭9 ♯11)
namesBySequenceIndex[1244] = '''Mela Gamanasrama,Raga Puriya,Peiraiotikos (Greece),Raga Marwa Thaat,Raga Partiravan,Raga Puriya Kalyan,Raga	Purvikalyani,Raga Sohani,Raga Bairari,Garian'''
namesBySequenceIndexNegative[1244] = ''''''
semitonesBySequenceIndex[1244] = [0, 1, 4, 6, 7, 9, 11]
poemBySequenceIndex[1244] = '''B D J L M P S'''
poemAuthors[1244] = ''''''
#1245: ䷻䷥ Composed Polarising: 1 3 2 1 2 2 1  : S R1 G3 M2 P D2 N3 : EkEf

#1246: C D♭ E F♯ G A♯ B :0 1 4 6 7 10 11: 1 ♭2 3 ♯4 5 ♯6 7: ma7(add ♭7 ♭9 ♯11)
namesBySequenceIndex[1245] = '''Mela Visvambhari,Lyrian'''
namesBySequenceIndexNegative[1245] = ''''''
semitonesBySequenceIndex[1245] = [0, 1, 4, 6, 7, 10, 11]
poemBySequenceIndex[1245] = '''B D J L M R S'''
poemAuthors[1245] = ''''''
#1246: ䷻䷼ Composed Truth: 1 3 2 1 3 1 1  : S R1 G3 M2 P D3 N3 : EkEm

#1247: C D♭ E F♯ G♯ A B♭ :0 1 4 6 8 9 10: 1 ♭2 3 ♯4 ♯5 6 ♭7: 13(♯5 ♭9 ♯11)
namesBySequenceIndex[1246] = '''Epyphian'''
namesBySequenceIndexNegative[1246] = ''''''
semitonesBySequenceIndex[1246] = [0, 1, 4, 6, 8, 9, 10]
poemBySequenceIndex[1246] = '''B D J L N P R'''
poemAuthors[1246] = ''''''
#1247: ䷻䷰ Composed Revolution: 1 3 2 2 1 1 2  : S R1 G3 M2 D1 D2 N2 : EkId

#1248: C D♭ E F♯ G♯ A B :0 1 4 6 8 9 11: 1 ♭2 3 ♯4 ♯5 6 7: ma13(♯5 ♭9 ♯11)
namesBySequenceIndex[1247] = '''Kagian'''
namesBySequenceIndexNegative[1247] = ''''''
semitonesBySequenceIndex[1247] = [0, 1, 4, 6, 8, 9, 11]
poemBySequenceIndex[1247] = '''B D J L N P S'''
poemAuthors[1247] = ''''''
#1248: ䷻䷝ Composed Distanced: 1 3 2 2 1 2 1  : S R1 G3 M2 D1 D2 N3 : EkIf

#1249: C D♭ E F♯ G♯ A♯ B :0 1 4 6 8 10 11: 1 ♭2 3 ♯4 ♯5 ♯6 7: ma7(♯5 add ♭7 ♭9 ♯11)
namesBySequenceIndex[1248] = '''Enigmatic (ascending),Ionathian'''
namesBySequenceIndexNegative[1248] = ''''''
semitonesBySequenceIndex[1248] = [0, 1, 4, 6, 8, 10, 11]
poemBySequenceIndex[1248] = '''B D J L N R S'''
poemAuthors[1248] = ''''''
#1249: ䷻䷤ Composed Family: 1 3 2 2 2 1 1  : S R1 G3 M2 D1 N2 N3 : EkIm

#1250: C D♭ E G♭ A B♭ B :0 1 4 6 9 10 11: 1 ♭2 3 ♭5 6 ♭7 7: ma7(♭5 add ♭7 ♭9 13)
namesBySequenceIndex[1249] = '''Kaptian'''
namesBySequenceIndexNegative[1249] = ''''''
semitonesBySequenceIndex[1249] = [0, 1, 4, 6, 9, 10, 11]
poemBySequenceIndex[1249] = '''B D J L P R S'''
poemAuthors[1249] = ''''''
#1250: ䷻䷘ Composed Integrity: 1 3 2 3 1 1 1  : S R1 G3 M2 D2 N2 N3 : EkAg

#1251: C D♭ E G A♭ A B♭ :0 1 4 7 8 9 10: 1 ♭2 3 5 ♭6 6 ♭7: 7(add ♭9 ♭13 13)
namesBySequenceIndex[1250] = '''Dathian'''
namesBySequenceIndexNegative[1250] = ''''''
semitonesBySequenceIndex[1250] = [0, 1, 4, 7, 8, 9, 10]
poemBySequenceIndex[1250] = '''B D J M N P R'''
poemAuthors[1250] = ''''''
#1251: ䷻䷛ Composed Excess: 1 3 3 1 1 1 2  : S R1 G3 P D1 D2 N2 : Ekod

#1252: C D♭ E G A♭ A B :0 1 4 7 8 9 11: 1 ♭2 3 5 ♭6 6 7: ma7(add ♭9 ♭13 13)
namesBySequenceIndex[1251] = '''Bygian'''
namesBySequenceIndexNegative[1251] = ''''''
semitonesBySequenceIndex[1251] = [0, 1, 4, 7, 8, 9, 11]
poemBySequenceIndex[1251] = '''B D J M N P S'''
poemAuthors[1251] = ''''''
#1252: ䷻䷱ Composed Holding: 1 3 3 1 1 2 1  : S R1 G3 P D1 D2 N3 : Ekof

#1253: C D♭ E G A♭ B♭ B :0 1 4 7 8 10 11: 1 ♭2 3 5 ♭6 ♭7 7: ma7(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1252] = '''Kylian'''
namesBySequenceIndexNegative[1252] = ''''''
semitonesBySequenceIndex[1252] = [0, 1, 4, 7, 8, 10, 11]
poemBySequenceIndex[1252] = '''B D J M N R S'''
poemAuthors[1252] = ''''''
#1253: ䷻䷸ Composed Osmosis: 1 3 3 1 2 1 1  : S R1 G3 P D1 N2 N3 : Ekom

#1254: C D♭ E G A B♭ B :0 1 4 7 9 10 11: 1 ♭2 3 5 6 ♭7 7: ma7(add ♭7 ♭9 13)
namesBySequenceIndex[1253] = '''Aeryrian'''
namesBySequenceIndexNegative[1253] = ''''''
semitonesBySequenceIndex[1253] = [0, 1, 4, 7, 9, 10, 11]
poemBySequenceIndex[1253] = '''B D J M P R S'''
poemAuthors[1253] = ''''''
#1254: ䷻䷅ Composed Arguing: 1 3 3 2 1 1 1  : S R1 G3 P D2 N2 N3 : Ekeg

#1255: C D♭ E G♯ A B♭ B :0 1 4 8 9 10 11: 1 ♭2 3 ♯5 6 ♭7 7: ma7(♯5 add ♭7 ♭9 13)
namesBySequenceIndex[1254] = '''Aeolarian'''
namesBySequenceIndexNegative[1254] = ''''''
semitonesBySequenceIndex[1254] = [0, 1, 4, 8, 9, 10, 11]
poemBySequenceIndex[1254] = '''B D J N P R S'''
poemAuthors[1254] = ''''''
#1255: ䷻䷠ Composed Retiring: 1 3 4 1 1 1 1  : S R1 G3 D1 D2 N2 N3 : Ekig

#1256: C D♭ F F♯ G A♭ A :0 1 5 6 7 8 9: 1 ♭2 4 ♯4 5 ♭6 6: 6sus4(add ♭6 ♭9 ♯11)
namesBySequenceIndex[1255] = '''Phrathian'''
namesBySequenceIndexNegative[1255] = ''''''
semitonesBySequenceIndex[1255] = [0, 1, 5, 6, 7, 8, 9]
poemBySequenceIndex[1255] = '''B D K L M N P'''
poemAuthors[1255] = ''''''
#1256: ䷨䷡ Diminishing Might: 1 4 1 1 1 1 3  : S R1 M1 M2 P D1 N1 : ElOb

#1257: C D♭ F F♯ G A♭ B♭ :0 1 5 6 7 8 10: 1 ♭2 4 ♯4 5 ♭6 ♭7: 11sus♯4(♭9 add ♭13)
namesBySequenceIndex[1256] = '''Zynian'''
namesBySequenceIndexNegative[1256] = ''''''
semitonesBySequenceIndex[1256] = [0, 1, 5, 6, 7, 8, 10]
poemBySequenceIndex[1256] = '''B D K L M N R'''
poemAuthors[1256] = ''''''
#1257: ䷨䷄ Diminishing Biding: 1 4 1 1 1 2 2  : S R1 M1 M2 P D1 N2 : ElOk

#1258: C D♭ F F♯ G A♭ B :0 1 5 6 7 8 11: 1 ♭2 4 ♯4 5 ♭6 7: ma11sus♯4(♭9 add ♭13)
namesBySequenceIndex[1257] = '''Danian'''
namesBySequenceIndexNegative[1257] = ''''''
semitonesBySequenceIndex[1257] = [0, 1, 5, 6, 7, 8, 11]
poemBySequenceIndex[1257] = '''B D K L M N S'''
poemAuthors[1257] = ''''''
#1258: ䷨䷙ Diminishing Cultivating: 1 4 1 1 1 3 1  : S R1 M1 M2 P D1 N3 : ElOl

#1259: C D♭ F F♯ G A B♭ :0 1 5 6 7 9 10: 1 ♭2 4 ♯4 5 6 ♭7: 13sus♯4(♭9)
namesBySequenceIndex[1258] = '''Katalian'''
namesBySequenceIndexNegative[1258] = ''''''
semitonesBySequenceIndex[1258] = [0, 1, 5, 6, 7, 9, 10]
poemBySequenceIndex[1258] = '''B D K L M P R'''
poemAuthors[1258] = ''''''
#1259: ䷨䷹ Diminishing Joyous: 1 4 1 1 2 1 2  : S R1 M1 M2 P D2 N2 : ElEd

#1260: C D♭ F F♯ G A B :0 1 5 6 7 9 11: 1 ♭2 4 ♯4 5 6 7: ma13sus♯4(♭9)
namesBySequenceIndex[1259] = '''Zyrian'''
namesBySequenceIndexNegative[1259] = ''''''
semitonesBySequenceIndex[1259] = [0, 1, 5, 6, 7, 9, 11]
poemBySequenceIndex[1259] = '''B D K L M P S'''
poemAuthors[1259] = ''''''
#1260: ䷨䷥ Diminishing Polarising: 1 4 1 1 2 2 1  : S R1 M1 M2 P D2 N3 : ElEf

#1261: C D♭ F F♯ G A♯ B :0 1 5 6 7 10 11: 1 ♭2 4 ♯4 5 ♯6 7: ma11sus♯4(♭9 add ♭7)
namesBySequenceIndex[1260] = '''Syptian'''
namesBySequenceIndexNegative[1260] = ''''''
semitonesBySequenceIndex[1260] = [0, 1, 5, 6, 7, 10, 11]
poemBySequenceIndex[1260] = '''B D K L M R S'''
poemAuthors[1260] = ''''''
#1261: ䷨䷼ Diminishing Truth: 1 4 1 1 3 1 1  : S R1 M1 M2 P D3 N3 : ElEm

#1262: C D♭ F G♭ A♭ A B♭ :0 1 5 6 8 9 10: 1 ♭2 4 ♭5 ♭6 6 ♭7: 13sus♯4(♯5 ♭9)
namesBySequenceIndex[1261] = '''Hijaz Major,Gypsy 2,Hijaz Major (Greece),Thalian'''
namesBySequenceIndexNegative[1261] = ''''''
semitonesBySequenceIndex[1261] = [0, 1, 5, 6, 8, 9, 10]
poemBySequenceIndex[1261] = '''B D K L N P R'''
poemAuthors[1261] = ''''''
#1262: ䷨䷰ Diminishing Revolution: 1 4 1 2 1 1 2  : S R1 M1 M2 D1 D2 N2 : ElId

#1263: C D♭ F G♭ A♭ A B :0 1 5 6 8 9 11: 1 ♭2 4 ♭5 ♭6 6 7: ma13sus♯4(♯5 ♭9)
namesBySequenceIndex[1262] = '''Aeolydian'''
namesBySequenceIndexNegative[1262] = ''''''
semitonesBySequenceIndex[1262] = [0, 1, 5, 6, 8, 9, 11]
poemBySequenceIndex[1262] = '''B D K L N P S'''
poemAuthors[1262] = ''''''
#1263: ䷨䷝ Diminishing Distanced: 1 4 1 2 1 2 1  : S R1 M1 M2 D1 D2 N3 : ElIf

#1264: C D♭ F G♭ A♭ B♭ B :0 1 5 6 8 10 11: 1 ♭2 4 ♭5 ♭6 ♭7 7: ma11sus♯4(♯5 ♭9 add ♭7)
namesBySequenceIndex[1263] = '''Zacrian'''
namesBySequenceIndexNegative[1263] = ''''''
semitonesBySequenceIndex[1263] = [0, 1, 5, 6, 8, 10, 11]
poemBySequenceIndex[1263] = '''B D K L N R S'''
poemAuthors[1263] = ''''''
#1264: ䷨䷤ Diminishing Family: 1 4 1 2 2 1 1  : S R1 M1 M2 D1 N2 N3 : ElIm

#1265: C D♭ F G♭ A B♭ B :0 1 5 6 9 10 11: 1 ♭2 4 ♭5 6 ♭7 7: ma13(♭5 ♭9 no 3 add ♭7)
namesBySequenceIndex[1264] = '''Dycrian'''
namesBySequenceIndexNegative[1264] = ''''''
semitonesBySequenceIndex[1264] = [0, 1, 5, 6, 9, 10, 11]
poemBySequenceIndex[1264] = '''B D K L P R S'''
poemAuthors[1264] = ''''''
#1265: ䷨䷘ Diminishing Integrity: 1 4 1 3 1 1 1  : S R1 M1 M2 D2 N2 N3 : ElAg

#1266: C D♭ F G A♭ A B♭ :0 1 5 7 8 9 10: 1 ♭2 4 5 ♭6 6 ♭7: 13(♭9 no 3 add ♭13)
namesBySequenceIndex[1265] = '''Sonian'''
namesBySequenceIndexNegative[1265] = ''''''
semitonesBySequenceIndex[1265] = [0, 1, 5, 7, 8, 9, 10]
poemBySequenceIndex[1265] = '''B D K M N P R'''
poemAuthors[1265] = ''''''
#1266: ䷨䷛ Diminishing Excess: 1 4 2 1 1 1 2  : S R1 M1 P D1 D2 N2 : Elod

#1267: C D♭ F G A♭ A B :0 1 5 7 8 9 11: 1 ♭2 4 5 ♭6 6 7: ma13(♭9 no 3 add ♭13)
namesBySequenceIndex[1266] = '''Gyptian'''
namesBySequenceIndexNegative[1266] = ''''''
semitonesBySequenceIndex[1266] = [0, 1, 5, 7, 8, 9, 11]
poemBySequenceIndex[1266] = '''B D K M N P S'''
poemAuthors[1266] = ''''''
#1267: ䷨䷱ Diminishing Holding: 1 4 2 1 1 2 1  : S R1 M1 P D1 D2 N3 : Elof

#1268: C D♭ F G A♭ B♭ B :0 1 5 7 8 10 11: 1 ♭2 4 5 ♭6 ♭7 7: ma7sus4(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1267] = '''Tharian'''
namesBySequenceIndexNegative[1267] = ''''''
semitonesBySequenceIndex[1267] = [0, 1, 5, 7, 8, 10, 11]
poemBySequenceIndex[1267] = '''B D K M N R S'''
poemAuthors[1267] = ''''''
#1268: ䷨䷸ Diminishing Osmosis: 1 4 2 1 2 1 1  : S R1 M1 P D1 N2 N3 : Elom

#1269: C D♭ F G A B♭ B :0 1 5 7 9 10 11: 1 ♭2 4 5 6 ♭7 7: ma13(♭9 no 3 add ♭7)
namesBySequenceIndex[1268] = '''Myrian'''
namesBySequenceIndexNegative[1268] = ''''''
semitonesBySequenceIndex[1268] = [0, 1, 5, 7, 9, 10, 11]
poemBySequenceIndex[1268] = '''B D K M P R S'''
poemAuthors[1268] = ''''''
#1269: ䷨䷅ Diminishing Arguing: 1 4 2 2 1 1 1  : S R1 M1 P D2 N2 N3 : Eleg

#1270: C D♭ F G♯ A B♭ B :0 1 5 8 9 10 11: 1 ♭2 4 ♯5 6 ♭7 7: ma13(♯5 ♭9 no 3 add ♭7)
namesBySequenceIndex[1269] = '''Aeryptian'''
namesBySequenceIndexNegative[1269] = ''''''
semitonesBySequenceIndex[1269] = [0, 1, 5, 8, 9, 10, 11]
poemBySequenceIndex[1269] = '''B D K N P R S'''
poemAuthors[1269] = ''''''
#1270: ䷨䷠ Diminishing Retiring: 1 4 3 1 1 1 1  : S R1 M1 D1 D2 N2 N3 : Elig

#1271: C D♭ F♯ G A♭ A B♭ :0 1 6 7 8 9 10: 1 ♭2 ♯4 5 ♭6 6 ♭7: 13(♭9 ♯11 no 3 add ♭13)
namesBySequenceIndex[1270] = ''''''
namesBySequenceIndexNegative[1270] = ''''''
semitonesBySequenceIndex[1270] = [0, 1, 6, 7, 8, 9, 10]
poemBySequenceIndex[1270] = '''B D L M N P R'''
poemAuthors[1270] = ''''''
#1271: ䷒䷪ Arrived Decision: 1 5 1 1 1 1 2  : S R1 M2 P D1 D2 N2 : EjOd

#1272: C D♭ F♯ G A♭ A B :0 1 6 7 8 9 11: 1 ♭2 ♯4 5 ♭6 6 7: ma13(♭9 ♯11 no 3 add ♭13)
namesBySequenceIndex[1271] = ''''''
namesBySequenceIndexNegative[1271] = ''''''
semitonesBySequenceIndex[1271] = [0, 1, 6, 7, 8, 9, 11]
poemBySequenceIndex[1271] = '''B D L M N P S'''
poemAuthors[1271] = ''''''
#1272: ䷒䷍ Arrived Resevoir: 1 5 1 1 1 2 1  : S R1 M2 P D1 D2 N3 : EjOf

#1273: C D♭ F♯ G A♭ B♭ B :0 1 6 7 8 10 11: 1 ♭2 ♯4 5 ♭6 ♭7 7: ma7sus♯4(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1272] = ''''''
namesBySequenceIndexNegative[1272] = ''''''
semitonesBySequenceIndex[1272] = [0, 1, 6, 7, 8, 10, 11]
poemBySequenceIndex[1272] = '''B D L M N R S'''
poemAuthors[1272] = ''''''
#1273: ䷒䷈ Arrived Homegrown: 1 5 1 1 2 1 1  : S R1 M2 P D1 N2 N3 : EjOm

#1274: C D♭ F♯ G A B♭ B :0 1 6 7 9 10 11: 1 ♭2 ♯4 5 6 ♭7 7: ma13(♭9 ♯11 no 3 add ♭7)
namesBySequenceIndex[1273] = ''''''
namesBySequenceIndexNegative[1273] = ''''''
semitonesBySequenceIndex[1273] = [0, 1, 6, 7, 9, 10, 11]
poemBySequenceIndex[1273] = '''B D L M P R S'''
poemAuthors[1273] = ''''''
#1274: ䷒䷉ Arrived Treading: 1 5 1 2 1 1 1  : S R1 M2 P D2 N2 N3 : EjEg

#1275: C D♭ F♯ G♯ A B♭ B :0 1 6 8 9 10 11: 1 ♭2 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♭9 ♯11 no 3 add ♭7)
namesBySequenceIndex[1274] = ''''''
namesBySequenceIndexNegative[1274] = ''''''
semitonesBySequenceIndex[1274] = [0, 1, 6, 8, 9, 10, 11]
poemBySequenceIndex[1274] = '''B D L N P R S'''
poemAuthors[1274] = ''''''
#1275: ䷒䷌ Arrived Fellowship: 1 5 2 1 1 1 1  : S R1 M2 D1 D2 N2 N3 : EjIg

#1276: C D♭ G A♭ A B♭ B :0 1 7 8 9 10 11: 1 ♭2 5 ♭6 6 ♭7 7: ma7sus♭2(add ♭7 ♭13 13)
namesBySequenceIndex[1275] = ''''''
namesBySequenceIndexNegative[1275] = ''''''
semitonesBySequenceIndex[1275] = [0, 1, 7, 8, 9, 10, 11]
poemBySequenceIndex[1275] = '''B D M N P R S'''
poemAuthors[1275] = ''''''
#1276: ䷒䷫ Arrived Coupling: 1 6 1 1 1 1 1  : S R1 P D1 D2 N2 N3 : Ejog

#1277: C D D♯ E F F♯ G :0 2 3 4 5 6 7: 1 2 ♯2 3 4 ♯4 5: 11(no 7 add ♯9 ♯11)
namesBySequenceIndex[1276] = ''''''
namesBySequenceIndexNegative[1276] = ''''''
semitonesBySequenceIndex[1276] = [0, 2, 3, 4, 5, 6, 7]
poemBySequenceIndex[1276] = '''B F G J K L M'''
poemAuthors[1276] = ''''''
#1277: ䷌䷒ Fellowship Arrived: 2 1 1 1 1 1 5  : S R2 R3 G3 M1 M2 P : IgEj

#1278: C D D♯ E F G♭ A♭ :0 2 3 4 5 6 8: 1 2 ♯2 3 4 ♭5 ♭6: 11(♯5 no 7 add ♯9 ♯11)
namesBySequenceIndex[1277] = '''Kogian'''
namesBySequenceIndexNegative[1277] = ''''''
semitonesBySequenceIndex[1277] = [0, 2, 3, 4, 5, 6, 8]
poemBySequenceIndex[1277] = '''B F G J K L N'''
poemAuthors[1277] = ''''''
#1278: ䷌䷣ Fellowship Eclipse: 2 1 1 1 1 2 4  : S R2 R3 G3 M1 M2 D1 : IgIj

#1279: C D D♯ E F G♭ A :0 2 3 4 5 6 9: 1 2 ♯2 3 4 ♭5 6: 13(♭5 no 7 add ♯9)
namesBySequenceIndex[1278] = '''Debussy's Heptatonic,Katodian'''
namesBySequenceIndexNegative[1278] = ''''''
semitonesBySequenceIndex[1278] = [0, 2, 3, 4, 5, 6, 9]
poemBySequenceIndex[1278] = '''B F G J K L P'''
poemAuthors[1278] = ''''''
#1279: ䷌䷲ Fellowship Thunder: 2 1 1 1 1 3 3  : S R2 R3 G3 M1 M2 D2 : IgAb

#1280: C D D♯ E F G♭ B♭ :0 2 3 4 5 6 10: 1 2 ♯2 3 4 ♭5 ♭7: 11(♭5 add ♯9)
namesBySequenceIndex[1279] = '''Bydian'''
namesBySequenceIndexNegative[1279] = ''''''
semitonesBySequenceIndex[1279] = [0, 2, 3, 4, 5, 6, 10]
poemBySequenceIndex[1279] = '''B F G J K L R'''
poemAuthors[1279] = ''''''
#1280: ䷌䷂ Fellowship Sprouting: 2 1 1 1 1 4 2  : S R2 R3 G3 M1 M2 N2 : IgAk

#1281: C D D♯ E F G♭ B :0 2 3 4 5 6 11: 1 2 ♯2 3 4 ♭5 7: ma11(♭5 add ♯9)
namesBySequenceIndex[1280] = ''''''
namesBySequenceIndexNegative[1280] = ''''''
semitonesBySequenceIndex[1280] = [0, 2, 3, 4, 5, 6, 11]
poemBySequenceIndex[1280] = '''B F G J K L S'''
poemAuthors[1280] = ''''''
#1281: ䷌䷚ Fellowship Nourishment: 2 1 1 1 1 5 1  : S R2 R3 G3 M1 M2 N3 : IgAl

#1282: C D D♯ E F G A♭ :0 2 3 4 5 7 8: 1 2 ♯2 3 4 5 ♭6: 11(no 7 add ♭6 ♯9)
namesBySequenceIndex[1281] = '''Gocrian'''
namesBySequenceIndexNegative[1281] = ''''''
semitonesBySequenceIndex[1281] = [0, 2, 3, 4, 5, 7, 8]
poemBySequenceIndex[1281] = '''B F G J K M N'''
poemAuthors[1281] = ''''''
#1282: ䷌䷭ Fellowship Ascending: 2 1 1 1 2 1 4  : S R2 R3 G3 M1 P D1 : Igoj

#1283: C D D♯ E F G A :0 2 3 4 5 7 9: 1 2 ♯2 3 4 5 6: 13(no 7 add ♯9)
namesBySequenceIndex[1282] = '''Mixonyphian'''
namesBySequenceIndexNegative[1282] = ''''''
semitonesBySequenceIndex[1282] = [0, 2, 3, 4, 5, 7, 9]
poemBySequenceIndex[1282] = '''B F G J K M P'''
poemAuthors[1282] = ''''''
#1283: ䷌䷧ Fellowship Deliverance: 2 1 1 1 2 2 3  : S R2 R3 G3 M1 P D2 : Igeb

#1284: C D D♯ E F G B♭ :0 2 3 4 5 7 10: 1 2 ♯2 3 4 5 ♭7: 11(add ♯9)
namesBySequenceIndex[1283] = '''Gyrian'''
namesBySequenceIndexNegative[1283] = ''''''
semitonesBySequenceIndex[1283] = [0, 2, 3, 4, 5, 7, 10]
poemBySequenceIndex[1283] = '''B F G J K M R'''
poemAuthors[1283] = ''''''
#1284: ䷌䷜ Fellowship Chasm: 2 1 1 1 2 3 2  : S R2 R3 G3 M1 P N2 : Igek

#1285: C D D♯ E F G B :0 2 3 4 5 7 11: 1 2 ♯2 3 4 5 7: ma11(add ♯9)
namesBySequenceIndex[1284] = '''Epothian'''
namesBySequenceIndexNegative[1284] = ''''''
semitonesBySequenceIndex[1284] = [0, 2, 3, 4, 5, 7, 11]
poemBySequenceIndex[1284] = '''B F G J K M S'''
poemAuthors[1284] = ''''''
#1285: ䷌䷃ Fellowship Naïve: 2 1 1 1 2 4 1  : S R2 R3 G3 M1 P N3 : Igel

#1286: C D D♯ E F G♯ A :0 2 3 4 5 8 9: 1 2 ♯2 3 4 ♯5 6: 13(♯5 no 7 add ♯9)
namesBySequenceIndex[1285] = '''Lygian'''
namesBySequenceIndexNegative[1285] = ''''''
semitonesBySequenceIndex[1285] = [0, 2, 3, 4, 5, 8, 9]
poemBySequenceIndex[1285] = '''B F G J K N P'''
poemAuthors[1285] = ''''''
#1286: ䷌䷽ Fellowship Detail: 2 1 1 1 3 1 3  : S R2 R3 G3 M1 D1 N1 : Igib

#1287: C D D♯ E F G♯ B♭ :0 2 3 4 5 8 10: 1 2 ♯2 3 4 ♯5 ♭7: 11(♯5 add ♯9)
namesBySequenceIndex[1286] = '''Madian'''
namesBySequenceIndexNegative[1286] = ''''''
semitonesBySequenceIndex[1286] = [0, 2, 3, 4, 5, 8, 10]
poemBySequenceIndex[1286] = '''B F G J K N R'''
poemAuthors[1286] = ''''''
#1287: ䷌䷦ Fellowship Limping: 2 1 1 1 3 2 2  : S R2 R3 G3 M1 D1 N2 : Igik

#1288: C D D♯ E F G♯ B :0 2 3 4 5 8 11: 1 2 ♯2 3 4 ♯5 7: ma11(♯5 add ♯9)
namesBySequenceIndex[1287] = '''Sythian'''
namesBySequenceIndexNegative[1287] = ''''''
semitonesBySequenceIndex[1287] = [0, 2, 3, 4, 5, 8, 11]
poemBySequenceIndex[1287] = '''B F G J K N S'''
poemAuthors[1287] = ''''''
#1288: ䷌䷳ Fellowship Stillness: 2 1 1 1 3 3 1  : S R2 R3 G3 M1 D1 N3 : Igil

#1289: C D D♯ E F A B♭ :0 2 3 4 5 9 10: 1 2 ♯2 3 4 6 ♭7: 13(no 5 add ♯9)
namesBySequenceIndex[1288] = '''Aeolodian'''
namesBySequenceIndexNegative[1288] = ''''''
semitonesBySequenceIndex[1288] = [0, 2, 3, 4, 5, 9, 10]
poemBySequenceIndex[1288] = '''B F G J K P R'''
poemAuthors[1288] = ''''''
#1289: ䷌䷬ Fellowship Clustering: 2 1 1 1 4 1 2  : S R2 R3 G3 M1 D2 N2 : Igad

#1290: C D D♯ E F A B :0 2 3 4 5 9 11: 1 2 ♯2 3 4 6 7: ma13(no 5 add ♯9)
namesBySequenceIndex[1289] = '''Frigonian,Ionogian'''
namesBySequenceIndexNegative[1289] = ''''''
semitonesBySequenceIndex[1289] = [0, 2, 3, 4, 5, 9, 11]
poemBySequenceIndex[1289] = '''B F G J K P S'''
poemAuthors[1289] = ''''''
#1290: ䷌䷢ Fellowship Prospering: 2 1 1 1 4 2 1  : S R2 R3 G3 M1 D2 N3 : Igaf

#1291: C D D♯ E F A♯ B :0 2 3 4 5 10 11: 1 2 ♯2 3 4 ♯6 7: ma11(no 5 add ♭7 ♯9)
namesBySequenceIndex[1290] = ''''''
namesBySequenceIndexNegative[1290] = ''''''
semitonesBySequenceIndex[1290] = [0, 2, 3, 4, 5, 10, 11]
poemBySequenceIndex[1290] = '''B F G J K R S'''
poemAuthors[1290] = ''''''
#1291: ䷌䷓ Fellowship Seeing: 2 1 1 1 5 1 1  : S R2 R3 G3 M1 D3 N3 : Igam

#1292: C D D♯ E F♯ G A♭ :0 2 3 4 6 7 8: 1 2 ♯2 3 ♯4 5 ♭6: ma(add ♭6 9 ♯9 ♯11)
namesBySequenceIndex[1291] = '''Stacrian'''
namesBySequenceIndexNegative[1291] = ''''''
semitonesBySequenceIndex[1291] = [0, 2, 3, 4, 6, 7, 8]
poemBySequenceIndex[1291] = '''B F G J L M N'''
poemAuthors[1291] = ''''''
#1292: ䷰䷊ Revolution Peace: 2 1 1 2 1 1 4  : S R2 R3 G3 M2 P D1 : IdOj

#1293: C D D♯ E F♯ G A :0 2 3 4 6 7 9: 1 2 ♯2 3 ♯4 5 6: 6/9(add ♯9 ♯11)
namesBySequenceIndex[1292] = '''Donian'''
namesBySequenceIndexNegative[1292] = ''''''
semitonesBySequenceIndex[1292] = [0, 2, 3, 4, 6, 7, 9]
poemBySequenceIndex[1292] = '''B F G J L M P'''
poemAuthors[1292] = ''''''
#1293: ䷰䷵ Revolution Betrothing: 2 1 1 2 1 2 3  : S R2 R3 G3 M2 P D2 : IdEb

#1294: C D D♯ E F♯ G B♭ :0 2 3 4 6 7 10: 1 2 ♯2 3 ♯4 5 ♭7: 9(add ♯9 ♯11)
namesBySequenceIndex[1293] = '''Lathian'''
namesBySequenceIndexNegative[1293] = ''''''
semitonesBySequenceIndex[1293] = [0, 2, 3, 4, 6, 7, 10]
poemBySequenceIndex[1293] = '''B F G J L M R'''
poemAuthors[1293] = ''''''
#1294: ䷰䷻ Revolution Composed: 2 1 1 2 1 3 2  : S R2 R3 G3 M2 P N2 : IdEk

#1295: C D D♯ E F♯ G B :0 2 3 4 6 7 11: 1 2 ♯2 3 ♯4 5 7: ma9(add ♯9 ♯11)
namesBySequenceIndex[1294] = '''Pygian'''
namesBySequenceIndexNegative[1294] = ''''''
semitonesBySequenceIndex[1294] = [0, 2, 3, 4, 6, 7, 11]
poemBySequenceIndex[1294] = '''B F G J L M S''' #because freedom grew huge kastles' lords' retiring
poemAuthors[1294] = ''''''
#1295: ䷰䷨ Revolution Diminishing: 2 1 1 2 1 4 1  : S R2 R3 G3 M2 P N3 : IdEl

#1296: C D D♯ E F♯ G♯ A :0 2 3 4 6 8 9: 1 2 ♯2 3 ♯4 ♯5 6: 6/9(♯5 add ♯9 ♯11)
namesBySequenceIndex[1295] = '''Rylian'''
namesBySequenceIndexNegative[1295] = ''''''
semitonesBySequenceIndex[1295] = [0, 2, 3, 4, 6, 8, 9]
poemBySequenceIndex[1295] = '''B F G J L N P'''
poemAuthors[1295] = ''''''
#1296: ䷰䷶ Revolution Abounding: 2 1 1 2 2 1 3  : S R2 R3 G3 M2 D1 N1 : IdIb

#1297: C D D♯ E F♯ G♯ B♭ :0 2 3 4 6 8 10: 1 2 ♯2 3 ♯4 ♯5 ♭7: 9(♯5 add ♯9 ♯11)
namesBySequenceIndex[1296] = '''Storian,Semilocrian ♭4'''
namesBySequenceIndexNegative[1296] = ''''''
semitonesBySequenceIndex[1296] = [0, 2, 3, 4, 6, 8, 10]
poemBySequenceIndex[1296] = '''B F G J L N R'''
poemAuthors[1296] = ''''''
#1297: ䷰䷾ Revolution Completed: 2 1 1 2 2 2 2  : S R2 R3 G3 M2 D1 N2 : IdIk

#1298: C D D♯ E F♯ G♯ B :0 2 3 4 6 8 11: 1 2 ♯2 3 ♯4 ♯5 7: ma9(♯5 add ♯9 ♯11)
namesBySequenceIndex[1297] = '''Stagian'''
namesBySequenceIndexNegative[1297] = ''''''
semitonesBySequenceIndex[1297] = [0, 2, 3, 4, 6, 8, 11]
poemBySequenceIndex[1297] = '''B F G J L N S'''
poemAuthors[1297] = ''''''
#1298: ䷰䷕ Revolution Adorning: 2 1 1 2 2 3 1  : S R2 R3 G3 M2 D1 N3 : IdIl

#1299: C D D♯ E G♭ A B♭ :0 2 3 4 6 9 10: 1 2 ♯2 3 ♭5 6 ♭7: 9(♭5 add ♯9 13)
namesBySequenceIndex[1298] = '''Synian'''
namesBySequenceIndexNegative[1298] = ''''''
semitonesBySequenceIndex[1298] = [0, 2, 3, 4, 6, 9, 10]
poemBySequenceIndex[1298] = '''B F G J L P R'''
poemAuthors[1298] = ''''''
#1299: ䷰䷐ Revolution Following: 2 1 1 2 3 1 2  : S R2 R3 G3 M2 D2 N2 : IdAd

#1300: C D D♯ E G♭ A B :0 2 3 4 6 9 11: 1 2 ♯2 3 ♭5 6 7: ma9(♭5 add ♯9 13)
namesBySequenceIndex[1299] = '''Sygian'''
namesBySequenceIndexNegative[1299] = ''''''
semitonesBySequenceIndex[1299] = [0, 2, 3, 4, 6, 9, 11]
poemBySequenceIndex[1299] = '''B F G J L P S'''
poemAuthors[1299] = ''''''
#1300: ䷰䷔ Revolution Biting: 2 1 1 2 3 2 1  : S R2 R3 G3 M2 D2 N3 : IdAf

#1301: C D D♯ E G♭ A♯ B :0 2 3 4 6 10 11: 1 2 ♯2 3 ♭5 ♯6 7: ma9(♭5 add ♭7 ♯9)
namesBySequenceIndex[1300] = '''Mylian'''
namesBySequenceIndexNegative[1300] = ''''''
semitonesBySequenceIndex[1300] = [0, 2, 3, 4, 6, 10, 11]
poemBySequenceIndex[1300] = '''B F G J L R S'''
poemAuthors[1300] = ''''''
#1301: ䷰䷩ Revolution Augmenting: 2 1 1 2 4 1 1  : S R2 R3 G3 M2 D3 N3 : IdAm

#1302: C D D♯ E G A♭ A :0 2 3 4 7 8 9: 1 2 ♯2 3 5 ♭6 6: 6/9(add ♯9 ♭13)
namesBySequenceIndex[1301] = '''Relative Blues,Raga Dvigandharabushini,Mythian'''
namesBySequenceIndexNegative[1301] = ''''''
semitonesBySequenceIndex[1301] = [0, 2, 3, 4, 7, 8, 9]
poemBySequenceIndex[1301] = '''B F G J M N P'''
poemAuthors[1301] = ''''''
#1302: ䷰䷟ Revolution Persevering: 2 1 1 3 1 1 3  : S R2 R3 G3 P D1 N1 : Idob

#1303: C D D♯ E G A♭ B♭ :0 2 3 4 7 8 10: 1 2 ♯2 3 5 ♭6 ♭7: 9(add ♯9 ♭13)
namesBySequenceIndex[1302] = '''Sabach,Aeolycrian'''
namesBySequenceIndexNegative[1302] = ''''''
semitonesBySequenceIndex[1302] = [0, 2, 3, 4, 7, 8, 10]
poemBySequenceIndex[1302] = '''B F G J M N R'''
poemAuthors[1302] = ''''''
#1303: ䷰䷯ Revolution Source: 2 1 1 3 1 2 2  : S R2 R3 G3 P D1 N2 : Idok

#1304: C D D♯ E G A♭ B :0 2 3 4 7 8 11: 1 2 ♯2 3 5 ♭6 7: ma9(add ♯9 ♭13)
namesBySequenceIndex[1303] = '''Sagian,Sabach Ma7 (Slonimsky)'''
namesBySequenceIndexNegative[1303] = ''''''
semitonesBySequenceIndex[1303] = [0, 2, 3, 4, 7, 8, 11]
poemBySequenceIndex[1303] = '''B F G J M N S'''
poemAuthors[1303] = ''''''
#1304: ䷰䷑ Revolution Correcting: 2 1 1 3 1 3 1  : S R2 R3 G3 P D1 N3 : Idol

#1305: C D D♯ E G A B♭ :0 2 3 4 7 9 10: 1 2 ♯2 3 5 6 ♭7: 9(add ♯9 13)
namesBySequenceIndex[1304] = '''Banshikicho,Bebop Minor,Banshikicho (Japan),Dogian'''
namesBySequenceIndexNegative[1304] = ''''''
semitonesBySequenceIndex[1304] = [0, 2, 3, 4, 7, 9, 10]
poemBySequenceIndex[1304] = '''B F G J M P R'''
poemAuthors[1304] = ''''''
#1305: ䷰䷮ Revolution Confining: 2 1 1 3 2 1 2  : S R2 R3 G3 P D2 N2 : Ided

#1306: C D D♯ E G A B :0 2 3 4 7 9 11: 1 2 ♯2 3 5 6 7: ma9(add ♯9 13)
namesBySequenceIndex[1305] = '''Epygian'''
namesBySequenceIndexNegative[1305] = ''''''
semitonesBySequenceIndex[1305] = [0, 2, 3, 4, 7, 9, 11]
poemBySequenceIndex[1305] = '''B F G J M P S'''
poemAuthors[1305] = ''''''
#1306: ䷰䷿ Revolution Unfinished: 2 1 1 3 2 2 1  : S R2 R3 G3 P D2 N3 : Idef

#1307: C D D♯ E G A♯ B :0 2 3 4 7 10 11: 1 2 ♯2 3 5 ♯6 7: ma9(add ♭7 ♯9)
namesBySequenceIndex[1306] = '''Aeolaptian'''
namesBySequenceIndexNegative[1306] = ''''''
semitonesBySequenceIndex[1306] = [0, 2, 3, 4, 7, 10, 11]
poemBySequenceIndex[1306] = '''B F G J M R S'''
poemAuthors[1306] = ''''''
#1307: ䷰䷺ Revolution Dispersing: 2 1 1 3 3 1 1  : S R2 R3 G3 P D3 N3 : Idem

#1308: C D D♯ E G♯ A B♭ :0 2 3 4 8 9 10: 1 2 ♯2 3 ♯5 6 ♭7: 9(♯5 add ♯9 13)
namesBySequenceIndex[1307] = '''Aeradian'''
namesBySequenceIndexNegative[1307] = ''''''
semitonesBySequenceIndex[1307] = [0, 2, 3, 4, 8, 9, 10]
poemBySequenceIndex[1307] = '''B F G J N P R'''
poemAuthors[1307] = ''''''
#1308: ䷰䷞ Revolution Conjoining: 2 1 1 4 1 1 2  : S R2 R3 G3 D1 D2 N2 : Idid

#1309: C D D♯ E G♯ A B :0 2 3 4 8 9 11: 1 2 ♯2 3 ♯5 6 7: ma9(♯5 add ♯9 13)
namesBySequenceIndex[1308] = '''Baptian'''
namesBySequenceIndexNegative[1308] = ''''''
semitonesBySequenceIndex[1308] = [0, 2, 3, 4, 8, 9, 11]
poemBySequenceIndex[1308] = '''B F G J N P S'''
poemAuthors[1308] = ''''''
#1309: ䷰䷷ Revolution Wanderer: 2 1 1 4 1 2 1  : S R2 R3 G3 D1 D2 N3 : Idif

#1310: C D D♯ E G♯ A♯ B :0 2 3 4 8 10 11: 1 2 ♯2 3 ♯5 ♯6 7: ma9(♯5 add ♭7 ♯9)
namesBySequenceIndex[1309] = '''Phrodian'''
namesBySequenceIndexNegative[1309] = ''''''
semitonesBySequenceIndex[1309] = [0, 2, 3, 4, 8, 10, 11]
poemBySequenceIndex[1309] = '''B F G J N R S'''
poemAuthors[1309] = ''''''
#1310: ䷰䷴ Revolution Infiltrating: 2 1 1 4 2 1 1  : S R2 R3 G3 D1 N2 N3 : Idim

#1311: C D D♯ E A B♭ B :0 2 3 4 9 10 11: 1 2 ♯2 3 6 ♭7 7: ma9(no 5 add ♭7 ♯9 13)
namesBySequenceIndex[1310] = ''''''
namesBySequenceIndexNegative[1310] = ''''''
semitonesBySequenceIndex[1310] = [0, 2, 3, 4, 9, 10, 11]
poemBySequenceIndex[1310] = '''B F G J P R S'''
poemAuthors[1310] = ''''''
#1311: ䷰䷋ Revolution Obstruction: 2 1 1 5 1 1 1  : S R2 R3 G3 D2 N2 N3 : Idag

#1312: C D E♭ F F♯ G A♭ :0 2 3 5 6 7 8: 1 2 ♭3 4 ♯4 5 ♭6: mi11(no 7 add ♭6 ♯11)
namesBySequenceIndex[1311] = '''Rygian'''
namesBySequenceIndexNegative[1311] = ''''''
semitonesBySequenceIndex[1311] = [0, 2, 3, 5, 6, 7, 8]
poemBySequenceIndex[1311] = '''B F G K L M N'''
poemAuthors[1311] = ''''''
#1312: ䷝䷊ Distanced Peace: 2 1 2 1 1 1 4  : S R2 G2 M1 M2 P D1 : IfOj

#1313: C D E♭ F F♯ G A :0 2 3 5 6 7 9: 1 2 ♭3 4 ♯4 5 6: mi13(no 7 add ♯11)
namesBySequenceIndex[1312] = '''Aeologian'''
namesBySequenceIndexNegative[1312] = ''''''
semitonesBySequenceIndex[1312] = [0, 2, 3, 5, 6, 7, 9]
poemBySequenceIndex[1312] = '''B F G K L M P'''
poemAuthors[1312] = ''''''
#1313: ䷝䷵ Distanced Betrothing: 2 1 2 1 1 2 3  : S R2 G2 M1 M2 P D2 : IfEb

#1314: C D E♭ F F♯ G B♭ :0 2 3 5 6 7 10: 1 2 ♭3 4 ♯4 5 ♭7: mi11(add ♯11)
namesBySequenceIndex[1313] = '''Raga Bahar (descending),Blues Add Nine,Aeodian'''
namesBySequenceIndexNegative[1313] = ''''''
semitonesBySequenceIndex[1313] = [0, 2, 3, 5, 6, 7, 10]
poemBySequenceIndex[1313] = '''B F G K L M R'''
poemAuthors[1313] = ''''''
#1314: ䷝䷻ Distanced Composed: 2 1 2 1 1 3 2  : S R2 G2 M1 M2 P N2 : IfEk

#1315: C D E♭ F F♯ G B :0 2 3 5 6 7 11: 1 2 ♭3 4 ♯4 5 7: mima11(add ♯11)
namesBySequenceIndex[1314] = '''Aerogian'''
namesBySequenceIndexNegative[1314] = ''''''
semitonesBySequenceIndex[1314] = [0, 2, 3, 5, 6, 7, 11]
poemBySequenceIndex[1314] = '''B F G K L M S'''
poemAuthors[1314] = ''''''
#1315: ䷝䷨ Distanced Diminishing: 2 1 2 1 1 4 1  : S R2 G2 M1 M2 P N3 : IfEl

#1316: C D E♭ F G♭ A♭ A :0 2 3 5 6 8 9: 1 2 ♭3 4 ♭5 ♭6 6: °13(no 7 add ♭6)
namesBySequenceIndex[1315] = '''Aeraptian'''
namesBySequenceIndexNegative[1315] = ''''''
semitonesBySequenceIndex[1315] = [0, 2, 3, 5, 6, 8, 9]
poemBySequenceIndex[1315] = '''B F G K L N P'''
poemAuthors[1315] = ''''''
#1316: ䷝䷶ Distanced Abounding: 2 1 2 1 2 1 3  : S R2 G2 M1 M2 D1 N1 : IfIb

#1317: C D E♭ F G♭ A♭ B♭ :0 2 3 5 6 8 10: 1 2 ♭3 4 ♭5 ♭6 ♭7: ⌀11(add ♭13)
namesBySequenceIndex[1316] = '''Semilocrian,Half Diminished,Minor Locrian,Locrian Natural 2,Minor Locrian,Minor ♭5,Altered Diminished,Lorian'''
namesBySequenceIndexNegative[1316] = ''''''
semitonesBySequenceIndex[1316] = [0, 2, 3, 5, 6, 8, 10]
poemBySequenceIndex[1316] = '''B F G K L N R'''
poemAuthors[1316] = ''''''
#1317: ䷝䷾ Distanced Completed: 2 1 2 1 2 2 2  : S R2 G2 M1 M2 D1 N2 : IfIk

#1318: C D E♭ F G♭ A♭ B :0 2 3 5 6 8 11: 1 2 ♭3 4 ♭5 ♭6 7: °ma11(add ♭13)
namesBySequenceIndex[1317] = '''Phrydian,Harmonic Minor ♭5'''
namesBySequenceIndexNegative[1317] = ''''''
semitonesBySequenceIndex[1317] = [0, 2, 3, 5, 6, 8, 11]
poemBySequenceIndex[1317] = '''B F G K L N S'''
poemAuthors[1317] = ''''''
#1318: ䷝䷕ Distanced Adorning: 2 1 2 1 2 3 1  : S R2 G2 M1 M2 D1 N3 : IfIl

#1319: C D E♭ F G♭ A B♭ :0 2 3 5 6 9 10: 1 2 ♭3 4 ♭5 6 ♭7: ⌀13
namesBySequenceIndex[1318] = '''Blues Heptatonic 1,Dorian ♭5,Kartzihiar (Greece),Maqam Karcigar (Iraq),Maqam Nahawand Murassah (Iraq),Katagian'''
namesBySequenceIndexNegative[1318] = ''''''
semitonesBySequenceIndex[1318] = [0, 2, 3, 5, 6, 9, 10]
poemBySequenceIndex[1318] = '''B F G K L P R'''
poemAuthors[1318] = ''''''
#1319: ䷝䷐ Distanced Following: 2 1 2 1 3 1 2  : S R2 G2 M1 M2 D2 N2 : IfAd

#1320: C D E♭ F G♭ A B :0 2 3 5 6 9 11: 1 2 ♭3 4 ♭5 6 7: °ma13
namesBySequenceIndex[1319] = '''Zangula,Jeths,Zangula (Nigeria),Gamian'''
namesBySequenceIndexNegative[1319] = ''''''
semitonesBySequenceIndex[1319] = [0, 2, 3, 5, 6, 9, 11]
poemBySequenceIndex[1319] = '''B F G K L P S'''
poemAuthors[1319] = ''''''
#1320: ䷝䷔ Distanced Biting: 2 1 2 1 3 2 1  : S R2 G2 M1 M2 D2 N3 : IfAf

#1321: C D E♭ F G♭ A♯ B :0 2 3 5 6 10 11: 1 2 ♭3 4 ♭5 ♯6 7: °ma11(add ♭7)
namesBySequenceIndex[1320] = '''Rolian'''
namesBySequenceIndexNegative[1320] = ''''''
semitonesBySequenceIndex[1320] = [0, 2, 3, 5, 6, 10, 11]
poemBySequenceIndex[1320] = '''B F G K L R S'''
poemAuthors[1320] = ''''''
#1321: ䷝䷩ Distanced Augmenting: 2 1 2 1 4 1 1  : S R2 G2 M1 M2 D3 N3 : IfAm

#1322: C D E♭ F G A♭ A :0 2 3 5 7 8 9: 1 2 ♭3 4 5 ♭6 6: mi13(no 7 add ♭6)
namesBySequenceIndex[1321] = '''Mela Jhankaradhvani,Mela Jhankaradhwani,Raga Jhankara Bhramavi,Phrorian'''
namesBySequenceIndexNegative[1321] = ''''''
semitonesBySequenceIndex[1321] = [0, 2, 3, 5, 7, 8, 9]
poemBySequenceIndex[1321] = '''B F G K M N P'''
poemAuthors[1321] = ''''''
#1322: ䷝䷟ Distanced Persevering: 2 1 2 2 1 1 3  : S R2 G2 M1 P D1 N1 : Ifob

#1323: C D E♭ F G A♭ B♭ :0 2 3 5 7 8 10: 1 2 ♭3 4 5 ♭6 ♭7: mi11(add ♭13)
namesBySequenceIndex[1322] = '''Aeolian,Natural Minor,Peruvian Minor,Melodic Minor (descending),Raga Adana (descending),Cushak (Armenia),Ezel (Ethiopia),Geez (Ethiopia),Se (Japan),Raga Bhairavi (descending),Mela Natabhairavi,Raga Jaunpuri,Raga Jingla,Raga Asavari That'''
namesBySequenceIndexNegative[1322] = ''''''
semitonesBySequenceIndex[1322] = [0, 2, 3, 5, 7, 8, 10]
poemBySequenceIndex[1322] = '''B F G K M N R'''
poemAuthors[1322] = ''''''
#1323: ䷝䷯ Distanced Source: 2 1 2 2 1 2 2  : S R2 G2 M1 P D1 N2 : Ifok

#1324: C D E♭ F G A♭ B :0 2 3 5 7 8 11: 1 2 ♭3 4 5 ♭6 7: mima11(add ♭13)
namesBySequenceIndex[1323] = '''Harmonic Minor,Mohammedan,Maqam Bayat-e-Esfahan (Iraq),Maqam Sultani Yakah (Iraq),Sultani Yakah,Zhalibny Minor,Raga Pilu That,Mela Kiravani,Mela Keeravani,Raga Kiranavali,Raga Kirvani,Raga Kalyana Vasantha,Raga Deshi 3,Mischung 4 (Germany),Mydian'''
namesBySequenceIndexNegative[1323] = ''''''
semitonesBySequenceIndex[1323] = [0, 2, 3, 5, 7, 8, 11]
poemBySequenceIndex[1323] = '''B F G K M N S'''
poemAuthors[1323] = ''''''
#1324: ䷝䷑ Distanced Correcting: 2 1 2 2 1 3 1  : S R2 G2 M1 P D1 N3 : Ifol

#1325: C D E♭ F G A B♭ :0 2 3 5 7 9 10: 1 2 ♭3 4 5 6 ♭7: mi13
namesBySequenceIndex[1324] = '''Dorian,Gregorian 8,Mischung 5 (Germany),Yu (China),Hyojo (Japan),Oshikicho (Japan),Nam (Vietnam),Raga Kafi That,Mela Kharaharapriya,Raga Bhairavi (ascending),Raga Kharapriya,Raga Shree (descending),Raga Bhimpalasi (descending),Raga Nayaki Kanada,Raga Sri,Raga Ritigaula,Raga Huseni,Raga Kanara,Raga Bageshri'''
namesBySequenceIndexNegative[1324] = ''''''
semitonesBySequenceIndex[1324] = [0, 2, 3, 5, 7, 9, 10]
poemBySequenceIndex[1324] = '''B F G K M P R'''
poemAuthors[1324] = ''''''
#1325: ䷝䷮ Distanced Confining: 2 1 2 2 2 1 2  : S R2 G2 M1 P D2 N2 : Ifed

#1326: C D E♭ F G A B :0 2 3 5 7 9 11: 1 2 ♭3 4 5 6 7: mima13
namesBySequenceIndex[1325] = '''Jazz Minor,Melodic Minor (ascending),Mischung 1 (Germany),Mela Gaurimanohari,Mela Gourimanohari,Raga Patdip,Raga Velavali,Raga Deshi 2,Bocrian'''
namesBySequenceIndexNegative[1325] = ''''''
semitonesBySequenceIndex[1325] = [0, 2, 3, 5, 7, 9, 11]
poemBySequenceIndex[1325] = '''B F G K M P S'''
poemAuthors[1325] = ''''''
#1326: ䷝䷿ Distanced Unfinished: 2 1 2 2 2 2 1  : S R2 G2 M1 P D2 N3 : Ifef

#1327: C D E♭ F G A♯ B :0 2 3 5 7 10 11: 1 2 ♭3 4 5 ♯6 7: mima11(add ♭7)
namesBySequenceIndex[1326] = '''Mela Varunapriya,Raga Viravasantham,Aeracrian'''
namesBySequenceIndexNegative[1326] = ''''''
semitonesBySequenceIndex[1326] = [0, 2, 3, 5, 7, 10, 11]
poemBySequenceIndex[1326] = '''B F G K M R S'''
poemAuthors[1326] = ''''''
#1327: ䷝䷺ Distanced Dispersing: 2 1 2 2 3 1 1  : S R2 G2 M1 P D3 N3 : Ifem

#1328: C D E♭ F G♯ A B♭ :0 2 3 5 8 9 10: 1 2 ♭3 4 ♯5 6 ♭7: mi13(♯5)
namesBySequenceIndex[1327] = '''Dalian'''
namesBySequenceIndexNegative[1327] = ''''''
semitonesBySequenceIndex[1327] = [0, 2, 3, 5, 8, 9, 10]
poemBySequenceIndex[1327] = '''B F G K N P R'''
poemAuthors[1327] = ''''''
#1328: ䷝䷞ Distanced Conjoining: 2 1 2 3 1 1 2  : S R2 G2 M1 D1 D2 N2 : Ifid

#1329: C D E♭ F G♯ A B :0 2 3 5 8 9 11: 1 2 ♭3 4 ♯5 6 7: mima13(♯5)
namesBySequenceIndex[1328] = '''Katothian,Jazz Minor ♯5'''
namesBySequenceIndexNegative[1328] = ''''''
semitonesBySequenceIndex[1328] = [0, 2, 3, 5, 8, 9, 11]
poemBySequenceIndex[1328] = '''B F G K N P S'''
poemAuthors[1328] = ''''''
#1329: ䷝䷷ Distanced Wanderer: 2 1 2 3 1 2 1  : S R2 G2 M1 D1 D2 N3 : Ifif

#1330: C D E♭ F G♯ A♯ B :0 2 3 5 8 10 11: 1 2 ♭3 4 ♯5 ♯6 7: mima11(♯5 add ♭7)
namesBySequenceIndex[1329] = '''Lodian'''
namesBySequenceIndexNegative[1329] = ''''''
semitonesBySequenceIndex[1329] = [0, 2, 3, 5, 8, 10, 11]
poemBySequenceIndex[1329] = '''B F G K N R S'''
poemAuthors[1329] = ''''''
#1330: ䷝䷴ Distanced Infiltrating: 2 1 2 3 2 1 1  : S R2 G2 M1 D1 N2 N3 : Ifim

#1331: C D E♭ F A B♭ B :0 2 3 5 9 10 11: 1 2 ♭3 4 6 ♭7 7: mima13(no 5 add ♭7)
namesBySequenceIndex[1330] = '''Boptian'''
namesBySequenceIndexNegative[1330] = ''''''
semitonesBySequenceIndex[1330] = [0, 2, 3, 5, 9, 10, 11]
poemBySequenceIndex[1330] = '''B F G K P R S'''
poemAuthors[1330] = ''''''
#1331: ䷝䷋ Distanced Obstruction: 2 1 2 4 1 1 1  : S R2 G2 M1 D2 N2 N3 : Ifag

#1332: C D E♭ F♯ G A♭ A :0 2 3 6 7 8 9: 1 2 ♭3 ♯4 5 ♭6 6: mi6/9(add ♯11 ♭13)
namesBySequenceIndex[1331] = '''Mela Syamalangi,Mela Shamalangi,Raga Shyamalam,Chromatic Hypodorian,Phryptian'''
namesBySequenceIndexNegative[1331] = ''''''
semitonesBySequenceIndex[1331] = [0, 2, 3, 6, 7, 8, 9]
poemBySequenceIndex[1331] = '''B F G L M N P'''
poemAuthors[1331] = ''''''
#1332: ䷶䷡ Abounding Might: 2 1 3 1 1 1 3  : S R2 G2 M2 P D1 N1 : IbOb

#1333: C D E♭ F♯ G A♭ B♭ :0 2 3 6 7 8 10: 1 2 ♭3 ♯4 5 ♭6 ♭7: mi9(add ♯11 ♭13)
namesBySequenceIndex[1332] = '''Gypsy 1,Mela Sanmukhapriya,Raga Asavari (descending),Hungarian Gypsy,Damian Emmanuel,Raga Simmendramadhyamam,Raga Camara,Minor Romani,Tyrian'''
namesBySequenceIndexNegative[1332] = ''''''
semitonesBySequenceIndex[1332] = [0, 2, 3, 6, 7, 8, 10]
poemBySequenceIndex[1332] = '''B F G L M N R'''
poemAuthors[1332] = ''''''
#1333: ䷶䷄ Abounding Biding: 2 1 3 1 1 2 2  : S R2 G2 M2 P D1 N2 : IbOk

#1334: C D E♭ F♯ G A♭ B :0 2 3 6 7 8 11: 1 2 ♭3 ♯4 5 ♭6 7: mima9(add ♯11 ♭13)
namesBySequenceIndex[1333] = '''Gypsy Minor,Double Harmonic Minor,Harmonic Minor ♯4,Hungarian Minor,Niavent (Egypt),Maqam Hisar (Iraq),Maqam Nawa Athar (Iraq),Niaventi Minor (Greece),Mela Simhendramadhyama,Raga Madhava Manohari,Algerian (first octave),Bogian'''
namesBySequenceIndexNegative[1333] = ''''''
semitonesBySequenceIndex[1333] = [0, 2, 3, 6, 7, 8, 11]
poemBySequenceIndex[1333] = '''B F G L M N S'''
poemAuthors[1333] = ''''''
#1334: ䷶䷙ Abounding Cultivating: 2 1 3 1 1 3 1  : S R2 G2 M2 P D1 N3 : IbOl

#1335: C D E♭ F♯ G A B♭ :0 2 3 6 7 9 10: 1 2 ♭3 ♯4 5 6 ♭7: mi13(♯11)
namesBySequenceIndex[1334] = '''Romanian Minor,Dorian ♯4,Gnossiennes,Ukranian Dorian,Tunisian,Kaffa (Ethiopia),Maqam Hedjaz (Iraq),Maqam Nakriz (Iraq) Misheberekh (Jewish),Nigriz (Greece),Peiraiotikos Minor (Greece),Souzinak (Greece),Ukrainian Minor,Mela Hemavati,Raga Desisimharavam,Katycrian'''
namesBySequenceIndexNegative[1334] = ''''''
semitonesBySequenceIndex[1334] = [0, 2, 3, 6, 7, 9, 10]
poemBySequenceIndex[1334] = '''B F G L M P R'''
poemAuthors[1334] = ''''''
#1335: ䷶䷹ Abounding Joyous: 2 1 3 1 2 1 2  : S R2 G2 M2 P D2 N2 : IbEd

#1336: C D E♭ F♯ G A B :0 2 3 6 7 9 11: 1 2 ♭3 ♯4 5 6 7: mima13(♯11)
namesBySequenceIndex[1335] = '''Mela Dharmavati,Raga Ambika,Jazz Minor ♯4,Raga Arunajualita,Raga Dumyaraga,Raga Madhuvanti,Lydian ♭3,Banian'''
namesBySequenceIndexNegative[1335] = ''''''
semitonesBySequenceIndex[1335] = [0, 2, 3, 6, 7, 9, 11]
poemBySequenceIndex[1335] = '''B F G L M P S'''
poemAuthors[1335] = ''''''
#1336: ䷶䷥ Abounding Polarising: 2 1 3 1 2 2 1  : S R2 G2 M2 P D2 N3 : IbEf

#1337: C D E♭ F♯ G A♯ B :0 2 3 6 7 10 11: 1 2 ♭3 ♯4 5 ♯6 7: mima9(add ♭7 ♯11)
namesBySequenceIndex[1336] = '''Mela Nitimati,Raga Kaikavasi,Raga Nisada,Zycrian'''
namesBySequenceIndexNegative[1336] = ''''''
semitonesBySequenceIndex[1336] = [0, 2, 3, 6, 7, 10, 11]
poemBySequenceIndex[1336] = '''B F G L M R S'''
poemAuthors[1336] = ''''''
#1337: ䷶䷼ Abounding Truth: 2 1 3 1 3 1 1  : S R2 G2 M2 P D3 N3 : IbEm

#1338: C D E♭ F♯ G♯ A B♭ :0 2 3 6 8 9 10: 1 2 ♭3 ♯4 ♯5 6 ♭7: ⌀13(♯11 add ♭13)
namesBySequenceIndex[1337] = '''Katyrian'''
namesBySequenceIndexNegative[1337] = ''''''
semitonesBySequenceIndex[1337] = [0, 2, 3, 6, 8, 9, 10]
poemBySequenceIndex[1337] = '''B F G L N P R'''
poemAuthors[1337] = ''''''
#1338: ䷶䷰ Abounding Revolution: 2 1 3 2 1 1 2  : S R2 G2 M2 D1 D2 N2 : IbId

#1339: C D E♭ F♯ G♯ A B :0 2 3 6 8 9 11: 1 2 ♭3 ♯4 ♯5 6 7: °ma13(♯11 add ♭13)
namesBySequenceIndex[1338] = '''Lylian'''
namesBySequenceIndexNegative[1338] = ''''''
semitonesBySequenceIndex[1338] = [0, 2, 3, 6, 8, 9, 11]
poemBySequenceIndex[1338] = '''B F G L N P S'''
poemAuthors[1338] = ''''''
#1339: ䷶䷝ Abounding Distanced: 2 1 3 2 1 2 1  : S R2 G2 M2 D1 D2 N3 : IbIf

#1340: C D E♭ F♯ G♯ A♯ B :0 2 3 6 8 10 11: 1 2 ♭3 ♯4 ♯5 ♯6 7: °ma9(add ♭7 ♭13)
namesBySequenceIndex[1339] = '''Stynian'''
namesBySequenceIndexNegative[1339] = ''''''
semitonesBySequenceIndex[1339] = [0, 2, 3, 6, 8, 10, 11]
poemBySequenceIndex[1339] = '''B F G L N R S'''
poemAuthors[1339] = ''''''
#1340: ䷶䷤ Abounding Family: 2 1 3 2 2 1 1  : S R2 G2 M2 D1 N2 N3 : IbIm

#1341: C D E♭ G♭ A B♭ B :0 2 3 6 9 10 11: 1 2 ♭3 ♭5 6 ♭7 7: °ma9(add ♭7 13)
namesBySequenceIndex[1340] = '''Mixodorian'''
namesBySequenceIndexNegative[1340] = ''''''
semitonesBySequenceIndex[1340] = [0, 2, 3, 6, 9, 10, 11]
poemBySequenceIndex[1340] = '''B F G L P R S'''
poemAuthors[1340] = ''''''
#1341: ䷶䷘ Abounding Integrity: 2 1 3 3 1 1 1  : S R2 G2 M2 D2 N2 N3 : IbAg

#1342: C D E♭ G A♭ A B♭ :0 2 3 7 8 9 10: 1 2 ♭3 5 ♭6 6 ♭7: mi9(add ♭13 13)
namesBySequenceIndex[1341] = '''Mocrian'''
namesBySequenceIndexNegative[1341] = ''''''
semitonesBySequenceIndex[1341] = [0, 2, 3, 7, 8, 9, 10]
poemBySequenceIndex[1341] = '''B F G M N P R'''
poemAuthors[1341] = ''''''
#1342: ䷶䷛ Abounding Excess: 2 1 4 1 1 1 2  : S R2 G2 P D1 D2 N2 : Ibod

#1343: C D E♭ G A♭ A B :0 2 3 7 8 9 11: 1 2 ♭3 5 ♭6 6 7: mima9(add ♭13 13)
namesBySequenceIndex[1342] = '''Thygian'''
namesBySequenceIndexNegative[1342] = ''''''
semitonesBySequenceIndex[1342] = [0, 2, 3, 7, 8, 9, 11]
poemBySequenceIndex[1342] = '''B F G M N P S'''
poemAuthors[1342] = ''''''
#1343: ䷶䷱ Abounding Holding: 2 1 4 1 1 2 1  : S R2 G2 P D1 D2 N3 : Ibof

#1344: C D E♭ G A♭ B♭ B :0 2 3 7 8 10 11: 1 2 ♭3 5 ♭6 ♭7 7: mima9(add ♭7 ♭13)
namesBySequenceIndex[1343] = '''Monian'''
namesBySequenceIndexNegative[1343] = ''''''
semitonesBySequenceIndex[1343] = [0, 2, 3, 7, 8, 10, 11]
poemBySequenceIndex[1343] = '''B F G M N R S'''
poemAuthors[1343] = ''''''
#1344: ䷶䷸ Abounding Osmosis: 2 1 4 1 2 1 1  : S R2 G2 P D1 N2 N3 : Ibom

#1345: C D E♭ G A B♭ B :0 2 3 7 9 10 11: 1 2 ♭3 5 6 ♭7 7: mima9(add ♭7 13)
namesBySequenceIndex[1344] = '''Kyrian'''
namesBySequenceIndexNegative[1344] = ''''''
semitonesBySequenceIndex[1344] = [0, 2, 3, 7, 9, 10, 11]
poemBySequenceIndex[1344] = '''B F G M P R S'''
poemAuthors[1344] = ''''''
#1345: ䷶䷅ Abounding Arguing: 2 1 4 2 1 1 1  : S R2 G2 P D2 N2 N3 : Ibeg

#1346: C D E♭ G♯ A B♭ B :0 2 3 8 9 10 11: 1 2 ♭3 ♯5 6 ♭7 7: mima9(♯5 add ♭7 13)
namesBySequenceIndex[1345] = ''''''
namesBySequenceIndexNegative[1345] = ''''''
semitonesBySequenceIndex[1345] = [0, 2, 3, 8, 9, 10, 11]
poemBySequenceIndex[1345] = '''B F G N P R S'''
poemAuthors[1345] = ''''''
#1346: ䷶䷠ Abounding Retiring: 2 1 5 1 1 1 1  : S R2 G2 D1 D2 N2 N3 : Ibig

#1347: C D E F F♯ G A♭ :0 2 4 5 6 7 8: 1 2 3 4 ♯4 5 ♭6: 11(no 7 add ♭6 ♯11)
namesBySequenceIndex[1346] = '''Katylian'''
namesBySequenceIndexNegative[1346] = ''''''
semitonesBySequenceIndex[1346] = [0, 2, 4, 5, 6, 7, 8]
poemBySequenceIndex[1346] = '''B F J K L M N'''
poemAuthors[1346] = ''''''
#1347: ䷤䷊ Family Peace: 2 2 1 1 1 1 4  : S R2 G3 M1 M2 P D1 : ImOj

#1348: C D E F F♯ G A :0 2 4 5 6 7 9: 1 2 3 4 ♯4 5 6: 13(no 7 add ♯11)
namesBySequenceIndex[1347] = '''Ionyptian'''
namesBySequenceIndexNegative[1347] = ''''''
semitonesBySequenceIndex[1347] = [0, 2, 4, 5, 6, 7, 9]
poemBySequenceIndex[1347] = '''B F J K L M P'''
poemAuthors[1347] = ''''''
#1348: ䷤䷵ Family Betrothing: 2 2 1 1 1 2 3  : S R2 G3 M1 M2 P D2 : ImEb

#1349: C D E F F♯ G B♭ :0 2 4 5 6 7 10: 1 2 3 4 ♯4 5 ♭7: 11(add ♯11)
namesBySequenceIndex[1348] = '''Katythian'''
namesBySequenceIndexNegative[1348] = ''''''
semitonesBySequenceIndex[1348] = [0, 2, 4, 5, 6, 7, 10]
poemBySequenceIndex[1348] = '''B F J K L M R'''
poemAuthors[1348] = ''''''
#1349: ䷤䷻ Family Composed: 2 2 1 1 1 3 2  : S R2 G3 M1 M2 P N2 : ImEk

#1350: C D E F F♯ G B :0 2 4 5 6 7 11: 1 2 3 4 ♯4 5 7: ma11(add ♯11)
namesBySequenceIndex[1349] = '''Gorian'''
namesBySequenceIndexNegative[1349] = ''''''
semitonesBySequenceIndex[1349] = [0, 2, 4, 5, 6, 7, 11]
poemBySequenceIndex[1349] = '''B F J K L M S'''
poemAuthors[1349] = ''''''
#1350: ䷤䷨ Family Diminishing: 2 2 1 1 1 4 1  : S R2 G3 M1 M2 P N3 : ImEl

#1351: C D E F G♭ A♭ A :0 2 4 5 6 8 9: 1 2 3 4 ♭5 ♭6 6: 13(♯5 no 7 add ♯11)
namesBySequenceIndex[1350] = '''Sathian'''
namesBySequenceIndexNegative[1350] = ''''''
semitonesBySequenceIndex[1350] = [0, 2, 4, 5, 6, 8, 9]
poemBySequenceIndex[1350] = '''B F J K L N P'''
poemAuthors[1350] = ''''''
#1351: ䷤䷶ Family Abounding: 2 2 1 1 2 1 3  : S R2 G3 M1 M2 D1 N1 : ImIb

#1352: C D E F G♭ A♭ B♭ :0 2 4 5 6 8 10: 1 2 3 4 ♭5 ♭6 ♭7: 11(♯5 add ♯11)
namesBySequenceIndex[1351] = '''Major Locrian,Gothian'''
namesBySequenceIndexNegative[1351] = ''''''
semitonesBySequenceIndex[1351] = [0, 2, 4, 5, 6, 8, 10]
poemBySequenceIndex[1351] = '''B F J K L N R'''
poemAuthors[1351] = ''''''
#1352: ䷤䷾ Family Completed: 2 2 1 1 2 2 2  : S R2 G3 M1 M2 D1 N2 : ImIk

#1353: C D E F G♭ A♭ B :0 2 4 5 6 8 11: 1 2 3 4 ♭5 ♭6 7: ma11(♯5 add ♯11)
namesBySequenceIndex[1352] = '''Malian'''
namesBySequenceIndexNegative[1352] = ''''''
semitonesBySequenceIndex[1352] = [0, 2, 4, 5, 6, 8, 11]
poemBySequenceIndex[1352] = '''B F J K L N S'''
poemAuthors[1352] = ''''''
#1353: ䷤䷕ Family Adorning: 2 2 1 1 2 3 1  : S R2 G3 M1 M2 D1 N3 : ImIl

#1354: C D E F G♭ A B♭ :0 2 4 5 6 9 10: 1 2 3 4 ♭5 6 ♭7: 13(♭5)
namesBySequenceIndex[1353] = '''Gylian,Mixolydian ♭5'''
namesBySequenceIndexNegative[1353] = ''''''
semitonesBySequenceIndex[1353] = [0, 2, 4, 5, 6, 9, 10]
poemBySequenceIndex[1353] = '''B F J K L P R'''
poemAuthors[1353] = ''''''
#1354: ䷤䷐ Family Following: 2 2 1 1 3 1 2  : S R2 G3 M1 M2 D2 N2 : ImAd

#1355: C D E F G♭ A B :0 2 4 5 6 9 11: 1 2 3 4 ♭5 6 7: ma13(♭5)
namesBySequenceIndex[1354] = '''Thodian,Major ♭5'''
namesBySequenceIndexNegative[1354] = ''''''
semitonesBySequenceIndex[1354] = [0, 2, 4, 5, 6, 9, 11]
poemBySequenceIndex[1354] = '''B F J K L P S'''
poemAuthors[1354] = ''''''
#1355: ䷤䷔ Family Biting: 2 2 1 1 3 2 1  : S R2 G3 M1 M2 D2 N3 : ImAf

#1356: C D E F G♭ A♯ B :0 2 4 5 6 10 11: 1 2 3 4 ♭5 ♯6 7: ma11(♭5 add ♭7)
namesBySequenceIndex[1355] = '''Aeolonian'''
namesBySequenceIndexNegative[1355] = ''''''
semitonesBySequenceIndex[1355] = [0, 2, 4, 5, 6, 10, 11]
poemBySequenceIndex[1355] = '''B F J K L R S'''
poemAuthors[1355] = ''''''
#1356: ䷤䷩ Family Augmenting: 2 2 1 1 4 1 1  : S R2 G3 M1 M2 D3 N3 : ImAm

#1357: C D E F G A♭ A :0 2 4 5 7 8 9: 1 2 3 4 5 ♭6 6: 13(no 7 add ♭6)
namesBySequenceIndex[1356] = '''Mela Maranjani,Mela Mararanjani,Raga Keseri,Bebop Major Heptatonic,Ionagian'''
namesBySequenceIndexNegative[1356] = ''''''
semitonesBySequenceIndex[1356] = [0, 2, 4, 5, 7, 8, 9]
poemBySequenceIndex[1356] = '''B F J K M N P'''
poemAuthors[1356] = ''''''
#1357: ䷤䷟ Family Persevering: 2 2 1 2 1 1 3  : S R2 G3 M1 P D1 N1 : Imob

#1358: C D E F G A♭ B♭ :0 2 4 5 7 8 10: 1 2 3 4 5 ♭6 ♭7: 11(add ♭13)
namesBySequenceIndex[1357] = '''Mela Charukesi,Melodic Major,Rite Of Spring,Aeolian Major,Major Minor,Mischung 6 (Germany),Hindu,Maqam Ussak (Iraq),Mela Carukesi,Raga Tarangini,Stydian,Mixolydian ♭6,Mixolydian ♭13'''
namesBySequenceIndexNegative[1357] = ''''''
semitonesBySequenceIndex[1357] = [0, 2, 4, 5, 7, 8, 10]
poemBySequenceIndex[1357] = '''B F J K M N R'''
poemAuthors[1357] = ''''''
#1358: ䷤䷯ Family Source: 2 2 1 2 1 2 2  : S R2 G3 M1 P D1 N2 : Imok

#1359: C D E F G A♭ B :0 2 4 5 7 8 11: 1 2 3 4 5 ♭6 7: ma11(add ♭13)
namesBySequenceIndex[1358] = '''Mela Sarasangi,Harmonic Major 1,Segiah,Tabahaniotiko (Greece),Mischung 2 (Germany),Raga Haripriya,Aerorian'''
namesBySequenceIndexNegative[1358] = ''''''
semitonesBySequenceIndex[1358] = [0, 2, 4, 5, 7, 8, 11]
poemBySequenceIndex[1358] = '''B F J K M N S'''
poemAuthors[1358] = ''''''
#1359: ䷤䷑ Family Correcting: 2 2 1 2 1 3 1  : S R2 G3 M1 P D1 N3 : Imol

#1360: C D E F G A B♭ :0 2 4 5 7 9 10: 1 2 3 4 5 6 ♭7: 13
namesBySequenceIndex[1359] = '''Mixolydian,Gregorian 2,Mischung 3 (Germany),Shang (China),Mela Harikamboji,Mela Harikambhoji,Raga Khamach (descending),Raga Kambodhi (descending),Raga Khamaj That,Raga Janjhuti,Raga Harini,Raga Khambhavati,Raga Surati,Raga Balahamsa'''
namesBySequenceIndexNegative[1359] = ''''''
semitonesBySequenceIndex[1359] = [0, 2, 4, 5, 7, 9, 10]
poemBySequenceIndex[1359] = '''B F J K M P R'''
poemAuthors[1359] = ''''''
#1360: ䷤䷮ Family Confining: 2 2 1 2 2 1 2  : S R2 G3 M1 P D2 N2 : Imed

#1361: C D E F G A B :0 2 4 5 7 9 11: 1 2 3 4 5 6 7: ma13
namesBySequenceIndex[1360] = '''Major,Ionian,Ghana Heptatonic,Ararai (Ethiopia),Xin (China),Maqam Cargah,Ajam Ashiran,Dastgah-e Mahur,Dastgah-e Rast Panjgah,Raga Bilaval That,Raga Arabhi (descending),Raga Bilahari (descending),Mela Shankarabharanam,Mela Dheerasankarabaranam'''
namesBySequenceIndexNegative[1360] = ''''''
semitonesBySequenceIndex[1360] = [0, 2, 4, 5, 7, 9, 11]
poemBySequenceIndex[1360] = '''Belonging Finds Joint Kinship Multiplying Personal Stories'''
poemAuthors[1360] = ''''''
#1361: ䷤䷿ Family Unfinished: 2 2 1 2 2 2 1  : S R2 G3 M1 P D2 N3 : Imef

#1362: C D E F G A♯ B :0 2 4 5 7 10 11: 1 2 3 4 5 ♯6 7: ma11(add ♭7)
namesBySequenceIndex[1361] = '''Mela Naganandini,Raga Samanta,Raga Nagabharanam,Gonian'''
namesBySequenceIndexNegative[1361] = ''''''
semitonesBySequenceIndex[1361] = [0, 2, 4, 5, 7, 10, 11]
poemBySequenceIndex[1361] = '''B F J K M R S'''
poemAuthors[1361] = ''''''
#1362: ䷤䷺ Family Dispersing: 2 2 1 2 3 1 1  : S R2 G3 M1 P D3 N3 : Imem

#1363: C D E F G♯ A B♭ :0 2 4 5 8 9 10: 1 2 3 4 ♯5 6 ♭7: 13(♯5)
namesBySequenceIndex[1362] = '''Lagian,Mixolydian ♯5'''
namesBySequenceIndexNegative[1362] = ''''''
semitonesBySequenceIndex[1362] = [0, 2, 4, 5, 8, 9, 10]
poemBySequenceIndex[1362] = '''B F J K N P R'''
poemAuthors[1362] = ''''''
#1363: ䷤䷞ Family Conjoining: 2 2 1 3 1 1 2  : S R2 G3 M1 D1 D2 N2 : Imid

#1364: C D E F G♯ A B :0 2 4 5 8 9 11: 1 2 3 4 ♯5 6 7: ma13(♯5)
namesBySequenceIndex[1363] = '''Phrothian,Ionian ♯5,Ionian Augmented,Harmonic Major 2'''
namesBySequenceIndexNegative[1363] = ''''''
semitonesBySequenceIndex[1363] = [0, 2, 4, 5, 8, 9, 11]
poemBySequenceIndex[1363] = '''B F J K N P S'''
poemAuthors[1363] = ''''''
#1364: ䷤䷷ Family Wanderer: 2 2 1 3 1 2 1  : S R2 G3 M1 D1 D2 N3 : Imif

#1365: C D E F G♯ A♯ B :0 2 4 5 8 10 11: 1 2 3 4 ♯5 ♯6 7: ma11(♯5 add ♭7)
namesBySequenceIndex[1364] = '''Katanian'''
namesBySequenceIndexNegative[1364] = ''''''
semitonesBySequenceIndex[1364] = [0, 2, 4, 5, 8, 10, 11]
poemBySequenceIndex[1364] = '''B F J K N R S'''
poemAuthors[1364] = ''''''
#1365: ䷤䷴ Family Infiltrating: 2 2 1 3 2 1 1  : S R2 G3 M1 D1 N2 N3 : Imim

#1366: C D E F A B♭ B :0 2 4 5 9 10 11: 1 2 3 4 6 ♭7 7: ma13(no 5 add ♭7)
namesBySequenceIndex[1365] = '''Raga Rageshri,Kygian'''
namesBySequenceIndexNegative[1365] = ''''''
semitonesBySequenceIndex[1365] = [0, 2, 4, 5, 9, 10, 11]
poemBySequenceIndex[1365] = '''B F J K P R S'''
poemAuthors[1365] = ''''''
#1366: ䷤䷋ Family Obstruction: 2 2 1 4 1 1 1  : S R2 G3 M1 D2 N2 N3 : Imag

#1367: C D E F♯ G A♭ A :0 2 4 6 7 8 9: 1 2 3 ♯4 5 ♭6 6: 6/9(add ♯11 ♭13)
namesBySequenceIndex[1366] = '''Mela Kantamani,Raga Srutiranjani,Raga Kuntala,Bacrian'''
namesBySequenceIndexNegative[1366] = ''''''
semitonesBySequenceIndex[1366] = [0, 2, 4, 6, 7, 8, 9]
poemBySequenceIndex[1366] = '''B F J L M N P'''
poemAuthors[1366] = ''''''
#1367: ䷾䷡ Completed Might: 2 2 2 1 1 1 3  : S R2 G3 M2 P D1 N1 : IkOb

#1368: C D E F♯ G A♭ B♭ :0 2 4 6 7 8 10: 1 2 3 ♯4 5 ♭6 ♭7: 9(add ♯11 ♭13)
namesBySequenceIndex[1367] = '''Mela Risabhapriya,Raga Ratipriya,Lydian Minor,Raga Rishabapriya,Lydian Dominant b6,Phryrian'''
namesBySequenceIndexNegative[1367] = ''''''
semitonesBySequenceIndex[1367] = [0, 2, 4, 6, 7, 8, 10]
poemBySequenceIndex[1367] = '''B F J L M N R'''
poemAuthors[1367] = ''''''
#1368: ䷾䷄ Completed Biding: 2 2 2 1 1 2 2  : S R2 G3 M2 P D1 N2 : IkOk

#1369: C D E F♯ G A♭ B :0 2 4 6 7 8 11: 1 2 3 ♯4 5 ♭6 7: ma9(add ♯11 ♭13)
namesBySequenceIndex[1368] = '''Mela Latangi,Raga Gitapriya,Raga Hamsalata,Ryphian,Pelog (Philippines Java Bali)'''
namesBySequenceIndexNegative[1368] = ''''''
semitonesBySequenceIndex[1368] = [0, 2, 4, 6, 7, 8, 11]
poemBySequenceIndex[1368] = '''B F J L M N S'''
poemAuthors[1368] = ''''''
#1369: ䷾䷙ Completed Cultivating: 2 2 2 1 1 3 1  : S R2 G3 M2 P D1 N3 : IkOl

#1370: C D E F♯ G A B♭ :0 2 4 6 7 9 10: 1 2 3 ♯4 5 6 ♭7: 13(♯11)
namesBySequenceIndex[1369] = '''Overtone,Acoustic,Lydian Dominant,Lydian ♭7,Mixolydian ♯4,Bartok,Mela Vacaspati,Raga Bhusavati or Bhusavali,Lythian'''
namesBySequenceIndexNegative[1369] = ''''''
semitonesBySequenceIndex[1369] = [0, 2, 4, 6, 7, 9, 10]
poemBySequenceIndex[1369] = '''B F J L M P R'''
poemAuthors[1369] = ''''''
#1370: ䷾䷹ Completed Joyous: 2 2 2 1 2 1 2  : S R2 G3 M2 P D2 N2 : IkEd

#1371: C D E F♯ G A B :0 2 4 6 7 9 11: 1 2 3 ♯4 5 6 7: ma13(♯11)
namesBySequenceIndex[1370] = '''Lydian,Mela Mecakalyani,Ping (China),Gu (China),Raga Shuddh Kalyan,Raga Kalyan That'''
namesBySequenceIndexNegative[1370] = ''''''
semitonesBySequenceIndex[1370] = [0, 2, 4, 6, 7, 9, 11]
poemBySequenceIndex[1370] = '''B F J L M P S'''
poemAuthors[1370] = ''''''
#1371: ䷾䷥ Completed Polarising: 2 2 2 1 2 2 1  : S R2 G3 M2 P D2 N3 : IkEf

#1372: C D E F♯ G A♯ B :0 2 4 6 7 10 11: 1 2 3 ♯4 5 ♯6 7: ma9(add ♭7 ♯11)
namesBySequenceIndex[1371] = '''Mela Citrambari,Raga Chaturangini 1,Lydian ♯6,Zagian'''
namesBySequenceIndexNegative[1371] = ''''''
semitonesBySequenceIndex[1371] = [0, 2, 4, 6, 7, 10, 11]
poemBySequenceIndex[1371] = '''B F J L M R S'''
poemAuthors[1371] = ''''''
#1372: ䷾䷼ Completed Truth: 2 2 2 1 3 1 1  : S R2 G3 M2 P D3 N3 : IkEm

#1373: C D E F♯ G♯ A B♭ :0 2 4 6 8 9 10: 1 2 3 ♯4 ♯5 6 ♭7: 13(♯5 ♯11)
namesBySequenceIndex[1372] = '''Aeroptian,Overtone ♯5,Synthetic Mixture ♯5'''
namesBySequenceIndexNegative[1372] = ''''''
semitonesBySequenceIndex[1372] = [0, 2, 4, 6, 8, 9, 10]
poemBySequenceIndex[1372] = '''B F J L N P R'''
poemAuthors[1372] = ''''''
#1373: ䷾䷰ Completed Revolution: 2 2 2 2 1 1 2  : S R2 G3 M2 D1 D2 N2 : IkId

#1374: C D E F♯ G♯ A B :0 2 4 6 8 9 11: 1 2 3 ♯4 ♯5 6 7: ma13(♯5 ♯11)
namesBySequenceIndex[1373] = '''Larian,Lydian ♯5,Lydian Augmented,Altered Lydian'''
namesBySequenceIndexNegative[1373] = ''''''
semitonesBySequenceIndex[1373] = [0, 2, 4, 6, 8, 9, 11]
poemBySequenceIndex[1373] = '''B F J L N P S'''
poemAuthors[1373] = ''''''
#1374: ䷾䷝ Completed Distanced: 2 2 2 2 1 2 1  : S R2 G3 M2 D1 D2 N3 : IkIf

#1375: C D E F♯ G♯ A♯ B :0 2 4 6 8 10 11: 1 2 3 ♯4 ♯5 ♯6 7: ma9(♯5 add ♭7 ♯11)
namesBySequenceIndex[1374] = '''Aeolynian,Leading Whole-Tone'''
namesBySequenceIndexNegative[1374] = ''''''
semitonesBySequenceIndex[1374] = [0, 2, 4, 6, 8, 10, 11]
poemBySequenceIndex[1374] = '''B F J L N R S'''
poemAuthors[1374] = ''''''
#1375: ䷾䷤ Completed Family: 2 2 2 2 2 1 1  : S R2 G3 M2 D1 N2 N3 : IkIm

#1376: C D E G♭ A B♭ B :0 2 4 6 9 10 11: 1 2 3 ♭5 6 ♭7 7: ma9(♭5 add ♭7 13)
namesBySequenceIndex[1375] = '''Mothian'''
namesBySequenceIndexNegative[1375] = ''''''
semitonesBySequenceIndex[1375] = [0, 2, 4, 6, 9, 10, 11]
poemBySequenceIndex[1375] = '''B F J L P R S'''
poemAuthors[1375] = ''''''
#1376: ䷾䷘ Completed Integrity: 2 2 2 3 1 1 1  : S R2 G3 M2 D2 N2 N3 : IkAg

#1377: C D E G A♭ A B♭ :0 2 4 7 8 9 10: 1 2 3 5 ♭6 6 ♭7: 9(add ♭13 13)
namesBySequenceIndex[1376] = '''Aeranian'''
namesBySequenceIndexNegative[1376] = ''''''
semitonesBySequenceIndex[1376] = [0, 2, 4, 7, 8, 9, 10]
poemBySequenceIndex[1376] = '''B F J M N P R'''
poemAuthors[1376] = ''''''
#1377: ䷾䷛ Completed Excess: 2 2 3 1 1 1 2  : S R2 G3 P D1 D2 N2 : Ikod

#1378: C D E G A♭ A B :0 2 4 7 8 9 11: 1 2 3 5 ♭6 6 7: ma9(add ♭13 13)
namesBySequenceIndex[1377] = '''Darian'''
namesBySequenceIndexNegative[1377] = ''''''
semitonesBySequenceIndex[1377] = [0, 2, 4, 7, 8, 9, 11]
poemBySequenceIndex[1377] = '''B F J M N P S'''
poemAuthors[1377] = ''''''
#1378: ䷾䷱ Completed Holding: 2 2 3 1 1 2 1  : S R2 G3 P D1 D2 N3 : Ikof

#1379: C D E G A♭ B♭ B :0 2 4 7 8 10 11: 1 2 3 5 ♭6 ♭7 7: ma9(add ♭7 ♭13)
namesBySequenceIndex[1378] = '''Kyptian'''
namesBySequenceIndexNegative[1378] = ''''''
semitonesBySequenceIndex[1378] = [0, 2, 4, 7, 8, 10, 11]
poemBySequenceIndex[1378] = '''B F J M N R S'''
poemAuthors[1378] = ''''''
#1379: ䷾䷸ Completed Osmosis: 2 2 3 1 2 1 1  : S R2 G3 P D1 N2 N3 : Ikom

#1380: C D E G A B♭ B :0 2 4 7 9 10 11: 1 2 3 5 6 ♭7 7: ma9(add ♭7 13)
namesBySequenceIndex[1379] = '''Gycrian'''
namesBySequenceIndexNegative[1379] = ''''''
semitonesBySequenceIndex[1379] = [0, 2, 4, 7, 9, 10, 11]
poemBySequenceIndex[1379] = '''B F J M P R S'''
poemAuthors[1379] = ''''''
#1380: ䷾䷅ Completed Arguing: 2 2 3 2 1 1 1  : S R2 G3 P D2 N2 N3 : Ikeg

#1381: C D E G♯ A B♭ B :0 2 4 8 9 10 11: 1 2 3 ♯5 6 ♭7 7: ma9(♯5 add ♭7 13)
namesBySequenceIndex[1380] = '''Phroptian'''
namesBySequenceIndexNegative[1380] = ''''''
semitonesBySequenceIndex[1380] = [0, 2, 4, 8, 9, 10, 11]
poemBySequenceIndex[1380] = '''B F J N P R S'''
poemAuthors[1380] = ''''''
#1381: ䷾䷠ Completed Retiring: 2 2 4 1 1 1 1  : S R2 G3 D1 D2 N2 N3 : Ikig

#1382: C D F F♯ G A♭ A :0 2 5 6 7 8 9: 1 2 4 ♯4 5 ♭6 6: 6/9sus4(add ♯11 ♭13)
namesBySequenceIndex[1381] = '''Rycrian'''
namesBySequenceIndexNegative[1381] = ''''''
semitonesBySequenceIndex[1381] = [0, 2, 5, 6, 7, 8, 9]
poemBySequenceIndex[1381] = '''B F K L M N P'''
poemAuthors[1381] = ''''''
#1382: ䷕䷡ Adorning Might: 2 3 1 1 1 1 3  : S R2 M1 M2 P D1 N1 : IlOb

#1383: C D F F♯ G A♭ B♭ :0 2 5 6 7 8 10: 1 2 4 ♯4 5 ♭6 ♭7: 11sus♯4(add ♭13)
namesBySequenceIndex[1382] = '''Ragian'''
namesBySequenceIndexNegative[1382] = ''''''
semitonesBySequenceIndex[1382] = [0, 2, 5, 6, 7, 8, 10]
poemBySequenceIndex[1382] = '''B F K L M N R'''
poemAuthors[1382] = ''''''
#1383: ䷕䷄ Adorning Biding: 2 3 1 1 1 2 2  : S R2 M1 M2 P D1 N2 : IlOk

#1384: C D F F♯ G A♭ B :0 2 5 6 7 8 11: 1 2 4 ♯4 5 ♭6 7: ma11sus♯4(add ♭13)
namesBySequenceIndex[1383] = '''Podian'''
namesBySequenceIndexNegative[1383] = ''''''
semitonesBySequenceIndex[1383] = [0, 2, 5, 6, 7, 8, 11]
poemBySequenceIndex[1383] = '''B F K L M N S'''
poemAuthors[1383] = ''''''
#1384: ䷕䷙ Adorning Cultivating: 2 3 1 1 1 3 1  : S R2 M1 M2 P D1 N3 : IlOl

#1385: C D F F♯ G A B♭ :0 2 5 6 7 9 10: 1 2 4 ♯4 5 6 ♭7: 13sus♯4
namesBySequenceIndex[1384] = '''Lonian'''
namesBySequenceIndexNegative[1384] = ''''''
semitonesBySequenceIndex[1384] = [0, 2, 5, 6, 7, 9, 10]
poemBySequenceIndex[1384] = '''B F K L M P R'''
poemAuthors[1384] = ''''''
#1385: ䷕䷹ Adorning Joyous: 2 3 1 1 2 1 2  : S R2 M1 M2 P D2 N2 : IlEd

#1386: C D F F♯ G A B :0 2 5 6 7 9 11: 1 2 4 ♯4 5 6 7: ma13sus♯4
namesBySequenceIndex[1385] = '''Zolian'''
namesBySequenceIndexNegative[1385] = ''''''
semitonesBySequenceIndex[1385] = [0, 2, 5, 6, 7, 9, 11]
poemBySequenceIndex[1385] = '''B F K L M P S'''
poemAuthors[1385] = ''''''
#1386: ䷕䷥ Adorning Polarising: 2 3 1 1 2 2 1  : S R2 M1 M2 P D2 N3 : IlEf

#1387: C D F F♯ G A♯ B :0 2 5 6 7 10 11: 1 2 4 ♯4 5 ♯6 7: ma11sus♯4(add ♭7)
namesBySequenceIndex[1386] = '''Phrynian'''
namesBySequenceIndexNegative[1386] = ''''''
semitonesBySequenceIndex[1386] = [0, 2, 5, 6, 7, 10, 11]
poemBySequenceIndex[1386] = '''B F K L M R S'''
poemAuthors[1386] = ''''''
#1387: ䷕䷼ Adorning Truth: 2 3 1 1 3 1 1  : S R2 M1 M2 P D3 N3 : IlEm

#1388: C D F G♭ A♭ A B♭ :0 2 5 6 8 9 10: 1 2 4 ♭5 ♭6 6 ♭7: 13sus♯4(♯5)
namesBySequenceIndex[1387] = '''Ionylian'''
namesBySequenceIndexNegative[1387] = ''''''
semitonesBySequenceIndex[1387] = [0, 2, 5, 6, 8, 9, 10]
poemBySequenceIndex[1387] = '''B F K L N P R'''
poemAuthors[1387] = ''''''
#1388: ䷕䷰ Adorning Revolution: 2 3 1 2 1 1 2  : S R2 M1 M2 D1 D2 N2 : IlId

#1389: C D F G♭ A♭ A B :0 2 5 6 8 9 11: 1 2 4 ♭5 ♭6 6 7: ma13sus♯4(♯5)
namesBySequenceIndex[1388] = '''Nohkan,Lydian Augmented ♯3,Lydian ♯5 ♯3,Kocrian'''
namesBySequenceIndexNegative[1388] = ''''''
semitonesBySequenceIndex[1388] = [0, 2, 5, 6, 8, 9, 11]
poemBySequenceIndex[1388] = '''B F K L N P S'''
poemAuthors[1388] = ''''''
#1389: ䷕䷝ Adorning Distanced: 2 3 1 2 1 2 1  : S R2 M1 M2 D1 D2 N3 : IlIf

#1390: C D F G♭ A♭ B♭ B :0 2 5 6 8 10 11: 1 2 4 ♭5 ♭6 ♭7 7: ma11sus♯4(♯5 add ♭7)
namesBySequenceIndex[1389] = '''Marian'''
namesBySequenceIndexNegative[1389] = ''''''
semitonesBySequenceIndex[1389] = [0, 2, 5, 6, 8, 10, 11]
poemBySequenceIndex[1389] = '''B F K L N R S'''
poemAuthors[1389] = ''''''
#1390: ䷕䷤ Adorning Family: 2 3 1 2 2 1 1  : S R2 M1 M2 D1 N2 N3 : IlIm

#1391: C D F G♭ A B♭ B :0 2 5 6 9 10 11: 1 2 4 ♭5 6 ♭7 7: ma13(♭5 no 3 add ♭7)
namesBySequenceIndex[1390] = '''Kodian'''
namesBySequenceIndexNegative[1390] = ''''''
semitonesBySequenceIndex[1390] = [0, 2, 5, 6, 9, 10, 11]
poemBySequenceIndex[1390] = '''B F K L P R S'''
poemAuthors[1390] = ''''''
#1391: ䷕䷘ Adorning Integrity: 2 3 1 3 1 1 1  : S R2 M1 M2 D2 N2 N3 : IlAg

#1392: C D F G A♭ A B♭ :0 2 5 7 8 9 10: 1 2 4 5 ♭6 6 ♭7: 13(no 3 add ♭13)
namesBySequenceIndex[1391] = '''Pyrian'''
namesBySequenceIndexNegative[1391] = ''''''
semitonesBySequenceIndex[1391] = [0, 2, 5, 7, 8, 9, 10]
poemBySequenceIndex[1391] = '''B F K M N P R'''
poemAuthors[1391] = ''''''
#1392: ䷕䷛ Adorning Excess: 2 3 2 1 1 1 2  : S R2 M1 P D1 D2 N2 : Ilod

#1393: C D F G A♭ A B :0 2 5 7 8 9 11: 1 2 4 5 ♭6 6 7: ma13(no 3 add ♭13)
namesBySequenceIndex[1392] = '''Noh,Ionolian'''
namesBySequenceIndexNegative[1392] = ''''''
semitonesBySequenceIndex[1392] = [0, 2, 5, 7, 8, 9, 11]
poemBySequenceIndex[1392] = '''B F K M N P S'''
poemAuthors[1392] = ''''''
#1393: ䷕䷱ Adorning Holding: 2 3 2 1 1 2 1  : S R2 M1 P D1 D2 N3 : Ilof

#1394: C D F G A♭ B♭ B :0 2 5 7 8 10 11: 1 2 4 5 ♭6 ♭7 7: ma9sus4(add ♭7 ♭13)
namesBySequenceIndex[1393] = '''Rathian'''
namesBySequenceIndexNegative[1393] = ''''''
semitonesBySequenceIndex[1393] = [0, 2, 5, 7, 8, 10, 11]
poemBySequenceIndex[1393] = '''B F K M N R S'''
poemAuthors[1393] = ''''''
#1394: ䷕䷸ Adorning Osmosis: 2 3 2 1 2 1 1  : S R2 M1 P D1 N2 N3 : Ilom

#1395: C D F G A B♭ B :0 2 5 7 9 10 11: 1 2 4 5 6 ♭7 7: ma13(no 3 add ♭7)
namesBySequenceIndex[1394] = '''Raga Sorati,Raga Sur Malhar,Zothian'''
namesBySequenceIndexNegative[1394] = ''''''
semitonesBySequenceIndex[1394] = [0, 2, 5, 7, 9, 10, 11]
poemBySequenceIndex[1394] = '''B F K M P R S'''
poemAuthors[1394] = ''''''
#1395: ䷕䷅ Adorning Arguing: 2 3 2 2 1 1 1  : S R2 M1 P D2 N2 N3 : Ileg

#1396: C D F G♯ A B♭ B :0 2 5 8 9 10 11: 1 2 4 ♯5 6 ♭7 7: ma13(♯5 no 3 add ♭7)
namesBySequenceIndex[1395] = '''Thanian'''
namesBySequenceIndexNegative[1395] = ''''''
semitonesBySequenceIndex[1395] = [0, 2, 5, 8, 9, 10, 11]
poemBySequenceIndex[1395] = '''B F K N P R S'''
poemAuthors[1395] = ''''''
#1396: ䷕䷠ Adorning Retiring: 2 3 3 1 1 1 1  : S R2 M1 D1 D2 N2 N3 : Ilig

#1397: C D F♯ G A♭ A B♭ :0 2 6 7 8 9 10: 1 2 ♯4 5 ♭6 6 ♭7: 13(♯11 no 3 add ♭13)
namesBySequenceIndex[1396] = '''Dydian'''
namesBySequenceIndexNegative[1396] = ''''''
semitonesBySequenceIndex[1396] = [0, 2, 6, 7, 8, 9, 10]
poemBySequenceIndex[1396] = '''B F L M N P R'''
poemAuthors[1396] = ''''''
#1397: ䷣䷪ Eclipse Decision: 2 4 1 1 1 1 2  : S R2 M2 P D1 D2 N2 : IjOd

#1398: C D F♯ G A♭ A B :0 2 6 7 8 9 11: 1 2 ♯4 5 ♭6 6 7: ma13(♯11 no 3 add ♭13)
namesBySequenceIndex[1397] = '''Thynian'''
namesBySequenceIndexNegative[1397] = ''''''
semitonesBySequenceIndex[1397] = [0, 2, 6, 7, 8, 9, 11]
poemBySequenceIndex[1397] = '''B F L M N P S'''
poemAuthors[1397] = ''''''
#1398: ䷣䷍ Eclipse Resevoir: 2 4 1 1 1 2 1  : S R2 M2 P D1 D2 N3 : IjOf

#1399: C D F♯ G A♭ B♭ B :0 2 6 7 8 10 11: 1 2 ♯4 5 ♭6 ♭7 7: ma9sus♯4(add ♭7 ♭13)
namesBySequenceIndex[1398] = '''Zocrian'''
namesBySequenceIndexNegative[1398] = ''''''
semitonesBySequenceIndex[1398] = [0, 2, 6, 7, 8, 10, 11]
poemBySequenceIndex[1398] = '''B F L M N R S'''
poemAuthors[1398] = ''''''
#1399: ䷣䷈ Eclipse Homegrown: 2 4 1 1 2 1 1  : S R2 M2 P D1 N2 N3 : IjOm

#1400: C D F♯ G A B♭ B :0 2 6 7 9 10 11: 1 2 ♯4 5 6 ♭7 7: ma13(♯11 no 3 add ♭7)
namesBySequenceIndex[1399] = '''Gyphian'''
namesBySequenceIndexNegative[1399] = ''''''
semitonesBySequenceIndex[1399] = [0, 2, 6, 7, 9, 10, 11]
poemBySequenceIndex[1399] = '''B F L M P R S'''
poemAuthors[1399] = ''''''
#1400: ䷣䷉ Eclipse Treading: 2 4 1 2 1 1 1  : S R2 M2 P D2 N2 N3 : IjEg

#1401: C D F♯ G♯ A B♭ B :0 2 6 8 9 10 11: 1 2 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♯11 no 3 add ♭7)
namesBySequenceIndex[1400] = '''Rydian'''
namesBySequenceIndexNegative[1400] = ''''''
semitonesBySequenceIndex[1400] = [0, 2, 6, 8, 9, 10, 11]
poemBySequenceIndex[1400] = '''B F L N P R S'''
poemAuthors[1400] = ''''''
#1401: ䷣䷌ Eclipse Fellowship: 2 4 2 1 1 1 1  : S R2 M2 D1 D2 N2 N3 : IjIg

#1402: C D G A♭ A B♭ B :0 2 7 8 9 10 11: 1 2 5 ♭6 6 ♭7 7: ma7sus2(add ♭7 ♭13 13)
namesBySequenceIndex[1401] = ''''''
namesBySequenceIndexNegative[1401] = ''''''
semitonesBySequenceIndex[1401] = [0, 2, 7, 8, 9, 10, 11]
poemBySequenceIndex[1401] = '''B F M N P R S'''
poemAuthors[1401] = ''''''
#1402: ䷣䷫ Eclipse Coupling: 2 5 1 1 1 1 1  : S R2 P D1 D2 N2 N3 : Ijog

#1403: C D♯ E F F♯ G A♭ :0 3 4 5 6 7 8: 1 ♯2 3 4 ♯4 5 ♭6: ma(add ♭6 ♯9 11 ♯11)
namesBySequenceIndex[1402] = '''Sanian'''
namesBySequenceIndexNegative[1402] = ''''''
semitonesBySequenceIndex[1402] = [0, 3, 4, 5, 6, 7, 8]
poemBySequenceIndex[1402] = '''B G J K L M N'''
poemAuthors[1402] = ''''''
#1403: ䷘䷊ Integrity Peace: 3 1 1 1 1 1 4  : S R3 G3 M1 M2 P D1 : AgOj

#1404: C D♯ E F F♯ G A :0 3 4 5 6 7 9: 1 ♯2 3 4 ♯4 5 6: 6(add ♯9 11 ♯11)
namesBySequenceIndex[1403] = '''Ponian'''
namesBySequenceIndexNegative[1403] = ''''''
semitonesBySequenceIndex[1403] = [0, 3, 4, 5, 6, 7, 9]
poemBySequenceIndex[1403] = '''B G J K L M P'''
poemAuthors[1403] = ''''''
#1404: ䷘䷵ Integrity Betrothing: 3 1 1 1 1 2 3  : S R3 G3 M1 M2 P D2 : AgEb

#1405: C D♯ E F F♯ G B♭ :0 3 4 5 6 7 10: 1 ♯2 3 4 ♯4 5 ♭7: 11(♯9 add ♯11)
namesBySequenceIndex[1404] = '''Blues Both Thirds,Ronian'''
namesBySequenceIndexNegative[1404] = ''''''
semitonesBySequenceIndex[1404] = [0, 3, 4, 5, 6, 7, 10]
poemBySequenceIndex[1404] = '''B G J K L M R'''
poemAuthors[1404] = ''''''
#1405: ䷘䷻ Integrity Composed: 3 1 1 1 1 3 2  : S R3 G3 M1 M2 P N2 : AgEk

#1406: C D♯ E F F♯ G B :0 3 4 5 6 7 11: 1 ♯2 3 4 ♯4 5 7: ma11(♯9 add ♯11)
namesBySequenceIndex[1405] = '''Thylian'''
namesBySequenceIndexNegative[1405] = ''''''
semitonesBySequenceIndex[1405] = [0, 3, 4, 5, 6, 7, 11]
poemBySequenceIndex[1405] = '''B G J K L M S'''
poemAuthors[1405] = ''''''
#1406: ䷘䷨ Integrity Diminishing: 3 1 1 1 1 4 1  : S R3 G3 M1 M2 P N3 : AgEl

#1407: C D♯ E F G♭ A♭ A :0 3 4 5 6 8 9: 1 ♯2 3 4 ♭5 ♭6 6: 6(♯5 add ♯9 11 ♯11)
namesBySequenceIndex[1406] = '''Borian'''
namesBySequenceIndexNegative[1406] = ''''''
semitonesBySequenceIndex[1406] = [0, 3, 4, 5, 6, 8, 9]
poemBySequenceIndex[1406] = '''B G J K L N P'''
poemAuthors[1406] = ''''''
#1407: ䷘䷶ Integrity Abounding: 3 1 1 1 2 1 3  : S R3 G3 M1 M2 D1 N1 : AgIb

#1408: C D♯ E F G♭ A♭ B♭ :0 3 4 5 6 8 10: 1 ♯2 3 4 ♭5 ♭6 ♭7: 11(♯5 ♯9 add ♯11)
namesBySequenceIndex[1407] = '''Spanish Heptatonic,Pagian'''
namesBySequenceIndexNegative[1407] = ''''''
semitonesBySequenceIndex[1407] = [0, 3, 4, 5, 6, 8, 10]
poemBySequenceIndex[1407] = '''B G J K L N R'''
poemAuthors[1407] = ''''''
#1408: ䷘䷾ Integrity Completed: 3 1 1 1 2 2 2  : S R3 G3 M1 M2 D1 N2 : AgIk

#1409: C D♯ E F G♭ A♭ B :0 3 4 5 6 8 11: 1 ♯2 3 4 ♭5 ♭6 7: ma11(♯5 ♯9 add ♯11)
namesBySequenceIndex[1408] = '''Rorian'''
namesBySequenceIndexNegative[1408] = ''''''
semitonesBySequenceIndex[1408] = [0, 3, 4, 5, 6, 8, 11]
poemBySequenceIndex[1408] = '''B G J K L N S'''
poemAuthors[1408] = ''''''
#1409: ䷘䷕ Integrity Adorning: 3 1 1 1 2 3 1  : S R3 G3 M1 M2 D1 N3 : AgIl

#1410: C D♯ E F G♭ A B♭ :0 3 4 5 6 9 10: 1 ♯2 3 4 ♭5 6 ♭7: 13(♭5 ♯9)
namesBySequenceIndex[1409] = '''Ionothian'''
namesBySequenceIndexNegative[1409] = ''''''
semitonesBySequenceIndex[1409] = [0, 3, 4, 5, 6, 9, 10]
poemBySequenceIndex[1409] = '''B G J K L P R'''
poemAuthors[1409] = ''''''
#1410: ䷘䷐ Integrity Following: 3 1 1 1 3 1 2  : S R3 G3 M1 M2 D2 N2 : AgAd

#1411: C D♯ E F G♭ A B :0 3 4 5 6 9 11: 1 ♯2 3 4 ♭5 6 7: ma13(♭5 ♯9)
namesBySequenceIndex[1410] = '''Aerycrian'''
namesBySequenceIndexNegative[1410] = ''''''
semitonesBySequenceIndex[1410] = [0, 3, 4, 5, 6, 9, 11]
poemBySequenceIndex[1410] = '''B G J K L P S'''
poemAuthors[1410] = ''''''
#1411: ䷘䷔ Integrity Biting: 3 1 1 1 3 2 1  : S R3 G3 M1 M2 D2 N3 : AgAf

#1412: C D♯ E F G♭ A♯ B :0 3 4 5 6 10 11: 1 ♯2 3 4 ♭5 ♯6 7: ma11(♭5 ♯9 add ♭7)
namesBySequenceIndex[1411] = '''Zathian'''
namesBySequenceIndexNegative[1411] = ''''''
semitonesBySequenceIndex[1411] = [0, 3, 4, 5, 6, 10, 11]
poemBySequenceIndex[1411] = '''B G J K L R S'''
poemAuthors[1411] = ''''''
#1412: ䷘䷩ Integrity Augmenting: 3 1 1 1 4 1 1  : S R3 G3 M1 M2 D3 N3 : AgAm

#1413: C D♯ E F G A♭ A :0 3 4 5 7 8 9: 1 ♯2 3 4 5 ♭6 6: 6(add ♭6 ♯9 11)
namesBySequenceIndex[1412] = '''Mela Yagapriya,Raga Kalahamsa,Stoptian'''
namesBySequenceIndexNegative[1412] = ''''''
semitonesBySequenceIndex[1412] = [0, 3, 4, 5, 7, 8, 9]
poemBySequenceIndex[1412] = '''B G J K M N P'''
poemAuthors[1412] = ''''''
#1413: ䷘䷟ Integrity Persevering: 3 1 1 2 1 1 3  : S R3 G3 M1 P D1 N1 : Agob

#1414: C D♯ E F G A♭ B♭ :0 3 4 5 7 8 10: 1 ♯2 3 4 5 ♭6 ♭7: 11(♯9 add ♭13)
namesBySequenceIndex[1413] = '''Raga Cudamani,Mela Ragavardhani,Mela Ragavardhini,Aerathian'''
namesBySequenceIndexNegative[1413] = ''''''
semitonesBySequenceIndex[1413] = [0, 3, 4, 5, 7, 8, 10]
poemBySequenceIndex[1413] = '''B G J K M N R'''
poemAuthors[1413] = ''''''
#1414: ䷘䷯ Integrity Source: 3 1 1 2 1 2 2  : S R3 G3 M1 P D1 N2 : Agok

#1415: C D♯ E F G A♭ B :0 3 4 5 7 8 11: 1 ♯2 3 4 5 ♭6 7: ma11(♯9 add ♭13)
namesBySequenceIndex[1414] = '''Sengiach,Sengah (Greece),Mela Gangeyabhusani,Mela Gangeyabhushani,Syrian'''
namesBySequenceIndexNegative[1414] = ''''''
semitonesBySequenceIndex[1414] = [0, 3, 4, 5, 7, 8, 11]
poemBySequenceIndex[1414] = '''B G J K M N S'''
poemAuthors[1414] = ''''''
#1415: ䷘䷑ Integrity Correcting: 3 1 1 2 1 3 1  : S R3 G3 M1 P D1 N3 : Agol

#1416: C D♯ E F G A B♭ :0 3 4 5 7 9 10: 1 ♯2 3 4 5 6 ♭7: 13(♯9)
namesBySequenceIndex[1415] = '''Rock ‘n Roll,Mela Vagadhisvari,Mela Vagadhibhusani,Raga Bhogachayanata,Raga Ganavaridhi,Raga Chayanata,Raga Nandkauns,Ionycrian'''
namesBySequenceIndexNegative[1415] = ''''''
semitonesBySequenceIndex[1415] = [0, 3, 4, 5, 7, 9, 10]
poemBySequenceIndex[1415] = '''B G J K M P R'''
poemAuthors[1415] = ''''''
#1416: ䷘䷮ Integrity Confining: 3 1 1 2 2 1 2  : S R3 G3 M1 P D2 N2 : Aged

#1417: C D♯ E F G A B :0 3 4 5 7 9 11: 1 ♯2 3 4 5 6 7: ma13(♯9)
namesBySequenceIndex[1416] = '''Mela Sulini,Houzam,Huzam,Ionian ♯2,Raga Sailadesakshi,Raga Trishuli,Dolian'''
namesBySequenceIndexNegative[1416] = ''''''
semitonesBySequenceIndex[1416] = [0, 3, 4, 5, 7, 9, 11]
poemBySequenceIndex[1416] = '''B G J K M P S'''
poemAuthors[1416] = ''''''
#1417: ䷘䷿ Integrity Unfinished: 3 1 1 2 2 2 1  : S R3 G3 M1 P D2 N3 : Agef

#1418: C D♯ E F G A♯ B :0 3 4 5 7 10 11: 1 ♯2 3 4 5 ♯6 7: ma11(♯9 add ♭7)
namesBySequenceIndex[1417] = '''Mela Chalanata,Raga None,Raga Nata/Nattai (ascending),Raga Madhuranjani,Ionacrian'''
namesBySequenceIndexNegative[1417] = ''''''
semitonesBySequenceIndex[1417] = [0, 3, 4, 5, 7, 10, 11]
poemBySequenceIndex[1417] = '''B G J K M R S'''
poemAuthors[1417] = ''''''
#1418: ䷘䷺ Integrity Dispersing: 3 1 1 2 3 1 1  : S R3 G3 M1 P D3 N3 : Agem

#1419: C D♯ E F G♯ A B♭ :0 3 4 5 8 9 10: 1 ♯2 3 4 ♯5 6 ♭7: 13(♯5 ♯9)
namesBySequenceIndex[1418] = '''Epogian'''
namesBySequenceIndexNegative[1418] = ''''''
semitonesBySequenceIndex[1418] = [0, 3, 4, 5, 8, 9, 10]
poemBySequenceIndex[1418] = '''B G J K N P R'''
poemAuthors[1418] = ''''''
#1419: ䷘䷞ Integrity Conjoining: 3 1 1 3 1 1 2  : S R3 G3 M1 D1 D2 N2 : Agid

#1420: C D♯ E F G♯ A B :0 3 4 5 8 9 11: 1 ♯2 3 4 ♯5 6 7: ma13(♯5 ♯9)
namesBySequenceIndex[1419] = '''Docrian,Ionian ♯2 ♯5'''
namesBySequenceIndexNegative[1419] = ''''''
semitonesBySequenceIndex[1419] = [0, 3, 4, 5, 8, 9, 11]
poemBySequenceIndex[1419] = '''B G J K N P S'''
poemAuthors[1419] = ''''''
#1420: ䷘䷷ Integrity Wanderer: 3 1 1 3 1 2 1  : S R3 G3 M1 D1 D2 N3 : Agif

#1421: C D♯ E F G♯ A♯ B :0 3 4 5 8 10 11: 1 ♯2 3 4 ♯5 ♯6 7: ma11(♯5 ♯9 add ♭7)
namesBySequenceIndex[1420] = '''Rothian,Chromatic Phrygian'''
namesBySequenceIndexNegative[1420] = ''''''
semitonesBySequenceIndex[1420] = [0, 3, 4, 5, 8, 10, 11]
poemBySequenceIndex[1420] = '''B G J K N R S'''
poemAuthors[1420] = ''''''
#1421: ䷘䷴ Integrity Infiltrating: 3 1 1 3 2 1 1  : S R3 G3 M1 D1 N2 N3 : Agim

#1422: C D♯ E F A B♭ B :0 3 4 5 9 10 11: 1 ♯2 3 4 6 ♭7 7: ma13(♯9 no 5 add ♭7)
namesBySequenceIndex[1421] = '''Thocrian'''
namesBySequenceIndexNegative[1421] = ''''''
semitonesBySequenceIndex[1421] = [0, 3, 4, 5, 9, 10, 11]
poemBySequenceIndex[1421] = '''B G J K P R S'''
poemAuthors[1421] = ''''''
#1422: ䷘䷋ Integrity Obstruction: 3 1 1 4 1 1 1  : S R3 G3 M1 D2 N2 N3 : Agag

#1423: C D♯ E F♯ G A♭ A :0 3 4 6 7 8 9: 1 ♯2 3 ♯4 5 ♭6 6: 6(add ♭6 ♯9 ♯11)
namesBySequenceIndex[1422] = '''Mela Sucaritra,Raga Santanamanjari,Raptian'''
namesBySequenceIndexNegative[1422] = ''''''
semitonesBySequenceIndex[1422] = [0, 3, 4, 6, 7, 8, 9]
poemBySequenceIndex[1422] = '''B G J L M N P'''
poemAuthors[1422] = ''''''
#1423: ䷐䷡ Following Might: 3 1 2 1 1 1 3  : S R3 G3 M2 P D1 N1 : AdOb

#1424: C D♯ E F♯ G A♭ B♭ :0 3 4 6 7 8 10: 1 ♯2 3 ♯4 5 ♭6 ♭7: 7(add ♯9 ♯11 ♭13)
namesBySequenceIndex[1423] = '''Mela Jyotisvarupini,Raga Jotismatti,Ionanian'''
namesBySequenceIndexNegative[1423] = ''''''
semitonesBySequenceIndex[1423] = [0, 3, 4, 6, 7, 8, 10]
poemBySequenceIndex[1423] = '''B G J L M N R'''
poemAuthors[1423] = ''''''
#1424: ䷐䷄ Following Biding: 3 1 2 1 1 2 2  : S R3 G3 M2 P D1 N2 : AdOk

#1425: C D♯ E F♯ G A♭ B :0 3 4 6 7 8 11: 1 ♯2 3 ♯4 5 ♭6 7: ma7(add ♯9 ♯11 ♭13)
namesBySequenceIndex[1424] = '''Mela Dhatuvardhani,Raga Devarashtra,Raga Dhauta Pancama,Barian'''
namesBySequenceIndexNegative[1424] = ''''''
semitonesBySequenceIndex[1424] = [0, 3, 4, 6, 7, 8, 11]
poemBySequenceIndex[1424] = '''B G J L M N S'''
poemAuthors[1424] = ''''''
#1425: ䷐䷙ Following Cultivating: 3 1 2 1 1 3 1  : S R3 G3 M2 P D1 N3 : AdOl

#1426: C D♯ E F♯ G A B♭ :0 3 4 6 7 9 10: 1 ♯2 3 ♯4 5 6 ♭7: 13(♯9 ♯11)
namesBySequenceIndex[1425] = '''Mela Nasikabhusani,Raga Nasamani,Hungarian Major,Mycrian'''
namesBySequenceIndexNegative[1425] = ''''''
semitonesBySequenceIndex[1425] = [0, 3, 4, 6, 7, 9, 10]
poemBySequenceIndex[1425] = '''B G J L M P R'''
poemAuthors[1425] = ''''''
#1426: ䷐䷹ Following Joyous: 3 1 2 1 2 1 2  : S R3 G3 M2 P D2 N2 : AdEd

#1427: C D♯ E F♯ G A B :0 3 4 6 7 9 11: 1 ♯2 3 ♯4 5 6 7: ma13(♯9 ♯11)
namesBySequenceIndex[1426] = '''Mela Kosalam,Lydian ♯2,Raga Kuksumakaram (or Kusumakaram),Bycrian'''
namesBySequenceIndexNegative[1426] = ''''''
semitonesBySequenceIndex[1426] = [0, 3, 4, 6, 7, 9, 11]
poemBySequenceIndex[1426] = '''B G J L M P S'''
poemAuthors[1426] = ''''''
#1427: ䷐䷥ Following Polarising: 3 1 2 1 2 2 1  : S R3 G3 M2 P D2 N3 : AdEf

#1428: C D♯ E F♯ G A♯ B :0 3 4 6 7 10 11: 1 ♯2 3 ♯4 5 ♯6 7: ma7(add ♭7 ♯9 ♯11)
namesBySequenceIndex[1427] = '''Mela Rasikapriya,Raga Rasamanjari 1,Raga Hamsagiri,Loptian,Lydian ♯2 ♯6'''
namesBySequenceIndexNegative[1427] = ''''''
semitonesBySequenceIndex[1427] = [0, 3, 4, 6, 7, 10, 11]
poemBySequenceIndex[1427] = '''B G J L M R S'''
poemAuthors[1427] = ''''''
#1428: ䷐䷼ Following Truth: 3 1 2 1 3 1 1  : S R3 G3 M2 P D3 N3 : AdEm

#1429: C D♯ E F♯ G♯ A B♭ :0 3 4 6 8 9 10: 1 ♯2 3 ♯4 ♯5 6 ♭7: 13(♯5 ♯9 ♯11)
namesBySequenceIndex[1428] = '''Korian'''
namesBySequenceIndexNegative[1428] = ''''''
semitonesBySequenceIndex[1428] = [0, 3, 4, 6, 8, 9, 10]
poemBySequenceIndex[1428] = '''B G J L N P R'''
poemAuthors[1428] = ''''''
#1429: ䷐䷰ Following Revolution: 3 1 2 2 1 1 2  : S R3 G3 M2 D1 D2 N2 : AdId

#1430: C D♯ E F♯ G♯ A B :0 3 4 6 8 9 11: 1 ♯2 3 ♯4 ♯5 6 7: ma13(♯5 ♯9 ♯11)
namesBySequenceIndex[1429] = '''Palian,Aeolian ♭1,Lydian Augmented ♯2'''
namesBySequenceIndexNegative[1429] = ''''''
semitonesBySequenceIndex[1429] = [0, 3, 4, 6, 8, 9, 11]
poemBySequenceIndex[1429] = '''B G J L N P S'''
poemAuthors[1429] = ''''''
#1430: ䷐䷝ Following Distanced: 3 1 2 2 1 2 1  : S R3 G3 M2 D1 D2 N3 : AdIf

#1431: C D♯ E F♯ G♯ A♯ B :0 3 4 6 8 10 11: 1 ♯2 3 ♯4 ♯5 ♯6 7: ma7(♯5 add ♭7 ♯9 ♯11)
namesBySequenceIndex[1430] = '''Golian'''
namesBySequenceIndexNegative[1430] = ''''''
semitonesBySequenceIndex[1430] = [0, 3, 4, 6, 8, 10, 11]
poemBySequenceIndex[1430] = '''B G J L N R S'''
poemAuthors[1430] = ''''''
#1431: ䷐䷤ Following Family: 3 1 2 2 2 1 1  : S R3 G3 M2 D1 N2 N3 : AdIm

#1432: C D♯ E G♭ A B♭ B :0 3 4 6 9 10 11: 1 ♯2 3 ♭5 6 ♭7 7: ma7(♭5 add ♭7 ♯9 13)
namesBySequenceIndex[1431] = '''Ranian'''
namesBySequenceIndexNegative[1431] = ''''''
semitonesBySequenceIndex[1431] = [0, 3, 4, 6, 9, 10, 11]
poemBySequenceIndex[1431] = '''B G J L P R S'''
poemAuthors[1431] = ''''''
#1432: ䷐䷘ Following Integrity: 3 1 2 3 1 1 1  : S R3 G3 M2 D2 N2 N3 : AdAg

#1433: C D♯ E G A♭ A B♭ :0 3 4 7 8 9 10: 1 ♯2 3 5 ♭6 6 ♭7: 7(add ♯9 ♭13 13)
namesBySequenceIndex[1432] = '''Zarian'''
namesBySequenceIndexNegative[1432] = ''''''
semitonesBySequenceIndex[1432] = [0, 3, 4, 7, 8, 9, 10]
poemBySequenceIndex[1432] = '''B G J M N P R'''
poemAuthors[1432] = ''''''
#1433: ䷐䷛ Following Excess: 3 1 3 1 1 1 2  : S R3 G3 P D1 D2 N2 : Adod

#1434: C D♯ E G A♭ A B :0 3 4 7 8 9 11: 1 ♯2 3 5 ♭6 6 7: ma7(add ♯9 ♭13 13)
namesBySequenceIndex[1433] = '''Tholian'''
namesBySequenceIndexNegative[1433] = ''''''
semitonesBySequenceIndex[1433] = [0, 3, 4, 7, 8, 9, 11]
poemBySequenceIndex[1433] = '''B G J M N P S'''
poemAuthors[1433] = ''''''
#1434: ䷐䷱ Following Holding: 3 1 3 1 1 2 1  : S R3 G3 P D1 D2 N3 : Adof

#1435: C D♯ E G A♭ B♭ B :0 3 4 7 8 10 11: 1 ♯2 3 5 ♭6 ♭7 7: ma7(add ♭7 ♯9 ♭13)
namesBySequenceIndex[1434] = '''Katathian'''
namesBySequenceIndexNegative[1434] = ''''''
semitonesBySequenceIndex[1434] = [0, 3, 4, 7, 8, 10, 11]
poemBySequenceIndex[1434] = '''B G J M N R S'''
poemAuthors[1434] = ''''''
#1435: ䷐䷸ Following Osmosis: 3 1 3 1 2 1 1  : S R3 G3 P D1 N2 N3 : Adom

#1436: C D♯ E G A B♭ B :0 3 4 7 9 10 11: 1 ♯2 3 5 6 ♭7 7: ma7(add ♭7 ♯9 13)
namesBySequenceIndex[1435] = '''Phrocrian'''
namesBySequenceIndexNegative[1435] = ''''''
semitonesBySequenceIndex[1435] = [0, 3, 4, 7, 9, 10, 11]
poemBySequenceIndex[1435] = '''B G J M P R S'''
poemAuthors[1435] = ''''''
#1436: ䷐䷅ Following Arguing: 3 1 3 2 1 1 1  : S R3 G3 P D2 N2 N3 : Adeg

#1437: C D♯ E G♯ A B♭ B :0 3 4 8 9 10 11: 1 ♯2 3 ♯5 6 ♭7 7: ma7(♯5 add ♭7 ♯9 13)
namesBySequenceIndex[1436] = '''Starian'''
namesBySequenceIndexNegative[1436] = ''''''
semitonesBySequenceIndex[1436] = [0, 3, 4, 8, 9, 10, 11]
poemBySequenceIndex[1436] = '''B G J N P R S'''
poemAuthors[1436] = ''''''
#1437: ䷐䷠ Following Retiring: 3 1 4 1 1 1 1  : S R3 G3 D1 D2 N2 N3 : Adig

#1438: C E♭ F F♯ G A♭ A :0 3 5 6 7 8 9: 1 ♭3 4 ♯4 5 ♭6 6: mi6(add ♭6 11 ♯11)
namesBySequenceIndex[1437] = '''Badian'''
namesBySequenceIndexNegative[1437] = ''''''
semitonesBySequenceIndex[1437] = [0, 3, 5, 6, 7, 8, 9]
poemBySequenceIndex[1437] = '''B G K L M N P'''
poemAuthors[1437] = ''''''
#1438: ䷔䷡ Biting Might: 3 2 1 1 1 1 3  : S G2 M1 M2 P D1 N1 : AfOb

#1439: C E♭ F F♯ G A♭ B♭ :0 3 5 6 7 8 10: 1 ♭3 4 ♯4 5 ♭6 ♭7: mi11(♯9 add ♯11 ♭13)
namesBySequenceIndex[1438] = '''Stathian'''
namesBySequenceIndexNegative[1438] = ''''''
semitonesBySequenceIndex[1438] = [0, 3, 5, 6, 7, 8, 10]
poemBySequenceIndex[1438] = '''B G K L M N R'''
poemAuthors[1438] = ''''''
#1439: ䷔䷄ Biting Biding: 3 2 1 1 1 2 2  : S G2 M1 M2 P D1 N2 : AfOk

#1440: C E♭ F F♯ G A♭ B :0 3 5 6 7 8 11: 1 ♭3 4 ♯4 5 ♭6 7: mima11(♯9 add ♯11 ♭13)
namesBySequenceIndex[1439] = '''Laptian'''
namesBySequenceIndexNegative[1439] = ''''''
semitonesBySequenceIndex[1439] = [0, 3, 5, 6, 7, 8, 11]
poemBySequenceIndex[1439] = '''B G K L M N S'''
poemAuthors[1439] = ''''''
#1440: ䷔䷙ Biting Cultivating: 3 2 1 1 1 3 1  : S G2 M1 M2 P D1 N3 : AfOl

#1441: C E♭ F F♯ G A B♭ :0 3 5 6 7 9 10: 1 ♭3 4 ♯4 5 6 ♭7: mi13(♯9 add ♯11)
namesBySequenceIndex[1440] = '''Doriblues,Blues Heptatonic 2,Rythian'''
namesBySequenceIndexNegative[1440] = ''''''
semitonesBySequenceIndex[1440] = [0, 3, 5, 6, 7, 9, 10]
poemBySequenceIndex[1440] = '''B G K L M P R'''
poemAuthors[1440] = ''''''
#1441: ䷔䷹ Biting Joyous: 3 2 1 1 2 1 2  : S G2 M1 M2 P D2 N2 : AfEd

#1442: C E♭ F F♯ G A B :0 3 5 6 7 9 11: 1 ♭3 4 ♯4 5 6 7: mima13(♯9 add ♯11)
namesBySequenceIndex[1441] = '''Eporian'''
namesBySequenceIndexNegative[1441] = ''''''
semitonesBySequenceIndex[1441] = [0, 3, 5, 6, 7, 9, 11]
poemBySequenceIndex[1441] = '''B G K L M P S'''
poemAuthors[1441] = ''''''
#1442: ䷔䷥ Biting Polarising: 3 2 1 1 2 2 1  : S G2 M1 M2 P D2 N3 : AfEf

#1443: C E♭ F F♯ G A♯ B :0 3 5 6 7 10 11: 1 ♭3 4 ♯4 5 ♯6 7: mima11(♯9 add ♭7 ♯11)
namesBySequenceIndex[1442] = '''Blues Leading Tone,Epathian'''
namesBySequenceIndexNegative[1442] = ''''''
semitonesBySequenceIndex[1442] = [0, 3, 5, 6, 7, 10, 11]
poemBySequenceIndex[1442] = '''B G K L M R S'''
poemAuthors[1442] = ''''''
#1443: ䷔䷼ Biting Truth: 3 2 1 1 3 1 1  : S G2 M1 M2 P D3 N3 : AfEm

#1444: C E♭ F G♭ A♭ A B♭ :0 3 5 6 8 9 10: 1 ♭3 4 ♭5 ♭6 6 ♭7: ⌀13(♯9 add ♭13)
namesBySequenceIndex[1443] = '''Ionopian'''
namesBySequenceIndexNegative[1443] = ''''''
semitonesBySequenceIndex[1443] = [0, 3, 5, 6, 8, 9, 10]
poemBySequenceIndex[1443] = '''B G K L N P R'''
poemAuthors[1443] = ''''''
#1444: ䷔䷰ Biting Revolution: 3 2 1 2 1 1 2  : S G2 M1 M2 D1 D2 N2 : AfId

#1445: C E♭ F G♭ A♭ A B :0 3 5 6 8 9 11: 1 ♭3 4 ♭5 ♭6 6 7: °ma13(♯9 add ♭13)
namesBySequenceIndex[1444] = '''Pogian'''
namesBySequenceIndexNegative[1444] = ''''''
semitonesBySequenceIndex[1444] = [0, 3, 5, 6, 8, 9, 11]
poemBySequenceIndex[1444] = '''B G K L N P S'''
poemAuthors[1444] = ''''''
#1445: ䷔䷝ Biting Distanced: 3 2 1 2 1 2 1  : S G2 M1 M2 D1 D2 N3 : AfIf

#1446: C E♭ F G♭ A♭ B♭ B :0 3 5 6 8 10 11: 1 ♭3 4 ♭5 ♭6 ♭7 7: °ma11(♯9 add ♭7 ♭13)
namesBySequenceIndex[1445] = '''Thonian'''
namesBySequenceIndexNegative[1445] = ''''''
semitonesBySequenceIndex[1445] = [0, 3, 5, 6, 8, 10, 11]
poemBySequenceIndex[1445] = '''B G K L N R S'''
poemAuthors[1445] = ''''''
#1446: ䷔䷤ Biting Family: 3 2 1 2 2 1 1  : S G2 M1 M2 D1 N2 N3 : AfIm

#1447: C E♭ F G♭ A B♭ B :0 3 5 6 9 10 11: 1 ♭3 4 ♭5 6 ♭7 7: °ma13(♯9 add ♭7)
namesBySequenceIndex[1446] = '''Katocrian'''
namesBySequenceIndexNegative[1446] = ''''''
semitonesBySequenceIndex[1446] = [0, 3, 5, 6, 9, 10, 11]
poemBySequenceIndex[1446] = '''B G K L P R S'''
poemAuthors[1446] = ''''''
#1447: ䷔䷘ Biting Integrity: 3 2 1 3 1 1 1  : S G2 M1 M2 D2 N2 N3 : AfAg

#1448: C E♭ F G A♭ A B♭ :0 3 5 7 8 9 10: 1 ♭3 4 5 ♭6 6 ♭7: mi13(♯9 add ♭13)
namesBySequenceIndex[1447] = '''Soptian'''
namesBySequenceIndexNegative[1447] = ''''''
semitonesBySequenceIndex[1447] = [0, 3, 5, 7, 8, 9, 10]
poemBySequenceIndex[1447] = '''B G K M N P R'''
poemAuthors[1447] = ''''''
#1448: ䷔䷛ Biting Excess: 3 2 2 1 1 1 2  : S G2 M1 P D1 D2 N2 : Afod

#1449: C E♭ F G A♭ A B :0 3 5 7 8 9 11: 1 ♭3 4 5 ♭6 6 7: mima13(♯9 add ♭13)
namesBySequenceIndex[1448] = '''Epacrian'''
namesBySequenceIndexNegative[1448] = ''''''
semitonesBySequenceIndex[1448] = [0, 3, 5, 7, 8, 9, 11]
poemBySequenceIndex[1448] = '''B G K M N P S'''
poemAuthors[1448] = ''''''
#1449: ䷔䷱ Biting Holding: 3 2 2 1 1 2 1  : S G2 M1 P D1 D2 N3 : Afof

#1450: C E♭ F G A♭ B♭ B :0 3 5 7 8 10 11: 1 ♭3 4 5 ♭6 ♭7 7: mima11(♯9 add ♭7 ♭13)
namesBySequenceIndex[1449] = '''Phrolian'''
namesBySequenceIndexNegative[1449] = ''''''
semitonesBySequenceIndex[1449] = [0, 3, 5, 7, 8, 10, 11]
poemBySequenceIndex[1449] = '''B G K M N R S'''
poemAuthors[1449] = ''''''
#1450: ䷔䷸ Biting Osmosis: 3 2 2 1 2 1 1  : S G2 M1 P D1 N2 N3 : Afom

#1451: C E♭ F G A B♭ B :0 3 5 7 9 10 11: 1 ♭3 4 5 6 ♭7 7: mima13(♯9 add ♭7)
namesBySequenceIndex[1450] = '''Raga Bahar (ascending),Phraptian'''
namesBySequenceIndexNegative[1450] = ''''''
semitonesBySequenceIndex[1450] = [0, 3, 5, 7, 9, 10, 11]
poemBySequenceIndex[1450] = '''B G K M P R S'''
poemAuthors[1450] = ''''''
#1451: ䷔䷅ Biting Arguing: 3 2 2 2 1 1 1  : S G2 M1 P D2 N2 N3 : Afeg

#1452: C E♭ F G♯ A B♭ B :0 3 5 8 9 10 11: 1 ♭3 4 ♯5 6 ♭7 7: mima13(♯5 ♯9 add ♭7)
namesBySequenceIndex[1451] = '''Morian'''
namesBySequenceIndexNegative[1451] = ''''''
semitonesBySequenceIndex[1451] = [0, 3, 5, 8, 9, 10, 11]
poemBySequenceIndex[1451] = '''B G K N P R S'''
poemAuthors[1451] = ''''''
#1452: ䷔䷠ Biting Retiring: 3 2 3 1 1 1 1  : S G2 M1 D1 D2 N2 N3 : Afig

#1453: C E♭ F♯ G A♭ A B♭ :0 3 6 7 8 9 10: 1 ♭3 ♯4 5 ♭6 6 ♭7: mi13(♯9 ♯11 add ♭13)
namesBySequenceIndex[1452] = '''Katoptian'''
namesBySequenceIndexNegative[1452] = ''''''
semitonesBySequenceIndex[1452] = [0, 3, 6, 7, 8, 9, 10]
poemBySequenceIndex[1452] = '''B G L M N P R'''
poemAuthors[1452] = ''''''
#1453: ䷲䷪ Thunder Decision: 3 3 1 1 1 1 2  : S G2 M2 P D1 D2 N2 : AbOd

#1454: C E♭ F♯ G A♭ A B :0 3 6 7 8 9 11: 1 ♭3 ♯4 5 ♭6 6 7: mima13(♯9 ♯11 add ♭13)
namesBySequenceIndex[1453] = '''Gacrian'''
namesBySequenceIndexNegative[1453] = ''''''
semitonesBySequenceIndex[1453] = [0, 3, 6, 7, 8, 9, 11]
poemBySequenceIndex[1453] = '''B G L M N P S'''
poemAuthors[1453] = ''''''
#1454: ䷲䷍ Thunder Resevoir: 3 3 1 1 1 2 1  : S G2 M2 P D1 D2 N3 : AbOf

#1455: C E♭ F♯ G A♭ B♭ B :0 3 6 7 8 10 11: 1 ♭3 ♯4 5 ♭6 ♭7 7: mima7(add ♭7 ♯11 ♭13)
namesBySequenceIndex[1454] = '''Stalian'''
namesBySequenceIndexNegative[1454] = ''''''
semitonesBySequenceIndex[1454] = [0, 3, 6, 7, 8, 10, 11]
poemBySequenceIndex[1454] = '''B G L M N R S'''
poemAuthors[1454] = ''''''
#1455: ䷲䷈ Thunder Homegrown: 3 3 1 1 2 1 1  : S G2 M2 P D1 N2 N3 : AbOm

#1456: C E♭ F♯ G A B♭ B :0 3 6 7 9 10 11: 1 ♭3 ♯4 5 6 ♭7 7: mima13(♯9 ♯11 add ♭7)
namesBySequenceIndex[1455] = '''Epagian'''
namesBySequenceIndexNegative[1455] = ''''''
semitonesBySequenceIndex[1455] = [0, 3, 6, 7, 9, 10, 11]
poemBySequenceIndex[1455] = '''B G L M P R S'''
poemAuthors[1455] = ''''''
#1456: ䷲䷉ Thunder Treading: 3 3 1 2 1 1 1  : S G2 M2 P D2 N2 N3 : AbEg

#1457: C E♭ F♯ G♯ A B♭ B :0 3 6 8 9 10 11: 1 ♭3 ♯4 ♯5 6 ♭7 7: °ma7(add ♭7 ♭13 13)
namesBySequenceIndex[1456] = '''Bonian'''
namesBySequenceIndexNegative[1456] = ''''''
semitonesBySequenceIndex[1456] = [0, 3, 6, 8, 9, 10, 11]
poemBySequenceIndex[1456] = '''B G L N P R S'''
poemAuthors[1456] = ''''''
#1457: ䷲䷌ Thunder Fellowship: 3 3 2 1 1 1 1  : S G2 M2 D1 D2 N2 N3 : AbIg

#1458: C E♭ G A♭ A B♭ B :0 3 7 8 9 10 11: 1 ♭3 5 ♭6 6 ♭7 7: mima7(add ♭7 ♭13 13)
namesBySequenceIndex[1457] = '''Kythian'''
namesBySequenceIndexNegative[1457] = ''''''
semitonesBySequenceIndex[1457] = [0, 3, 7, 8, 9, 10, 11]
poemBySequenceIndex[1457] = '''B G M N P R S'''
poemAuthors[1457] = ''''''
#1458: ䷲䷫ Thunder Coupling: 3 4 1 1 1 1 1  : S G2 P D1 D2 N2 N3 : Abog

#1459: C E F F♯ G A♭ A :0 4 5 6 7 8 9: 1 3 4 ♯4 5 ♭6 6: 6(add ♭6 11 ♯11)
namesBySequenceIndex[1458] = '''Katyptian'''
namesBySequenceIndexNegative[1458] = ''''''
semitonesBySequenceIndex[1458] = [0, 4, 5, 6, 7, 8, 9]
poemBySequenceIndex[1458] = '''B J K L M N P'''
poemAuthors[1458] = ''''''
#1459: ䷩䷡ Augmenting Might: 4 1 1 1 1 1 3  : S G3 M1 M2 P D1 N1 : AmOb

#1460: C E F F♯ G A♭ B♭ :0 4 5 6 7 8 10: 1 3 4 ♯4 5 ♭6 ♭7: 7(add 11 ♯11 ♭13)
namesBySequenceIndex[1459] = '''Stanian'''
namesBySequenceIndexNegative[1459] = ''''''
semitonesBySequenceIndex[1459] = [0, 4, 5, 6, 7, 8, 10]
poemBySequenceIndex[1459] = '''B J K L M N R'''
poemAuthors[1459] = ''''''
#1460: ䷩䷄ Augmenting Biding: 4 1 1 1 1 2 2  : S G3 M1 M2 P D1 N2 : AmOk

#1461: C E F F♯ G A♭ B :0 4 5 6 7 8 11: 1 3 4 ♯4 5 ♭6 7: ma7(add 11 ♯11 ♭13)
namesBySequenceIndex[1460] = '''Thycrian'''
namesBySequenceIndexNegative[1460] = ''''''
semitonesBySequenceIndex[1460] = [0, 4, 5, 6, 7, 8, 11]
poemBySequenceIndex[1460] = '''B J K L M N S'''
poemAuthors[1460] = ''''''
#1461: ䷩䷙ Augmenting Cultivating: 4 1 1 1 1 3 1  : S G3 M1 M2 P D1 N3 : AmOl

#1462: C E F F♯ G A B♭ :0 4 5 6 7 9 10: 1 3 4 ♯4 5 6 ♭7: 13(no 9 add ♯11)
namesBySequenceIndex[1461] = '''Saptian'''
namesBySequenceIndexNegative[1461] = ''''''
semitonesBySequenceIndex[1461] = [0, 4, 5, 6, 7, 9, 10]
poemBySequenceIndex[1461] = '''B J K L M P R'''
poemAuthors[1461] = ''''''
#1462: ䷩䷹ Augmenting Joyous: 4 1 1 1 2 1 2  : S G3 M1 M2 P D2 N2 : AmEd

#1463: C E F F♯ G A B :0 4 5 6 7 9 11: 1 3 4 ♯4 5 6 7: ma13(no 9 add ♯11)
namesBySequenceIndex[1462] = '''Zogian'''
namesBySequenceIndexNegative[1462] = ''''''
semitonesBySequenceIndex[1462] = [0, 4, 5, 6, 7, 9, 11]
poemBySequenceIndex[1462] = '''B J K L M P S'''
poemAuthors[1462] = ''''''
#1463: ䷩䷥ Augmenting Polarising: 4 1 1 1 2 2 1  : S G3 M1 M2 P D2 N3 : AmEf

#1464: C E F F♯ G A♯ B :0 4 5 6 7 10 11: 1 3 4 ♯4 5 ♯6 7: ma7(add ♭7 11 ♯11)
namesBySequenceIndex[1463] = '''Aeolacrian'''
namesBySequenceIndexNegative[1463] = ''''''
semitonesBySequenceIndex[1463] = [0, 4, 5, 6, 7, 10, 11]
poemBySequenceIndex[1463] = '''B J K L M R S'''
poemAuthors[1463] = ''''''
#1464: ䷩䷼ Augmenting Truth: 4 1 1 1 3 1 1  : S G3 M1 M2 P D3 N3 : AmEm

#1465: C E F G♭ A♭ A B♭ :0 4 5 6 8 9 10: 1 3 4 ♭5 ♭6 6 ♭7: 13(♯5 no 9 add ♯11)
namesBySequenceIndex[1464] = '''Katacrian'''
namesBySequenceIndexNegative[1464] = ''''''
semitonesBySequenceIndex[1464] = [0, 4, 5, 6, 8, 9, 10]
poemBySequenceIndex[1464] = '''B J K L N P R'''
poemAuthors[1464] = ''''''
#1465: ䷩䷰ Augmenting Revolution: 4 1 1 2 1 1 2  : S G3 M1 M2 D1 D2 N2 : AmId

#1466: C E F G♭ A♭ A B :0 4 5 6 8 9 11: 1 3 4 ♭5 ♭6 6 7: ma13(♯5 no 9 add ♯11)
namesBySequenceIndex[1465] = '''Aeolathian'''
namesBySequenceIndexNegative[1465] = ''''''
semitonesBySequenceIndex[1465] = [0, 4, 5, 6, 8, 9, 11]
poemBySequenceIndex[1465] = '''B J K L N P S'''
poemAuthors[1465] = ''''''
#1466: ䷩䷝ Augmenting Distanced: 4 1 1 2 1 2 1  : S G3 M1 M2 D1 D2 N3 : AmIf

#1467: C E F G♭ A♭ B♭ B :0 4 5 6 8 10 11: 1 3 4 ♭5 ♭6 ♭7 7: ma7(♯5 add ♭7 11 ♯11)
namesBySequenceIndex[1466] = '''Thacrian'''
namesBySequenceIndexNegative[1466] = ''''''
semitonesBySequenceIndex[1466] = [0, 4, 5, 6, 8, 10, 11]
poemBySequenceIndex[1466] = '''B J K L N R S'''
poemAuthors[1466] = ''''''
#1467: ䷩䷤ Augmenting Family: 4 1 1 2 2 1 1  : S G3 M1 M2 D1 N2 N3 : AmIm

#1468: C E F G♭ A B♭ B :0 4 5 6 9 10 11: 1 3 4 ♭5 6 ♭7 7: ma13(♭5 no 9 add ♭7)
namesBySequenceIndex[1467] = '''Ionarian'''
namesBySequenceIndexNegative[1467] = ''''''
semitonesBySequenceIndex[1467] = [0, 4, 5, 6, 9, 10, 11]
poemBySequenceIndex[1467] = '''B J K L P R S'''
poemAuthors[1467] = ''''''
#1468: ䷩䷘ Augmenting Integrity: 4 1 1 3 1 1 1  : S G3 M1 M2 D2 N2 N3 : AmAg

#1469: C E F G A♭ A B♭ :0 4 5 7 8 9 10: 1 3 4 5 ♭6 6 ♭7: 13(no 9 add ♭13)
namesBySequenceIndex[1468] = '''Stylian'''
namesBySequenceIndexNegative[1468] = ''''''
semitonesBySequenceIndex[1468] = [0, 4, 5, 7, 8, 9, 10]
poemBySequenceIndex[1468] = '''B J K M N P R'''
poemAuthors[1468] = ''''''
#1469: ䷩䷛ Augmenting Excess: 4 1 2 1 1 1 2  : S G3 M1 P D1 D2 N2 : Amod

#1470: C E F G A♭ A B :0 4 5 7 8 9 11: 1 3 4 5 ♭6 6 7: ma13(no 9 add ♭13)
namesBySequenceIndex[1469] = '''Stythian'''
namesBySequenceIndexNegative[1469] = ''''''
semitonesBySequenceIndex[1469] = [0, 4, 5, 7, 8, 9, 11]
poemBySequenceIndex[1469] = '''B J K M N P S'''
poemAuthors[1469] = ''''''
#1470: ䷩䷱ Augmenting Holding: 4 1 2 1 1 2 1  : S G3 M1 P D1 D2 N3 : Amof

#1471: C E F G A♭ B♭ B :0 4 5 7 8 10 11: 1 3 4 5 ♭6 ♭7 7: ma7(add ♭7 11 ♭13)
namesBySequenceIndex[1470] = '''Stygian'''
namesBySequenceIndexNegative[1470] = ''''''
semitonesBySequenceIndex[1470] = [0, 4, 5, 7, 8, 10, 11]
poemBySequenceIndex[1470] = '''B J K M N R S'''
poemAuthors[1470] = ''''''
#1471: ䷩䷸ Augmenting Osmosis: 4 1 2 1 2 1 1  : S G3 M1 P D1 N2 N3 : Amom

#1472: C E F G A B♭ B :0 4 5 7 9 10 11: 1 3 4 5 6 ♭7 7: ma13(no 9 add ♭7)
namesBySequenceIndex[1471] = '''Raga Khamach (ascending),Raga Madhuri,Ionythian'''
namesBySequenceIndexNegative[1471] = ''''''
semitonesBySequenceIndex[1471] = [0, 4, 5, 7, 9, 10, 11]
poemBySequenceIndex[1471] = '''B J K M P R S'''
poemAuthors[1471] = ''''''
#1472: ䷩䷅ Augmenting Arguing: 4 1 2 2 1 1 1  : S G3 M1 P D2 N2 N3 : Ameg

#1473: C E F G♯ A B♭ B :0 4 5 8 9 10 11: 1 3 4 ♯5 6 ♭7 7: ma13(♯5 no 9 add ♭7)
namesBySequenceIndex[1472] = '''Parian'''
namesBySequenceIndexNegative[1472] = ''''''
semitonesBySequenceIndex[1472] = [0, 4, 5, 8, 9, 10, 11]
poemBySequenceIndex[1472] = '''B J K N P R S'''
poemAuthors[1472] = ''''''
#1473: ䷩䷠ Augmenting Retiring: 4 1 3 1 1 1 1  : S G3 M1 D1 D2 N2 N3 : Amig

#1474: C E F♯ G A♭ A B♭ :0 4 6 7 8 9 10: 1 3 ♯4 5 ♭6 6 ♭7: 7(add ♯11 ♭13 13)
namesBySequenceIndex[1473] = '''Gydian'''
namesBySequenceIndexNegative[1473] = ''''''
semitonesBySequenceIndex[1473] = [0, 4, 6, 7, 8, 9, 10]
poemBySequenceIndex[1473] = '''B J L M N P R'''
poemAuthors[1473] = ''''''
#1474: ䷂䷪ Sprouting Decision: 4 2 1 1 1 1 2  : S G3 M2 P D1 D2 N2 : AkOd

#1475: C E F♯ G A♭ A B :0 4 6 7 8 9 11: 1 3 ♯4 5 ♭6 6 7: ma7(add ♯11 ♭13 13)
namesBySequenceIndex[1474] = '''Epycrian'''
namesBySequenceIndexNegative[1474] = ''''''
semitonesBySequenceIndex[1474] = [0, 4, 6, 7, 8, 9, 11]
poemBySequenceIndex[1474] = '''B J L M N P S'''
poemAuthors[1474] = ''''''
#1475: ䷂䷍ Sprouting Resevoir: 4 2 1 1 1 2 1  : S G3 M2 P D1 D2 N3 : AkOf

#1476: C E F♯ G A♭ B♭ B :0 4 6 7 8 10 11: 1 3 ♯4 5 ♭6 ♭7 7: ma7(add ♭7 ♯11 ♭13)
namesBySequenceIndex[1475] = '''Katogian'''
namesBySequenceIndexNegative[1475] = ''''''
semitonesBySequenceIndex[1475] = [0, 4, 6, 7, 8, 10, 11]
poemBySequenceIndex[1475] = '''B J L M N R S'''
poemAuthors[1475] = ''''''
#1476: ䷂䷈ Sprouting Homegrown: 4 2 1 1 2 1 1  : S G3 M2 P D1 N2 N3 : AkOm

#1477: C E F♯ G A B♭ B :0 4 6 7 9 10 11: 1 3 ♯4 5 6 ♭7 7: ma7(add ♭7 ♯11 13)
namesBySequenceIndex[1476] = '''Aeopian'''
namesBySequenceIndexNegative[1476] = ''''''
semitonesBySequenceIndex[1476] = [0, 4, 6, 7, 9, 10, 11]
poemBySequenceIndex[1476] = '''B J L M P R S'''
poemAuthors[1476] = ''''''
#1477: ䷂䷉ Sprouting Treading: 4 2 1 2 1 1 1  : S G3 M2 P D2 N2 N3 : AkEg

#1478: C E F♯ G♯ A B♭ B :0 4 6 8 9 10 11: 1 3 ♯4 ♯5 6 ♭7 7: ma7(♯5 add ♭7 ♯11 13)
namesBySequenceIndex[1477] = '''Pythian'''
namesBySequenceIndexNegative[1477] = ''''''
semitonesBySequenceIndex[1477] = [0, 4, 6, 8, 9, 10, 11]
poemBySequenceIndex[1477] = '''B J L N P R S'''
poemAuthors[1477] = ''''''
#1478: ䷂䷌ Sprouting Fellowship: 4 2 2 1 1 1 1  : S G3 M2 D1 D2 N2 N3 : AkIg

#1479: C E G A♭ A B♭ B :0 4 7 8 9 10 11: 1 3 5 ♭6 6 ♭7 7: ma7(add ♭7 ♭13 13)
namesBySequenceIndex[1478] = '''Thadian'''
namesBySequenceIndexNegative[1478] = ''''''
semitonesBySequenceIndex[1478] = [0, 4, 7, 8, 9, 10, 11]
poemBySequenceIndex[1478] = '''B J M N P R S'''
poemAuthors[1478] = ''''''
#1479: ䷂䷫ Sprouting Coupling: 4 3 1 1 1 1 1  : S G3 P D1 D2 N2 N3 : Akog

#1480: C F F♯ G A♭ A B♭ :0 5 6 7 8 9 10: 1 4 ♯4 5 ♭6 6 ♭7: 13sus♯4(no 9 add ♭13)
namesBySequenceIndex[1479] = ''''''
namesBySequenceIndexNegative[1479] = ''''''
semitonesBySequenceIndex[1479] = [0, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1479] = '''B K L M N P R'''
poemAuthors[1479] = ''''''
#1480: ䷚䷪ Nourishment Decision: 5 1 1 1 1 1 2  : S M1 M2 P D1 D2 N2 : AlOd

#1481: C F F♯ G A♭ A B :0 5 6 7 8 9 11: 1 4 ♯4 5 ♭6 6 7: ma13sus♯4(no 9 add ♭13)
namesBySequenceIndex[1480] = ''''''
namesBySequenceIndexNegative[1480] = ''''''
semitonesBySequenceIndex[1480] = [0, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1480] = '''B K L M N P S'''
poemAuthors[1480] = ''''''
#1481: ䷚䷍ Nourishment Resevoir: 5 1 1 1 1 2 1  : S M1 M2 P D1 D2 N3 : AlOf

#1482: C F F♯ G A♭ B♭ B :0 5 6 7 8 10 11: 1 4 ♯4 5 ♭6 ♭7 7: ma7sus4(add ♭7 ♯11 ♭13)
namesBySequenceIndex[1481] = ''''''
namesBySequenceIndexNegative[1481] = ''''''
semitonesBySequenceIndex[1481] = [0, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1481] = '''B K L M N R S'''
poemAuthors[1481] = ''''''
#1482: ䷚䷈ Nourishment Homegrown: 5 1 1 1 2 1 1  : S M1 M2 P D1 N2 N3 : AlOm

#1483: C F F♯ G A B♭ B :0 5 6 7 9 10 11: 1 4 ♯4 5 6 ♭7 7: ma13sus♯4(no 9 add ♭7)
namesBySequenceIndex[1482] = ''''''
namesBySequenceIndexNegative[1482] = ''''''
semitonesBySequenceIndex[1482] = [0, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1482] = '''B K L M P R S'''
poemAuthors[1482] = ''''''
#1483: ䷚䷉ Nourishment Treading: 5 1 1 2 1 1 1  : S M1 M2 P D2 N2 N3 : AlEg

#1484: C F G♭ A♭ A B♭ B :0 5 6 8 9 10 11: 1 4 ♭5 ♭6 6 ♭7 7: ma13sus♯4(♯5 no 9 add ♭7)
namesBySequenceIndex[1483] = ''''''
namesBySequenceIndexNegative[1483] = ''''''
semitonesBySequenceIndex[1483] = [0, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1483] = '''B K L N P R S'''
poemAuthors[1483] = ''''''
#1484: ䷚䷌ Nourishment Fellowship: 5 1 2 1 1 1 1  : S M1 M2 D1 D2 N2 N3 : AlIg

#1485: C F G A♭ A B♭ B :0 5 7 8 9 10 11: 1 4 5 ♭6 6 ♭7 7: ma7sus4(add ♭7 ♭13 13)
namesBySequenceIndex[1484] = ''''''
namesBySequenceIndexNegative[1484] = ''''''
semitonesBySequenceIndex[1484] = [0, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1484] = '''B K M N P R S'''
poemAuthors[1484] = ''''''
#1485: ䷚䷫ Nourishment Coupling: 5 2 1 1 1 1 1  : S M1 P D1 D2 N2 N3 : Alog

#1486: C F♯ G A♭ A B♭ B :0 6 7 8 9 10 11: 1 ♯4 5 ♭6 6 ♭7 7: ma7sus♯4(add ♭7 ♭13 13)
namesBySequenceIndex[1485] = ''''''
namesBySequenceIndexNegative[1485] = ''''''
semitonesBySequenceIndex[1485] = [0, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1485] = '''B L M N P R S'''
poemAuthors[1485] = ''''''
#1486: ䷗䷀ Returning Creative: 6 1 1 1 1 1 1  : S M2 P D1 D2 N2 N3 : AjOg

#1487: C D♭ D E♭ E F F♯ G :0 1 2 3 4 5 6 7: 1 ♭2 2 ♭3 3 4 ♯4 5: 11(no 7 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1486] = ''''''
namesBySequenceIndexNegative[1486] = ''''''
semitonesBySequenceIndex[1486] = [0, 1, 2, 3, 4, 5, 6, 7]
poemBySequenceIndex[1486] = '''B D F G J K L M'''
poemAuthors[1486] = ''''''
#1487: ䷀䷒ Creative Arrived: 1 1 1 1 1 1 1 5  : S R1 R2 G2 G3 M1 M2 P : OgEj

#1488: C D♭ D E♭ E F G♭ A♭ :0 1 2 3 4 5 6 8: 1 ♭2 2 ♭3 3 4 ♭5 ♭6: 11(♯5 no 7 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1487] = '''Logyllic'''
namesBySequenceIndexNegative[1487] = ''''''
semitonesBySequenceIndex[1487] = [0, 1, 2, 3, 4, 5, 6, 8]
poemBySequenceIndex[1487] = '''B D F G J K L N'''
poemAuthors[1487] = ''''''
#1488: ䷀䷣ Creative Eclipse: 1 1 1 1 1 1 2 4  : S R1 R2 G2 G3 M1 M2 D1 : OgIj

#1489: C D♭ D E♭ E F G♭ A :0 1 2 3 4 5 6 9: 1 ♭2 2 ♭3 3 4 ♭5 6: 13(♭5 no 7 add ♭9 ♯9)
namesBySequenceIndex[1488] = '''Ionaryllic'''
namesBySequenceIndexNegative[1488] = ''''''
semitonesBySequenceIndex[1488] = [0, 1, 2, 3, 4, 5, 6, 9]
poemBySequenceIndex[1488] = '''B D F G J K L P'''
poemAuthors[1488] = ''''''
#1489: ䷀䷲ Creative Thunder: 1 1 1 1 1 1 3 3  : S R1 R2 G2 G3 M1 M2 D2 : OgAb

#1490: C D♭ D E♭ E F G♭ B♭ :0 1 2 3 4 5 6 10: 1 ♭2 2 ♭3 3 4 ♭5 ♭7: 11(♭5 add ♭9 ♯9)
namesBySequenceIndex[1489] = '''Mythyllic'''
namesBySequenceIndexNegative[1489] = ''''''
semitonesBySequenceIndex[1489] = [0, 1, 2, 3, 4, 5, 6, 10]
poemBySequenceIndex[1489] = '''B D F G J K L R'''
poemAuthors[1489] = ''''''
#1490: ䷀䷂ Creative Sprouting: 1 1 1 1 1 1 4 2  : S R1 R2 G2 G3 M1 M2 N2 : OgAk

#1491: C D♭ D E♭ E F G♭ B :0 1 2 3 4 5 6 11: 1 ♭2 2 ♭3 3 4 ♭5 7: ma11(♭5 add ♭9 ♯9)
namesBySequenceIndex[1490] = ''''''
namesBySequenceIndexNegative[1490] = ''''''
semitonesBySequenceIndex[1490] = [0, 1, 2, 3, 4, 5, 6, 11]
poemBySequenceIndex[1490] = '''B D F G J K L S'''
poemAuthors[1490] = ''''''
#1491: ䷀䷚ Creative Nourishment: 1 1 1 1 1 1 5 1  : S R1 R2 G2 G3 M1 M2 N3 : OgAl

#1492: C D♭ D E♭ E F G A♭ :0 1 2 3 4 5 7 8: 1 ♭2 2 ♭3 3 4 5 ♭6: 11(no 7 add ♭6 ♭9 ♯9)
namesBySequenceIndex[1491] = '''Thyphyllic'''
namesBySequenceIndexNegative[1491] = ''''''
semitonesBySequenceIndex[1491] = [0, 1, 2, 3, 4, 5, 7, 8]
poemBySequenceIndex[1491] = '''B D F G J K M N'''
poemAuthors[1491] = ''''''
#1492: ䷀䷭ Creative Ascending: 1 1 1 1 1 2 1 4  : S R1 R2 G2 G3 M1 P D1 : Ogoj

#1493: C D♭ D E♭ E F G A :0 1 2 3 4 5 7 9: 1 ♭2 2 ♭3 3 4 5 6: 13(no 7 add ♭9 ♯9)
namesBySequenceIndex[1492] = '''Aerocryllic'''
namesBySequenceIndexNegative[1492] = ''''''
semitonesBySequenceIndex[1492] = [0, 1, 2, 3, 4, 5, 7, 9]
poemBySequenceIndex[1492] = '''B D F G J K M P'''
poemAuthors[1492] = ''''''
#1493: ䷀䷧ Creative Deliverance: 1 1 1 1 1 2 2 3  : S R1 R2 G2 G3 M1 P D2 : Ogeb

#1494: C D♭ D E♭ E F G B♭ :0 1 2 3 4 5 7 10: 1 ♭2 2 ♭3 3 4 5 ♭7: 11(add ♭9 ♯9)
namesBySequenceIndex[1493] = ''''''
namesBySequenceIndexNegative[1493] = ''''''
semitonesBySequenceIndex[1493] = [0, 1, 2, 3, 4, 5, 7, 10]
poemBySequenceIndex[1493] = '''B D F G J K M R'''
poemAuthors[1493] = ''''''
#1494: ䷀䷜ Creative Chasm: 1 1 1 1 1 2 3 2  : S R1 R2 G2 G3 M1 P N2 : Ogek

#1495: C D♭ D E♭ E F G B :0 1 2 3 4 5 7 11: 1 ♭2 2 ♭3 3 4 5 7: ma11(add ♭9 ♯9)
namesBySequenceIndex[1494] = '''Dacryllic'''
namesBySequenceIndexNegative[1494] = ''''''
semitonesBySequenceIndex[1494] = [0, 1, 2, 3, 4, 5, 7, 11]
poemBySequenceIndex[1494] = '''B D F G J K M S'''
poemAuthors[1494] = ''''''
#1495: ䷀䷃ Creative Naïve: 1 1 1 1 1 2 4 1  : S R1 R2 G2 G3 M1 P N3 : Ogel

#1496: C D♭ D E♭ E F G♯ A :0 1 2 3 4 5 8 9: 1 ♭2 2 ♭3 3 4 ♯5 6: 13(♯5 no 7 add ♭9 ♯9)
namesBySequenceIndex[1495] = '''Rodyllic'''
namesBySequenceIndexNegative[1495] = ''''''
semitonesBySequenceIndex[1495] = [0, 1, 2, 3, 4, 5, 8, 9]
poemBySequenceIndex[1495] = '''B D F G J K N P'''
poemAuthors[1495] = ''''''
#1496: ䷀䷽ Creative Detail: 1 1 1 1 1 3 1 3  : S R1 R2 G2 G3 M1 D1 N1 : Ogib

#1497: C D♭ D E♭ E F G♯ B♭ :0 1 2 3 4 5 8 10: 1 ♭2 2 ♭3 3 4 ♯5 ♭7: 11(♯5 add ♭9 ♯9)
namesBySequenceIndex[1496] = '''Zalyllic'''
namesBySequenceIndexNegative[1496] = ''''''
semitonesBySequenceIndex[1496] = [0, 1, 2, 3, 4, 5, 8, 10]
poemBySequenceIndex[1496] = '''B D F G J K N R'''
poemAuthors[1496] = ''''''
#1497: ䷀䷦ Creative Limping: 1 1 1 1 1 3 2 2  : S R1 R2 G2 G3 M1 D1 N2 : Ogik

#1498: C D♭ D E♭ E F G♯ B :0 1 2 3 4 5 8 11: 1 ♭2 2 ♭3 3 4 ♯5 7: ma11(♯5 add ♭9 ♯9)
namesBySequenceIndex[1497] = '''Laryllic'''
namesBySequenceIndexNegative[1497] = ''''''
semitonesBySequenceIndex[1497] = [0, 1, 2, 3, 4, 5, 8, 11]
poemBySequenceIndex[1497] = '''B D F G J K N S'''
poemAuthors[1497] = ''''''
#1498: ䷀䷳ Creative Stillness: 1 1 1 1 1 3 3 1  : S R1 R2 G2 G3 M1 D1 N3 : Ogil

#1499: C D♭ D E♭ E F A B♭ :0 1 2 3 4 5 9 10: 1 ♭2 2 ♭3 3 4 6 ♭7: 13(no 5 add ♭9 ♯9)
namesBySequenceIndex[1498] = '''Pocryllic'''
namesBySequenceIndexNegative[1498] = ''''''
semitonesBySequenceIndex[1498] = [0, 1, 2, 3, 4, 5, 9, 10]
poemBySequenceIndex[1498] = '''B D F G J K P R'''
poemAuthors[1498] = ''''''
#1499: ䷀䷬ Creative Clustering: 1 1 1 1 1 4 1 2  : S R1 R2 G2 G3 M1 D2 N2 : Ogad

#1500: C D♭ D E♭ E F A B :0 1 2 3 4 5 9 11: 1 ♭2 2 ♭3 3 4 6 7: ma13(no 5 add ♭9 ♯9)
namesBySequenceIndex[1499] = '''Aerylyllic'''
namesBySequenceIndexNegative[1499] = ''''''
semitonesBySequenceIndex[1499] = [0, 1, 2, 3, 4, 5, 9, 11]
poemBySequenceIndex[1499] = '''B D F G J K P S'''
poemAuthors[1499] = ''''''
#1500: ䷀䷢ Creative Prospering: 1 1 1 1 1 4 2 1  : S R1 R2 G2 G3 M1 D2 N3 : Ogaf

#1501: C D♭ D E♭ E F A♯ B :0 1 2 3 4 5 10 11: 1 ♭2 2 ♭3 3 4 ♯6 7: ma11(no 5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1500] = ''''''
namesBySequenceIndexNegative[1500] = ''''''
semitonesBySequenceIndex[1500] = [0, 1, 2, 3, 4, 5, 10, 11]
poemBySequenceIndex[1500] = '''B D F G J K R S'''
poemAuthors[1500] = ''''''
#1501: ䷀䷓ Creative Seeing: 1 1 1 1 1 5 1 1  : S R1 R2 G2 G3 M1 D3 N3 : Ogam

#1502: C D♭ D E♭ E F♯ G A♭ :0 1 2 3 4 6 7 8: 1 ♭2 2 ♭3 3 ♯4 5 ♭6: ma(add ♭6 ♭9 9 ♯9 ♯11)
namesBySequenceIndex[1501] = '''Kocryllic'''
namesBySequenceIndexNegative[1501] = ''''''
semitonesBySequenceIndex[1501] = [0, 1, 2, 3, 4, 6, 7, 8]
poemBySequenceIndex[1501] = '''B D F G J L M N'''
poemAuthors[1501] = ''''''
#1502: ䷪䷊ Decision Peace: 1 1 1 1 2 1 1 4  : S R1 R2 G2 G3 M2 P D1 : OdOj

#1503: C D♭ D E♭ E F♯ G A :0 1 2 3 4 6 7 9: 1 ♭2 2 ♭3 3 ♯4 5 6: 6/9(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1502] = '''Sylyllic'''
namesBySequenceIndexNegative[1502] = ''''''
semitonesBySequenceIndex[1502] = [0, 1, 2, 3, 4, 6, 7, 9]
poemBySequenceIndex[1502] = '''B D F G J L M P'''
poemAuthors[1502] = ''''''
#1503: ䷪䷵ Decision Betrothing: 1 1 1 1 2 1 2 3  : S R1 R2 G2 G3 M2 P D2 : OdEb

#1504: C D♭ D E♭ E F♯ G B♭ :0 1 2 3 4 6 7 10: 1 ♭2 2 ♭3 3 ♯4 5 ♭7: 9(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1503] = '''Mygyllic'''
namesBySequenceIndexNegative[1503] = ''''''
semitonesBySequenceIndex[1503] = [0, 1, 2, 3, 4, 6, 7, 10]
poemBySequenceIndex[1503] = '''B D F G J L M R'''
poemAuthors[1503] = ''''''
#1504: ䷪䷻ Decision Composed: 1 1 1 1 2 1 3 2  : S R1 R2 G2 G3 M2 P N2 : OdEk

#1505: C D♭ D E♭ E F♯ G B :0 1 2 3 4 6 7 11: 1 ♭2 2 ♭3 3 ♯4 5 7: ma9(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1504] = '''Poptyllic'''
namesBySequenceIndexNegative[1504] = ''''''
semitonesBySequenceIndex[1504] = [0, 1, 2, 3, 4, 6, 7, 11]
poemBySequenceIndex[1504] = '''B D F G J L M S'''
poemAuthors[1504] = ''''''
#1505: ䷪䷨ Decision Diminishing: 1 1 1 1 2 1 4 1  : S R1 R2 G2 G3 M2 P N3 : OdEl

#1506: C D♭ D E♭ E F♯ G♯ A :0 1 2 3 4 6 8 9: 1 ♭2 2 ♭3 3 ♯4 ♯5 6: 6/9(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1505] = '''Pyryllic'''
namesBySequenceIndexNegative[1505] = ''''''
semitonesBySequenceIndex[1505] = [0, 1, 2, 3, 4, 6, 8, 9]
poemBySequenceIndex[1505] = '''B D F G J L N P'''
poemAuthors[1505] = ''''''
#1506: ䷪䷶ Decision Abounding: 1 1 1 1 2 2 1 3  : S R1 R2 G2 G3 M2 D1 N1 : OdIb

#1507: C D♭ D E♭ E F♯ G♯ B♭ :0 1 2 3 4 6 8 10: 1 ♭2 2 ♭3 3 ♯4 ♯5 ♭7: 9(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1506] = '''Bothyllic'''
namesBySequenceIndexNegative[1506] = ''''''
semitonesBySequenceIndex[1506] = [0, 1, 2, 3, 4, 6, 8, 10]
poemBySequenceIndex[1506] = '''B D F G J L N R'''
poemAuthors[1506] = ''''''
#1507: ䷪䷾ Decision Completed: 1 1 1 1 2 2 2 2  : S R1 R2 G2 G3 M2 D1 N2 : OdIk

#1508: C D♭ D E♭ E F♯ G♯ B :0 1 2 3 4 6 8 11: 1 ♭2 2 ♭3 3 ♯4 ♯5 7: ma9(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1507] = '''Zanyllic'''
namesBySequenceIndexNegative[1507] = ''''''
semitonesBySequenceIndex[1507] = [0, 1, 2, 3, 4, 6, 8, 11]
poemBySequenceIndex[1507] = '''B D F G J L N S'''
poemAuthors[1507] = ''''''
#1508: ䷪䷕ Decision Adorning: 1 1 1 1 2 2 3 1  : S R1 R2 G2 G3 M2 D1 N3 : OdIl

#1509: C D♭ D E♭ E G♭ A B♭ :0 1 2 3 4 6 9 10: 1 ♭2 2 ♭3 3 ♭5 6 ♭7: 9(♭5 add ♭9 ♯9 13)
namesBySequenceIndex[1508] = '''Rynyllic'''
namesBySequenceIndexNegative[1508] = ''''''
semitonesBySequenceIndex[1508] = [0, 1, 2, 3, 4, 6, 9, 10]
poemBySequenceIndex[1508] = '''B D F G J L P R'''
poemAuthors[1508] = ''''''
#1509: ䷪䷐ Decision Following: 1 1 1 1 2 3 1 2  : S R1 R2 G2 G3 M2 D2 N2 : OdAd

#1510: C D♭ D E♭ E G♭ A B :0 1 2 3 4 6 9 11: 1 ♭2 2 ♭3 3 ♭5 6 7: ma9(♭5 add ♭9 ♯9 13)
namesBySequenceIndex[1509] = ''''''
namesBySequenceIndexNegative[1509] = ''''''
semitonesBySequenceIndex[1509] = [0, 1, 2, 3, 4, 6, 9, 11]
poemBySequenceIndex[1509] = '''B D F G J L P S'''
poemAuthors[1509] = ''''''
#1510: ䷪䷔ Decision Biting: 1 1 1 1 2 3 2 1  : S R1 R2 G2 G3 M2 D2 N3 : OdAf

#1511: C D♭ D E♭ E G♭ A♯ B :0 1 2 3 4 6 10 11: 1 ♭2 2 ♭3 3 ♭5 ♯6 7: ma9(♭5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1510] = '''Thynyllic'''
namesBySequenceIndexNegative[1510] = ''''''
semitonesBySequenceIndex[1510] = [0, 1, 2, 3, 4, 6, 10, 11]
poemBySequenceIndex[1510] = '''B D F G J L R S'''
poemAuthors[1510] = ''''''
#1511: ䷪䷩ Decision Augmenting: 1 1 1 1 2 4 1 1  : S R1 R2 G2 G3 M2 D3 N3 : OdAm

#1512: C D♭ D E♭ E G A♭ A :0 1 2 3 4 7 8 9: 1 ♭2 2 ♭3 3 5 ♭6 6: 6/9(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1511] = '''Gaptyllic'''
namesBySequenceIndexNegative[1511] = ''''''
semitonesBySequenceIndex[1511] = [0, 1, 2, 3, 4, 7, 8, 9]
poemBySequenceIndex[1511] = '''B D F G J M N P'''
poemAuthors[1511] = ''''''
#1512: ䷪䷟ Decision Persevering: 1 1 1 1 3 1 1 3  : S R1 R2 G2 G3 P D1 N1 : Odob

#1513: C D♭ D E♭ E G A♭ B♭ :0 1 2 3 4 7 8 10: 1 ♭2 2 ♭3 3 5 ♭6 ♭7: 9(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1512] = '''Rolyllic'''
namesBySequenceIndexNegative[1512] = ''''''
semitonesBySequenceIndex[1512] = [0, 1, 2, 3, 4, 7, 8, 10]
poemBySequenceIndex[1512] = '''B D F G J M N R'''
poemAuthors[1512] = ''''''
#1513: ䷪䷯ Decision Source: 1 1 1 1 3 1 2 2  : S R1 R2 G2 G3 P D1 N2 : Odok

#1514: C D♭ D E♭ E G A♭ B :0 1 2 3 4 7 8 11: 1 ♭2 2 ♭3 3 5 ♭6 7: ma9(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1513] = '''Ionathyllic'''
namesBySequenceIndexNegative[1513] = ''''''
semitonesBySequenceIndex[1513] = [0, 1, 2, 3, 4, 7, 8, 11]
poemBySequenceIndex[1513] = '''B D F G J M N S'''
poemAuthors[1513] = ''''''
#1514: ䷪䷑ Decision Correcting: 1 1 1 1 3 1 3 1  : S R1 R2 G2 G3 P D1 N3 : Odol

#1515: C D♭ D E♭ E G A B♭ :0 1 2 3 4 7 9 10: 1 ♭2 2 ♭3 3 5 6 ♭7: 9(add ♭9 ♯9 13)
namesBySequenceIndex[1514] = '''Phrodyllic'''
namesBySequenceIndexNegative[1514] = ''''''
semitonesBySequenceIndex[1514] = [0, 1, 2, 3, 4, 7, 9, 10]
poemBySequenceIndex[1514] = '''B D F G J M P R'''
poemAuthors[1514] = ''''''
#1515: ䷪䷮ Decision Confining: 1 1 1 1 3 2 1 2  : S R1 R2 G2 G3 P D2 N2 : Oded

#1516: C D♭ D E♭ E G A B :0 1 2 3 4 7 9 11: 1 ♭2 2 ♭3 3 5 6 7: ma9(add ♭9 ♯9 13)
namesBySequenceIndex[1515] = '''Zocryllic'''
namesBySequenceIndexNegative[1515] = ''''''
semitonesBySequenceIndex[1515] = [0, 1, 2, 3, 4, 7, 9, 11]
poemBySequenceIndex[1515] = '''B D F G J M P S'''
poemAuthors[1515] = ''''''
#1516: ䷪䷿ Decision Unfinished: 1 1 1 1 3 2 2 1  : S R1 R2 G2 G3 P D2 N3 : Odef

#1517: C D♭ D E♭ E G A♯ B :0 1 2 3 4 7 10 11: 1 ♭2 2 ♭3 3 5 ♯6 7: ma9(add ♭7 ♭9 ♯9)
namesBySequenceIndex[1516] = '''Kataptyllic'''
namesBySequenceIndexNegative[1516] = ''''''
semitonesBySequenceIndex[1516] = [0, 1, 2, 3, 4, 7, 10, 11]
poemBySequenceIndex[1516] = '''B D F G J M R S'''
poemAuthors[1516] = ''''''
#1517: ䷪䷺ Decision Dispersing: 1 1 1 1 3 3 1 1  : S R1 R2 G2 G3 P D3 N3 : Odem

#1518: C D♭ D E♭ E G♯ A B♭ :0 1 2 3 4 8 9 10: 1 ♭2 2 ♭3 3 ♯5 6 ♭7: 9(♯5 add ♭9 ♯9 13)
namesBySequenceIndex[1517] = '''Phralyllic'''
namesBySequenceIndexNegative[1517] = ''''''
semitonesBySequenceIndex[1517] = [0, 1, 2, 3, 4, 8, 9, 10]
poemBySequenceIndex[1517] = '''B D F G J N P R'''
poemAuthors[1517] = ''''''
#1518: ䷪䷞ Decision Conjoining: 1 1 1 1 4 1 1 2  : S R1 R2 G2 G3 D1 D2 N2 : Odid

#1519: C D♭ D E♭ E G♯ A B :0 1 2 3 4 8 9 11: 1 ♭2 2 ♭3 3 ♯5 6 7: ma9(♯5 add ♭9 ♯9 13)
namesBySequenceIndex[1518] = '''Phracryllic'''
namesBySequenceIndexNegative[1518] = ''''''
semitonesBySequenceIndex[1518] = [0, 1, 2, 3, 4, 8, 9, 11]
poemBySequenceIndex[1518] = '''B D F G J N P S'''
poemAuthors[1518] = ''''''
#1519: ䷪䷷ Decision Wanderer: 1 1 1 1 4 1 2 1  : S R1 R2 G2 G3 D1 D2 N3 : Odif

#1520: C D♭ D E♭ E G♯ A♯ B :0 1 2 3 4 8 10 11: 1 ♭2 2 ♭3 3 ♯5 ♯6 7: ma9(♯5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1519] = '''Bonyllic'''
namesBySequenceIndexNegative[1519] = ''''''
semitonesBySequenceIndex[1519] = [0, 1, 2, 3, 4, 8, 10, 11]
poemBySequenceIndex[1519] = '''B D F G J N R S'''
poemAuthors[1519] = ''''''
#1520: ䷪䷴ Decision Infiltrating: 1 1 1 1 4 2 1 1  : S R1 R2 G2 G3 D1 N2 N3 : Odim

#1521: C D♭ D E♭ E A B♭ B :0 1 2 3 4 9 10 11: 1 ♭2 2 ♭3 3 6 ♭7 7: ma9(no 5 add ♭7 ♭9 ♯9 13)
namesBySequenceIndex[1520] = ''''''
namesBySequenceIndexNegative[1520] = ''''''
semitonesBySequenceIndex[1520] = [0, 1, 2, 3, 4, 9, 10, 11]
poemBySequenceIndex[1520] = '''B D F G J P R S'''
poemAuthors[1520] = ''''''
#1521: ䷪䷋ Decision Obstruction: 1 1 1 1 5 1 1 1  : S R1 R2 G2 G3 D2 N2 N3 : Odag

#1522: C D♭ D E♭ F F♯ G A♭ :0 1 2 3 5 6 7 8: 1 ♭2 2 ♭3 4 ♯4 5 ♭6: mi11(no 7 add ♭6 ♭9 ♯11)
namesBySequenceIndex[1521] = '''Bocryllic'''
namesBySequenceIndexNegative[1521] = ''''''
semitonesBySequenceIndex[1521] = [0, 1, 2, 3, 5, 6, 7, 8]
poemBySequenceIndex[1521] = '''B D F G K L M N'''
poemAuthors[1521] = ''''''
#1522: ䷍䷊ Resevoir Peace: 1 1 1 2 1 1 1 4  : S R1 R2 G2 M1 M2 P D1 : OfOj

#1523: C D♭ D E♭ F F♯ G A :0 1 2 3 5 6 7 9: 1 ♭2 2 ♭3 4 ♯4 5 6: mi13(no 7 add ♭9 ♯11)
namesBySequenceIndex[1522] = ''''''
namesBySequenceIndexNegative[1522] = ''''''
semitonesBySequenceIndex[1522] = [0, 1, 2, 3, 5, 6, 7, 9]
poemBySequenceIndex[1522] = '''B D F G K L M P'''
poemAuthors[1522] = ''''''
#1523: ䷍䷵ Resevoir Betrothing: 1 1 1 2 1 1 2 3  : S R1 R2 G2 M1 M2 P D2 : OfEb

#1524: C D♭ D E♭ F F♯ G B♭ :0 1 2 3 5 6 7 10: 1 ♭2 2 ♭3 4 ♯4 5 ♭7: mi11(add ♭9 ♯11)
namesBySequenceIndex[1523] = '''Stynyllic'''
namesBySequenceIndexNegative[1523] = ''''''
semitonesBySequenceIndex[1523] = [0, 1, 2, 3, 5, 6, 7, 10]
poemBySequenceIndex[1523] = '''B D F G K L M R'''
poemAuthors[1523] = ''''''
#1524: ䷍䷻ Resevoir Composed: 1 1 1 2 1 1 3 2  : S R1 R2 G2 M1 M2 P N2 : OfEk

#1525: C D♭ D E♭ F F♯ G B :0 1 2 3 5 6 7 11: 1 ♭2 2 ♭3 4 ♯4 5 7: mima11(add ♭9 ♯11)
namesBySequenceIndex[1524] = '''Lodyllic'''
namesBySequenceIndexNegative[1524] = ''''''
semitonesBySequenceIndex[1524] = [0, 1, 2, 3, 5, 6, 7, 11]
poemBySequenceIndex[1524] = '''B D F G K L M S'''
poemAuthors[1524] = ''''''
#1525: ䷍䷨ Resevoir Diminishing: 1 1 1 2 1 1 4 1  : S R1 R2 G2 M1 M2 P N3 : OfEl

#1526: C D♭ D E♭ F G♭ A♭ A :0 1 2 3 5 6 8 9: 1 ♭2 2 ♭3 4 ♭5 ♭6 6: °13(no 7 add ♭6 ♭9)
namesBySequenceIndex[1525] = '''Aeolocryllic'''
namesBySequenceIndexNegative[1525] = ''''''
semitonesBySequenceIndex[1525] = [0, 1, 2, 3, 5, 6, 8, 9]
poemBySequenceIndex[1525] = '''B D F G K L N P'''
poemAuthors[1525] = ''''''
#1526: ䷍䷶ Resevoir Abounding: 1 1 1 2 1 2 1 3  : S R1 R2 G2 M1 M2 D1 N1 : OfIb

#1527: C D♭ D E♭ F G♭ A♭ B♭ :0 1 2 3 5 6 8 10: 1 ♭2 2 ♭3 4 ♭5 ♭6 ♭7: ⌀11(add ♭9 ♭13)
namesBySequenceIndex[1526] = '''Aeradyllic'''
namesBySequenceIndexNegative[1526] = ''''''
semitonesBySequenceIndex[1526] = [0, 1, 2, 3, 5, 6, 8, 10]
poemBySequenceIndex[1526] = '''B D F G K L N R'''
poemAuthors[1526] = ''''''
#1527: ䷍䷾ Resevoir Completed: 1 1 1 2 1 2 2 2  : S R1 R2 G2 M1 M2 D1 N2 : OfIk

#1528: C D♭ D E♭ F G♭ A♭ B :0 1 2 3 5 6 8 11: 1 ♭2 2 ♭3 4 ♭5 ♭6 7: °ma11(add ♭9 ♭13)
namesBySequenceIndex[1527] = '''Lothyllic'''
namesBySequenceIndexNegative[1527] = ''''''
semitonesBySequenceIndex[1527] = [0, 1, 2, 3, 5, 6, 8, 11]
poemBySequenceIndex[1527] = '''B D F G K L N S'''
poemAuthors[1527] = ''''''
#1528: ䷍䷕ Resevoir Adorning: 1 1 1 2 1 2 3 1  : S R1 R2 G2 M1 M2 D1 N3 : OfIl

#1529: C D♭ D E♭ F G♭ A B♭ :0 1 2 3 5 6 9 10: 1 ♭2 2 ♭3 4 ♭5 6 ♭7: ⌀13(add ♭9)
namesBySequenceIndex[1528] = '''Polyllic'''
namesBySequenceIndexNegative[1528] = ''''''
semitonesBySequenceIndex[1528] = [0, 1, 2, 3, 5, 6, 9, 10]
poemBySequenceIndex[1528] = '''B D F G K L P R'''
poemAuthors[1528] = ''''''
#1529: ䷍䷐ Resevoir Following: 1 1 1 2 1 3 1 2  : S R1 R2 G2 M1 M2 D2 N2 : OfAd

#1530: C D♭ D E♭ F G♭ A B :0 1 2 3 5 6 9 11: 1 ♭2 2 ♭3 4 ♭5 6 7: °ma13(add ♭9)
namesBySequenceIndex[1529] = '''Lylyllic'''
namesBySequenceIndexNegative[1529] = ''''''
semitonesBySequenceIndex[1529] = [0, 1, 2, 3, 5, 6, 9, 11]
poemBySequenceIndex[1529] = '''B D F G K L P S'''
poemAuthors[1529] = ''''''
#1530: ䷍䷔ Resevoir Biting: 1 1 1 2 1 3 2 1  : S R1 R2 G2 M1 M2 D2 N3 : OfAf

#1531: C D♭ D E♭ F G♭ A♯ B :0 1 2 3 5 6 10 11: 1 ♭2 2 ♭3 4 ♭5 ♯6 7: °ma11(add ♭7 ♭9)
namesBySequenceIndex[1530] = '''Mixonyllic'''
namesBySequenceIndexNegative[1530] = ''''''
semitonesBySequenceIndex[1530] = [0, 1, 2, 3, 5, 6, 10, 11]
poemBySequenceIndex[1530] = '''B D F G K L R S'''
poemAuthors[1530] = ''''''
#1531: ䷍䷩ Resevoir Augmenting: 1 1 1 2 1 4 1 1  : S R1 R2 G2 M1 M2 D3 N3 : OfAm

#1532: C D♭ D E♭ F G A♭ A :0 1 2 3 5 7 8 9: 1 ♭2 2 ♭3 4 5 ♭6 6: mi13(no 7 add ♭6 ♭9)
namesBySequenceIndex[1531] = '''Aerygyllic'''
namesBySequenceIndexNegative[1531] = ''''''
semitonesBySequenceIndex[1531] = [0, 1, 2, 3, 5, 7, 8, 9]
poemBySequenceIndex[1531] = '''B D F G K M N P'''
poemAuthors[1531] = ''''''
#1532: ䷍䷟ Resevoir Persevering: 1 1 1 2 2 1 1 3  : S R1 R2 G2 M1 P D1 N1 : Ofob

#1533: C D♭ D E♭ F G A♭ B♭ :0 1 2 3 5 7 8 10: 1 ♭2 2 ♭3 4 5 ♭6 ♭7: mi11(add ♭9 ♭13)
namesBySequenceIndex[1532] = '''Soryllic,Phrygian Add ♮9,Phrygian Aeolian ♭4'''
namesBySequenceIndexNegative[1532] = ''''''
semitonesBySequenceIndex[1532] = [0, 1, 2, 3, 5, 7, 8, 10]
poemBySequenceIndex[1532] = '''B D F G K M N R'''
poemAuthors[1532] = ''''''
#1533: ䷍䷯ Resevoir Source: 1 1 1 2 2 1 2 2  : S R1 R2 G2 M1 P D1 N2 : Ofok

#1534: C D♭ D E♭ F G A♭ B :0 1 2 3 5 7 8 11: 1 ♭2 2 ♭3 4 5 ♭6 7: mima11(add ♭9 ♭13)
namesBySequenceIndex[1533] = '''Rycryllic,Harmonic Neapolitan Minor'''
namesBySequenceIndexNegative[1533] = ''''''
semitonesBySequenceIndex[1533] = [0, 1, 2, 3, 5, 7, 8, 11]
poemBySequenceIndex[1533] = '''B D F G K M N S'''
poemAuthors[1533] = ''''''
#1534: ䷍䷑ Resevoir Correcting: 1 1 1 2 2 1 3 1  : S R1 R2 G2 M1 P D1 N3 : Ofol

#1535: C D♭ D E♭ F G A B♭ :0 1 2 3 5 7 9 10: 1 ♭2 2 ♭3 4 5 6 ♭7: mi13(add ♭9)
namesBySequenceIndex[1534] = '''Adonai Malakh,Adonai Malakh (Jewish),Ragyllic'''
namesBySequenceIndexNegative[1534] = ''''''
semitonesBySequenceIndex[1534] = [0, 1, 2, 3, 5, 7, 9, 10]
poemBySequenceIndex[1534] = '''B D F G K M P R'''
poemAuthors[1534] = ''''''
#1535: ䷍䷮ Resevoir Confining: 1 1 1 2 2 2 1 2  : S R1 R2 G2 M1 P D2 N2 : Ofed

#1536: C D♭ D E♭ F G A B :0 1 2 3 5 7 9 11: 1 ♭2 2 ♭3 4 5 6 7: mima13(add ♭9)
namesBySequenceIndex[1535] = '''Gynyllic'''
namesBySequenceIndexNegative[1535] = ''''''
semitonesBySequenceIndex[1535] = [0, 1, 2, 3, 5, 7, 9, 11]
poemBySequenceIndex[1535] = '''B D F G K M P S'''
poemAuthors[1535] = ''''''
#1536: ䷍䷿ Resevoir Unfinished: 1 1 1 2 2 2 2 1  : S R1 R2 G2 M1 P D2 N3 : Ofef

#1537: C D♭ D E♭ F G A♯ B :0 1 2 3 5 7 10 11: 1 ♭2 2 ♭3 4 5 ♯6 7: mima11(add ♭7 ♭9)
namesBySequenceIndex[1536] = '''Aeolonyllic'''
namesBySequenceIndexNegative[1536] = ''''''
semitonesBySequenceIndex[1536] = [0, 1, 2, 3, 5, 7, 10, 11]
poemBySequenceIndex[1536] = '''B D F G K M R S'''
poemAuthors[1536] = ''''''
#1537: ䷍䷺ Resevoir Dispersing: 1 1 1 2 2 3 1 1  : S R1 R2 G2 M1 P D3 N3 : Ofem

#1538: C D♭ D E♭ F G♯ A B♭ :0 1 2 3 5 8 9 10: 1 ♭2 2 ♭3 4 ♯5 6 ♭7: mi13(♯5 add ♭9)
namesBySequenceIndex[1537] = '''Zogyllic'''
namesBySequenceIndexNegative[1537] = ''''''
semitonesBySequenceIndex[1537] = [0, 1, 2, 3, 5, 8, 9, 10]
poemBySequenceIndex[1537] = '''B D F G K N P R'''
poemAuthors[1537] = ''''''
#1538: ䷍䷞ Resevoir Conjoining: 1 1 1 2 3 1 1 2  : S R1 R2 G2 M1 D1 D2 N2 : Ofid

#1539: C D♭ D E♭ F G♯ A B :0 1 2 3 5 8 9 11: 1 ♭2 2 ♭3 4 ♯5 6 7: mima13(♯5 add ♭9)
namesBySequenceIndex[1538] = '''Aerogyllic'''
namesBySequenceIndexNegative[1538] = ''''''
semitonesBySequenceIndex[1538] = [0, 1, 2, 3, 5, 8, 9, 11]
poemBySequenceIndex[1538] = '''B D F G K N P S'''
poemAuthors[1538] = ''''''
#1539: ䷍䷷ Resevoir Wanderer: 1 1 1 2 3 1 2 1  : S R1 R2 G2 M1 D1 D2 N3 : Ofif

#1540: C D♭ D E♭ F G♯ A♯ B :0 1 2 3 5 8 10 11: 1 ♭2 2 ♭3 4 ♯5 ♯6 7: mima11(♯5 add ♭7 ♭9)
namesBySequenceIndex[1539] = ''''''
namesBySequenceIndexNegative[1539] = ''''''
semitonesBySequenceIndex[1539] = [0, 1, 2, 3, 5, 8, 10, 11]
poemBySequenceIndex[1539] = '''B D F G K N R S'''
poemAuthors[1539] = ''''''
#1540: ䷍䷴ Resevoir Infiltrating: 1 1 1 2 3 2 1 1  : S R1 R2 G2 M1 D1 N2 N3 : Ofim

#1541: C D♭ D E♭ F A B♭ B :0 1 2 3 5 9 10 11: 1 ♭2 2 ♭3 4 6 ♭7 7: mima13(no 5 add ♭7 ♭9)
namesBySequenceIndex[1540] = '''Gydyllic'''
namesBySequenceIndexNegative[1540] = ''''''
semitonesBySequenceIndex[1540] = [0, 1, 2, 3, 5, 9, 10, 11]
poemBySequenceIndex[1540] = '''B D F G K P R S'''
poemAuthors[1540] = ''''''
#1541: ䷍䷋ Resevoir Obstruction: 1 1 1 2 4 1 1 1  : S R1 R2 G2 M1 D2 N2 N3 : Ofag

#1542: C D♭ D E♭ F♯ G A♭ A :0 1 2 3 6 7 8 9: 1 ♭2 2 ♭3 ♯4 5 ♭6 6: mi6/9(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1541] = '''Katogyllic'''
namesBySequenceIndexNegative[1541] = ''''''
semitonesBySequenceIndex[1541] = [0, 1, 2, 3, 6, 7, 8, 9]
poemBySequenceIndex[1541] = '''B D F G L M N P'''
poemAuthors[1541] = ''''''
#1542: ䷡䷡ Might Might : 1 1 1 3 1 1 1 3  : S R1 R2 G2 M2 P D1 N1 : ObOb

#1543: C D♭ D E♭ F♯ G A♭ B♭ :0 1 2 3 6 7 8 10: 1 ♭2 2 ♭3 ♯4 5 ♭6 ♭7: mi9(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1542] = '''Mothyllic'''
namesBySequenceIndexNegative[1542] = ''''''
semitonesBySequenceIndex[1542] = [0, 1, 2, 3, 6, 7, 8, 10]
poemBySequenceIndex[1542] = '''B D F G L M N R'''
poemAuthors[1542] = ''''''
#1543: ䷡䷄ Might Biding: 1 1 1 3 1 1 2 2  : S R1 R2 G2 M2 P D1 N2 : ObOk

#1544: C D♭ D E♭ F♯ G A♭ B :0 1 2 3 6 7 8 11: 1 ♭2 2 ♭3 ♯4 5 ♭6 7: mima9(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1543] = '''Aeroptyllic,Hungarian Minor (add ♭2)'''
namesBySequenceIndexNegative[1543] = ''''''
semitonesBySequenceIndex[1543] = [0, 1, 2, 3, 6, 7, 8, 11]
poemBySequenceIndex[1543] = '''B D F G L M N S'''
poemAuthors[1543] = ''''''
#1544: ䷡䷙ Might Cultivating: 1 1 1 3 1 1 3 1  : S R1 R2 G2 M2 P D1 N3 : ObOl

#1545: C D♭ D E♭ F♯ G A B♭ :0 1 2 3 6 7 9 10: 1 ♭2 2 ♭3 ♯4 5 6 ♭7: mi13(♯11 add ♭9)
namesBySequenceIndex[1544] = '''Epigyllic'''
namesBySequenceIndexNegative[1544] = ''''''
semitonesBySequenceIndex[1544] = [0, 1, 2, 3, 6, 7, 9, 10]
poemBySequenceIndex[1544] = '''B D F G L M P R'''
poemAuthors[1544] = ''''''
#1545: ䷡䷹ Might Joyous: 1 1 1 3 1 2 1 2  : S R1 R2 G2 M2 P D2 N2 : ObEd

#1546: C D♭ D E♭ F♯ G A B :0 1 2 3 6 7 9 11: 1 ♭2 2 ♭3 ♯4 5 6 7: mima13(♯11 add ♭9)
namesBySequenceIndex[1545] = '''Katydyllic'''
namesBySequenceIndexNegative[1545] = ''''''
semitonesBySequenceIndex[1545] = [0, 1, 2, 3, 6, 7, 9, 11]
poemBySequenceIndex[1545] = '''B D F G L M P S'''
poemAuthors[1545] = ''''''
#1546: ䷡䷥ Might Polarising: 1 1 1 3 1 2 2 1  : S R1 R2 G2 M2 P D2 N3 : ObEf

#1547: C D♭ D E♭ F♯ G A♯ B :0 1 2 3 6 7 10 11: 1 ♭2 2 ♭3 ♯4 5 ♯6 7: mima9(add ♭7 ♭9 ♯11)
namesBySequenceIndex[1546] = '''Pythyllic'''
namesBySequenceIndexNegative[1546] = ''''''
semitonesBySequenceIndex[1546] = [0, 1, 2, 3, 6, 7, 10, 11]
poemBySequenceIndex[1546] = '''B D F G L M R S'''
poemAuthors[1546] = ''''''
#1547: ䷡䷼ Might Truth: 1 1 1 3 1 3 1 1  : S R1 R2 G2 M2 P D3 N3 : ObEm

#1548: C D♭ D E♭ F♯ G♯ A B♭ :0 1 2 3 6 8 9 10: 1 ♭2 2 ♭3 ♯4 ♯5 6 ♭7: ⌀13(♯11 add ♭9 ♭13)
namesBySequenceIndex[1547] = '''Aeolyllic'''
namesBySequenceIndexNegative[1547] = ''''''
semitonesBySequenceIndex[1547] = [0, 1, 2, 3, 6, 8, 9, 10]
poemBySequenceIndex[1547] = '''B D F G L N P R'''
poemAuthors[1547] = ''''''
#1548: ䷡䷰ Might Revolution: 1 1 1 3 2 1 1 2  : S R1 R2 G2 M2 D1 D2 N2 : ObId

#1549: C D♭ D E♭ F♯ G♯ A B :0 1 2 3 6 8 9 11: 1 ♭2 2 ♭3 ♯4 ♯5 6 7: °ma13(♯11 add ♭9 ♭13)
namesBySequenceIndex[1548] = '''Aeoryllic'''
namesBySequenceIndexNegative[1548] = ''''''
semitonesBySequenceIndex[1548] = [0, 1, 2, 3, 6, 8, 9, 11]
poemBySequenceIndex[1548] = '''B D F G L N P S'''
poemAuthors[1548] = ''''''
#1549: ䷡䷝ Might Distanced: 1 1 1 3 2 1 2 1  : S R1 R2 G2 M2 D1 D2 N3 : ObIf

#1550: C D♭ D E♭ F♯ G♯ A♯ B :0 1 2 3 6 8 10 11: 1 ♭2 2 ♭3 ♯4 ♯5 ♯6 7: °ma9(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1549] = '''Katocryllic'''
namesBySequenceIndexNegative[1549] = ''''''
semitonesBySequenceIndex[1549] = [0, 1, 2, 3, 6, 8, 10, 11]
poemBySequenceIndex[1549] = '''B D F G L N R S'''
poemAuthors[1549] = ''''''
#1550: ䷡䷤ Might Family: 1 1 1 3 2 2 1 1  : S R1 R2 G2 M2 D1 N2 N3 : ObIm

#1551: C D♭ D E♭ G♭ A B♭ B :0 1 2 3 6 9 10 11: 1 ♭2 2 ♭3 ♭5 6 ♭7 7: °ma9(add ♭7 ♭9 13)
namesBySequenceIndex[1550] = '''Sonyllic'''
namesBySequenceIndexNegative[1550] = ''''''
semitonesBySequenceIndex[1550] = [0, 1, 2, 3, 6, 9, 10, 11]
poemBySequenceIndex[1550] = '''B D F G L P R S'''
poemAuthors[1550] = ''''''
#1551: ䷡䷘ Might Integrity: 1 1 1 3 3 1 1 1  : S R1 R2 G2 M2 D2 N2 N3 : ObAg

#1552: C D♭ D E♭ G A♭ A B♭ :0 1 2 3 7 8 9 10: 1 ♭2 2 ♭3 5 ♭6 6 ♭7: mi9(add ♭9 ♭13 13)
namesBySequenceIndex[1551] = '''Mycryllic'''
namesBySequenceIndexNegative[1551] = ''''''
semitonesBySequenceIndex[1551] = [0, 1, 2, 3, 7, 8, 9, 10]
poemBySequenceIndex[1551] = '''B D F G M N P R'''
poemAuthors[1551] = ''''''
#1552: ䷡䷛ Might Excess: 1 1 1 4 1 1 1 2  : S R1 R2 G2 P D1 D2 N2 : Obod

#1553: C D♭ D E♭ G A♭ A B :0 1 2 3 7 8 9 11: 1 ♭2 2 ♭3 5 ♭6 6 7: mima9(add ♭9 ♭13 13)
namesBySequenceIndex[1552] = '''Dygyllic'''
namesBySequenceIndexNegative[1552] = ''''''
semitonesBySequenceIndex[1552] = [0, 1, 2, 3, 7, 8, 9, 11]
poemBySequenceIndex[1552] = '''B D F G M N P S'''
poemAuthors[1552] = ''''''
#1553: ䷡䷱ Might Holding: 1 1 1 4 1 1 2 1  : S R1 R2 G2 P D1 D2 N3 : Obof

#1554: C D♭ D E♭ G A♭ B♭ B :0 1 2 3 7 8 10 11: 1 ♭2 2 ♭3 5 ♭6 ♭7 7: mima9(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1553] = '''Gyryllic'''
namesBySequenceIndexNegative[1553] = ''''''
semitonesBySequenceIndex[1553] = [0, 1, 2, 3, 7, 8, 10, 11]
poemBySequenceIndex[1553] = '''B D F G M N R S'''
poemAuthors[1553] = ''''''
#1554: ䷡䷸ Might Osmosis: 1 1 1 4 1 2 1 1  : S R1 R2 G2 P D1 N2 N3 : Obom

#1555: C D♭ D E♭ G A B♭ B :0 1 2 3 7 9 10 11: 1 ♭2 2 ♭3 5 6 ♭7 7: mima9(add ♭7 ♭9 13)
namesBySequenceIndex[1554] = '''Tholyllic'''
namesBySequenceIndexNegative[1554] = ''''''
semitonesBySequenceIndex[1554] = [0, 1, 2, 3, 7, 9, 10, 11]
poemBySequenceIndex[1554] = '''B D F G M P R S'''
poemAuthors[1554] = ''''''
#1555: ䷡䷅ Might Arguing: 1 1 1 4 2 1 1 1  : S R1 R2 G2 P D2 N2 N3 : Obeg

#1556: C D♭ D E♭ G♯ A B♭ B :0 1 2 3 8 9 10 11: 1 ♭2 2 ♭3 ♯5 6 ♭7 7: mima9(♯5 add ♭7 ♭9 13)
namesBySequenceIndex[1555] = ''''''
namesBySequenceIndexNegative[1555] = ''''''
semitonesBySequenceIndex[1555] = [0, 1, 2, 3, 8, 9, 10, 11]
poemBySequenceIndex[1555] = '''B D F G N P R S'''
poemAuthors[1555] = ''''''
#1556: ䷡䷠ Might Retiring: 1 1 1 5 1 1 1 1  : S R1 R2 G2 D1 D2 N2 N3 : Obig

#1557: C D♭ D E F F♯ G A♭ :0 1 2 4 5 6 7 8: 1 ♭2 2 3 4 ♯4 5 ♭6: 11(no 7 add ♭6 ♭9 ♯11)
namesBySequenceIndex[1556] = '''Thoptyllic'''
namesBySequenceIndexNegative[1556] = ''''''
semitonesBySequenceIndex[1556] = [0, 1, 2, 4, 5, 6, 7, 8]
poemBySequenceIndex[1556] = '''B D F J K L M N'''
poemAuthors[1556] = ''''''
#1557: ䷈䷊ Homegrown Peace: 1 1 2 1 1 1 1 4  : S R1 R2 G3 M1 M2 P D1 : OmOj

#1558: C D♭ D E F F♯ G A :0 1 2 4 5 6 7 9: 1 ♭2 2 3 4 ♯4 5 6: 13(no 7 add ♭9 ♯11)
namesBySequenceIndex[1557] = '''Katalyllic'''
namesBySequenceIndexNegative[1557] = ''''''
semitonesBySequenceIndex[1557] = [0, 1, 2, 4, 5, 6, 7, 9]
poemBySequenceIndex[1557] = '''B D F J K L M P'''
poemAuthors[1557] = ''''''
#1558: ䷈䷵ Homegrown Betrothing: 1 1 2 1 1 1 2 3  : S R1 R2 G3 M1 M2 P D2 : OmEb

#1559: C D♭ D E F F♯ G B♭ :0 1 2 4 5 6 7 10: 1 ♭2 2 3 4 ♯4 5 ♭7: 11(add ♭9 ♯11)
namesBySequenceIndex[1558] = '''Kolyllic'''
namesBySequenceIndexNegative[1558] = ''''''
semitonesBySequenceIndex[1558] = [0, 1, 2, 4, 5, 6, 7, 10]
poemBySequenceIndex[1558] = '''B D F J K L M R'''
poemAuthors[1558] = ''''''
#1559: ䷈䷻ Homegrown Composed: 1 1 2 1 1 1 3 2  : S R1 R2 G3 M1 M2 P N2 : OmEk

#1560: C D♭ D E F F♯ G B :0 1 2 4 5 6 7 11: 1 ♭2 2 3 4 ♯4 5 7: ma11(add ♭9 ♯11)
namesBySequenceIndex[1559] = '''Kogyllic'''
namesBySequenceIndexNegative[1559] = ''''''
semitonesBySequenceIndex[1559] = [0, 1, 2, 4, 5, 6, 7, 11]
poemBySequenceIndex[1559] = '''B D F J K L M S'''
poemAuthors[1559] = ''''''
#1560: ䷈䷨ Homegrown Diminishing: 1 1 2 1 1 1 4 1  : S R1 R2 G3 M1 M2 P N3 : OmEl

#1561: C D♭ D E F G♭ A♭ A :0 1 2 4 5 6 8 9: 1 ♭2 2 3 4 ♭5 ♭6 6: 13(♯5 no 7 add ♭9 ♯11)
namesBySequenceIndex[1560] = '''Sathyllic'''
namesBySequenceIndexNegative[1560] = ''''''
semitonesBySequenceIndex[1560] = [0, 1, 2, 4, 5, 6, 8, 9]
poemBySequenceIndex[1560] = '''B D F J K L N P'''
poemAuthors[1560] = ''''''
#1561: ䷈䷶ Homegrown Abounding: 1 1 2 1 1 2 1 3  : S R1 R2 G3 M1 M2 D1 N1 : OmIb

#1562: C D♭ D E F G♭ A♭ B♭ :0 1 2 4 5 6 8 10: 1 ♭2 2 3 4 ♭5 ♭6 ♭7: 11(♯5 add ♭9 ♯11)
namesBySequenceIndex[1561] = '''Syryllic'''
namesBySequenceIndexNegative[1561] = ''''''
semitonesBySequenceIndex[1561] = [0, 1, 2, 4, 5, 6, 8, 10]
poemBySequenceIndex[1561] = '''B D F J K L N R'''
poemAuthors[1561] = ''''''
#1562: ䷈䷾ Homegrown Completed: 1 1 2 1 1 2 2 2  : S R1 R2 G3 M1 M2 D1 N2 : OmIk

#1563: C D♭ D E F G♭ A♭ B :0 1 2 4 5 6 8 11: 1 ♭2 2 3 4 ♭5 ♭6 7: ma11(♯5 add ♭9 ♯11)
namesBySequenceIndex[1562] = ''''''
namesBySequenceIndexNegative[1562] = ''''''
semitonesBySequenceIndex[1562] = [0, 1, 2, 4, 5, 6, 8, 11]
poemBySequenceIndex[1562] = '''B D F J K L N S'''
poemAuthors[1562] = ''''''
#1563: ䷈䷕ Homegrown Adorning: 1 1 2 1 1 2 3 1  : S R1 R2 G3 M1 M2 D1 N3 : OmIl

#1564: C D♭ D E F G♭ A B♭ :0 1 2 4 5 6 9 10: 1 ♭2 2 3 4 ♭5 6 ♭7: 13(♭5 add ♭9)
namesBySequenceIndex[1563] = '''Katygyllic'''
namesBySequenceIndexNegative[1563] = ''''''
semitonesBySequenceIndex[1563] = [0, 1, 2, 4, 5, 6, 9, 10]
poemBySequenceIndex[1563] = '''B D F J K L P R'''
poemAuthors[1563] = ''''''
#1564: ䷈䷐ Homegrown Following: 1 1 2 1 1 3 1 2  : S R1 R2 G3 M1 M2 D2 N2 : OmAd

#1565: C D♭ D E F G♭ A B :0 1 2 4 5 6 9 11: 1 ♭2 2 3 4 ♭5 6 7: ma13(♭5 add ♭9)
namesBySequenceIndex[1564] = '''Rathyllic'''
namesBySequenceIndexNegative[1564] = ''''''
semitonesBySequenceIndex[1564] = [0, 1, 2, 4, 5, 6, 9, 11]
poemBySequenceIndex[1564] = '''B D F J K L P S'''
poemAuthors[1564] = ''''''
#1565: ䷈䷔ Homegrown Biting: 1 1 2 1 1 3 2 1  : S R1 R2 G3 M1 M2 D2 N3 : OmAf

#1566: C D♭ D E F G♭ A♯ B :0 1 2 4 5 6 10 11: 1 ♭2 2 3 4 ♭5 ♯6 7: ma11(♭5 add ♭7 ♭9)
namesBySequenceIndex[1565] = '''Bynyllic'''
namesBySequenceIndexNegative[1565] = ''''''
semitonesBySequenceIndex[1565] = [0, 1, 2, 4, 5, 6, 10, 11]
poemBySequenceIndex[1565] = '''B D F J K L R S'''
poemAuthors[1565] = ''''''
#1566: ䷈䷩ Homegrown Augmenting: 1 1 2 1 1 4 1 1  : S R1 R2 G3 M1 M2 D3 N3 : OmAm

#1567: C D♭ D E F G A♭ A :0 1 2 4 5 7 8 9: 1 ♭2 2 3 4 5 ♭6 6: 13(no 7 add ♭6 ♭9)
namesBySequenceIndex[1566] = '''Thogyllic'''
namesBySequenceIndexNegative[1566] = ''''''
semitonesBySequenceIndex[1566] = [0, 1, 2, 4, 5, 7, 8, 9]
poemBySequenceIndex[1566] = '''B D F J K M N P'''
poemAuthors[1566] = ''''''
#1567: ䷈䷟ Homegrown Persevering: 1 1 2 1 2 1 1 3  : S R1 R2 G3 M1 P D1 N1 : Omob

#1568: C D♭ D E F G A♭ B♭ :0 1 2 4 5 7 8 10: 1 ♭2 2 3 4 5 ♭6 ♭7: 11(add ♭9 ♭13)
namesBySequenceIndex[1567] = ''''''
namesBySequenceIndexNegative[1567] = ''''''
semitonesBySequenceIndex[1567] = [0, 1, 2, 4, 5, 7, 8, 10]
poemBySequenceIndex[1567] = '''B D F J K M N R'''
poemAuthors[1567] = ''''''
#1568: ䷈䷯ Homegrown Source: 1 1 2 1 2 1 2 2  : S R1 R2 G3 M1 P D1 N2 : Omok

#1569: C D♭ D E F G A♭ B :0 1 2 4 5 7 8 11: 1 ♭2 2 3 4 5 ♭6 7: ma11(add ♭9 ♭13)
namesBySequenceIndex[1568] = '''Phroptyllic'''
namesBySequenceIndexNegative[1568] = ''''''
semitonesBySequenceIndex[1568] = [0, 1, 2, 4, 5, 7, 8, 11]
poemBySequenceIndex[1568] = '''B D F J K M N S'''
poemAuthors[1568] = ''''''
#1569: ䷈䷑ Homegrown Correcting: 1 1 2 1 2 1 3 1  : S R1 R2 G3 M1 P D1 N3 : Omol

#1570: C D♭ D E F G A B♭ :0 1 2 4 5 7 9 10: 1 ♭2 2 3 4 5 6 ♭7: 13(add ♭9)
namesBySequenceIndex[1569] = '''Lyryllic'''
namesBySequenceIndexNegative[1569] = ''''''
semitonesBySequenceIndex[1569] = [0, 1, 2, 4, 5, 7, 9, 10]
poemBySequenceIndex[1569] = '''B D F J K M P R'''
poemAuthors[1569] = ''''''
#1570: ䷈䷮ Homegrown Confining: 1 1 2 1 2 2 1 2  : S R1 R2 G3 M1 P D2 N2 : Omed

#1571: C D♭ D E F G A B :0 1 2 4 5 7 9 11: 1 ♭2 2 3 4 5 6 7: ma13(add ♭9)
namesBySequenceIndex[1570] = '''Staptyllic'''
namesBySequenceIndexNegative[1570] = ''''''
semitonesBySequenceIndex[1570] = [0, 1, 2, 4, 5, 7, 9, 11]
poemBySequenceIndex[1570] = '''B D F J K M P S'''
poemAuthors[1570] = ''''''
#1571: ䷈䷿ Homegrown Unfinished: 1 1 2 1 2 2 2 1  : S R1 R2 G3 M1 P D2 N3 : Omef

#1572: C D♭ D E F G A♯ B :0 1 2 4 5 7 10 11: 1 ♭2 2 3 4 5 ♯6 7: ma11(add ♭7 ♭9)
namesBySequenceIndex[1571] = '''Daryllic'''
namesBySequenceIndexNegative[1571] = ''''''
semitonesBySequenceIndex[1571] = [0, 1, 2, 4, 5, 7, 10, 11]
poemBySequenceIndex[1571] = '''B D F J K M R S'''
poemAuthors[1571] = ''''''
#1572: ䷈䷺ Homegrown Dispersing: 1 1 2 1 2 3 1 1  : S R1 R2 G3 M1 P D3 N3 : Omem

#1573: C D♭ D E F G♯ A B♭ :0 1 2 4 5 8 9 10: 1 ♭2 2 3 4 ♯5 6 ♭7: 13(♯5 add ♭9)
namesBySequenceIndex[1572] = '''Thacryllic'''
namesBySequenceIndexNegative[1572] = ''''''
semitonesBySequenceIndex[1572] = [0, 1, 2, 4, 5, 8, 9, 10]
poemBySequenceIndex[1572] = '''B D F J K N P R'''
poemAuthors[1572] = ''''''
#1573: ䷈䷞ Homegrown Conjoining: 1 1 2 1 3 1 1 2  : S R1 R2 G3 M1 D1 D2 N2 : Omid

#1574: C D♭ D E F G♯ A B :0 1 2 4 5 8 9 11: 1 ♭2 2 3 4 ♯5 6 7: ma13(♯5 add ♭9)
namesBySequenceIndex[1573] = '''Stanyllic'''
namesBySequenceIndexNegative[1573] = ''''''
semitonesBySequenceIndex[1573] = [0, 1, 2, 4, 5, 8, 9, 11]
poemBySequenceIndex[1573] = '''B D F J K N P S'''
poemAuthors[1573] = ''''''
#1574: ䷈䷷ Homegrown Wanderer: 1 1 2 1 3 1 2 1  : S R1 R2 G3 M1 D1 D2 N3 : Omif

#1575: C D♭ D E F G♯ A♯ B :0 1 2 4 5 8 10 11: 1 ♭2 2 3 4 ♯5 ♯6 7: ma11(♯5 add ♭7 ♭9)
namesBySequenceIndex[1574] = '''Daptyllic'''
namesBySequenceIndexNegative[1574] = ''''''
semitonesBySequenceIndex[1574] = [0, 1, 2, 4, 5, 8, 10, 11]
poemBySequenceIndex[1574] = '''B D F J K N R S'''
poemAuthors[1574] = ''''''
#1575: ䷈䷴ Homegrown Infiltrating: 1 1 2 1 3 2 1 1  : S R1 R2 G3 M1 D1 N2 N3 : Omim

#1576: C D♭ D E F A B♭ B :0 1 2 4 5 9 10 11: 1 ♭2 2 3 4 6 ♭7 7: ma13(no 5 add ♭7 ♭9)
namesBySequenceIndex[1575] = '''Paptyllic'''
namesBySequenceIndexNegative[1575] = ''''''
semitonesBySequenceIndex[1575] = [0, 1, 2, 4, 5, 9, 10, 11]
poemBySequenceIndex[1575] = '''B D F J K P R S'''
poemAuthors[1575] = ''''''
#1576: ䷈䷋ Homegrown Obstruction: 1 1 2 1 4 1 1 1  : S R1 R2 G3 M1 D2 N2 N3 : Omag

#1577: C D♭ D E F♯ G A♭ A :0 1 2 4 6 7 8 9: 1 ♭2 2 3 ♯4 5 ♭6 6: 6/9(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1576] = '''Thocryllic'''
namesBySequenceIndexNegative[1576] = ''''''
semitonesBySequenceIndex[1576] = [0, 1, 2, 4, 6, 7, 8, 9]
poemBySequenceIndex[1576] = '''B D F J L M N P'''
poemAuthors[1576] = ''''''
#1577: ䷄䷡ Biding Might: 1 1 2 2 1 1 1 3  : S R1 R2 G3 M2 P D1 N1 : OkOb

#1578: C D♭ D E F♯ G A♭ B♭ :0 1 2 4 6 7 8 10: 1 ♭2 2 3 ♯4 5 ♭6 ♭7: 9(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1577] = '''Epidyllic'''
namesBySequenceIndexNegative[1577] = ''''''
semitonesBySequenceIndex[1577] = [0, 1, 2, 4, 6, 7, 8, 10]
poemBySequenceIndex[1577] = '''B D F J L M N R'''
poemAuthors[1577] = ''''''
#1578: ䷄䷄ Biding Biding : 1 1 2 2 1 1 2 2  : S R1 R2 G3 M2 P D1 N2 : OkOk

#1579: C D♭ D E F♯ G A♭ B :0 1 2 4 6 7 8 11: 1 ♭2 2 3 ♯4 5 ♭6 7: ma9(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1578] = '''Dathyllic'''
namesBySequenceIndexNegative[1578] = ''''''
semitonesBySequenceIndex[1578] = [0, 1, 2, 4, 6, 7, 8, 11]
poemBySequenceIndex[1578] = '''B D F J L M N S'''
poemAuthors[1578] = ''''''
#1579: ䷄䷙ Biding Cultivating: 1 1 2 2 1 1 3 1  : S R1 R2 G3 M2 P D1 N3 : OkOl

#1580: C D♭ D E F♯ G A B♭ :0 1 2 4 6 7 9 10: 1 ♭2 2 3 ♯4 5 6 ♭7: 13(♯11 add ♭9)
namesBySequenceIndex[1579] = '''Aeolyryllic'''
namesBySequenceIndexNegative[1579] = ''''''
semitonesBySequenceIndex[1579] = [0, 1, 2, 4, 6, 7, 9, 10]
poemBySequenceIndex[1579] = '''B D F J L M P R'''
poemAuthors[1579] = ''''''
#1580: ䷄䷹ Biding Joyous: 1 1 2 2 1 2 1 2  : S R1 R2 G3 M2 P D2 N2 : OkEd

#1581: C D♭ D E F♯ G A B :0 1 2 4 6 7 9 11: 1 ♭2 2 3 ♯4 5 6 7: ma13(♯11 add ♭9)
namesBySequenceIndex[1580] = '''Godyllic'''
namesBySequenceIndexNegative[1580] = ''''''
semitonesBySequenceIndex[1580] = [0, 1, 2, 4, 6, 7, 9, 11]
poemBySequenceIndex[1580] = '''B D F J L M P S'''
poemAuthors[1580] = ''''''
#1581: ䷄䷥ Biding Polarising: 1 1 2 2 1 2 2 1  : S R1 R2 G3 M2 P D2 N3 : OkEf

#1582: C D♭ D E F♯ G A♯ B :0 1 2 4 6 7 10 11: 1 ♭2 2 3 ♯4 5 ♯6 7: ma9(add ♭7 ♭9 ♯11)
namesBySequenceIndex[1581] = '''Phrathyllic'''
namesBySequenceIndexNegative[1581] = ''''''
semitonesBySequenceIndex[1581] = [0, 1, 2, 4, 6, 7, 10, 11]
poemBySequenceIndex[1581] = '''B D F J L M R S'''
poemAuthors[1581] = ''''''
#1582: ䷄䷼ Biding Truth: 1 1 2 2 1 3 1 1  : S R1 R2 G3 M2 P D3 N3 : OkEm

#1583: C D♭ D E F♯ G♯ A B♭ :0 1 2 4 6 8 9 10: 1 ♭2 2 3 ♯4 ♯5 6 ♭7: 13(♯5 ♯11 add ♭9)
namesBySequenceIndex[1582] = '''Mixoryllic'''
namesBySequenceIndexNegative[1582] = ''''''
semitonesBySequenceIndex[1582] = [0, 1, 2, 4, 6, 8, 9, 10]
poemBySequenceIndex[1582] = '''B D F J L N P R'''
poemAuthors[1582] = ''''''
#1583: ䷄䷰ Biding Revolution: 1 1 2 2 2 1 1 2  : S R1 R2 G3 M2 D1 D2 N2 : OkId

#1584: C D♭ D E F♯ G♯ A B :0 1 2 4 6 8 9 11: 1 ♭2 2 3 ♯4 ♯5 6 7: ma13(♯5 ♯11 add ♭9)
namesBySequenceIndex[1583] = '''Gothyllic'''
namesBySequenceIndexNegative[1583] = ''''''
semitonesBySequenceIndex[1583] = [0, 1, 2, 4, 6, 8, 9, 11]
poemBySequenceIndex[1583] = '''B D F J L N P S'''
poemAuthors[1583] = ''''''
#1584: ䷄䷝ Biding Distanced: 1 1 2 2 2 1 2 1  : S R1 R2 G3 M2 D1 D2 N3 : OkIf

#1585: C D♭ D E F♯ G♯ A♯ B :0 1 2 4 6 8 10 11: 1 ♭2 2 3 ♯4 ♯5 ♯6 7: ma9(♯5 add ♭7 ♭9 ♯11)
namesBySequenceIndex[1584] = '''Ionaptyllic'''
namesBySequenceIndexNegative[1584] = ''''''
semitonesBySequenceIndex[1584] = [0, 1, 2, 4, 6, 8, 10, 11]
poemBySequenceIndex[1584] = '''B D F J L N R S'''
poemAuthors[1584] = ''''''
#1585: ䷄䷤ Biding Family: 1 1 2 2 2 2 1 1  : S R1 R2 G3 M2 D1 N2 N3 : OkIm

#1586: C D♭ D E G♭ A B♭ B :0 1 2 4 6 9 10 11: 1 ♭2 2 3 ♭5 6 ♭7 7: ma9(♭5 add ♭7 ♭9 13)
namesBySequenceIndex[1585] = '''Aeonyllic'''
namesBySequenceIndexNegative[1585] = ''''''
semitonesBySequenceIndex[1585] = [0, 1, 2, 4, 6, 9, 10, 11]
poemBySequenceIndex[1585] = '''B D F J L P R S'''
poemAuthors[1585] = ''''''
#1586: ䷄䷘ Biding Integrity: 1 1 2 2 3 1 1 1  : S R1 R2 G3 M2 D2 N2 N3 : OkAg

#1587: C D♭ D E G A♭ A B♭ :0 1 2 4 7 8 9 10: 1 ♭2 2 3 5 ♭6 6 ♭7: 9(add ♭9 ♭13 13)
namesBySequenceIndex[1586] = ''''''
namesBySequenceIndexNegative[1586] = ''''''
semitonesBySequenceIndex[1586] = [0, 1, 2, 4, 7, 8, 9, 10]
poemBySequenceIndex[1586] = '''B D F J M N P R'''
poemAuthors[1586] = ''''''
#1587: ䷄䷛ Biding Excess: 1 1 2 3 1 1 1 2  : S R1 R2 G3 P D1 D2 N2 : Okod

#1588: C D♭ D E G A♭ A B :0 1 2 4 7 8 9 11: 1 ♭2 2 3 5 ♭6 6 7: ma9(add ♭9 ♭13 13)
namesBySequenceIndex[1587] = '''Madyllic'''
namesBySequenceIndexNegative[1587] = ''''''
semitonesBySequenceIndex[1587] = [0, 1, 2, 4, 7, 8, 9, 11]
poemBySequenceIndex[1587] = '''B D F J M N P S'''
poemAuthors[1587] = ''''''
#1588: ䷄䷱ Biding Holding: 1 1 2 3 1 1 2 1  : S R1 R2 G3 P D1 D2 N3 : Okof

#1589: C D♭ D E G A♭ B♭ B :0 1 2 4 7 8 10 11: 1 ♭2 2 3 5 ♭6 ♭7 7: ma9(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1588] = '''Rothyllic'''
namesBySequenceIndexNegative[1588] = ''''''
semitonesBySequenceIndex[1588] = [0, 1, 2, 4, 7, 8, 10, 11]
poemBySequenceIndex[1588] = '''B D F J M N R S'''
poemAuthors[1588] = ''''''
#1589: ䷄䷸ Biding Osmosis: 1 1 2 3 1 2 1 1  : S R1 R2 G3 P D1 N2 N3 : Okom

#1590: C D♭ D E G A B♭ B :0 1 2 4 7 9 10 11: 1 ♭2 2 3 5 6 ♭7 7: ma9(add ♭7 ♭9 13)
namesBySequenceIndex[1589] = ''''''
namesBySequenceIndexNegative[1589] = ''''''
semitonesBySequenceIndex[1589] = [0, 1, 2, 4, 7, 9, 10, 11]
poemBySequenceIndex[1589] = '''B D F J M P R S'''
poemAuthors[1589] = ''''''
#1590: ䷄䷅ Biding Arguing: 1 1 2 3 2 1 1 1  : S R1 R2 G3 P D2 N2 N3 : Okeg

#1591: C D♭ D E G♯ A B♭ B :0 1 2 4 8 9 10 11: 1 ♭2 2 3 ♯5 6 ♭7 7: ma9(♯5 add ♭7 ♭9 13)
namesBySequenceIndex[1590] = '''Eparyllic'''
namesBySequenceIndexNegative[1590] = ''''''
semitonesBySequenceIndex[1590] = [0, 1, 2, 4, 8, 9, 10, 11]
poemBySequenceIndex[1590] = '''B D F J N P R S'''
poemAuthors[1590] = ''''''
#1591: ䷄䷠ Biding Retiring: 1 1 2 4 1 1 1 1  : S R1 R2 G3 D1 D2 N2 N3 : Okig

#1592: C D♭ D F F♯ G A♭ A :0 1 2 5 6 7 8 9: 1 ♭2 2 4 ♯4 5 ♭6 6: 6/9sus4(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1591] = '''Ionodyllic'''
namesBySequenceIndexNegative[1591] = ''''''
semitonesBySequenceIndex[1591] = [0, 1, 2, 5, 6, 7, 8, 9]
poemBySequenceIndex[1591] = '''B D F K L M N P'''
poemAuthors[1591] = ''''''
#1592: ䷙䷡ Cultivating Might: 1 1 3 1 1 1 1 3  : S R1 G1 M1 M2 P D1 N1 : OlOb

#1593: C D♭ D F F♯ G A♭ B♭ :0 1 2 5 6 7 8 10: 1 ♭2 2 4 ♯4 5 ♭6 ♭7: 11sus♯4(add ♭9 ♭13)
namesBySequenceIndex[1592] = '''Styptyllic'''
namesBySequenceIndexNegative[1592] = ''''''
semitonesBySequenceIndex[1592] = [0, 1, 2, 5, 6, 7, 8, 10]
poemBySequenceIndex[1592] = '''B D F K L M N R'''
poemAuthors[1592] = ''''''
#1593: ䷙䷄ Cultivating Biding: 1 1 3 1 1 1 2 2  : S R1 G1 M1 M2 P D1 N2 : OlOk

#1594: C D♭ D F F♯ G A♭ B :0 1 2 5 6 7 8 11: 1 ♭2 2 4 ♯4 5 ♭6 7: ma11sus♯4(add ♭9 ♭13)
namesBySequenceIndex[1593] = '''Messiaen 4th Mode,Zygyllic'''
namesBySequenceIndexNegative[1593] = ''''''
semitonesBySequenceIndex[1593] = [0, 1, 2, 5, 6, 7, 8, 11]
poemBySequenceIndex[1593] = '''B D F K L M N S'''
poemAuthors[1593] = ''''''
#1594: ䷙䷙ Cultivating Cultivating : 1 1 3 1 1 1 3 1  : S R1 G1 M1 M2 P D1 N3 : OlOl

#1595: C D♭ D F F♯ G A B♭ :0 1 2 5 6 7 9 10: 1 ♭2 2 4 ♯4 5 6 ♭7: 13sus♯4(add ♭9)
namesBySequenceIndex[1594] = '''Dyryllic'''
namesBySequenceIndexNegative[1594] = ''''''
semitonesBySequenceIndex[1594] = [0, 1, 2, 5, 6, 7, 9, 10]
poemBySequenceIndex[1594] = '''B D F K L M P R'''
poemAuthors[1594] = ''''''
#1595: ䷙䷹ Cultivating Joyous: 1 1 3 1 1 2 1 2  : S R1 G1 M1 M2 P D2 N2 : OlEd

#1596: C D♭ D F F♯ G A B :0 1 2 5 6 7 9 11: 1 ♭2 2 4 ♯4 5 6 7: ma13sus♯4(add ♭9)
namesBySequenceIndex[1595] = '''Mixothyllic'''
namesBySequenceIndexNegative[1595] = ''''''
semitonesBySequenceIndex[1595] = [0, 1, 2, 5, 6, 7, 9, 11]
poemBySequenceIndex[1595] = '''B D F K L M P S'''
poemAuthors[1595] = ''''''
#1596: ䷙䷥ Cultivating Polarising: 1 1 3 1 1 2 2 1  : S R1 G1 M1 M2 P D2 N3 : OlEf

#1597: C D♭ D F F♯ G A♯ B :0 1 2 5 6 7 10 11: 1 ♭2 2 4 ♯4 5 ♯6 7: ma11sus♯4(add ♭7 ♭9)
namesBySequenceIndex[1596] = '''Mylyllic'''
namesBySequenceIndexNegative[1596] = ''''''
semitonesBySequenceIndex[1596] = [0, 1, 2, 5, 6, 7, 10, 11]
poemBySequenceIndex[1596] = '''B D F K L M R S'''
poemAuthors[1596] = ''''''
#1597: ䷙䷼ Cultivating Truth: 1 1 3 1 1 3 1 1  : S R1 G1 M1 M2 P D3 N3 : OlEm

#1598: C D♭ D F G♭ A♭ A B♭ :0 1 2 5 6 8 9 10: 1 ♭2 2 4 ♭5 ♭6 6 ♭7: 13sus♯4(♯5 add ♭9)
namesBySequenceIndex[1597] = '''Salyllic'''
namesBySequenceIndexNegative[1597] = ''''''
semitonesBySequenceIndex[1597] = [0, 1, 2, 5, 6, 8, 9, 10]
poemBySequenceIndex[1597] = '''B D F K L N P R'''
poemAuthors[1597] = ''''''
#1598: ䷙䷰ Cultivating Revolution: 1 1 3 1 2 1 1 2  : S R1 G1 M1 M2 D1 D2 N2 : OlId

#1599: C D♭ D F G♭ A♭ A B :0 1 2 5 6 8 9 11: 1 ♭2 2 4 ♭5 ♭6 6 7: ma13sus♯4(♯5 add ♭9)
namesBySequenceIndex[1598] = '''Molyllic'''
namesBySequenceIndexNegative[1598] = ''''''
semitonesBySequenceIndex[1598] = [0, 1, 2, 5, 6, 8, 9, 11]
poemBySequenceIndex[1598] = '''B D F K L N P S'''
poemAuthors[1598] = ''''''
#1599: ䷙䷝ Cultivating Distanced: 1 1 3 1 2 1 2 1  : S R1 G1 M1 M2 D1 D2 N3 : OlIf

#1600: C D♭ D F G♭ A♭ B♭ B :0 1 2 5 6 8 10 11: 1 ♭2 2 4 ♭5 ♭6 ♭7 7: ma11sus♯4(♯5 add ♭7 ♭9)
namesBySequenceIndex[1599] = '''Zyptyllic'''
namesBySequenceIndexNegative[1599] = ''''''
semitonesBySequenceIndex[1599] = [0, 1, 2, 5, 6, 8, 10, 11]
poemBySequenceIndex[1599] = '''B D F K L N R S'''
poemAuthors[1599] = ''''''
#1600: ䷙䷤ Cultivating Family: 1 1 3 1 2 2 1 1  : S R1 G1 M1 M2 D1 N2 N3 : OlIm

#1601: C D♭ D F G♭ A B♭ B :0 1 2 5 6 9 10 11: 1 ♭2 2 4 ♭5 6 ♭7 7: ma13sus♭2(♭5 add ♭7)
namesBySequenceIndex[1600] = '''Zonyllic'''
namesBySequenceIndexNegative[1600] = ''''''
semitonesBySequenceIndex[1600] = [0, 1, 2, 5, 6, 9, 10, 11]
poemBySequenceIndex[1600] = '''B D F K L P R S'''
poemAuthors[1600] = ''''''
#1601: ䷙䷘ Cultivating Integrity: 1 1 3 1 3 1 1 1  : S R1 G1 M1 M2 D2 N2 N3 : OlAg

#1602: C D♭ D F G A♭ A B♭ :0 1 2 5 7 8 9 10: 1 ♭2 2 4 5 ♭6 6 ♭7: 13sus♭2(add ♭13)
namesBySequenceIndex[1601] = '''Katolyllic'''
namesBySequenceIndexNegative[1601] = ''''''
semitonesBySequenceIndex[1601] = [0, 1, 2, 5, 7, 8, 9, 10]
poemBySequenceIndex[1601] = '''B D F K M N P R'''
poemAuthors[1601] = ''''''
#1602: ䷙䷛ Cultivating Excess: 1 1 3 2 1 1 1 2  : S R1 G1 M1 P D1 D2 N2 : Olod

#1603: C D♭ D F G A♭ A B :0 1 2 5 7 8 9 11: 1 ♭2 2 4 5 ♭6 6 7: ma13sus♭2(add ♭13)
namesBySequenceIndex[1602] = '''Zythyllic'''
namesBySequenceIndexNegative[1602] = ''''''
semitonesBySequenceIndex[1602] = [0, 1, 2, 5, 7, 8, 9, 11]
poemBySequenceIndex[1602] = '''B D F K M N P S'''
poemAuthors[1602] = ''''''
#1603: ䷙䷱ Cultivating Holding: 1 1 3 2 1 1 2 1  : S R1 G1 M1 P D1 D2 N3 : Olof

#1604: C D♭ D F G A♭ B♭ B :0 1 2 5 7 8 10 11: 1 ♭2 2 4 5 ♭6 ♭7 7: ma11sus♭2(add ♭7 ♭13)
namesBySequenceIndex[1603] = '''Banyllic'''
namesBySequenceIndexNegative[1603] = ''''''
semitonesBySequenceIndex[1603] = [0, 1, 2, 5, 7, 8, 10, 11]
poemBySequenceIndex[1603] = '''B D F K M N R S'''
poemAuthors[1603] = ''''''
#1604: ䷙䷸ Cultivating Osmosis: 1 1 3 2 1 2 1 1  : S R1 G1 M1 P D1 N2 N3 : Olom

#1605: C D♭ D F G A B♭ B :0 1 2 5 7 9 10 11: 1 ♭2 2 4 5 6 ♭7 7: ma13sus♭2(add ♭7)
namesBySequenceIndex[1604] = '''Aerathyllic'''
namesBySequenceIndexNegative[1604] = ''''''
semitonesBySequenceIndex[1604] = [0, 1, 2, 5, 7, 9, 10, 11]
poemBySequenceIndex[1604] = '''B D F K M P R S'''
poemAuthors[1604] = ''''''
#1605: ䷙䷅ Cultivating Arguing: 1 1 3 2 2 1 1 1  : S R1 G1 M1 P D2 N2 N3 : Oleg

#1606: C D♭ D F G♯ A B♭ B :0 1 2 5 8 9 10 11: 1 ♭2 2 4 ♯5 6 ♭7 7: ma13sus♭2(♯5 add ♭7)
namesBySequenceIndex[1605] = '''Pathyllic'''
namesBySequenceIndexNegative[1605] = ''''''
semitonesBySequenceIndex[1605] = [0, 1, 2, 5, 8, 9, 10, 11]
poemBySequenceIndex[1605] = '''B D F K N P R S'''
poemAuthors[1605] = ''''''
#1606: ䷙䷠ Cultivating Retiring: 1 1 3 3 1 1 1 1  : S R1 G1 M1 D1 D2 N2 N3 : Olig

#1607: C D♭ D F♯ G A♭ A B♭ :0 1 2 6 7 8 9 10: 1 ♭2 2 ♯4 5 ♭6 6 ♭7: 13sus♭2(♯11 add ♭13)
namesBySequenceIndex[1606] = '''Phryptyllic'''
namesBySequenceIndexNegative[1606] = ''''''
semitonesBySequenceIndex[1606] = [0, 1, 2, 6, 7, 8, 9, 10]
poemBySequenceIndex[1606] = '''B D F L M N P R'''
poemAuthors[1606] = ''''''
#1607: ䷊䷪ Peace Decision: 1 1 4 1 1 1 1 2  : S R1 G1 M2 P D1 D2 N2 : OjOd

#1608: C D♭ D F♯ G A♭ A B :0 1 2 6 7 8 9 11: 1 ♭2 2 ♯4 5 ♭6 6 7: ma13sus♭2(♯11 add ♭13)
namesBySequenceIndex[1607] = '''Laptyllic'''
namesBySequenceIndexNegative[1607] = ''''''
semitonesBySequenceIndex[1607] = [0, 1, 2, 6, 7, 8, 9, 11]
poemBySequenceIndex[1607] = '''B D F L M N P S'''
poemAuthors[1607] = ''''''
#1608: ䷊䷍ Peace Resevoir: 1 1 4 1 1 1 2 1  : S R1 G1 M2 P D1 D2 N3 : OjOf

#1609: C D♭ D F♯ G A♭ B♭ B :0 1 2 6 7 8 10 11: 1 ♭2 2 ♯4 5 ♭6 ♭7 7: ma9sus♯4(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1608] = '''Ronyllic'''
namesBySequenceIndexNegative[1608] = ''''''
semitonesBySequenceIndex[1608] = [0, 1, 2, 6, 7, 8, 10, 11]
poemBySequenceIndex[1608] = '''B D F L M N R S'''
poemAuthors[1608] = ''''''
#1609: ䷊䷈ Peace Homegrown: 1 1 4 1 1 2 1 1  : S R1 G1 M2 P D1 N2 N3 : OjOm

#1610: C D♭ D F♯ G A B♭ B :0 1 2 6 7 9 10 11: 1 ♭2 2 ♯4 5 6 ♭7 7: ma13sus♭2(♯11 add ♭7)
namesBySequenceIndex[1609] = '''Phrygyllic'''
namesBySequenceIndexNegative[1609] = ''''''
semitonesBySequenceIndex[1609] = [0, 1, 2, 6, 7, 9, 10, 11]
poemBySequenceIndex[1609] = '''B D F L M P R S'''
poemAuthors[1609] = ''''''
#1610: ䷊䷉ Peace Treading: 1 1 4 1 2 1 1 1  : S R1 G1 M2 P D2 N2 N3 : OjEg

#1611: C D♭ D F♯ G♯ A B♭ B :0 1 2 6 8 9 10 11: 1 ♭2 2 ♯4 ♯5 6 ♭7 7: ma13sus♭2(♯5 ♯11 add ♭7)
namesBySequenceIndex[1610] = '''Katyryllic'''
namesBySequenceIndexNegative[1610] = ''''''
semitonesBySequenceIndex[1610] = [0, 1, 2, 6, 8, 9, 10, 11]
poemBySequenceIndex[1610] = '''B D F L N P R S'''
poemAuthors[1610] = ''''''
#1611: ䷊䷌ Peace Fellowship: 1 1 4 2 1 1 1 1  : S R1 G1 M2 D1 D2 N2 N3 : OjIg

#1612: C D♭ D G A♭ A B♭ B :0 1 2 7 8 9 10 11: 1 ♭2 2 5 ♭6 6 ♭7 7: ma9sus♭2(add ♭7 ♭13 13)
namesBySequenceIndex[1611] = ''''''
namesBySequenceIndexNegative[1611] = ''''''
semitonesBySequenceIndex[1611] = [0, 1, 2, 7, 8, 9, 10, 11]
poemBySequenceIndex[1611] = '''B D F M N P R S'''
poemAuthors[1611] = ''''''
#1612: ䷊䷫ Peace Coupling: 1 1 5 1 1 1 1 1  : S R1 G1 P D1 D2 N2 N3 : Ojog

#1613: C D♭ D♯ E F F♯ G A♭ :0 1 3 4 5 6 7 8: 1 ♭2 ♯2 3 4 ♯4 5 ♭6: ma(add ♭6 ♭9 ♯9 11 ♯11)
namesBySequenceIndex[1612] = '''Moryllic'''
namesBySequenceIndexNegative[1612] = ''''''
semitonesBySequenceIndex[1612] = [0, 1, 3, 4, 5, 6, 7, 8]
poemBySequenceIndex[1612] = '''B D G J K L M N'''
poemAuthors[1612] = ''''''
#1613: ䷉䷊ Treading Peace: 1 2 1 1 1 1 1 4  : S R1 R3 G3 M1 M2 P D1 : EgOj

#1614: C D♭ D♯ E F F♯ G A :0 1 3 4 5 6 7 9: 1 ♭2 ♯2 3 4 ♯4 5 6: 6(add ♭9 ♯9 11 ♯11)
namesBySequenceIndex[1613] = '''Doryllic'''
namesBySequenceIndexNegative[1613] = ''''''
semitonesBySequenceIndex[1613] = [0, 1, 3, 4, 5, 6, 7, 9]
poemBySequenceIndex[1613] = '''B D G J K L M P'''
poemAuthors[1613] = ''''''
#1614: ䷉䷵ Treading Betrothing: 1 2 1 1 1 1 2 3  : S R1 R3 G3 M1 M2 P D2 : EgEb

#1615: C D♭ D♯ E F F♯ G B♭ :0 1 3 4 5 6 7 10: 1 ♭2 ♯2 3 4 ♯4 5 ♭7: 11(♭9 add ♯9 ♯11)
namesBySequenceIndex[1614] = '''Stagyllic'''
namesBySequenceIndexNegative[1614] = ''''''
semitonesBySequenceIndex[1614] = [0, 1, 3, 4, 5, 6, 7, 10]
poemBySequenceIndex[1614] = '''B D G J K L M R'''
poemAuthors[1614] = ''''''
#1615: ䷉䷻ Treading Composed: 1 2 1 1 1 1 3 2  : S R1 R3 G3 M1 M2 P N2 : EgEk

#1616: C D♭ D♯ E F F♯ G B :0 1 3 4 5 6 7 11: 1 ♭2 ♯2 3 4 ♯4 5 7: ma11(♭9 add ♯9 ♯11)
namesBySequenceIndex[1615] = '''Phraptyllic'''
namesBySequenceIndexNegative[1615] = ''''''
semitonesBySequenceIndex[1615] = [0, 1, 3, 4, 5, 6, 7, 11]
poemBySequenceIndex[1615] = '''B D G J K L M S'''
poemAuthors[1615] = ''''''
#1616: ䷉䷨ Treading Diminishing: 1 2 1 1 1 1 4 1  : S R1 R3 G3 M1 M2 P N3 : EgEl

#1617: C D♭ D♯ E F G♭ A♭ A :0 1 3 4 5 6 8 9: 1 ♭2 ♯2 3 4 ♭5 ♭6 6: 6(♯5 add ♭9 ♯9 11 ♯11)
namesBySequenceIndex[1616] = '''Ionilyllic'''
namesBySequenceIndexNegative[1616] = ''''''
semitonesBySequenceIndex[1616] = [0, 1, 3, 4, 5, 6, 8, 9]
poemBySequenceIndex[1616] = '''B D G J K L N P'''
poemAuthors[1616] = ''''''
#1617: ䷉䷶ Treading Abounding: 1 2 1 1 1 2 1 3  : S R1 R3 G3 M1 M2 D1 N1 : EgIb

#1618: C D♭ D♯ E F G♭ A♭ B♭ :0 1 3 4 5 6 8 10: 1 ♭2 ♯2 3 4 ♭5 ♭6 ♭7: 11(♯5 ♭9 add ♯9 ♯11)
namesBySequenceIndex[1617] = '''Esplá,Ahava Rabba (Jewish),Spanish Octatonic,Epinyllic'''
namesBySequenceIndexNegative[1617] = ''''''
semitonesBySequenceIndex[1617] = [0, 1, 3, 4, 5, 6, 8, 10]
poemBySequenceIndex[1617] = '''B D G J K L N R'''
poemAuthors[1617] = ''''''
#1618: ䷉䷾ Treading Completed: 1 2 1 1 1 2 2 2  : S R1 R3 G3 M1 M2 D1 N2 : EgIk

#1619: C D♭ D♯ E F G♭ A♭ B :0 1 3 4 5 6 8 11: 1 ♭2 ♯2 3 4 ♭5 ♭6 7: ma11(♯5 ♭9 add ♯9 ♯11)
namesBySequenceIndex[1618] = '''Katoryllic'''
namesBySequenceIndexNegative[1618] = ''''''
semitonesBySequenceIndex[1618] = [0, 1, 3, 4, 5, 6, 8, 11]
poemBySequenceIndex[1618] = '''B D G J K L N S'''
poemAuthors[1618] = ''''''
#1619: ䷉䷕ Treading Adorning: 1 2 1 1 1 2 3 1  : S R1 R3 G3 M1 M2 D1 N3 : EgIl

#1620: C D♭ D♯ E F G♭ A B♭ :0 1 3 4 5 6 9 10: 1 ♭2 ♯2 3 4 ♭5 6 ♭7: 13(♭5 ♭9 add ♯9)
namesBySequenceIndex[1619] = '''Maqam Shadd’araban,Maqam Shadd’araban (Iraq),Magyllic'''
namesBySequenceIndexNegative[1619] = ''''''
semitonesBySequenceIndex[1619] = [0, 1, 3, 4, 5, 6, 9, 10]
poemBySequenceIndex[1619] = '''B D G J K L P R'''
poemAuthors[1619] = ''''''
#1620: ䷉䷐ Treading Following: 1 2 1 1 1 3 1 2  : S R1 R3 G3 M1 M2 D2 N2 : EgAd

#1621: C D♭ D♯ E F G♭ A B :0 1 3 4 5 6 9 11: 1 ♭2 ♯2 3 4 ♭5 6 7: ma13(♭5 ♭9 add ♯9)
namesBySequenceIndex[1620] = '''Thodyllic'''
namesBySequenceIndexNegative[1620] = ''''''
semitonesBySequenceIndex[1620] = [0, 1, 3, 4, 5, 6, 9, 11]
poemBySequenceIndex[1620] = '''B D G J K L P S'''
poemAuthors[1620] = ''''''
#1621: ䷉䷔ Treading Biting: 1 2 1 1 1 3 2 1  : S R1 R3 G3 M1 M2 D2 N3 : EgAf

#1622: C D♭ D♯ E F G♭ A♯ B :0 1 3 4 5 6 10 11: 1 ♭2 ♯2 3 4 ♭5 ♯6 7: ma11(♭5 ♭9 add ♭7 ♯9)
namesBySequenceIndex[1621] = '''Raryllic'''
namesBySequenceIndexNegative[1621] = ''''''
semitonesBySequenceIndex[1621] = [0, 1, 3, 4, 5, 6, 10, 11]
poemBySequenceIndex[1621] = '''B D G J K L R S'''
poemAuthors[1621] = ''''''
#1622: ䷉䷩ Treading Augmenting: 1 2 1 1 1 4 1 1  : S R1 R3 G3 M1 M2 D3 N3 : EgAm

#1623: C D♭ D♯ E F G A♭ A :0 1 3 4 5 7 8 9: 1 ♭2 ♯2 3 4 5 ♭6 6: 6(add ♭6 ♭9 ♯9 11)
namesBySequenceIndex[1622] = '''Ionogyllic'''
namesBySequenceIndexNegative[1622] = ''''''
semitonesBySequenceIndex[1622] = [0, 1, 3, 4, 5, 7, 8, 9]
poemBySequenceIndex[1622] = '''B D G J K M N P'''
poemAuthors[1622] = ''''''
#1623: ䷉䷟ Treading Persevering: 1 2 1 1 2 1 1 3  : S R1 R3 G3 M1 P D1 N1 : Egob

#1624: C D♭ D♯ E F G A♭ B♭ :0 1 3 4 5 7 8 10: 1 ♭2 ♯2 3 4 5 ♭6 ♭7: 11(♭9 add ♯9 ♭13)
namesBySequenceIndex[1623] = '''Flamenco,Spanish Phrygian,Phrygian Add ♮3,Thydyllic'''
namesBySequenceIndexNegative[1623] = ''''''
semitonesBySequenceIndex[1623] = [0, 1, 3, 4, 5, 7, 8, 10]
poemBySequenceIndex[1623] = '''B D G J K M N R'''
poemAuthors[1623] = ''''''
#1624: ䷉䷯ Treading Source: 1 2 1 1 2 1 2 2  : S R1 R3 G3 M1 P D1 N2 : Egok

#1625: C D♭ D♯ E F G A♭ B :0 1 3 4 5 7 8 11: 1 ♭2 ♯2 3 4 5 ♭6 7: ma11(♭9 add ♯9 ♭13)
namesBySequenceIndex[1624] = '''Layllic'''
namesBySequenceIndexNegative[1624] = ''''''
semitonesBySequenceIndex[1624] = [0, 1, 3, 4, 5, 7, 8, 11]
poemBySequenceIndex[1624] = '''B D G J K M N S'''
poemAuthors[1624] = ''''''
#1625: ䷉䷑ Treading Correcting: 1 2 1 1 2 1 3 1  : S R1 R3 G3 M1 P D1 N3 : Egol

#1626: C D♭ D♯ E F G A B♭ :0 1 3 4 5 7 9 10: 1 ♭2 ♯2 3 4 5 6 ♭7: 13(♭9 add ♯9)
namesBySequenceIndex[1625] = '''LG Octatonic,Poryllic'''
namesBySequenceIndexNegative[1625] = ''''''
semitonesBySequenceIndex[1625] = [0, 1, 3, 4, 5, 7, 9, 10]
poemBySequenceIndex[1625] = '''B D G J K M P R'''
poemAuthors[1625] = ''''''
#1626: ䷉䷮ Treading Confining: 1 2 1 1 2 2 1 2  : S R1 R3 G3 M1 P D2 N2 : Eged

#1627: C D♭ D♯ E F G A B :0 1 3 4 5 7 9 11: 1 ♭2 ♯2 3 4 5 6 7: ma13(♭9 add ♯9)
namesBySequenceIndex[1626] = '''Stythyllic'''
namesBySequenceIndexNegative[1626] = ''''''
semitonesBySequenceIndex[1626] = [0, 1, 3, 4, 5, 7, 9, 11]
poemBySequenceIndex[1626] = '''B D G J K M P S'''
poemAuthors[1626] = ''''''
#1627: ䷉䷿ Treading Unfinished: 1 2 1 1 2 2 2 1  : S R1 R3 G3 M1 P D2 N3 : Egef

#1628: C D♭ D♯ E F G A♯ B :0 1 3 4 5 7 10 11: 1 ♭2 ♯2 3 4 5 ♯6 7: ma11(♭9 add ♭7 ♯9)
namesBySequenceIndex[1627] = ''''''
namesBySequenceIndexNegative[1627] = ''''''
semitonesBySequenceIndex[1627] = [0, 1, 3, 4, 5, 7, 10, 11]
poemBySequenceIndex[1627] = '''B D G J K M R S'''
poemAuthors[1627] = ''''''
#1628: ䷉䷺ Treading Dispersing: 1 2 1 1 2 3 1 1  : S R1 R3 G3 M1 P D3 N3 : Egem

#1629: C D♭ D♯ E F G♯ A B♭ :0 1 3 4 5 8 9 10: 1 ♭2 ♯2 3 4 ♯5 6 ♭7: 13(♯5 ♭9 add ♯9)
namesBySequenceIndex[1628] = '''Zacryllic'''
namesBySequenceIndexNegative[1628] = ''''''
semitonesBySequenceIndex[1628] = [0, 1, 3, 4, 5, 8, 9, 10]
poemBySequenceIndex[1628] = '''B D G J K N P R'''
poemAuthors[1628] = ''''''
#1629: ䷉䷞ Treading Conjoining: 1 2 1 1 3 1 1 2  : S R1 R3 G3 M1 D1 D2 N2 : Egid

#1630: C D♭ D♯ E F G♯ A B :0 1 3 4 5 8 9 11: 1 ♭2 ♯2 3 4 ♯5 6 7: ma13(♯5 ♭9 add ♯9)
namesBySequenceIndex[1629] = '''Ganyllic'''
namesBySequenceIndexNegative[1629] = ''''''
semitonesBySequenceIndex[1629] = [0, 1, 3, 4, 5, 8, 9, 11]
poemBySequenceIndex[1629] = '''B D G J K N P S'''
poemAuthors[1629] = ''''''
#1630: ䷉䷷ Treading Wanderer: 1 2 1 1 3 1 2 1  : S R1 R3 G3 M1 D1 D2 N3 : Egif

#1631: C D♭ D♯ E F G♯ A♯ B :0 1 3 4 5 8 10 11: 1 ♭2 ♯2 3 4 ♯5 ♯6 7: ma11(♯5 ♭9 add ♭7 ♯9)
namesBySequenceIndex[1630] = '''Aeryptyllic'''
namesBySequenceIndexNegative[1630] = ''''''
semitonesBySequenceIndex[1630] = [0, 1, 3, 4, 5, 8, 10, 11]
poemBySequenceIndex[1630] = '''B D G J K N R S'''
poemAuthors[1630] = ''''''
#1631: ䷉䷴ Treading Infiltrating: 1 2 1 1 3 2 1 1  : S R1 R3 G3 M1 D1 N2 N3 : Egim

#1632: C D♭ D♯ E F A B♭ B :0 1 3 4 5 9 10 11: 1 ♭2 ♯2 3 4 6 ♭7 7: ma13(♭9 no 5 add ♭7 ♯9)
namesBySequenceIndex[1631] = '''Kydyllic'''
namesBySequenceIndexNegative[1631] = ''''''
semitonesBySequenceIndex[1631] = [0, 1, 3, 4, 5, 9, 10, 11]
poemBySequenceIndex[1631] = '''B D G J K P R S'''
poemAuthors[1631] = ''''''
#1632: ䷉䷋ Treading Obstruction: 1 2 1 1 4 1 1 1  : S R1 R3 G3 M1 D2 N2 N3 : Egag

#1633: C D♭ D♯ E F♯ G A♭ A :0 1 3 4 6 7 8 9: 1 ♭2 ♯2 3 ♯4 5 ♭6 6: 6(add ♭6 ♭9 ♯9 ♯11)
namesBySequenceIndex[1632] = '''Aeraptyllic'''
namesBySequenceIndexNegative[1632] = ''''''
semitonesBySequenceIndex[1632] = [0, 1, 3, 4, 6, 7, 8, 9]
poemBySequenceIndex[1632] = '''B D G J L M N P'''
poemAuthors[1632] = ''''''
#1633: ䷹䷡ Joyous Might: 1 2 1 2 1 1 1 3  : S R1 R3 G3 M2 P D1 N1 : EdOb

#1634: C D♭ D♯ E F♯ G A♭ B♭ :0 1 3 4 6 7 8 10: 1 ♭2 ♯2 3 ♯4 5 ♭6 ♭7: 7(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1633] = '''Stonyllic'''
namesBySequenceIndexNegative[1633] = ''''''
semitonesBySequenceIndex[1633] = [0, 1, 3, 4, 6, 7, 8, 10]
poemBySequenceIndex[1633] = '''B D G J L M N R'''
poemAuthors[1633] = ''''''
#1634: ䷹䷄ Joyous Biding: 1 2 1 2 1 1 2 2  : S R1 R3 G3 M2 P D1 N2 : EdOk

#1635: C D♭ D♯ E F♯ G A♭ B :0 1 3 4 6 7 8 11: 1 ♭2 ♯2 3 ♯4 5 ♭6 7: ma7(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1634] = '''Rygyllic'''
namesBySequenceIndexNegative[1634] = ''''''
semitonesBySequenceIndex[1634] = [0, 1, 3, 4, 6, 7, 8, 11]
poemBySequenceIndex[1634] = '''B D G J L M N S'''
poemAuthors[1634] = ''''''
#1635: ䷹䷙ Joyous Cultivating: 1 2 1 2 1 1 3 1  : S R1 R3 G3 M2 P D1 N3 : EdOl

#1636: C D♭ D♯ E F♯ G A B♭ :0 1 3 4 6 7 9 10: 1 ♭2 ♯2 3 ♯4 5 6 ♭7: 13(♭9 ♯11 add ♯9)
namesBySequenceIndex[1635] = '''Diminished Dominant,Diminished Half-Tone,Messiaen 2nd Mode,Alpha Chord,Semitone-Tone,Distance Model 1 2,Ladyllic'''
namesBySequenceIndexNegative[1635] = ''''''
semitonesBySequenceIndex[1635] = [0, 1, 3, 4, 6, 7, 9, 10]
poemBySequenceIndex[1635] = '''B D G J L M P R'''
poemAuthors[1635] = ''''''
#1636: ䷹䷹ Joyous Joyous : 1 2 1 2 1 2 1 2  : S R1 R3 G3 M2 P D2 N2 : EdEd

#1637: C D♭ D♯ E F♯ G A B :0 1 3 4 6 7 9 11: 1 ♭2 ♯2 3 ♯4 5 6 7: ma13(♭9 ♯11 add ♯9)
namesBySequenceIndex[1636] = '''Shostakovich'''
namesBySequenceIndexNegative[1636] = ''''''
semitonesBySequenceIndex[1636] = [0, 1, 3, 4, 6, 7, 9, 11]
poemBySequenceIndex[1636] = '''B D G J L M P S'''
poemAuthors[1636] = ''''''
#1637: ䷹䷥ Joyous Polarising: 1 2 1 2 1 2 2 1  : S R1 R3 G3 M2 P D2 N3 : EdEf

#1638: C D♭ D♯ E F♯ G A♯ B :0 1 3 4 6 7 10 11: 1 ♭2 ♯2 3 ♯4 5 ♯6 7: ma7(add ♭7 ♭9 ♯9 ♯11)
namesBySequenceIndex[1637] = '''Kodyllic'''
namesBySequenceIndexNegative[1637] = ''''''
semitonesBySequenceIndex[1637] = [0, 1, 3, 4, 6, 7, 10, 11]
poemBySequenceIndex[1637] = '''B D G J L M R S'''
poemAuthors[1637] = ''''''
#1638: ䷹䷼ Joyous Truth: 1 2 1 2 1 3 1 1  : S R1 R3 G3 M2 P D3 N3 : EdEm

#1639: C D♭ D♯ E F♯ G♯ A B♭ :0 1 3 4 6 8 9 10: 1 ♭2 ♯2 3 ♯4 ♯5 6 ♭7: 13(♯5 ♭9 ♯11 add ♯9)
namesBySequenceIndex[1638] = ''''''
namesBySequenceIndexNegative[1638] = ''''''
semitonesBySequenceIndex[1638] = [0, 1, 3, 4, 6, 8, 9, 10]
poemBySequenceIndex[1638] = '''B D G J L N P R'''
poemAuthors[1638] = ''''''
#1639: ䷹䷰ Joyous Revolution: 1 2 1 2 2 1 1 2  : S R1 R3 G3 M2 D1 D2 N2 : EdId

#1640: C D♭ D♯ E F♯ G♯ A B :0 1 3 4 6 8 9 11: 1 ♭2 ♯2 3 ♯4 ♯5 6 7: ma13(♯5 ♭9 ♯11 add ♯9)
namesBySequenceIndex[1639] = '''Magen Abot,Mogyllic'''
namesBySequenceIndexNegative[1639] = ''''''
semitonesBySequenceIndex[1639] = [0, 1, 3, 4, 6, 8, 9, 11]
poemBySequenceIndex[1639] = '''B D G J L N P S'''
poemAuthors[1639] = ''''''
#1640: ䷹䷝ Joyous Distanced: 1 2 1 2 2 1 2 1  : S R1 R3 G3 M2 D1 D2 N3 : EdIf

#1641: C D♭ D♯ E F♯ G♯ A♯ B :0 1 3 4 6 8 10 11: 1 ♭2 ♯2 3 ♯4 ♯5 ♯6 7: ma7(♯5 add ♭7 ♭9 ♯9 ♯11)
namesBySequenceIndex[1640] = '''Danyllic'''
namesBySequenceIndexNegative[1640] = ''''''
semitonesBySequenceIndex[1640] = [0, 1, 3, 4, 6, 8, 10, 11]
poemBySequenceIndex[1640] = '''B D G J L N R S'''
poemAuthors[1640] = ''''''
#1641: ䷹䷤ Joyous Family: 1 2 1 2 2 2 1 1  : S R1 R3 G3 M2 D1 N2 N3 : EdIm

#1642: C D♭ D♯ E G♭ A B♭ B :0 1 3 4 6 9 10 11: 1 ♭2 ♯2 3 ♭5 6 ♭7 7: ma7(♭5 add ♭7 ♭9 ♯9 13)
namesBySequenceIndex[1641] = '''Monyllic'''
namesBySequenceIndexNegative[1641] = ''''''
semitonesBySequenceIndex[1641] = [0, 1, 3, 4, 6, 9, 10, 11]
poemBySequenceIndex[1641] = '''B D G J L P R S'''
poemAuthors[1641] = ''''''
#1642: ䷹䷘ Joyous Integrity: 1 2 1 2 3 1 1 1  : S R1 R3 G3 M2 D2 N2 N3 : EdAg

#1643: C D♭ D♯ E G A♭ A B♭ :0 1 3 4 7 8 9 10: 1 ♭2 ♯2 3 5 ♭6 6 ♭7: 7(add ♭9 ♯9 ♭13 13)
namesBySequenceIndex[1642] = '''Ionoyllic'''
namesBySequenceIndexNegative[1642] = ''''''
semitonesBySequenceIndex[1642] = [0, 1, 3, 4, 7, 8, 9, 10]
poemBySequenceIndex[1642] = '''B D G J M N P R'''
poemAuthors[1642] = ''''''
#1643: ䷹䷛ Joyous Excess: 1 2 1 3 1 1 1 2  : S R1 R3 G3 P D1 D2 N2 : Edod

#1644: C D♭ D♯ E G A♭ A B :0 1 3 4 7 8 9 11: 1 ♭2 ♯2 3 5 ♭6 6 7: ma7(add ♭9 ♯9 ♭13 13)
namesBySequenceIndex[1643] = '''Aeolynyllic'''
namesBySequenceIndexNegative[1643] = ''''''
semitonesBySequenceIndex[1643] = [0, 1, 3, 4, 7, 8, 9, 11]
poemBySequenceIndex[1643] = '''B D G J M N P S'''
poemAuthors[1643] = ''''''
#1644: ䷹䷱ Joyous Holding: 1 2 1 3 1 1 2 1  : S R1 R3 G3 P D1 D2 N3 : Edof

#1645: C D♭ D♯ E G A♭ B♭ B :0 1 3 4 7 8 10 11: 1 ♭2 ♯2 3 5 ♭6 ♭7 7: ma7(add ♭7 ♭9 ♯9 ♭13)
namesBySequenceIndex[1644] = '''Mixotharyllic'''
namesBySequenceIndexNegative[1644] = ''''''
semitonesBySequenceIndex[1644] = [0, 1, 3, 4, 7, 8, 10, 11]
poemBySequenceIndex[1644] = '''B D G J M N R S'''
poemAuthors[1644] = ''''''
#1645: ䷹䷸ Joyous Osmosis: 1 2 1 3 1 2 1 1  : S R1 R3 G3 P D1 N2 N3 : Edom

#1646: C D♭ D♯ E G A B♭ B :0 1 3 4 7 9 10 11: 1 ♭2 ♯2 3 5 6 ♭7 7: ma7(add ♭7 ♭9 ♯9 13)
namesBySequenceIndex[1645] = '''Ioninyllic'''
namesBySequenceIndexNegative[1645] = ''''''
semitonesBySequenceIndex[1645] = [0, 1, 3, 4, 7, 9, 10, 11]
poemBySequenceIndex[1645] = '''B D G J M P R S'''
poemAuthors[1645] = ''''''
#1646: ䷹䷅ Joyous Arguing: 1 2 1 3 2 1 1 1  : S R1 R3 G3 P D2 N2 N3 : Edeg

#1647: C D♭ D♯ E G♯ A B♭ B :0 1 3 4 8 9 10 11: 1 ♭2 ♯2 3 ♯5 6 ♭7 7: ma7(♯5 add ♭7 ♭9 ♯9 13)
namesBySequenceIndex[1646] = '''Storyllic'''
namesBySequenceIndexNegative[1646] = ''''''
semitonesBySequenceIndex[1646] = [0, 1, 3, 4, 8, 9, 10, 11]
poemBySequenceIndex[1646] = '''B D G J N P R S'''
poemAuthors[1646] = ''''''
#1647: ䷹䷠ Joyous Retiring: 1 2 1 4 1 1 1 1  : S R1 R3 G3 D1 D2 N2 N3 : Edig

#1648: C D♭ E♭ F F♯ G A♭ A :0 1 3 5 6 7 8 9: 1 ♭2 ♭3 4 ♯4 5 ♭6 6: mi6(add ♭6 ♭9 11 ♯11)
namesBySequenceIndex[1647] = '''Ionyryllic'''
namesBySequenceIndexNegative[1647] = ''''''
semitonesBySequenceIndex[1647] = [0, 1, 3, 5, 6, 7, 8, 9]
poemBySequenceIndex[1647] = '''B D G K L M N P'''
poemAuthors[1647] = ''''''
#1648: ䷥䷡ Polarising Might: 1 2 2 1 1 1 1 3  : S R1 G2 M1 M2 P D1 N1 : EfOb

#1649: C D♭ E♭ F F♯ G A♭ B♭ :0 1 3 5 6 7 8 10: 1 ♭2 ♭3 4 ♯4 5 ♭6 ♭7: mi11(♭9 add ♯11 ♭13)
namesBySequenceIndex[1648] = '''Bebop Locrian,Phrygian Locrian,Solyllic'''
namesBySequenceIndexNegative[1648] = ''''''
semitonesBySequenceIndex[1648] = [0, 1, 3, 5, 6, 7, 8, 10]
poemBySequenceIndex[1648] = '''B D G K L M N R'''
poemAuthors[1648] = ''''''
#1649: ䷥䷄ Polarising Biding: 1 2 2 1 1 1 2 2  : S R1 G2 M1 M2 P D1 N2 : EfOk

#1650: C D♭ E♭ F F♯ G A♭ B :0 1 3 5 6 7 8 11: 1 ♭2 ♭3 4 ♯4 5 ♭6 7: mima11(♭9 add ♯11 ♭13)
namesBySequenceIndex[1649] = '''Bebop Half-Diminished,Lacryllic'''
namesBySequenceIndexNegative[1649] = ''''''
semitonesBySequenceIndex[1649] = [0, 1, 3, 5, 6, 7, 8, 11]
poemBySequenceIndex[1649] = '''B D G K L M N S'''
poemAuthors[1649] = ''''''
#1650: ䷥䷙ Polarising Cultivating: 1 2 2 1 1 1 3 1  : S R1 G2 M1 M2 P D1 N3 : EfOl

#1651: C D♭ E♭ F F♯ G A B♭ :0 1 3 5 6 7 9 10: 1 ♭2 ♭3 4 ♯4 5 6 ♭7: mi13(♭9 add ♯11)
namesBySequenceIndex[1650] = ''''''
namesBySequenceIndexNegative[1650] = ''''''
semitonesBySequenceIndex[1650] = [0, 1, 3, 5, 6, 7, 9, 10]
poemBySequenceIndex[1650] = '''B D G K L M P R'''
poemAuthors[1650] = ''''''
#1651: ䷥䷹ Polarising Joyous: 1 2 2 1 1 2 1 2  : S R1 G2 M1 M2 P D2 N2 : EfEd

#1652: C D♭ E♭ F F♯ G A B :0 1 3 5 6 7 9 11: 1 ♭2 ♭3 4 ♯4 5 6 7: mima13(♭9 add ♯11)
namesBySequenceIndex[1651] = '''Van Der Host,Kaptyllic'''
namesBySequenceIndexNegative[1651] = ''''''
semitonesBySequenceIndex[1651] = [0, 1, 3, 5, 6, 7, 9, 11]
poemBySequenceIndex[1651] = '''B D G K L M P S'''
poemAuthors[1651] = ''''''
#1652: ䷥䷥ Polarising Polarising : 1 2 2 1 1 2 2 1  : S R1 G2 M1 M2 P D2 N3 : EfEf

#1653: C D♭ E♭ F F♯ G A♯ B :0 1 3 5 6 7 10 11: 1 ♭2 ♭3 4 ♯4 5 ♯6 7: mima11(♭9 add ♭7 ♯11)
namesBySequenceIndex[1652] = '''Boptyllic'''
namesBySequenceIndexNegative[1652] = ''''''
semitonesBySequenceIndex[1652] = [0, 1, 3, 5, 6, 7, 10, 11]
poemBySequenceIndex[1652] = '''B D G K L M R S'''
poemAuthors[1652] = ''''''
#1653: ䷥䷼ Polarising Truth: 1 2 2 1 1 3 1 1  : S R1 G2 M1 M2 P D3 N3 : EfEm

#1654: C D♭ E♭ F G♭ A♭ A B♭ :0 1 3 5 6 8 9 10: 1 ♭2 ♭3 4 ♭5 ♭6 6 ♭7: ⌀13(♭9 add ♭13)
namesBySequenceIndex[1653] = '''Moptyllic'''
namesBySequenceIndexNegative[1653] = ''''''
semitonesBySequenceIndex[1653] = [0, 1, 3, 5, 6, 8, 9, 10]
poemBySequenceIndex[1653] = '''B D G K L N P R'''
poemAuthors[1653] = ''''''
#1654: ䷥䷰ Polarising Revolution: 1 2 2 1 2 1 1 2  : S R1 G2 M1 M2 D1 D2 N2 : EfId

#1655: C D♭ E♭ F G♭ A♭ A B :0 1 3 5 6 8 9 11: 1 ♭2 ♭3 4 ♭5 ♭6 6 7: °ma13(♭9 add ♭13)
namesBySequenceIndex[1654] = '''Baryllic'''
namesBySequenceIndexNegative[1654] = ''''''
semitonesBySequenceIndex[1654] = [0, 1, 3, 5, 6, 8, 9, 11]
poemBySequenceIndex[1654] = '''B D G K L N P S'''
poemAuthors[1654] = ''''''
#1655: ䷥䷝ Polarising Distanced: 1 2 2 1 2 1 2 1  : S R1 G2 M1 M2 D1 D2 N3 : EfIf

#1656: C D♭ E♭ F G♭ A♭ B♭ B :0 1 3 5 6 8 10 11: 1 ♭2 ♭3 4 ♭5 ♭6 ♭7 7: °ma11(♭9 add ♭7 ♭13)
namesBySequenceIndex[1655] = '''Prokofiev,Epiphyllic'''
namesBySequenceIndexNegative[1655] = ''''''
semitonesBySequenceIndex[1655] = [0, 1, 3, 5, 6, 8, 10, 11]
poemBySequenceIndex[1655] = '''B D G K L N R S'''
poemAuthors[1655] = ''''''
#1656: ䷥䷤ Polarising Family: 1 2 2 1 2 2 1 1  : S R1 G2 M1 M2 D1 N2 N3 : EfIm

#1657: C D♭ E♭ F G♭ A B♭ B :0 1 3 5 6 9 10 11: 1 ♭2 ♭3 4 ♭5 6 ♭7 7: °ma13(♭9 add ♭7)
namesBySequenceIndex[1656] = '''Badyllic'''
namesBySequenceIndexNegative[1656] = ''''''
semitonesBySequenceIndex[1656] = [0, 1, 3, 5, 6, 9, 10, 11]
poemBySequenceIndex[1656] = '''B D G K L P R S'''
poemAuthors[1656] = ''''''
#1657: ䷥䷘ Polarising Integrity: 1 2 2 1 3 1 1 1  : S R1 G2 M1 M2 D2 N2 N3 : EfAg

#1658: C D♭ E♭ F G A♭ A B♭ :0 1 3 5 7 8 9 10: 1 ♭2 ♭3 4 5 ♭6 6 ♭7: mi13(♭9 add ♭13)
namesBySequenceIndex[1657] = '''Epocryllic'''
namesBySequenceIndexNegative[1657] = ''''''
semitonesBySequenceIndex[1657] = [0, 1, 3, 5, 7, 8, 9, 10]
poemBySequenceIndex[1657] = '''B D G K M N P R'''
poemAuthors[1657] = ''''''
#1658: ䷥䷛ Polarising Excess: 1 2 2 2 1 1 1 2  : S R1 G2 M1 P D1 D2 N2 : Efod

#1659: C D♭ E♭ F G A♭ A B :0 1 3 5 7 8 9 11: 1 ♭2 ♭3 4 5 ♭6 6 7: mima13(♭9 add ♭13)
namesBySequenceIndex[1658] = '''Thanyllic'''
namesBySequenceIndexNegative[1658] = ''''''
semitonesBySequenceIndex[1658] = [0, 1, 3, 5, 7, 8, 9, 11]
poemBySequenceIndex[1658] = '''B D G K M N P S'''
poemAuthors[1658] = ''''''
#1659: ䷥䷱ Polarising Holding: 1 2 2 2 1 1 2 1  : S R1 G2 M1 P D1 D2 N3 : Efof

#1660: C D♭ E♭ F G A♭ B♭ B :0 1 3 5 7 8 10 11: 1 ♭2 ♭3 4 5 ♭6 ♭7 7: mima11(♭9 add ♭7 ♭13)
namesBySequenceIndex[1659] = '''Hamel,Gregorian 3,Lythyllic'''
namesBySequenceIndexNegative[1659] = ''''''
semitonesBySequenceIndex[1659] = [0, 1, 3, 5, 7, 8, 10, 11]
poemBySequenceIndex[1659] = '''B D G K M N R S'''
poemAuthors[1659] = ''''''
#1660: ䷥䷸ Polarising Osmosis: 1 2 2 2 1 2 1 1  : S R1 G2 M1 P D1 N2 N3 : Efom

#1661: C D♭ E♭ F G A B♭ B :0 1 3 5 7 9 10 11: 1 ♭2 ♭3 4 5 6 ♭7 7: mima13(♭9 add ♭7)
namesBySequenceIndex[1660] = '''Phryryllic'''
namesBySequenceIndexNegative[1660] = ''''''
semitonesBySequenceIndex[1660] = [0, 1, 3, 5, 7, 9, 10, 11]
poemBySequenceIndex[1660] = '''B D G K M P R S'''
poemAuthors[1660] = ''''''
#1661: ䷥䷅ Polarising Arguing: 1 2 2 2 2 1 1 1  : S R1 G2 M1 P D2 N2 N3 : Efeg

#1662: C D♭ E♭ F G♯ A B♭ B :0 1 3 5 8 9 10 11: 1 ♭2 ♭3 4 ♯5 6 ♭7 7: mima13(♯5 ♭9 add ♭7)
namesBySequenceIndex[1661] = '''Kyryllic'''
namesBySequenceIndexNegative[1661] = ''''''
semitonesBySequenceIndex[1661] = [0, 1, 3, 5, 8, 9, 10, 11]
poemBySequenceIndex[1661] = '''B D G K N P R S'''
poemAuthors[1661] = ''''''
#1662: ䷥䷠ Polarising Retiring: 1 2 2 3 1 1 1 1  : S R1 G2 M1 D1 D2 N2 N3 : Efig

#1663: C D♭ E♭ F♯ G A♭ A B♭ :0 1 3 6 7 8 9 10: 1 ♭2 ♭3 ♯4 5 ♭6 6 ♭7: mi13(♭9 ♯11 add ♭13)
namesBySequenceIndex[1662] = '''Aeolacryllic'''
namesBySequenceIndexNegative[1662] = ''''''
semitonesBySequenceIndex[1662] = [0, 1, 3, 6, 7, 8, 9, 10]
poemBySequenceIndex[1662] = '''B D G L M N P R'''
poemAuthors[1662] = ''''''
#1663: ䷵䷪ Betrothing Decision: 1 2 3 1 1 1 1 2  : S R1 G2 M2 P D1 D2 N2 : EbOd

#1664: C D♭ E♭ F♯ G A♭ A B :0 1 3 6 7 8 9 11: 1 ♭2 ♭3 ♯4 5 ♭6 6 7: mima13(♭9 ♯11 add ♭13)
namesBySequenceIndex[1663] = ''''''
namesBySequenceIndexNegative[1663] = ''''''
semitonesBySequenceIndex[1663] = [0, 1, 3, 6, 7, 8, 9, 11]
poemBySequenceIndex[1663] = '''B D G L M N P S'''
poemAuthors[1663] = ''''''
#1664: ䷵䷍ Betrothing Resevoir: 1 2 3 1 1 1 2 1  : S R1 G2 M2 P D1 D2 N3 : EbOf

#1665: C D♭ E♭ F♯ G A♭ B♭ B :0 1 3 6 7 8 10 11: 1 ♭2 ♭3 ♯4 5 ♭6 ♭7 7: mima7(add ♭7 ♭9 ♯11 ♭13)
namesBySequenceIndex[1664] = '''Neveseri,Neveseri (Greece),Dycryllic'''
namesBySequenceIndexNegative[1664] = ''''''
semitonesBySequenceIndex[1664] = [0, 1, 3, 6, 7, 8, 10, 11]
poemBySequenceIndex[1664] = '''B D G L M N R S'''
poemAuthors[1664] = ''''''
#1665: ䷵䷈ Betrothing Homegrown: 1 2 3 1 1 2 1 1  : S R1 G2 M2 P D1 N2 N3 : EbOm

#1666: C D♭ E♭ F♯ G A B♭ B :0 1 3 6 7 9 10 11: 1 ♭2 ♭3 ♯4 5 6 ♭7 7: mima13(♭9 ♯11 add ♭7)
namesBySequenceIndex[1665] = '''Kagyllic'''
namesBySequenceIndexNegative[1665] = ''''''
semitonesBySequenceIndex[1665] = [0, 1, 3, 6, 7, 9, 10, 11]
poemBySequenceIndex[1665] = '''B D G L M P R S'''
poemAuthors[1665] = ''''''
#1666: ䷵䷉ Betrothing Treading: 1 2 3 1 2 1 1 1  : S R1 G2 M2 P D2 N2 N3 : EbEg

#1667: C D♭ E♭ F♯ G♯ A B♭ B :0 1 3 6 8 9 10 11: 1 ♭2 ♭3 ♯4 ♯5 6 ♭7 7: °ma13(♭9 ♯11 add ♭7 ♭13)
namesBySequenceIndex[1666] = ''''''
namesBySequenceIndexNegative[1666] = ''''''
semitonesBySequenceIndex[1666] = [0, 1, 3, 6, 8, 9, 10, 11]
poemBySequenceIndex[1666] = '''B D G L N P R S'''
poemAuthors[1666] = ''''''
#1667: ䷵䷌ Betrothing Fellowship: 1 2 3 2 1 1 1 1  : S R1 G2 M2 D1 D2 N2 N3 : EbIg

#1668: C D♭ E♭ G A♭ A B♭ B :0 1 3 7 8 9 10 11: 1 ♭2 ♭3 5 ♭6 6 ♭7 7: mima7(add ♭7 ♭9 ♭13 13)
namesBySequenceIndex[1667] = '''Dynyllic'''
namesBySequenceIndexNegative[1667] = ''''''
semitonesBySequenceIndex[1667] = [0, 1, 3, 7, 8, 9, 10, 11]
poemBySequenceIndex[1667] = '''B D G M N P R S'''
poemAuthors[1667] = ''''''
#1668: ䷵䷫ Betrothing Coupling: 1 2 4 1 1 1 1 1  : S R1 G2 P D1 D2 N2 N3 : Ebog

#1669: C D♭ E F F♯ G A♭ A :0 1 4 5 6 7 8 9: 1 ♭2 3 4 ♯4 5 ♭6 6: 6(add ♭6 ♭9 11 ♯11)
namesBySequenceIndex[1668] = '''Kycryllic'''
namesBySequenceIndexNegative[1668] = ''''''
semitonesBySequenceIndex[1668] = [0, 1, 4, 5, 6, 7, 8, 9]
poemBySequenceIndex[1668] = '''B D J K L M N P'''
poemAuthors[1668] = ''''''
#1669: ䷼䷡ Truth Might: 1 3 1 1 1 1 1 3  : S R1 G3 M1 M2 P D1 N1 : EmOb

#1670: C D♭ E F F♯ G A♭ B♭ :0 1 4 5 6 7 8 10: 1 ♭2 3 4 ♯4 5 ♭6 ♭7: 11(♭9 add ♯11 ♭13)
namesBySequenceIndex[1669] = '''Zothyllic'''
namesBySequenceIndexNegative[1669] = ''''''
semitonesBySequenceIndex[1669] = [0, 1, 4, 5, 6, 7, 8, 10]
poemBySequenceIndex[1669] = '''B D J K L M N R'''
poemAuthors[1669] = ''''''
#1670: ䷼䷄ Truth Biding: 1 3 1 1 1 1 2 2  : S R1 G3 M1 M2 P D1 N2 : EmOk

#1671: C D♭ E F F♯ G A♭ B :0 1 4 5 6 7 8 11: 1 ♭2 3 4 ♯4 5 ♭6 7: ma11(♭9 add ♯11 ♭13)
namesBySequenceIndex[1670] = '''Raga Ramkali,Raga Ramakri,Bythyllic'''
namesBySequenceIndexNegative[1670] = ''''''
semitonesBySequenceIndex[1670] = [0, 1, 4, 5, 6, 7, 8, 11]
poemBySequenceIndex[1670] = '''B D J K L M N S'''
poemAuthors[1670] = ''''''
#1671: ䷼䷙ Truth Cultivating: 1 3 1 1 1 1 3 1  : S R1 G3 M1 M2 P D1 N3 : EmOl

#1672: C D♭ E F F♯ G A B♭ :0 1 4 5 6 7 9 10: 1 ♭2 3 4 ♯4 5 6 ♭7: 13(♭9 add ♯11)
namesBySequenceIndex[1671] = '''Aerythyllic'''
namesBySequenceIndexNegative[1671] = ''''''
semitonesBySequenceIndex[1671] = [0, 1, 4, 5, 6, 7, 9, 10]
poemBySequenceIndex[1671] = '''B D J K L M P R'''
poemAuthors[1671] = ''''''
#1672: ䷼䷹ Truth Joyous: 1 3 1 1 1 2 1 2  : S R1 G3 M1 M2 P D2 N2 : EmEd

#1673: C D♭ E F F♯ G A B :0 1 4 5 6 7 9 11: 1 ♭2 3 4 ♯4 5 6 7: ma13(♭9 add ♯11)
namesBySequenceIndex[1672] = '''Buzurg,Buzurg (Arabia),Zolyllic'''
namesBySequenceIndexNegative[1672] = ''''''
semitonesBySequenceIndex[1672] = [0, 1, 4, 5, 6, 7, 9, 11]
poemBySequenceIndex[1672] = '''B D J K L M P S'''
poemAuthors[1672] = ''''''
#1673: ䷼䷥ Truth Polarising: 1 3 1 1 1 2 2 1  : S R1 G3 M1 M2 P D2 N3 : EmEf

#1674: C D♭ E F F♯ G A♯ B :0 1 4 5 6 7 10 11: 1 ♭2 3 4 ♯4 5 ♯6 7: ma11(♭9 add ♭7 ♯11)
namesBySequenceIndex[1673] = '''Aeralyllic'''
namesBySequenceIndexNegative[1673] = ''''''
semitonesBySequenceIndex[1673] = [0, 1, 4, 5, 6, 7, 10, 11]
poemBySequenceIndex[1673] = '''B D J K L M R S'''
poemAuthors[1673] = ''''''
#1674: ䷼䷼ Truth Truth : 1 3 1 1 1 3 1 1  : S R1 G3 M1 M2 P D3 N3 : EmEm

#1675: C D♭ E F G♭ A♭ A B♭ :0 1 4 5 6 8 9 10: 1 ♭2 3 4 ♭5 ♭6 6 ♭7: 13(♯5 ♭9 add ♯11)
namesBySequenceIndex[1674] = '''Lynyllic'''
namesBySequenceIndexNegative[1674] = ''''''
semitonesBySequenceIndex[1674] = [0, 1, 4, 5, 6, 8, 9, 10]
poemBySequenceIndex[1674] = '''B D J K L N P R'''
poemAuthors[1674] = ''''''
#1675: ䷼䷰ Truth Revolution: 1 3 1 1 2 1 1 2  : S R1 G3 M1 M2 D1 D2 N2 : EmId

#1676: C D♭ E F G♭ A♭ A B :0 1 4 5 6 8 9 11: 1 ♭2 3 4 ♭5 ♭6 6 7: ma13(♯5 ♭9 add ♯11)
namesBySequenceIndex[1675] = '''Zathyllic'''
namesBySequenceIndexNegative[1675] = ''''''
semitonesBySequenceIndex[1675] = [0, 1, 4, 5, 6, 8, 9, 11]
poemBySequenceIndex[1675] = '''B D J K L N P S'''
poemAuthors[1675] = ''''''
#1676: ䷼䷝ Truth Distanced: 1 3 1 1 2 1 2 1  : S R1 G3 M1 M2 D1 D2 N3 : EmIf

#1677: C D♭ E F G♭ A♭ B♭ B :0 1 4 5 6 8 10 11: 1 ♭2 3 4 ♭5 ♭6 ♭7 7: ma11(♯5 ♭9 add ♭7 ♯11)
namesBySequenceIndex[1676] = '''Enigmatic Mixed,Bodyllic'''
namesBySequenceIndexNegative[1676] = ''''''
semitonesBySequenceIndex[1676] = [0, 1, 4, 5, 6, 8, 10, 11]
poemBySequenceIndex[1676] = '''B D J K L N R S'''
poemAuthors[1676] = ''''''
#1677: ䷼䷤ Truth Family: 1 3 1 1 2 2 1 1  : S R1 G3 M1 M2 D1 N2 N3 : EmIm

#1678: C D♭ E F G♭ A B♭ B :0 1 4 5 6 9 10 11: 1 ♭2 3 4 ♭5 6 ♭7 7: ma13(♭5 ♭9 add ♭7)
namesBySequenceIndex[1677] = '''Oriental,Galyllic'''
namesBySequenceIndexNegative[1677] = ''''''
semitonesBySequenceIndex[1677] = [0, 1, 4, 5, 6, 9, 10, 11]
poemBySequenceIndex[1677] = '''B D J K L P R S'''
poemAuthors[1677] = ''''''
#1678: ䷼䷘ Truth Integrity: 1 3 1 1 3 1 1 1  : S R1 G3 M1 M2 D2 N2 N3 : EmAg

#1679: C D♭ E F G A♭ A B♭ :0 1 4 5 7 8 9 10: 1 ♭2 3 4 5 ♭6 6 ♭7: 13(♭9 add ♭13)
namesBySequenceIndex[1678] = '''Aerynyllic'''
namesBySequenceIndexNegative[1678] = ''''''
semitonesBySequenceIndex[1678] = [0, 1, 4, 5, 7, 8, 9, 10]
poemBySequenceIndex[1678] = '''B D J K M N P R'''
poemAuthors[1678] = ''''''
#1679: ䷼䷛ Truth Excess: 1 3 1 2 1 1 1 2  : S R1 G3 M1 P D1 D2 N2 : Emod

#1680: C D♭ E F G A♭ A B :0 1 4 5 7 8 9 11: 1 ♭2 3 4 5 ♭6 6 7: ma13(♭9 add ♭13)
namesBySequenceIndex[1679] = '''Raga Saurashtra,Sanyllic'''
namesBySequenceIndexNegative[1679] = ''''''
semitonesBySequenceIndex[1679] = [0, 1, 4, 5, 7, 8, 9, 11]
poemBySequenceIndex[1679] = '''B D J K M N P S'''
poemAuthors[1679] = ''''''
#1680: ䷼䷱ Truth Holding: 1 3 1 2 1 1 2 1  : S R1 G3 M1 P D1 D2 N3 : Emof

#1681: C D♭ E F G A♭ B♭ B :0 1 4 5 7 8 10 11: 1 ♭2 3 4 5 ♭6 ♭7 7: ma11(♭9 add ♭7 ♭13)
namesBySequenceIndex[1680] = '''Maqam Hijaz,Ponyllic'''
namesBySequenceIndexNegative[1680] = ''''''
semitonesBySequenceIndex[1680] = [0, 1, 4, 5, 7, 8, 10, 11]
poemBySequenceIndex[1680] = '''B D J K M N R S'''
poemAuthors[1680] = ''''''
#1681: ䷼䷸ Truth Osmosis: 1 3 1 2 1 2 1 1  : S R1 G3 M1 P D1 N2 N3 : Emom

#1682: C D♭ E F G A B♭ B :0 1 4 5 7 9 10 11: 1 ♭2 3 4 5 6 ♭7 7: ma13(♭9 add ♭7)
namesBySequenceIndex[1681] = '''Modyllic'''
namesBySequenceIndexNegative[1681] = ''''''
semitonesBySequenceIndex[1681] = [0, 1, 4, 5, 7, 9, 10, 11]
poemBySequenceIndex[1681] = '''B D J K M P R S'''
poemAuthors[1681] = ''''''
#1682: ䷼䷅ Truth Arguing: 1 3 1 2 2 1 1 1  : S R1 G3 M1 P D2 N2 N3 : Emeg

#1683: C D♭ E F G♯ A B♭ B :0 1 4 5 8 9 10 11: 1 ♭2 3 4 ♯5 6 ♭7 7: ma13(♯5 ♭9 add ♭7)
namesBySequenceIndex[1682] = '''Ryryllic'''
namesBySequenceIndexNegative[1682] = ''''''
semitonesBySequenceIndex[1682] = [0, 1, 4, 5, 8, 9, 10, 11]
poemBySequenceIndex[1682] = '''B D J K N P R S'''
poemAuthors[1682] = ''''''
#1683: ䷼䷠ Truth Retiring: 1 3 1 3 1 1 1 1  : S R1 G3 M1 D1 D2 N2 N3 : Emig

#1684: C D♭ E F♯ G A♭ A B♭ :0 1 4 6 7 8 9 10: 1 ♭2 3 ♯4 5 ♭6 6 ♭7: 13(♭9 ♯11 add ♭13)
namesBySequenceIndex[1683] = '''Lolyllic'''
namesBySequenceIndexNegative[1683] = ''''''
semitonesBySequenceIndex[1683] = [0, 1, 4, 6, 7, 8, 9, 10]
poemBySequenceIndex[1683] = '''B D J L M N P R'''
poemAuthors[1683] = ''''''
#1684: ䷻䷪ Composed Decision: 1 3 2 1 1 1 1 2  : S R1 G3 M2 P D1 D2 N2 : EkOd

#1685: C D♭ E F♯ G A♭ A B :0 1 4 6 7 8 9 11: 1 ♭2 3 ♯4 5 ♭6 6 7: ma13(♭9 ♯11 add ♭13)
namesBySequenceIndex[1684] = '''Rythyllic'''
namesBySequenceIndexNegative[1684] = ''''''
semitonesBySequenceIndex[1684] = [0, 1, 4, 6, 7, 8, 9, 11]
poemBySequenceIndex[1684] = '''B D J L M N P S'''
poemAuthors[1684] = ''''''
#1685: ䷻䷍ Composed Resevoir: 1 3 2 1 1 1 2 1  : S R1 G3 M2 P D1 D2 N3 : EkOf

#1686: C D♭ E F♯ G A♭ B♭ B :0 1 4 6 7 8 10 11: 1 ♭2 3 ♯4 5 ♭6 ♭7 7: ma7(add ♭7 ♭9 ♯11 ♭13)
namesBySequenceIndex[1685] = '''Aeoryllic'''
namesBySequenceIndexNegative[1685] = ''''''
semitonesBySequenceIndex[1685] = [0, 1, 4, 6, 7, 8, 10, 11]
poemBySequenceIndex[1685] = '''B D J L M N R S'''
poemAuthors[1685] = ''''''
#1686: ䷻䷈ Composed Homegrown: 1 3 2 1 1 2 1 1  : S R1 G3 M2 P D1 N2 N3 : EkOm

#1687: C D♭ E F♯ G A B♭ B :0 1 4 6 7 9 10 11: 1 ♭2 3 ♯4 5 6 ♭7 7: ma13(♭9 ♯11 add ♭7)
namesBySequenceIndex[1686] = '''Epothyllic'''
namesBySequenceIndexNegative[1686] = ''''''
semitonesBySequenceIndex[1686] = [0, 1, 4, 6, 7, 9, 10, 11]
poemBySequenceIndex[1686] = '''B D J L M P R S'''
poemAuthors[1686] = ''''''
#1687: ䷻䷉ Composed Treading: 1 3 2 1 2 1 1 1  : S R1 G3 M2 P D2 N2 N3 : EkEg

#1688: C D♭ E F♯ G♯ A B♭ B :0 1 4 6 8 9 10 11: 1 ♭2 3 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♭9 ♯11 add ♭7)
namesBySequenceIndex[1687] = '''Stoptyllic'''
namesBySequenceIndexNegative[1687] = ''''''
semitonesBySequenceIndex[1687] = [0, 1, 4, 6, 8, 9, 10, 11]
poemBySequenceIndex[1687] = '''B D J L N P R S'''
poemAuthors[1687] = ''''''
#1688: ䷻䷌ Composed Fellowship: 1 3 2 2 1 1 1 1  : S R1 G3 M2 D1 D2 N2 N3 : EkIg

#1689: C D♭ E G A♭ A B♭ B :0 1 4 7 8 9 10 11: 1 ♭2 3 5 ♭6 6 ♭7 7: ma7(add ♭7 ♭9 ♭13 13)
namesBySequenceIndex[1688] = '''Loryllic'''
namesBySequenceIndexNegative[1688] = ''''''
semitonesBySequenceIndex[1688] = [0, 1, 4, 7, 8, 9, 10, 11]
poemBySequenceIndex[1688] = '''B D J M N P R S'''
poemAuthors[1688] = ''''''
#1689: ䷻䷫ Composed Coupling: 1 3 3 1 1 1 1 1  : S R1 G3 P D1 D2 N2 N3 : Ekog

#1690: C D♭ F F♯ G A♭ A B♭ :0 1 5 6 7 8 9 10: 1 ♭2 4 ♯4 5 ♭6 6 ♭7: 13sus♯4(♭9 add ♭13)
namesBySequenceIndex[1689] = '''Palyllic'''
namesBySequenceIndexNegative[1689] = ''''''
semitonesBySequenceIndex[1689] = [0, 1, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1689] = '''B D K L M N P R'''
poemAuthors[1689] = ''''''
#1690: ䷨䷪ Diminishing Decision: 1 4 1 1 1 1 1 2  : S R1 M1 M2 P D1 D2 N2 : ElOd

#1691: C D♭ F F♯ G A♭ A B :0 1 5 6 7 8 9 11: 1 ♭2 4 ♯4 5 ♭6 6 7: ma13sus♯4(♭9 add ♭13)
namesBySequenceIndex[1690] = '''Ionayllic'''
namesBySequenceIndexNegative[1690] = ''''''
semitonesBySequenceIndex[1690] = [0, 1, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1690] = '''B D K L M N P S'''
poemAuthors[1690] = ''''''
#1691: ䷨䷍ Diminishing Resevoir: 1 4 1 1 1 1 2 1  : S R1 M1 M2 P D1 D2 N3 : ElOf

#1692: C D♭ F F♯ G A♭ B♭ B :0 1 5 6 7 8 10 11: 1 ♭2 4 ♯4 5 ♭6 ♭7 7: ma11sus♯4(♭9 add ♭7 ♭13)
namesBySequenceIndex[1691] = '''Pylyllic'''
namesBySequenceIndexNegative[1691] = ''''''
semitonesBySequenceIndex[1691] = [0, 1, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1691] = '''B D K L M N R S'''
poemAuthors[1691] = ''''''
#1692: ䷨䷈ Diminishing Homegrown: 1 4 1 1 1 2 1 1  : S R1 M1 M2 P D1 N2 N3 : ElOm

#1693: C D♭ F F♯ G A B♭ B :0 1 5 6 7 9 10 11: 1 ♭2 4 ♯4 5 6 ♭7 7: ma13sus♯4(♭9 add ♭7)
namesBySequenceIndex[1692] = '''Epogyllic'''
namesBySequenceIndexNegative[1692] = ''''''
semitonesBySequenceIndex[1692] = [0, 1, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1692] = '''B D K L M P R S'''
poemAuthors[1692] = ''''''
#1693: ䷨䷉ Diminishing Treading: 1 4 1 1 2 1 1 1  : S R1 M1 M2 P D2 N2 N3 : ElEg

#1694: C D♭ F G♭ A♭ A B♭ B :0 1 5 6 8 9 10 11: 1 ♭2 4 ♭5 ♭6 6 ♭7 7: ma13sus♯4(♯5 ♭9 add ♭7)
namesBySequenceIndex[1693] = '''Dogyllic'''
namesBySequenceIndexNegative[1693] = ''''''
semitonesBySequenceIndex[1693] = [0, 1, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1693] = '''B D K L N P R S'''
poemAuthors[1693] = ''''''
#1694: ䷨䷌ Diminishing Fellowship: 1 4 1 2 1 1 1 1  : S R1 M1 M2 D1 D2 N2 N3 : ElIg

#1695: C D♭ F G A♭ A B♭ B :0 1 5 7 8 9 10 11: 1 ♭2 4 5 ♭6 6 ♭7 7: ma13(♭9 no 3 add ♭7 ♭13)
namesBySequenceIndex[1694] = '''Sadyllic'''
namesBySequenceIndexNegative[1694] = ''''''
semitonesBySequenceIndex[1694] = [0, 1, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1694] = '''B D K M N P R S'''
poemAuthors[1694] = ''''''
#1695: ䷨䷫ Diminishing Coupling: 1 4 2 1 1 1 1 1  : S R1 M1 P D1 D2 N2 N3 : Elog

#1696: C D♭ F♯ G A♭ A B♭ B :0 1 6 7 8 9 10 11: 1 ♭2 ♯4 5 ♭6 6 ♭7 7: ma13(♭9 ♯11 no 3 add ♭7 ♭13)
namesBySequenceIndex[1695] = ''''''
namesBySequenceIndexNegative[1695] = ''''''
semitonesBySequenceIndex[1695] = [0, 1, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1695] = '''B D L M N P R S'''
poemAuthors[1695] = ''''''
#1696: ䷒䷀ Arrived Creative: 1 5 1 1 1 1 1 1  : S R1 M2 P D1 D2 N2 N3 : EjOg

#1697: C D D♯ E F F♯ G A♭ :0 2 3 4 5 6 7 8: 1 2 ♯2 3 4 ♯4 5 ♭6: 11(no 7 add ♭6 ♯9 ♯11)
namesBySequenceIndex[1696] = '''Ionothyllic'''
namesBySequenceIndexNegative[1696] = ''''''
semitonesBySequenceIndex[1696] = [0, 2, 3, 4, 5, 6, 7, 8]
poemBySequenceIndex[1696] = '''B F G J K L M N'''
poemAuthors[1696] = ''''''
#1697: ䷌䷊ Fellowship Peace: 2 1 1 1 1 1 1 4  : S R2 R3 G3 M1 M2 P D1 : IgOj

#1698: C D D♯ E F F♯ G A :0 2 3 4 5 6 7 9: 1 2 ♯2 3 4 ♯4 5 6: 13(no 7 add ♯9 ♯11)
namesBySequenceIndex[1697] = ''''''
namesBySequenceIndexNegative[1697] = ''''''
semitonesBySequenceIndex[1697] = [0, 2, 3, 4, 5, 6, 7, 9]
poemBySequenceIndex[1697] = '''B F G J K L M P'''
poemAuthors[1697] = ''''''
#1698: ䷌䷵ Fellowship Betrothing: 2 1 1 1 1 1 2 3  : S R2 R3 G3 M1 M2 P D2 : IgEb

#1699: C D D♯ E F F♯ G B♭ :0 2 3 4 5 6 7 10: 1 2 ♯2 3 4 ♯4 5 ♭7: 11(add ♯9 ♯11)
namesBySequenceIndex[1698] = '''Zadyllic'''
namesBySequenceIndexNegative[1698] = ''''''
semitonesBySequenceIndex[1698] = [0, 2, 3, 4, 5, 6, 7, 10]
poemBySequenceIndex[1698] = '''B F G J K L M R'''
poemAuthors[1698] = ''''''
#1699: ䷌䷻ Fellowship Composed: 2 1 1 1 1 1 3 2  : S R2 R3 G3 M1 M2 P N2 : IgEk

#1700: C D D♯ E F F♯ G B :0 2 3 4 5 6 7 11: 1 2 ♯2 3 4 ♯4 5 7: ma11(add ♯9 ♯11)
namesBySequenceIndex[1699] = '''Bydyllic'''
namesBySequenceIndexNegative[1699] = ''''''
semitonesBySequenceIndex[1699] = [0, 2, 3, 4, 5, 6, 7, 11]
poemBySequenceIndex[1699] = '''B F G J K L M S'''
poemAuthors[1699] = ''''''
#1700: ䷌䷨ Fellowship Diminishing: 2 1 1 1 1 1 4 1  : S R2 R3 G3 M1 M2 P N3 : IgEl

#1701: C D D♯ E F G♭ A♭ A :0 2 3 4 5 6 8 9: 1 2 ♯2 3 4 ♭5 ♭6 6: 13(♯5 no 7 add ♯9 ♯11)
namesBySequenceIndex[1700] = '''Pycryllic'''
namesBySequenceIndexNegative[1700] = ''''''
semitonesBySequenceIndex[1700] = [0, 2, 3, 4, 5, 6, 8, 9]
poemBySequenceIndex[1700] = '''B F G J K L N P'''
poemAuthors[1700] = ''''''
#1701: ䷌䷶ Fellowship Abounding: 2 1 1 1 1 2 1 3  : S R2 R3 G3 M1 M2 D1 N1 : IgIb

#1702: C D D♯ E F G♭ A♭ B♭ :0 2 3 4 5 6 8 10: 1 2 ♯2 3 4 ♭5 ♭6 ♭7: 11(♯5 add ♯9 ♯11)
namesBySequenceIndex[1701] = '''Goryllic'''
namesBySequenceIndexNegative[1701] = ''''''
semitonesBySequenceIndex[1701] = [0, 2, 3, 4, 5, 6, 8, 10]
poemBySequenceIndex[1701] = '''B F G J K L N R'''
poemAuthors[1701] = ''''''
#1702: ䷌䷾ Fellowship Completed: 2 1 1 1 1 2 2 2  : S R2 R3 G3 M1 M2 D1 N2 : IgIk

#1703: C D D♯ E F G♭ A♭ B :0 2 3 4 5 6 8 11: 1 2 ♯2 3 4 ♭5 ♭6 7: ma11(♯5 add ♯9 ♯11)
namesBySequenceIndex[1702] = '''Kadyllic'''
namesBySequenceIndexNegative[1702] = ''''''
semitonesBySequenceIndex[1702] = [0, 2, 3, 4, 5, 6, 8, 11]
poemBySequenceIndex[1702] = '''B F G J K L N S'''
poemAuthors[1702] = ''''''
#1703: ䷌䷕ Fellowship Adorning: 2 1 1 1 1 2 3 1  : S R2 R3 G3 M1 M2 D1 N3 : IgIl

#1704: C D D♯ E F G♭ A B♭ :0 2 3 4 5 6 9 10: 1 2 ♯2 3 4 ♭5 6 ♭7: 13(♭5 add ♯9)
namesBySequenceIndex[1703] = '''Gonyllic'''
namesBySequenceIndexNegative[1703] = ''''''
semitonesBySequenceIndex[1703] = [0, 2, 3, 4, 5, 6, 9, 10]
poemBySequenceIndex[1703] = '''B F G J K L P R'''
poemAuthors[1703] = ''''''
#1704: ䷌䷐ Fellowship Following: 2 1 1 1 1 3 1 2  : S R2 R3 G3 M1 M2 D2 N2 : IgAd

#1705: C D D♯ E F G♭ A B :0 2 3 4 5 6 9 11: 1 2 ♯2 3 4 ♭5 6 7: ma13(♭5 add ♯9)
namesBySequenceIndex[1704] = '''Ionoryllic'''
namesBySequenceIndexNegative[1704] = ''''''
semitonesBySequenceIndex[1704] = [0, 2, 3, 4, 5, 6, 9, 11]
poemBySequenceIndex[1704] = '''B F G J K L P S'''
poemAuthors[1704] = ''''''
#1705: ䷌䷔ Fellowship Biting: 2 1 1 1 1 3 2 1  : S R2 R3 G3 M1 M2 D2 N3 : IgAf

#1706: C D D♯ E F G♭ A♯ B :0 2 3 4 5 6 10 11: 1 2 ♯2 3 4 ♭5 ♯6 7: ma11(♭5 add ♭7 ♯9)
namesBySequenceIndex[1705] = '''Gylyllic'''
namesBySequenceIndexNegative[1705] = ''''''
semitonesBySequenceIndex[1705] = [0, 2, 3, 4, 5, 6, 10, 11]
poemBySequenceIndex[1705] = '''B F G J K L R S'''
poemAuthors[1705] = ''''''
#1706: ䷌䷩ Fellowship Augmenting: 2 1 1 1 1 4 1 1  : S R2 R3 G3 M1 M2 D3 N3 : IgAm

#1707: C D D♯ E F G A♭ A :0 2 3 4 5 7 8 9: 1 2 ♯2 3 4 5 ♭6 6: 13(no 7 add ♭6 ♯9)
namesBySequenceIndex[1706] = '''Phronyllic'''
namesBySequenceIndexNegative[1706] = ''''''
semitonesBySequenceIndex[1706] = [0, 2, 3, 4, 5, 7, 8, 9]
poemBySequenceIndex[1706] = '''B F G J K M N P'''
poemAuthors[1706] = ''''''
#1707: ䷌䷟ Fellowship Persevering: 2 1 1 1 2 1 1 3  : S R2 R3 G3 M1 P D1 N1 : Igob

#1708: C D D♯ E F G A♭ B♭ :0 2 3 4 5 7 8 10: 1 2 ♯2 3 4 5 ♭6 ♭7: 11(add ♯9 ♭13)
namesBySequenceIndex[1707] = '''Epiryllic'''
namesBySequenceIndexNegative[1707] = ''''''
semitonesBySequenceIndex[1707] = [0, 2, 3, 4, 5, 7, 8, 10]
poemBySequenceIndex[1707] = '''B F G J K M N R'''
poemAuthors[1707] = ''''''
#1708: ䷌䷯ Fellowship Source: 2 1 1 1 2 1 2 2  : S R2 R3 G3 M1 P D1 N2 : Igok

#1709: C D D♯ E F G A♭ B :0 2 3 4 5 7 8 11: 1 2 ♯2 3 4 5 ♭6 7: ma11(add ♯9 ♭13)
namesBySequenceIndex[1708] = '''Manyllic'''
namesBySequenceIndexNegative[1708] = ''''''
semitonesBySequenceIndex[1708] = [0, 2, 3, 4, 5, 7, 8, 11]
poemBySequenceIndex[1708] = '''B F G J K M N S'''
poemAuthors[1708] = ''''''
#1709: ䷌䷑ Fellowship Correcting: 2 1 1 1 2 1 3 1  : S R2 R3 G3 M1 P D1 N3 : Igol

#1710: C D D♯ E F G A B♭ :0 2 3 4 5 7 9 10: 1 2 ♯2 3 4 5 6 ♭7: 13(add ♯9)
namesBySequenceIndex[1709] = '''Raga Zilla,Minor Bebop,Bebop Dorian,Mixolydian Dorian,Mixodyllic'''
namesBySequenceIndexNegative[1709] = ''''''
semitonesBySequenceIndex[1709] = [0, 2, 3, 4, 5, 7, 9, 10]
poemBySequenceIndex[1709] = '''B F G J K M P R'''
poemAuthors[1709] = ''''''
#1710: ䷌䷮ Fellowship Confining: 2 1 1 1 2 2 1 2  : S R2 R3 G3 M1 P D2 N2 : Iged

#1711: C D D♯ E F G A B :0 2 3 4 5 7 9 11: 1 2 ♯2 3 4 5 6 7: ma13(add ♯9)
namesBySequenceIndex[1710] = '''Katagyllic'''
namesBySequenceIndexNegative[1710] = ''''''
semitonesBySequenceIndex[1710] = [0, 2, 3, 4, 5, 7, 9, 11]
poemBySequenceIndex[1710] = '''B F G J K M P S'''
poemAuthors[1710] = ''''''
#1711: ䷌䷿ Fellowship Unfinished: 2 1 1 1 2 2 2 1  : S R2 R3 G3 M1 P D2 N3 : Igef

#1712: C D D♯ E F G A♯ B :0 2 3 4 5 7 10 11: 1 2 ♯2 3 4 5 ♯6 7: ma11(add ♭7 ♯9)
namesBySequenceIndex[1711] = '''Dodyllic'''
namesBySequenceIndexNegative[1711] = ''''''
semitonesBySequenceIndex[1711] = [0, 2, 3, 4, 5, 7, 10, 11]
poemBySequenceIndex[1711] = '''B F G J K M R S'''
poemAuthors[1711] = ''''''
#1712: ䷌䷺ Fellowship Dispersing: 2 1 1 1 2 3 1 1  : S R2 R3 G3 M1 P D3 N3 : Igem

#1713: C D D♯ E F G♯ A B♭ :0 2 3 4 5 8 9 10: 1 2 ♯2 3 4 ♯5 6 ♭7: 13(♯5 add ♯9)
namesBySequenceIndex[1712] = '''Maryllic'''
namesBySequenceIndexNegative[1712] = ''''''
semitonesBySequenceIndex[1712] = [0, 2, 3, 4, 5, 8, 9, 10]
poemBySequenceIndex[1712] = '''B F G J K N P R'''
poemAuthors[1712] = ''''''
#1713: ䷌䷞ Fellowship Conjoining: 2 1 1 1 3 1 1 2  : S R2 R3 G3 M1 D1 D2 N2 : Igid

#1714: C D D♯ E F G♯ A B :0 2 3 4 5 8 9 11: 1 2 ♯2 3 4 ♯5 6 7: ma13(♯5 add ♯9)
namesBySequenceIndex[1713] = '''Phrylyllic'''
namesBySequenceIndexNegative[1713] = ''''''
semitonesBySequenceIndex[1713] = [0, 2, 3, 4, 5, 8, 9, 11]
poemBySequenceIndex[1713] = '''B F G J K N P S'''
poemAuthors[1713] = ''''''
#1714: ䷌䷷ Fellowship Wanderer: 2 1 1 1 3 1 2 1  : S R2 R3 G3 M1 D1 D2 N3 : Igif

#1715: C D D♯ E F G♯ A♯ B :0 2 3 4 5 8 10 11: 1 2 ♯2 3 4 ♯5 ♯6 7: ma11(♯5 add ♭7 ♯9)
namesBySequenceIndex[1714] = '''Socryllic'''
namesBySequenceIndexNegative[1714] = ''''''
semitonesBySequenceIndex[1714] = [0, 2, 3, 4, 5, 8, 10, 11]
poemBySequenceIndex[1714] = '''B F G J K N R S'''
poemAuthors[1714] = ''''''
#1715: ䷌䷴ Fellowship Infiltrating: 2 1 1 1 3 2 1 1  : S R2 R3 G3 M1 D1 N2 N3 : Igim

#1716: C D D♯ E F A B♭ B :0 2 3 4 5 9 10 11: 1 2 ♯2 3 4 6 ♭7 7: ma13(no 5 add ♭7 ♯9)
namesBySequenceIndex[1715] = '''Zycryllic'''
namesBySequenceIndexNegative[1715] = ''''''
semitonesBySequenceIndex[1715] = [0, 2, 3, 4, 5, 9, 10, 11]
poemBySequenceIndex[1715] = '''B F G J K P R S'''
poemAuthors[1715] = ''''''
#1716: ䷌䷋ Fellowship Obstruction: 2 1 1 1 4 1 1 1  : S R2 R3 G3 M1 D2 N2 N3 : Igag

#1717: C D D♯ E F♯ G A♭ A :0 2 3 4 6 7 8 9: 1 2 ♯2 3 ♯4 5 ♭6 6: 6/9(add ♯9 ♯11 ♭13)
namesBySequenceIndex[1716] = '''Phrolyllic'''
namesBySequenceIndexNegative[1716] = ''''''
semitonesBySequenceIndex[1716] = [0, 2, 3, 4, 6, 7, 8, 9]
poemBySequenceIndex[1716] = '''B F G J L M N P'''
poemAuthors[1716] = ''''''
#1717: ䷰䷡ Revolution Might: 2 1 1 2 1 1 1 3  : S R2 R3 G3 M2 P D1 N1 : IdOb

#1718: C D D♯ E F♯ G A♭ B♭ :0 2 3 4 6 7 8 10: 1 2 ♯2 3 ♯4 5 ♭6 ♭7: 9(add ♯9 ♯11 ♭13)
namesBySequenceIndex[1717] = '''Stygyllic'''
namesBySequenceIndexNegative[1717] = ''''''
semitonesBySequenceIndex[1717] = [0, 2, 3, 4, 6, 7, 8, 10]
poemBySequenceIndex[1717] = '''B F G J L M N R'''
poemAuthors[1717] = ''''''
#1718: ䷰䷄ Revolution Biding: 2 1 1 2 1 1 2 2  : S R2 R3 G3 M2 P D1 N2 : IdOk

#1719: C D D♯ E F♯ G A♭ B :0 2 3 4 6 7 8 11: 1 2 ♯2 3 ♯4 5 ♭6 7: ma9(add ♯9 ♯11 ♭13)
namesBySequenceIndex[1718] = '''Aeolaryllic'''
namesBySequenceIndexNegative[1718] = ''''''
semitonesBySequenceIndex[1718] = [0, 2, 3, 4, 6, 7, 8, 11]
poemBySequenceIndex[1718] = '''B F G J L M N S'''
poemAuthors[1718] = ''''''
#1719: ䷰䷙ Revolution Cultivating: 2 1 1 2 1 1 3 1  : S R2 R3 G3 M2 P D1 N3 : IdOl

#1720: C D D♯ E F♯ G A B♭ :0 2 3 4 6 7 9 10: 1 2 ♯2 3 ♯4 5 6 ♭7: 13(♯11 add ♯9)
namesBySequenceIndex[1719] = ''''''
namesBySequenceIndexNegative[1719] = ''''''
semitonesBySequenceIndex[1719] = [0, 2, 3, 4, 6, 7, 9, 10]
poemBySequenceIndex[1719] = '''B F G J L M P R'''
poemAuthors[1719] = ''''''
#1720: ䷰䷹ Revolution Joyous: 2 1 1 2 1 2 1 2  : S R2 R3 G3 M2 P D2 N2 : IdEd

#1721: C D D♯ E F♯ G A B :0 2 3 4 6 7 9 11: 1 2 ♯2 3 ♯4 5 6 7: ma13(♯11 add ♯9)
namesBySequenceIndex[1720] = '''Gycryllic'''
namesBySequenceIndexNegative[1720] = ''''''
semitonesBySequenceIndex[1720] = [0, 2, 3, 4, 6, 7, 9, 11]
poemBySequenceIndex[1720] = '''B F G J L M P S'''
poemAuthors[1720] = ''''''
#1721: ䷰䷥ Revolution Polarising: 2 1 1 2 1 2 2 1  : S R2 R3 G3 M2 P D2 N3 : IdEf

#1722: C D D♯ E F♯ G A♯ B :0 2 3 4 6 7 10 11: 1 2 ♯2 3 ♯4 5 ♯6 7: ma9(add ♭7 ♯9 ♯11)
namesBySequenceIndex[1721] = '''Saryllic'''
namesBySequenceIndexNegative[1721] = ''''''
semitonesBySequenceIndex[1721] = [0, 2, 3, 4, 6, 7, 10, 11]
poemBySequenceIndex[1721] = '''B F G J L M R S'''
poemAuthors[1721] = ''''''
#1722: ䷰䷼ Revolution Truth: 2 1 1 2 1 3 1 1  : S R2 R3 G3 M2 P D3 N3 : IdEm

#1723: C D D♯ E F♯ G♯ A B♭ :0 2 3 4 6 8 9 10: 1 2 ♯2 3 ♯4 ♯5 6 ♭7: 13(♯5 ♯11 add ♯9)
namesBySequenceIndex[1722] = '''Epotyllic'''
namesBySequenceIndexNegative[1722] = ''''''
semitonesBySequenceIndex[1722] = [0, 2, 3, 4, 6, 8, 9, 10]
poemBySequenceIndex[1722] = '''B F G J L N P R'''
poemAuthors[1722] = ''''''
#1723: ䷰䷰ Revolution Revolution : 2 1 1 2 2 1 1 2  : S R2 R3 G3 M2 D1 D2 N2 : IdId

#1724: C D D♯ E F♯ G♯ A B :0 2 3 4 6 8 9 11: 1 2 ♯2 3 ♯4 ♯5 6 7: ma13(♯5 ♯11 add ♯9)
namesBySequenceIndex[1723] = '''Mocryllic'''
namesBySequenceIndexNegative[1723] = ''''''
semitonesBySequenceIndex[1723] = [0, 2, 3, 4, 6, 8, 9, 11]
poemBySequenceIndex[1723] = '''B F G J L N P S'''
poemAuthors[1723] = ''''''
#1724: ䷰䷝ Revolution Distanced: 2 1 1 2 2 1 2 1  : S R2 R3 G3 M2 D1 D2 N3 : IdIf

#1725: C D D♯ E F♯ G♯ A♯ B :0 2 3 4 6 8 10 11: 1 2 ♯2 3 ♯4 ♯5 ♯6 7: ma9(♯5 add ♭7 ♯9 ♯11)
namesBySequenceIndex[1724] = '''Aerothyllic'''
namesBySequenceIndexNegative[1724] = ''''''
semitonesBySequenceIndex[1724] = [0, 2, 3, 4, 6, 8, 10, 11]
poemBySequenceIndex[1724] = '''B F G J L N R S'''
poemAuthors[1724] = ''''''
#1725: ䷰䷤ Revolution Family: 2 1 1 2 2 2 1 1  : S R2 R3 G3 M2 D1 N2 N3 : IdIm

#1726: C D D♯ E G♭ A B♭ B :0 2 3 4 6 9 10 11: 1 2 ♯2 3 ♭5 6 ♭7 7: ma9(♭5 add ♭7 ♯9 13)
namesBySequenceIndex[1725] = ''''''
namesBySequenceIndexNegative[1725] = ''''''
semitonesBySequenceIndex[1725] = [0, 2, 3, 4, 6, 9, 10, 11]
poemBySequenceIndex[1725] = '''B F G J L P R S'''
poemAuthors[1725] = ''''''
#1726: ䷰䷘ Revolution Integrity: 2 1 1 2 3 1 1 1  : S R2 R3 G3 M2 D2 N2 N3 : IdAg

#1727: C D D♯ E G A♭ A B♭ :0 2 3 4 7 8 9 10: 1 2 ♯2 3 5 ♭6 6 ♭7: 9(add ♯9 ♭13 13)
namesBySequenceIndex[1726] = '''Mathyllic'''
namesBySequenceIndexNegative[1726] = ''''''
semitonesBySequenceIndex[1726] = [0, 2, 3, 4, 7, 8, 9, 10]
poemBySequenceIndex[1726] = '''B F G J M N P R'''
poemAuthors[1726] = ''''''
#1727: ䷰䷛ Revolution Excess: 2 1 1 3 1 1 1 2  : S R2 R3 G3 P D1 D2 N2 : Idod

#1728: C D D♯ E G A♭ A B :0 2 3 4 7 8 9 11: 1 2 ♯2 3 5 ♭6 6 7: ma9(add ♯9 ♭13 13)
namesBySequenceIndex[1727] = '''Panyllic'''
namesBySequenceIndexNegative[1727] = ''''''
semitonesBySequenceIndex[1727] = [0, 2, 3, 4, 7, 8, 9, 11]
poemBySequenceIndex[1727] = '''B F G J M N P S'''
poemAuthors[1727] = ''''''
#1728: ䷰䷱ Revolution Holding: 2 1 1 3 1 1 2 1  : S R2 R3 G3 P D1 D2 N3 : Idof

#1729: C D D♯ E G A♭ B♭ B :0 2 3 4 7 8 10 11: 1 2 ♯2 3 5 ♭6 ♭7 7: ma9(add ♭7 ♯9 ♭13)
namesBySequenceIndex[1728] = '''Kyptyllic'''
namesBySequenceIndexNegative[1728] = ''''''
semitonesBySequenceIndex[1728] = [0, 2, 3, 4, 7, 8, 10, 11]
poemBySequenceIndex[1728] = '''B F G J M N R S'''
poemAuthors[1728] = ''''''
#1729: ䷰䷸ Revolution Osmosis: 2 1 1 3 1 2 1 1  : S R2 R3 G3 P D1 N2 N3 : Idom

#1730: C D D♯ E G A B♭ B :0 2 3 4 7 9 10 11: 1 2 ♯2 3 5 6 ♭7 7: ma9(add ♭7 ♯9 13)
namesBySequenceIndex[1729] = '''Zydyllic'''
namesBySequenceIndexNegative[1729] = ''''''
semitonesBySequenceIndex[1729] = [0, 2, 3, 4, 7, 9, 10, 11]
poemBySequenceIndex[1729] = '''B F G J M P R S'''
poemAuthors[1729] = ''''''
#1730: ䷰䷅ Revolution Arguing: 2 1 1 3 2 1 1 1  : S R2 R3 G3 P D2 N2 N3 : Ideg

#1731: C D D♯ E G♯ A B♭ B :0 2 3 4 8 9 10 11: 1 2 ♯2 3 ♯5 6 ♭7 7: ma9(♯5 add ♭7 ♯9 13)
namesBySequenceIndex[1730] = '''Bygyllic'''
namesBySequenceIndexNegative[1730] = ''''''
semitonesBySequenceIndex[1730] = [0, 2, 3, 4, 8, 9, 10, 11]
poemBySequenceIndex[1730] = '''B F G J N P R S'''
poemAuthors[1730] = ''''''
#1731: ䷰䷠ Revolution Retiring: 2 1 1 4 1 1 1 1  : S R2 R3 G3 D1 D2 N2 N3 : Idig

#1732: C D E♭ F F♯ G A♭ A :0 2 3 5 6 7 8 9: 1 2 ♭3 4 ♯4 5 ♭6 6: mi13(no 7 add ♭6 ♯11)
namesBySequenceIndex[1731] = '''Radyllic'''
namesBySequenceIndexNegative[1731] = ''''''
semitonesBySequenceIndex[1731] = [0, 2, 3, 5, 6, 7, 8, 9]
poemBySequenceIndex[1731] = '''B F G K L M N P'''
poemAuthors[1731] = ''''''
#1732: ䷝䷡ Distanced Might: 2 1 2 1 1 1 1 3  : S R2 G2 M1 M2 P D1 N1 : IfOb

#1733: C D E♭ F F♯ G A♭ B♭ :0 2 3 5 6 7 8 10: 1 2 ♭3 4 ♯4 5 ♭6 ♭7: mi11(add ♯11 ♭13)
namesBySequenceIndex[1732] = '''Sagyllic'''
namesBySequenceIndexNegative[1732] = ''''''
semitonesBySequenceIndex[1732] = [0, 2, 3, 5, 6, 7, 8, 10]
poemBySequenceIndex[1732] = '''B F G K L M N R'''
poemAuthors[1732] = ''''''
#1733: ䷝䷄ Distanced Biding: 2 1 2 1 1 1 2 2  : S R2 G2 M1 M2 P D1 N2 : IfOk

#1734: C D E♭ F F♯ G A♭ B :0 2 3 5 6 7 8 11: 1 2 ♭3 4 ♯4 5 ♭6 7: mima11(add ♯11 ♭13)
namesBySequenceIndex[1733] = '''Algerian Octatonic,Katadyllic'''
namesBySequenceIndexNegative[1733] = ''''''
semitonesBySequenceIndex[1733] = [0, 2, 3, 5, 6, 7, 8, 11]
poemBySequenceIndex[1733] = '''B F G K L M N S'''
poemAuthors[1733] = ''''''
#1734: ䷝䷙ Distanced Cultivating: 2 1 2 1 1 1 3 1  : S R2 G2 M1 M2 P D1 N3 : IfOl

#1735: C D E♭ F F♯ G A B♭ :0 2 3 5 6 7 9 10: 1 2 ♭3 4 ♯4 5 6 ♭7: mi13(add ♯11)
namesBySequenceIndex[1734] = '''Bluedorian,Blues Octatonic,Aeoloryllic'''
namesBySequenceIndexNegative[1734] = ''''''
semitonesBySequenceIndex[1734] = [0, 2, 3, 5, 6, 7, 9, 10]
poemBySequenceIndex[1734] = '''B F G K L M P R'''
poemAuthors[1734] = ''''''
#1735: ䷝䷹ Distanced Joyous: 2 1 2 1 1 2 1 2  : S R2 G2 M1 M2 P D2 N2 : IfEd

#1736: C D E♭ F F♯ G A B :0 2 3 5 6 7 9 11: 1 2 ♭3 4 ♯4 5 6 7: mima13(add ♯11)
namesBySequenceIndex[1735] = '''Bluelodic,Stalyllic'''
namesBySequenceIndexNegative[1735] = ''''''
semitonesBySequenceIndex[1735] = [0, 2, 3, 5, 6, 7, 9, 11]
poemBySequenceIndex[1735] = '''B F G K L M P S'''
poemAuthors[1735] = ''''''
#1736: ䷝䷥ Distanced Polarising: 2 1 2 1 1 2 2 1  : S R2 G2 M1 M2 P D2 N3 : IfEf

#1737: C D E♭ F F♯ G A♯ B :0 2 3 5 6 7 10 11: 1 2 ♭3 4 ♯4 5 ♯6 7: mima11(add ♭7 ♯11)
namesBySequenceIndex[1736] = '''Bycryllic'''
namesBySequenceIndexNegative[1736] = ''''''
semitonesBySequenceIndex[1736] = [0, 2, 3, 5, 6, 7, 10, 11]
poemBySequenceIndex[1736] = '''B F G K L M R S'''
poemAuthors[1736] = ''''''
#1737: ䷝䷼ Distanced Truth: 2 1 2 1 1 3 1 1  : S R2 G2 M1 M2 P D3 N3 : IfEm

#1738: C D E♭ F G♭ A♭ A B♭ :0 2 3 5 6 8 9 10: 1 2 ♭3 4 ♭5 ♭6 6 ♭7: ⌀13(add ♭13)
namesBySequenceIndex[1737] = '''Ionidyllic'''
namesBySequenceIndexNegative[1737] = ''''''
semitonesBySequenceIndex[1737] = [0, 2, 3, 5, 6, 8, 9, 10]
poemBySequenceIndex[1737] = '''B F G K L N P R'''
poemAuthors[1737] = ''''''
#1738: ䷝䷰ Distanced Revolution: 2 1 2 1 2 1 1 2  : S R2 G2 M1 M2 D1 D2 N2 : IfId

#1739: C D E♭ F G♭ A♭ A B :0 2 3 5 6 8 9 11: 1 2 ♭3 4 ♭5 ♭6 6 7: °ma13(add ♭13)
namesBySequenceIndex[1738] = '''Diminished,Tone-Semitone,Octatonic,Whole-Tone Diminished,Modus Conjunctus,Epadyllic'''
namesBySequenceIndexNegative[1738] = ''''''
semitonesBySequenceIndex[1738] = [0, 2, 3, 5, 6, 8, 9, 11]
poemBySequenceIndex[1738] = '''B F G K L N P S'''
poemAuthors[1738] = ''''''
#1739: ䷝䷝ Distanced Distanced : 2 1 2 1 2 1 2 1  : S R2 G2 M1 M2 D1 D2 N3 : IfIf

#1740: C D E♭ F G♭ A♭ B♭ B :0 2 3 5 6 8 10 11: 1 2 ♭3 4 ♭5 ♭6 ♭7 7: °ma11(add ♭7 ♭13)
namesBySequenceIndex[1739] = ''''''
namesBySequenceIndexNegative[1739] = ''''''
semitonesBySequenceIndex[1739] = [0, 2, 3, 5, 6, 8, 10, 11]
poemBySequenceIndex[1739] = '''B F G K L N R S'''
poemAuthors[1739] = ''''''
#1740: ䷝䷤ Distanced Family: 2 1 2 1 2 2 1 1  : S R2 G2 M1 M2 D1 N2 N3 : IfIm

#1741: C D E♭ F G♭ A B♭ B :0 2 3 5 6 9 10 11: 1 2 ♭3 4 ♭5 6 ♭7 7: °ma13(add ♭7)
namesBySequenceIndex[1740] = '''Epaptyllic'''
namesBySequenceIndexNegative[1740] = ''''''
semitonesBySequenceIndex[1740] = [0, 2, 3, 5, 6, 9, 10, 11]
poemBySequenceIndex[1740] = '''B F G K L P R S'''
poemAuthors[1740] = ''''''
#1741: ䷝䷘ Distanced Integrity: 2 1 2 1 3 1 1 1  : S R2 G2 M1 M2 D2 N2 N3 : IfAg

#1742: C D E♭ F G A♭ A B♭ :0 2 3 5 7 8 9 10: 1 2 ♭3 4 5 ♭6 6 ♭7: mi13(add ♭13)
namesBySequenceIndex[1741] = '''Raga Deshi 1,Dorian Aeolian,Gregorian 1,Raga Anandabhairavi,Raga Mukhari,Raga Manji,Gadyllic'''
namesBySequenceIndexNegative[1741] = ''''''
semitonesBySequenceIndex[1741] = [0, 2, 3, 5, 7, 8, 9, 10]
poemBySequenceIndex[1741] = '''B F G K M N P R'''
poemAuthors[1741] = ''''''
#1742: ䷝䷛ Distanced Excess: 2 1 2 2 1 1 1 2  : S R2 G2 M1 P D1 D2 N2 : Ifod

#1743: C D E♭ F G A♭ A B :0 2 3 5 7 8 9 11: 1 2 ♭3 4 5 ♭6 6 7: mima13(add ♭13)
namesBySequenceIndex[1742] = '''Zirafkend,Bebop Melodic Minor,Zirafkend (Arabia)'''
namesBySequenceIndexNegative[1742] = ''''''
semitonesBySequenceIndex[1742] = [0, 2, 3, 5, 7, 8, 9, 11]
poemBySequenceIndex[1742] = '''B F G K M N P S'''
poemAuthors[1742] = ''''''
#1743: ䷝䷱ Distanced Holding: 2 1 2 2 1 1 2 1  : S R2 G2 M1 P D1 D2 N3 : Ifof

#1744: C D E♭ F G A♭ B♭ B :0 2 3 5 7 8 10 11: 1 2 ♭3 4 5 ♭6 ♭7 7: mima11(add ♭7 ♭13)
namesBySequenceIndex[1743] = '''Raga Suha Kanada,Bebop Natural Minor,Utility Minor,Gregorian 4,Maqam Nahawand (Iraq),Maqam Farahfaza (Iraq),Katodyllic'''
namesBySequenceIndexNegative[1743] = ''''''
semitonesBySequenceIndex[1743] = [0, 2, 3, 5, 7, 8, 10, 11]
poemBySequenceIndex[1743] = '''B F G K M N R S'''
poemAuthors[1743] = ''''''
#1744: ䷝䷸ Distanced Osmosis: 2 1 2 2 1 2 1 1  : S R2 G2 M1 P D1 N2 N3 : Ifom

#1745: C D E♭ F G A B♭ B :0 2 3 5 7 9 10 11: 1 2 ♭3 4 5 6 ♭7 7: mima13(add ♭7)
namesBySequenceIndex[1744] = '''Raga Bahar,Raga Sindhura,Raga Mian Ki Malhar,Goptyllic'''
namesBySequenceIndexNegative[1744] = ''''''
semitonesBySequenceIndex[1744] = [0, 2, 3, 5, 7, 9, 10, 11]
poemBySequenceIndex[1744] = '''B F G K M P R S'''
poemAuthors[1744] = ''''''
#1745: ䷝䷅ Distanced Arguing: 2 1 2 2 2 1 1 1  : S R2 G2 M1 P D2 N2 N3 : Ifeg

#1746: C D E♭ F G♯ A B♭ B :0 2 3 5 8 9 10 11: 1 2 ♭3 4 ♯5 6 ♭7 7: mima13(♯5 add ♭7)
namesBySequenceIndex[1745] = '''Styryllic'''
namesBySequenceIndexNegative[1745] = ''''''
semitonesBySequenceIndex[1745] = [0, 2, 3, 5, 8, 9, 10, 11]
poemBySequenceIndex[1745] = '''B F G K N P R S'''
poemAuthors[1745] = ''''''
#1746: ䷝䷠ Distanced Retiring: 2 1 2 3 1 1 1 1  : S R2 G2 M1 D1 D2 N2 N3 : Ifig

#1747: C D E♭ F♯ G A♭ A B♭ :0 2 3 6 7 8 9 10: 1 2 ♭3 ♯4 5 ♭6 6 ♭7: mi13(♯11 add ♭13)
namesBySequenceIndex[1746] = '''Raga Chinthamani,Staryllic'''
namesBySequenceIndexNegative[1746] = ''''''
semitonesBySequenceIndex[1746] = [0, 2, 3, 6, 7, 8, 9, 10]
poemBySequenceIndex[1746] = '''B F G L M N P R'''
poemAuthors[1746] = ''''''
#1747: ䷶䷪ Abounding Decision: 2 1 3 1 1 1 1 2  : S R2 G2 M2 P D1 D2 N2 : IbOd

#1748: C D E♭ F♯ G A♭ A B :0 2 3 6 7 8 9 11: 1 2 ♭3 ♯4 5 ♭6 6 7: mima13(♯11 add ♭13)
namesBySequenceIndex[1747] = '''Gyptyllic'''
namesBySequenceIndexNegative[1747] = ''''''
semitonesBySequenceIndex[1747] = [0, 2, 3, 6, 7, 8, 9, 11]
poemBySequenceIndex[1747] = '''B F G L M N P S'''
poemAuthors[1747] = ''''''
#1748: ䷶䷍ Abounding Resevoir: 2 1 3 1 1 1 2 1  : S R2 G2 M2 P D1 D2 N3 : IbOf

#1749: C D E♭ F♯ G A♭ B♭ B :0 2 3 6 7 8 10 11: 1 2 ♭3 ♯4 5 ♭6 ♭7 7: mima9(add ♭7 ♯11 ♭13)
namesBySequenceIndex[1748] = '''Thadyllic'''
namesBySequenceIndexNegative[1748] = ''''''
semitonesBySequenceIndex[1748] = [0, 2, 3, 6, 7, 8, 10, 11]
poemBySequenceIndex[1748] = '''B F G L M N R S'''
poemAuthors[1748] = ''''''
#1749: ䷶䷈ Abounding Homegrown: 2 1 3 1 1 2 1 1  : S R2 G2 M2 P D1 N2 N3 : IbOm

#1750: C D E♭ F♯ G A B♭ B :0 2 3 6 7 9 10 11: 1 2 ♭3 ♯4 5 6 ♭7 7: mima13(♯11 add ♭7)
namesBySequenceIndex[1749] = '''Eporyllic'''
namesBySequenceIndexNegative[1749] = ''''''
semitonesBySequenceIndex[1749] = [0, 2, 3, 6, 7, 9, 10, 11]
poemBySequenceIndex[1749] = '''B F G L M P R S'''
poemAuthors[1749] = ''''''
#1750: ䷶䷉ Abounding Treading: 2 1 3 1 2 1 1 1  : S R2 G2 M2 P D2 N2 N3 : IbEg

#1751: C D E♭ F♯ G♯ A B♭ B :0 2 3 6 8 9 10 11: 1 2 ♭3 ♯4 ♯5 6 ♭7 7: °ma13(♯11 add ♭7 ♭13)
namesBySequenceIndex[1750] = '''Epaphyllic'''
namesBySequenceIndexNegative[1750] = ''''''
semitonesBySequenceIndex[1750] = [0, 2, 3, 6, 8, 9, 10, 11]
poemBySequenceIndex[1750] = '''B F G L N P R S'''
poemAuthors[1750] = ''''''
#1751: ䷶䷌ Abounding Fellowship: 2 1 3 2 1 1 1 1  : S R2 G2 M2 D1 D2 N2 N3 : IbIg

#1752: C D E♭ G A♭ A B♭ B :0 2 3 7 8 9 10 11: 1 2 ♭3 5 ♭6 6 ♭7 7: mima9(add ♭7 ♭13 13)
namesBySequenceIndex[1751] = '''Phrycryllic'''
namesBySequenceIndexNegative[1751] = ''''''
semitonesBySequenceIndex[1751] = [0, 2, 3, 7, 8, 9, 10, 11]
poemBySequenceIndex[1751] = '''B F G M N P R S'''
poemAuthors[1751] = ''''''
#1752: ䷶䷫ Abounding Coupling: 2 1 4 1 1 1 1 1  : S R2 G2 P D1 D2 N2 N3 : Ibog

#1753: C D E F F♯ G A♭ A :0 2 4 5 6 7 8 9: 1 2 3 4 ♯4 5 ♭6 6: 13(no 7 add ♭6 ♯11)
namesBySequenceIndex[1752] = '''Stydyllic'''
namesBySequenceIndexNegative[1752] = ''''''
semitonesBySequenceIndex[1752] = [0, 2, 4, 5, 6, 7, 8, 9]
poemBySequenceIndex[1752] = '''B F J K L M N P'''
poemAuthors[1752] = ''''''
#1753: ䷤䷡ Family Might: 2 2 1 1 1 1 1 3  : S R2 G3 M1 M2 P D1 N1 : ImOb

#1754: C D E F F♯ G A♭ B♭ :0 2 4 5 6 7 8 10: 1 2 3 4 ♯4 5 ♭6 ♭7: 11(add ♯11 ♭13)
namesBySequenceIndex[1753] = '''Sodyllic'''
namesBySequenceIndexNegative[1753] = ''''''
semitonesBySequenceIndex[1753] = [0, 2, 4, 5, 6, 7, 8, 10]
poemBySequenceIndex[1753] = '''B F J K L M N R'''
poemAuthors[1753] = ''''''
#1754: ䷤䷄ Family Biding: 2 2 1 1 1 1 2 2  : S R2 G3 M1 M2 P D1 N2 : ImOk

#1755: C D E F F♯ G A♭ B :0 2 4 5 6 7 8 11: 1 2 3 4 ♯4 5 ♭6 7: ma11(add ♯11 ♭13)
namesBySequenceIndex[1754] = '''Rydyllic'''
namesBySequenceIndexNegative[1754] = ''''''
semitonesBySequenceIndex[1754] = [0, 2, 4, 5, 6, 7, 8, 11]
poemBySequenceIndex[1754] = '''B F J K L M N S'''
poemAuthors[1754] = ''''''
#1755: ䷤䷙ Family Cultivating: 2 2 1 1 1 1 3 1  : S R2 G3 M1 M2 P D1 N3 : ImOl

#1756: C D E F F♯ G A B♭ :0 2 4 5 6 7 9 10: 1 2 3 4 ♯4 5 6 ♭7: 13(add ♯11)
namesBySequenceIndex[1755] = '''Gocryllic'''
namesBySequenceIndexNegative[1755] = ''''''
semitonesBySequenceIndex[1755] = [0, 2, 4, 5, 6, 7, 9, 10]
poemBySequenceIndex[1755] = '''B F J K L M P R'''
poemAuthors[1755] = ''''''
#1756: ䷤䷹ Family Joyous: 2 2 1 1 1 2 1 2  : S R2 G3 M1 M2 P D2 N2 : ImEd

#1757: C D E F F♯ G A B :0 2 4 5 6 7 9 11: 1 2 3 4 ♯4 5 6 7: ma13(add ♯11)
namesBySequenceIndex[1756] = '''Ichikotsucho,Major-Lydian Mixed,Gregorian 5,Genus Diatonicum Veterum Correctum,Kubilai (Mongolia),Ishikotsucho (Japan),Raga Bihag,Raga Gaud Sarang,Raga Hamir Kalyani,Raga Bihag (descending),Raga Kedar,Raga Yaman/Kalyan,Raga Chayanat,Zylyllic'''
namesBySequenceIndexNegative[1756] = ''''''
semitonesBySequenceIndex[1756] = [0, 2, 4, 5, 6, 7, 9, 11]
poemBySequenceIndex[1756] = '''B F J K L M P S'''
poemAuthors[1756] = ''''''
#1757: ䷤䷥ Family Polarising: 2 2 1 1 1 2 2 1  : S R2 G3 M1 M2 P D2 N3 : ImEf

#1758: C D E F F♯ G A♯ B :0 2 4 5 6 7 10 11: 1 2 3 4 ♯4 5 ♯6 7: ma11(add ♭7 ♯11)
namesBySequenceIndex[1757] = '''Katynyllic'''
namesBySequenceIndexNegative[1757] = ''''''
semitonesBySequenceIndex[1757] = [0, 2, 4, 5, 6, 7, 10, 11]
poemBySequenceIndex[1757] = '''B F J K L M R S'''
poemAuthors[1757] = ''''''
#1758: ䷤䷼ Family Truth: 2 2 1 1 1 3 1 1  : S R2 G3 M1 M2 P D3 N3 : ImEm

#1759: C D E F G♭ A♭ A B♭ :0 2 4 5 6 8 9 10: 1 2 3 4 ♭5 ♭6 6 ♭7: 13(♯5 add ♯11)
namesBySequenceIndex[1758] = '''Epicryllic'''
namesBySequenceIndexNegative[1758] = ''''''
semitonesBySequenceIndex[1758] = [0, 2, 4, 5, 6, 8, 9, 10]
poemBySequenceIndex[1758] = '''B F J K L N P R'''
poemAuthors[1758] = ''''''
#1759: ䷤䷰ Family Revolution: 2 2 1 1 2 1 1 2  : S R2 G3 M1 M2 D1 D2 N2 : ImId

#1760: C D E F G♭ A♭ A B :0 2 4 5 6 8 9 11: 1 2 3 4 ♭5 ♭6 6 7: ma13(♯5 add ♯11)
namesBySequenceIndex[1759] = ''''''
namesBySequenceIndexNegative[1759] = ''''''
semitonesBySequenceIndex[1759] = [0, 2, 4, 5, 6, 8, 9, 11]
poemBySequenceIndex[1759] = '''B F J K L N P S'''
poemAuthors[1759] = ''''''
#1760: ䷤䷝ Family Distanced: 2 2 1 1 2 1 2 1  : S R2 G3 M1 M2 D1 D2 N3 : ImIf

#1761: C D E F G♭ A♭ B♭ B :0 2 4 5 6 8 10 11: 1 2 3 4 ♭5 ♭6 ♭7 7: ma11(♯5 add ♭7 ♯11)
namesBySequenceIndex[1760] = '''Messiaen 6th Mode,Roryllic'''
namesBySequenceIndexNegative[1760] = ''''''
semitonesBySequenceIndex[1760] = [0, 2, 4, 5, 6, 8, 10, 11]
poemBySequenceIndex[1760] = '''B F J K L N R S'''
poemAuthors[1760] = ''''''
#1761: ䷤䷤ Family Family : 2 2 1 1 2 2 1 1  : S R2 G3 M1 M2 D1 N2 N3 : ImIm

#1762: C D E F G♭ A B♭ B :0 2 4 5 6 9 10 11: 1 2 3 4 ♭5 6 ♭7 7: ma13(♭5 add ♭7)
namesBySequenceIndex[1761] = '''Bagyllic'''
namesBySequenceIndexNegative[1761] = ''''''
semitonesBySequenceIndex[1761] = [0, 2, 4, 5, 6, 9, 10, 11]
poemBySequenceIndex[1761] = '''B F J K L P R S'''
poemAuthors[1761] = ''''''
#1762: ䷤䷘ Family Integrity: 2 2 1 1 3 1 1 1  : S R2 G3 M1 M2 D2 N2 N3 : ImAg

#1763: C D E F G A♭ A B♭ :0 2 4 5 7 8 9 10: 1 2 3 4 5 ♭6 6 ♭7: 13(add ♭13)
namesBySequenceIndex[1762] = '''Zyryllic'''
namesBySequenceIndexNegative[1762] = ''''''
semitonesBySequenceIndex[1762] = [0, 2, 4, 5, 7, 8, 9, 10]
poemBySequenceIndex[1762] = '''B F J K M N P R'''
poemAuthors[1762] = ''''''
#1763: ䷤䷛ Family Excess: 2 2 1 2 1 1 1 2  : S R2 G3 M1 P D1 D2 N2 : Imod

#1764: C D E F G A♭ A B :0 2 4 5 7 8 9 11: 1 2 3 4 5 ♭6 6 7: ma13(add ♭13)
namesBySequenceIndex[1763] = '''Bebop Major,Ionoptyllic'''
namesBySequenceIndexNegative[1763] = ''''''
semitonesBySequenceIndex[1763] = [0, 2, 4, 5, 7, 8, 9, 11]
poemBySequenceIndex[1763] = '''B F J K M N P S'''
poemAuthors[1763] = ''''''
#1764: ䷤䷱ Family Holding: 2 2 1 2 1 1 2 1  : S R2 G3 M1 P D1 D2 N3 : Imof

#1765: C D E F G A♭ B♭ B :0 2 4 5 7 8 10 11: 1 2 3 4 5 ♭6 ♭7 7: ma11(add ♭7 ♭13)
namesBySequenceIndex[1764] = '''Stogyllic'''
namesBySequenceIndexNegative[1764] = ''''''
semitonesBySequenceIndex[1764] = [0, 2, 4, 5, 7, 8, 10, 11]
poemBySequenceIndex[1764] = '''B F J K M N R S'''
poemAuthors[1764] = ''''''
#1765: ䷤䷸ Family Osmosis: 2 2 1 2 1 2 1 1  : S R2 G3 M1 P D1 N2 N3 : Imom

#1766: C D E F G A B♭ B :0 2 4 5 7 9 10 11: 1 2 3 4 5 6 ♭7 7: ma13(add ♭7)
namesBySequenceIndex[1765] = '''Bebop,Bebop Dominant,Bebop Mixolydian,Major Mixolydian,Gregorian 6,Genus Diatonicum,Chinese Eight-Tone,Rast (Greece),Maqam Shawq Awir (Iraq),Raga Khamaj,Raga Desh Malhar,Raga Alhaiya Bilaval,Raga Bihagara,Raga Devagandhari,Aerycryllic'''
namesBySequenceIndexNegative[1765] = ''''''
semitonesBySequenceIndex[1765] = [0, 2, 4, 5, 7, 9, 10, 11]
poemBySequenceIndex[1765] = '''B F J K M P R S'''
poemAuthors[1765] = ''''''
#1766: ䷤䷅ Family Arguing: 2 2 1 2 2 1 1 1  : S R2 G3 M1 P D2 N2 N3 : Imeg

#1767: C D E F G♯ A B♭ B :0 2 4 5 8 9 10 11: 1 2 3 4 ♯5 6 ♭7 7: ma13(♯5 add ♭7)
namesBySequenceIndex[1766] = '''Phrocryllic'''
namesBySequenceIndexNegative[1766] = ''''''
semitonesBySequenceIndex[1766] = [0, 2, 4, 5, 8, 9, 10, 11]
poemBySequenceIndex[1766] = '''B F J K N P R S'''
poemAuthors[1766] = ''''''
#1767: ䷤䷠ Family Retiring: 2 2 1 3 1 1 1 1  : S R2 G3 M1 D1 D2 N2 N3 : Imig

#1768: C D E F♯ G A♭ A B♭ :0 2 4 6 7 8 9 10: 1 2 3 ♯4 5 ♭6 6 ♭7: 13(♯11 add ♭13)
namesBySequenceIndex[1767] = '''Gygyllic'''
namesBySequenceIndexNegative[1767] = ''''''
semitonesBySequenceIndex[1767] = [0, 2, 4, 6, 7, 8, 9, 10]
poemBySequenceIndex[1767] = '''B F J L M N P R'''
poemAuthors[1767] = ''''''
#1768: ䷾䷪ Completed Decision: 2 2 2 1 1 1 1 2  : S R2 G3 M2 P D1 D2 N2 : IkOd

#1769: C D E F♯ G A♭ A B :0 2 4 6 7 8 9 11: 1 2 3 ♯4 5 ♭6 6 7: ma13(♯11 add ♭13)
namesBySequenceIndex[1768] = '''Ionocryllic'''
namesBySequenceIndexNegative[1768] = ''''''
semitonesBySequenceIndex[1768] = [0, 2, 4, 6, 7, 8, 9, 11]
poemBySequenceIndex[1768] = '''B F J L M N P S'''
poemAuthors[1768] = ''''''
#1769: ䷾䷍ Completed Resevoir: 2 2 2 1 1 1 2 1  : S R2 G3 M2 P D1 D2 N3 : IkOf

#1770: C D E F♯ G A♭ B♭ B :0 2 4 6 7 8 10 11: 1 2 3 ♯4 5 ♭6 ♭7 7: ma9(add ♭7 ♯11 ♭13)
namesBySequenceIndex[1769] = '''Racryllic'''
namesBySequenceIndexNegative[1769] = ''''''
semitonesBySequenceIndex[1769] = [0, 2, 4, 6, 7, 8, 10, 11]
poemBySequenceIndex[1769] = '''B F J L M N R S'''
poemAuthors[1769] = ''''''
#1770: ䷾䷈ Completed Homegrown: 2 2 2 1 1 2 1 1  : S R2 G3 M2 P D1 N2 N3 : IkOm

#1771: C D E F♯ G A B♭ B :0 2 4 6 7 9 10 11: 1 2 3 ♯4 5 6 ♭7 7: ma13(♯11 add ♭7)
namesBySequenceIndex[1770] = '''Rocryllic'''
namesBySequenceIndexNegative[1770] = ''''''
semitonesBySequenceIndex[1770] = [0, 2, 4, 6, 7, 9, 10, 11]
poemBySequenceIndex[1770] = '''B F J L M P R S'''
poemAuthors[1770] = ''''''
#1771: ䷾䷉ Completed Treading: 2 2 2 1 2 1 1 1  : S R2 G3 M2 P D2 N2 N3 : IkEg

#1772: C D E F♯ G♯ A B♭ B :0 2 4 6 8 9 10 11: 1 2 3 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♯11 add ♭7)
namesBySequenceIndex[1771] = '''Thyryllic'''
namesBySequenceIndexNegative[1771] = ''''''
semitonesBySequenceIndex[1771] = [0, 2, 4, 6, 8, 9, 10, 11]
poemBySequenceIndex[1771] = '''B F J L N P R S'''
poemAuthors[1771] = ''''''
#1772: ䷾䷌ Completed Fellowship: 2 2 2 2 1 1 1 1  : S R2 G3 M2 D1 D2 N2 N3 : IkIg

#1773: C D E G A♭ A B♭ B :0 2 4 7 8 9 10 11: 1 2 3 5 ♭6 6 ♭7 7: ma9(add ♭7 ♭13 13)
namesBySequenceIndex[1772] = '''Sythyllic'''
namesBySequenceIndexNegative[1772] = ''''''
semitonesBySequenceIndex[1772] = [0, 2, 4, 7, 8, 9, 10, 11]
poemBySequenceIndex[1772] = '''B F J M N P R S'''
poemAuthors[1772] = ''''''
#1773: ䷾䷫ Completed Coupling: 2 2 3 1 1 1 1 1  : S R2 G3 P D1 D2 N2 N3 : Ikog

#1774: C D F F♯ G A♭ A B♭ :0 2 5 6 7 8 9 10: 1 2 4 ♯4 5 ♭6 6 ♭7: 13sus♯4(add ♭13)
namesBySequenceIndex[1773] = '''Katycryllic'''
namesBySequenceIndexNegative[1773] = ''''''
semitonesBySequenceIndex[1773] = [0, 2, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1773] = '''B F K L M N P R'''
poemAuthors[1773] = ''''''
#1774: ䷕䷪ Adorning Decision: 2 3 1 1 1 1 1 2  : S R2 M1 M2 P D1 D2 N2 : IlOd

#1775: C D F F♯ G A♭ A B :0 2 5 6 7 8 9 11: 1 2 4 ♯4 5 ♭6 6 7: ma13sus♯4(add ♭13)
namesBySequenceIndex[1774] = '''Raptyllic'''
namesBySequenceIndexNegative[1774] = ''''''
semitonesBySequenceIndex[1774] = [0, 2, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1774] = '''B F K L M N P S'''
poemAuthors[1774] = ''''''
#1775: ䷕䷍ Adorning Resevoir: 2 3 1 1 1 1 2 1  : S R2 M1 M2 P D1 D2 N3 : IlOf

#1776: C D F F♯ G A♭ B♭ B :0 2 5 6 7 8 10 11: 1 2 4 ♯4 5 ♭6 ♭7 7: ma11sus♯4(add ♭7 ♭13)
namesBySequenceIndex[1775] = ''''''
namesBySequenceIndexNegative[1775] = ''''''
semitonesBySequenceIndex[1775] = [0, 2, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1775] = '''B F K L M N R S'''
poemAuthors[1775] = ''''''
#1776: ䷕䷈ Adorning Homegrown: 2 3 1 1 1 2 1 1  : S R2 M1 M2 P D1 N2 N3 : IlOm

#1777: C D F F♯ G A B♭ B :0 2 5 6 7 9 10 11: 1 2 4 ♯4 5 6 ♭7 7: ma13sus♯4(add ♭7)
namesBySequenceIndex[1776] = '''Aeologyllic'''
namesBySequenceIndexNegative[1776] = ''''''
semitonesBySequenceIndex[1776] = [0, 2, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1776] = '''B F K L M P R S'''
poemAuthors[1776] = ''''''
#1777: ䷕䷉ Adorning Treading: 2 3 1 1 2 1 1 1  : S R2 M1 M2 P D2 N2 N3 : IlEg

#1778: C D F G♭ A♭ A B♭ B :0 2 5 6 8 9 10 11: 1 2 4 ♭5 ♭6 6 ♭7 7: ma13sus♯4(♯5 add ♭7)
namesBySequenceIndex[1777] = '''Stathyllic'''
namesBySequenceIndexNegative[1777] = ''''''
semitonesBySequenceIndex[1777] = [0, 2, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1777] = '''B F K L N P R S'''
poemAuthors[1777] = ''''''
#1778: ䷕䷌ Adorning Fellowship: 2 3 1 2 1 1 1 1  : S R2 M1 M2 D1 D2 N2 N3 : IlIg

#1779: C D F G A♭ A B♭ B :0 2 5 7 8 9 10 11: 1 2 4 5 ♭6 6 ♭7 7: ma13(no 3 add ♭7 ♭13)
namesBySequenceIndex[1778] = ''''''
namesBySequenceIndexNegative[1778] = ''''''
semitonesBySequenceIndex[1778] = [0, 2, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1778] = '''B F K M N P R S'''
poemAuthors[1778] = ''''''
#1779: ䷕䷫ Adorning Coupling: 2 3 2 1 1 1 1 1  : S R2 M1 P D1 D2 N2 N3 : Ilog

#1780: C D F♯ G A♭ A B♭ B :0 2 6 7 8 9 10 11: 1 2 ♯4 5 ♭6 6 ♭7 7: ma13(♯11 no 3 add ♭7 ♭13)
namesBySequenceIndex[1779] = '''Ionyllic'''
namesBySequenceIndexNegative[1779] = ''''''
semitonesBySequenceIndex[1779] = [0, 2, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1779] = '''B F L M N P R S'''
poemAuthors[1779] = ''''''
#1780: ䷣䷀ Eclipse Creative: 2 4 1 1 1 1 1 1  : S R2 M2 P D1 D2 N2 N3 : IjOg

#1781: C D♯ E F F♯ G A♭ A :0 3 4 5 6 7 8 9: 1 ♯2 3 4 ♯4 5 ♭6 6: 6(add ♭6 ♯9 11 ♯11)
namesBySequenceIndex[1780] = '''Dythyllic'''
namesBySequenceIndexNegative[1780] = ''''''
semitonesBySequenceIndex[1780] = [0, 3, 4, 5, 6, 7, 8, 9]
poemBySequenceIndex[1780] = '''B G J K L M N P'''
poemAuthors[1780] = ''''''
#1781: ䷘䷡ Integrity Might: 3 1 1 1 1 1 1 3  : S R3 G3 M1 M2 P D1 N1 : AgOb

#1782: C D♯ E F F♯ G A♭ B♭ :0 3 4 5 6 7 8 10: 1 ♯2 3 4 ♯4 5 ♭6 ♭7: 11(♯9 add ♯11 ♭13)
namesBySequenceIndex[1781] = '''Kataryllic'''
namesBySequenceIndexNegative[1781] = ''''''
semitonesBySequenceIndex[1781] = [0, 3, 4, 5, 6, 7, 8, 10]
poemBySequenceIndex[1781] = '''B G J K L M N R'''
poemAuthors[1781] = ''''''
#1782: ䷘䷄ Integrity Biding: 3 1 1 1 1 1 2 2  : S R3 G3 M1 M2 P D1 N2 : AgOk

#1783: C D♯ E F F♯ G A♭ B :0 3 4 5 6 7 8 11: 1 ♯2 3 4 ♯4 5 ♭6 7: ma11(♯9 add ♯11 ♭13)
namesBySequenceIndex[1782] = '''Aeolaptyllic'''
namesBySequenceIndexNegative[1782] = ''''''
semitonesBySequenceIndex[1782] = [0, 3, 4, 5, 6, 7, 8, 11]
poemBySequenceIndex[1782] = '''B G J K L M N S'''
poemAuthors[1782] = ''''''
#1783: ䷘䷙ Integrity Cultivating: 3 1 1 1 1 1 3 1  : S R3 G3 M1 M2 P D1 N3 : AgOl

#1784: C D♯ E F F♯ G A B♭ :0 3 4 5 6 7 9 10: 1 ♯2 3 4 ♯4 5 6 ♭7: 13(♯9 add ♯11)
namesBySequenceIndex[1783] = '''Tharyllic'''
namesBySequenceIndexNegative[1783] = ''''''
semitonesBySequenceIndex[1783] = [0, 3, 4, 5, 6, 7, 9, 10]
poemBySequenceIndex[1783] = '''B G J K L M P R'''
poemAuthors[1783] = ''''''
#1784: ䷘䷹ Integrity Joyous: 3 1 1 1 1 2 1 2  : S R3 G3 M1 M2 P D2 N2 : AgEd

#1785: C D♯ E F F♯ G A B :0 3 4 5 6 7 9 11: 1 ♯2 3 4 ♯4 5 6 7: ma13(♯9 add ♯11)
namesBySequenceIndex[1784] = '''Gythyllic'''
namesBySequenceIndexNegative[1784] = ''''''
semitonesBySequenceIndex[1784] = [0, 3, 4, 5, 6, 7, 9, 11]
poemBySequenceIndex[1784] = '''B G J K L M P S'''
poemAuthors[1784] = ''''''
#1785: ䷘䷥ Integrity Polarising: 3 1 1 1 1 2 2 1  : S R3 G3 M1 M2 P D2 N3 : AgEf

#1786: C D♯ E F F♯ G A♯ B :0 3 4 5 6 7 10 11: 1 ♯2 3 4 ♯4 5 ♯6 7: ma11(♯9 add ♭7 ♯11)
namesBySequenceIndex[1785] = '''Epagyllic'''
namesBySequenceIndexNegative[1785] = ''''''
semitonesBySequenceIndex[1785] = [0, 3, 4, 5, 6, 7, 10, 11]
poemBySequenceIndex[1785] = '''B G J K L M R S'''
poemAuthors[1785] = ''''''
#1786: ䷘䷼ Integrity Truth: 3 1 1 1 1 3 1 1  : S R3 G3 M1 M2 P D3 N3 : AgEm

#1787: C D♯ E F G♭ A♭ A B♭ :0 3 4 5 6 8 9 10: 1 ♯2 3 4 ♭5 ♭6 6 ♭7: 13(♯5 ♯9 add ♯11)
namesBySequenceIndex[1786] = ''''''
namesBySequenceIndexNegative[1786] = ''''''
semitonesBySequenceIndex[1786] = [0, 3, 4, 5, 6, 8, 9, 10]
poemBySequenceIndex[1786] = '''B G J K L N P R'''
poemAuthors[1786] = ''''''
#1787: ䷘䷰ Integrity Revolution: 3 1 1 1 2 1 1 2  : S R3 G3 M1 M2 D1 D2 N2 : AgId

#1788: C D♯ E F G♭ A♭ A B :0 3 4 5 6 8 9 11: 1 ♯2 3 4 ♭5 ♭6 6 7: ma13(♯5 ♯9 add ♯11)
namesBySequenceIndex[1787] = '''Aeolathyllic'''
namesBySequenceIndexNegative[1787] = ''''''
semitonesBySequenceIndex[1787] = [0, 3, 4, 5, 6, 8, 9, 11]
poemBySequenceIndex[1787] = '''B G J K L N P S'''
poemAuthors[1787] = ''''''
#1788: ䷘䷝ Integrity Distanced: 3 1 1 1 2 1 2 1  : S R3 G3 M1 M2 D1 D2 N3 : AgIf

#1789: C D♯ E F G♭ A♭ B♭ B :0 3 4 5 6 8 10 11: 1 ♯2 3 4 ♭5 ♭6 ♭7 7: ma11(♯5 ♯9 add ♭7 ♯11)
namesBySequenceIndex[1788] = '''Bacryllic'''
namesBySequenceIndexNegative[1788] = ''''''
semitonesBySequenceIndex[1788] = [0, 3, 4, 5, 6, 8, 10, 11]
poemBySequenceIndex[1788] = '''B G J K L N R S'''
poemAuthors[1788] = ''''''
#1789: ䷘䷤ Integrity Family: 3 1 1 1 2 2 1 1  : S R3 G3 M1 M2 D1 N2 N3 : AgIm

#1790: C D♯ E F G♭ A B♭ B :0 3 4 5 6 9 10 11: 1 ♯2 3 4 ♭5 6 ♭7 7: ma13(♭5 ♯9 add ♭7)
namesBySequenceIndex[1789] = '''Sydyllic'''
namesBySequenceIndexNegative[1789] = ''''''
semitonesBySequenceIndex[1789] = [0, 3, 4, 5, 6, 9, 10, 11]
poemBySequenceIndex[1789] = '''B G J K L P R S'''
poemAuthors[1789] = ''''''
#1790: ䷘䷘ Integrity Integrity : 3 1 1 1 3 1 1 1  : S R3 G3 M1 M2 D2 N2 N3 : AgAg

#1791: C D♯ E F G A♭ A B♭ :0 3 4 5 7 8 9 10: 1 ♯2 3 4 5 ♭6 6 ♭7: 13(♯9 add ♭13)
namesBySequenceIndex[1790] = '''Dagyllic'''
namesBySequenceIndexNegative[1790] = ''''''
semitonesBySequenceIndex[1790] = [0, 3, 4, 5, 7, 8, 9, 10]
poemBySequenceIndex[1790] = '''B G J K M N P R'''
poemAuthors[1790] = ''''''
#1791: ䷘䷛ Integrity Excess: 3 1 1 2 1 1 1 2  : S R3 G3 M1 P D1 D2 N2 : Agod

#1792: C D♯ E F G A♭ A B :0 3 4 5 7 8 9 11: 1 ♯2 3 4 5 ♭6 6 7: ma13(♯9 add ♭13)
namesBySequenceIndex[1791] = '''Lonyllic'''
namesBySequenceIndexNegative[1791] = ''''''
semitonesBySequenceIndex[1791] = [0, 3, 4, 5, 7, 8, 9, 11]
poemBySequenceIndex[1791] = '''B G J K M N P S'''
poemAuthors[1791] = ''''''
#1792: ䷘䷱ Integrity Holding: 3 1 1 2 1 1 2 1  : S R3 G3 M1 P D1 D2 N3 : Agof

#1793: C D♯ E F G A♭ B♭ B :0 3 4 5 7 8 10 11: 1 ♯2 3 4 5 ♭6 ♭7 7: ma11(♯9 add ♭7 ♭13)
namesBySequenceIndex[1792] = '''Dydyllic'''
namesBySequenceIndexNegative[1792] = ''''''
semitonesBySequenceIndex[1792] = [0, 3, 4, 5, 7, 8, 10, 11]
poemBySequenceIndex[1792] = '''B G J K M N R S'''
poemAuthors[1792] = ''''''
#1793: ䷘䷸ Integrity Osmosis: 3 1 1 2 1 2 1 1  : S R3 G3 M1 P D1 N2 N3 : Agom

#1794: C D♯ E F G A B♭ B :0 3 4 5 7 9 10 11: 1 ♯2 3 4 5 6 ♭7 7: ma13(♯9 add ♭7)
namesBySequenceIndex[1793] = '''Eponyllic'''
namesBySequenceIndexNegative[1793] = ''''''
semitonesBySequenceIndex[1793] = [0, 3, 4, 5, 7, 9, 10, 11]
poemBySequenceIndex[1793] = '''B G J K M P R S'''
poemAuthors[1793] = ''''''
#1794: ䷘䷅ Integrity Arguing: 3 1 1 2 2 1 1 1  : S R3 G3 M1 P D2 N2 N3 : Ageg

#1795: C D♯ E F G♯ A B♭ B :0 3 4 5 8 9 10 11: 1 ♯2 3 4 ♯5 6 ♭7 7: ma13(♯5 ♯9 add ♭7)
namesBySequenceIndex[1794] = '''Kalyllic'''
namesBySequenceIndexNegative[1794] = ''''''
semitonesBySequenceIndex[1794] = [0, 3, 4, 5, 8, 9, 10, 11]
poemBySequenceIndex[1794] = '''B G J K N P R S'''
poemAuthors[1794] = ''''''
#1795: ䷘䷠ Integrity Retiring: 3 1 1 3 1 1 1 1  : S R3 G3 M1 D1 D2 N2 N3 : Agig

#1796: C D♯ E F♯ G A♭ A B♭ :0 3 4 6 7 8 9 10: 1 ♯2 3 ♯4 5 ♭6 6 ♭7: 13(♯9 ♯11 add ♭13)
namesBySequenceIndex[1795] = '''Stacryllic'''
namesBySequenceIndexNegative[1795] = ''''''
semitonesBySequenceIndex[1795] = [0, 3, 4, 6, 7, 8, 9, 10]
poemBySequenceIndex[1795] = '''B G J L M N P R'''
poemAuthors[1795] = ''''''
#1796: ䷐䷪ Following Decision: 3 1 2 1 1 1 1 2  : S R3 G3 M2 P D1 D2 N2 : AdOd

#1797: C D♯ E F♯ G A♭ A B :0 3 4 6 7 8 9 11: 1 ♯2 3 ♯4 5 ♭6 6 7: ma13(♯9 ♯11 add ♭13)
namesBySequenceIndex[1796] = '''Doptyllic'''
namesBySequenceIndexNegative[1796] = ''''''
semitonesBySequenceIndex[1796] = [0, 3, 4, 6, 7, 8, 9, 11]
poemBySequenceIndex[1796] = '''B G J L M N P S'''
poemAuthors[1796] = ''''''
#1797: ䷐䷍ Following Resevoir: 3 1 2 1 1 1 2 1  : S R3 G3 M2 P D1 D2 N3 : AdOf

#1798: C D♯ E F♯ G A♭ B♭ B :0 3 4 6 7 8 10 11: 1 ♯2 3 ♯4 5 ♭6 ♭7 7: ma7(add ♭7 ♯9 ♯11 ♭13)
namesBySequenceIndex[1797] = '''Thyptyllic'''
namesBySequenceIndexNegative[1797] = ''''''
semitonesBySequenceIndex[1797] = [0, 3, 4, 6, 7, 8, 10, 11]
poemBySequenceIndex[1797] = '''B G J L M N R S'''
poemAuthors[1797] = ''''''
#1798: ䷐䷈ Following Homegrown: 3 1 2 1 1 2 1 1  : S R3 G3 M2 P D1 N2 N3 : AdOm

#1799: C D♯ E F♯ G A B♭ B :0 3 4 6 7 9 10 11: 1 ♯2 3 ♯4 5 6 ♭7 7: ma13(♯9 ♯11 add ♭7)
namesBySequenceIndex[1798] = '''Maptyllic'''
namesBySequenceIndexNegative[1798] = ''''''
semitonesBySequenceIndex[1798] = [0, 3, 4, 6, 7, 9, 10, 11]
poemBySequenceIndex[1798] = '''B G J L M P R S'''
poemAuthors[1798] = ''''''
#1799: ䷐䷉ Following Treading: 3 1 2 1 2 1 1 1  : S R3 G3 M2 P D2 N2 N3 : AdEg

#1800: C D♯ E F♯ G♯ A B♭ B :0 3 4 6 8 9 10 11: 1 ♯2 3 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♯9 ♯11 add ♭7)
namesBySequenceIndex[1799] = '''Aeolothyllic'''
namesBySequenceIndexNegative[1799] = ''''''
semitonesBySequenceIndex[1799] = [0, 3, 4, 6, 8, 9, 10, 11]
poemBySequenceIndex[1799] = '''B G J L N P R S'''
poemAuthors[1799] = ''''''
#1800: ䷐䷌ Following Fellowship: 3 1 2 2 1 1 1 1  : S R3 G3 M2 D1 D2 N2 N3 : AdIg

#1801: C D♯ E G A♭ A B♭ B :0 3 4 7 8 9 10 11: 1 ♯2 3 5 ♭6 6 ♭7 7: ma7(add ♭7 ♯9 ♭13 13)
namesBySequenceIndex[1800] = '''Ioniptyllic'''
namesBySequenceIndexNegative[1800] = ''''''
semitonesBySequenceIndex[1800] = [0, 3, 4, 7, 8, 9, 10, 11]
poemBySequenceIndex[1800] = '''B G J M N P R S'''
poemAuthors[1800] = ''''''
#1801: ䷐䷫ Following Coupling: 3 1 3 1 1 1 1 1  : S R3 G3 P D1 D2 N2 N3 : Adog

#1802: C E♭ F F♯ G A♭ A B♭ :0 3 5 6 7 8 9 10: 1 ♭3 4 ♯4 5 ♭6 6 ♭7: mi13(♯9 add ♯11 ♭13)
namesBySequenceIndex[1801] = ''''''
namesBySequenceIndexNegative[1801] = ''''''
semitonesBySequenceIndex[1801] = [0, 3, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1801] = '''B G K L M N P R'''
poemAuthors[1801] = ''''''
#1802: ䷔䷪ Biting Decision: 3 2 1 1 1 1 1 2  : S G2 M1 M2 P D1 D2 N2 : AfOd

#1803: C E♭ F F♯ G A♭ A B :0 3 5 6 7 8 9 11: 1 ♭3 4 ♯4 5 ♭6 6 7: mima13(♯9 add ♯11 ♭13)
namesBySequenceIndex[1802] = '''Aeronyllic'''
namesBySequenceIndexNegative[1802] = ''''''
semitonesBySequenceIndex[1802] = [0, 3, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1802] = '''B G K L M N P S'''
poemAuthors[1802] = ''''''
#1803: ䷔䷍ Biting Resevoir: 3 2 1 1 1 1 2 1  : S G2 M1 M2 P D1 D2 N3 : AfOf

#1804: C E♭ F F♯ G A♭ B♭ B :0 3 5 6 7 8 10 11: 1 ♭3 4 ♯4 5 ♭6 ♭7 7: mima11(♯9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[1803] = '''Pothyllic'''
namesBySequenceIndexNegative[1803] = ''''''
semitonesBySequenceIndex[1803] = [0, 3, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1803] = '''B G K L M N R S'''
poemAuthors[1803] = ''''''
#1804: ䷔䷈ Biting Homegrown: 3 2 1 1 1 2 1 1  : S G2 M1 M2 P D1 N2 N3 : AfOm

#1805: C E♭ F F♯ G A B♭ B :0 3 5 6 7 9 10 11: 1 ♭3 4 ♯4 5 6 ♭7 7: mima13(♯9 add ♭7 ♯11)
namesBySequenceIndex[1804] = '''Zoryllic'''
namesBySequenceIndexNegative[1804] = ''''''
semitonesBySequenceIndex[1804] = [0, 3, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1804] = '''B G K L M P R S'''
poemAuthors[1804] = ''''''
#1805: ䷔䷉ Biting Treading: 3 2 1 1 2 1 1 1  : S G2 M1 M2 P D2 N2 N3 : AfEg

#1806: C E♭ F G♭ A♭ A B♭ B :0 3 5 6 8 9 10 11: 1 ♭3 4 ♭5 ♭6 6 ♭7 7: °ma13(♯9 add ♭7 ♭13)
namesBySequenceIndex[1805] = '''Lydyllic'''
namesBySequenceIndexNegative[1805] = ''''''
semitonesBySequenceIndex[1805] = [0, 3, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1805] = '''B G K L N P R S'''
poemAuthors[1805] = ''''''
#1806: ䷔䷌ Biting Fellowship: 3 2 1 2 1 1 1 1  : S G2 M1 M2 D1 D2 N2 N3 : AfIg

#1807: C E♭ F G A♭ A B♭ B :0 3 5 7 8 9 10 11: 1 ♭3 4 5 ♭6 6 ♭7 7: mima13(♯9 add ♭7 ♭13)
namesBySequenceIndex[1806] = '''Phranyllic'''
namesBySequenceIndexNegative[1806] = ''''''
semitonesBySequenceIndex[1806] = [0, 3, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1806] = '''B G K M N P R S'''
poemAuthors[1806] = ''''''
#1807: ䷔䷫ Biting Coupling: 3 2 2 1 1 1 1 1  : S G2 M1 P D1 D2 N2 N3 : Afog

#1808: C E♭ F♯ G A♭ A B♭ B :0 3 6 7 8 9 10 11: 1 ♭3 ♯4 5 ♭6 6 ♭7 7: mima13(♯9 ♯11 add ♭7 ♭13)
namesBySequenceIndex[1807] = '''Zaryllic'''
namesBySequenceIndexNegative[1807] = ''''''
semitonesBySequenceIndex[1807] = [0, 3, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1807] = '''B G L M N P R S'''
poemAuthors[1807] = ''''''
#1808: ䷲䷀ Thunder Creative: 3 3 1 1 1 1 1 1  : S G2 M2 P D1 D2 N2 N3 : AbOg

#1809: C E F F♯ G A♭ A B♭ :0 4 5 6 7 8 9 10: 1 3 4 ♯4 5 ♭6 6 ♭7: 13(no 9 add ♯11 ♭13)
namesBySequenceIndex[1808] = '''Stolyllic'''
namesBySequenceIndexNegative[1808] = ''''''
semitonesBySequenceIndex[1808] = [0, 4, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1808] = '''B J K L M N P R'''
poemAuthors[1808] = ''''''
#1809: ䷩䷪ Augmenting Decision: 4 1 1 1 1 1 1 2  : S G3 M1 M2 P D1 D2 N2 : AmOd

#1810: C E F F♯ G A♭ A B :0 4 5 6 7 8 9 11: 1 3 4 ♯4 5 ♭6 6 7: ma13(no 9 add ♯11 ♭13)
namesBySequenceIndex[1809] = '''Phroryllic'''
namesBySequenceIndexNegative[1809] = ''''''
semitonesBySequenceIndex[1809] = [0, 4, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1809] = '''B J K L M N P S'''
poemAuthors[1809] = ''''''
#1810: ䷩䷍ Augmenting Resevoir: 4 1 1 1 1 1 2 1  : S G3 M1 M2 P D1 D2 N3 : AmOf

#1811: C E F F♯ G A♭ B♭ B :0 4 5 6 7 8 10 11: 1 3 4 ♯4 5 ♭6 ♭7 7: ma7(add ♭7 11 ♯11 ♭13)
namesBySequenceIndex[1810] = '''Aeoladyllic'''
namesBySequenceIndexNegative[1810] = ''''''
semitonesBySequenceIndex[1810] = [0, 4, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1810] = '''B J K L M N R S'''
poemAuthors[1810] = ''''''
#1811: ䷩䷈ Augmenting Homegrown: 4 1 1 1 1 2 1 1  : S G3 M1 M2 P D1 N2 N3 : AmOm

#1812: C E F F♯ G A B♭ B :0 4 5 6 7 9 10 11: 1 3 4 ♯4 5 6 ♭7 7: ma13(no 9 add ♭7 ♯11)
namesBySequenceIndex[1811] = '''Pynyllic'''
namesBySequenceIndexNegative[1811] = ''''''
semitonesBySequenceIndex[1811] = [0, 4, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1811] = '''B J K L M P R S'''
poemAuthors[1811] = ''''''
#1812: ䷩䷉ Augmenting Treading: 4 1 1 1 2 1 1 1  : S G3 M1 M2 P D2 N2 N3 : AmEg

#1813: C E F G♭ A♭ A B♭ B :0 4 5 6 8 9 10 11: 1 3 4 ♭5 ♭6 6 ♭7 7: ma13(♯5 no 9 add ♭7 ♯11)
namesBySequenceIndex[1812] = '''Thagyllic'''
namesBySequenceIndexNegative[1812] = ''''''
semitonesBySequenceIndex[1812] = [0, 4, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1812] = '''B J K L N P R S'''
poemAuthors[1812] = ''''''
#1813: ䷩䷌ Augmenting Fellowship: 4 1 1 2 1 1 1 1  : S G3 M1 M2 D1 D2 N2 N3 : AmIg

#1814: C E F G A♭ A B♭ B :0 4 5 7 8 9 10 11: 1 3 4 5 ♭6 6 ♭7 7: ma13(no 9 add ♭7 ♭13)
namesBySequenceIndex[1813] = '''Dolyllic'''
namesBySequenceIndexNegative[1813] = ''''''
semitonesBySequenceIndex[1813] = [0, 4, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1813] = '''B J K M N P R S'''
poemAuthors[1813] = ''''''
#1814: ䷩䷫ Augmenting Coupling: 4 1 2 1 1 1 1 1  : S G3 M1 P D1 D2 N2 N3 : Amog

#1815: C E F♯ G A♭ A B♭ B :0 4 6 7 8 9 10 11: 1 3 ♯4 5 ♭6 6 ♭7 7: ma7(add ♭7 ♯11 ♭13 13)
namesBySequenceIndex[1814] = '''Stycryllic'''
namesBySequenceIndexNegative[1814] = ''''''
semitonesBySequenceIndex[1814] = [0, 4, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1814] = '''B J L M N P R S'''
poemAuthors[1814] = ''''''
#1815: ䷂䷀ Sprouting Creative: 4 2 1 1 1 1 1 1  : S G3 M2 P D1 D2 N2 N3 : AkOg

#1816: C F F♯ G A♭ A B♭ B :0 5 6 7 8 9 10 11: 1 4 ♯4 5 ♭6 6 ♭7 7: ma13sus♯4(no 9 add ♭7 ♭13)
namesBySequenceIndex[1815] = ''''''
namesBySequenceIndexNegative[1815] = ''''''
semitonesBySequenceIndex[1815] = [0, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1815] = '''B K L M N P R S'''
poemAuthors[1815] = ''''''
#1816: ䷚䷀ Nourishment Creative: 5 1 1 1 1 1 1 1  : S M1 M2 P D1 D2 N2 N3 : AlOg

#1817: C D♭ D E♭ E F F♯ G A♭ :0 1 2 3 4 5 6 7 8: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭6: 11(no 7 add ♭6 ♭9 ♯9 ♯11)
namesBySequenceIndex[1816] = '''Polygic'''
namesBySequenceIndexNegative[1816] = ''''''
semitonesBySequenceIndex[1816] = [0, 1, 2, 3, 4, 5, 6, 7, 8]
poemBySequenceIndex[1816] = '''B D F G J K L M N'''
poemAuthors[1816] = ''''''
#1817: ䷀䷊ Creative Peace: 1 1 1 1 1 1 1 1 4  : S R1 R2 G2 G3 M1 M2 P D1 : OgOj

#1818: C D♭ D E♭ E F F♯ G A :0 1 2 3 4 5 6 7 9: 1 ♭2 2 ♭3 3 4 ♯4 5 6: 13(no 7 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1817] = '''Raptygic'''
namesBySequenceIndexNegative[1817] = ''''''
semitonesBySequenceIndex[1817] = [0, 1, 2, 3, 4, 5, 6, 7, 9]
poemBySequenceIndex[1817] = '''B D F G J K L M P'''
poemAuthors[1817] = ''''''
#1818: ䷀䷵ Creative Betrothing: 1 1 1 1 1 1 1 2 3  : S R1 R2 G2 G3 M1 M2 P D2 : OgEb

#1819: C D♭ D E♭ E F F♯ G B♭ :0 1 2 3 4 5 6 7 10: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭7: 11(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1818] = '''Sarygic'''
namesBySequenceIndexNegative[1818] = ''''''
semitonesBySequenceIndex[1818] = [0, 1, 2, 3, 4, 5, 6, 7, 10]
poemBySequenceIndex[1818] = '''B D F G J K L M R'''
poemAuthors[1818] = ''''''
#1819: ䷀䷻ Creative Composed: 1 1 1 1 1 1 1 3 2  : S R1 R2 G2 G3 M1 M2 P N2 : OgEk

#1820: C D♭ D E♭ E F F♯ G B :0 1 2 3 4 5 6 7 11: 1 ♭2 2 ♭3 3 4 ♯4 5 7: ma11(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1819] = '''Stanygic'''
namesBySequenceIndexNegative[1819] = ''''''
semitonesBySequenceIndex[1819] = [0, 1, 2, 3, 4, 5, 6, 7, 11]
poemBySequenceIndex[1819] = '''B D F G J K L M S'''
poemAuthors[1819] = ''''''
#1820: ䷀䷨ Creative Diminishing: 1 1 1 1 1 1 1 4 1  : S R1 R2 G2 G3 M1 M2 P N3 : OgEl

#1821: C D♭ D E♭ E F G♭ A♭ A :0 1 2 3 4 5 6 8 9: 1 ♭2 2 ♭3 3 4 ♭5 ♭6 6: 13(♯5 no 7 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1820] = '''Aeolathygic'''
namesBySequenceIndexNegative[1820] = ''''''
semitonesBySequenceIndex[1820] = [0, 1, 2, 3, 4, 5, 6, 8, 9]
poemBySequenceIndex[1820] = '''B D F G J K L N P'''
poemAuthors[1820] = ''''''
#1821: ䷀䷶ Creative Abounding: 1 1 1 1 1 1 2 1 3  : S R1 R2 G2 G3 M1 M2 D1 N1 : OgIb

#1822: C D♭ D E♭ E F G♭ A♭ B♭ :0 1 2 3 4 5 6 8 10: 1 ♭2 2 ♭3 3 4 ♭5 ♭6 ♭7: 11(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1821] = '''Tharygic'''
namesBySequenceIndexNegative[1821] = ''''''
semitonesBySequenceIndex[1821] = [0, 1, 2, 3, 4, 5, 6, 8, 10]
poemBySequenceIndex[1821] = '''B D F G J K L N R'''
poemAuthors[1821] = ''''''
#1822: ䷀䷾ Creative Completed: 1 1 1 1 1 1 2 2 2  : S R1 R2 G2 G3 M1 M2 D1 N2 : OgIk

#1823: C D♭ D E♭ E F G♭ A♭ B :0 1 2 3 4 5 6 8 11: 1 ♭2 2 ♭3 3 4 ♭5 ♭6 7: ma11(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1822] = '''Gythygic'''
namesBySequenceIndexNegative[1822] = ''''''
semitonesBySequenceIndex[1822] = [0, 1, 2, 3, 4, 5, 6, 8, 11]
poemBySequenceIndex[1822] = '''B D F G J K L N S'''
poemAuthors[1822] = ''''''
#1823: ䷀䷕ Creative Adorning: 1 1 1 1 1 1 2 3 1  : S R1 R2 G2 G3 M1 M2 D1 N3 : OgIl

#1824: C D♭ D E♭ E F G♭ A B♭ :0 1 2 3 4 5 6 9 10: 1 ♭2 2 ♭3 3 4 ♭5 6 ♭7: 13(♭5 add ♭9 ♯9)
namesBySequenceIndex[1823] = '''Lydygic'''
namesBySequenceIndexNegative[1823] = ''''''
semitonesBySequenceIndex[1823] = [0, 1, 2, 3, 4, 5, 6, 9, 10]
poemBySequenceIndex[1823] = '''B D F G J K L P R'''
poemAuthors[1823] = ''''''
#1824: ䷀䷐ Creative Following: 1 1 1 1 1 1 3 1 2  : S R1 R2 G2 G3 M1 M2 D2 N2 : OgAd

#1825: C D♭ D E♭ E F G♭ A B :0 1 2 3 4 5 6 9 11: 1 ♭2 2 ♭3 3 4 ♭5 6 7: ma13(♭5 add ♭9 ♯9)
namesBySequenceIndex[1824] = '''Thacrygic'''
namesBySequenceIndexNegative[1824] = ''''''
semitonesBySequenceIndex[1824] = [0, 1, 2, 3, 4, 5, 6, 9, 11]
poemBySequenceIndex[1824] = '''B D F G J K L P S'''
poemAuthors[1824] = ''''''
#1825: ䷀䷔ Creative Biting: 1 1 1 1 1 1 3 2 1  : S R1 R2 G2 G3 M1 M2 D2 N3 : OgAf

#1826: C D♭ D E♭ E F G♭ A♯ B :0 1 2 3 4 5 6 10 11: 1 ♭2 2 ♭3 3 4 ♭5 ♯6 7: ma11(♭5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1825] = '''Thaptygic'''
namesBySequenceIndexNegative[1825] = ''''''
semitonesBySequenceIndex[1825] = [0, 1, 2, 3, 4, 5, 6, 10, 11]
poemBySequenceIndex[1825] = '''B D F G J K L R S'''
poemAuthors[1825] = ''''''
#1826: ䷀䷩ Creative Augmenting: 1 1 1 1 1 1 4 1 1  : S R1 R2 G2 G3 M1 M2 D3 N3 : OgAm

#1827: C D♭ D E♭ E F G A♭ A :0 1 2 3 4 5 7 8 9: 1 ♭2 2 ♭3 3 4 5 ♭6 6: 13(no 7 add ♭6 ♭9 ♯9)
namesBySequenceIndex[1826] = '''Katylygic'''
namesBySequenceIndexNegative[1826] = ''''''
semitonesBySequenceIndex[1826] = [0, 1, 2, 3, 4, 5, 7, 8, 9]
poemBySequenceIndex[1826] = '''B D F G J K M N P'''
poemAuthors[1826] = ''''''
#1827: ䷀䷟ Creative Persevering: 1 1 1 1 1 2 1 1 3  : S R1 R2 G2 G3 M1 P D1 N1 : Ogob

#1828: C D♭ D E♭ E F G A♭ B♭ :0 1 2 3 4 5 7 8 10: 1 ♭2 2 ♭3 3 4 5 ♭6 ♭7: 11(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1827] = '''Radygic'''
namesBySequenceIndexNegative[1827] = ''''''
semitonesBySequenceIndex[1827] = [0, 1, 2, 3, 4, 5, 7, 8, 10]
poemBySequenceIndex[1827] = '''B D F G J K M N R'''
poemAuthors[1827] = ''''''
#1828: ䷀䷯ Creative Source: 1 1 1 1 1 2 1 2 2  : S R1 R2 G2 G3 M1 P D1 N2 : Ogok

#1829: C D♭ D E♭ E F G A♭ B :0 1 2 3 4 5 7 8 11: 1 ♭2 2 ♭3 3 4 5 ♭6 7: ma11(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1828] = '''Aeolocrygic'''
namesBySequenceIndexNegative[1828] = ''''''
semitonesBySequenceIndex[1828] = [0, 1, 2, 3, 4, 5, 7, 8, 11]
poemBySequenceIndex[1828] = '''B D F G J K M N S'''
poemAuthors[1828] = ''''''
#1829: ䷀䷑ Creative Correcting: 1 1 1 1 1 2 1 3 1  : S R1 R2 G2 G3 M1 P D1 N3 : Ogol

#1830: C D♭ D E♭ E F G A B♭ :0 1 2 3 4 5 7 9 10: 1 ♭2 2 ♭3 3 4 5 6 ♭7: 13(add ♭9 ♯9)
namesBySequenceIndex[1829] = '''Sydygic'''
namesBySequenceIndexNegative[1829] = ''''''
semitonesBySequenceIndex[1829] = [0, 1, 2, 3, 4, 5, 7, 9, 10]
poemBySequenceIndex[1829] = '''B D F G J K M P R'''
poemAuthors[1829] = ''''''
#1830: ䷀䷮ Creative Confining: 1 1 1 1 1 2 2 1 2  : S R1 R2 G2 G3 M1 P D2 N2 : Oged

#1831: C D♭ D E♭ E F G A B :0 1 2 3 4 5 7 9 11: 1 ♭2 2 ♭3 3 4 5 6 7: ma13(add ♭9 ♯9)
namesBySequenceIndex[1830] = '''Sylygic'''
namesBySequenceIndexNegative[1830] = ''''''
semitonesBySequenceIndex[1830] = [0, 1, 2, 3, 4, 5, 7, 9, 11]
poemBySequenceIndex[1830] = '''B D F G J K M P S'''
poemAuthors[1830] = ''''''
#1831: ䷀䷿ Creative Unfinished: 1 1 1 1 1 2 2 2 1  : S R1 R2 G2 G3 M1 P D2 N3 : Ogef

#1832: C D♭ D E♭ E F G A♯ B :0 1 2 3 4 5 7 10 11: 1 ♭2 2 ♭3 3 4 5 ♯6 7: ma11(add ♭7 ♭9 ♯9)
namesBySequenceIndex[1831] = '''Pyrygic'''
namesBySequenceIndexNegative[1831] = ''''''
semitonesBySequenceIndex[1831] = [0, 1, 2, 3, 4, 5, 7, 10, 11]
poemBySequenceIndex[1831] = '''B D F G J K M R S'''
poemAuthors[1831] = ''''''
#1832: ䷀䷺ Creative Dispersing: 1 1 1 1 1 2 3 1 1  : S R1 R2 G2 G3 M1 P D3 N3 : Ogem

#1833: C D♭ D E♭ E F G♯ A B♭ :0 1 2 3 4 5 8 9 10: 1 ♭2 2 ♭3 3 4 ♯5 6 ♭7: 13(♯5 add ♭9 ♯9)
namesBySequenceIndex[1832] = '''Gaptygic'''
namesBySequenceIndexNegative[1832] = ''''''
semitonesBySequenceIndex[1832] = [0, 1, 2, 3, 4, 5, 8, 9, 10]
poemBySequenceIndex[1832] = '''B D F G J K N P R'''
poemAuthors[1832] = ''''''
#1833: ䷀䷞ Creative Conjoining: 1 1 1 1 1 3 1 1 2  : S R1 R2 G2 G3 M1 D1 D2 N2 : Ogid

#1834: C D♭ D E♭ E F G♯ A B :0 1 2 3 4 5 8 9 11: 1 ♭2 2 ♭3 3 4 ♯5 6 7: ma13(♯5 add ♭9 ♯9)
namesBySequenceIndex[1833] = '''Stadygic'''
namesBySequenceIndexNegative[1833] = ''''''
semitonesBySequenceIndex[1833] = [0, 1, 2, 3, 4, 5, 8, 9, 11]
poemBySequenceIndex[1833] = '''B D F G J K N P S'''
poemAuthors[1833] = ''''''
#1834: ䷀䷷ Creative Wanderer: 1 1 1 1 1 3 1 2 1  : S R1 R2 G2 G3 M1 D1 D2 N3 : Ogif

#1835: C D♭ D E♭ E F G♯ A♯ B :0 1 2 3 4 5 8 10 11: 1 ♭2 2 ♭3 3 4 ♯5 ♯6 7: ma11(♯5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1834] = '''Aeolynygic'''
namesBySequenceIndexNegative[1834] = ''''''
semitonesBySequenceIndex[1834] = [0, 1, 2, 3, 4, 5, 8, 10, 11]
poemBySequenceIndex[1834] = '''B D F G J K N R S'''
poemAuthors[1834] = ''''''
#1835: ䷀䷴ Creative Infiltrating: 1 1 1 1 1 3 2 1 1  : S R1 R2 G2 G3 M1 D1 N2 N3 : Ogim

#1836: C D♭ D E♭ E F A B♭ B :0 1 2 3 4 5 9 10 11: 1 ♭2 2 ♭3 3 4 6 ♭7 7: ma13(no 5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1835] = '''Eporygic'''
namesBySequenceIndexNegative[1835] = ''''''
semitonesBySequenceIndex[1835] = [0, 1, 2, 3, 4, 5, 9, 10, 11]
poemBySequenceIndex[1835] = '''B D F G J K P R S'''
poemAuthors[1835] = ''''''
#1836: ䷀䷋ Creative Obstruction: 1 1 1 1 1 4 1 1 1  : S R1 R2 G2 G3 M1 D2 N2 N3 : Ogag

#1837: C D♭ D E♭ E F♯ G A♭ A :0 1 2 3 4 6 7 8 9: 1 ♭2 2 ♭3 3 ♯4 5 ♭6 6: 6/9(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1836] = '''Aeolygic'''
namesBySequenceIndexNegative[1836] = ''''''
semitonesBySequenceIndex[1836] = [0, 1, 2, 3, 4, 6, 7, 8, 9]
poemBySequenceIndex[1836] = '''B D F G J L M N P'''
poemAuthors[1836] = ''''''
#1837: ䷪䷡ Decision Might: 1 1 1 1 2 1 1 1 3  : S R1 R2 G2 G3 M2 P D1 N1 : OdOb

#1838: C D♭ D E♭ E F♯ G A♭ B♭ :0 1 2 3 4 6 7 8 10: 1 ♭2 2 ♭3 3 ♯4 5 ♭6 ♭7: 9(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1837] = '''Padygic'''
namesBySequenceIndexNegative[1837] = ''''''
semitonesBySequenceIndex[1837] = [0, 1, 2, 3, 4, 6, 7, 8, 10]
poemBySequenceIndex[1837] = '''B D F G J L M N R'''
poemAuthors[1837] = ''''''
#1838: ䷪䷄ Decision Biding: 1 1 1 1 2 1 1 2 2  : S R1 R2 G2 G3 M2 P D1 N2 : OdOk

#1839: C D♭ D E♭ E F♯ G A♭ B :0 1 2 3 4 6 7 8 11: 1 ♭2 2 ♭3 3 ♯4 5 ♭6 7: ma9(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1838] = '''Phradygic'''
namesBySequenceIndexNegative[1838] = ''''''
semitonesBySequenceIndex[1838] = [0, 1, 2, 3, 4, 6, 7, 8, 11]
poemBySequenceIndex[1838] = '''B D F G J L M N S'''
poemAuthors[1838] = ''''''
#1839: ䷪䷙ Decision Cultivating: 1 1 1 1 2 1 1 3 1  : S R1 R2 G2 G3 M2 P D1 N3 : OdOl

#1840: C D♭ D E♭ E F♯ G A B♭ :0 1 2 3 4 6 7 9 10: 1 ♭2 2 ♭3 3 ♯4 5 6 ♭7: 13(♯11 add ♭9 ♯9)
namesBySequenceIndex[1839] = '''Pylygic'''
namesBySequenceIndexNegative[1839] = ''''''
semitonesBySequenceIndex[1839] = [0, 1, 2, 3, 4, 6, 7, 9, 10]
poemBySequenceIndex[1839] = '''B D F G J L M P R'''
poemAuthors[1839] = ''''''
#1840: ䷪䷹ Decision Joyous: 1 1 1 1 2 1 2 1 2  : S R1 R2 G2 G3 M2 P D2 N2 : OdEd

#1841: C D♭ D E♭ E F♯ G A B :0 1 2 3 4 6 7 9 11: 1 ♭2 2 ♭3 3 ♯4 5 6 7: ma13(♯11 add ♭9 ♯9)
namesBySequenceIndex[1840] = '''Gothygic'''
namesBySequenceIndexNegative[1840] = ''''''
semitonesBySequenceIndex[1840] = [0, 1, 2, 3, 4, 6, 7, 9, 11]
poemBySequenceIndex[1840] = '''B D F G J L M P S'''
poemAuthors[1840] = ''''''
#1841: ䷪䷥ Decision Polarising: 1 1 1 1 2 1 2 2 1  : S R1 R2 G2 G3 M2 P D2 N3 : OdEf

#1842: C D♭ D E♭ E F♯ G A♯ B :0 1 2 3 4 6 7 10 11: 1 ♭2 2 ♭3 3 ♯4 5 ♯6 7: ma9(add ♭7 ♭9 ♯9 ♯11)
namesBySequenceIndex[1841] = '''Phroptygic'''
namesBySequenceIndexNegative[1841] = ''''''
semitonesBySequenceIndex[1841] = [0, 1, 2, 3, 4, 6, 7, 10, 11]
poemBySequenceIndex[1841] = '''B D F G J L M R S'''
poemAuthors[1841] = ''''''
#1842: ䷪䷼ Decision Truth: 1 1 1 1 2 1 3 1 1  : S R1 R2 G2 G3 M2 P D3 N3 : OdEm

#1843: C D♭ D E♭ E F♯ G♯ A B♭ :0 1 2 3 4 6 8 9 10: 1 ♭2 2 ♭3 3 ♯4 ♯5 6 ♭7: 13(♯5 ♯11 add ♭9 ♯9)
namesBySequenceIndex[1842] = '''Aerocrygic'''
namesBySequenceIndexNegative[1842] = ''''''
semitonesBySequenceIndex[1842] = [0, 1, 2, 3, 4, 6, 8, 9, 10]
poemBySequenceIndex[1842] = '''B D F G J L N P R'''
poemAuthors[1842] = ''''''
#1843: ䷪䷰ Decision Revolution: 1 1 1 1 2 2 1 1 2  : S R1 R2 G2 G3 M2 D1 D2 N2 : OdId

#1844: C D♭ D E♭ E F♯ G♯ A B :0 1 2 3 4 6 8 9 11: 1 ♭2 2 ♭3 3 ♯4 ♯5 6 7: ma13(♯5 ♯11 add ♭9 ♯9)
namesBySequenceIndex[1843] = '''Katygic'''
namesBySequenceIndexNegative[1843] = ''''''
semitonesBySequenceIndex[1843] = [0, 1, 2, 3, 4, 6, 8, 9, 11]
poemBySequenceIndex[1843] = '''B D F G J L N P S'''
poemAuthors[1843] = ''''''
#1844: ䷪䷝ Decision Distanced: 1 1 1 1 2 2 1 2 1  : S R1 R2 G2 G3 M2 D1 D2 N3 : OdIf

#1845: C D♭ D E♭ E F♯ G♯ A♯ B :0 1 2 3 4 6 8 10 11: 1 ♭2 2 ♭3 3 ♯4 ♯5 ♯6 7: ma9(♯5 add ♭7 ♭9 ♯9 ♯11)
namesBySequenceIndex[1844] = '''Lothygic'''
namesBySequenceIndexNegative[1844] = ''''''
semitonesBySequenceIndex[1844] = [0, 1, 2, 3, 4, 6, 8, 10, 11]
poemBySequenceIndex[1844] = '''B D F G J L N R S'''
poemAuthors[1844] = ''''''
#1845: ䷪䷤ Decision Family: 1 1 1 1 2 2 2 1 1  : S R1 R2 G2 G3 M2 D1 N2 N3 : OdIm

#1846: C D♭ D E♭ E G♭ A B♭ B :0 1 2 3 4 6 9 10 11: 1 ♭2 2 ♭3 3 ♭5 6 ♭7 7: ma9(♭5 add ♭7 ♭9 ♯9 13)
namesBySequenceIndex[1845] = '''Rycrygic'''
namesBySequenceIndexNegative[1845] = ''''''
semitonesBySequenceIndex[1845] = [0, 1, 2, 3, 4, 6, 9, 10, 11]
poemBySequenceIndex[1845] = '''B D F G J L P R S'''
poemAuthors[1845] = ''''''
#1846: ䷪䷘ Decision Integrity: 1 1 1 1 2 3 1 1 1  : S R1 R2 G2 G3 M2 D2 N2 N3 : OdAg

#1847: C D♭ D E♭ E G A♭ A B♭ :0 1 2 3 4 7 8 9 10: 1 ♭2 2 ♭3 3 5 ♭6 6 ♭7: 9(add ♭9 ♯9 ♭13 13)
namesBySequenceIndex[1846] = '''Marygic'''
namesBySequenceIndexNegative[1846] = ''''''
semitonesBySequenceIndex[1846] = [0, 1, 2, 3, 4, 7, 8, 9, 10]
poemBySequenceIndex[1846] = '''B D F G J M N P R'''
poemAuthors[1846] = ''''''
#1847: ䷪䷛ Decision Excess: 1 1 1 1 3 1 1 1 2  : S R1 R2 G2 G3 P D1 D2 N2 : Odod

#1848: C D♭ D E♭ E G A♭ A B :0 1 2 3 4 7 8 9 11: 1 ♭2 2 ♭3 3 5 ♭6 6 7: ma9(add ♭9 ♯9 ♭13 13)
namesBySequenceIndex[1847] = '''Aeroptygic'''
namesBySequenceIndexNegative[1847] = ''''''
semitonesBySequenceIndex[1847] = [0, 1, 2, 3, 4, 7, 8, 9, 11]
poemBySequenceIndex[1847] = '''B D F G J M N P S'''
poemAuthors[1847] = ''''''
#1848: ䷪䷱ Decision Holding: 1 1 1 1 3 1 1 2 1  : S R1 R2 G2 G3 P D1 D2 N3 : Odof

#1849: C D♭ D E♭ E G A♭ B♭ B :0 1 2 3 4 7 8 10 11: 1 ♭2 2 ♭3 3 5 ♭6 ♭7 7: ma9(add ♭7 ♭9 ♯9 ♭13)
namesBySequenceIndex[1848] = '''Byptygic'''
namesBySequenceIndexNegative[1848] = ''''''
semitonesBySequenceIndex[1848] = [0, 1, 2, 3, 4, 7, 8, 10, 11]
poemBySequenceIndex[1848] = '''B D F G J M N R S'''
poemAuthors[1848] = ''''''
#1849: ䷪䷸ Decision Osmosis: 1 1 1 1 3 1 2 1 1  : S R1 R2 G2 G3 P D1 N2 N3 : Odom

#1850: C D♭ D E♭ E G A B♭ B :0 1 2 3 4 7 9 10 11: 1 ♭2 2 ♭3 3 5 6 ♭7 7: ma9(add ♭7 ♭9 ♯9 13)
namesBySequenceIndex[1849] = '''Thadygic'''
namesBySequenceIndexNegative[1849] = ''''''
semitonesBySequenceIndex[1849] = [0, 1, 2, 3, 4, 7, 9, 10, 11]
poemBySequenceIndex[1849] = '''B D F G J M P R S'''
poemAuthors[1849] = ''''''
#1850: ䷪䷅ Decision Arguing: 1 1 1 1 3 2 1 1 1  : S R1 R2 G2 G3 P D2 N2 N3 : Odeg

#1851: C D♭ D E♭ E G♯ A B♭ B :0 1 2 3 4 8 9 10 11: 1 ♭2 2 ♭3 3 ♯5 6 ♭7 7: ma9(♯5 add ♭7 ♭9 ♯9 13)
namesBySequenceIndex[1850] = '''Aerynygic'''
namesBySequenceIndexNegative[1850] = ''''''
semitonesBySequenceIndex[1850] = [0, 1, 2, 3, 4, 8, 9, 10, 11]
poemBySequenceIndex[1850] = '''B D F G J N P R S'''
poemAuthors[1850] = ''''''
#1851: ䷪䷠ Decision Retiring: 1 1 1 1 4 1 1 1 1  : S R1 R2 G2 G3 D1 D2 N2 N3 : Odig

#1852: C D♭ D E♭ F F♯ G A♭ A :0 1 2 3 5 6 7 8 9: 1 ♭2 2 ♭3 4 ♯4 5 ♭6 6: mi13(no 7 add ♭6 ♭9 ♯11)
namesBySequenceIndex[1851] = '''Epitygic'''
namesBySequenceIndexNegative[1851] = ''''''
semitonesBySequenceIndex[1851] = [0, 1, 2, 3, 5, 6, 7, 8, 9]
poemBySequenceIndex[1851] = '''B D F G K L M N P'''
poemAuthors[1851] = ''''''
#1852: ䷍䷡ Resevoir Might: 1 1 1 2 1 1 1 1 3  : S R1 R2 G2 M1 M2 P D1 N1 : OfOb

#1853: C D♭ D E♭ F F♯ G A♭ B♭ :0 1 2 3 5 6 7 8 10: 1 ♭2 2 ♭3 4 ♯4 5 ♭6 ♭7: mi11(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1852] = '''Solygic'''
namesBySequenceIndexNegative[1852] = ''''''
semitonesBySequenceIndex[1852] = [0, 1, 2, 3, 5, 6, 7, 8, 10]
poemBySequenceIndex[1852] = '''B D F G K L M N R'''
poemAuthors[1852] = ''''''
#1853: ䷍䷄ Resevoir Biding: 1 1 1 2 1 1 1 2 2  : S R1 R2 G2 M1 M2 P D1 N2 : OfOk

#1854: C D♭ D E♭ F F♯ G A♭ B :0 1 2 3 5 6 7 8 11: 1 ♭2 2 ♭3 4 ♯4 5 ♭6 7: mima11(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1853] = '''Dydygic'''
namesBySequenceIndexNegative[1853] = ''''''
semitonesBySequenceIndex[1853] = [0, 1, 2, 3, 5, 6, 7, 8, 11]
poemBySequenceIndex[1853] = '''B D F G K L M N S'''
poemAuthors[1853] = ''''''
#1854: ䷍䷙ Resevoir Cultivating: 1 1 1 2 1 1 1 3 1  : S R1 R2 G2 M1 M2 P D1 N3 : OfOl

#1855: C D♭ D E♭ F F♯ G A B♭ :0 1 2 3 5 6 7 9 10: 1 ♭2 2 ♭3 4 ♯4 5 6 ♭7: mi13(add ♭9 ♯11)
namesBySequenceIndex[1854] = '''Lyrygic'''
namesBySequenceIndexNegative[1854] = ''''''
semitonesBySequenceIndex[1854] = [0, 1, 2, 3, 5, 6, 7, 9, 10]
poemBySequenceIndex[1854] = '''B D F G K L M P R'''
poemAuthors[1854] = ''''''
#1855: ䷍䷹ Resevoir Joyous: 1 1 1 2 1 1 2 1 2  : S R1 R2 G2 M1 M2 P D2 N2 : OfEd

#1856: C D♭ D E♭ F F♯ G A B :0 1 2 3 5 6 7 9 11: 1 ♭2 2 ♭3 4 ♯4 5 6 7: mima13(add ♭9 ♯11)
namesBySequenceIndex[1855] = '''Epilygic'''
namesBySequenceIndexNegative[1855] = ''''''
semitonesBySequenceIndex[1855] = [0, 1, 2, 3, 5, 6, 7, 9, 11]
poemBySequenceIndex[1855] = '''B D F G K L M P S'''
poemAuthors[1855] = ''''''
#1856: ䷍䷥ Resevoir Polarising: 1 1 1 2 1 1 2 2 1  : S R1 R2 G2 M1 M2 P D2 N3 : OfEf

#1857: C D♭ D E♭ F F♯ G A♯ B :0 1 2 3 5 6 7 10 11: 1 ♭2 2 ♭3 4 ♯4 5 ♯6 7: mima11(add ♭7 ♭9 ♯11)
namesBySequenceIndex[1856] = '''Mixodygic'''
namesBySequenceIndexNegative[1856] = ''''''
semitonesBySequenceIndex[1856] = [0, 1, 2, 3, 5, 6, 7, 10, 11]
poemBySequenceIndex[1856] = '''B D F G K L M R S'''
poemAuthors[1856] = ''''''
#1857: ䷍䷼ Resevoir Truth: 1 1 1 2 1 1 3 1 1  : S R1 R2 G2 M1 M2 P D3 N3 : OfEm

#1858: C D♭ D E♭ F G♭ A♭ A B♭ :0 1 2 3 5 6 8 9 10: 1 ♭2 2 ♭3 4 ♭5 ♭6 6 ♭7: ⌀13(add ♭9 ♭13)
namesBySequenceIndex[1857] = '''Rocrygic'''
namesBySequenceIndexNegative[1857] = ''''''
semitonesBySequenceIndex[1857] = [0, 1, 2, 3, 5, 6, 8, 9, 10]
poemBySequenceIndex[1857] = '''B D F G K L N P R'''
poemAuthors[1857] = ''''''
#1858: ䷍䷰ Resevoir Revolution: 1 1 1 2 1 2 1 1 2  : S R1 R2 G2 M1 M2 D1 D2 N2 : OfId

#1859: C D♭ D E♭ F G♭ A♭ A B :0 1 2 3 5 6 8 9 11: 1 ♭2 2 ♭3 4 ♭5 ♭6 6 7: °ma13(add ♭9 ♭13)
namesBySequenceIndex[1858] = '''Rodygic'''
namesBySequenceIndexNegative[1858] = ''''''
semitonesBySequenceIndex[1858] = [0, 1, 2, 3, 5, 6, 8, 9, 11]
poemBySequenceIndex[1858] = '''B D F G K L N P S'''
poemAuthors[1858] = ''''''
#1859: ䷍䷝ Resevoir Distanced: 1 1 1 2 1 2 1 2 1  : S R1 R2 G2 M1 M2 D1 D2 N3 : OfIf

#1860: C D♭ D E♭ F G♭ A♭ B♭ B :0 1 2 3 5 6 8 10 11: 1 ♭2 2 ♭3 4 ♭5 ♭6 ♭7 7: °ma11(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1859] = '''Lythygic'''
namesBySequenceIndexNegative[1859] = ''''''
semitonesBySequenceIndex[1859] = [0, 1, 2, 3, 5, 6, 8, 10, 11]
poemBySequenceIndex[1859] = '''B D F G K L N R S'''
poemAuthors[1859] = ''''''
#1860: ䷍䷤ Resevoir Family: 1 1 1 2 1 2 2 1 1  : S R1 R2 G2 M1 M2 D1 N2 N3 : OfIm

#1861: C D♭ D E♭ F G♭ A B♭ B :0 1 2 3 5 6 9 10 11: 1 ♭2 2 ♭3 4 ♭5 6 ♭7 7: °ma13(add ♭7 ♭9)
namesBySequenceIndex[1860] = '''Kodygic'''
namesBySequenceIndexNegative[1860] = ''''''
semitonesBySequenceIndex[1860] = [0, 1, 2, 3, 5, 6, 9, 10, 11]
poemBySequenceIndex[1860] = '''B D F G K L P R S'''
poemAuthors[1860] = ''''''
#1861: ䷍䷘ Resevoir Integrity: 1 1 1 2 1 3 1 1 1  : S R1 R2 G2 M1 M2 D2 N2 N3 : OfAg

#1862: C D♭ D E♭ F G A♭ A B♭ :0 1 2 3 5 7 8 9 10: 1 ♭2 2 ♭3 4 5 ♭6 6 ♭7: mi13(add ♭9 ♭13)
namesBySequenceIndex[1861] = '''Godygic,Chromatic Diatonic Dorian'''
namesBySequenceIndexNegative[1861] = ''''''
semitonesBySequenceIndex[1861] = [0, 1, 2, 3, 5, 7, 8, 9, 10]
poemBySequenceIndex[1861] = '''B D F G K M N P R'''
poemAuthors[1861] = ''''''
#1862: ䷍䷛ Resevoir Excess: 1 1 1 2 2 1 1 1 2  : S R1 R2 G2 M1 P D1 D2 N2 : Ofod

#1863: C D♭ D E♭ F G A♭ A B :0 1 2 3 5 7 8 9 11: 1 ♭2 2 ♭3 4 5 ♭6 6 7: mima13(add ♭9 ♭13)
namesBySequenceIndex[1862] = '''Zanygic'''
namesBySequenceIndexNegative[1862] = ''''''
semitonesBySequenceIndex[1862] = [0, 1, 2, 3, 5, 7, 8, 9, 11]
poemBySequenceIndex[1862] = '''B D F G K M N P S'''
poemAuthors[1862] = ''''''
#1863: ䷍䷱ Resevoir Holding: 1 1 1 2 2 1 1 2 1  : S R1 R2 G2 M1 P D1 D2 N3 : Ofof

#1864: C D♭ D E♭ F G A♭ B♭ B :0 1 2 3 5 7 8 10 11: 1 ♭2 2 ♭3 4 5 ♭6 ♭7 7: mima11(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1863] = '''Zyphygic'''
namesBySequenceIndexNegative[1863] = ''''''
semitonesBySequenceIndex[1863] = [0, 1, 2, 3, 5, 7, 8, 10, 11]
poemBySequenceIndex[1863] = '''B D F G K M N R S'''
poemAuthors[1863] = ''''''
#1864: ䷍䷸ Resevoir Osmosis: 1 1 1 2 2 1 2 1 1  : S R1 R2 G2 M1 P D1 N2 N3 : Ofom

#1865: C D♭ D E♭ F G A B♭ B :0 1 2 3 5 7 9 10 11: 1 ♭2 2 ♭3 4 5 6 ♭7 7: mima13(add ♭7 ♭9)
namesBySequenceIndex[1864] = '''Darygic'''
namesBySequenceIndexNegative[1864] = ''''''
semitonesBySequenceIndex[1864] = [0, 1, 2, 3, 5, 7, 9, 10, 11]
poemBySequenceIndex[1864] = '''B D F G K M P R S'''
poemAuthors[1864] = ''''''
#1865: ䷍䷅ Resevoir Arguing: 1 1 1 2 2 2 1 1 1  : S R1 R2 G2 M1 P D2 N2 N3 : Ofeg

#1866: C D♭ D E♭ F G♯ A B♭ B :0 1 2 3 5 8 9 10 11: 1 ♭2 2 ♭3 4 ♯5 6 ♭7 7: mima13(♯5 add ♭7 ♭9)
namesBySequenceIndex[1865] = '''Phrathygic'''
namesBySequenceIndexNegative[1865] = ''''''
semitonesBySequenceIndex[1865] = [0, 1, 2, 3, 5, 8, 9, 10, 11]
poemBySequenceIndex[1865] = '''B D F G K N P R S'''
poemAuthors[1865] = ''''''
#1866: ䷍䷠ Resevoir Retiring: 1 1 1 2 3 1 1 1 1  : S R1 R2 G2 M1 D1 D2 N2 N3 : Ofig

#1867: C D♭ D E♭ F♯ G A♭ A B♭ :0 1 2 3 6 7 8 9 10: 1 ♭2 2 ♭3 ♯4 5 ♭6 6 ♭7: mi13(♯11 add ♭9 ♭13)
namesBySequenceIndex[1866] = '''Zacrygic'''
namesBySequenceIndexNegative[1866] = ''''''
semitonesBySequenceIndex[1866] = [0, 1, 2, 3, 6, 7, 8, 9, 10]
poemBySequenceIndex[1866] = '''B D F G L M N P R'''
poemAuthors[1866] = ''''''
#1867: ䷡䷪ Might Decision: 1 1 1 3 1 1 1 1 2  : S R1 R2 G2 M2 P D1 D2 N2 : ObOd

#1868: C D♭ D E♭ F♯ G A♭ A B :0 1 2 3 6 7 8 9 11: 1 ♭2 2 ♭3 ♯4 5 ♭6 6 7: mima13(♯11 add ♭9 ♭13)
namesBySequenceIndex[1867] = '''Mothygic'''
namesBySequenceIndexNegative[1867] = ''''''
semitonesBySequenceIndex[1867] = [0, 1, 2, 3, 6, 7, 8, 9, 11]
poemBySequenceIndex[1867] = '''B D F G L M N P S'''
poemAuthors[1867] = ''''''
#1868: ䷡䷍ Might Resevoir: 1 1 1 3 1 1 1 2 1  : S R1 R2 G2 M2 P D1 D2 N3 : ObOf

#1869: C D♭ D E♭ F♯ G A♭ B♭ B :0 1 2 3 6 7 8 10 11: 1 ♭2 2 ♭3 ♯4 5 ♭6 ♭7 7: mima9(add ♭7 ♭9 ♯11 ♭13)
namesBySequenceIndex[1868] = '''Mylygic'''
namesBySequenceIndexNegative[1868] = ''''''
semitonesBySequenceIndex[1868] = [0, 1, 2, 3, 6, 7, 8, 10, 11]
poemBySequenceIndex[1868] = '''B D F G L M N R S'''
poemAuthors[1868] = ''''''
#1869: ䷡䷈ Might Homegrown: 1 1 1 3 1 1 2 1 1  : S R1 R2 G2 M2 P D1 N2 N3 : ObOm

#1870: C D♭ D E♭ F♯ G A B♭ B :0 1 2 3 6 7 9 10 11: 1 ♭2 2 ♭3 ♯4 5 6 ♭7 7: mima13(♯11 add ♭7 ♭9)
namesBySequenceIndex[1869] = '''Stodygic'''
namesBySequenceIndexNegative[1869] = ''''''
semitonesBySequenceIndex[1869] = [0, 1, 2, 3, 6, 7, 9, 10, 11]
poemBySequenceIndex[1869] = '''B D F G L M P R S'''
poemAuthors[1869] = ''''''
#1870: ䷡䷉ Might Treading: 1 1 1 3 1 2 1 1 1  : S R1 R2 G2 M2 P D2 N2 N3 : ObEg

#1871: C D♭ D E♭ F♯ G♯ A B♭ B :0 1 2 3 6 8 9 10 11: 1 ♭2 2 ♭3 ♯4 ♯5 6 ♭7 7: °ma13(♯11 add ♭7 ♭9 ♭13)
namesBySequenceIndex[1870] = '''Lynygic'''
namesBySequenceIndexNegative[1870] = ''''''
semitonesBySequenceIndex[1870] = [0, 1, 2, 3, 6, 8, 9, 10, 11]
poemBySequenceIndex[1870] = '''B D F G L N P R S'''
poemAuthors[1870] = ''''''
#1871: ䷡䷌ Might Fellowship: 1 1 1 3 2 1 1 1 1  : S R1 R2 G2 M2 D1 D2 N2 N3 : ObIg

#1872: C D♭ D E♭ G A♭ A B♭ B :0 1 2 3 7 8 9 10 11: 1 ♭2 2 ♭3 5 ♭6 6 ♭7 7: mima9(add ♭7 ♭9 ♭13 13)
namesBySequenceIndex[1871] = '''Thyptygic'''
namesBySequenceIndexNegative[1871] = ''''''
semitonesBySequenceIndex[1871] = [0, 1, 2, 3, 7, 8, 9, 10, 11]
poemBySequenceIndex[1871] = '''B D F G M N P R S'''
poemAuthors[1871] = ''''''
#1872: ䷡䷫ Might Coupling: 1 1 1 4 1 1 1 1 1  : S R1 R2 G2 P D1 D2 N2 N3 : Obog

#1873: C D♭ D E F F♯ G A♭ A :0 1 2 4 5 6 7 8 9: 1 ♭2 2 3 4 ♯4 5 ♭6 6: 13(no 7 add ♭6 ♭9 ♯11)
namesBySequenceIndex[1872] = '''Ionodygic'''
namesBySequenceIndexNegative[1872] = ''''''
semitonesBySequenceIndex[1872] = [0, 1, 2, 4, 5, 6, 7, 8, 9]
poemBySequenceIndex[1872] = '''B D F J K L M N P'''
poemAuthors[1872] = ''''''
#1873: ䷈䷡ Homegrown Might: 1 1 2 1 1 1 1 1 3  : S R1 R2 G3 M1 M2 P D1 N1 : OmOb

#1874: C D♭ D E F F♯ G A♭ B♭ :0 1 2 4 5 6 7 8 10: 1 ♭2 2 3 4 ♯4 5 ♭6 ♭7: 11(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1873] = '''Aeolyrigic'''
namesBySequenceIndexNegative[1873] = ''''''
semitonesBySequenceIndex[1873] = [0, 1, 2, 4, 5, 6, 7, 8, 10]
poemBySequenceIndex[1873] = '''B D F J K L M N R'''
poemAuthors[1873] = ''''''
#1874: ䷈䷄ Homegrown Biding: 1 1 2 1 1 1 1 2 2  : S R1 R2 G3 M1 M2 P D1 N2 : OmOk

#1875: C D♭ D E F F♯ G A♭ B :0 1 2 4 5 6 7 8 11: 1 ♭2 2 3 4 ♯4 5 ♭6 7: ma11(add ♭9 ♯11 ♭13)
namesBySequenceIndex[1874] = '''Thocrygic'''
namesBySequenceIndexNegative[1874] = ''''''
semitonesBySequenceIndex[1874] = [0, 1, 2, 4, 5, 6, 7, 8, 11]
poemBySequenceIndex[1874] = '''B D F J K L M N S'''
poemAuthors[1874] = ''''''
#1875: ䷈䷙ Homegrown Cultivating: 1 1 2 1 1 1 1 3 1  : S R1 R2 G3 M1 M2 P D1 N3 : OmOl

#1876: C D♭ D E F F♯ G A B♭ :0 1 2 4 5 6 7 9 10: 1 ♭2 2 3 4 ♯4 5 6 ♭7: 13(add ♭9 ♯11)
namesBySequenceIndex[1875] = '''Youlan,Danygic'''
namesBySequenceIndexNegative[1875] = ''''''
semitonesBySequenceIndex[1875] = [0, 1, 2, 4, 5, 6, 7, 9, 10]
poemBySequenceIndex[1875] = '''B D F J K L M P R'''
poemAuthors[1875] = ''''''
#1876: ䷈䷹ Homegrown Joyous: 1 1 2 1 1 1 2 1 2  : S R1 R2 G3 M1 M2 P D2 N2 : OmEd

#1877: C D♭ D E F F♯ G A B :0 1 2 4 5 6 7 9 11: 1 ♭2 2 3 4 ♯4 5 6 7: ma13(add ♭9 ♯11)
namesBySequenceIndex[1876] = '''Zylygic'''
namesBySequenceIndexNegative[1876] = ''''''
semitonesBySequenceIndex[1876] = [0, 1, 2, 4, 5, 6, 7, 9, 11]
poemBySequenceIndex[1876] = '''B D F J K L M P S'''
poemAuthors[1876] = ''''''
#1877: ䷈䷥ Homegrown Polarising: 1 1 2 1 1 1 2 2 1  : S R1 R2 G3 M1 M2 P D2 N3 : OmEf

#1878: C D♭ D E F F♯ G A♯ B :0 1 2 4 5 6 7 10 11: 1 ♭2 2 3 4 ♯4 5 ♯6 7: ma11(add ♭7 ♭9 ♯11)
namesBySequenceIndex[1877] = '''Tholygic'''
namesBySequenceIndexNegative[1877] = ''''''
semitonesBySequenceIndex[1877] = [0, 1, 2, 4, 5, 6, 7, 10, 11]
poemBySequenceIndex[1877] = '''B D F J K L M R S'''
poemAuthors[1877] = ''''''
#1878: ䷈䷼ Homegrown Truth: 1 1 2 1 1 1 3 1 1  : S R1 R2 G3 M1 M2 P D3 N3 : OmEm

#1879: C D♭ D E F G♭ A♭ A B♭ :0 1 2 4 5 6 8 9 10: 1 ♭2 2 3 4 ♭5 ♭6 6 ♭7: 13(♯5 add ♭9 ♯11)
namesBySequenceIndex[1878] = '''Stynygic'''
namesBySequenceIndexNegative[1878] = ''''''
semitonesBySequenceIndex[1878] = [0, 1, 2, 4, 5, 6, 8, 9, 10]
poemBySequenceIndex[1878] = '''B D F J K L N P R'''
poemAuthors[1878] = ''''''
#1879: ䷈䷰ Homegrown Revolution: 1 1 2 1 1 2 1 1 2  : S R1 R2 G3 M1 M2 D1 D2 N2 : OmId

#1880: C D♭ D E F G♭ A♭ A B :0 1 2 4 5 6 8 9 11: 1 ♭2 2 3 4 ♭5 ♭6 6 7: ma13(♯5 add ♭9 ♯11)
namesBySequenceIndex[1879] = '''Modygic'''
namesBySequenceIndexNegative[1879] = ''''''
semitonesBySequenceIndex[1879] = [0, 1, 2, 4, 5, 6, 8, 9, 11]
poemBySequenceIndex[1879] = '''B D F J K L N P S'''
poemAuthors[1879] = ''''''
#1880: ䷈䷝ Homegrown Distanced: 1 1 2 1 1 2 1 2 1  : S R1 R2 G3 M1 M2 D1 D2 N3 : OmIf

#1881: C D♭ D E F G♭ A♭ B♭ B :0 1 2 4 5 6 8 10 11: 1 ♭2 2 3 4 ♭5 ♭6 ♭7 7: ma11(♯5 add ♭7 ♭9 ♯11)
namesBySequenceIndex[1880] = '''Kynygic'''
namesBySequenceIndexNegative[1880] = ''''''
semitonesBySequenceIndex[1880] = [0, 1, 2, 4, 5, 6, 8, 10, 11]
poemBySequenceIndex[1880] = '''B D F J K L N R S'''
poemAuthors[1880] = ''''''
#1881: ䷈䷤ Homegrown Family: 1 1 2 1 1 2 2 1 1  : S R1 R2 G3 M1 M2 D1 N2 N3 : OmIm

#1882: C D♭ D E F G♭ A B♭ B :0 1 2 4 5 6 9 10 11: 1 ♭2 2 3 4 ♭5 6 ♭7 7: ma13(♭5 add ♭7 ♭9)
namesBySequenceIndex[1881] = '''Katalygic'''
namesBySequenceIndexNegative[1881] = ''''''
semitonesBySequenceIndex[1881] = [0, 1, 2, 4, 5, 6, 9, 10, 11]
poemBySequenceIndex[1881] = '''B D F J K L P R S'''
poemAuthors[1881] = ''''''
#1882: ䷈䷘ Homegrown Integrity: 1 1 2 1 1 3 1 1 1  : S R1 R2 G3 M1 M2 D2 N2 N3 : OmAg

#1883: C D♭ D E F G A♭ A B♭ :0 1 2 4 5 7 8 9 10: 1 ♭2 2 3 4 5 ♭6 6 ♭7: 13(add ♭9 ♭13)
namesBySequenceIndex[1882] = '''Ionocrygic'''
namesBySequenceIndexNegative[1882] = ''''''
semitonesBySequenceIndex[1882] = [0, 1, 2, 4, 5, 7, 8, 9, 10]
poemBySequenceIndex[1882] = '''B D F J K M N P R'''
poemAuthors[1882] = ''''''
#1883: ䷈䷛ Homegrown Excess: 1 1 2 1 2 1 1 1 2  : S R1 R2 G3 M1 P D1 D2 N2 : Omod

#1884: C D♭ D E F G A♭ A B :0 1 2 4 5 7 8 9 11: 1 ♭2 2 3 4 5 ♭6 6 7: ma13(add ♭9 ♭13)
namesBySequenceIndex[1883] = '''Zyrygic,Chromatic Permutated Diatonic Dorian'''
namesBySequenceIndexNegative[1883] = ''''''
semitonesBySequenceIndex[1883] = [0, 1, 2, 4, 5, 7, 8, 9, 11]
poemBySequenceIndex[1883] = '''B D F J K M N P S'''
poemAuthors[1883] = ''''''
#1884: ䷈䷱ Homegrown Holding: 1 1 2 1 2 1 1 2 1  : S R1 R2 G3 M1 P D1 D2 N3 : Omof

#1885: C D♭ D E F G A♭ B♭ B :0 1 2 4 5 7 8 10 11: 1 ♭2 2 3 4 5 ♭6 ♭7 7: ma11(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1884] = '''Epolygic'''
namesBySequenceIndexNegative[1884] = ''''''
semitonesBySequenceIndex[1884] = [0, 1, 2, 4, 5, 7, 8, 10, 11]
poemBySequenceIndex[1884] = '''B D F J K M N R S'''
poemAuthors[1884] = ''''''
#1885: ䷈䷸ Homegrown Osmosis: 1 1 2 1 2 1 2 1 1  : S R1 R2 G3 M1 P D1 N2 N3 : Omom

#1886: C D♭ D E F G A B♭ B :0 1 2 4 5 7 9 10 11: 1 ♭2 2 3 4 5 6 ♭7 7: ma13(add ♭7 ♭9)
namesBySequenceIndex[1885] = '''Bebop Chromatic,Bacrygic'''
namesBySequenceIndexNegative[1885] = ''''''
semitonesBySequenceIndex[1885] = [0, 1, 2, 4, 5, 7, 9, 10, 11]
poemBySequenceIndex[1885] = '''B D F J K M P R S'''
poemAuthors[1885] = ''''''
#1886: ䷈䷅ Homegrown Arguing: 1 1 2 1 2 2 1 1 1  : S R1 R2 G3 M1 P D2 N2 N3 : Omeg

#1887: C D♭ D E F G♯ A B♭ B :0 1 2 4 5 8 9 10 11: 1 ♭2 2 3 4 ♯5 6 ♭7 7: ma13(♯5 add ♭7 ♭9)
namesBySequenceIndex[1886] = '''Eparygic'''
namesBySequenceIndexNegative[1886] = ''''''
semitonesBySequenceIndex[1886] = [0, 1, 2, 4, 5, 8, 9, 10, 11]
poemBySequenceIndex[1886] = '''B D F J K N P R S'''
poemAuthors[1886] = ''''''
#1887: ䷈䷠ Homegrown Retiring: 1 1 2 1 3 1 1 1 1  : S R1 R2 G3 M1 D1 D2 N2 N3 : Omig

#1888: C D♭ D E F♯ G A♭ A B♭ :0 1 2 4 6 7 8 9 10: 1 ♭2 2 3 ♯4 5 ♭6 6 ♭7: 13(♯11 add ♭9 ♭13)
namesBySequenceIndex[1887] = '''Stonygic'''
namesBySequenceIndexNegative[1887] = ''''''
semitonesBySequenceIndex[1887] = [0, 1, 2, 4, 6, 7, 8, 9, 10]
poemBySequenceIndex[1887] = '''B D F J L M N P R'''
poemAuthors[1887] = ''''''
#1888: ䷄䷪ Biding Decision: 1 1 2 2 1 1 1 1 2  : S R1 R2 G3 M2 P D1 D2 N2 : OkOd

#1889: C D♭ D E F♯ G A♭ A B :0 1 2 4 6 7 8 9 11: 1 ♭2 2 3 ♯4 5 ♭6 6 7: ma13(♯11 add ♭9 ♭13)
namesBySequenceIndex[1888] = '''Epithygic'''
namesBySequenceIndexNegative[1888] = ''''''
semitonesBySequenceIndex[1888] = [0, 1, 2, 4, 6, 7, 8, 9, 11]
poemBySequenceIndex[1888] = '''B D F J L M N P S'''
poemAuthors[1888] = ''''''
#1889: ䷄䷍ Biding Resevoir: 1 1 2 2 1 1 1 2 1  : S R1 R2 G3 M2 P D1 D2 N3 : OkOf

#1890: C D♭ D E F♯ G A♭ B♭ B :0 1 2 4 6 7 8 10 11: 1 ♭2 2 3 ♯4 5 ♭6 ♭7 7: ma9(add ♭7 ♭9 ♯11 ♭13)
namesBySequenceIndex[1889] = '''Symmetrical Nonatonic,Aeolonygic'''
namesBySequenceIndexNegative[1889] = ''''''
semitonesBySequenceIndex[1889] = [0, 1, 2, 4, 6, 7, 8, 10, 11]
poemBySequenceIndex[1889] = '''B D F J L M N R S'''
poemAuthors[1889] = ''''''
#1890: ䷄䷈ Biding Homegrown: 1 1 2 2 1 1 2 1 1  : S R1 R2 G3 M2 P D1 N2 N3 : OkOm

#1891: C D♭ D E F♯ G A B♭ B :0 1 2 4 6 7 9 10 11: 1 ♭2 2 3 ♯4 5 6 ♭7 7: ma13(♯11 add ♭7 ♭9)
namesBySequenceIndex[1890] = '''Aeralygic'''
namesBySequenceIndexNegative[1890] = ''''''
semitonesBySequenceIndex[1890] = [0, 1, 2, 4, 6, 7, 9, 10, 11]
poemBySequenceIndex[1890] = '''B D F J L M P R S'''
poemAuthors[1890] = ''''''
#1891: ䷄䷉ Biding Treading: 1 1 2 2 1 2 1 1 1  : S R1 R2 G3 M2 P D2 N2 N3 : OkEg

#1892: C D♭ D E F♯ G♯ A B♭ B :0 1 2 4 6 8 9 10 11: 1 ♭2 2 3 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♯11 add ♭7 ♭9)
namesBySequenceIndex[1891] = '''Monygic'''
namesBySequenceIndexNegative[1891] = ''''''
semitonesBySequenceIndex[1891] = [0, 1, 2, 4, 6, 8, 9, 10, 11]
poemBySequenceIndex[1891] = '''B D F J L N P R S'''
poemAuthors[1891] = ''''''
#1892: ䷄䷌ Biding Fellowship: 1 1 2 2 2 1 1 1 1  : S R1 R2 G3 M2 D1 D2 N2 N3 : OkIg

#1893: C D♭ D E G A♭ A B♭ B :0 1 2 4 7 8 9 10 11: 1 ♭2 2 3 5 ♭6 6 ♭7 7: ma9(add ♭7 ♭9 ♭13 13)
namesBySequenceIndex[1892] = '''Badygic'''
namesBySequenceIndexNegative[1892] = ''''''
semitonesBySequenceIndex[1892] = [0, 1, 2, 4, 7, 8, 9, 10, 11]
poemBySequenceIndex[1892] = '''B D F J M N P R S'''
poemAuthors[1892] = ''''''
#1893: ䷄䷫ Biding Coupling: 1 1 2 3 1 1 1 1 1  : S R1 R2 G3 P D1 D2 N2 N3 : Okog

#1894: C D♭ D F F♯ G A♭ A B♭ :0 1 2 5 6 7 8 9 10: 1 ♭2 2 4 ♯4 5 ♭6 6 ♭7: 13sus♯4(add ♭9 ♭13)
namesBySequenceIndex[1893] = '''Zodygic'''
namesBySequenceIndexNegative[1893] = ''''''
semitonesBySequenceIndex[1893] = [0, 1, 2, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1893] = '''B D F K L M N P R'''
poemAuthors[1893] = ''''''
#1894: ䷙䷪ Cultivating Decision: 1 1 3 1 1 1 1 1 2  : S R1 G1 M1 M2 P D1 D2 N2 : OlOd

#1895: C D♭ D F F♯ G A♭ A B :0 1 2 5 6 7 8 9 11: 1 ♭2 2 4 ♯4 5 ♭6 6 7: ma13sus♯4(add ♭9 ♭13)
namesBySequenceIndex[1894] = '''Panygic'''
namesBySequenceIndexNegative[1894] = ''''''
semitonesBySequenceIndex[1894] = [0, 1, 2, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1894] = '''B D F K L M N P S'''
poemAuthors[1894] = ''''''
#1895: ䷙䷍ Cultivating Resevoir: 1 1 3 1 1 1 1 2 1  : S R1 G1 M1 M2 P D1 D2 N3 : OlOf

#1896: C D♭ D F F♯ G A♭ B♭ B :0 1 2 5 6 7 8 10 11: 1 ♭2 2 4 ♯4 5 ♭6 ♭7 7: ma11sus♯4(add ♭7 ♭9 ♭13)
namesBySequenceIndex[1895] = '''Thophygic'''
namesBySequenceIndexNegative[1895] = ''''''
semitonesBySequenceIndex[1895] = [0, 1, 2, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1895] = '''B D F K L M N R S'''
poemAuthors[1895] = ''''''
#1896: ䷙䷈ Cultivating Homegrown: 1 1 3 1 1 1 2 1 1  : S R1 G1 M1 M2 P D1 N2 N3 : OlOm

#1897: C D♭ D F F♯ G A B♭ B :0 1 2 5 6 7 9 10 11: 1 ♭2 2 4 ♯4 5 6 ♭7 7: ma13sus♯4(add ♭7 ♭9)
namesBySequenceIndex[1896] = '''Galygic'''
namesBySequenceIndexNegative[1896] = ''''''
semitonesBySequenceIndex[1896] = [0, 1, 2, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1896] = '''B D F K L M P R S'''
poemAuthors[1896] = ''''''
#1897: ䷙䷉ Cultivating Treading: 1 1 3 1 1 2 1 1 1  : S R1 G1 M1 M2 P D2 N2 N3 : OlEg

#1898: C D♭ D F G♭ A♭ A B♭ B :0 1 2 5 6 8 9 10 11: 1 ♭2 2 4 ♭5 ♭6 6 ♭7 7: ma13sus♯4(♯5 add ♭7 ♭9)
namesBySequenceIndex[1897] = '''Zynygic'''
namesBySequenceIndexNegative[1897] = ''''''
semitonesBySequenceIndex[1897] = [0, 1, 2, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1897] = '''B D F K L N P R S'''
poemAuthors[1897] = ''''''
#1898: ䷙䷌ Cultivating Fellowship: 1 1 3 1 2 1 1 1 1  : S R1 G1 M1 M2 D1 D2 N2 N3 : OlIg

#1899: C D♭ D F G A♭ A B♭ B :0 1 2 5 7 8 9 10 11: 1 ♭2 2 4 5 ♭6 6 ♭7 7: ma13sus♭2(add ♭7 ♭13)
namesBySequenceIndex[1898] = '''Doptygic'''
namesBySequenceIndexNegative[1898] = ''''''
semitonesBySequenceIndex[1898] = [0, 1, 2, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1898] = '''B D F K M N P R S'''
poemAuthors[1898] = ''''''
#1899: ䷙䷫ Cultivating Coupling: 1 1 3 2 1 1 1 1 1  : S R1 G1 M1 P D1 D2 N2 N3 : Olog

#1900: C D♭ D F♯ G A♭ A B♭ B :0 1 2 6 7 8 9 10 11: 1 ♭2 2 ♯4 5 ♭6 6 ♭7 7: ma13sus♭2(♯11 add ♭7 ♭13)
namesBySequenceIndex[1899] = '''Ionogygic'''
namesBySequenceIndexNegative[1899] = ''''''
semitonesBySequenceIndex[1899] = [0, 1, 2, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1899] = '''B D F L M N P R S'''
poemAuthors[1899] = ''''''
#1900: ䷊䷀ Peace Creative: 1 1 4 1 1 1 1 1 1  : S R1 G1 M2 P D1 D2 N2 N3 : OjOg

#1901: C D♭ D♯ E F F♯ G A♭ A :0 1 3 4 5 6 7 8 9: 1 ♭2 ♯2 3 4 ♯4 5 ♭6 6: 6(add ♭6 ♭9 ♯9 11 ♯11)
namesBySequenceIndex[1900] = '''Aeranygic'''
namesBySequenceIndexNegative[1900] = ''''''
semitonesBySequenceIndex[1900] = [0, 1, 3, 4, 5, 6, 7, 8, 9]
poemBySequenceIndex[1900] = '''B D G J K L M N P'''
poemAuthors[1900] = ''''''
#1901: ䷉䷡ Treading Might: 1 2 1 1 1 1 1 1 3  : S R1 R3 G3 M1 M2 P D1 N1 : EgOb

#1902: C D♭ D♯ E F F♯ G A♭ B♭ :0 1 3 4 5 6 7 8 10: 1 ♭2 ♯2 3 4 ♯4 5 ♭6 ♭7: 11(♭9 add ♯9 ♯11 ♭13)
namesBySequenceIndex[1901] = '''Styptygic'''
namesBySequenceIndexNegative[1901] = ''''''
semitonesBySequenceIndex[1901] = [0, 1, 3, 4, 5, 6, 7, 8, 10]
poemBySequenceIndex[1901] = '''B D G J K L M N R'''
poemAuthors[1901] = ''''''
#1902: ䷉䷄ Treading Biding: 1 2 1 1 1 1 1 2 2  : S R1 R3 G3 M1 M2 P D1 N2 : EgOk

#1903: C D♭ D♯ E F F♯ G A♭ B :0 1 3 4 5 6 7 8 11: 1 ♭2 ♯2 3 4 ♯4 5 ♭6 7: ma11(♭9 add ♯9 ♯11 ♭13)
namesBySequenceIndex[1902] = '''Bythygic'''
namesBySequenceIndexNegative[1902] = ''''''
semitonesBySequenceIndex[1902] = [0, 1, 3, 4, 5, 6, 7, 8, 11]
poemBySequenceIndex[1902] = '''B D G J K L M N S'''
poemAuthors[1902] = ''''''
#1903: ䷉䷙ Treading Cultivating: 1 2 1 1 1 1 1 3 1  : S R1 R3 G3 M1 M2 P D1 N3 : EgOl

#1904: C D♭ D♯ E F F♯ G A B♭ :0 1 3 4 5 6 7 9 10: 1 ♭2 ♯2 3 4 ♯4 5 6 ♭7: 13(♭9 add ♯9 ♯11)
namesBySequenceIndex[1903] = '''Mycrygic,Dominant Nonatonic'''
namesBySequenceIndexNegative[1903] = ''''''
semitonesBySequenceIndex[1903] = [0, 1, 3, 4, 5, 6, 7, 9, 10]
poemBySequenceIndex[1903] = '''B D G J K L M P R'''
poemAuthors[1903] = ''''''
#1904: ䷉䷹ Treading Joyous: 1 2 1 1 1 1 2 1 2  : S R1 R3 G3 M1 M2 P D2 N2 : EgEd

#1905: C D♭ D♯ E F F♯ G A B :0 1 3 4 5 6 7 9 11: 1 ♭2 ♯2 3 4 ♯4 5 6 7: ma13(♭9 add ♯9 ♯11)
namesBySequenceIndex[1904] = '''Barygic'''
namesBySequenceIndexNegative[1904] = ''''''
semitonesBySequenceIndex[1904] = [0, 1, 3, 4, 5, 6, 7, 9, 11]
poemBySequenceIndex[1904] = '''B D G J K L M P S'''
poemAuthors[1904] = ''''''
#1905: ䷉䷥ Treading Polarising: 1 2 1 1 1 1 2 2 1  : S R1 R3 G3 M1 M2 P D2 N3 : EgEf

#1906: C D♭ D♯ E F F♯ G A♯ B :0 1 3 4 5 6 7 10 11: 1 ♭2 ♯2 3 4 ♯4 5 ♯6 7: ma11(♭9 add ♭7 ♯9 ♯11)
namesBySequenceIndex[1905] = '''Lacrygic'''
namesBySequenceIndexNegative[1905] = ''''''
semitonesBySequenceIndex[1905] = [0, 1, 3, 4, 5, 6, 7, 10, 11]
poemBySequenceIndex[1905] = '''B D G J K L M R S'''
poemAuthors[1905] = ''''''
#1906: ䷉䷼ Treading Truth: 1 2 1 1 1 1 3 1 1  : S R1 R3 G3 M1 M2 P D3 N3 : EgEm

#1907: C D♭ D♯ E F G♭ A♭ A B♭ :0 1 3 4 5 6 8 9 10: 1 ♭2 ♯2 3 4 ♭5 ♭6 6 ♭7: 13(♯5 ♭9 add ♯9 ♯11)
namesBySequenceIndex[1906] = '''Thydygic'''
namesBySequenceIndexNegative[1906] = ''''''
semitonesBySequenceIndex[1906] = [0, 1, 3, 4, 5, 6, 8, 9, 10]
poemBySequenceIndex[1906] = '''B D G J K L N P R'''
poemAuthors[1906] = ''''''
#1907: ䷉䷰ Treading Revolution: 1 2 1 1 1 2 1 1 2  : S R1 R3 G3 M1 M2 D1 D2 N2 : EgId

#1908: C D♭ D♯ E F G♭ A♭ A B :0 1 3 4 5 6 8 9 11: 1 ♭2 ♯2 3 4 ♭5 ♭6 6 7: ma13(♯5 ♭9 add ♯9 ♯11)
namesBySequenceIndex[1907] = '''Goptygic'''
namesBySequenceIndexNegative[1907] = ''''''
semitonesBySequenceIndex[1907] = [0, 1, 3, 4, 5, 6, 8, 9, 11]
poemBySequenceIndex[1907] = '''B D G J K L N P S'''
poemAuthors[1907] = ''''''
#1908: ䷉䷝ Treading Distanced: 1 2 1 1 1 2 1 2 1  : S R1 R3 G3 M1 M2 D1 D2 N3 : EgIf

#1909: C D♭ D♯ E F G♭ A♭ B♭ B :0 1 3 4 5 6 8 10 11: 1 ♭2 ♯2 3 4 ♭5 ♭6 ♭7 7: ma11(♯5 ♭9 add ♭7 ♯9 ♯11)
namesBySequenceIndex[1908] = '''Garygic'''
namesBySequenceIndexNegative[1908] = ''''''
semitonesBySequenceIndex[1908] = [0, 1, 3, 4, 5, 6, 8, 10, 11]
poemBySequenceIndex[1908] = '''B D G J K L N R S'''
poemAuthors[1908] = ''''''
#1909: ䷉䷤ Treading Family: 1 2 1 1 1 2 2 1 1  : S R1 R3 G3 M1 M2 D1 N2 N3 : EgIm

#1910: C D♭ D♯ E F G♭ A B♭ B :0 1 3 4 5 6 9 10 11: 1 ♭2 ♯2 3 4 ♭5 6 ♭7 7: ma13(♭5 ♭9 add ♭7 ♯9)
namesBySequenceIndex[1909] = '''Rynygic'''
namesBySequenceIndexNegative[1909] = ''''''
semitonesBySequenceIndex[1909] = [0, 1, 3, 4, 5, 6, 9, 10, 11]
poemBySequenceIndex[1909] = '''B D G J K L P R S'''
poemAuthors[1909] = ''''''
#1910: ䷉䷘ Treading Integrity: 1 2 1 1 1 3 1 1 1  : S R1 R3 G3 M1 M2 D2 N2 N3 : EgAg

#1911: C D♭ D♯ E F G A♭ A B♭ :0 1 3 4 5 7 8 9 10: 1 ♭2 ♯2 3 4 5 ♭6 6 ♭7: 13(♭9 add ♯9 ♭13)
namesBySequenceIndex[1910] = '''Aeradygic'''
namesBySequenceIndexNegative[1910] = ''''''
semitonesBySequenceIndex[1910] = [0, 1, 3, 4, 5, 7, 8, 9, 10]
poemBySequenceIndex[1910] = '''B D G J K M N P R'''
poemAuthors[1910] = ''''''
#1911: ䷉䷛ Treading Excess: 1 2 1 1 2 1 1 1 2  : S R1 R3 G3 M1 P D1 D2 N2 : Egod

#1912: C D♭ D♯ E F G A♭ A B :0 1 3 4 5 7 8 9 11: 1 ♭2 ♯2 3 4 5 ♭6 6 7: ma13(♭9 add ♯9 ♭13)
namesBySequenceIndex[1911] = '''Tcherepnin,Tcherepnin (Russia),Genus Chromaticum,Zydygic'''
namesBySequenceIndexNegative[1911] = ''''''
semitonesBySequenceIndex[1911] = [0, 1, 3, 4, 5, 7, 8, 9, 11]
poemBySequenceIndex[1911] = '''B D G J K M N P S'''
poemAuthors[1911] = ''''''
#1912: ䷉䷱ Treading Holding: 1 2 1 1 2 1 1 2 1  : S R1 R3 G3 M1 P D1 D2 N3 : Egof

#1913: C D♭ D♯ E F G A♭ B♭ B :0 1 3 4 5 7 8 10 11: 1 ♭2 ♯2 3 4 5 ♭6 ♭7 7: ma11(♭9 add ♭7 ♯9 ♭13)
namesBySequenceIndex[1912] = '''Moorish Phrygian,Moorish Phrygian (Spain),Katodygic'''
namesBySequenceIndexNegative[1912] = ''''''
semitonesBySequenceIndex[1912] = [0, 1, 3, 4, 5, 7, 8, 10, 11]
poemBySequenceIndex[1912] = '''B D G J K M N R S'''
poemAuthors[1912] = ''''''
#1913: ䷉䷸ Treading Osmosis: 1 2 1 1 2 1 2 1 1  : S R1 R3 G3 M1 P D1 N2 N3 : Egom

#1914: C D♭ D♯ E F G A B♭ B :0 1 3 4 5 7 9 10 11: 1 ♭2 ♯2 3 4 5 6 ♭7 7: ma13(♭9 add ♭7 ♯9)
namesBySequenceIndex[1913] = '''Stophygic'''
namesBySequenceIndexNegative[1913] = ''''''
semitonesBySequenceIndex[1913] = [0, 1, 3, 4, 5, 7, 9, 10, 11]
poemBySequenceIndex[1913] = '''B D G J K M P R S'''
poemAuthors[1913] = ''''''
#1914: ䷉䷅ Treading Arguing: 1 2 1 1 2 2 1 1 1  : S R1 R3 G3 M1 P D2 N2 N3 : Egeg

#1915: C D♭ D♯ E F G♯ A B♭ B :0 1 3 4 5 8 9 10 11: 1 ♭2 ♯2 3 4 ♯5 6 ♭7 7: ma13(♯5 ♭9 add ♭7 ♯9)
namesBySequenceIndex[1914] = '''Katorygic'''
namesBySequenceIndexNegative[1914] = ''''''
semitonesBySequenceIndex[1914] = [0, 1, 3, 4, 5, 8, 9, 10, 11]
poemBySequenceIndex[1914] = '''B D G J K N P R S'''
poemAuthors[1914] = ''''''
#1915: ䷉䷠ Treading Retiring: 1 2 1 1 3 1 1 1 1  : S R1 R3 G3 M1 D1 D2 N2 N3 : Egig

#1916: C D♭ D♯ E F♯ G A♭ A B♭ :0 1 3 4 6 7 8 9 10: 1 ♭2 ♯2 3 ♯4 5 ♭6 6 ♭7: 13(♭9 ♯11 add ♯9 ♭13)
namesBySequenceIndex[1915] = '''Raphygic'''
namesBySequenceIndexNegative[1915] = ''''''
semitonesBySequenceIndex[1915] = [0, 1, 3, 4, 6, 7, 8, 9, 10]
poemBySequenceIndex[1915] = '''B D G J L M N P R'''
poemAuthors[1915] = ''''''
#1916: ䷹䷪ Joyous Decision: 1 2 1 2 1 1 1 1 2  : S R1 R3 G3 M2 P D1 D2 N2 : EdOd

#1917: C D♭ D♯ E F♯ G A♭ A B :0 1 3 4 6 7 8 9 11: 1 ♭2 ♯2 3 ♯4 5 ♭6 6 7: ma13(♭9 ♯11 add ♯9 ♭13)
namesBySequenceIndex[1916] = '''Gocrygic'''
namesBySequenceIndexNegative[1916] = ''''''
semitonesBySequenceIndex[1916] = [0, 1, 3, 4, 6, 7, 8, 9, 11]
poemBySequenceIndex[1916] = '''B D G J L M N P S'''
poemAuthors[1916] = ''''''
#1917: ䷹䷍ Joyous Resevoir: 1 2 1 2 1 1 1 2 1  : S R1 R3 G3 M2 P D1 D2 N3 : EdOf

#1918: C D♭ D♯ E F♯ G A♭ B♭ B :0 1 3 4 6 7 8 10 11: 1 ♭2 ♯2 3 ♯4 5 ♭6 ♭7 7: ma7(add ♭7 ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1917] = '''Sadygic'''
namesBySequenceIndexNegative[1917] = ''''''
semitonesBySequenceIndex[1917] = [0, 1, 3, 4, 6, 7, 8, 10, 11]
poemBySequenceIndex[1917] = '''B D G J L M N R S'''
poemAuthors[1917] = ''''''
#1918: ䷹䷈ Joyous Homegrown: 1 2 1 2 1 1 2 1 1  : S R1 R3 G3 M2 P D1 N2 N3 : EdOm

#1919: C D♭ D♯ E F♯ G A B♭ B :0 1 3 4 6 7 9 10 11: 1 ♭2 ♯2 3 ♯4 5 6 ♭7 7: ma13(♭9 ♯11 add ♭7 ♯9)
namesBySequenceIndex[1918] = '''Epidygic'''
namesBySequenceIndexNegative[1918] = ''''''
semitonesBySequenceIndex[1918] = [0, 1, 3, 4, 6, 7, 9, 10, 11]
poemBySequenceIndex[1918] = '''B D G J L M P R S'''
poemAuthors[1918] = ''''''
#1919: ䷹䷉ Joyous Treading: 1 2 1 2 1 2 1 1 1  : S R1 R3 G3 M2 P D2 N2 N3 : EdEg

#1920: C D♭ D♯ E F♯ G♯ A B♭ B :0 1 3 4 6 8 9 10 11: 1 ♭2 ♯2 3 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♭9 ♯11 add ♭7 ♯9)
namesBySequenceIndex[1919] = '''Aerygic'''
namesBySequenceIndexNegative[1919] = ''''''
semitonesBySequenceIndex[1919] = [0, 1, 3, 4, 6, 8, 9, 10, 11]
poemBySequenceIndex[1919] = '''B D G J L N P R S'''
poemAuthors[1919] = ''''''
#1920: ䷹䷌ Joyous Fellowship: 1 2 1 2 2 1 1 1 1  : S R1 R3 G3 M2 D1 D2 N2 N3 : EdIg

#1921: C D♭ D♯ E G A♭ A B♭ B :0 1 3 4 7 8 9 10 11: 1 ♭2 ♯2 3 5 ♭6 6 ♭7 7: ma7(add ♭7 ♭9 ♯9 ♭13 13)
namesBySequenceIndex[1920] = '''Ionygic'''
namesBySequenceIndexNegative[1920] = ''''''
semitonesBySequenceIndex[1920] = [0, 1, 3, 4, 7, 8, 9, 10, 11]
poemBySequenceIndex[1920] = '''B D G J M N P R S'''
poemAuthors[1920] = ''''''
#1921: ䷹䷫ Joyous Coupling: 1 2 1 3 1 1 1 1 1  : S R1 R3 G3 P D1 D2 N2 N3 : Edog

#1922: C D♭ E♭ F F♯ G A♭ A B♭ :0 1 3 5 6 7 8 9 10: 1 ♭2 ♭3 4 ♯4 5 ♭6 6 ♭7: mi13(♭9 add ♯11 ♭13)
namesBySequenceIndex[1921] = '''Boptygic'''
namesBySequenceIndexNegative[1921] = ''''''
semitonesBySequenceIndex[1921] = [0, 1, 3, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1921] = '''B D G K L M N P R'''
poemAuthors[1921] = ''''''
#1922: ䷥䷪ Polarising Decision: 1 2 2 1 1 1 1 1 2  : S R1 G2 M1 M2 P D1 D2 N2 : EfOd

#1923: C D♭ E♭ F F♯ G A♭ A B :0 1 3 5 6 7 8 9 11: 1 ♭2 ♭3 4 ♯4 5 ♭6 6 7: mima13(♭9 add ♯11 ♭13)
namesBySequenceIndex[1922] = '''Stalygic'''
namesBySequenceIndexNegative[1922] = ''''''
semitonesBySequenceIndex[1922] = [0, 1, 3, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1922] = '''B D G K L M N P S'''
poemAuthors[1922] = ''''''
#1923: ䷥䷍ Polarising Resevoir: 1 2 2 1 1 1 1 2 1  : S R1 G2 M1 M2 P D1 D2 N3 : EfOf

#1924: C D♭ E♭ F F♯ G A♭ B♭ B :0 1 3 5 6 7 8 10 11: 1 ♭2 ♭3 4 ♯4 5 ♭6 ♭7 7: mima11(♭9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[1923] = '''Ionoptygic'''
namesBySequenceIndexNegative[1923] = ''''''
semitonesBySequenceIndex[1923] = [0, 1, 3, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1923] = '''B D G K L M N R S'''
poemAuthors[1923] = ''''''
#1924: ䷥䷈ Polarising Homegrown: 1 2 2 1 1 1 2 1 1  : S R1 G2 M1 M2 P D1 N2 N3 : EfOm

#1925: C D♭ E♭ F F♯ G A B♭ B :0 1 3 5 6 7 9 10 11: 1 ♭2 ♭3 4 ♯4 5 6 ♭7 7: mima13(♭9 add ♭7 ♯11)
namesBySequenceIndex[1924] = '''Aeolanygic'''
namesBySequenceIndexNegative[1924] = ''''''
semitonesBySequenceIndex[1924] = [0, 1, 3, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1924] = '''B D G K L M P R S'''
poemAuthors[1924] = ''''''
#1925: ䷥䷉ Polarising Treading: 1 2 2 1 1 2 1 1 1  : S R1 G2 M1 M2 P D2 N2 N3 : EfEg

#1926: C D♭ E♭ F G♭ A♭ A B♭ B :0 1 3 5 6 8 9 10 11: 1 ♭2 ♭3 4 ♭5 ♭6 6 ♭7 7: °ma13(♭9 add ♭7 ♭13)
namesBySequenceIndex[1925] = '''Ryptygic'''
namesBySequenceIndexNegative[1925] = ''''''
semitonesBySequenceIndex[1925] = [0, 1, 3, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1925] = '''B D G K L N P R S'''
poemAuthors[1925] = ''''''
#1926: ䷥䷌ Polarising Fellowship: 1 2 2 1 2 1 1 1 1  : S R1 G2 M1 M2 D1 D2 N2 N3 : EfIg

#1927: C D♭ E♭ F G A♭ A B♭ B :0 1 3 5 7 8 9 10 11: 1 ♭2 ♭3 4 5 ♭6 6 ♭7 7: mima13(♭9 add ♭7 ♭13)
namesBySequenceIndex[1926] = '''Styrygic'''
namesBySequenceIndexNegative[1926] = ''''''
semitonesBySequenceIndex[1926] = [0, 1, 3, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1926] = '''B D G K M N P R S'''
poemAuthors[1926] = ''''''
#1927: ䷥䷫ Polarising Coupling: 1 2 2 2 1 1 1 1 1  : S R1 G2 M1 P D1 D2 N2 N3 : Efog

#1928: C D♭ E♭ F♯ G A♭ A B♭ B :0 1 3 6 7 8 9 10 11: 1 ♭2 ♭3 ♯4 5 ♭6 6 ♭7 7: mima13(♭9 ♯11 add ♭7 ♭13)
namesBySequenceIndex[1927] = '''Phrocrygic'''
namesBySequenceIndexNegative[1927] = ''''''
semitonesBySequenceIndex[1927] = [0, 1, 3, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1927] = '''B D G L M N P R S'''
poemAuthors[1927] = ''''''
#1928: ䷵䷀ Betrothing Creative: 1 2 3 1 1 1 1 1 1  : S R1 G2 M2 P D1 D2 N2 N3 : EbOg

#1929: C D♭ E F F♯ G A♭ A B♭ :0 1 4 5 6 7 8 9 10: 1 ♭2 3 4 ♯4 5 ♭6 6 ♭7: 13(♭9 add ♯11 ♭13)
namesBySequenceIndex[1928] = '''Aerythygic'''
namesBySequenceIndexNegative[1928] = ''''''
semitonesBySequenceIndex[1928] = [0, 1, 4, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1928] = '''B D J K L M N P R'''
poemAuthors[1928] = ''''''
#1929: ䷼䷪ Truth Decision: 1 3 1 1 1 1 1 1 2  : S R1 G3 M1 M2 P D1 D2 N2 : EmOd

#1930: C D♭ E F F♯ G A♭ A B :0 1 4 5 6 7 8 9 11: 1 ♭2 3 4 ♯4 5 ♭6 6 7: ma13(♭9 add ♯11 ♭13)
namesBySequenceIndex[1929] = '''Madygic'''
namesBySequenceIndexNegative[1929] = ''''''
semitonesBySequenceIndex[1929] = [0, 1, 4, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1929] = '''B D J K L M N P S'''
poemAuthors[1929] = ''''''
#1930: ䷼䷍ Truth Resevoir: 1 3 1 1 1 1 1 2 1  : S R1 G3 M1 M2 P D1 D2 N3 : EmOf

#1931: C D♭ E F F♯ G A♭ B♭ B :0 1 4 5 6 7 8 10 11: 1 ♭2 3 4 ♯4 5 ♭6 ♭7 7: ma11(♭9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[1930] = '''Dyrygic'''
namesBySequenceIndexNegative[1930] = ''''''
semitonesBySequenceIndex[1930] = [0, 1, 4, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1930] = '''B D J K L M N R S'''
poemAuthors[1930] = ''''''
#1931: ䷼䷈ Truth Homegrown: 1 3 1 1 1 1 2 1 1  : S R1 G3 M1 M2 P D1 N2 N3 : EmOm

#1932: C D♭ E F F♯ G A B♭ B :0 1 4 5 6 7 9 10 11: 1 ♭2 3 4 ♯4 5 6 ♭7 7: ma13(♭9 add ♭7 ♯11)
namesBySequenceIndex[1931] = '''Bodygic'''
namesBySequenceIndexNegative[1931] = ''''''
semitonesBySequenceIndex[1931] = [0, 1, 4, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1931] = '''B D J K L M P R S'''
poemAuthors[1931] = ''''''
#1932: ䷼䷉ Truth Treading: 1 3 1 1 1 2 1 1 1  : S R1 G3 M1 M2 P D2 N2 N3 : EmEg

#1933: C D♭ E F G♭ A♭ A B♭ B :0 1 4 5 6 8 9 10 11: 1 ♭2 3 4 ♭5 ♭6 6 ♭7 7: ma13(♯5 ♭9 add ♭7 ♯11)
namesBySequenceIndex[1932] = '''Pothygic'''
namesBySequenceIndexNegative[1932] = ''''''
semitonesBySequenceIndex[1932] = [0, 1, 4, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1932] = '''B D J K L N P R S'''
poemAuthors[1932] = ''''''
#1933: ䷼䷌ Truth Fellowship: 1 3 1 1 2 1 1 1 1  : S R1 G3 M1 M2 D1 D2 N2 N3 : EmIg

#1934: C D♭ E F G A♭ A B♭ B :0 1 4 5 7 8 9 10 11: 1 ♭2 3 4 5 ♭6 6 ♭7 7: ma13(♭9 add ♭7 ♭13)
namesBySequenceIndex[1933] = '''Lonygic'''
namesBySequenceIndexNegative[1933] = ''''''
semitonesBySequenceIndex[1933] = [0, 1, 4, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1933] = '''B D J K M N P R S'''
poemAuthors[1933] = ''''''
#1934: ䷼䷫ Truth Coupling: 1 3 1 2 1 1 1 1 1  : S R1 G3 M1 P D1 D2 N2 N3 : Emog

#1935: C D♭ E F♯ G A♭ A B♭ B :0 1 4 6 7 8 9 10 11: 1 ♭2 3 ♯4 5 ♭6 6 ♭7 7: ma13(♭9 ♯11 add ♭7 ♭13)
namesBySequenceIndex[1934] = '''Ionilygic'''
namesBySequenceIndexNegative[1934] = ''''''
semitonesBySequenceIndex[1934] = [0, 1, 4, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1934] = '''B D J L M N P R S'''
poemAuthors[1934] = ''''''
#1935: ䷻䷀ Composed Creative: 1 3 2 1 1 1 1 1 1  : S R1 G3 M2 P D1 D2 N2 N3 : EkOg

#1936: C D♭ F F♯ G A♭ A B♭ B :0 1 5 6 7 8 9 10 11: 1 ♭2 4 ♯4 5 ♭6 6 ♭7 7: ma13sus♯4(♭9 add ♭7 ♭13)
namesBySequenceIndex[1935] = '''Aeolarygic'''
namesBySequenceIndexNegative[1935] = ''''''
semitonesBySequenceIndex[1935] = [0, 1, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1935] = '''B D K L M N P R S'''
poemAuthors[1935] = ''''''
#1936: ䷨䷀ Diminishing Creative: 1 4 1 1 1 1 1 1 1  : S R1 M1 M2 P D1 D2 N2 N3 : ElOg

#1937: C D D♯ E F F♯ G A♭ A :0 2 3 4 5 6 7 8 9: 1 2 ♯2 3 4 ♯4 5 ♭6 6: 13(no 7 add ♭6 ♯9 ♯11)
namesBySequenceIndex[1936] = '''Ladygic'''
namesBySequenceIndexNegative[1936] = ''''''
semitonesBySequenceIndex[1936] = [0, 2, 3, 4, 5, 6, 7, 8, 9]
poemBySequenceIndex[1936] = '''B F G J K L M N P'''
poemAuthors[1936] = ''''''
#1937: ䷌䷡ Fellowship Might: 2 1 1 1 1 1 1 1 3  : S R2 R3 G3 M1 M2 P D1 N1 : IgOb

#1938: C D D♯ E F F♯ G A♭ B♭ :0 2 3 4 5 6 7 8 10: 1 2 ♯2 3 4 ♯4 5 ♭6 ♭7: 11(add ♯9 ♯11 ♭13)
namesBySequenceIndex[1937] = '''Katycrygic'''
namesBySequenceIndexNegative[1937] = ''''''
semitonesBySequenceIndex[1937] = [0, 2, 3, 4, 5, 6, 7, 8, 10]
poemBySequenceIndex[1937] = '''B F G J K L M N R'''
poemAuthors[1937] = ''''''
#1938: ䷌䷄ Fellowship Biding: 2 1 1 1 1 1 1 2 2  : S R2 R3 G3 M1 M2 P D1 N2 : IgOk

#1939: C D D♯ E F F♯ G A♭ B :0 2 3 4 5 6 7 8 11: 1 2 ♯2 3 4 ♯4 5 ♭6 7: ma11(add ♯9 ♯11 ♭13)
namesBySequenceIndex[1938] = '''Dothygic'''
namesBySequenceIndexNegative[1938] = ''''''
semitonesBySequenceIndex[1938] = [0, 2, 3, 4, 5, 6, 7, 8, 11]
poemBySequenceIndex[1938] = '''B F G J K L M N S'''
poemAuthors[1938] = ''''''
#1939: ䷌䷙ Fellowship Cultivating: 2 1 1 1 1 1 1 3 1  : S R2 R3 G3 M1 M2 P D1 N3 : IgOl

#1940: C D D♯ E F F♯ G A B♭ :0 2 3 4 5 6 7 9 10: 1 2 ♯2 3 4 ♯4 5 6 ♭7: 13(add ♯9 ♯11)
namesBySequenceIndex[1939] = '''Katagygic,Blues Mixolydian'''
namesBySequenceIndexNegative[1939] = ''''''
semitonesBySequenceIndex[1939] = [0, 2, 3, 4, 5, 6, 7, 9, 10]
poemBySequenceIndex[1939] = '''B F G J K L M P R'''
poemAuthors[1939] = ''''''
#1940: ䷌䷹ Fellowship Joyous: 2 1 1 1 1 1 2 1 2  : S R2 R3 G3 M1 M2 P D2 N2 : IgEd

#1941: C D D♯ E F F♯ G A B :0 2 3 4 5 6 7 9 11: 1 2 ♯2 3 4 ♯4 5 6 7: ma13(add ♯9 ♯11)
namesBySequenceIndex[1940] = '''Zolygic'''
namesBySequenceIndexNegative[1940] = ''''''
semitonesBySequenceIndex[1940] = [0, 2, 3, 4, 5, 6, 7, 9, 11]
poemBySequenceIndex[1940] = '''B F G J K L M P S'''
poemAuthors[1940] = ''''''
#1941: ䷌䷥ Fellowship Polarising: 2 1 1 1 1 1 2 2 1  : S R2 R3 G3 M1 M2 P D2 N3 : IgEf

#1942: C D D♯ E F F♯ G A♯ B :0 2 3 4 5 6 7 10 11: 1 2 ♯2 3 4 ♯4 5 ♯6 7: ma11(add ♭7 ♯9 ♯11)
namesBySequenceIndex[1941] = '''Mixolygic'''
namesBySequenceIndexNegative[1941] = ''''''
semitonesBySequenceIndex[1941] = [0, 2, 3, 4, 5, 6, 7, 10, 11]
poemBySequenceIndex[1941] = '''B F G J K L M R S'''
poemAuthors[1941] = ''''''
#1942: ䷌䷼ Fellowship Truth: 2 1 1 1 1 1 3 1 1  : S R2 R3 G3 M1 M2 P D3 N3 : IgEm

#1943: C D D♯ E F G♭ A♭ A B♭ :0 2 3 4 5 6 8 9 10: 1 2 ♯2 3 4 ♭5 ♭6 6 ♭7: 13(♯5 add ♯9 ♯11)
namesBySequenceIndex[1942] = '''Sacrygic'''
namesBySequenceIndexNegative[1942] = ''''''
semitonesBySequenceIndex[1942] = [0, 2, 3, 4, 5, 6, 8, 9, 10]
poemBySequenceIndex[1942] = '''B F G J K L N P R'''
poemAuthors[1942] = ''''''
#1943: ䷌䷰ Fellowship Revolution: 2 1 1 1 1 2 1 1 2  : S R2 R3 G3 M1 M2 D1 D2 N2 : IgId

#1944: C D D♯ E F G♭ A♭ A B :0 2 3 4 5 6 8 9 11: 1 2 ♯2 3 4 ♭5 ♭6 6 7: ma13(♯5 add ♯9 ♯11)
namesBySequenceIndex[1943] = '''Laptygic'''
namesBySequenceIndexNegative[1943] = ''''''
semitonesBySequenceIndex[1943] = [0, 2, 3, 4, 5, 6, 8, 9, 11]
poemBySequenceIndex[1943] = '''B F G J K L N P S'''
poemAuthors[1943] = ''''''
#1944: ䷌䷝ Fellowship Distanced: 2 1 1 1 1 2 1 2 1  : S R2 R3 G3 M1 M2 D1 D2 N3 : IgIf

#1945: C D D♯ E F G♭ A♭ B♭ B :0 2 3 4 5 6 8 10 11: 1 2 ♯2 3 4 ♭5 ♭6 ♭7 7: ma11(♯5 add ♭7 ♯9 ♯11)
namesBySequenceIndex[1944] = '''Katarygic'''
namesBySequenceIndexNegative[1944] = ''''''
semitonesBySequenceIndex[1944] = [0, 2, 3, 4, 5, 6, 8, 10, 11]
poemBySequenceIndex[1944] = '''B F G J K L N R S'''
poemAuthors[1944] = ''''''
#1945: ䷌䷤ Fellowship Family: 2 1 1 1 1 2 2 1 1  : S R2 R3 G3 M1 M2 D1 N2 N3 : IgIm

#1946: C D D♯ E F G♭ A B♭ B :0 2 3 4 5 6 9 10 11: 1 2 ♯2 3 4 ♭5 6 ♭7 7: ma13(♭5 add ♭7 ♯9)
namesBySequenceIndex[1945] = '''Katynygic'''
namesBySequenceIndexNegative[1945] = ''''''
semitonesBySequenceIndex[1945] = [0, 2, 3, 4, 5, 6, 9, 10, 11]
poemBySequenceIndex[1945] = '''B F G J K L P R S'''
poemAuthors[1945] = ''''''
#1946: ䷌䷘ Fellowship Integrity: 2 1 1 1 1 3 1 1 1  : S R2 R3 G3 M1 M2 D2 N2 N3 : IgAg

#1947: C D D♯ E F G A♭ A B♭ :0 2 3 4 5 7 8 9 10: 1 2 ♯2 3 4 5 ♭6 6 ♭7: 13(add ♯9 ♭13)
namesBySequenceIndex[1946] = '''Houseini,Gadygic'''
namesBySequenceIndexNegative[1946] = ''''''
semitonesBySequenceIndex[1946] = [0, 2, 3, 4, 5, 7, 8, 9, 10]
poemBySequenceIndex[1946] = '''B F G J K M N P R'''
poemAuthors[1946] = ''''''
#1947: ䷌䷛ Fellowship Excess: 2 1 1 1 2 1 1 1 2  : S R2 R3 G3 M1 P D1 D2 N2 : Igod

#1948: C D D♯ E F G A♭ A B :0 2 3 4 5 7 8 9 11: 1 2 ♯2 3 4 5 ♭6 6 7: ma13(add ♯9 ♭13)
namesBySequenceIndex[1947] = '''Gycrygic'''
namesBySequenceIndexNegative[1947] = ''''''
semitonesBySequenceIndex[1947] = [0, 2, 3, 4, 5, 7, 8, 9, 11]
poemBySequenceIndex[1947] = '''B F G J K M N P S'''
poemAuthors[1947] = ''''''
#1948: ䷌䷱ Fellowship Holding: 2 1 1 1 2 1 1 2 1  : S R2 R3 G3 M1 P D1 D2 N3 : Igof

#1949: C D D♯ E F G A♭ B♭ B :0 2 3 4 5 7 8 10 11: 1 2 ♯2 3 4 5 ♭6 ♭7 7: ma11(add ♭7 ♯9 ♭13)
namesBySequenceIndex[1948] = '''Epocrygic'''
namesBySequenceIndexNegative[1948] = ''''''
semitonesBySequenceIndex[1948] = [0, 2, 3, 4, 5, 7, 8, 10, 11]
poemBySequenceIndex[1948] = '''B F G J K M N R S'''
poemAuthors[1948] = ''''''
#1949: ䷌䷸ Fellowship Osmosis: 2 1 1 1 2 1 2 1 1  : S R2 R3 G3 M1 P D1 N2 N3 : Igom

#1950: C D D♯ E F G A B♭ B :0 2 3 4 5 7 9 10 11: 1 2 ♯2 3 4 5 6 ♭7 7: ma13(add ♭7 ♯9)
namesBySequenceIndex[1949] = '''Raga Malgunji,Blues Enneatonic,Raga Ramdasi Malhar,Major-Dorian Mixed,Sorygic'''
namesBySequenceIndexNegative[1949] = ''''''
semitonesBySequenceIndex[1949] = [0, 2, 3, 4, 5, 7, 9, 10, 11]
poemBySequenceIndex[1949] = '''B F G J K M P R S'''
poemAuthors[1949] = ''''''
#1950: ䷌䷅ Fellowship Arguing: 2 1 1 1 2 2 1 1 1  : S R2 R3 G3 M1 P D2 N2 N3 : Igeg

#1951: C D D♯ E F G♯ A B♭ B :0 2 3 4 5 8 9 10 11: 1 2 ♯2 3 4 ♯5 6 ♭7 7: ma13(♯5 add ♭7 ♯9)
namesBySequenceIndex[1950] = '''Bycrygic'''
namesBySequenceIndexNegative[1950] = ''''''
semitonesBySequenceIndex[1950] = [0, 2, 3, 4, 5, 8, 9, 10, 11]
poemBySequenceIndex[1950] = '''B F G J K N P R S'''
poemAuthors[1950] = ''''''
#1951: ䷌䷠ Fellowship Retiring: 2 1 1 1 3 1 1 1 1  : S R2 R3 G3 M1 D1 D2 N2 N3 : Igig

#1952: C D D♯ E F♯ G A♭ A B♭ :0 2 3 4 6 7 8 9 10: 1 2 ♯2 3 ♯4 5 ♭6 6 ♭7: 13(♯11 add ♯9 ♭13)
namesBySequenceIndex[1951] = '''Mocrygic'''
namesBySequenceIndexNegative[1951] = ''''''
semitonesBySequenceIndex[1951] = [0, 2, 3, 4, 6, 7, 8, 9, 10]
poemBySequenceIndex[1951] = '''B F G J L M N P R'''
poemAuthors[1951] = ''''''
#1952: ䷰䷪ Revolution Decision: 2 1 1 2 1 1 1 1 2  : S R2 R3 G3 M2 P D1 D2 N2 : IdOd

#1953: C D D♯ E F♯ G A♭ A B :0 2 3 4 6 7 8 9 11: 1 2 ♯2 3 ♯4 5 ♭6 6 7: ma13(♯11 add ♯9 ♭13)
namesBySequenceIndex[1952] = '''Staptygic'''
namesBySequenceIndexNegative[1952] = ''''''
semitonesBySequenceIndex[1952] = [0, 2, 3, 4, 6, 7, 8, 9, 11]
poemBySequenceIndex[1952] = '''B F G J L M N P S'''
poemAuthors[1952] = ''''''
#1953: ䷰䷍ Revolution Resevoir: 2 1 1 2 1 1 1 2 1  : S R2 R3 G3 M2 P D1 D2 N3 : IdOf

#1954: C D D♯ E F♯ G A♭ B♭ B :0 2 3 4 6 7 8 10 11: 1 2 ♯2 3 ♯4 5 ♭6 ♭7 7: ma9(add ♭7 ♯9 ♯11 ♭13)
namesBySequenceIndex[1953] = '''Messiaen 3rd Mode,Genus Chromaticum (III),Phronygic'''
namesBySequenceIndexNegative[1953] = ''''''
semitonesBySequenceIndex[1953] = [0, 2, 3, 4, 6, 7, 8, 10, 11]
poemBySequenceIndex[1953] = '''B F G J L M N R S'''
poemAuthors[1953] = ''''''
#1954: ䷰䷈ Revolution Homegrown: 2 1 1 2 1 1 2 1 1  : S R2 R3 G3 M2 P D1 N2 N3 : IdOm

#1955: C D D♯ E F♯ G A B♭ B :0 2 3 4 6 7 9 10 11: 1 2 ♯2 3 ♯4 5 6 ♭7 7: ma13(♯11 add ♭7 ♯9)
namesBySequenceIndex[1954] = '''Moptygic'''
namesBySequenceIndexNegative[1954] = ''''''
semitonesBySequenceIndex[1954] = [0, 2, 3, 4, 6, 7, 9, 10, 11]
poemBySequenceIndex[1954] = '''B F G J L M P R S'''
poemAuthors[1954] = ''''''
#1955: ䷰䷉ Revolution Treading: 2 1 1 2 1 2 1 1 1  : S R2 R3 G3 M2 P D2 N2 N3 : IdEg

#1956: C D D♯ E F♯ G♯ A B♭ B :0 2 3 4 6 8 9 10 11: 1 2 ♯2 3 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♯11 add ♭7 ♯9)
namesBySequenceIndex[1955] = '''Ionidygic'''
namesBySequenceIndexNegative[1955] = ''''''
semitonesBySequenceIndex[1955] = [0, 2, 3, 4, 6, 8, 9, 10, 11]
poemBySequenceIndex[1955] = '''B F G J L N P R S'''
poemAuthors[1955] = ''''''
#1956: ䷰䷌ Revolution Fellowship: 2 1 1 2 2 1 1 1 1  : S R2 R3 G3 M2 D1 D2 N2 N3 : IdIg

#1957: C D D♯ E G A♭ A B♭ B :0 2 3 4 7 8 9 10 11: 1 2 ♯2 3 5 ♭6 6 ♭7 7: ma9(add ♭7 ♯9 ♭13 13)
namesBySequenceIndex[1956] = '''Dogygic'''
namesBySequenceIndexNegative[1956] = ''''''
semitonesBySequenceIndex[1956] = [0, 2, 3, 4, 7, 8, 9, 10, 11]
poemBySequenceIndex[1956] = '''B F G J M N P R S'''
poemAuthors[1956] = ''''''
#1957: ䷰䷫ Revolution Coupling: 2 1 1 3 1 1 1 1 1  : S R2 R3 G3 P D1 D2 N2 N3 : Idog

#1958: C D E♭ F F♯ G A♭ A B♭ :0 2 3 5 6 7 8 9 10: 1 2 ♭3 4 ♯4 5 ♭6 6 ♭7: mi13(add ♯11 ♭13)
namesBySequenceIndex[1957] = '''Kiourdi,Mathygic'''
namesBySequenceIndexNegative[1957] = ''''''
semitonesBySequenceIndex[1957] = [0, 2, 3, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1957] = '''B F G K L M N P R'''
poemAuthors[1957] = ''''''
#1958: ䷝䷪ Distanced Decision: 2 1 2 1 1 1 1 1 2  : S R2 G2 M1 M2 P D1 D2 N2 : IfOd

#1959: C D E♭ F F♯ G A♭ A B :0 2 3 5 6 7 8 9 11: 1 2 ♭3 4 ♯4 5 ♭6 6 7: mima13(add ♯11 ♭13)
namesBySequenceIndex[1958] = '''Zycrygic'''
namesBySequenceIndexNegative[1958] = ''''''
semitonesBySequenceIndex[1958] = [0, 2, 3, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1958] = '''B F G K L M N P S'''
poemAuthors[1958] = ''''''
#1959: ䷝䷍ Distanced Resevoir: 2 1 2 1 1 1 1 2 1  : S R2 G2 M1 M2 P D1 D2 N3 : IfOf

#1960: C D E♭ F F♯ G A♭ B♭ B :0 2 3 5 6 7 8 10 11: 1 2 ♭3 4 ♯4 5 ♭6 ♭7 7: mima11(add ♭7 ♯11 ♭13)
namesBySequenceIndex[1959] = '''Aeolorygic'''
namesBySequenceIndexNegative[1959] = ''''''
semitonesBySequenceIndex[1959] = [0, 2, 3, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1959] = '''B F G K L M N R S'''
poemAuthors[1959] = ''''''
#1960: ䷝䷈ Distanced Homegrown: 2 1 2 1 1 1 2 1 1  : S R2 G2 M1 M2 P D1 N2 N3 : IfOm

#1961: C D E♭ F F♯ G A B♭ B :0 2 3 5 6 7 9 10 11: 1 2 ♭3 4 ♯4 5 6 ♭7 7: mima13(add ♭7 ♯11)
namesBySequenceIndex[1960] = '''Epyrygic'''
namesBySequenceIndexNegative[1960] = ''''''
semitonesBySequenceIndex[1960] = [0, 2, 3, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1960] = '''B F G K L M P R S'''
poemAuthors[1960] = ''''''
#1961: ䷝䷉ Distanced Treading: 2 1 2 1 1 2 1 1 1  : S R2 G2 M1 M2 P D2 N2 N3 : IfEg

#1962: C D E♭ F G♭ A♭ A B♭ B :0 2 3 5 6 8 9 10 11: 1 2 ♭3 4 ♭5 ♭6 6 ♭7 7: °ma13(add ♭7 ♭13)
namesBySequenceIndex[1961] = '''Koptygic'''
namesBySequenceIndexNegative[1961] = ''''''
semitonesBySequenceIndex[1961] = [0, 2, 3, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1961] = '''B F G K L N P R S'''
poemAuthors[1961] = ''''''
#1962: ䷝䷌ Distanced Fellowship: 2 1 2 1 2 1 1 1 1  : S R2 G2 M1 M2 D1 D2 N2 N3 : IfIg

#1963: C D E♭ F G A♭ A B♭ B :0 2 3 5 7 8 9 10 11: 1 2 ♭3 4 5 ♭6 6 ♭7 7: mima13(add ♭7 ♭13)
namesBySequenceIndex[1962] = '''Raga Pilu,Word Scale,Full Minor All Flats,Dathygic'''
namesBySequenceIndexNegative[1962] = ''''''
semitonesBySequenceIndex[1962] = [0, 2, 3, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1962] = '''B F G K M N P R S'''
poemAuthors[1962] = ''''''
#1963: ䷝䷫ Distanced Coupling: 2 1 2 2 1 1 1 1 1  : S R2 G2 M1 P D1 D2 N2 N3 : Ifog

#1964: C D E♭ F♯ G A♭ A B♭ B :0 2 3 6 7 8 9 10 11: 1 2 ♭3 ♯4 5 ♭6 6 ♭7 7: mima13(♯11 add ♭7 ♭13)
namesBySequenceIndex[1963] = '''Gyptygic'''
namesBySequenceIndexNegative[1963] = ''''''
semitonesBySequenceIndex[1963] = [0, 2, 3, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1963] = '''B F G L M N P R S'''
poemAuthors[1963] = ''''''
#1964: ䷶䷀ Abounding Creative: 2 1 3 1 1 1 1 1 1  : S R2 G2 M2 P D1 D2 N2 N3 : IbOg

#1965: C D E F F♯ G A♭ A B♭ :0 2 4 5 6 7 8 9 10: 1 2 3 4 ♯4 5 ♭6 6 ♭7: 13(add ♯11 ♭13)
namesBySequenceIndex[1964] = '''Sythygic'''
namesBySequenceIndexNegative[1964] = ''''''
semitonesBySequenceIndex[1964] = [0, 2, 4, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1964] = '''B F J K L M N P R'''
poemAuthors[1964] = ''''''
#1965: ䷤䷪ Family Decision: 2 2 1 1 1 1 1 1 2  : S R2 G3 M1 M2 P D1 D2 N2 : ImOd

#1966: C D E F F♯ G A♭ A B :0 2 4 5 6 7 8 9 11: 1 2 3 4 ♯4 5 ♭6 6 7: ma13(add ♯11 ♭13)
namesBySequenceIndex[1965] = '''Apinygic'''
namesBySequenceIndexNegative[1965] = ''''''
semitonesBySequenceIndex[1965] = [0, 2, 4, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1965] = '''B F J K L M N P S'''
poemAuthors[1965] = ''''''
#1966: ䷤䷍ Family Resevoir: 2 2 1 1 1 1 1 2 1  : S R2 G3 M1 M2 P D1 D2 N3 : ImOf

#1967: C D E F F♯ G A♭ B♭ B :0 2 4 5 6 7 8 10 11: 1 2 3 4 ♯4 5 ♭6 ♭7 7: ma11(add ♭7 ♯11 ♭13)
namesBySequenceIndex[1966] = '''Kaptygic'''
namesBySequenceIndexNegative[1966] = ''''''
semitonesBySequenceIndex[1966] = [0, 2, 4, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1966] = '''B F J K L M N R S'''
poemAuthors[1966] = ''''''
#1967: ䷤䷈ Family Homegrown: 2 2 1 1 1 1 2 1 1  : S R2 G3 M1 M2 P D1 N2 N3 : ImOm

#1968: C D E F F♯ G A B♭ B :0 2 4 5 6 7 9 10 11: 1 2 3 4 ♯4 5 6 ♭7 7: ma13(add ♭7 ♯11)
namesBySequenceIndex[1967] = '''Ryo,Taishikicho (Japan),Lydian Mixolydian,Aerycrygic'''
namesBySequenceIndexNegative[1967] = ''''''
semitonesBySequenceIndex[1967] = [0, 2, 4, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1967] = '''B F J K L M P R S'''
poemAuthors[1967] = ''''''
#1968: ䷤䷉ Family Treading: 2 2 1 1 1 2 1 1 1  : S R2 G3 M1 M2 P D2 N2 N3 : ImEg

#1969: C D E F G♭ A♭ A B♭ B :0 2 4 5 6 8 9 10 11: 1 2 3 4 ♭5 ♭6 6 ♭7 7: ma13(♯5 add ♭7 ♯11)
namesBySequenceIndex[1968] = '''Porygic'''
namesBySequenceIndexNegative[1968] = ''''''
semitonesBySequenceIndex[1968] = [0, 2, 4, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1968] = '''B F J K L N P R S'''
poemAuthors[1968] = ''''''
#1969: ䷤䷌ Family Fellowship: 2 2 1 1 2 1 1 1 1  : S R2 G3 M1 M2 D1 D2 N2 N3 : ImIg

#1970: C D E F G A♭ A B♭ B :0 2 4 5 7 8 9 10 11: 1 2 3 4 5 ♭6 6 ♭7 7: ma13(add ♭7 ♭13)
namesBySequenceIndex[1969] = '''Raga Pahadi,Bagygic'''
namesBySequenceIndexNegative[1969] = ''''''
semitonesBySequenceIndex[1969] = [0, 2, 4, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1969] = '''B F J K M N P R S'''
poemAuthors[1969] = ''''''
#1970: ䷤䷫ Family Coupling: 2 2 1 2 1 1 1 1 1  : S R2 G3 M1 P D1 D2 N2 N3 : Imog

#1971: C D E F♯ G A♭ A B♭ B :0 2 4 6 7 8 9 10 11: 1 2 3 ♯4 5 ♭6 6 ♭7 7: ma13(♯11 add ♭7 ♭13)
namesBySequenceIndex[1970] = '''Kyrygic'''
namesBySequenceIndexNegative[1970] = ''''''
semitonesBySequenceIndex[1970] = [0, 2, 4, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1970] = '''B F J L M N P R S'''
poemAuthors[1970] = ''''''
#1971: ䷾䷀ Completed Creative: 2 2 2 1 1 1 1 1 1  : S R2 G3 M2 P D1 D2 N2 N3 : IkOg

#1972: C D F F♯ G A♭ A B♭ B :0 2 5 6 7 8 9 10 11: 1 2 4 ♯4 5 ♭6 6 ♭7 7: ma13sus♯4(add ♭7 ♭13)
namesBySequenceIndex[1971] = '''Starygic'''
namesBySequenceIndexNegative[1971] = ''''''
semitonesBySequenceIndex[1971] = [0, 2, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1971] = '''B F K L M N P R S'''
poemAuthors[1971] = ''''''
#1972: ䷕䷀ Adorning Creative: 2 3 1 1 1 1 1 1 1  : S R2 M1 M2 P D1 D2 N2 N3 : IlOg

#1973: C D♯ E F F♯ G A♭ A B♭ :0 3 4 5 6 7 8 9 10: 1 ♯2 3 4 ♯4 5 ♭6 6 ♭7: 13(♯9 add ♯11 ♭13)
namesBySequenceIndex[1972] = '''Aeolacrygic'''
namesBySequenceIndexNegative[1972] = ''''''
semitonesBySequenceIndex[1972] = [0, 3, 4, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[1972] = '''B G J K L M N P R'''
poemAuthors[1972] = ''''''
#1973: ䷘䷪ Integrity Decision: 3 1 1 1 1 1 1 1 2  : S R3 G3 M1 M2 P D1 D2 N2 : AgOd

#1974: C D♯ E F F♯ G A♭ A B :0 3 4 5 6 7 8 9 11: 1 ♯2 3 4 ♯4 5 ♭6 6 7: ma13(♯9 add ♯11 ♭13)
namesBySequenceIndex[1973] = '''Zothygic'''
namesBySequenceIndexNegative[1973] = ''''''
semitonesBySequenceIndex[1973] = [0, 3, 4, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[1973] = '''B G J K L M N P S'''
poemAuthors[1973] = ''''''
#1974: ䷘䷍ Integrity Resevoir: 3 1 1 1 1 1 1 2 1  : S R3 G3 M1 M2 P D1 D2 N3 : AgOf

#1975: C D♯ E F F♯ G A♭ B♭ B :0 3 4 5 6 7 8 10 11: 1 ♯2 3 4 ♯4 5 ♭6 ♭7 7: ma11(♯9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[1974] = '''Loptygic'''
namesBySequenceIndexNegative[1974] = ''''''
semitonesBySequenceIndex[1974] = [0, 3, 4, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[1974] = '''B G J K L M N R S'''
poemAuthors[1974] = ''''''
#1975: ䷘䷈ Integrity Homegrown: 3 1 1 1 1 1 2 1 1  : S R3 G3 M1 M2 P D1 N2 N3 : AgOm

#1976: C D♯ E F F♯ G A B♭ B :0 3 4 5 6 7 9 10 11: 1 ♯2 3 4 ♯4 5 6 ♭7 7: ma13(♯9 add ♭7 ♯11)
namesBySequenceIndex[1975] = '''Dycrygic'''
namesBySequenceIndexNegative[1975] = ''''''
semitonesBySequenceIndex[1975] = [0, 3, 4, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[1975] = '''B G J K L M P R S'''
poemAuthors[1975] = ''''''
#1976: ䷘䷉ Integrity Treading: 3 1 1 1 1 2 1 1 1  : S R3 G3 M1 M2 P D2 N2 N3 : AgEg

#1977: C D♯ E F G♭ A♭ A B♭ B :0 3 4 5 6 8 9 10 11: 1 ♯2 3 4 ♭5 ♭6 6 ♭7 7: ma13(♯5 ♯9 add ♭7 ♯11)
namesBySequenceIndex[1976] = '''Zathygic'''
namesBySequenceIndexNegative[1976] = ''''''
semitonesBySequenceIndex[1976] = [0, 3, 4, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[1976] = '''B G J K L N P R S'''
poemAuthors[1976] = ''''''
#1977: ䷘䷌ Integrity Fellowship: 3 1 1 1 2 1 1 1 1  : S R3 G3 M1 M2 D1 D2 N2 N3 : AgIg

#1978: C D♯ E F G A♭ A B♭ B :0 3 4 5 7 8 9 10 11: 1 ♯2 3 4 5 ♭6 6 ♭7 7: ma13(♯9 add ♭7 ♭13)
namesBySequenceIndex[1977] = '''Kalygic'''
namesBySequenceIndexNegative[1977] = ''''''
semitonesBySequenceIndex[1977] = [0, 3, 4, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[1977] = '''B G J K M N P R S'''
poemAuthors[1977] = ''''''
#1978: ䷘䷫ Integrity Coupling: 3 1 1 2 1 1 1 1 1  : S R3 G3 M1 P D1 D2 N2 N3 : Agog

#1979: C D♯ E F♯ G A♭ A B♭ B :0 3 4 6 7 8 9 10 11: 1 ♯2 3 ♯4 5 ♭6 6 ♭7 7: ma13(♯9 ♯11 add ♭7 ♭13)
namesBySequenceIndex[1978] = '''Phrygic'''
namesBySequenceIndexNegative[1978] = ''''''
semitonesBySequenceIndex[1978] = [0, 3, 4, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1978] = '''B G J L M N P R S'''
poemAuthors[1978] = ''''''
#1979: ䷐䷀ Following Creative: 3 1 2 1 1 1 1 1 1  : S R3 G3 M2 P D1 D2 N2 N3 : AdOg

#1980: C E♭ F F♯ G A♭ A B♭ B :0 3 5 6 7 8 9 10 11: 1 ♭3 4 ♯4 5 ♭6 6 ♭7 7: mima13(♯9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[1979] = '''Sathygic'''
namesBySequenceIndexNegative[1979] = ''''''
semitonesBySequenceIndex[1979] = [0, 3, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1979] = '''B G K L M N P R S'''
poemAuthors[1979] = ''''''
#1980: ䷔䷀ Biting Creative: 3 2 1 1 1 1 1 1 1  : S G2 M1 M2 P D1 D2 N2 N3 : AfOg

#1981: C E F F♯ G A♭ A B♭ B :0 4 5 6 7 8 9 10 11: 1 3 4 ♯4 5 ♭6 6 ♭7 7: ma13(no 9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[1980] = '''Manygic'''
namesBySequenceIndexNegative[1980] = ''''''
semitonesBySequenceIndex[1980] = [0, 4, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[1980] = '''B J K L M N P R S'''
poemAuthors[1980] = ''''''
#1981: ䷩䷀ Augmenting Creative: 4 1 1 1 1 1 1 1 1  : S G3 M1 M2 P D1 D2 N2 N3 : AmOg

#1982: C D♭ D E♭ E F F♯ G A♭ A :0 1 2 3 4 5 6 7 8 9: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭6 6: 13(no 7 add ♭6 ♭9 ♯9 ♯11)
namesBySequenceIndex[1981] = '''Dodyllian'''
namesBySequenceIndexNegative[1981] = ''''''
semitonesBySequenceIndex[1981] = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
poemBySequenceIndex[1981] = '''B D F G J K L M N P'''
poemAuthors[1981] = ''''''
#1982: ䷀䷡ Creative Might: 1 1 1 1 1 1 1 1 1 3  : S R1 R2 G2 G3 M1 M2 P D1 N1 : OgOb

#1983: C D♭ D E♭ E F F♯ G A♭ B♭ :0 1 2 3 4 5 6 7 8 10: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭6 ♭7: 11(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1982] = '''Mixodyllian'''
namesBySequenceIndexNegative[1982] = ''''''
semitonesBySequenceIndex[1982] = [0, 1, 2, 3, 4, 5, 6, 7, 8, 10]
poemBySequenceIndex[1982] = '''B D F G J K L M N R'''
poemAuthors[1982] = ''''''
#1983: ䷀䷄ Creative Biding: 1 1 1 1 1 1 1 1 2 2  : S R1 R2 G2 G3 M1 M2 P D1 N2 : OgOk

#1984: C D♭ D E♭ E F F♯ G A♭ B :0 1 2 3 4 5 6 7 8 11: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭6 7: ma11(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1983] = '''Zogyllian'''
namesBySequenceIndexNegative[1983] = ''''''
semitonesBySequenceIndex[1983] = [0, 1, 2, 3, 4, 5, 6, 7, 8, 11]
poemBySequenceIndex[1983] = '''B D F G J K L M N S'''
poemAuthors[1983] = ''''''
#1984: ䷀䷙ Creative Cultivating: 1 1 1 1 1 1 1 1 3 1  : S R1 R2 G2 G3 M1 M2 P D1 N3 : OgOl

#1985: C D♭ D E♭ E F F♯ G A B♭ :0 1 2 3 4 5 6 7 9 10: 1 ♭2 2 ♭3 3 4 ♯4 5 6 ♭7: 13(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1984] = '''Aerygyllian'''
namesBySequenceIndexNegative[1984] = ''''''
semitonesBySequenceIndex[1984] = [0, 1, 2, 3, 4, 5, 6, 7, 9, 10]
poemBySequenceIndex[1984] = '''B D F G J K L M P R'''
poemAuthors[1984] = ''''''
#1985: ䷀䷹ Creative Joyous: 1 1 1 1 1 1 1 2 1 2  : S R1 R2 G2 G3 M1 M2 P D2 N2 : OgEd

#1986: C D♭ D E♭ E F F♯ G A B :0 1 2 3 4 5 6 7 9 11: 1 ♭2 2 ♭3 3 4 ♯4 5 6 7: ma13(add ♭9 ♯9 ♯11)
namesBySequenceIndex[1985] = '''Aeradyllian'''
namesBySequenceIndexNegative[1985] = ''''''
semitonesBySequenceIndex[1985] = [0, 1, 2, 3, 4, 5, 6, 7, 9, 11]
poemBySequenceIndex[1985] = '''B D F G J K L M P S'''
poemAuthors[1985] = ''''''
#1986: ䷀䷥ Creative Polarising: 1 1 1 1 1 1 1 2 2 1  : S R1 R2 G2 G3 M1 M2 P D2 N3 : OgEf

#1987: C D♭ D E♭ E F F♯ G A♯ B :0 1 2 3 4 5 6 7 10 11: 1 ♭2 2 ♭3 3 4 ♯4 5 ♯6 7: ma11(add ♭7 ♭9 ♯9 ♯11)
namesBySequenceIndex[1986] = '''Madyllian'''
namesBySequenceIndexNegative[1986] = ''''''
semitonesBySequenceIndex[1986] = [0, 1, 2, 3, 4, 5, 6, 7, 10, 11]
poemBySequenceIndex[1986] = '''B D F G J K L M R S'''
poemAuthors[1986] = ''''''
#1987: ䷀䷼ Creative Truth: 1 1 1 1 1 1 1 3 1 1  : S R1 R2 G2 G3 M1 M2 P D3 N3 : OgEm

#1988: C D♭ D E♭ E F G♭ A♭ A B♭ :0 1 2 3 4 5 6 8 9 10: 1 ♭2 2 ♭3 3 4 ♭5 ♭6 6 ♭7: 13(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1987] = '''Rocryllian'''
namesBySequenceIndexNegative[1987] = ''''''
semitonesBySequenceIndex[1987] = [0, 1, 2, 3, 4, 5, 6, 8, 9, 10]
poemBySequenceIndex[1987] = '''B D F G J K L N P R'''
poemAuthors[1987] = ''''''
#1988: ䷀䷰ Creative Revolution: 1 1 1 1 1 1 2 1 1 2  : S R1 R2 G2 G3 M1 M2 D1 D2 N2 : OgId

#1989: C D♭ D E♭ E F G♭ A♭ A B :0 1 2 3 4 5 6 8 9 11: 1 ♭2 2 ♭3 3 4 ♭5 ♭6 6 7: ma13(♯5 add ♭9 ♯9 ♯11)
namesBySequenceIndex[1988] = '''Dathyllian'''
namesBySequenceIndexNegative[1988] = ''''''
semitonesBySequenceIndex[1988] = [0, 1, 2, 3, 4, 5, 6, 8, 9, 11]
poemBySequenceIndex[1988] = '''B D F G J K L N P S'''
poemAuthors[1988] = ''''''
#1989: ䷀䷝ Creative Distanced: 1 1 1 1 1 1 2 1 2 1  : S R1 R2 G2 G3 M1 M2 D1 D2 N3 : OgIf

#1990: C D♭ D E♭ E F G♭ A♭ B♭ B :0 1 2 3 4 5 6 8 10 11: 1 ♭2 2 ♭3 3 4 ♭5 ♭6 ♭7 7: ma11(♯5 add ♭7 ♭9 ♯9 ♯11)
namesBySequenceIndex[1989] = '''Ryptyllian'''
namesBySequenceIndexNegative[1989] = ''''''
semitonesBySequenceIndex[1989] = [0, 1, 2, 3, 4, 5, 6, 8, 10, 11]
poemBySequenceIndex[1989] = '''B D F G J K L N R S'''
poemAuthors[1989] = ''''''
#1990: ䷀䷤ Creative Family: 1 1 1 1 1 1 2 2 1 1  : S R1 R2 G2 G3 M1 M2 D1 N2 N3 : OgIm

#1991: C D♭ D E♭ E F G♭ A B♭ B :0 1 2 3 4 5 6 9 10 11: 1 ♭2 2 ♭3 3 4 ♭5 6 ♭7 7: ma13(♭5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1990] = '''Dycryllian'''
namesBySequenceIndexNegative[1990] = ''''''
semitonesBySequenceIndex[1990] = [0, 1, 2, 3, 4, 5, 6, 9, 10, 11]
poemBySequenceIndex[1990] = '''B D F G J K L P R S'''
poemAuthors[1990] = ''''''
#1991: ䷀䷘ Creative Integrity: 1 1 1 1 1 1 3 1 1 1  : S R1 R2 G2 G3 M1 M2 D2 N2 N3 : OgAg

#1992: C D♭ D E♭ E F G A♭ A B♭ :0 1 2 3 4 5 7 8 9 10: 1 ♭2 2 ♭3 3 4 5 ♭6 6 ♭7: 13(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1991] = '''Soryllian'''
namesBySequenceIndexNegative[1991] = ''''''
semitonesBySequenceIndex[1991] = [0, 1, 2, 3, 4, 5, 7, 8, 9, 10]
poemBySequenceIndex[1991] = '''B D F G J K M N P R'''
poemAuthors[1991] = ''''''
#1992: ䷀䷛ Creative Excess: 1 1 1 1 1 2 1 1 1 2  : S R1 R2 G2 G3 M1 P D1 D2 N2 : Ogod

#1993: C D♭ D E♭ E F G A♭ A B :0 1 2 3 4 5 7 8 9 11: 1 ♭2 2 ♭3 3 4 5 ♭6 6 7: ma13(add ♭9 ♯9 ♭13)
namesBySequenceIndex[1992] = '''Zyryllian'''
namesBySequenceIndexNegative[1992] = ''''''
semitonesBySequenceIndex[1992] = [0, 1, 2, 3, 4, 5, 7, 8, 9, 11]
poemBySequenceIndex[1992] = '''B D F G J K M N P S'''
poemAuthors[1992] = ''''''
#1993: ䷀䷱ Creative Holding: 1 1 1 1 1 2 1 1 2 1  : S R1 R2 G2 G3 M1 P D1 D2 N3 : Ogof

#1994: C D♭ D E♭ E F G A♭ B♭ B :0 1 2 3 4 5 7 8 10 11: 1 ♭2 2 ♭3 3 4 5 ♭6 ♭7 7: ma11(add ♭7 ♭9 ♯9 ♭13)
namesBySequenceIndex[1993] = '''Raga Sindhi-Bhairavi,Boptyllian'''
namesBySequenceIndexNegative[1993] = ''''''
semitonesBySequenceIndex[1993] = [0, 1, 2, 3, 4, 5, 7, 8, 10, 11]
poemBySequenceIndex[1993] = '''B D F G J K M N R S'''
poemAuthors[1993] = ''''''
#1994: ䷀䷸ Creative Osmosis: 1 1 1 1 1 2 1 2 1 1  : S R1 R2 G2 G3 M1 P D1 N2 N3 : Ogom

#1995: C D♭ D E♭ E F G A B♭ B :0 1 2 3 4 5 7 9 10 11: 1 ♭2 2 ♭3 3 4 5 6 ♭7 7: ma13(add ♭7 ♭9 ♯9)
namesBySequenceIndex[1994] = '''Loptyllian'''
namesBySequenceIndexNegative[1994] = ''''''
semitonesBySequenceIndex[1994] = [0, 1, 2, 3, 4, 5, 7, 9, 10, 11]
poemBySequenceIndex[1994] = '''B D F G J K M P R S'''
poemAuthors[1994] = ''''''
#1995: ䷀䷅ Creative Arguing: 1 1 1 1 1 2 2 1 1 1  : S R1 R2 G2 G3 M1 P D2 N2 N3 : Ogeg

#1996: C D♭ D E♭ E F G♯ A B♭ B :0 1 2 3 4 5 8 9 10 11: 1 ♭2 2 ♭3 3 4 ♯5 6 ♭7 7: ma13(♯5 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1995] = '''Aeogyllian'''
namesBySequenceIndexNegative[1995] = ''''''
semitonesBySequenceIndex[1995] = [0, 1, 2, 3, 4, 5, 8, 9, 10, 11]
poemBySequenceIndex[1995] = '''B D F G J K N P R S'''
poemAuthors[1995] = ''''''
#1996: ䷀䷠ Creative Retiring: 1 1 1 1 1 3 1 1 1 1  : S R1 R2 G2 G3 M1 D1 D2 N2 N3 : Ogig

#1997: C D♭ D E♭ E F♯ G A♭ A B♭ :0 1 2 3 4 6 7 8 9 10: 1 ♭2 2 ♭3 3 ♯4 5 ♭6 6 ♭7: 13(♯11 add ♭9 ♯9 ♭13)
namesBySequenceIndex[1996] = '''Epiryllian'''
namesBySequenceIndexNegative[1996] = ''''''
semitonesBySequenceIndex[1996] = [0, 1, 2, 3, 4, 6, 7, 8, 9, 10]
poemBySequenceIndex[1996] = '''B D F G J L M N P R'''
poemAuthors[1996] = ''''''
#1997: ䷪䷪ Decision Decision : 1 1 1 1 2 1 1 1 1 2  : S R1 R2 G2 G3 M2 P D1 D2 N2 : OdOd

#1998: C D♭ D E♭ E F♯ G A♭ A B :0 1 2 3 4 6 7 8 9 11: 1 ♭2 2 ♭3 3 ♯4 5 ♭6 6 7: ma13(♯11 add ♭9 ♯9 ♭13)
namesBySequenceIndex[1997] = '''Godyllian'''
namesBySequenceIndexNegative[1997] = ''''''
semitonesBySequenceIndex[1997] = [0, 1, 2, 3, 4, 6, 7, 8, 9, 11]
poemBySequenceIndex[1997] = '''B D F G J L M N P S'''
poemAuthors[1997] = ''''''
#1998: ䷪䷍ Decision Resevoir: 1 1 1 1 2 1 1 1 2 1  : S R1 R2 G2 G3 M2 P D1 D2 N3 : OdOf

#1999: C D♭ D E♭ E F♯ G A♭ B♭ B :0 1 2 3 4 6 7 8 10 11: 1 ♭2 2 ♭3 3 ♯4 5 ♭6 ♭7 7: ma9(add ♭7 ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[1998] = '''Sagyllian'''
namesBySequenceIndexNegative[1998] = ''''''
semitonesBySequenceIndex[1998] = [0, 1, 2, 3, 4, 6, 7, 8, 10, 11]
poemBySequenceIndex[1998] = '''B D F G J L M N R S'''
poemAuthors[1998] = ''''''
#1999: ䷪䷈ Decision Homegrown: 1 1 1 1 2 1 1 2 1 1  : S R1 R2 G2 G3 M2 P D1 N2 N3 : OdOm

#2000: C D♭ D E♭ E F♯ G A B♭ B :0 1 2 3 4 6 7 9 10 11: 1 ♭2 2 ♭3 3 ♯4 5 6 ♭7 7: ma13(♯11 add ♭7 ♭9 ♯9)
namesBySequenceIndex[1999] = '''Bagyllian'''
namesBySequenceIndexNegative[1999] = ''''''
semitonesBySequenceIndex[1999] = [0, 1, 2, 3, 4, 6, 7, 9, 10, 11]
poemBySequenceIndex[1999] = '''B D F G J L M P R S'''
poemAuthors[1999] = ''''''
#2000: ䷪䷉ Decision Treading: 1 1 1 1 2 1 2 1 1 1  : S R1 R2 G2 G3 M2 P D2 N2 N3 : OdEg

#2001: C D♭ D E♭ E F♯ G♯ A B♭ B :0 1 2 3 4 6 8 9 10 11: 1 ♭2 2 ♭3 3 ♯4 ♯5 6 ♭7 7: ma13(♯5 ♯11 add ♭7 ♭9 ♯9)
namesBySequenceIndex[2000] = '''Kataphyllian'''
namesBySequenceIndexNegative[2000] = ''''''
semitonesBySequenceIndex[2000] = [0, 1, 2, 3, 4, 6, 8, 9, 10, 11]
poemBySequenceIndex[2000] = '''B D F G J L N P R S'''
poemAuthors[2000] = ''''''
#2001: ䷪䷌ Decision Fellowship: 1 1 1 1 2 2 1 1 1 1  : S R1 R2 G2 G3 M2 D1 D2 N2 N3 : OdIg

#2002: C D♭ D E♭ E G A♭ A B♭ B :0 1 2 3 4 7 8 9 10 11: 1 ♭2 2 ♭3 3 5 ♭6 6 ♭7 7: ma9(add ♭7 ♭9 ♯9 ♭13 13)
namesBySequenceIndex[2001] = '''Dydyllian'''
namesBySequenceIndexNegative[2001] = ''''''
semitonesBySequenceIndex[2001] = [0, 1, 2, 3, 4, 7, 8, 9, 10, 11]
poemBySequenceIndex[2001] = '''B D F G J M N P R S'''
poemAuthors[2001] = ''''''
#2002: ䷪䷫ Decision Coupling: 1 1 1 1 3 1 1 1 1 1  : S R1 R2 G2 G3 P D1 D2 N2 N3 : Odog

#2003: C D♭ D E♭ F F♯ G A♭ A B♭ :0 1 2 3 5 6 7 8 9 10: 1 ♭2 2 ♭3 4 ♯4 5 ♭6 6 ♭7: mi13(add ♭9 ♯11 ♭13)
namesBySequenceIndex[2002] = '''Gadyllian'''
namesBySequenceIndexNegative[2002] = ''''''
semitonesBySequenceIndex[2002] = [0, 1, 2, 3, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[2002] = '''B D F G K L M N P R'''
poemAuthors[2002] = ''''''
#2003: ䷍䷪ Resevoir Decision: 1 1 1 2 1 1 1 1 1 2  : S R1 R2 G2 M1 M2 P D1 D2 N2 : OfOd

#2004: C D♭ D E♭ F F♯ G A♭ A B :0 1 2 3 5 6 7 8 9 11: 1 ♭2 2 ♭3 4 ♯4 5 ♭6 6 7: mima13(add ♭9 ♯11 ♭13)
namesBySequenceIndex[2003] = '''Messiaen 7th Mode,Lyryllian'''
namesBySequenceIndexNegative[2003] = ''''''
semitonesBySequenceIndex[2003] = [0, 1, 2, 3, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[2003] = '''B D F G K L M N P S'''
poemAuthors[2003] = ''''''
#2004: ䷍䷍ Resevoir Resevoir : 1 1 1 2 1 1 1 1 2 1  : S R1 R2 G2 M1 M2 P D1 D2 N3 : OfOf

#2005: C D♭ D E♭ F F♯ G A♭ B♭ B :0 1 2 3 5 6 7 8 10 11: 1 ♭2 2 ♭3 4 ♯4 5 ♭6 ♭7 7: mima11(add ♭7 ♭9 ♯11 ♭13)
namesBySequenceIndex[2004] = '''Epityllian'''
namesBySequenceIndexNegative[2004] = ''''''
semitonesBySequenceIndex[2004] = [0, 1, 2, 3, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[2004] = '''B D F G K L M N R S'''
poemAuthors[2004] = ''''''
#2005: ䷍䷈ Resevoir Homegrown: 1 1 1 2 1 1 1 2 1 1  : S R1 R2 G2 M1 M2 P D1 N2 N3 : OfOm

#2006: C D♭ D E♭ F F♯ G A B♭ B :0 1 2 3 5 6 7 9 10 11: 1 ♭2 2 ♭3 4 ♯4 5 6 ♭7 7: mima13(add ♭7 ♭9 ♯11)
namesBySequenceIndex[2005] = '''Epinyllian'''
namesBySequenceIndexNegative[2005] = ''''''
semitonesBySequenceIndex[2005] = [0, 1, 2, 3, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[2005] = '''B D F G K L M P R S'''
poemAuthors[2005] = ''''''
#2006: ䷍䷉ Resevoir Treading: 1 1 1 2 1 1 2 1 1 1  : S R1 R2 G2 M1 M2 P D2 N2 N3 : OfEg

#2007: C D♭ D E♭ F G♭ A♭ A B♭ B :0 1 2 3 5 6 8 9 10 11: 1 ♭2 2 ♭3 4 ♭5 ♭6 6 ♭7 7: °ma13(add ♭7 ♭9 ♭13)
namesBySequenceIndex[2006] = '''Mathyllian'''
namesBySequenceIndexNegative[2006] = ''''''
semitonesBySequenceIndex[2006] = [0, 1, 2, 3, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[2006] = '''B D F G K L N P R S'''
poemAuthors[2006] = ''''''
#2007: ䷍䷌ Resevoir Fellowship: 1 1 1 2 1 2 1 1 1 1  : S R1 R2 G2 M1 M2 D1 D2 N2 N3 : OfIg

#2008: C D♭ D E♭ F G A♭ A B♭ B :0 1 2 3 5 7 8 9 10 11: 1 ♭2 2 ♭3 4 5 ♭6 6 ♭7 7: mima13(add ♭7 ♭9 ♭13)
namesBySequenceIndex[2007] = '''Phradyllian'''
namesBySequenceIndexNegative[2007] = ''''''
semitonesBySequenceIndex[2007] = [0, 1, 2, 3, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[2007] = '''B D F G K M N P R S'''
poemAuthors[2007] = ''''''
#2008: ䷍䷫ Resevoir Coupling: 1 1 1 2 2 1 1 1 1 1  : S R1 R2 G2 M1 P D1 D2 N2 N3 : Ofog

#2009: C D♭ D E♭ F♯ G A♭ A B♭ B :0 1 2 3 6 7 8 9 10 11: 1 ♭2 2 ♭3 ♯4 5 ♭6 6 ♭7 7: mima13(♯11 add ♭7 ♭9 ♭13)
namesBySequenceIndex[2008] = '''Thogyllian'''
namesBySequenceIndexNegative[2008] = ''''''
semitonesBySequenceIndex[2008] = [0, 1, 2, 3, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2008] = '''B D F G L M N P R S'''
poemAuthors[2008] = ''''''
#2009: ䷡䷀ Might Creative: 1 1 1 3 1 1 1 1 1 1  : S R1 R2 G2 M2 P D1 D2 N2 N3 : ObOg

#2010: C D♭ D E F F♯ G A♭ A B♭ :0 1 2 4 5 6 7 8 9 10: 1 ♭2 2 3 4 ♯4 5 ♭6 6 ♭7: 13(add ♭9 ♯11 ♭13)
namesBySequenceIndex[2009] = '''Danyllian'''
namesBySequenceIndexNegative[2009] = ''''''
semitonesBySequenceIndex[2009] = [0, 1, 2, 4, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[2009] = '''B D F J K L M N P R'''
poemAuthors[2009] = ''''''
#2010: ䷈䷪ Homegrown Decision: 1 1 2 1 1 1 1 1 1 2  : S R1 R2 G3 M1 M2 P D1 D2 N2 : OmOd

#2011: C D♭ D E F F♯ G A♭ A B :0 1 2 4 5 6 7 8 9 11: 1 ♭2 2 3 4 ♯4 5 ♭6 6 7: ma13(add ♭9 ♯11 ♭13)
namesBySequenceIndex[2010] = '''Solyllian'''
namesBySequenceIndexNegative[2010] = ''''''
semitonesBySequenceIndex[2010] = [0, 1, 2, 4, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[2010] = '''B D F J K L M N P S'''
poemAuthors[2010] = ''''''
#2011: ䷈䷍ Homegrown Resevoir: 1 1 2 1 1 1 1 1 2 1  : S R1 R2 G3 M1 M2 P D1 D2 N3 : OmOf

#2012: C D♭ D E F F♯ G A♭ B♭ B :0 1 2 4 5 6 7 8 10 11: 1 ♭2 2 3 4 ♯4 5 ♭6 ♭7 7: ma11(add ♭7 ♭9 ♯11 ♭13)
namesBySequenceIndex[2011] = '''Symmetrical Decatonic,Mogyllian'''
namesBySequenceIndexNegative[2011] = ''''''
semitonesBySequenceIndex[2011] = [0, 1, 2, 4, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[2011] = '''B D F J K L M N R S'''
poemAuthors[2011] = ''''''
#2012: ䷈䷈ Homegrown Homegrown : 1 1 2 1 1 1 1 2 1 1  : S R1 R2 G3 M1 M2 P D1 N2 N3 : OmOm

#2013: C D♭ D E F F♯ G A B♭ B :0 1 2 4 5 6 7 9 10 11: 1 ♭2 2 3 4 ♯4 5 6 ♭7 7: ma13(add ♭7 ♭9 ♯11)
namesBySequenceIndex[2012] = '''Ionyllian'''
namesBySequenceIndexNegative[2012] = ''''''
semitonesBySequenceIndex[2012] = [0, 1, 2, 4, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[2012] = '''B D F J K L M P R S'''
poemAuthors[2012] = ''''''
#2013: ䷈䷉ Homegrown Treading: 1 1 2 1 1 1 2 1 1 1  : S R1 R2 G3 M1 M2 P D2 N2 N3 : OmEg

#2014: C D♭ D E F G♭ A♭ A B♭ B :0 1 2 4 5 6 8 9 10 11: 1 ♭2 2 3 4 ♭5 ♭6 6 ♭7 7: ma13(♯5 add ♭7 ♭9 ♯11)
namesBySequenceIndex[2013] = '''Katagyllian'''
namesBySequenceIndexNegative[2013] = ''''''
semitonesBySequenceIndex[2013] = [0, 1, 2, 4, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[2013] = '''B D F J K L N P R S'''
poemAuthors[2013] = ''''''
#2014: ䷈䷌ Homegrown Fellowship: 1 1 2 1 1 2 1 1 1 1  : S R1 R2 G3 M1 M2 D1 D2 N2 N3 : OmIg

#2015: C D♭ D E F G A♭ A B♭ B :0 1 2 4 5 7 8 9 10 11: 1 ♭2 2 3 4 5 ♭6 6 ♭7 7: ma13(add ♭7 ♭9 ♭13)
namesBySequenceIndex[2014] = '''Styptyllian'''
namesBySequenceIndexNegative[2014] = ''''''
semitonesBySequenceIndex[2014] = [0, 1, 2, 4, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[2014] = '''B D F J K M N P R S'''
poemAuthors[2014] = ''''''
#2015: ䷈䷫ Homegrown Coupling: 1 1 2 1 2 1 1 1 1 1  : S R1 R2 G3 M1 P D1 D2 N2 N3 : Omog

#2016: C D♭ D E F♯ G A♭ A B♭ B :0 1 2 4 6 7 8 9 10 11: 1 ♭2 2 3 ♯4 5 ♭6 6 ♭7 7: ma13(♯11 add ♭7 ♭9 ♭13)
namesBySequenceIndex[2015] = '''Dagyllian'''
namesBySequenceIndexNegative[2015] = ''''''
semitonesBySequenceIndex[2015] = [0, 1, 2, 4, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2015] = '''B D F J L M N P R S'''
poemAuthors[2015] = ''''''
#2016: ䷄䷀ Biding Creative: 1 1 2 2 1 1 1 1 1 1  : S R1 R2 G3 M2 P D1 D2 N2 N3 : OkOg

#2017: C D♭ D F F♯ G A♭ A B♭ B :0 1 2 5 6 7 8 9 10 11: 1 ♭2 2 4 ♯4 5 ♭6 6 ♭7 7: ma13sus♯4(add ♭7 ♭9 ♭13)
namesBySequenceIndex[2016] = '''Rygyllian'''
namesBySequenceIndexNegative[2016] = ''''''
semitonesBySequenceIndex[2016] = [0, 1, 2, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2016] = '''B D F K L M N P R S'''
poemAuthors[2016] = ''''''
#2017: ䷙䷀ Cultivating Creative: 1 1 3 1 1 1 1 1 1 1  : S R1 G1 M1 M2 P D1 D2 N2 N3 : OlOg

#2018: C D♭ D♯ E F F♯ G A♭ A B♭ :0 1 3 4 5 6 7 8 9 10: 1 ♭2 ♯2 3 4 ♯4 5 ♭6 6 ♭7: 13(♭9 add ♯9 ♯11 ♭13)
namesBySequenceIndex[2017] = '''Maqam Tarzanuyn,Lythyllian'''
namesBySequenceIndexNegative[2017] = ''''''
semitonesBySequenceIndex[2017] = [0, 1, 3, 4, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[2017] = '''B D G J K L M N P R'''
poemAuthors[2017] = ''''''
#2018: ䷉䷪ Treading Decision: 1 2 1 1 1 1 1 1 1 2  : S R1 R3 G3 M1 M2 P D1 D2 N2 : EgOd

#2019: C D♭ D♯ E F F♯ G A♭ A B :0 1 3 4 5 6 7 8 9 11: 1 ♭2 ♯2 3 4 ♯4 5 ♭6 6 7: ma13(♭9 add ♯9 ♯11 ♭13)
namesBySequenceIndex[2018] = '''Goptyllian'''
namesBySequenceIndexNegative[2018] = ''''''
semitonesBySequenceIndex[2018] = [0, 1, 3, 4, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[2018] = '''B D G J K L M N P S'''
poemAuthors[2018] = ''''''
#2019: ䷉䷍ Treading Resevoir: 1 2 1 1 1 1 1 1 2 1  : S R1 R3 G3 M1 M2 P D1 D2 N3 : EgOf

#2020: C D♭ D♯ E F F♯ G A♭ B♭ B :0 1 3 4 5 6 7 8 10 11: 1 ♭2 ♯2 3 4 ♯4 5 ♭6 ♭7 7: ma11(♭9 add ♭7 ♯9 ♯11 ♭13)
namesBySequenceIndex[2019] = '''Zyphyllian'''
namesBySequenceIndexNegative[2019] = ''''''
semitonesBySequenceIndex[2019] = [0, 1, 3, 4, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[2019] = '''B D G J K L M N R S'''
poemAuthors[2019] = ''''''
#2020: ䷉䷈ Treading Homegrown: 1 2 1 1 1 1 1 2 1 1  : S R1 R3 G3 M1 M2 P D1 N2 N3 : EgOm

#2021: C D♭ D♯ E F F♯ G A B♭ B :0 1 3 4 5 6 7 9 10 11: 1 ♭2 ♯2 3 4 ♯4 5 6 ♭7 7: ma13(♭9 add ♭7 ♯9 ♯11)
namesBySequenceIndex[2020] = '''Katodyllian'''
namesBySequenceIndexNegative[2020] = ''''''
semitonesBySequenceIndex[2020] = [0, 1, 3, 4, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[2020] = '''B D G J K L M P R S'''
poemAuthors[2020] = ''''''
#2021: ䷉䷉ Treading Treading : 1 2 1 1 1 1 2 1 1 1  : S R1 R3 G3 M1 M2 P D2 N2 N3 : EgEg

#2022: C D♭ D♯ E F G♭ A♭ A B♭ B :0 1 3 4 5 6 8 9 10 11: 1 ♭2 ♯2 3 4 ♭5 ♭6 6 ♭7 7: ma13(♯5 ♭9 add ♭7 ♯9 ♯11)
namesBySequenceIndex[2021] = '''Aeoryllian'''
namesBySequenceIndexNegative[2021] = ''''''
semitonesBySequenceIndex[2021] = [0, 1, 3, 4, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[2021] = '''B D G J K L N P R S'''
poemAuthors[2021] = ''''''
#2022: ䷉䷌ Treading Fellowship: 1 2 1 1 1 2 1 1 1 1  : S R1 R3 G3 M1 M2 D1 D2 N2 N3 : EgIg

#2023: C D♭ D♯ E F G A♭ A B♭ B :0 1 3 4 5 7 8 9 10 11: 1 ♭2 ♯2 3 4 5 ♭6 6 ♭7 7: ma13(♭9 add ♭7 ♯9 ♭13)
namesBySequenceIndex[2022] = '''Ragyllian'''
namesBySequenceIndexNegative[2022] = ''''''
semitonesBySequenceIndex[2022] = [0, 1, 3, 4, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[2022] = '''B D G J K M N P R S'''
poemAuthors[2022] = ''''''
#2023: ䷉䷫ Treading Coupling: 1 2 1 1 2 1 1 1 1 1  : S R1 R3 G3 M1 P D1 D2 N2 N3 : Egog

#2024: C D♭ D♯ E F♯ G A♭ A B♭ B :0 1 3 4 6 7 8 9 10 11: 1 ♭2 ♯2 3 ♯4 5 ♭6 6 ♭7 7: ma13(♭9 ♯11 add ♭7 ♯9 ♭13)
namesBySequenceIndex[2023] = '''Zolyllian'''
namesBySequenceIndexNegative[2023] = ''''''
semitonesBySequenceIndex[2023] = [0, 1, 3, 4, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2023] = '''B D G J L M N P R S'''
poemAuthors[2023] = ''''''
#2024: ䷹䷀ Joyous Creative: 1 2 1 2 1 1 1 1 1 1  : S R1 R3 G3 M2 P D1 D2 N2 N3 : EdOg

#2025: C D♭ E♭ F F♯ G A♭ A B♭ B :0 1 3 5 6 7 8 9 10 11: 1 ♭2 ♭3 4 ♯4 5 ♭6 6 ♭7 7: mima13(♭9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[2024] = '''Katyllian'''
namesBySequenceIndexNegative[2024] = ''''''
semitonesBySequenceIndex[2024] = [0, 1, 3, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2024] = '''B D G K L M N P R S'''
poemAuthors[2024] = ''''''
#2025: ䷥䷀ Polarising Creative: 1 2 2 1 1 1 1 1 1 1  : S R1 G2 M1 M2 P D1 D2 N2 N3 : EfOg

#2026: C D♭ E F F♯ G A♭ A B♭ B :0 1 4 5 6 7 8 9 10 11: 1 ♭2 3 4 ♯4 5 ♭6 6 ♭7 7: ma13(♭9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[2025] = '''Bathyllian'''
namesBySequenceIndexNegative[2025] = ''''''
semitonesBySequenceIndex[2025] = [0, 1, 4, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2025] = '''B D J K L M N P R S'''
poemAuthors[2025] = ''''''
#2026: ䷼䷀ Truth Creative: 1 3 1 1 1 1 1 1 1 1  : S R1 G3 M1 M2 P D1 D2 N2 N3 : EmOg

#2027: C D D♯ E F F♯ G A♭ A B♭ :0 2 3 4 5 6 7 8 9 10: 1 2 ♯2 3 4 ♯4 5 ♭6 6 ♭7: 13(add ♯9 ♯11 ♭13)
namesBySequenceIndex[2026] = '''Katogyllian'''
namesBySequenceIndexNegative[2026] = ''''''
semitonesBySequenceIndex[2026] = [0, 2, 3, 4, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[2026] = '''B F G J K L M N P R'''
poemAuthors[2026] = ''''''
#2027: ䷌䷪ Fellowship Decision: 2 1 1 1 1 1 1 1 1 2  : S R2 R3 G3 M1 M2 P D1 D2 N2 : IgOd

#2028: C D D♯ E F F♯ G A♭ A B :0 2 3 4 5 6 7 8 9 11: 1 2 ♯2 3 4 ♯4 5 ♭6 6 7: ma13(add ♯9 ♯11 ♭13)
namesBySequenceIndex[2027] = '''Maqam Shawq Afza,Bacryllian'''
namesBySequenceIndexNegative[2027] = ''''''
semitonesBySequenceIndex[2027] = [0, 2, 3, 4, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[2027] = '''B F G J K L M N P S'''
poemAuthors[2027] = ''''''
#2028: ䷌䷍ Fellowship Resevoir: 2 1 1 1 1 1 1 1 2 1  : S R2 R3 G3 M1 M2 P D1 D2 N3 : IgOf

#2029: C D D♯ E F F♯ G A♭ B♭ B :0 2 3 4 5 6 7 8 10 11: 1 2 ♯2 3 4 ♯4 5 ♭6 ♭7 7: ma11(add ♭7 ♯9 ♯11 ♭13)
namesBySequenceIndex[2028] = '''Epocryllian'''
namesBySequenceIndexNegative[2028] = ''''''
semitonesBySequenceIndex[2028] = [0, 2, 3, 4, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[2028] = '''B F G J K L M N R S'''
poemAuthors[2028] = ''''''
#2029: ䷌䷈ Fellowship Homegrown: 2 1 1 1 1 1 1 2 1 1  : S R2 R3 G3 M1 M2 P D1 N2 N3 : IgOm

#2030: C D D♯ E F F♯ G A B♭ B :0 2 3 4 5 6 7 9 10 11: 1 2 ♯2 3 4 ♯4 5 6 ♭7 7: ma13(add ♭7 ♯9 ♯11)
namesBySequenceIndex[2029] = '''Garyllian,Minor Pentatonic with Leading Tones'''
namesBySequenceIndexNegative[2029] = ''''''
semitonesBySequenceIndex[2029] = [0, 2, 3, 4, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[2029] = '''B F G J K L M P R S'''
poemAuthors[2029] = ''''''
#2030: ䷌䷉ Fellowship Treading: 2 1 1 1 1 1 2 1 1 1  : S R2 R3 G3 M1 M2 P D2 N2 N3 : IgEg

#2031: C D D♯ E F G♭ A♭ A B♭ B :0 2 3 4 5 6 8 9 10 11: 1 2 ♯2 3 4 ♭5 ♭6 6 ♭7 7: ma13(♯5 add ♭7 ♯9 ♯11)
namesBySequenceIndex[2030] = '''Thydyllian'''
namesBySequenceIndexNegative[2030] = ''''''
semitonesBySequenceIndex[2030] = [0, 2, 3, 4, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[2030] = '''B F G J K L N P R S'''
poemAuthors[2030] = ''''''
#2031: ䷌䷌ Fellowship Fellowship : 2 1 1 1 1 2 1 1 1 1  : S R2 R3 G3 M1 M2 D1 D2 N2 N3 : IgIg

#2032: C D D♯ E F G A♭ A B♭ B :0 2 3 4 5 7 8 9 10 11: 1 2 ♯2 3 4 5 ♭6 6 ♭7 7: ma13(add ♭7 ♯9 ♭13)
namesBySequenceIndex[2031] = '''Major Minor Mixed,Aerycryllian'''
namesBySequenceIndexNegative[2031] = ''''''
semitonesBySequenceIndex[2031] = [0, 2, 3, 4, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[2031] = '''B F G J K M N P R S'''
poemAuthors[2031] = ''''''
#2032: ䷌䷫ Fellowship Coupling: 2 1 1 1 2 1 1 1 1 1  : S R2 R3 G3 M1 P D1 D2 N2 N3 : Igog

#2033: C D D♯ E F♯ G A♭ A B♭ B :0 2 3 4 6 7 8 9 10 11: 1 2 ♯2 3 ♯4 5 ♭6 6 ♭7 7: ma13(♯11 add ♭7 ♯9 ♭13)
namesBySequenceIndex[2032] = '''Staptyllian'''
namesBySequenceIndexNegative[2032] = ''''''
semitonesBySequenceIndex[2032] = [0, 2, 3, 4, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2032] = '''B F G J L M N P R S'''
poemAuthors[2032] = ''''''
#2033: ䷰䷀ Revolution Creative: 2 1 1 2 1 1 1 1 1 1  : S R2 R3 G3 M2 P D1 D2 N2 N3 : IdOg

#2034: C D E♭ F F♯ G A♭ A B♭ B :0 2 3 5 6 7 8 9 10 11: 1 2 ♭3 4 ♯4 5 ♭6 6 ♭7 7: mima13(add ♭7 ♯11 ♭13)
namesBySequenceIndex[2033] = '''Gothyllian'''
namesBySequenceIndexNegative[2033] = ''''''
semitonesBySequenceIndex[2033] = [0, 2, 3, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2033] = '''B F G K L M N P R S'''
poemAuthors[2033] = ''''''
#2034: ䷝䷀ Distanced Creative: 2 1 2 1 1 1 1 1 1 1  : S R2 G2 M1 M2 P D1 D2 N2 N3 : IfOg

#2035: C D E F F♯ G A♭ A B♭ B :0 2 4 5 6 7 8 9 10 11: 1 2 3 4 ♯4 5 ♭6 6 ♭7 7: ma13(add ♭7 ♯11 ♭13)
namesBySequenceIndex[2034] = '''Sydyllian'''
namesBySequenceIndexNegative[2034] = ''''''
semitonesBySequenceIndex[2034] = [0, 2, 4, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2034] = '''B F J K L M N P R S'''
poemAuthors[2034] = ''''''
#2035: ䷤䷀ Family Creative: 2 2 1 1 1 1 1 1 1 1  : S R2 G3 M1 M2 P D1 D2 N2 N3 : ImOg

#2036: C D♯ E F F♯ G A♭ A B♭ B :0 3 4 5 6 7 8 9 10 11: 1 ♯2 3 4 ♯4 5 ♭6 6 ♭7 7: ma13(♯9 add ♭7 ♯11 ♭13)
namesBySequenceIndex[2035] = '''Katoryllian'''
namesBySequenceIndexNegative[2035] = ''''''
semitonesBySequenceIndex[2035] = [0, 3, 4, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2035] = '''B G J K L M N P R S'''
poemAuthors[2035] = ''''''
#2036: ䷘䷀ Integrity Creative: 3 1 1 1 1 1 1 1 1 1  : S R3 G3 M1 M2 P D1 D2 N2 N3 : AgOg

#2037: C D♭ D E♭ E F F♯ G A♭ A B♭ :0 1 2 3 4 5 6 7 8 9 10: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭6 6 ♭7: 13(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[2036] = '''Monatic'''
namesBySequenceIndexNegative[2036] = ''''''
semitonesBySequenceIndex[2036] = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
poemBySequenceIndex[2036] = '''B D F G J K L M N P R'''
poemAuthors[2036] = ''''''
#2037: ䷀䷪ Creative Decision: 1 1 1 1 1 1 1 1 1 1 2  : S R1 R2 G2 G3 M1 M2 P D1 D2 N2 : OgOd

#2038: C D♭ D E♭ E F F♯ G A♭ A B :0 1 2 3 4 5 6 7 8 9 11: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭6 6 7: ma13(add ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[2037] = '''Solatic'''
namesBySequenceIndexNegative[2037] = ''''''
semitonesBySequenceIndex[2037] = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11]
poemBySequenceIndex[2037] = '''B D F G J K L M N P S'''
poemAuthors[2037] = ''''''
#2038: ䷀䷍ Creative Resevoir: 1 1 1 1 1 1 1 1 1 2 1  : S R1 R2 G2 G3 M1 M2 P D1 D2 N3 : OgOf

#2039: C D♭ D E♭ E F F♯ G A♭ B♭ B :0 1 2 3 4 5 6 7 8 10 11: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭6 ♭7 7: ma11(add ♭7 ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[2038] = '''Zylatic'''
namesBySequenceIndexNegative[2038] = ''''''
semitonesBySequenceIndex[2038] = [0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11]
poemBySequenceIndex[2038] = '''B D F G J K L M N R S'''
poemAuthors[2038] = ''''''
#2039: ䷀䷈ Creative Homegrown: 1 1 1 1 1 1 1 1 2 1 1  : S R1 R2 G2 G3 M1 M2 P D1 N2 N3 : OgOm

#2040: C D♭ D E♭ E F F♯ G A B♭ B :0 1 2 3 4 5 6 7 9 10 11: 1 ♭2 2 ♭3 3 4 ♯4 5 6 ♭7 7: ma13(add ♭7 ♭9 ♯9 ♯11)
namesBySequenceIndex[2039] = '''Mixolatic'''
namesBySequenceIndexNegative[2039] = ''''''
semitonesBySequenceIndex[2039] = [0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11]
poemBySequenceIndex[2039] = '''B D F G J K L M P R S'''
poemAuthors[2039] = ''''''
#2040: ䷀䷉ Creative Treading: 1 1 1 1 1 1 1 2 1 1 1  : S R1 R2 G2 G3 M1 M2 P D2 N2 N3 : OgEg

#2041: C D♭ D E♭ E F G♭ A♭ A B♭ B :0 1 2 3 4 5 6 8 9 10 11: 1 ♭2 2 ♭3 3 4 ♭5 ♭6 6 ♭7 7: ma13(♯5 add ♭7 ♭9 ♯9 ♯11)
namesBySequenceIndex[2040] = '''Soratic'''
namesBySequenceIndexNegative[2040] = ''''''
semitonesBySequenceIndex[2040] = [0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11]
poemBySequenceIndex[2040] = '''B D F G J K L N P R S'''
poemAuthors[2040] = ''''''
#2041: ䷀䷌ Creative Fellowship: 1 1 1 1 1 1 2 1 1 1 1  : S R1 R2 G2 G3 M1 M2 D1 D2 N2 N3 : OgIg

#2042: C D♭ D E♭ E F G A♭ A B♭ B :0 1 2 3 4 5 7 8 9 10 11: 1 ♭2 2 ♭3 3 4 5 ♭6 6 ♭7 7: ma13(add ♭7 ♭9 ♯9 ♭13)
namesBySequenceIndex[2041] = '''Godatic'''
namesBySequenceIndexNegative[2041] = ''''''
semitonesBySequenceIndex[2041] = [0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11]
poemBySequenceIndex[2041] = '''B D F G J K M N P R S'''
poemAuthors[2041] = ''''''
#2042: ䷀䷫ Creative Coupling: 1 1 1 1 1 2 1 1 1 1 1  : S R1 R2 G2 G3 M1 P D1 D2 N2 N3 : Ogog

#2043: C D♭ D E♭ E F♯ G A♭ A B♭ B :0 1 2 3 4 6 7 8 9 10 11: 1 ♭2 2 ♭3 3 ♯4 5 ♭6 6 ♭7 7: ma13(♯11 add ♭7 ♭9 ♯9 ♭13)
namesBySequenceIndex[2042] = '''Eptatic'''
namesBySequenceIndexNegative[2042] = ''''''
semitonesBySequenceIndex[2042] = [0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2042] = '''B D F G J L M N P R S'''
poemAuthors[2042] = ''''''
#2043: ䷪䷀ Decision Creative: 1 1 1 1 2 1 1 1 1 1 1  : S R1 R2 G2 G3 M2 P D1 D2 N2 N3 : OdOg

#2044: C D♭ D E♭ F F♯ G A♭ A B♭ B :0 1 2 3 5 6 7 8 9 10 11: 1 ♭2 2 ♭3 4 ♯4 5 ♭6 6 ♭7 7: mima13(add ♭7 ♭9 ♯11 ♭13)
namesBySequenceIndex[2043] = '''Ionatic'''
namesBySequenceIndexNegative[2043] = ''''''
semitonesBySequenceIndex[2043] = [0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2043] = '''B D F G K L M N P R S'''
poemAuthors[2043] = ''''''
#2044: ䷍䷀ Resevoir Creative: 1 1 1 2 1 1 1 1 1 1 1  : S R1 R2 G2 M1 M2 P D1 D2 N2 N3 : OfOg

#2045: C D♭ D E F F♯ G A♭ A B♭ B :0 1 2 4 5 6 7 8 9 10 11: 1 ♭2 2 3 4 ♯4 5 ♭6 6 ♭7 7: ma13(add ♭7 ♭9 ♯11 ♭13)
namesBySequenceIndex[2044] = '''Aeolatic'''
namesBySequenceIndexNegative[2044] = ''''''
semitonesBySequenceIndex[2044] = [0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2044] = '''B D F J K L M N P R S'''
poemAuthors[2044] = ''''''
#2045: ䷈䷀ Homegrown Creative: 1 1 2 1 1 1 1 1 1 1 1  : S R1 R2 G3 M1 M2 P D1 D2 N2 N3 : OmOg

#2046: C D♭ D♯ E F F♯ G A♭ A B♭ B :0 1 3 4 5 6 7 8 9 10 11: 1 ♭2 ♯2 3 4 ♯4 5 ♭6 6 ♭7 7: ma13(♭9 add ♭7 ♯9 ♯11 ♭13)
namesBySequenceIndex[2045] = '''Thydatic'''
namesBySequenceIndexNegative[2045] = ''''''
semitonesBySequenceIndex[2045] = [0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2045] = '''B D G J K L M N P R S'''
poemAuthors[2045] = ''''''
#2046: ䷉䷀ Treading Creative: 1 2 1 1 1 1 1 1 1 1 1  : S R1 R3 G3 M1 M2 P D1 D2 N2 N3 : EgOg

#2047: C D D♯ E F F♯ G A♭ A B♭ B :0 2 3 4 5 6 7 8 9 10 11: 1 2 ♯2 3 4 ♯4 5 ♭6 6 ♭7 7: ma13(add ♭7 ♯9 ♯11 ♭13)
namesBySequenceIndex[2046] = '''Aerycratic'''
namesBySequenceIndexNegative[2046] = ''''''
semitonesBySequenceIndex[2046] = [0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2046] = '''B F G J K L M N P R S'''
poemAuthors[2046] = ''''''
#2047: ䷌䷀ Fellowship Creative: 2 1 1 1 1 1 1 1 1 1 1  : S R2 R3 G3 M1 M2 P D1 D2 N2 N3 : IgOg

#2048: C D♭ D E♭ E F F♯ G A♭ A B♭ B :0 1 2 3 4 5 6 7 8 9 10 11: 1 ♭2 2 ♭3 3 4 ♯4 5 ♭6 6 ♭7 7: ma13(add ♭7 ♭9 ♯9 ♯11 ♭13)
namesBySequenceIndex[2047] = '''Chromatic,Édrihanian'''
namesBySequenceIndexNegative[2047] = ''''''
semitonesBySequenceIndex[2047] = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
poemBySequenceIndex[2047] = '''Birthing Darkly Fog Grows Jostling Kognitive. Love Mundane Never Precise Rarely Simple'''
poemAuthors[2047] = '''Michael Cenkner'''
#2048: ䷀䷀ Creative Creative : 1 1 1 1 1 1 1 1 1 1 1 1  : S R1 R2 G2 G3 M1 M2 P D1 D2 N2 N3 : OgOg